#!/bin/sh
# *************************************************************************
#
# Licensed Materials - Property of IBM
#
# 5724-CYN
#
# (C) Copyright IBM Corp. 2005,2010  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *************************************************************************
# ######################################################################################
# ######################################################################################
# ######################################################################################
# 
#                               F U N C T I O N S
#
# ######################################################################################
# ######################################################################################
# ######################################################################################

print_syntax ()
{
  cat << EOF

   $SCRIPT       Version $VERSION

Command line syntax is:

   $SCRIPT [srcPath]

   where..
       srcPath - Path for the source binaries you want to install/refresh. 
                 Default: Current directory (\$PWD)
EOF

  exit 1
}

Abort ()
{
    echo "Aborting..."
    cd $INITIDIR
    exit 1
}

flagAndAbort ()
{
    echo "ERROR: $1 Please correct the problem and retry."
    Abort
}

DLTLIB ()
{
    curlib=$1

    if [ -d /QSYS.LIB/$curlib.LIB ]
    then
        system 'DLTLIB '$curlib 2>/dev/null
        rc=$?

        if [ "$rc" != "0" ]
        then
            flagAndAbort "Unable to delete library $curlib."
        fi
    fi
}

CRTLIB ()
{
    curlib=$1

    if [ -d /QSYS.LIB/$curlib.LIB ]
    then
        flagAndAbort "Library $curlib already exist."
    else
        system 'CRTLIB '$curlib 2>/dev/null
        rc=$?

        if [ "$rc" != "0" ]
        then
            flagAndAbort "Unable to create library $curlib."
        fi
    fi
}

CRTSAVF ()
{
    curlib=$1
    cursavf=$2

    if [ -e /QSYS.LIB/$curlib.LIB/$cursavf.FILE ]
    then
        flagAndAbort "SAVF file $curlib/$cursavf already exist."
    else
        system 'CRTSAVF FILE('$curlib/$cursavf')' 2>/dev/null
        rc=$?

        if [ "$rc" != "0" ]
        then
            flagAndAbort "Unable to create SAVF file $curlib/$cursavf."
        fi
    fi
}

_CP ()
{
    srcFile=$1
    tgtFile=$2

    if [ ! -e $srcFile ]
    then
        flagAndAbort "File $srcFile does not exist."
    else
        cp $srcFile $tgtFile 2>/dev/null
        rc=$?

        if [ "$rc" != "0" ]
        then
            flagAndAbort "Unable to copy $srcFile to $tgtFile."
        fi
    fi
}

TRACE ()
{
    if [ "$TRACING" != "" ]
    then
        echo "    $1"
    fi
}

RSTOBJ ()
{
    curobject=$1
    curlib=$2
    cursavf=$3

    TRACE "Starting RSTOBJ [Parameters: $*]..."
    TRACE "    Checking if /QSYS.LIB/$curlib.LIB/$curobject.SRVPGM exists"

    if [ -e /QSYS.LIB/$curlib.LIB/$curobject.SRVPGM ]
    then
        TRACE "        /QSYS.LIB/$curlib.LIB/$curobject.SRVPGM already exists"
        flagAndAbort "Service program $curlib/$curobject already exists."
    else
        TRACE "        /QSYS.LIB/$curlib.LIB/$curobject.SRVPGM does not exist"
        TRACE "    RSTOBJ OBJ($curobject) SAVLIB($curlib) DEV(*SAVF) SAVF($curlib/$cursavf) OBJTYPE(*SRVPGM)"
        system 'RSTOBJ OBJ('$curobject') SAVLIB('$curlib') DEV(*SAVF) OBJTYPE(*SRVPGM) SAVF('$curlib/$cursavf') RSTLIB('$curlib') FRCOBJCVN(*YES)' 2>/dev/null
        rc=$?
        TRACE "        rc=$rc"

        if [ "$rc" != "0" ]
        then
            TRACE "    Checking if /QSYS.LIB/$curlib.LIB/$curobject.SRVPGM got created even with rc != 0"

            if [ ! -e /QSYS.LIB/$curlib.LIB/$curobject.SRVPGM ]
            then
                TRACE "        RSTOBJ failed"
                flagAndAbort "Unable to restore service program $curlib/$curobject."
            else
                TRACE "        /QSYS.LIB/$curlib.LIB/$curobject.SRVPGM got created"
            fi
        fi
    fi

    TRACE "Ending RSTOBJ..."
}

_MV ()
{
    srcFile=$1
    tgtFile=$2

    TRACE "Starting _MV [Parameters: $*]..."
    TRACE "    Checking if $srcFile exists"

    if [ ! -e $srcFile ]
    then
        TRACE "        $srcFile does not exist"
        flagAndAbort "File $srcFile does not exist."
    else
        TRACE "        $srcFile exists"
        TRACE "    mv $srcFile $tgtFile"
        mv $srcFile $tgtFile 2>/dev/null
        rc=$?
        TRACE "        rc=$rc"

        if [ "$rc" != "0" ]
        then
            TRACE "        _MV failed"
            flagAndAbort "Unable to move $srcFile to $tgtFile."
        else
            TRACE "        $srcFile has been moved to $tgtFile"
        fi
    fi

    TRACE "Ending _MV..."
}

DLTSRVPGM ()
{
    curlib=$1
    curobj=$2

    TRACE "Starting DLTSRVPGM [Parameters: $*]..."
    TRACE "    Checking if /QSYS.LIB/$curlib.LIB/$curobj.SRVPGM exists"

    if [ ! -e /QSYS.LIB/$curlib.LIB/$curobj.SRVPGM ]
    then
        TRACE "        /QSYS.LIB/$curlib.LIB/$curobj.SRVPGM does not exist"
    else
        TRACE "        /QSYS.LIB/$curlib.LIB/$curobj.SRVPGM already exists"
        TRACE "    DLTSRVPGM SRVPGM($curlib/$curobj)"
        system 'DLTSRVPGM SRVPGM('$curlib/$curobj')' 2>/dev/null
        rc=$?
        TRACE "        rc=$rc"

        if [ "$rc" != "0" ]
        then
            TRACE "        DLTSRVPGM failed"
            flagAndAbort "Unable to delete /QSYS.LIB/$curlib.LIB/$curobj.SRVPGM"
        else
            TRACE "        /QSYS.LIB/$curlib.LIB/$curobj.SRVPGM has been deleted"
        fi
    fi

    TRACE "Ending DLTSRVPGM..."
}

DLTF ()
{
    curlib=$1
    curobj=$2

    TRACE "Starting DLTF [Parameters: $*]..."
    TRACE "    Checking if /QSYS.LIB/$curlib.LIB/$curobj.FILE exists"

    if [ ! -e /QSYS.LIB/$curlib.LIB/$curobj.FILE ]
    then
        TRACE "        /QSYS.LIB/$curlib.LIB/$curobj.FILE does not exist"
    else
        TRACE "        /QSYS.LIB/$curlib.LIB/$curobj.FILE already exists"
        TRACE "    DLTF FILE($curlib/$curobj)"
        system 'DLTF FILE('$curlib/$curobj')' 2>/dev/null
        rc=$?
        TRACE "        rc=$rc"

        if [ "$rc" != "0" ]
        then
            TRACE "        DLTF failed"
            flagAndAbort "Unable to delete /QSYS.LIB/$curlib.LIB/$curobj.FILE"
        else
            TRACE "        /QSYS.LIB/$curlib.LIB/$curobj.FILE has been deleted"
        fi
    fi

    TRACE "Ending DLTF..."
}

UPDSRVPGM ()
{
    curlib=$1
    cursrvpgm=$2

    TRACE "Starting UPDSRVPGM [Parameters: $*]..."
    TRACE "    Checking if /QSYS.LIB/$curlib.LIB/$cursrvpgm.SRVPGM exists"

    if [ ! -e /QSYS.LIB/$curlib.LIB/$cursrvpgm.SRVPGM ]
    then
        TRACE "        /QSYS.LIB/$curlib.LIB/$cursrvpgm.SRVPGM does not exist"
        flagAndAbort "Service program $curlib/$cursrvpgm does not exist."
    else
        TRACE "        /QSYS.LIB/$curlib.LIB/$cursrvpgm.SRVPGM already exists"
        TRACE "    UPDSRVPGM SRVPGM($curlib/$cursrvpgm) MODULE(*NONE)"
        system 'UPDSRVPGM SRVPGM('$curlib/$cursrvpgm') MODULE(*NONE)' 2>/dev/null
        rc=$?
        TRACE "        rc=$rc"

        if [ "$rc" != "0" ]
        then
            TRACE "        UPDSRVPGM failed"
            flagAndAbort "Unable to update service program $curlib/$cursrvpgm."
        else
            TRACE "        /QSYS.LIB/$curlib.LIB/$cursrvpgm.SRVPGM got updated"
        fi
    fi

    TRACE "Ending UPDSRVPGM..."
}

restoreAMPASS ()
{
    if [ ! -e /QSYS.LIB/QGPL.LIB/AMPASS.SRVPGM ]
    then
        TRACE "        /QSYS.LIB/QGPL.LIB/AMPASS.SRVPGM does not exist"
        
        if [ -e $SRCDIR/toolkit/lib/os400/ampass.savf ]
        then
            TRACE "        Source binaries were found..."
        else
            flagAndAbort "$SRCDIR does not contain the expected source binaries."
        fi

        echo "Restoring service program AMPASS ..."
        DLTF QGPL AMPASS
        CRTSAVF QGPL AMPASS
        _CP $1/toolkit/lib/os400/ampass.savf /qsys.lib/qgpl.lib/AMPASS.FILE
        DLTSRVPGM QGPL AMPASS
        RSTOBJ AMPASS QGPL AMPASS
        UPDSRVPGM QGPL AMPASS
    else
        TRACE "        /QSYS.LIB/QGPL.LIB/AMPASS.SRVPGM exists. No need to restor it."
    fi
}

# ######################################################################################
# ######################################################################################
# ######################################################################################
# 
#                             E N D   O F   F U N C T I O N S
#
# ######################################################################################
# ######################################################################################
# ######################################################################################


SRCDIR=$1
INITIDIR=$PWD
SCRIPT=`basename $0`
VERSION=1.0
TRACING=

TRACE "Starting $SCRIPT..."

if [ "$1" == "?" -o "$1" == "-?" ]
then
    print_syntax
fi

if [ -z $SRCDIR ]
then
    SRCDIR=$PWD
else 
    if [ ! -d $SRCDIR ]
    then
        flagAndAbort "$SRCDIR is not a valid directory."
    fi    
fi

TRACE "    Selected source path: $SRCDIR"
cd $SRCDIR

TRACING=
restoreAMPASS $SRCDIR

exit 0

