#! /bin/sh

#####################################################################         
# Licensed Materials - Property of IBM
#  
# 5724-L62  
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.       
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# 
##################################################################### 

##################################################################### 
# Requirements to execute this script:
#  1. Provide an executable Java 1.5 or higher in the PATH. 
#  2  Do one of the following:
#      1) export the ITCAM_CONFIG variable   - or -
#      2) supply the -config command line argument  - or -
#      3) launch this script from the <config_home>/bin directory. 
#  3. Execute with a WAS administrative userid
#####################################################################
   set -o logical
 
   if [ "$1" = "-reconfig" ]
   then                      
       SCRIPT_DESCR="reconfiguration"         
   else                      
       SCRIPT_DESCR="configuration"
   fi                        
 
   SCRIPT_PY="config.py"
   
   VERBOSE="0" 
   
   # Routine to print path statements 
   # Input: path_var, pathprefix
   print_path() {
     paths=$path_var
     while [ -n "$paths" ]; do
         path=$(echo "$paths" | cut -d ":" -f 1)
         paths=$(echo "$paths" | cut -s -d ":" -f 2-)
         print "$pathprefix$path"
         pathprefix="                    "
     done
   }
   
   #######################################################################
   #  Verify JAVA_HOME 
   #######################################################################

   # Look for an executable Java program in the path.
   # If one is found, set JAVA_HOME if java is found in bin.

   JAVA_EXE=""
   javaloc=""
   paths=$PATH
   while [ -n "$paths" ]; do
     path=$(echo "$paths" | cut -d ":" -f 1)
     paths=$(echo "$paths" | cut -s -d ":" -f 2-)
     if [ -x "$path/java" ]; then
       JAVA_EXE="java"
       javaloc="$path"
       if [ -z "$JAVA_HOME" -a "$(basename $path)" = "bin" ]; then
         export JAVA_HOME=$(dirname $path)
       fi
       break
     fi
   done

   # If no viable Java was found in the path, check JAVA_HOME.

   if [ -z "$JAVA_EXE" ]; then
     if [ -n "$JAVA_HOME" ]; then
       if [ -x "$JAVA_HOME/bin/java" ]; then
         JAVA_EXE=$JAVA_HOME/bin/java
         javaloc="$JAVA_HOME"
       fi
     fi
   fi
   
   if [ -z "$JAVA_EXE" ]; then
     print "*Error*  Unable to locate java program"
     exit 1
   fi
  
   # Check  Java SDK level
   vers=$(java -version 2>&1 | grep -e 'version' | cut -d " " -f 3 | tr -d '\"')  
   v1=$(echo $vers | cut -d "." -f 1)   
   v2=$(echo $vers | cut -d "." -f 2)   
   vc=$((v1*10 + v2))
   if [ vc -lt 15 ]; then
     echo "*ERROR* Java version must be 1.5 or higher"
     exit 1
   fi
   
   #######################################################################
   # Locate ITCAM configuration location from the current directory
   #######################################################################
  
   TMP=`dirname "$0"`/..
   ITCAM_CONFIG_LOC=`cd $TMP; pwd`

   # Verify the CONFIG location is valid
   if [ ! -f "$ITCAM_CONFIG_LOC/bin/config.sh" -o ! -d "$ITCAM_CONFIG_LOC/runtime" ]; then
     print "You must run this script from the ITCAM <config_home/bin> directory" 
     exit 1
   fi
   
   #####################################################################################
   #  Prepare for and launch Jython configuration script 
   #  Input:  $ITCAM_CONFIG_LOC
   #####################################################################################
   
   if [ -z "$CLASSPATH" ]; then
     CLASSPATH=$ITCAM_CONFIG_LOC/bin/jython/jython.jar:$ITCAM_CONFIG_LOC/itcamdc/lib/installbundle.jar
   else
     CLASSPATH=$ITCAM_CONFIG_LOC/bin/jython/jython.jar:$ITCAM_CONFIG_LOC/itcamdc/lib/installbundle.jar:$CLASSPATH
   fi
   if [ -z "$PYTHONPATH" ]; then
     PYTHONPATH=$ITCAM_CONFIG_LOC/bin/lib
   else
     PYTHONPATH=$ITCAM_CONFIG_LOC/bin/lib:$PYTHONPATH
   fi
  
   PERF_JVM_OPTIONS="-Xms256m -Xmx256m"
   PATH=$JAVA_HOME/bin:$PATH
   export PATH
   
   CMD="$JAVA_EXE $PERF_JVM_OPTIONS" 
   CMD="$CMD -Xnoargsconversion"
   CMD="$CMD -Dfile.encoding=ISO8859-1"
   CMD="$CMD -Dpython.console=org.python.util.InteractiveConsole"
   CMD="$CMD -Dpython.path=$PYTHONPATH" 
   CMD="$CMD -classpath $CLASSPATH org.python.util.jython" 
   CMD="$CMD $SCRIPT_PY  $*" 

   if [ $VERBOSE = "1" ]; then
     print
     print "Launching ITCAM $SCRIPT_DESCR"
     print
     print "  JAVA_EXE:         $JAVA_EXE"
     print "  JAVA_HOME:        $JAVA_HOME"
     print "  ITCAM_CONFIG_LOC: $ITCAM_CONFIG_LOC"
     pathprefix="  CLASSPATH:        "
     path_var=$CLASSPATH
     print_path
     pathprefix="  PYTHONPATH:       "
     path_var=$PYTHONPATH
     print_path
     pathprefix="  PATH:             "
     path_var=$PATH
     print_path
     print
     print $CMD
     print 
   fi
   
   cd $ITCAM_CONFIG_LOC/bin
   set +u
   $CMD
   CMDRC=$?
 
exit $CMDRC
