#####################################################################
# Licensed Materials - Property of IBM
#
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
#
# This module creates an itcam template.  The template is used for
# dynamic clusters.  The template are created for distributed and 
# ZOS.
#
#####################################################################

import os, sys, traceback

# Eliminate need to prefix import modules with "lib."
sys.path.append("lib")

import createPCL_config as createPCL
import createPDL
import resourcebundle
import debugUtil
import checkversion
import utils
import configtema
import get_wasroot
import configms
import configsmf
import configsoa
import display_config
import configtpv
import configde
import configtemplate
import configtt
import inputproperties
import createTemplateDCInput
import configtemplategc
import platformUtil
import debugUtil as log

from ConfigurationException import ConfigurationException
from resourcebundle import getmsg as getmsg
from constants import constants
from debugUtil import logDebug as debug
from ConsoleWriter import ConsoleWriter 
from ConfigTemplateTask import ConfigTemplateTask
from PrintWrapper import PrintWrapper 


product = "ITCAM/AD 7.2"
fixlev = "HAAD720 AK00000"




try:

    #############
    # Initialize
    #############
    

    regular_process = True
    continue_config = True
    
    #
    # Instantiate the global constants class
    #
    common_constants = constants()
    
    #
    # Set the print wrapper for stdout and stderr.
    #
    PrintWrapper(common_constants.STDOUT)
    PrintWrapper(common_constants.STDERR)

    debug("Initialization of "+product+" "+fixlev)

    #
    # Initialize the resource bundle
    #
    resourcebundle.set_resourcebundle()

    
    
    ############################
    # Display a startup message
    ############################

    print
    print
    print "%s" % getmsg("FLOWERBOX_TEMPLATEXD","Create ITCAM XD templates")
    print


    #
    # Create the data structures
    #
    pdl = createPDL.make_pdl(createPCL.make_pcl())
    debugUtil.logDumpPCL(pdl["PCL"][0], "Initial creation of PCL with %d entries" % len(pdl["PCL"][0]))
    debugUtil.logDumpPDL(pdl, "Initial creation of PDL with %d entries" % len(pdl), short=False)

    createPDL.proc_envvar(pdl)
    createPDL.proc_cliopt(pdl,sys.argv)
    debugUtil.logDumpPDL(pdl, "PDL after environment variables and command line arguments")

    # Perhaps all we are going to do is display the help text.
    if pdl["HELP"][0]:
        #debugUtil.set_logging(False)
        print "%s" % getmsg("HELP_MIGRATE")
        print
        sys.exit(0)
     
    # Default log and message file names
    if pdl["LOG"][0] is None:
        pdl["LOG"][0:2] = "configtemplate-trace.log", "configtemplatexd"
    if pdl["MESSAGE"][0] is None:
        pdl["MESSAGE"][0:2] = "configtemplate-message.log", "configtemplatexd"
    
    
    # Default and validate the config home.
    if pdl["CONFIGHOME"][0] is None:
        cwd = os.getcwd()
        if cwd.endswith("/bin"):
            config_home = cwd.replace("/bin","")
            pdl["CONFIGHOME"][0:2] = config_home, "configtemplatexd.py"
        else:
            print "%s" % getmsg("CFG5028E", "ITCAM_CONFIG is not set and current directory is not a valid config_home")
            sys.exit(1)


    # Initialize the log utility and interrogate version.properties.
    log.set_logging(pdl["LOG"][0],pdl["MESSAGE"][0], pdl["CONFIGHOME"][0])
    if log.isLogBad():
        print "%s" % getmsg("CFG5027W", "Unable to open log file {0}, diagnostic output will be directed to the console.", [pdl["LOG"][0]])
    else:
        # Redirect sys.stdout
        if pdl["LOGCONSOLE"][0] is True:
            log_file = debugUtil.getLogFilename()
            log_dir = os.path.dirname(log_file)
            console_wtr = ConsoleWriter(log_dir + os.sep + "configtemplate-console.log" )
            # Start with flower box printed above 
            console_wtr.log("%s" % getmsg("FLOWERBOX_TEMPLATEXD","Create ITCAM XD templates"))
    
    
    print "%s" % getmsg("CFG5043I", "Log file name: {0}", [log.getLogFilename()])

    version = checkversion.get_current_version(pdl)
    assert version != -1, getmsg("CFG1070E", "Invalid version number  - {0}", [version])
    utils.setData(pdl, "VERSION", version, "migrate")
    
    utils.setData(pdl, "CREATETEMPLATE", True, "createtemplate")
    #
    # Prompt to select the DMGR home.
    #
    get_wasroot.get_wasroot(pdl)
    #
    # Set a dummpy server name into SERVERS pdl as it is needed
    # for other processing.
    #
    servers = []
    servers.append("XDDUMMY")
    utils.setData(pdl, "SERVERS", servers, "createtemplatexd")
    
    # Config template
    configtemplate.config_template(pdl)
    
    osname = utils.getDataString(pdl, "TEMPLATEOS")
    
    # Configure SOA DC (not for OS400)
    if (osname is not "OS400" or not "OS400-64"):
        configsoa.config_soa(pdl)
    
    # Configure Tema
    configtema.config_tema(pdl)

    # Configure Managing Server
    configms.config_ms(pdl)

    # Config TTAPI
    continue_config = configtt.config_tt(pdl)
    if not continue_config:
        raise ConfigurationException(getmsg("CFG2051I", "The XD template creation was bypassed."))
    
    # Configure SMF only for zos.
    if (osname.startswith("zos")):
        configsmf.config_smf(pdl)

    # Configure TPV
    configtpv.config_tpv(pdl)
    
    # Configure DE
    configde.config_de(pdl)
    
    # Configure GC
    configtemplategc.config_gc(pdl)

        
    #
    # Create the DC input.
    #
    createTemplateDCInput.createManualInputFile(pdl)
    
    # Create input.properties files
    inputproperties.process_inputproperties(pdl)
                
    #
    # Call config task to create the templates.
    #
    config_task = ConfigTemplateTask(pdl, common_constants.CONFIGTEMPLATE, "configtemplatexd_inputlist", "configtemplatexd.log", "configtemplatexd_wsadmin.py")
    config_task.process()
    
    rc = pdl["RETURNCODE"][0]
    if (rc == 0):
        print "\n%s\n" % getmsg("CFG2048I","Creation of XD template has successfully completed with return code {0}.",(pdl["RETURNCODE"][0],))
        
    else:
        print "\n%s\n" % getmsg("CFG2049E","Creation of XD template did not complete successfully with return code {0}.",(pdl["RETURNCODE"][0],))
            
#former assert, unrecoverable error caused by environment settings or input parameters
#fixable by user, not a program error, error text goes to the console
except ConfigurationException, (instance):
    print "%s" % getmsg("CFG5026E", "Critical error in {0}", ["configtemplatexd.py"])
    print "%s" % instance.parameter #print exception message
    sys.exit(1)

except AssertionError:

    print "%s" % getmsg("CFG5022E", "Internal program error detected in {0}", ["configtemplatexd.py"])
    try:
        debugUtil.logException("Assertion error in configtemplatexd.py")
        debugUtil.logDumpStack("Stack data for diagnostic purposes")
    except:
        _type, _val, _tb = sys.exc_info()
        try:
            print "\n%s\n" % getmsg("CFG1003E", "Logging routine failed, traceback follows")
        except:
            print "\nLogging routine failed, traceback follows\n"
        traceback.print_exception(_type,_val,_tb)
    sys.exit(1)

except KeyboardInterrupt:   #User entered terminate character in prompt
    print "%s" % getmsg("CFG2050I", "Exiting the ITCAM Data Collector for WebSphere XD Template Creation Utility.")
    sys.exit(0)

except SystemExit:   # Generic exit (after an exception was already caught and handled)
    print "%s" % getmsg("CFG2050I", "Exiting the ITCAM Data Collector for WebSphere XD Template Creation Utility.")
    sys.exit(1)

except:
    _exctype, _excval, _exctb = sys.exc_info()

    try:
        msg = getmsg("CFG2056E", "Unexpected exception detected in configtemplatexd.py")
    except:
        msg = "Unexpected exception detected in configtemplatexd.py"
    print "\n%s\n" % msg

    try:
        debugUtil.logException('Error in main routine')
    except:
        try:
            print "\n%s\n" % getmsg("CFG1003E", "Logging routine failed, traceback follows")
        except:
            print "\nLogging routine failed, traceback follows\n"
    traceback.print_exception(_exctype,_excval,_exctb)
    print
    sys.exit(1)
