#!/bin/sh

#####################################################################         
# Licensed Materials - Property of IBM
#  
# 5724-L62  
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.       
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# 
##################################################################### 

# Set ITCAM_CONFIGHOME if undefined
if [ -z "$ITCAM_CONFIGHOME" ]; then
    TMP=`dirname "$0"`/..
    ITCAM_CONFIGHOME=`cd $TMP; pwd`
    export ITCAM_CONFIGHOME
fi

# Check ITCAM_CONFIGHOME is valid
if [ ! -f "$ITCAM_CONFIGHOME/bin/config.sh" ]; then
    echo The ITCAM_CONFIGHOME environment variable is not defined correctly
    echo This environment variable is needed to run this program
    exit 1
fi

# Check JAVA_HOME is valid 
if [ -z "$JAVA_HOME" -o ! -f "$JAVA_HOME"/bin/java ]; then
    echo The JAVA_HOME environment variable is not defined correctly
    echo This environment variable is needed to run this program
    exit 1
fi

#
# TODO: Should we preserve user CLASSPATH and PYTHONPATH or reset?
#
if [ -z "$CLASSPATH" ]; then
    CLASSPATH=$ITCAM_CONFIGHOME/bin/jython/jython.jar:$ITCAM_CONFIGHOME/bin/lib/installbundle.jar
else
    CLASSPATH=$CLASSPATH:$ITCAM_CONFIGHOME/bin/jython/jython.jar:$ITCAM_CONFIGHOME/bin/lib/installbundle.jar
fi
if [ -z "$PYTHONPATH" ]; then
    PYTHONPATH=$ITCAM_CONFIGHOME/bin/lib
else
    PYTHONPATH=$PYTHONPATH:$ITCAM_CONFIGHOME/bin/lib
fi
PYTHONPATH=$PYTHONPATH:$ITCAM_CONFIGHOME/bin/lib
ITCAM_LOG=$ITCAM_CONFIGHOME/data/config.log

JAVA_CMD=$JAVA_HOME/bin/java
if [ "$(uname)" = "AIX" ]
then
 JAVA_OPTS="-Xms256m -Xmx256m"
else 
 JAVA_OPTS=
fi
JAVA_PROPS="-Dpython.path=$PYTHONPATH "

echo
#echo Launching ITCAM configuration...
echo
#echo JAVA_HOME:        $JAVA_HOME
#echo
#echo ITCAM_CONFIGHOME: $ITCAM_CONFIGHOME
#echo
#echo CLASSPATH:        $CLASSPATH
#echo
#echo PYTHONPATH:       $PYTHONPATH
#echo
#echo Log file:         $ITCAM_LOG

# Execute the DC configurator
"$JAVA_CMD" \
  -classpath "$CLASSPATH" $JAVA_OPTS $JAVA_PROPS \
  org.python.util.jython \
  "$ITCAM_CONFIGHOME"/bin/deletetemplatexd.py $*
  
#get return code from the DC configurator execution
rc=$?

if [ -t 0 ]; then
    # jython turns echo off
    stty echo
fi

#return the exit code 
exit $rc
