#####################################################################
# Licensed Materials - Property of IBM
#
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
#
# This module delete itcam template.  The template is used for
# dynamic clusters.  
#
#####################################################################

import os, sys, traceback

# Eliminate need to prefix import modules with "lib."
sys.path.append("lib")

import createPCL_config as createPCL
import createPDL
import resourcebundle
import debugUtil
import checkversion
import utils
import get_wasroot
import deletetemplate
import debugUtil as log

from ConfigurationException import ConfigurationException
from resourcebundle import getmsg as getmsg
from constants import constants
from debugUtil import logDebug as debug
from ConsoleWriter import ConsoleWriter 
from PrintWrapper import PrintWrapper 


product = "ITCAM/AD 7.2"
fixlev = "HAAD720 AK00000"




try:

    #############
    # Initialize
    #############
    
    #
    # Instantiate the global constants class
    #
    common_constants = constants()
    
    #
    # Set the print wrapper for stdout and stderr.
    #
    PrintWrapper(common_constants.STDOUT)
    PrintWrapper(common_constants.STDERR)

    regular_process = True
    continue_config = True

    debug("Initialization of "+product+" "+fixlev)

    #
    # Initialize the resource bundle
    #
    resourcebundle.set_resourcebundle()

        
    ############################
    # Display a startup message
    ############################

    print
    print
    print "%s" % getmsg("FLOWERBOX_DELETETEMPLATEXD","Delete XD templates")
    print


    #
    # Create the data structures
    #
    pdl = createPDL.make_pdl(createPCL.make_pcl())
    debugUtil.logDumpPCL(pdl["PCL"][0], "Initial creation of PCL with %d entries" % len(pdl["PCL"][0]))
    debugUtil.logDumpPDL(pdl, "Initial creation of PDL with %d entries" % len(pdl), short=False)

    createPDL.proc_envvar(pdl)
    createPDL.proc_cliopt(pdl,sys.argv)
    debugUtil.logDumpPDL(pdl, "PDL after environment variables and command line arguments")

    # Perhaps all we are going to do is display the help text.
    if pdl["HELP"][0]:
        #debugUtil.set_logging(False)
        print "%s" % getmsg("HELP_MIGRATE")
        print
        sys.exit(0)
     
    # Default log and message file names
    if pdl["LOG"][0] is None:
        pdl["LOG"][0:2] = "deletetemplate-trace.log", "deletetemplatexd"
    if pdl["MESSAGE"][0] is None:
        pdl["MESSAGE"][0:2] = "deletetemplate-message.log", "deletetemplatexd"
    
    
    # Default and validate the config home.
    if pdl["CONFIGHOME"][0] is None:
        cwd = os.getcwd()
        if cwd.endswith("/bin"):
            config_home = cwd.replace("/bin","")
            pdl["CONFIGHOME"][0:2] = config_home, "deletetemplatexd.py"
        else:
            print "%s" % getmsg("CFG5028E", "ITCAM_CONFIG is not set and current directory is not a valid config_home")
            sys.exit(1)


    # Initialize the log utility and interrogate version.properties.
    log.set_logging(pdl["LOG"][0],pdl["MESSAGE"][0], pdl["CONFIGHOME"][0])
    if log.isLogBad():
        print "%s" % getmsg("CFG5027W", "Unable to open log file {0}, diagnostic output will be directed to the console.", [pdl["LOG"][0]])
    else:
        # Redirect sys.stdout
        if pdl["LOGCONSOLE"][0] is True:
            log_file = debugUtil.getLogFilename()
            log_dir = os.path.dirname(log_file)
            console_wtr = ConsoleWriter(log_dir + os.sep + "deletetemplate-console.log" )
            # Start with flower box printed above 
            console_wtr.log("%s" % getmsg("FLOWERBOX_DELETETEMPLATEXD","Delete XD templates"))
    
    
    print "%s" % getmsg("CFG5043I", "Log file name: {0}", [log.getLogFilename()])

    version = checkversion.get_current_version(pdl)
    assert version != -1, getmsg("CFG1070E", "Invalid version number  - {0}", [version])
    utils.setData(pdl, "VERSION", version, "migrate")
    
    utils.setData(pdl, "DELETETEMPLATE", True, "deletetemplatexd")
    #
    # Prompt to select the DMGR home.
    #
    get_wasroot.get_wasroot(pdl)
    
    #
    # Start displaying the XD templates and delete the selected ones.
    #
    deletetemplate.delete_template(pdl); 
    
    rc = pdl["RETURNCODE"][0]
    if (rc == 0):
        print "\n%s\n" % getmsg("CFG2058I","Deletion of XD template has successfully completed with return code {0}.",(pdl["RETURNCODE"][0],))
        
    else:
        print "\n%s\n" % getmsg("CFG2059E","Deletion of XD template did not complete successfully with return code {0}.",(pdl["RETURNCODE"][0],))
            
#former assert, unrecoverable error caused by environment settings or input parameters
#fixable by user, not a program error, error text goes to the console
except ConfigurationException, (instance):
    print "%s" % getmsg("CFG5026E", "Critical error in {0}", ["deletetemplatexd.py"])
    print "%s" % instance.parameter #print exception message
    sys.exit(1)

except AssertionError:

    print "%s" % getmsg("CFG5022E", "Internal program error detected in {0}", ["deletetemplatexd.py"])
    try:
        debugUtil.logException("Assertion error in deletetemplatexd.py")
        debugUtil.logDumpStack("Stack data for diagnostic purposes")
    except:
        _type, _val, _tb = sys.exc_info()
        try:
            print "\n%s\n" % getmsg("CFG1003E", "Logging routine failed, traceback follows")
        except:
            print "\nLogging routine failed, traceback follows\n"
        traceback.print_exception(_type,_val,_tb)
    sys.exit(1)

except KeyboardInterrupt:   #User entered terminate character in prompt
    print "%s" % getmsg("CFG2055I", "Exiting the ITCAM Data Collector for WebSphere XD Template Deletion Utility.")
    sys.exit(0)

except SystemExit:   # Generic exit (after an exception was already caught and handled)
    print "%s" % getmsg("CFG2055I", "Exiting the ITCAM Data Collector for WebSphere XD Template Deletion Utility.")
    sys.exit(1)

except:
    _exctype, _excval, _exctb = sys.exc_info()

    try:
        msg = getmsg("CFG2057E", "Unexpected exception detected in deletetemplatexd.py")
    except:
        msg = "Unexpected exception detected in deletetemplatexd.py"
    print "\n%s\n" % msg

    try:
        debugUtil.logException('Error in main routine')
    except:
        try:
            print "\n%s\n" % getmsg("CFG1003E", "Logging routine failed, traceback follows")
        except:
            print "\nLogging routine failed, traceback follows\n"
    traceback.print_exception(_exctype,_excval,_exctb)
    print
    sys.exit(1)
