@echo off

@REM #####################################################################         
@REM # Licensed Materials - Property of IBM
@REM #  
@REM # 5724-L62  
@REM #
@REM # (C) Copyright IBM Corp. 2012 All Rights Reserved.       
@REM #
@REM # US Government Users Restricted Rights - Use, duplication or
@REM # disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
@REM # 
@REM ##################################################################### 

setlocal enableextensions enabledelayedexpansion

set TMPDIR=

@REM Extract the options from the command line

set WORKDIR=%~dp0
set RESPONSE_FILE=
set SILENT=
set RESP_FILE_REQUIRED=

@REM Locate Java Home in the path if JAVA_HOME is not defined
set temp_path=%PATH%
:findJavaHome
if not defined JAVA_HOME if defined temp_path (
  for /F "tokens=1,* delims=;" %%D in ("%temp_path%") do (
    if not "%%~D" == "" if exist "%%~D\java.exe" if "%%~nD" == "bin" set JAVA_HOME="%%~dpD"
    set temp_path=%%E
  )
  goto findJavaHome
)

@REM Validate Java Home
set JAVA_EXE=%JAVA_HOME%\bin\java
if not exist "%JAVA_EXE%.exe" set JAVA_EXE=%JAVA_HOME%\jre\bin\java
if not exist "%JAVA_EXE%.exe" goto err_noJavaHome

:get_opts
    if "%~1" == "-h" goto usage
    if "%~1" == "-d" goto set_config_home
    if "%~1" == "-r" goto set_response_file
    goto main

:set_response_file
    if defined SILENT goto err_parmConflict
    set RESPONSE_FILE=%2
    set SILENT=y
    set RESP_FILE_REQUIRED=y
    goto got_optarg

:set_config_home
    if defined SILENT goto err_parmConflict
    set ITCAM_CONFIGHOME=%2
    set SILENT=y
    goto got_optarg

:got_optarg
    shift
    shift
    goto get_opts

:main

@REM Get the name of the file to extract
if "%~1" == "" goto err_commandline
set GDC_ARCHIVE=%1


@REM Set DC Version
set DC_VERSION=7.3.0.10.0

@REM Dequote RESPONSE_FILE
set dequote=###%RESPONSE_FILE%###
set dequote=%dequote:###"=%
set dequote=%dequote:"###=%
set dequote=%dequote:###=%
set RESPONSE_FILE=%dequote%

@REM Dequote ITCAM_CONFIGHOME
set dequote=###%ITCAM_CONFIGHOME%###
set dequote=%dequote:###"=%
set dequote=%dequote:"###=%
set dequote=%dequote:###=%
set ITCAM_CONFIGHOME=%dequote%


@REM Dequote GDC_ARCHIVE
set dequote=###%GDC_ARCHIVE%###
set dequote=%dequote:###"=%
set dequote=%dequote:"###=%
set dequote=%dequote:###=%
set GDC_ARCHIVE=%dequote%

@REM Remove quotes from JAVA_HOME
set dequote=###%JAVA_HOME%###
set dequote=%dequote:###"=%
set dequote=%dequote:"###=%
set dequote=%dequote:###=%
set JAVA_HOME=%dequote%

@REM Remove quotes from TEMP
set dequote=###%TEMP%###
set dequote=%dequote:###"=%
set dequote=%dequote:"###=%
set dequote=%dequote:###=%
set TEMP=%dequote%

@REM check that valid response file is specified in case of silent install with -r flag
if not "%RESP_FILE_REQUIRED%" == "" (
   if "%RESPONSE_FILE%" == "%%EP_RSP_FILE%%" goto err_skipExtraction
   if not exist "%RESPONSE_FILE%" goto err_invalidRspFile
)

@REM Create a temporary directory
if not defined TEMP goto err_noTemp
if not exist "%TEMP%" goto err_noTemp
set TMPDIR=%TEMP%.\tmpdir_%RANDOM%
mkdir "%TMPDIR%"

@REM The list of files to extract
set FILE_LIST=^
 %DC_VERSION%/bin/jython/jython.jar^
 %DC_VERSION%/bin/lib/installbundle.jar^
 %DC_VERSION%/bin/lib/gdc_extract.py^
 %DC_VERSION%/bin/lib/debugUtil.py^
 %DC_VERSION%/bin/lib/doprompt.py^
 %DC_VERSION%/bin/lib/platformUtil.py^
 %DC_VERSION%/bin/lib/resourcebundle.py^
 %DC_VERSION%/bin/lib/ConfigurationException.py^
 %DC_VERSION%/bin/lib/PrintWrapper.py^
 %DC_VERSION%/bin/lib/constants.py^
 %DC_VERSION%/bin/lib/utils.py
 
@REM Extract the archive to the temporary directory
set RC=0
"%JAVA_EXE%" -jar "%WORKDIR%\gdc_extract.jar" "%TMPDIR%" "%GDC_ARCHIVE%" %FILE_LIST%
set RC=%ERRORLEVEL%
if not "%RC%" == "0" goto err_java_bootstrap
if not exist "%TMPDIR%\%DC_VERSION%\bin\lib\gdc_extract.py" goto err_extractFailed

set C_PATH=%TMPDIR%/%DC_VERSION%/bin/jython/jython.jar;%TMPDIR%/%DC_VERSION%/bin/lib/installbundle.jar

@REM The file to which the value of ITCAM_CONFIGHOME will be written by the jython script
set TMPFILE=%TMPDIR%\%DC_VERSION%\configdir.dat

@REM Execute the extracted jython bootstrap
set JAVA_OPTS=
if defined RESPONSE_FILE set JAVA_OPTS=-Dresponse.file="%RESPONSE_FILE%"
if defined ITCAM_CONFIGHOME set JAVA_OPTS=-Dconfig.home="%ITCAM_CONFIGHOME%"
set JAVA_PROPS=-Ddc.version=%DC_VERSION% -Dfile.encoding=UTF-8 -Dpython.path="%TMPDIR%\%DC_VERSION%\bin;%TMPDIR%\%DC_VERSION%\bin\lib"
pushd "%TMPDIR%\%DC_VERSION%\bin"
"%JAVA_EXE%" %JAVA_OPTS% %JAVA_PROPS% -cp "%C_PATH%" org.python.util.jython "%TMPDIR%\%DC_VERSION%\bin\lib\gdc_extract.py" "%GDC_ARCHIVE%" "%TMPFILE%"
set RC=%ERRORLEVEL%
popd
if not "%RC%" == "0" goto err_jython_bootstrap

@REM Read configuration home from the output file
if not exist "%TMPFILE%" goto err_confighome
set /p ITCAM_CONFIGHOME=<"%TMPFILE%"
if not defined ITCAM_CONFIGHOME goto err_confighome
if not exist "%ITCAM_CONFIGHOME%" goto err_confighome

@REM Clean up temporary dir
rmdir /s /q "%TMPDIR%"

endlocal & set ITCAM_CONFIGHOME=%ITCAM_CONFIGHOME%\%DC_VERSION%& set RESPONSE_FILE=%RESPONSE_FILE%& set SILENT=%SILENT%


@REM Execute the DC configurator (unless silent mode)
if "%SILENT%" == "" (
    call "%ITCAM_CONFIGHOME%\bin\config.bat"
)
set RC=%ERRORLEVEL%
@REM if not "%RC%" == "0" if not "%SILENT%" == "y" goto err_configScript
goto done


:err_parmConflict
    set RC=1
    set msgid="CFG5061E"
    set default_msg="CFG5061E Parameter conflict, both -d and -r flags may not be used together"
    set print_usage=1
    goto print_error
    

:err_commandline
    set RC=1
    set msgid="CFG5058E"
    set default_msg="CFG5058E Specify the archive file to extract"
    set print_usage=1
    goto print_error
    

:err_noTemp
    set RC=1
    set msgid="CFG5059E"
    set default_msg="CFG5059E No temporary directory"
    goto print_error

:err_noJavaHome
    set RC=1
    echo ERROR: Unable to locate Java Home
    goto done

:err_java_bootstrap
    if "%RC%" == "" set RC=1
    set msgid="CFG5060E"
    set default_msg="CFG5060E Failure in Java bootstrap"
    goto print_error

:err_extractFailed
    set RC=1
    set msgid="CFG5057E"
    set default_msg="CFG5057E Failed to extract files from archive"
    goto print_error

:err_confighome
    set RC=1
    set msgid="CFG5062E"
    set default_msg="CFG5062E Failed to get the value of ITCAM_CONFIGHOME"
    goto print_error

:err_jython_bootstrap
    if "%RC%" == "" set RC=1
    set msgid="CFG5054E"
    set default_msg="CFG5054E Failure in Jython bootstrap"
    goto print_error

:err_configScript
    if "%RC%" == "" set RC=1
    set msgid="CFG5045E"
    set default_msg="CFG5045E The configuration process is being terminated due to errors listed above."
    goto print_error
    
:err_skipExtraction
    set RC=0
    set msgid="CFG5055I"
	set default_msg="CFG5055I Skipping DC extraction, EP_RSP_FILE property is not set"
	goto print_error
	
:err_invalidRspFile
    set RC=1
    set msgid="CFG5056E"
    set param="%RESPONSE_FILE%"
	set default_msg="CFG5056E Specified response file %RESPONSE_FILE% not found on the system"
	goto print_error

:usage
    set msgid="GDC_EXTRACTOR_USAGE"
    set default_msg="Usage: %~n0 [-h] [-d config_home^|-r response_file] archive_file archive_file: the archive file to extract, where -d config_home: the directory to extract the archive file to, -r response_file: the response file for silent installation, -h: display this help message"
    goto print_error

:print_error
	if not exist "%WORKDIR%\installbundle.jar" (
		echo %default_msg%
		goto done
	)
	"%JAVA_EXE%" -cp "%WORKDIR%\installbundle.jar" com.ibm.tivoli.itcam.install.common.resources.MessagePrinter "%msgid%" "%default_msg%" 
	goto done

:done
@REM The tmp dir will be there in some error scenarios
if defined TMPDIR if exist "%TMPDIR%" rmdir /s /q "%TMPDIR%"
exit /b %RC%
    
