######################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
   This class inherits from the WsadminTask, which executes a configdatacolltorc call to
   configure selected servers in a profile.  This class
   overwrites the method calls that are specifically for the config call.
   The actual call to the config jython call is done in the base class.
'''

fixlev = "38001" # Last APAR, defect or story to update this module

from WsadminTask import WsadminTask
from constants import constants
from debugUtil import logDebug as debug

import debugUtil as log
import utils
import os

class ConfigTask(WsadminTask):

    probe_serverindex_list = None
    wtype = -1
    common_constants = None


    def __init__(self, pdl_obj, wtype, inputlist_fname, logf, jython_call, probe_serverindex_list):
        '''
           This is the constructor routine.

           @param self - Reference to this object
           @param pdl - PDL reference
           @param wtype - Caller's type like config, unconfig,.....
           @param inputlist_fname - input list file name
           @param logf - log file name
           @param probe_serverindex_list - A list contains dictionary which the key is the server index and its value of 1 or 0, which
                 0 means to start the server and 1 means not to restart as it is dynamically refreshed
        '''

        debug("Entry __init__ %s" % fixlev)

        WsadminTask.__init__(self, pdl_obj, wtype, inputlist_fname, logf, jython_call)
        self.probe_serverindex_list = probe_serverindex_list
        self.wtype = wtype
        debug("Config task type is : %s" % (wtype))
        #
        # Instantiate common constants class
        #
        self.common_constants = constants()
        debug("Constants reference is: %s" % (self.common_constants))

        debug("Exit __init__")


    def set_inputlist_entries(self, inputlist_file, server_indexlist):
        '''
           This routine saves the following info in the config inoutlist:
           1st position = type that caused the config to be processed (config, reconfig, migrate)
                          the type is an integer number.
           2nd position = 1 if server needs to be notified
                          0 no need to notifiy server
           3rd position = Cell name
           4th position = Node name
           5th position = Server name
           6th position = input list location
           The entries are separated by a "^".

           @param inputlist_file - input list file fd
           @param server_indexlist - Index list of the selected servers for a given group
        '''

        probe = 0;

        debug("Entry set_inputlist_entries %s" % fixlev)

        debug("configtask set_inputlist_entries is called")
        runtimedir = os.path.normpath(os.path.join(self.pdl["CONFIGHOME"][0], "runtime"))
        for serveri in server_indexlist:
            inputPropertyFilePrefix = utils.getServerBasedFilePrefix(self.pdl, serveri)
            wascell = utils.getData(self.pdl, "WASCELL", serveri)
            wasnode = utils.getData(self.pdl, "WASNODE", serveri)
            wasserver = utils.getData(self.pdl, "SERVERS", serveri)
            debug("WAS cell: %s  WAS Node: %s  WAS Server: %s" % (wascell, wasnode, wasserver))
            #
            # Check the selected server index in the probe server dictionary and if there is a match,
            # make a note of it, so the first position is set appropriately.  This is an indicator
            # which determines if there is a need to issue the mbean call for dynamic reconfig.
            #
            if self.probe_serverindex_list != None:
                for serverpi_dict in range(len(self.probe_serverindex_list)):
                    serverdict_entry = self.probe_serverindex_list[serverpi_dict]
                    if serverdict_entry.has_key(serveri) == True:
                        probe = serverdict_entry[serveri]
                        break

            #
            # Save the input list entries.  The entries are separated by a "^".
            #
            inputPropertyFile = str(self.wtype) + "^" + str(probe) + "^" + wascell + "^" + wasnode + "^" + wasserver + "^" + os.path.normpath(os.path.join(runtimedir, inputPropertyFilePrefix + "input.properties"))
            debug("inputPropertyFile: " + inputPropertyFile)
            inputlist_file.write(inputPropertyFile)
            inputlist_file.write("\n")

        debug("Exit set_inputlist_entries")
