#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
#
# This class replaces the current sys.stdout. Each print line
# received is written to that sys.stdout and to a designated log file.
#
##################################################################### 
'''
   ConsoleWriter writes to the console(stdout) and to a 
   designated log file. 

'''

import os, sys

import utils
import codecs
import debugUtil as log
from   debugUtil import logDebug as debug
import shared_globals as shared 
 
class ConsoleWriter(object):     
    
    logfile = None
    file    = None
    log_enabled = False
    
    # original console based sys.stdout 
    sys_stdout = None
            
    def __init__(self, logpath): 
        '''
            Constructor
            @param logpath - path for console log
        '''    	 
        debug("Entry __init__")
        
        # 1. Open the console log 
        self.logfile = logpath 
        try: 
            utils.rename_archival_file(self.logfile)       
            self.file = codecs.open(self.logfile, 'w', 'utf-8')
            self.log_enabled = True
            shared.console_writer = self
            debug("  Console logging to: %s" % self.logfile)
        except:
            log.logException("  Cannot open console log: %s" % self.logfile )
            debug("  Console logging is disabled")
            # temp
            sys.stdout.print("Console logging is disabled")
            # Log failure 
            return  
            
        # 2. Save original reference to sys.stdout   
        self.sys_stdout = sys.stdout
        # 3. Make this class the new sys.stdout   
        sys.stdout = self  
            
        debug("Exit  __init__")
        
    def __del__(self):
    	'''
    	    Similar to java finalizer
    	'''
    	if self.sys_stdout is not None:
            sys.stdout = self.sys_stdout
        shared.console_writer = None
        if file is not None:
            self.file.flush()	
            self.file.close()
        log_enabled = False
        debug("ConsoleWriter has been deleted.")
   
    def write(self, line):
        ''' 
              Writes the line to both stdout and the console log 
        '''
        # 1. Write to the console
        self.sys_stdout.write(line)
        
        # 2. Write to the console log
        self.log(line)   
   
    def log(self, line):
       '''
           Log the line.  
           Can be called from within or outside this class.   
       '''
       if self.log_enabled is True: 
           try:    
               self.file.write(line)
           except:
               # Disable logging 
               self.log_enabled = False
               # Restore original sys_stdout  
               sys.stdout = self.sys_stdout    
               log.logException("  Write to console-log failed")
               debug("  Console output logging disabled; original sys.stdout restored")
                
    def flush(self): 
        # 1 stdout stream
        self.sys_stdout.flush()
    	# 2 console log  
    	if self.log_enabled: 
            self.file.flush()
 
    def close(self): 
     	# Close only the console log 
      	if file is not None:
            self.file.close()            
       
  
