######################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
   This class inherits from the WsadminTask, which executes a deletetemplatexd_wsadmin call to
   delete selected itcam templates for xd.
   The actual call to the config jython call is done in the base class.
'''

# Note: This code was created using FindServersTask as a model, but many comments
#       unfortunately were not updated and still mistakenly refer to FindServersTask.

fixlev = "38001" # Last APAR, defect or story to update this module

import os

import debugUtil as log
import utils

from WsadminTask import WsadminTask
from constants import constants
from resourcebundle import getmsg
from debugUtil import logDebug as debug

class DeleteTemplateTask(WsadminTask):

    wtype = -1
    common_constants = None
    delete_template = None


    def __init__(self, pdl_obj, wtype, inputlist_fname, logf, jython_call):
        '''
           This is the constructor routine.

           @param self - Reference to this object
           @param pdl - PDL reference
           @param wtype - Caller's type like config, unconfig,.....
           @param inputlist_fname - input list file name
           @param logf - log file name
           @parma data_file - Data file to save the discovered template names
        '''

        debug("Entry WsadminTask.__init__ %s" % fixlev)

        WsadminTask.__init__(self, pdl_obj, wtype, inputlist_fname, logf, jython_call)
        self.wtype = wtype
        debug("Config task type is : %s" % (wtype))
        self.delete_template = getmsg("DELETE_TEMPLATE", "delete template ")
        #
        # Instantiate common constants class
        #
        self.common_constants = constants()
        debug("Constants reference is: %s" % (self.common_constants))

        debug("Exit WsadminTask.__init__")


    def process(self, serverindex = None):
        '''
           Constructs the wsadmin command and starts executing it.   The discovered
           templates will be deleted.

           @param self - Reference to this object
           @param serverindex - Not used in this task, and it has a value of None
        '''

        debug("Entry WsadminTask.process %s" % fixlev)

        try:
            self.create_inputlist()
            command = self.constructCommand()
            wasprofile = utils.getData(self.pdl, "PROFILE")
            wasprofilehome = utils.getData(self.pdl, "PROFILEHOME")
            message = getmsg("CFG2060I", "Processing deleting the XD templates ")
            print "%s" % message
            #
            # Start wsadmin command.  The wsdadmin runs in a separate thread
            #
            self.start_wsadmin(command, wasprofilehome, wasprofile)

            debug("Exit WsadminTask.process")

        except Exception, errmsg:
            utils.setData(self.pdl, "RETURNCODE", 1, "ConfigTemplateTask")
            msg1 = "Error occurred processing deletetemplatexd_wsadmin call - %s" % (errmsg)
            log.logException(msg1)
            msgl =[]
            msgl.append(self.delete_template)
            msgl.append(errmsg)
            print getmsg("CFG2038E", "Error occurred while processing {0} call - {1}", msgl)
            raise errmsg


    def get_progress_file_location(self, server_indexlist = None):
        '''
           This routine gets the progress file location for this task.

           @param inputlist_file - input list file fd
           @param server_indexlist - Not used in this task, and it has a value of None
           @return - Progress file location
        '''

        debug("Entry WsadminTask.get_progress_file_location %s" % fixlev)

        wasprofile = utils.getData(self.pdl, "PROFILE")
        progressfile_prefix = wasprofile + "." + "deletexdtemplate" + "." + "progress"
        progressfile_loc = self.pdl["CONFIGHOME"][0] + os.sep + "data" + os.sep + progressfile_prefix

        debug("Exit WsadminTask.get_progress_file_location")

        return progressfile_loc


    def set_wsadmin_data(self, serveri = None):
        '''
           This routine sets the wsadmin data from PDL.  This is the default implementation which
           gets the data from the PDL.

           @param serveri - Not used in this task, and it has a value of None
        '''

        debug("Entry WsadminTask.set_wsadmin_data %s" % fixlev)

        self.wasProfileHome = utils.getData(self.pdl, "PROFILEHOME")
        debug("wasProfileHome: " + self.wasProfileHome)
        self.wroot = utils.getData(self.pdl, "WASROOT")
        debug("WASROOT: " + self.wroot)
        self.profile = utils.getData(self.pdl, "PROFILE")
        debug("profile: " + self.profile)
        self.conntype = utils.getData(self.pdl, "CONNTYPE")
        self.host = utils.getData(self.pdl, "HOST")
        self.port = utils.getDataString(self.pdl, "PORT")

        debug("Exit set_wsadmin_data ")


    def set_inputlist_entries(self, inputlist_file, server_indexlist = None):
        '''
           This routine sets the input list entry record for the template task.  It contains
           the data discovery file location separated by "^"

           @param inputlist_file - input list file fd
           @param server_indexlist - Not used in this task, and it has a value of None
        '''

        debug("Entry WsadminTask.set_inputlist_entries %s" % fixlev)

        discovered_entries = ""

        #
        # Get the list of selected itcam templates and write it to the inoutlist
        # file.  Each entry are separated by a "^".
        #
        templatelist = utils.getData(self.pdl, "TEMPLATELIST")
        debug("Discovered itcam templates: %s" % (templatelist))
        for drecord in templatelist:
            drecord = drecord.strip()
            discovered_entries+= drecord + "^"

        debug("Discovered records: " + discovered_entries)
        inputlist_file.write(discovered_entries)
        inputlist_file.write("\n")

        debug("Exit WsadminTask.set_inputlist_entries")

    def get_wasprofilehome(self):
        '''
           This routine gets the WAS profile home

           @return - WAS profile home
        '''

        debug("get_wasprofilehome of DeleteTemplateTask is called")
        return self.wasProfileHome


    def get_wasroot(self):
        '''
           This routine gets WAS root.

           @return - WAS root
        '''

        debug("get_wasroot of DeleteTemplateTask is called")
        return self.wroot


    def get_profile_name(self):
        '''
           This routine gets WAS profile name

           @return - WAS profile name
        '''

        debug("get_profile_name of DeleteTemplateTask is called")
        return self.profile


    def get_washost(self):
        '''
           This routine gets WAS host name.

           @return - WAS host name
        '''

        debug("get_washost of DeleteTemplateTask is called")
        return self.host


    def get_wasport(self):
        '''
           This routine gets WAS port.

           @return - WAS port number
        '''

        debug("get_wasport of DeleteTemplateTask is called")
        return self.port


    def get_wasconntype(self):
        '''
           This routine gets WAS connection type.

           @return - WAS password
        '''

        debug("get_wasconntype of WsadminTask is called")
        return self.conntype


    def get_logfile_loc(self, serveri = None):
        '''
           This routine gets log file location

           @param serveri - Not used in this task, and it has a value of None
           @return - Log file location
        '''

        debug("get_logfile_loc of configtemplatexd_wsadmin is called")
        datadir = self.pdl["CONFIGHOME"][0] + os.sep + "data"
        wasprofile = utils.getData(self.pdl, "PROFILE")
        logfile_loc  = datadir + os.sep + wasprofile + "." + self.log_file_name

        return logfile_loc
