#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
#
# This class replaces the current sys.stdout. Each print line
# received is written to that sys.stdout and to a designated log file.
#
##################################################################### 
'''
   This class is a wrapper for the print statement.  Every print statement
   will use the java System.out or System.err to print the message.

'''

import sys
import java.lang.System as System

from constants import constants
 
class PrintWrapper(object):     
    
    
    sys_stdout = None
    sys_stderr = None
    std_type = None
    printstream = None
            
            
    def __init__(self, stdtype): 
        '''
            Constructor
            
            @param stdtype - what type of print it is, stdout or stderr
            
        '''    
        
        #
        # Instantiate common constants class
        #
        self.common_constants = constants()     
        
        self.std_type = stdtype
        #  
        # Save original reference to sys.stdout/sys.stderr based on the type.
        #  
        if (stdtype == self.common_constants.STDOUT):
            self.sys_stdout = sys.stdout
            sys.stdout = self
            self.printstream = System.out
        else:
            self.sys_stderr = sys.stderr   
            sys.stderr = self 
            self.printstream = System.err
        
        
        
    def __del__(self):
        '''
            Similar to java finalizer
        '''
        
        if self.sys_stdout is not None:
            sys.stdout = self.sys_stdout
        elif self.sys_stderr is not None:
            sys.stderr = self.sys_stderr    
        
   
    def write(self, line):
        ''' 
              Writes the line stdout/stderr.
              
              @param line - Line to be printed
        '''
        
        if self.sys_stdout is not None:
            self.printstream.print(line)
        elif self.sys_stderr is not None:
            self.printstream.print(line) 
        
        
   
    def flush(self): 
        '''
           Flushes the stream.
        '''
        
        if self.sys_stdout is not None:
            self.printstream.flush()
        elif self.sys_stderr is not None:
            self.printstream.flush()       
        
 
    def close(self): 
        '''
           Closes the stream.
        '''
        
        if self.sys_stdout is not None:
            self.printstream.close()
        elif self.sys_stderr is not None:
            self.printstream.close()
