#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved. 
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
   This class handles the progress processing.  This calss could be instantiated in
   2 ways.  One is to pass an input list, which its forst record contains
   the progress file location.  This progress file is shared between the main thread
   and the jython program that was deployed by wsadmin.  This file contains the progress
   information for the main thrread to display to the console.
   The second way of instantiation is using this object within the main thread.  In this
   case the progress file name is passed to it for processing.
'''

import os
import codecs
import debugUtil as log
import resourcebundle as rmsg

from constants import constants
from debugUtil import logDebug as debug        


class Progress:
    
    common_constants = None
    inputlist_file_loc = None
    progress_file = None
    ptype = -1
    
        
    
    def __init__(self, ptype, inputlist_file_loc = None, progress_file_loc = None):
        '''
           This is the constructor routine.
           
           @param inputlist_file_loc - input list file location
           @param progress_file_loc - progress file li=ocation
           @param ptype - Process type like config, unconfig,...
                
        '''
        
        debug("Entry __init__")
        
        #
        # Instantiate common constants class
        #
        self.common_constants = constants()
        log.logDebug("Constants reference is: %s" % (self.common_constants))
        self.ptype = ptype
        log.logDebug("Process type is - %s" % (self.ptype))
        #
        # If progress file location is specified, then the file will be open, notice
        # that input list location and progress file location are mutually exclusive.
        #
        if (progress_file_loc != None):
            self.progress_file = codecs.open(progress_file_loc, mode='w', encoding='utf-8')
        else:    
            self.inputlist_file_loc = inputlist_file_loc
            debug("input list file location: %s" % (self.inputlist_file_loc))
            
        debug("Exit __init__")
        
        
    def open_progressfile(self):
        '''
           Reads the first record from the input list and gets the WAS full name
           and the progress file location.
           
        '''
        
        ptype = "None"
        
        debug("Entry open_progressfile")
        
        try:
            #
            # set the text base on the type.  This module is common to all the processing
            # types.
            #
            if (self.ptype == self.common_constants.CONFIG):
                ptype = self.common_constants.COMMON_CONFIG_LOG_TEXT
            elif (self.ptype == self.common_constants.UNCONFIG):
                ptype = self.common_constants.COMMON_UNCONFIG_LOG_TEXT    
            elif (self.ptype == self.common_constants.MIGRATE):
                ptype = self.common_constants.COMMON_MIGRATE_LOG_TEXT   
            elif (self.ptype == self.common_constants.FINDSERVERS):
                ptype = self.common_constants.COMMON_FINDSERVERS_LOG_TEXT   
            else:
                ptype = "None"             
            #
            # Open the input list file.
            #    
            inputlist_file = codecs.open(self.inputlist_file_loc, mode='r', encoding='utf-8')
            #
            # The first record is the progress file location, so it is read and the
            # the file is open for  writing.
            #
            progress_file_loc = inputlist_file.readline().strip()
            log.logDebug("Progress file: %s " % (progress_file_loc))
            self.progress_file = codecs.open(progress_file_loc, mode='w', encoding='utf-8')
            log.logDebug("Progress file location opened successfully  - %s " % (progress_file_loc))
            
        except Exception, errmsg:
            log.logException("%s - %s" % (ptype, errmsg))
            raise RuntimeError
        
        debug("Exit open_progressfile - %s" % (inputlist_file))
        
        return inputlist_file


    def write_progress_start(self, entity_name = ""):
        '''
           Writes the start message and repeat message to the progress file.
           
           @param entity_name - entity name
       '''
        
        start_msg = "None"
        
        debug("Entry write_progress_start - %s" % (entity_name))
        
        debug("Start record: %s" % (entity_name))
        #
        # Based on the process type, set its corresponding string, and set the first record
        # in the progress file.
        #
        log.logDebug("Caller type is - %s" % (self.ptype))  
        msgl =[]
        if (self.ptype == self.common_constants.MBEAN):
            msgl.append(entity_name)
            start_msg = rmsg.getmsg("START_MBEANCALL", "Start mbean call for {0}", msgl)
        elif (self.ptype == self.common_constants.CONFIG):
            msgl.append(entity_name)
            start_msg = rmsg.getmsg("START_CONFIGURING", "Start configuring {0}", msgl) 
        elif (self.ptype == self.common_constants.UNCONFIG):
            msgl.append(entity_name)
            start_msg = rmsg.getmsg("START_UNCONFIGURING", "Start unconfiguring {0}", msgl)   
        elif (self.ptype == self.common_constants.MIGRATE):
            msgl.append(entity_name)
            start_msg = rmsg.getmsg("START_MIGRATING", "Start migrating {0}", msgl) 
        elif (self.ptype == self.common_constants.MSUNCONFIG):
            msgl.append(entity_name)
            start_msg = rmsg.getmsg("START_MSUNCONFIG", "Start MS unconfiguration for {0}", msgl) 
        elif (self.ptype == self.common_constants.FINDSERVERS):
            msgl.append(entity_name)
            start_msg = rmsg.getmsg("START_FINDSERVERS", "Start finding servers for profile {0}", msgl)  
        elif (self.ptype == self.common_constants.CONFIGTEMPLATE):
            msgl.append(entity_name)
            start_msg = rmsg.getmsg("START_XDTEMPLATE", "Start creating XD template {0}", msgl)  
        elif (self.ptype == self.common_constants.DISCOVERTEMPLATE):
            start_msg = rmsg.getmsg("START_DISCOVER_XDTEMPLATE", "Start discovering XD templates")  
        elif (self.ptype == self.common_constants.DELETETEMPLATE):
            start_msg = rmsg.getmsg("START_DELETE_XDTEMPLATE", "Start deleting XD templates")                
        else:
            start_msg = "None"          
        start_msg = start_msg + self.common_constants.SEPARATOR_FOR_REPEAT_MESSAGE + self.common_constants.NOINDENT 
        self.progress_file.write(start_msg)
        self.progress_file.write(os.linesep)
        #
        # Write the repeat message to the progress file.  
        #
        msgl =[]
        msgl.append(entity_name)
        msg = rmsg.getmsg("PROCESSING", "Processing {0}...", msgl)
        msg = msg + self.common_constants.SEPARATOR_FOR_REPEAT_MESSAGE + self.common_constants.REPEAT
        self.progress_file.write(msg)
        self.progress_file.write(os.linesep)
        self.progress_file.flush()  

        debug("Exit write_progress_start")

    def write_progress(self, msg):
        '''
           Writes a message to the progress file.
           
           @param msg - A message
           @param progress_file - Progress file fd
        '''
        
        debug("Entry write_progress - %s" % (msg))
        log.logDebug("Progress record: %s" % (msg))
        self.progress_file.write(msg)
        self.progress_file.write(os.linesep)
        self.progress_file.flush()  
        
        debug("Exit write_progress")
        
        
    def write_progress_noindent(self, msg):
        '''
           Writes a message to the progress file with no indentation.
           
           @param msg - A message
           @param progress_file - Progress file fd
        '''
        
        debug("Entry write_progress_noindent - %s" % (msg))
        
        debug("Progress record: %s" % (msg))
        msg = msg + self.common_constants.SEPARATOR_FOR_REPEAT_MESSAGE + self.common_constants.NOINDENT 
        self.progress_file.write(msg)
        self.progress_file.write(os.linesep)
        self.progress_file.flush()      
        
        debug("Exit write_progress_noindent")
        
        
    def write_progress_repeat(self, msg):
        '''
           Writes a message to the progress file with an indication that the
           message will be displayed on the console repeatedly.
           
           @param msg - A message
           @param progress_file - Progress file fd
        '''
        
        debug("Entry write_progress_repeat - %s" % (msg))
        
        debug("Progress record: %s" % (msg))
        msg = msg + self.common_constants.SEPARATOR_FOR_REPEAT_MESSAGE + self.common_constants.REPEAT 
        log.logDebug("Progress repeat record: %s" % (msg))
        self.progress_file.write(msg)
        self.progress_file.write(os.linesep)
        self.progress_file.flush()  
        
        debug("Exit write_progress_repeat")
          
    
    def write_progress_finish(self, entity_name, ftype):
        '''
           Writes the finished message to the progress file.
           
           @param entity_name - entity name
           @param ftype - 0 means successful, 1 means unsuccessful
       '''
        
        finish_msg = "None"
        stat = "None"
        
        debug("Entry write_progress_finish - %s - %s" % (entity_name, ftype))
        
        debug("Finish record: %s" % (entity_name))
        #
        # Based on the process type, set its corresponding string, and set the first record
        # in the progress file.
        #
        if (ftype == self.common_constants.SUCCESSFUL):
            stat = rmsg.getmsg("SUCCESSFULLY", "successfully")
        else:
            stat = rmsg.getmsg("UNSUCCESSFULLY", "unsuccessfully")    
        debug("Caller type is - %s" % (self.ptype))  
        msgl =[]
        if (self.ptype == self.common_constants.MBEAN):
            msgl.append(entity_name)
            msgl.append(stat)
            finish_msg = rmsg.getmsg("FINISH_MBEANCALL", "Finished mbean call for {0} {1}", msgl)
        elif (self.ptype == self.common_constants.CONFIG):
            msgl.append(entity_name)
            msgl.append(stat)
            finish_msg = rmsg.getmsg("FINISH_CONFIGURING", "Finished Configuring {0} {1}", msgl) 
        elif (self.ptype == self.common_constants.UNCONFIG):
            msgl.append(entity_name)
            msgl.append(stat)
            finish_msg = rmsg.getmsg("FINISH_UNCONFIGURING", "Finished Unconfiguring {0} {1}", msgl)   
        elif (self.ptype == self.common_constants.MIGRATE):
            msgl.append(entity_name)
            msgl.append(stat)
            finish_msg = rmsg.getmsg("FINISH_MIGRATING", "Finished Migrating {0} {1}", msgl) 
        elif (self.ptype == self.common_constants.MSUNCONFIG):
            msgl.append(entity_name)
            finish_msg = rmsg.getmsg("FINISH_MSUNCONFIG", "Finished MS unconfiguration for {0}", msgl)
        elif (self.ptype == self.common_constants.FINDSERVERS):
            msgl.append(entity_name)
            finish_msg = rmsg.getmsg("FINISH_FINDSERVERS", "Finished finding servers for profile {0} {1}", msgl) 
        elif (self.ptype == self.common_constants.CONFIGTEMPLATE):
            msgl.append(entity_name)
            finish_msg = rmsg.getmsg("FINISH_XDTEMPLATE", "Finished creating XD template - {0}", msgl)  
        elif (self.ptype == self.common_constants.DISCOVERTEMPLATE):
            finish_msg = rmsg.getmsg("FINISH_DISCOVER_XDTEMPLATE", "Finished discovering XD templates")  
        elif (self.ptype == self.common_constants.DELETETEMPLATE):
            finish_msg = rmsg.getmsg("FINISH_DELETE_XDTEMPLATE", "Finished deleting XD templates")         
        else:
            finish_msg = "None"             
        finish_msg = finish_msg + self.common_constants.SEPARATOR_FOR_REPEAT_MESSAGE + self.common_constants.NOINDENT 
        self.progress_file.write(finish_msg)
        self.progress_file.write(os.linesep)
        self.progress_file.flush()
    
        debug("Exit write_progress_finish")
    
    
    def close_file(self):
        '''
           Close the progress file
        '''
        
        self.progress_file.close()   

