#####################################################################     
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

'''
    Invoke configDataCollector.py 
'''
#####################################################################
# 16Jan2012 MEC 15905 If command fails, scan log looking for success messages.
# 
#####################################################################

import debugUtil as log

from constants import constants
from ConfigTask import ConfigTask
from debugUtil import logDebug as debug
import os
import wlpConfigDC


def configDataCollector(pdl, probe_serverindex_list = None):
    '''
       Creates configtask object which executes the wsadmin for the configuring the servers.
       
       @param pdl - PDL reference
       @param probe_serverindex_list - A list of server indexes which are needed to be notified because there
                                       are probe changes.
    '''       
        
    debug("Entry configDataCollector ")
    
    try:
        ctype = -1
            
        #
        # Instantiate common constants class
        #
        common_constants = constants()    
        
        reconfigflag = pdl["RECONFIG"][0]
        migrateflag = pdl["MIGRATE"][0]
        if (reconfigflag):  
            debug("Processing reconfig")
            ctype = common_constants.RECONFIG
        elif (migrateflag):
            debug("Processing migrate")
            ctype = common_constants.MIGRATE    
        else:
            debug("Processing config")
            ctype = common_constants.CONFIG              
        config_task = ConfigTask(pdl, ctype, "config_inputlist", "configdatacollector.log", "configDataCollector.py", probe_serverindex_list)
        config_task.process()
        debug("Exit configDataCollector")
        
    except Exception, errmsg: 
        msg1 = "Error occurred processing configuration - %s" % (errmsg)  
        log.logException(msg1)
        log.logDebug("configDataCollector exit")
        raise errmsg
        
          
        

#####################################################################
# Unit Testing
#####################################################################

if __name__ == '__main__':

    import createPCL_config as createPCL
    import createPDL
    import debugUtil

    debugUtil.set_logging()
    debug("Unit Test for configDC")

    pdl = createPDL.make_pdl(createPCL.make_pcl())

    configDataCollector(pdl)



