#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
    configDataCollector.py
    usage wsdamin.sh -f configDataCollector.py filename
'''
import sys
import re
import os
import time
import codecs
import traceback
import shutil

from java.util import Map
from java.util import StringTokenizer
from java.util import Vector
from time import strftime
from java.lang import System
from java.io   import File

# On Windows platforms the python.path property has problems.
# Make an explict addition to the the import path.
sys.path.append("lib")

import platformUtil
import debugUtil as log
import resourcebundle
import mbeancall

from constants import constants
from Progress import Progress
from debugUtil import logDebug as debug
##Global variables
VERSIONSTRING = "ITCAM 7.3 AK0000"
SCRIPT = "configDataCollector"
SCRIPTNAME = SCRIPT + ".py"

common_constants = None

# This is used for properties and variable substitution entries
timestamp = strftime("%Y-%m-%d %H:%M:%S")
tmp = ("%s %s" % (VERSIONSTRING, timestamp))
description_entry = ["description", tmp]

nodeName = ""
serverName = ""
serverid = ""

javaProcServant = ""
javaProcControl = ""

# These are set and checked in getProcessDefs
jvmId = ""
servantJvmId = ""
controlJvmId = ""

# This is set after parameters are validated
runtime_path = ""

variableMap = ""
variableMapArray = {}
variableMapServerid = ""
serverVariableEntries = []
maximumHeapSize = ""

product = ""
control_classpath = ""
genericargs = ""
sharedLib = ""
serverVersion = ""
pipusc = ""
pipusc_value = ""
server_runtime = ""
itcamdc_home = ""
server_classpath = ""
server_bootclasspath = ""
server_pmiservice_statisticSet = ""
smf = "No"
containertimeouts = ""
doSynchronize = 1
server_amhome_flag = 0
JVMSecurityPolicy_flag = 0
ecam_flag = 0
control_environments = []
control_properties = []

pmiService_flag = 0
extraRMI_flag = 0

quiet_flag = 0
failed_flag = 0

probe = 0
wascell = "None"
wasnode = "None"
wasserver = "None"
current_inputproperties = "None"
summary = {}
config_type = -1
old_am_home = ""
isStandalone = 0
configComplete = 0
###End of global variables####


###Initialize#####
# Catch in case jiti.jar is not provided
try:
    from com.cyanea.installer.zos import SetupUtil
    jiti_flag = 1
except Exception:
    jiti_flag = 0
    print " \n*Warning*  Unable to load SetupUtil for jiti unconfigure\n "


# Get policy cleanup property for unconfigDataCollector only

  # The server.policy file is common within a node, so we do not want to
  # unconfigure it unless that choice has been approved earlier from
  # the unconfig.sh script.

if (SCRIPT == "unconfigDataCollector" ):
    if (System.getProperty("itcam.cleanuppolicy") == "" ):
        serverPolicy_flag = 0
    else:
        serverPolicy_flag = 1
        print "itcam.cleanuppolicy option set"
else:
    serverPolicy_flag=0

  # These values are overridden by system property entries

#######End initialize#########

def logDebug(msg):
    print msg

def is_zOS(platform):
    if (platform == "ZOS"): return 1
    return 0

def to2DList(str):
    '''
        returns result of AdminConfig call as a list of lists
        like in AdminConfig.show(<PMIServiceId> )
    '''
    result = []
    item_list = toList(str)
    logDebug("item_list=%s" % item_list)
    for item in item_list:
        pair = wsadminToList(item)
        logDebug("pair values: %s" % pair)
        result.append(pair)
    return result

def toList(str):
    if (str.strip() == ""): return []
    lineSeparator = System.getProperty('line.separator')
    _list = str.split(lineSeparator)
    return _list

def regexp(pattern, string, flags=0):
        if(re.compile(pattern, flags).search(string)==None): return 0
        else: return 1

def regexpn(pattern, string, flags=0):
        r = re.compile(pattern, flags).subn("", string)
        return r[1]

def join (lst, separator=" "):
    strg = ""
    for element in lst:
        strg += str(element) +separator
    strg = strg[0:len(strg)-len(separator)]
    return strg


def wsadminToList(inStr):
    outList=[]
    if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
        tmpList = inStr[1:-1].split() #splits space-separated lists,
    else:
        tmpList = inStr.split("\n")   #splits for Windows or Linux
    for item in tmpList:
        item = item.rstrip();         #removes any Windows "\r"
        if (len(item)>0):
            outList.append(item)
    return outList

def split_path(path_str):
    '''
        splits path by unix or windows style separator
        replacement for split(";:") in jacl

    '''
    rex = re.compile("[:]")
    if platformUtil.isWindows():
        rex = re.compile("[;]")
    return rex.split(path_str)


def isUnderAmHome(path, am_home):
    '''
        check if a path is under a directory
    '''
    logDebug("isUnderAmHome entry %s %s" % (path, am_home))
    isChildDir = 0
    
    if am_home != None and am_home != "":
        try:
            dc_home = os.path.dirname(am_home)
            canonical_path = File(path).getCanonicalPath()
            canonical_dc_home = File(dc_home).getCanonicalPath()
            logDebug("checking %s is under %s" % (canonical_path, canonical_dc_home))
            if canonical_path.startswith(canonical_dc_home): isChildDir = 1
        except Exception, errmsg:
            logDebug("Error in isUnderAmHome %s" % errmsg)
            
        logDebug("isUnderAmHome exit %s" % isChildDir)
    return isChildDir

errorCode = ""
errorInfo = ""

##############################################################
#
#  Procedure  checkObsolete
#
#  Check for obsolete path values, that is ones that contain
#
#   - /itcam/WebSphere/DC/
#   - /itcamdc/lib/wpsaspect.jar
#   - ${ITCAM61HOME}
#
##############################################################

def checkObsolete ( path ):

        global old_am_home
        
        obsolete_flag = 0
        if isUnderAmHome(path, old_am_home) == 1:
            logDebug("marking '%s' as obsolete, path is under old am.home " % path)
            obsolete_flag = 1
        elif (path.find("/itcam/WebSphere/DC/") != -1):
            obsolete_flag = 1
        elif (path.find("/itcamdc/lib/wpsaspect.jar") != -1):
            obsolete_flag = 1
        elif (path.find("${ITCAM61HOME}") != -1):
            obsolete_flag = 1
                
        logDebug("checkObsolete exit = %s" % str(obsolete_flag))
        return obsolete_flag


def attrAsList(str):
    rex=re.compile("\[(.*)\]")
    attrValue = rex.split(str)[1]
    return attrValue.split()


def getSubstitutionEntryValue(var_name):
    '''
        get variable value
    '''
    global AdminConfig
    global serverid
    logDebug("getSubstitutionEntryValue entry %s" % var_name)
    varValue = None
    try:
        variables = toList(AdminConfig.list("VariableSubstitutionEntry", serverid))
        for variable in variables:
            getVarName = AdminConfig.showAttribute(variable, "symbolicName")
            logDebug("varName %s" % (getVarName))
            if getVarName == var_name:
                varValue = AdminConfig.showAttribute(variable, "value" )
    except:
        logDebug("Error in getSubstitutionEntryValue")
    logDebug("varName %s is set to %s" % (getVarName, varValue))
    return varValue

def get_am_home(propId):
    global AdminConfig
    
    logDebug("get_am_home entry")
    prop_value = AdminConfig.showAttribute(propId, "value" )
    if prop_value != None and prop_value.find("${ITCAMDCHOME}") > -1:
        itcamdchome = getSubstitutionEntryValue("ITCAMDCHOME")
        if itcamdchome != None:
            logDebug("replacing value with %s" % itcamdchome) 
            prop_value = prop_value.replace("${ITCAMDCHOME}", itcamdchome)
    logDebug("get_am_home exit %s" % prop_value)    
    return prop_value

def getJVMMode():
    global AdminTask

    logDebug(" getJVMMode()")    
    logDebug("nodeName="+nodeName+", serverName="+serverName)
    
    serverSDK = AdminTask.getServerSDK('-nodeName ' + nodeName + ' -serverName ' + serverName)
    serverBitness = serverSDK[-5:-3]
     
    logDebug("serverBitness=" + serverBitness)
    logDebug(" getJVMMode() exit")    

    return serverBitness


####
# restore input.properties file from a backup if configuration failed
# remove backup file if configuration was successful
#
####
def restore_input_properties():
    global current_inputproperties
    global config_type
    global configComplete
    
    logDebug("restore_input_properties entry configComplete=%s configType=%s current_inputproperties=%s" % (str(configComplete), str(config_type), current_inputproperties))
    try:
        if (current_inputproperties != "None" and
           (config_type == str(common_constants.CONFIG) or config_type == str(common_constants.MIGRATE) or config_type == str(common_constants.RECONFIG))):
            current_inputproperties_org = current_inputproperties + "_org"
            if configComplete == 0:  #dataCollector wasn't configured, no AdminConfig.save() executed, restoring to saved input.properties
                if os.path.exists(current_inputproperties_org):
                    shutil.copy(current_inputproperties_org, current_inputproperties)
                    os.remove(current_inputproperties_org)
                    log.logDebug("restored original input.properties file %s" % (current_inputproperties_org))
                else:
                    os.remove(current_inputproperties)
                    log.logDebug("input.properties file was removed - %s" % (current_inputproperties))
            else:  #configuration complete, failure  after AdminConfig.save()
                if os.path.exists(current_inputproperties_org):
                    os.remove(current_inputproperties_org)
                    log.logDebug("AdminConfig.save() executed, removing stored input.properties file %s" % current_inputproperties_org)
                else:
                    log.logDebug("no backup file found %s" % current_inputproperties_org)
    except Exception, remerrmsg:
        print "input.properties file could not be restores - %s - %s" % (current_inputproperties, remerrmsg)
    logDebug("restore_input_properties exit")

# config
##############################################################
#
#  Procedure configEnvs
#  Arguments config id, new environments
#
##############################################################
#updateEnvironments list of dictionaries
def configEnvs ( javaProcId, updateEnvironments ):
        global AdminConfig
        global nodeName, serverName, serverid
        global description_entry, old_am_home


        if (platformUtil.IS_OS400_OS() and config_type == str(common_constants.RECONFIG)):
            logDebug("nodeName="+nodeName+", serverName="+serverName)
            logDebug("calling getJVMMode()")
            bitness = getJVMMode()
            
        logDebug("configEnvs %s %s " %(javaProcId,updateEnvironments))
        print "Create/Modify java process with %s environment variables" % str(updateEnvironments)

        logDebug("%s updating environment entries " % str(len(updateEnvironments)))
        for ue in updateEnvironments:
            logDebug("item=%s" % ue)
        # Read in the environment from the Java process

        logDebug("Reading in name/value pairs from existing environment")
        '''
        '[PATH(cells/x336r1s39-vm02Node01Cell/nodes/x336r1s39-vm02Node02/servers/server1
        |server.xml#Property_1319649805390) _JVM_THREAD_DUMP_BUFFER_SIZE(cells/x336r1s39
        -vm02Node01Cell/nodes/x336r1s39-vm02Node02/servers/server1|server.xml#Property_1
        319649805562) NLSPATH(cells/x336r1s39-vm02Node01Cell/nodes/x336r1s39-vm02Node02/
                              servers/server1|server.xml#Property_1324073901586)]'
        '''
        environs = toList(AdminConfig.showAttribute(javaProcId, "environment" ))
        logDebug("environs %s" % environs)
        if (len(environs) != 1):
                print " \n*Warning*  Unexpected %s environment entries for %s should be 1.\n " % (str(len(environs)), serverName)
                print "           Process will continue.  However, there may be a WAS configuration issue.\n "


        env_list = attrAsList(environs[0])
        environs = env_list
        env_dic = {}
        for envid in environs:
            logDebug("envid=%s" % envid)
            logDebug( "%s environment IDs read for %s" % (str(len(env_list)), serverName))
            if (envid.find('(') > -1 and envid.find(')') > -1):
                var_name = envid.split('(')[0]
                var_value = envid[len(var_name):]
                env_dic[var_name] = var_value
            envid.split('(')

        # Create the arrays from the update entries

        updates = {}
        hits = {}
        for ue in updateEnvironments:
            nm = ue[0][1]
            val = ue[1][1]
            hits[nm] = 0
            updates[nm] = val
            logDebug("%s=%s" % (nm, val))




        # Process the environment variables
        # Special handling for paths such as NLSPATH and LIBPATH

        logDebug("  Processing environment entries:")
        
        logDebug("am.home value is %s" %  old_am_home)
        
        for envid in environs:
            has_property = envid.find("#Property_")
            if (has_property == -1): continue
            enm = AdminConfig.showAttribute(envid, "name" )
            logDebug("Environment name = %s" % enm)
            if updates.get(enm, None) == None : continue
            hits[enm] = 1
            eval = AdminConfig.showAttribute(envid, "value" )
            logDebug("Environment value = %s" % eval)
            val = updates[enm]
            logDebug("Update value      = %s" % val)

            if (enm == "LIBPATH" and platformUtil.IS_OS400_OS() and config_type == str(common_constants.RECONFIG)):
                if bitness == "32":
                    val = "/lib:${ITCAMDCHOME}/toolkit/lib/os400-pase"
                else:
                    val = "/lib:${ITCAMDCHOME}/toolkit/lib/os400-pase64"
                    
                logDebug("Update value (overriden) = %s" % val)

            # Must merge the path updates
            if (enm == "NLSPATH" or enm == "LIBPATH" or enm == "LD_LIBRARY_PATH" or enm == "PATH" or enm == "SHLIB_PATH"):
                logDebug("Merging %s  with existing path list" % enm)
                paths = split_path(val)
                epaths = split_path(eval)
                newPaths = paths
                for epath in epaths:
                    # Drop any existing path that is in the new paths
                    for path in paths:
                        if (path == epath):
                            logDebug("%s present in existing path " % (epath))
                            epath = ""
                            break


                    if (epath == ""):  continue
                    # Drop any ITCAMDCHOME path, it is assumed to be ours
                    if (epath.find("${ITCAMDCHOME}") != -1):
                        logDebug("Existing %s is an old ITCAM entry and will be dropped" % epath)
                        continue

                    # Drop any paths from a previous release
                    if (checkObsolete(epath ) ):
                        print " \n*Warning*  Obsolete "+enm+" entry removed: "+epath+"\n "
                        print "           If this entry was not originally set by ITCAM,"
                        print "           you may wish to manually replace it.\n "
                        continue


                    logDebug("Existing %s entry will be retained in %s " % (epath, enm))
                    newPaths.append(epath)

                pathDelimiter = ":"
                if platformUtil.isWindows():
                    pathDelimiter = ";"
                newPaths = join( newPaths, pathDelimiter)
                newVal = newPaths

                # Update existing entry with merged paths
                # (We presume that the new value will never be empty)

                if (newVal == eval):
                    print "Environment variable already set: "+enm+" = "+eval
                else:
                    print "Updating environment variable "+nm+" to merged path: "+newVal
                    newEnv = [["name", enm], ["value", newVal], description_entry]
                    AdminConfig.modify(envid, newEnv )

                continue


            if (val == eval):
                print "Environment variable already set: "+enm+" = "+eval
            else:
                print "Updating environment variable "+nm+" from \""+eval+"\" to \""+val+"\""
                newEnv = [["name", enm], ["value", val], description_entry]
                AdminConfig.modify(envid, newEnv )




        logDebug(hits)
        for nm in updates.keys():
            if (hits[nm] > 0):
                logDebug("Value %s already processed, skipping..." % nm)
                continue
            logDebug("Found new name %s" % nm)
            val = updates[nm]
            print "Creating new environment "+nm+"="+val
            newEnv = [["name", nm], ["value", val], description_entry]
            AdminConfig.create("Property", javaProcId, newEnv, "environment" )


        logDebug(" configEnvs() exit")



def configCustomService ( newCustomService ):
        '''
            config
            Procedure configCustomService
            Arguments: server config id,
            newCustomService
        '''

        global AdminConfig
        global serverid


        logDebug("configCustomService() entry")
        for attr in newCustomService:
            logDebug("attr=%s" % attr)

        for attr in newCustomService:
            attrName = attr[0]
            attrValue = attr[1]
            if ( attrName == "classname" ):
                newClassName = attrValue
            elif ( attrName == "enable" ):
                newEnable = attrValue
            elif ( attrName == "classpath" ):
                newClassPath = attrValue
            elif ( attrName == "displayName" ):
                newServiceName = attrValue
            elif ( attrName == "description" ):
                newDesctiption = attrValue
            elif ( attrName == "default" ):
                logDebug("ignore "+attrName+"="+attrValue)



        print "Configure custom service "+newServiceName
        customServiceIds = AdminConfig.list("CustomService", serverid )
        if (customServiceIds != ""):
            customServiceIds_list = toList(customServiceIds)
            print "Custom Service "+customServiceIds+" exists"
            for tmpId in customServiceIds_list:
                oldClassName = AdminConfig.showAttribute(tmpId, "classname" )
                oldServiceName = AdminConfig.showAttribute(tmpId, "displayName" )
                oldEnable = AdminConfig.showAttribute(tmpId, "enable" )
                oldClassPath = AdminConfig.showAttribute(tmpId, "classpath" )
                if (oldClassName == newClassName):
                    print "Removing existing "+newServiceName+" custom service"
                    AdminConfig.remove(tmpId )
                    break
        AdminConfig.create("CustomService", serverid, newCustomService )
        logDebug("configCustomService() exit")



def unconfigSharedLib ( libraries, nodeName ):
    '''
        unconfig
        Procedure  unconfigSharedLib
        Arguments: libraries

        Note: This unconfigures the ITCAM/DC WPS shared library
            for the whole node.  This should be okay because
            there is only one WebSphere_Portal per node.
    '''

    logDebug("unconfigSharedLib(%s) entry" % libraries )
    libId = AdminConfig.getid("/Node:"+nodeName+"/Library:WPSlib/" )
    if (libId== ""):
        print "WPS not installed.  Skipping WPSlib classPath unconfiguration"
        logDebug("unconfigSharedLib() exit = false")
        return "false"
    logDebug("WPSlib ID = %s" % libId)
    nodeId = AdminConfig.getid("/Node:"+nodeName+"/" )
    logDebug("Node ID = %s" % nodeId)
    # Attributes for the new WPSlib Library object consist
    # of the old objects attributes with the classPath replaced

    currentPaths = AdminConfig.showAttribute(libId, "classPath" )
    newPaths = unmergePaths(currentPaths, libraries )
    classpathDelimiter = ":"
    if platformUtil.isWindows():
        classpathDelimiter = ";"
    if (newPaths == join( split_path(currentPaths), classpathDelimiter)  ):
        print "WPS shared library "+libraries+" already unconfigured"
    else:
        print "Removing ITCAM from WPSlib classPath"
        logDebug("Constructing new WPSlib attributes")
        attrs = to2DList(AdminConfig.show(libId ))
        newWPSlib = []
        for attr in attrs:
            if (attr[0] == "description"):
                logDebug("Skipping description attribute %s" % attr)
                continue
            if (attr[0] == "classPath"):
                # classPath normally uses semicolons for separator character
                newWPSlib.append(["classPath", join( newPaths.split(":" ), ";")  ])
                continue
            logDebug("Using existing attribute = %s, value = %s" % (attr[0], attr[1]))
            newWPSlib.append(attr)
        AdminConfig.remove(libId )
        libId = AdminConfig.create("Library", nodeId, newWPSlib )
        logDebug("New WPSlib ID = %s" % libId)

    logDebug("unconfigSharedLib() exit = true")
    return "true"


# config
##############################################################
#
#  Procedure  configSharedLib
#  Argument:  libraries
#
#  Because the classPath attribute is merged instead of replaced
#  by modify, we have to recreate the WPSlib object to change the
#  classPath.  This would not be necessary if we had unsetAttributes,
#  but that is available only in WAS 7.0 and later.
#
#  This is specifically for configuring the class path of an existing
#  WPSlib Library object.
#
#  Shared libararies are configured at the node, so ${ITCAMDCHOME} can
#  not be used because it is configured at the node.
#
#  Admin Console: Environment/Shared Libraries/WPSlib
#
##############################################################

def configSharedLib ( libraries ):
        global AdminConfig
        global nodeName
        global description_entry

        logDebug("  configSharedLib(%s) entry for node '%s'" % (libraries, nodeName))

        libId = AdminConfig.getid("/Node:"+nodeName+"/Library:WPSlib/" ).strip()
        if (libId == "" or libId == None):
            print "WPS not installed.  Skipping WPSlib classPath configuration"
            logDebug("configSharedLib() exit = false")
            return "false"

        logDebug(" WPSlib ID = %s" % libId)

        nodeId = AdminConfig.getid("/Node:"+nodeName+"/" )

        logDebug("Node ID = %s" % nodeId)
        # Attributes for the new WPSlib Library object consist
        # of the old objects attributes with the classPath replaced

        currentPaths = AdminConfig.showAttribute(libId, "classPath" )
        newPaths = mergePathsAfter(currentPaths, libraries )
        classpathDelimiter = ":"
        if platformUtil.isWindows():
            classpathDelimiter = ";"
        if (newPaths == join(split_path(currentPaths), classpathDelimiter) ):
            print "WPS shared library "+libraries+" already configured"
        else:
            print "Adding ITCAM to WPSlib classPath"
            logDebug("Constructing new WPSlib attributes")
            attrs = to2DList(AdminConfig.show(libId ))
            newWPSlib = []
            newWPSlib.append(description_entry)
            for attr in attrs:
                print "attr=%s" % attr
                if (attr[0] == "description"):
                    logDebug("Skipping description attribute = %s" % attr[1])
                    continue
                if (attr[0] == "classPath"):
                    logDebug("Existing classPath value    =\n  %s  " % join(split_path(attr[1]), "\n"))
                    logDebug("Replacement classPath value =\n  %s  " % join (split_path(newPaths), "\n" ))
                    # classPath normally uses semicolons for separator character
                    newWPSlib.append(["classPath", join( newPaths.split(":" ), ";")  ])
                    continue
                logDebug("Using existing attribute = %s value = %s" % (attr[0], attr[1]))
                newWPSlib.append(attr)

            AdminConfig.remove(libId )
            libId = AdminConfig.create("Library", nodeId, newWPSlib )
            logDebug("New WPSlib ID = %s" % libId)


        logDebug("configSharedLib() exit")
        return "true"


# both
##############################################################
#
#  Procedure  configSmf
#  Arguments: serverid
#             enable  true or false
##############################################################

def configSmf ( enable ):
        global AdminConfig
        global serverid
        global javaProcControl

        logDebug("configSmf("+enable+") entry")

        environments = wsadminToList(AdminConfig.list("Property", javaProcControl ))

        # If we are unconfiguring, remove the SMF container and server activity
        # records because they may have been configured by a back-level
        # version of configDataCollector.

        # Note: Consider removing after 21-Oct-2009

        if (enable == "false"):
            setControlEnv(environments, "server_SMF_server_activity_enabled", enable )
            setControlEnv(environments, "server_SMF_container_activity_enabled", enable )


        setControlEnv(environments, "server_SMF_server_interval_enabled", enable )
        setControlEnv(environments, "server_SMF_container_interval_enabled", enable )

        setControlEnv(environments, "server_SMF_interval_length", 120 )

        logDebug("  configSmf() exit")



# unconfig
##############################################################
#
#  Procedure  deleteJvmArgs
#  Arguments: target, configured, current
#
#  Any JVM option not found in the current options which is
#  found in both the target and configured options is inferred
#  to have been deleted.  These are removed from the target
#  and returned to the caller.
#
#  Note that debug tracing is commented out in this routine
#  because the voluminous output obscures the flow of code
#  more likely to fail.  This low-level routine was debugged
#  and tested outside the wsadmin environment.
#
##############################################################

def deleteJvmArgs ( target_opts, config_opts, exist_opts ):

        # if { $debug_flag } { puts " \n[clock format [clock seconds] -format %T] *DEBUG*  unmergeJvmArgs() entry" }
        # if { $debug_flag } { puts   "[clock format [clock seconds] -format %T] *DEBUG*    target:  \"$target_opts\"" }
        # if { $debug_flag } { puts   "[clock format [clock seconds] -format %T] *DEBUG*    config:  \"$config_opts\"" }
        # if { $debug_flag } { puts   "[clock format [clock seconds] -format %T] *DEBUG*    current: \"$exist_opts\"\n " }

        #target_opts = join( target_opts, " ")
        #target_opts = target_opts.split( )
        #config_opts = join( config_opts, " ")
        #config_opts = config_opts.split( )
        #exist_opts = join( exist_opts, " ")
        #exist_opts = exist_opts.split( )

        merged = []
        for item in target_opts:
            # First check if the item was copied to the configured args
            dup_flag = 0
            if (item == ""): continue
            # if { $debug_flag } { puts "[clock format [clock seconds] -format %T] *DEBUG*    target_opts item: \"$item\"" }
            for configitem in config_opts:
                if (item == configitem):
                    dup_flag = 1
                    break

            # If it is in both, see if it has been removed from the current args
            if (dup_flag):
                keep_flag = 0
                for existitem in exist_opts:
                    if (item == existitem):
                        keep_flag = 1
                        break

            else:
                keep_flag = 1

            # Accumlate all non-deleted items
            if (keep_flag):
                merged.append( item )
            else:
                logDebug("deleted item = %s" % item)




        # if { $debug_flag } { puts "[clock format [clock seconds] -format %T] *DEBUG*    delete result:"; displayJvmArgs "" $merged }
        # if { $debug_flag } { puts "[clock format [clock seconds] -format %T] *DEBUG*  deleteJvmArgs() exit\n " }
        merged = join( merged, " ")
        return merged


# config
##############################################################
#
#  Procedure  doSynchronize
#  Arguments: none
#
#  Returns    none
##############################################################

def __doSynchronize():

        global AdminConfig, AdminControl
        global nodeName, serverName, serverid
        global common_constants

        stat = 0

        #
        # Synchronize with this node if Network Deployment
        # ND has a DeploymentManager object, Base does not
        # There will only be one cell returned.
        # Cell type will be STANDALONE or DISTRIBUTED.
        #
        logDebug("__doSynchronize entry")
        try:
            cell = AdminConfig.getid("/Cell:/") 
            celltype = AdminConfig.showAttribute(cell, "cellType" )
            logDebug("celltype=%s" % celltype)
            if (celltype == "DISTRIBUTED"):
                logDebug("Synchronizing domain manager")
                print "\n%s %s" % (common_constants.SYNC_STARTED, nodeName)
                #print "\nSynchronizing with node %s" % nodeName
                sync = AdminControl.completeObjectName("type=NodeSync,node="+nodeName+",*" )
                if (sync == "" or sync == None):
                    logDebug("Unable to locate NodeSync for node %s. Manual Synchronization is required after the configuration " % nodeName)
                    stat = 1
                    msg = resourcebundle.getmsg("CFG1150W", "Configuration completed, but synchronization of node {0} with the Deployment Manager failed. Manual synchronization is required.", [nodeName])
                    progress_obj.write_progress(msg) 
                else:
                    rc = AdminControl.invoke(sync, "sync" )
                    if (rc == "true"):
                        print "%s on %s" % (common_constants.SYNC_COMPLETED, nodeName)
                    else:
                        stat = 1
                        logDebug("*Error*  Failure synchronizing on node %s, result = %s" % (nodeName, str(rc)))
                        msg = resourcebundle.getmsg("CFG1150W", "Configuration completed, but synchronization of node {0} with the Deployment Manager failed. Manual synchronization is required.", [nodeName])
                        progress_obj.write_progress(msg) 
            else:
                logDebug("No DMGR, synchronization not necessary.")
        except Exception, msg:
            logDebug("Error occurred while synchronizing: %s" % msg )
            stat = 1
            msg = resourcebundle.getmsg("CFG1150W", "Configuration completed, but synchronization of node {0} with the Deployment Manager failed. Manual synchronization is required.", [nodeName])
            progress_obj.write_progress(msg)
            
        logDebug("__doSynchronize exit %s" % str(stat))
        return stat

def hasNodeagent():
    
    global AdminConfig, AdminControl
    global nodeName, doSynchronize, isStandalone
        
    result = 0
    
    logDebug("hasNodeagent entry")
    try:
        cell = AdminConfig.getid("/Cell:/" )
        celltype = AdminConfig.showAttribute(cell, "cellType" )
        logDebug("celltype=%s" % celltype) 
        if (celltype == "DISTRIBUTED"):
            logDebug("checking if nodeagent is up")
            print "\n%s %s" % (common_constants.SYNC_STARTED, nodeName)
            sync = AdminControl.completeObjectName("type=NodeSync,node="+nodeName+",*" )
            logDebug("nodeName=%s sync=%s" % (nodeName, str(sync)))
            if (sync == "" or sync == None):
                doSynchronize = 0
                result = 1
                logDebug("Nodeagent for node %s is not running " % nodeName)
            else:
                doSynchronize = 1
        else:
            logDebug("Standalone node, we are OK to proceed") 
            doSynchronize = 0
            isStandalone = 1
    except Exception, msg:
        logDebug("Error in hasNodeAgent: %s" % msg )
   
    logDebug("hasNodeagent, exit %s" % str(result))    
    return result


# both
##############################################################
#
#  Procedure  diffJvmArgs
#  Arguments:
#
#  Find the difference between two strings.  That is, remove every word
#  found in the second string from the first string.
#
#  Note that debug tracing is commented out in this routine
#  because the voluminous output obscures the flow of code
#  more likely to fail.  This low-level routine was debugged
#  and tested outside the wsadmin environment.
#
##############################################################

def diffJvmArgs ( exist_opts, minus_opts ):

        logDebug("diffJvmArgs() entry")
        logDebug(" current: %s" % exist_opts)
        logDebug("minus: %s" % minus_opts)

        #exist_opts = join( exist_opts, " ")
        #exist_opts = exist_opts.split( )
        #minus_opts = join( minus_opts, " ")
        #minus_opts = minus_opts.split( )

        extras = []
        for item in exist_opts:
            for olditem in minus_opts:
                if (item == olditem):
                    item = ""
                    break
            if (item != ""): extras.append(item)

        diff = " ".join(extras)

        logDebug(" diffJvmArgs() result = %s" % diff)
        return diff



# config
##############################################################
#
#  Procedure  getArg
#  Arguments: arg
#             argList
#  Returns argument from list if present, "" if not
#
##############################################################
def getArg ( arg, argList ):

    logDebug("arg=%s" % arg)
    retArg = ""
    for argument in argList:
        if (argument.find(arg) == 0):
            retArg = argument
            break
    logDebug("input arg=%s return arg=%s" % (arg, retArg))
    return retArg


# both
##############################################################
#
#  Procedure displayJvmArgs
#
##############################################################

def displayJvmArgs ( title, jvmargs ):
        if (title != ""):
                if (jvmargs == ""):
                        print title+": (none)"
                        return

                print " "
                print title+":"


        print " "
        hit_flag = 0
        for arg in jvmargs.split( ):
            if (arg == ""):  continue
            hit_flag = 1
            if (arg.find(":") == arg.rfind(":") ):
                print "  " + arg
            else:
                args = arg.split(":" )
                argi = 0  #forStart
                while ( argi < len(args)  ):  #forTest
                    print "%d %s" % (argi, args[argi])
                    argi += 1  #forNext


        if ( not hit_flag): print "  (none)"

        print " "



# config
##############################################################
#
#  Procedure configJVM
#  Arguments: new java command line argument
#
##############################################################

def configJVM ( newGenericJvmArguments ):
        global AdminConfig
        global AdminTask
        global product
        global jvmId
        global nodeName, serverName, serverid
        global serverVersion
        global wasnode
        global wasserver

        logDebug("configJVM() entry")


        foundRTT = "false"
        runam = ""

        print "Modify JVM command line arguments"
        oldGenericJvmArguments = AdminConfig.showAttribute(jvmId, "genericJvmArguments" )

        print "oldGenericJvmArguments: ", oldGenericJvmArguments

        #Check if agentlib:am* is part of oldGenericJvmArguments.
        #If yes, remove it from oldGenericJvmArguments.
        updateOldArgList = []
        if oldGenericJvmArguments != None and oldGenericJvmArguments != "":
            oldArgList = oldGenericJvmArguments.split()
            addArgToList = "true"
            for oldArg in oldArgList:
                if oldArg.find("-agentlib:am") == 0:
                    addArgToList = "false"
                elif (oldArg.startswith("-Xbootclasspath/a:") or oldArg.startswith("-Xbootclasspath/p:")):
                    index = oldArg.index(":")
                    bootCPList = oldArg[index+1:len(oldArg)]
                    print "bootCPList: ", bootCPList
                    #Check if any jar files from older version of ITCAM
                    #If yes, remove them from bootclasspath as we do not want to save them in AM_OLD_JVM_ARG.
                    bootCPToAdd = []
                    classpathdelimiter = ":"
                    if platformUtil.isWindows():
                        classpathdelimiter = ";"
                    cpjars = bootCPList.split(classpathdelimiter)
                    oldArg = ""
                    for cpjar in cpjars:
                        print "cpjar: ", cpjar
                        if cpjar.find("${ITCAM") == 0:
                            continue
                        else:
                            bootCPToAdd.append(cpjar)
                    if len(bootCPToAdd) > 0:
                        oldArg = join(bootCPToAdd, os.sep)
                        addArgToList = "true"
                if addArgToList == "true":
                    updateOldArgList.append(oldArg)

        oldGenericJvmArguments = join( updateOldArgList, " ")
        print "oldGenericJvmArguments after cleaning up: ", oldGenericJvmArguments

        # Save the old generic JVM arguments in a variable, so unconfig can find it
        am_old_jvm_args = [["symbolicName", "AM_OLD_JVM_ARGS"], ["value", oldGenericJvmArguments]]

        # Check if the variable already exists (the user is running config again)
        oldvarid = getServerVariableID("AM_OLD_JVM_ARGS" )
        if (oldvarid == ""):
            configServerVariable(am_old_jvm_args )
        else:
            print "Variable AM_OLD_JVM_ARGS already exists - left in place"
            logDebug("Saved generic JVM arguments are: ")
            displayJvmArgs("", AdminConfig.showAttribute(oldvarid, "value" ) )


        logDebug("Current generic JVM arguments are: ")
        displayJvmArgs("", oldGenericJvmArguments )

        newArgList = newGenericJvmArguments.split()
        oldArgList = oldGenericJvmArguments.split()
        jvmArgList = []
        if (oldGenericJvmArguments != ""):
            logDebug("New generic JVM arguments are:")
            displayJvmArgs("", newGenericJvmArguments )
            oldArgList = oldGenericJvmArguments.split()

            # First go through oldGenericJvmArguments, and see if
            # there are replacements in newGenericArgs

            for oldArg in oldArgList:
                logDebug("oldArg=%s" % oldArg)
                if (oldArg.startswith("-Xrunjvmpi:")):
                    runam = oldArg.split("-Xrunjvmpi:")[1]
                    #if { [regexp -- {^-Xrunjvmpi:(.+)$} $oldArg "" runam] } {
                    if (int(serverVersion) >= 61):
                        print "Found ITCAM for RTT in WAS 6.1. Leave -Xrunjvmpi"
                        jvmArgList.append(oldArg)
                    else:
                        print "Found ITCAM for RTT libjvmpi. Converted to -Xrunam:"+runam
                        jvmArgList.append("-Xrunam:"+runam)


                # For WAS 6.1, -Xrunam is replaced by -agentlib:am, jiti.jar is
                # removed from boot class path, and the injector path is no
                # longer present.  This should allow graceful migration from
                # WAS 6.0 to WAS 6.1 by simply rerunning setupwas.sh.
                # The -Xrunam* in new list replaces the -Xrunam* in the old list
                elif (oldArg.find("-Xrunam") == 0):
                    runam = getArg("-Xrunam", newArgList )
                    if (runam == ""):  runam = getArg("-agentlib:am", newArgList )
                    print "Found ITCAM -Xrunam for WAS. Migrating to "+runam
                    jvmArgList.append(runam)

                # The -agentlib in new list replaces the -agentlib in the old list
                elif (oldArg.find("-agentlib:am") == 0):
                    agentlib = getArg("-agentlib:am", newArgList )
                    print "Found ITCAM -agentlib:am in old args. Now "+agentlib
                    jvmArgList.append(agentlib)

                # Merge paths for -Xbootclasspath/a: and -Xbootclasspath/p:
                elif (oldArg.startswith("-Xbootclasspath/a:") or oldArg.startswith("-Xbootclasspath/p:")):
                    # elseif { [regexp -- {^(-Xbootclasspath/[ap]:).*} $oldArg oldBootClasspath arg] } {
                    arg = oldArg.split(':')[0] + ':'
                    oldBootClasspath = arg
                    newBootClasspath = getArg(arg, newArgList )
                    bootClasspath = mergePaths(oldBootClasspath, newBootClasspath )
                    # If using JVMTI now and RTT not present, remove any old jiti libraries
                    if (newGenericJvmArguments.find("-agentlib:am") >= 0):
                        if (oldGenericJvmArguments.find("-Xrunjvmpi") < 0):
                            print "Removing unused jiti libraries from bootClasspath"
                            bootClasspath = removeJiti(bootClasspath )


                    print "Merged boot class "+bootClasspath
                    jvmArgList.append(bootClasspath)

                # Any properties in old list will be dropped if they are also in new list
                elif (oldArg.startswith("-D") and len(oldArg.split('=')) == 2):
                    #TODO} elseif { [regexp -- {^-D([^=]+)=(.*)$} $oldArg x prop ] } {
                    prop = oldArg.split('=')[1]
                    arg = getArg("-D"+prop+"=", newArgList )
                    if (arg == ""):
                        jvmArgList.append(oldArg)
                    else:
                        print "Property "+prop+" updated to "+arg


                # Any -verbose from old list will be used if not found in new list
                elif (oldArg.find("-verbose") == 0):
                    for arg in newArgList:
                        if (arg== oldArg):
                            oldArg = ""
                            break


                    if (oldArg== ""):
                        print "Duplicate item "+arg+" dropped"
                    else:
                        jvmArgList.append(oldArg)


                # Drop IProbeInectorManager if using JVMTI
                elif (oldArg.find("-Dcom.ibm.tivoli.jiti.injector.IProbeInjectorManager") == 0):
                    if (newGenericJvmArguments.find("-agentlib:am") >= 0):
                        print " Using JVMTI. Dropping com.ibm.tivoli.jiti.injector.IProbeInjectorManager"
                    else:
                        jvmArgList.append(oldArg)


                # Drop obsolete ITCAM_JVM_ARGS
                elif (oldArg == "${ITCAM_JVM_ARGS}"):
                    print "Dropping obsolete entry "+oldArg

                # Drop erroneous entry jlog.common.dir.CYN=
                elif (oldArg.find("jlog.common.dir.CYN=") == 0):
                    print "Dropping erroneous entry "+oldArg

                # Drop obsolete entry -Declipse.security=
                elif (oldArg.find("-Declipse.security=") == 0):
                    print "Dropping erroneous entry "+oldArg

                # Drop no longer needed entry -Xnoclassgc
                elif ("-Xnoclassgc"== oldArg):
                    print "Dropping obsolete entry "+oldArg

                # Drop duplicate entry -Xverbosegclog:
                elif (oldArg.find("-Xverbosegclog:") == 0):
                    print "Dropping conflicting entry "+oldArg

                # Anything else will be passed through
                else:
                    print "Pass through: "+oldArg
                    jvmArgList.append(oldArg)


            logDebug("JVM arg list updates: ")
            #jvmArgList = join( jvmArgList)
            displayJvmArgs("", join(jvmArgList) )

            # Now go through the newArglist, see if there are new ones to go in.

            print "Examining new Jvm arg list"
            for newArg in newArgList:

                # Only one runam in the final list
                if (newArg.find("-Xrunam") == 0):
                    arg = getArg("-Xrunam", jvmArgList )
                    if (arg == ""): jvmArgList.append(newArg)


                # if a bootpath was merged, do not add the one in the new list
                elif (newArg.startswith("-Xbootclasspath/a:") or newArg.startswith("-Xbootclasspath/p:")):
                    #TODO} elseif { [regexp -- {(-Xbootclasspath/[ap]:).*} $newArg x prop] } {
                    prop = newArg.split(':')[0] + ':'
                    arg = getArg(prop, jvmArgList )
                    if (arg == ""):
                        jvmArgList.append(newArg)


                # Only one -verbose:gc in the final list
                elif (newArg.find("-verbose:gc") == 0):
                    arg = getArg("-verbose:gc", jvmArgList )
                    print "getArg -verbose:gc found "+arg
                    if (arg == ""):
                        jvmArgList.append(newArg)

                # Add all new elements that are not already present in the final list
                else:
                    for arg in jvmArgList:
                        if (arg == newArg):
                            print "newArg "+newArg+" already present"
                            newArg = ""
                            break


                    if (newArg != ""):
                        print "Adding newArg "+newArg
                        jvmArgList.append(newArg)



            jvmArgList = join( jvmArgList, " ")
            genericJvmArgs = jvmArgList
        else:
            # No oldArgs
            genericJvmArgs = newGenericJvmArguments


        # If AM_CONFIG_JVM_ARGS exists, we have been configured before.
        # In that case, determine if any of the current parameters are extras
        # defined after configuration, and remove those from the parameters
        # from the value to be stored in AM_CONFIG_JVM_ARGS.  This is so that
        # unconfig will recognize those extras as new parameters.

        configJvmArgs = genericJvmArgs
        configvarid = getServerVariableID("AM_CONFIG_JVM_ARGS" )
        if (configvarid != ""):
            logDebug(" Will replace existing AM_CONFIG_JVM_ARG")
            # Find any options in the current JVM options which aren't in AM_CONFIG_JVM_ARGS.
            # These will be extras defined since the last configuration.

            configValue = AdminConfig.showAttribute(configvarid, "value" )
            extraValue = diffJvmArgs(oldGenericJvmArguments, configValue )
            if (extraValue != ""):
                logDebug("Added since last configuration: %s" % extraValue)
                # Remove the extra JVM options from the AM_CONFIG_JVM_ARGS options.
                # This will cause unconfig to recognize them as defined since configuration.
                configJvmArgs = diffJvmArgs(configJvmArgs, extraValue )


            # But things may also be removed.
            #   If in AM_OLD_JVM_ARGS
            #   And in AM_CONFIG_JVM_ARGS
            #   But not in current JVM args
            #   Then it was deleted,
            #   So it must be added back to the new AM_CONFIG_JVM_ARGS for unconfig.sh

            if (oldvarid != ""):
                oldValue = AdminConfig.showAttribute(oldvarid, "value" )
                configJvmArgsList = configJvmArgs.split( )
                for item in oldValue.split( ):
                    if (item == ""): continue
                    logDebug("item=%s" % item)
                    # Is item from AM_OLD_JVM_ARGS in AM_CONFIG_JVM_ARGS?
                    additem = ""
                    for configitem in configValue.split( ):
                        if (configitem == item):
                            additem = "yes"
                            break


                    # And is item not in current JVM args?
                    if (additem != "yes"): continue
                    for currentitem in genericJvmArgs.split( ):
                        if (currentitem == item):
                            additem = "no"
                            break
                    if (additem != "yes"): continue
                    # Then it must be reinstated into new AM_CONFIG_JVM_ARGS
                    logDebug("Deleted item found to retain in AM_CONFIG_JVM_ARGS: %s"  % item)
                    configJvmArgsList.append(item)

                #configJvmArgsList = join( configJvmArgsList, " ")
                configJvmArgs = join( configJvmArgsList, " ")
                print "configJvmArgs = %s" % configJvmArgs

        else:
            logDebug("No previous AM_CONFIG_JVM_ARG")

        displayJvmArgs("Final merged generic JVM arguments", genericJvmArgs )

        # Save the configured generic JVM arguments in a variable, so unconfig can identify changes
        am_config_jvm_args = [["symbolicName", "AM_CONFIG_JVM_ARGS"], ["value", configJvmArgs]]
        configServerVariable(am_config_jvm_args )

        # modify genericJvmArguments
        process_type_arg = ""
        if platform == "ZOS": 
            process_type_arg = "-processType Servant"    
        print "Jvm Arguments  %s" % (genericJvmArgs)
        argString = "[ %s -serverName %s -nodeName %s -genericJvmArguments \"%s\"]" % (process_type_arg, wasserver, wasnode, genericJvmArgs)
        print "Setting jvm argument ARGS using  %s" % (argString)
        AdminTask.setJVMProperties(argString)

        logDebug("configJVM() exit")



# config
##############################################################
#
#  Procedure  removeJiti
#  Arguments: bootClasspath
#
#  Returns:   The modified bootClasspath
#
#
##############################################################

def removeJiti ( bootClasspath ):

        retVal = ""
        boots = bootClasspath.split(":" )
        newboots = []
        for boot in boots:
            if ("boot".find("jiti.jar") >= 0):
                    continue
            else:
                newboots.append(boot)

        retVal = join(newboots, ':')
        return retVal


# both
#############################################################
#
#  Procedure unconfigInjector
#  Unconfigure the JITI Primary Injector File, which is obsolete
#
#############################################################
def unconfigInjector (  ):
        global AdminConfig
        global server_runtime
        global quiet_flag
        global jvmId
        global jiti_flag


        logDebug("unconfigInjector() entry")

        PRIMARY_INJECTOR_FILE = "com.ibm.tivoli.jiti.injector.ProbeInjectorManagerChain.primaryInjectorFile"

        # Unlike other property scans, this will be quick, since there will probably be at most
        # one property for the Servant JVM ID.  So there is no point in keeping a cache of properties here.

        objectId = ""
        value = ""
        logDebug(" Looking for "+PRIMARY_INJECTOR_FILE+" property.")

        properties = toList(AdminConfig.list("Property", jvmId ))
        logDebug(" %d property objects found." % len(properties))
        for property in properties:
            logDebug(" Object ID = "+property)
            if ( property.find("#Property_") == -1 ): continue
            # Catch any errors in showAttribute
            try:
                name = AdminConfig.showAttribute(property, "name" )
            except:
                _type_, _value_, _tbck_ = sys.exc_info()
                print " \n*Warning*  showAttribute failed for property ID "+property
                print "           " + str(_value_) +"\n "
                continue
            logDebug("Name = "+name)
            if (name == PRIMARY_INJECTOR_FILE):
                value = AdminConfig.showAttribute(property, "value" )
                print "System property "+name+" exists,"
                print "  value = \""+value+"\""
                objectId = property
                break



        # Have the unconfigInjector remove the name from the properties file,
        # then it returns the name to revert to.  We either update the
        # property or remove it, depending upon whether a reversion name was returned.

        # However, if JITI is not available the best we can do is remove the property

        if (objectId != ""):
            # The catch is in case the jiti jar has not been provided to wsadmin
            # It should not be necessary because we already checked in the prolog
            if (jiti_flag):
                try:
                    setup_util = SetupUtil(  ) #?PROBLEM? (jacl 61) JAVA::NEW SetupUtil might require:  from ?? import SetupUtil
                    # Note: Java null is returned as empty string, not [java::null]
                    injectorPropertyValue = setup_util.unconfigInjector(value )
                    print "injectorPropertyValue being set to \""+injectorPropertyValue+"\""
                except:
                    _type_, _value_, _tbck_ = sys.exc_info()
                    ermsg = str(_value_)
                    # Note ermsg is included in $::errorInfo, so unneeded if the latter is dumped
                    print " *Warning*  Unconfig injector failure " + ermsg
                    print " processing continues\n "
                    injectorPropertyValue = ""
            else:
                logDebug("Removing obsolete injector property")
                injectorPropertyValue = ""
            if (injectorPropertyValue == ""):
                logDebug("Reversion injector file name is null")
                AdminConfig.remove(objectId )
                logDebug("Injector property "+PRIMARY_INJECTOR_FILE+" has been removed")
            else:
                logDebug("Reversion injector file name is \""+injectorPropertyValue+"\"")
                injectorProperty = [["name", PRIMARY_INJECTOR_FILE], ["value", injectorPropertyValue]]
                AdminConfig.modify(objectId, injectorProperty )
                logDebug("Injector property "+PRIMARY_INJECTOR_FILE+" has been reverted")

        else:
            logDebug("No injector property "+PRIMARY_INJECTOR_FILE+" found")
            if ( not quiet_flag):
                print "Injector property "+PRIMARY_INJECTOR_FILE+" does not exist"


        logDebug("  unconfigInjector() exit")




def subsNodeVariable (varstring, nodename):
    '''
        Procedure subsNodeVariable
        Arguments: variable string, node name
        This does recursive replacement of node variables.
    '''
    global variableEntries, variableEntriesMapNodename, variableEntriesMap
    global AdminConfig

    variableEntriesMapNodename = ""
    logDebug("subsNodeVariable(%s %s) "  % (varstring, nodename))
    # Check if we need to get a new variable map
    logDebug("variableEntries not set")
    variableEntries = []
    variableEntriesMapNodename = nodename
    variableEntriesMap={}

    if (nodename != variableEntriesMapNodename):
        logDebug("cached node was " + str(variableEntriesMapNodename))
        variableEntries = ""


    # Build a variable map array
    if (len(variableEntries) == 0):
        logDebug("need to build variableEntriesMap")
        variableEntries = toList(AdminConfig.getid("/Node:"+nodename+"/VariableMap:/VariableSubstitutionEntry:/" ))
        for entry in variableEntries:
            # Skip if not VariableSubstitutionEntry
            position = entry.find("#VariableSubstitutionEntry_")
            if (position == -1): continue
            try:
                varName = AdminConfig.showAttribute(entry, "symbolicName" )
            except:
                type, value, tbck = sys.exc_info()
                logDebug("No symbolicName for node entry %s . Error: %s" % (entry, str(value)))
                continue
            #endtry
            try:
                varValue = AdminConfig.showAttribute(entry, "value" )
            except:
                type, value, tbck = sys.exc_info()
                logDebug("No value for node entry %s .  Error: %s " % (str(entry), str(value)))
                continue
            #endtry
            variableEntriesMap[varName]=varValue

        variableEntriesMapNodename = nodename


    while (varstring.find('{') > -1 and varstring.find('}') > -1):
        varName = varstring.split('${', 1)[1].split('}', 1)[0]
        leftPart=varstring.split('${',1)[0]
        rightPart=varstring.split('}',1)[1]
        #print "varstring=%s varName=%s leftPart=%s rightPart=%s" % (varstring, varName, leftPart, rightPart)
        if (variableEntriesMap.get(varName, -1) == -1):
            logDebug("Unable to locate ${"+varName+"}")
            varValue = ""
        else:
            varValue = variableEntriesMap[varName]
        logDebug("replace ${"+varName+"} => \""+varValue+"\"")
        var = leftPart+varValue+rightPart
        varstring=var
    #endWhile

    logDebug("subsNodeVariable %s=%s" % (varstring, var))
    return var


# config
##############################################################
#
#  Procedure  configServerVariable
#  Arguments: variable
#
##############################################################

def configServerVariable ( newVariable ):
        global AdminConfig
        global AdminTask
        global nodeName, serverName, serverid
        global variableMap, variableMapArray
        global description_entry
        global wascell
        global wasnode
        global wasserver

        logDebug(" configServerVariable(%s) entry" % newVariable)

        _J2J_bracket_ = newVariable[0][0]
        if (_J2J_bracket_ != "symbolicName" or newVariable[1][0] != "value"):
            raise RuntimeError, "configServerVariable bad parameter = "+newVariable


        newSymbolicName = newVariable[0][1]
        newValue = newVariable[1][1].strip()


        initServerVariableMapCache( )

        logDebug("description_entry=%s" % description_entry)
        newVariable.append(description_entry)
        logDebug("map array=%s" %  variableMapArray)
        logDebug("new symb name=%s new val=%s" % (newSymbolicName, newValue))
        ent = variableMapArray.get(newSymbolicName, None)
        logDebug("Entry from map array %s" % (ent))
        if (ent != None):
            oldValue = ent
            logDebug("variable %s is already defined with value '%s'" % (newSymbolicName, oldValue))
            if (oldValue != newValue):
                print "Modify variable "+newSymbolicName
                argString = u"[-variableName %s -scope Cell=%s,Node=%s,Server=%s ]" % (newSymbolicName, wascell, wasnode, wasserver)
                print "Removing variables ARGS     %s" % (argString)
                AdminTask.removeVariable(argString)
                if newValue.find('"') > -1:
                    newValue = newValue.replace('"', '&quot;')
                argString = u"[-variableName %s -scope Cell=%s,Node=%s,Server=%s -variableValue \"%s\" -variableDescription '%s']" % (newSymbolicName, wascell, wasnode, wasserver, newValue, description_entry[1])
                print "Modifying variable ARGS     %s" % (argString)
                AdminTask.setVariable(argString)
                variableMapArray[newSymbolicName] = newValue
            else:
                print "No change for variable "+newSymbolicName

        else:
            if newValue.find('"') > -1:
                newValue = newValue.replace('"', '&quot;')
            print "create new variable "+newSymbolicName
            argString = u"[-variableName %s -scope Cell=%s,Node=%s,Server=%s -variableValue \"%s\" -variableDescription '%s']" % (newSymbolicName, wascell, wasnode, wasserver, newValue, description_entry[1])
            print "Setting variable ARGS     %s" % (argString)
            AdminTask.setVariable(argString)
            variableMapArray[newSymbolicName] = newValue


        logDebug("configServerVariable() exit")



# Both
#############################################################
#
#  Procedure  getProcessDefs
#  Arguments: serverid
#
#  Gets the JavaProcessDefs and JVM IDs for the z/OS
#  control and servant regions
##############################################################

def getProcessDefs ( platform ):
        global AdminConfig
        global nodeName, serverName, serverid
        global javaProcServant
        global javaProcControl
        global jvmId, servantJvmId, controlJvmId

        logDebug("getProcessDefs(%s)" % serverid)


        jvmId = ""
        servantJvmId = ""
        controlJvmId = ""
        processDefs = []
        # Walk over Java Process Definitions looking for control and servant regions
        # Remember the ProcessDef and JavaVirtualMachine object IDs for each

        processDefs = toList(AdminConfig.list("JavaProcessDef", serverid ))
        #processDefs.append(processDef)
        #TODO
        for process in processDefs:
                logDebug("getProcessDefs process = %s"  % process)

                # z/OS Has multiple ProcessDefs, Control, Servant, Adjunct
                type = AdminConfig.showAttribute(process, "processType" )
                # Carol: only z/OS has type value returned.
                logDebug("   type = %s" % type)

                if (is_zOS(platform) and type == "Control"):
                        javaProcControl = process
                        print "   jvmidstr=%s" % AdminConfig.showAttribute(process, "jvmEntries" )
                        jvmIdStr = wsadminToList(AdminConfig.showAttribute(process, "jvmEntries" ))
                        jvmIdStr = join( jvmIdStr, " ")
                        controlJvmId = jvmIdStr
                        logDebug("   getProcessDefs is Control")
                elif (type == "Servant" or not is_zOS(platform)):
                        javaProcServant = process
                        jvmIdStr = wsadminToList(AdminConfig.showAttribute(process, "jvmEntries" ))
                        print "jvmidstr=%s" % AdminConfig.showAttribute(process, "jvmEntries" )
                        jvmIdStr = join( jvmIdStr, " ")
                        servantJvmId = jvmIdStr
                        print "   Carol -- servantJvmId: %s" % servantJvmId
                        logDebug("   getProcessDefs is Servant")
                elif (type == "Adjunct"):
                        # We don't currently care about an Adjunct
                        logDebug("   getProcessDefs is Adjunct")


                else:
                        logDebug("   getProcessDefs is unexpected type = %s " % type)



        if ( len(controlJvmId) == 0 ):
                #error "Unable to locate Control region JVM, configuration abandoned"
                print "  no control JvmId"

        if ( len(servantJvmId) == 0  ):
                #error "Unable to locate Servant region JVM, configuration abandoned"
                print "  no servntJvmid"


        # default "jvmId" is the servant on z/OS
        jvmId = servantJvmId

        logDebug(" default JVM ID: %s" % jvmId)
        logDebug(" control JVM ID: %s" % controlJvmId)
        logDebug(" servant JVM ID: %s" % servantJvmId)
        logDebug("getProcessDefs exit")

        return



# config
##############################################################
#
#  Procedure configPMIService
#
##############################################################

def configPMIService (  ):
        global AdminConfig
        global serverid

        logDebug(" configPmiService() entry")


        PMIServiceId = AdminConfig.list("PMIService", serverid )
        logDebug("PMI service ID = %s" % PMIServiceId)


        if (PMIServiceId == "" or PMIServiceId == None):
            raise RuntimeError, "PMIService doesn't exist"


        #if (len(PMIServiceId) > 1):
        #    raise RuntimeError, "More than 1 PMIService exists"


        pmiats = to2DList(AdminConfig.show(PMIServiceId ))
        enable = ""
        for pmiat in pmiats:
            if ( pmiat[0] == "enable" ): enable = pmiat[1]


        if (enable != "true"):
            print "Enable PMI service"
            AdminConfig.modify(PMIServiceId, [["enable", "true"]] )
        else:
            print "PMI service is already enabled"


        logDebug("configPmiService() exit")



# unconfig
##############################################################
#
#  Procedure unconfigCustomService
#  Arguments: server config id,
#             newCustomService
#
##############################################################

def unconfigCustomService ( newCustomService ):
        global AdminConfig
        global serverid

        logDebug("unconfigCustomService() entry")
        for attr in newCustomService:
            logDebug("attr=%s" % attr)


        for attr in newCustomService:
            attrName = attr[0]
            attrValue = attr[1]
            if ( attrName == "classname" ):
                newClassName = attrValue
            elif ( attrName == "enable" ):
                newEnable = attrValue
            elif ( attrName == "classpath" ):
                newClassPath = attrValue
            elif ( attrName == "displayName" ):
                newServiceName = attrValue
            elif ( attrName == "description" ):
                newDesctiption = attrValue
            elif ( attrName == "default" ):
                logDebug("ignore "+attrName+"="+attrValue)


        print "Unconfigure custom service "+newServiceName
        customServiceIds = AdminConfig.list("CustomService", serverid )
        if (customServiceIds != ""):
            for tmpId in customServiceIds:
                #?PROBLEM? (jacl 38) previous FOREACH_ITEM_IN_LIST may need variable.split("xx")
                oldClassName = AdminConfig.showAttribute(tmpId, "classname" )
                oldServiceName = AdminConfig.showAttribute(tmpId, "displayName" )
                oldEnable = AdminConfig.showAttribute(tmpId, "enable" )
                oldClassPath = AdminConfig.showAttribute(tmpId, "classpath" )
                if (oldServiceName == newServiceName):
                    print "Custom Service "+newServiceName+" exists, removing it"
                    AdminConfig.remove(tmpId )
                    break




        logDebug("unconfigCustomService() exit")



##############################################################
#
#  Procedure configHeapSize $serverid $maximumHeapSize
#  Arguments: maximumHeapSize
#
##############################################################
def configHeapSize ( maximumHeapSize ):
        global AdminConfig
        global serverid, platform, jvmId

        logDebug("configHeapSize entry maxHeap=%d" %  maximumHeapSize)
        logDebug("jvmId=%s" % jvmId)
        #removed code for getting jvmId, will use global jvmId value
        if (jvmId == "" or jvmId == None):
            raise Exception("ERROR: JVM does not exist in server %s" % serverid)

        logDebug("Modify JVM maximumHeapSize arguments")
        oldmaximumHeapSize = AdminConfig.showAttribute(jvmId, "maximumHeapSize")
        print "INFO: current maximumHeapSize is %s" % str(oldmaximumHeapSize)
        print "##############################################################"
        print "INFO: new maximumHeapSize is %s" % str(maximumHeapSize)
        #modify maximumHeapSize
        mhattr = [["maximumHeapSize", maximumHeapSize]]
        AdminConfig.modify(jvmId, mhattr )
        logDebug("configHeapSize exit")



# both
##############################################################
#
#  Procedure  setControlEnv
#  Arguments: environments      current environment variables
#             var               variable to add
#             newValue          new value to set
#
#  Sets a control process environment variable
#
#  Note that new entries will not be created if the value is to
#  be "false".  This is to avoid creating unneeded entries for
#  container and server activity records, which are obsolete.
#
##############################################################

def setControlEnv ( environments, entry, newValue ):
        global AdminConfig
        global javaProcControl
        global description_entry

        logDebug(" setControlEnv(%s, %s) entry" % (entry, newValue))

        parm = [["name", entry], ["value", newValue], description_entry]
        index = 0
        i = -1
        envid = ""
        for var in environments:
            print var
            if (var.startswith(entry)):
                logDebug("found %s at position %d" % (entry, index))
                i = index
                envid = var
                break
            index += 1
        if (i < 0):
            if (newValue != "false"):
                logDebug("New environment variable %s" % entry)
                AdminConfig.create("Property", javaProcControl, parm, "environment" )
            else:
                logDebug("Not creating "+entry+" because it would be false")

        else:
            #envid = environments[i]
            value = AdminConfig.showAttribute(envid, "value" )
            if (value == newValue):
                logDebug("Already set to %s" % newValue)

            else:
                logDebug("Modifying existing Old value = %s" % value)
                logDebug("New value = %s" % newValue)
                AdminConfig.modify(envid, parm )


        logDebug(" setControlEnv() exit")



# both
##############################################################
#
#  Procedure  initServerVariableMapCache
#  Arguments: none
#
##############################################################

def initServerVariableMapCache (  ):
        global AdminConfig
        global serverid
        global variableMap, variableMapArray, variableMapServerid, serverVariableEntries


        logDebug("  initServerVariableMapCache() entry")


        # The array doesn't need to be rebuilt if additions and deletions are maintained
        # If an entry has been added, the variable map must be rebuilt
        # If an entry has been removed, the variable map must be rebuilt

        if ( len(variableMap) == 0  ):
                logDebug("   First variable map cache")
                variableMap = ""
                variableMapServerid = serverid
                # Add a dummy entry to define 2-dimensional array
                variableMapArray = {}


        if (variableMapServerid != serverid):
                logDebug("Server for variable map cache has changed")
                variableMap = ""
                variableMapServerid = serverid
                serverVariableEntries = []


        if (len(variableMap) == 0):
            logDebug("Creating new variable map entries cache")
            serverVariableEntries = []
            variableMapArray = {}
            # Build the array

            variableMap = AdminConfig.list("VariableMap", serverid )
            if (variableMap == ""):
                print " \n*Error*  VariableMap doesn't exist in server "+serverid+"\n "
                raise RuntimeError, "VariableMap doesn't exist in server %s" % serverid

            serverVariableEntries = wsadminToList(AdminConfig.showAttribute(variableMap, "entries" ))
            for entryId in serverVariableEntries:
                subst_found = entryId.find("#VariableSubstitutionEntry_")
                if (subst_found == -1): continue

                try:
                    name = AdminConfig.showAttribute(entryId, "symbolicName" )
                except:
                    logDebug("No symbolicName for server entry %s " % entryId)
                    print " \n*Warning*  Invalid WAS server variable object "+entryId
                    #print "           "+errmsg+"\n "
                    continue

                try:
                    value = AdminConfig.showAttribute(entryId, "value" )
                except:
                    logDebug("No value for server entry %s " % entryId)
                    print " \n*Warning*  Invalid WAS server variable object %s" % entryId
                    #print "           "+errmsg+"\n "
                    continue


                logDebug("Entry ID = %s" % entryId)
                logDebug("%s=%s" % (name, value))
                #variableMapArray[name] = [name, [entryId, value]]
                variableMapArray[name] = value
            logDebug("%d properties in cache" % (len(variableMapArray)))

        logDebug("initServerVariableMapCache() exit")



# unconfig
##############################################################
#
#  Procedure  editServerPolicy
#  Arguments: "config" or "unconfig"
#
#  Note: editServerPolicy is obsolete for configuration, it
#        is only retained to remove existing changes.
#
##############################################################

def editServerPolicy ( action ):
        global AdminConfig
        global variableEntries
        global propDir
        global serverid
        global runtime_path
        global itcamdc_home
        global nodeName

        logDebug(" editServerPolicy(%s) entry" % action)
        # This flag is only needed for extreme debugging
        debugx_flag = 1

        # Establish the full path to the server policy file by resolving
        # the value of WAS_PROPS_DIR at the node level.

        print "Editing Server Policy file"
        propDir = subsNodeVariable("${WAS_PROPS_DIR}", nodeName )
        print "WAS_PROPS_DIR="+propDir
        policyFile = propDir+"/server.policy"
        logDebug("server policy = %s" % policyFile)


        # Establish the full path to the data collector policy additions file.

        dcFile = runtime_path+"/datacollector.policy.asc"
        logDebug(" dcFile = %s" % dcFile)

        #
        # Find an unused name for the backup copy of the file
        # (Also find the last used name)
        #

        lastFile = ""
        i = 1
        zero_str = "0000"
        while (1):
            if (i < 1000):
                    suffix = "." + zero_str[:-(len(str(i)))] + str(i)
            else:
                    suffix = "." + str(i)

            backupFile = policyFile + suffix
            try:
                backupDesc = open( backupFile, "r" )
                lastFile = backupFile
                backupDesc.close()
                i = (i + 1)
                if (i > 500):
                    print " "
                    print "*Warning*  Policy file backup limit reached: "+backupFile
                    print " "
                    print "           After this point, the current server.policy file"
                    print "           will be backed up to server.policy"+suffix+" each"
                    print "           time, overlaying the previous backup."
                    print " "
                    print "           If you need these files for diagnostic purposes,"
                    print "           you may wish to copy them to another directory"
                    print "           before running this script again."
                    break
            except:
                # Success!  We found a file name that did not open!
                break
        #endWhile

        #
        # Read in the last backup file to compare with the existing policy file
        # We will consistently trim trailing blank lines from files read to
        # avoid adding blank lines to the edited file and making backups of
        # files that differ only in trailing blank lines.
        #

        lastBuffer = ""
        if (lastFile != ""):
            try:
                lastDesc = open( lastFile, "r" )
                tmp_list = []
                while 1:
                    line = lastDesc.readline()
                    if (not line): break
                    if (line.strip() == ""): continue
                    tmp_list.append(line)
                #endwhile
                lastBuffer = ''.join(tmp_list)
                lastBuffer = lastBuffer + "\n"
                lastDesc.close()
            except:
                type, value, tbck = sys.exc_info()
                logDebug("Unable to read last backup = "+lastFile+"\nError Mesage = " + value)

        if (debugx_flag):
            logDebug(" Last buffer ="+lastBuffer+"<eof>")

        #
        # Read in the existing policy file, both to edit it and to back it up
        #
        try:
            policyDesc = open( policyFile, "r" )
        except:
            type, value, tbck = sys.exc_info()
            print " "
            print "*Error*  Failure opening policy descriptor to read"
            print "         File is "+policyFile
            print "         Error msg is " + value
            print " "
            print "         There is probably a permissions issue that needs to"
            print "         be corrected before the script can run successfully."
            print " "
            raise RuntimeError, "Unable to open "+policyFile+"\n"+value


        tmp_list = []
        while 1:
            line = policyDesc.readline()
            if (not line): break
            if (line.strip() == ""): continue
            tmp_list.append(line)
        oldBuffer = ''.join(tmp_list)
        oldBuffer = oldBuffer+"\n"
        policyDesc.close()

        buffer = oldBuffer
        if (debugx_flag):
            logDebug("   buffer ="+buffer+"<eof>")


        if (lastBuffer == oldBuffer):
            logDebug(" Last backup equals current server.policy, no backup needed")
        else:
            logDebug("Last backup does not equal current server.policy, backup needed")


        #
        # Read in the datacollector policy additions file.
        #
        # The contents of this file will be added to server.policy to configure
        # and removed to unconfigure.
        #

        try:
            dcDesc = open( dcFile, "r" )
        except:
            type, value, tbck = sys.exc_info()
            logDebug(" Error opening DC policy file "+dcFile+" to read")
            logDebug("  Error message is %s" % value)
            logDebug("File is obsolete, unconfiguration is unnecessary")
            logDebug("editServerPolicy() Exit")
            return

        tmp_list = []
        while 1:
             line = dcDesc.readline()
             if (not line): break
             if (line.strip() == ""): continue
             tmp_list.append(line)
        dcbuffer = ''.join(tmp_list)
        dcbuffer = dcbuffer+"\n"
        dcDesc.close()
        if (debugx_flag):
                logDebug(" dcbuffer="+dcbuffer+"<eof>")


        #
        # Remove any existing update from the buffer.
        # Obviously this is very dependent upon the content of the
        # datacollector.policy file.  The contents of this file have
        # not changed since WSAM became ITCAM.
        #

        # TCL regexps that would be helpful but don't work in JACL: (?:re) *? +? ??

        # JACL in WAS 5.1 does not support "end-1" as in "[lrange $array 0 end-1]".

        # JACL in WAS 5.1 has a bug in regular expression processing, so we have
        # to use "string first ..." and "string last ..." instead.

        # JACL in WAS 5.1 also doesn't support the 3rd parameter of "string first ..."

        # The left part is whatever preceeds our inserted text
        # The mid part is our inserted text
        # The right part is whatever follows our inserted text

        leftPart = buffer
        midPart = ""
        rightPart = ""

        p = buffer.rfind("\n// Licensed Materials - Property of IBM")
        if (p != -1):
            logDebug("Insert begins at  %s" % str(p))
            q = buffer[p:].find("datacollector.policy")
            if (q != -1):
                q = q + p
                logDebug("Confirmation begins at %s" % str(q))
                r = buffer[q:].find("\n};")
                if (r != -1):
                    r = r+q
                    logDebug("Insert ends at %s" % str(q))
                    leftPart = buffer[0:p+1]
                    p = p+1
                    r = r+2
                    midPart = buffer[p:r+1]
                    r = r+1
                    rightPart = buffer[r:]
                else:
                    logDebug("No end of insert found")
            else:
                 logDebug(" No confirmation found")

        if (debugx_flag):
            logDebug(" Left part ="+leftPart+"<eof>")

        if (debugx_flag):
            logDebug("Mid part ="+midPart+"<eof>")

        if (debugx_flag):
            logDebug("Right part ="+rightPart+"<eof>")


        if (midPart != ""):
            print "Existing standard update found in server policy"
            log = []
            lines = leftPart.split("\n" )
            if ("end" not in lines and len(lines) > 1):
                #lines = lines[0, (len(lines) - 2)]
                lines=lines[:-1]

            log.append([["old", lines]])

            lines = midPart.split("\n" )
            #_J2J_bracket_ = lines["end"]
            if ("end" not in lines and len(lines) > 1):
                #lines = lines[0, (len(lines) - 2)]
                lines=lines[:-1]

            log.append([["del", lines]])

            lines = rightPart.split("\n" )
            #_J2J_bracket_ = lines["end"]
            if ("end" not in lines and len(lines) > 1):
                #lines = lines[0, (len(lines) - 2)]
                lines=lines[:-1]


            log.append([["old", lines]])

            leftPart = leftPart.rstrip(" \t\n")
            leftPart = leftPart+"\n"
            if (debugx_flag):
                logDebug(" Trimmed left part ="+leftPart+"<eof>")


            rightPart = rightPart.rstrip(" \t\n")
            rightPart = rightPart+"\n"
            if (debugx_flag):
                logDebug(" Trimmed right part ="+rightPart+"<eof>")


            buffer = leftPart+rightPart
            rightPart = buffer.rstrip(" \t\n")
            rightPart = buffer+"\n"
        else:
            print "No previous update found in server.policy"
            log = []

            lines = buffer.split("\n" )
            #_J2J_bracket_ = lines["end"]
            if ("end" not in lines and len(lines) > 1):
                #lines = lines[0, (len(lines) - 2)]
                lines=lines[:-1]

            log.append([["old", lines]])

        if (debugx_flag):
            logDebug("Cleaned buffer ="+buffer+"<eof>")


        # If we are configuring, add the datacollector.policy file to the server.policy file.

        if (action == "config"):
            print "The server.policy file is being configured."
            buffer = buffer+"\n\n\n\n"+dcbuffer

            lines = dcbuffer.split("\n" )
            #_J2J_bracket_ = lines["end"]
            if ("end" not in lines and len(lines) > 1):
                #lines = lines[0, (len(lines) - 2)]
                lines=lines[:-1]
            log.append([["new", lines]])
        else:
            print "The server.policy file is being unconfigured."


        logDebug("Changes for server.policy:")
        for log1 in log:
            log0 = log1[0][0] + ":"
            if (log0 == "old:"):
                log0 = "    "
            for log2 in log1[0][1]:
                logDebug(" %s %s " % (log0, log2))

        if (debugx_flag):
            logDebug("    Updated buffer ="+buffer+"<eof>")

        # This check is defered to this point in order to the debug dump of the edit
        # to be done.

        if (leftPart.rstrip(" \t\n") == ""):
            print " "
            print "*Error*  The current WebSphere server.policy file appears to"
            print "         be empty or invalid."
            print "         File is "+policyFile
            print " "
            print "         It is possible the file has been edited incorrectly."
            print "         Either restore the server.policy file from a known"
            print "         good backup and rerun "+SCRIPTNAME+", or contact IBM"
            print "         support for assistance."
            print " "
            raise RuntimeError, "Unable to edit "+policyFile


        if (lastBuffer == buffer):
            logDebug(" Backup not needed, matches updated file")


        #
        # Write the updated buffer back over the policy file, if it was changed
        #
        # buffer     - The updated buffer to be written to server.policy
        # oldBuffer  - The original unedited contents of server.policy
        # lastBuffer - The last backup of server.policy we were able to read, or ""
        #

        if (oldBuffer != buffer):

            # Create a backup copy of the server policy file if needed
            # It will not be needed if the last backup matched either the
            # current file or the updated one we're about to write.

            if (lastBuffer != oldBuffer  and lastBuffer != buffer):
                print "Creating backup #"+i+"..."
                try:
                    backupDesc = open( backupFile, "w" )
                    backupDesc.write(oldBuffer)
                    backupDesc.close()
                except:
                    type, value, tbck = sys.exc_info()
                    print " "
                    print "*Error*  Error opening backup file to write"
                    print "         File name is " + backupFile
                    print "         Error msg is " + str(value)
                    print " "
                    print "         There is probably a permissions problem that must be"
                    print "         corrected for the script to run successfully."
                    print " "
                    raise RuntimeError, "Unable to create a backup of the server policy file\n %s" % value
            else:
                    print "Backup # %d bypassed, it is not needed" % i


            # Write the updates

            try:
                policyDesc = open( policyFile, "w" )
                if (debugx_flag):
                    logDebug(" Final buffer ="+buffer+"<eof>")

                policyDesc.write(buffer)
                policyDesc.close()
                print "The server.policy file has been updated."
            except:
                type, value, tbck = sys.exc_info()
                print " "
                print "*Error*  Error opening server policy file to write"
                print "           File name is "+policyFile
                print "           Error msg is "+str(value)
                print " "
                print "           There is probably a permissions problem that must be"
                print "           corrected for the script to run successfully."
                print " "
                raise RuntimeError, "Unable to open server policy file to write\n %s" % value

        else:
                print "Update unnecessary, no changes are needed to file contents."


        logDebug(" editServerPolicy() Exit")



def configSystemProperties ( jvmId, newSysProperties ):
    '''
        config
        Procedure configSystemProperties
        Arguments java system properties
    '''
    global AdminConfig
    global description_entry
    global old_am_home
    
    logDebug("configSystemProperties() entry")
    logDebug("JVM ID = %s" % jvmId)
    
    logDebug("System properties to create:" )
    for newSysProperty in newSysProperties:
        logDebug("   %s" % newSysProperty)

    print "Create/Modify JVM system properties"

    oldSysPropertyIds = wsadminToList(AdminConfig.list("Property", jvmId ))
    for newProperty in newSysProperties:
        #print "newProperty=%s" % newProperty
        logDebug("newProperty=%s" % newProperty)
        newProperty.append(description_entry)
        newName = newProperty[0][1]
        newValue = newProperty[1][1]
        #print "new "+newName+" = \""+newValue+"\""
        logDebug("new %s =%s" % (newName,newValue))
        objectId = ""
        for oldPropertyId in oldSysPropertyIds:
            if (oldPropertyId == ""): continue
            oldPropertyName = AdminConfig.showAttribute(oldPropertyId, "name" )
            if (oldPropertyName == "am.home"):
                if old_am_home == None or old_am_home == "":
                    old_am_home = get_am_home(oldPropertyId)
                debug("old_am_home=%s" % old_am_home)
            if (oldPropertyName == newName):
                #print "System property "+oldPropertyName+" exists"
                logDebug("System property "+oldPropertyName+" exists")
                objectId = oldPropertyId
                break


        if (objectId != ""):
            oldValue = AdminConfig.showAttribute(objectId, "value" )
            if (oldValue != newValue):
                print "Modify property "+newName+" from \""+oldValue+"\" to \""+newValue+"\""
                AdminConfig.modify(objectId, newProperty )
            else:
                print "No change for SystemProperty "+newName+" = "+oldValue

        else:
            print "create new property "+newName+" = \""+newValue+"\""
            AdminConfig.create("Property", jvmId, newProperty, "systemProperties" )



    logDebug("configSystemProperties() exit")


# unconfig
##############################################################
#
#  Procedure  unconfigServerVariable
#  Arguments: variable name
#
##############################################################

def unconfigServerVariable ( varname ):
        global AdminConfig
        global AdminTask
        global nodeName, serverName, serverid
        global variableMap, variableMapArray
        global quiet_flag

        logDebug("unconfigServerVariable("+varname+") entry")

        initServerVariableMapCache( )

        ent = variableMapArray.get(varname, None)
        if (ent != None):
            print "Removing WebSphere variable " + varname
            argString = u"[-variableName %s -scope Cell=%s,Node=%s,Server=%s ]" % (varname, wascell, wasnode, wasserver)
            print "Removing variables ARGS     %s" % (argString)
            AdminTask.removeVariable(argString)
            del variableMapArray[varname]
            variableMap = ""
        else:
            print "Variable "+varname+" not found, ignored"



        logDebug("unconfigServerVariable() exit")



# config
##############################################################
#
#  Procedure  mergePaths
#  Arguments: base classpath
#             new classpath
#  Returns  Updated classpath string
#
#  Given a new classpath and an old classpath, merges the
#  new paths into the old paths, with the new paths before the
#  existing paths.
#
#  Note: Empty braces may have been introduced in previous configuration
#
##############################################################

def mergePaths ( basePaths, newPaths ):

        logDebug("mergePaths entry" )
        if basePaths != None:
            basePaths = basePaths.strip()
        if newPaths != None:
            newPaths = newPaths.strip()
        logDebug("Base:   "+basePaths)
        logDebug("New:    "+newPaths)


        results = []
        used = []
        # Put in the new path entries first, which includes the option keyword
        for path in split_path(newPaths):
            if (path.strip( ) == ""): continue
            if (path.strip( ) == "{}"): continue
            logDebug("New path entry: "+path)
            used.append(path)
            results.append(path)


        # Now put in all base path entries which are not duplicate, ITCAM product, or obsolete
        # Note that removing the duplicates will handle the keyword option, if present
        for path in split_path(basePaths):
            if (path.strip( ) == ""):  continue
            if (path.strip( ) == "{}"):  continue
            logDebug("Base path entry: %s" % path)

            # Do not add entries we have already added
            if path in used:
                logDebug("Duplicate entry %s dropped." % path)
                continue
            if (path.find("${ITCAMDCHOME}") != -1):
                logDebug("ITCAM product entry dropped.")
                continue

            if (checkObsolete(path ) ):
                print " \n*Warning*  Obsolete entry "+path+" removed "
                print "           If this entry was not originally set by ITCAM,"
                print "           you may wish to replace it manually. "
                continue
            results.append(path)


        results = join( results, ":")
        result = results

        logDebug("mergePaths exit = %s " % results)
        return result


# unconfig
##############################################################
#
#  Procedure  unmergePaths
#  Arguments: existing paths
#             added paths
#  Returns  Updated classpath string
#
#  Given an existing classpath, removes paths provided in
#  the addedPath parameter.
#
#  Note: Empty braces may have been introduced in previous configuration
#
##############################################################

def unmergePaths ( currentPaths, addedPaths ):
        global AdminConfig
        global product
        global jvmId
        global serverid

        logDebug("  unMergePaths entry")
        logDebug("Current %s" % currentPaths)
        logDebug("Added: %s " % addedPaths)


        # Build a lookup array of the added paths
        used = []
        for path in split_path(addedPaths):
            if (path.strip( ) == ""): continue
            if (path.strip( ) == "{}"): continue
            logDebug(" Path entry: %s " % path)
            used.append(path)


        results = []

        # Remove added paths, ITCAM product paths, and obsolete paths
        logDebug("Current paths: %s " % currentPaths)

        for path in split_path(currentPaths):
            if (path.strip( ) == ""): continue
            if (path.strip( ) == "{}"): continue
            logDebug("Path entry: %s" % path)
            if (path in used):
                logDebug("Added entry dropped.")
                continue

            if (path.find("${ITCAMDCHOME}") != -1):
                logDebug("ITCAM product entry dropped.")
                continue

            if (checkObsolete(path) ):
                print " \n*Warning*  Obsolete entry "+path+" removed\n "
                print "           If this entry was not originally set by ITCAM,"
                print "           you may wish to replace it manually.\n "
                continue

            results.append(path)

        results = join( results, ":")
        result = results

        logDebug("unmergePaths exit %s" % result)
        return result



# config
##############################################################
#
#  Procedure  mergePathsAfter
#  Arguments: base classpath
#             new classpath
#  Returns  Updated classpath string
#
#  Given a new classpath and an old classpath, merges the
#  new paths into the old paths, with the new paths following the
#  existing paths.
#
#  Note: Empty braces may have been introduced in previous configuration
#
##############################################################

def mergePathsAfter ( basePaths, newPaths ):

        logDebug(" mergePathsAfter entry" )
        if basePaths != None:
            basePaths = basePaths.strip()
        if newPaths != None:
            newPaths = newPaths.strip()
        logDebug("Base: %s  " % basePaths)
        logDebug("New:  %s  " % newPaths)

        used = []
        # Build an array of new path entries for checking
        for path in split_path(newPaths):
            if (path.strip() == ""): continue
            if (path.strip() == "{}"): continue
            used.append(path)


        results = []

        # Now put in all base path entries which are not duplicate, ITCAM product, or obsolete
        for path in split_path(basePaths):
            if (path.strip( ) == ""): continue
            if (path.strip() == "{}"): continue
            logDebug("Base path entry: " + path)
            # Drop the entries we are going to add


            if (path in used):
                logDebug("Duplicate entry dropped. %s " % path)
                continue


            if (path.find("${ITCAMDCHOME}") != -1):
                logDebug("ITCAM product entry dropped. %s " % path)
                continue


            if (checkObsolete(path ) ):
                print " \n*Warning*  Obsolete entry "+path+" removed\n "
                print "           If this entry was not originally set by ITCAM,"
                print "           you may wish to replace it manually.\n "
                continue


            results.append(path)


        # Put in the new path entries after the existing paths
        for path in split_path(newPaths):
            if (path.strip( ) == ""): continue
            if (path.strip( ) == "{}"): continue
            logDebug("New path entry: %s" % path)
            results.append(path)


        results = join( results, ":")
        result = results

        logDebug("mergePaths exit =\n  %s" % results)
        return result




# Both
##################################################################
#
#  Procedure getServerVariableID
#  Arguments: variable name
#  Returns: Object ID for the server variable map
#
#  Note: Does lookup for server variables only.  Cache optimization
#        is not indicated because it is used to lookup the id for
#        AM_OLD_JVM_ARGS one time only.
##################################################################

def getServerVariableID ( name ):
        global AdminConfig
        global serverVariableEntries
        global nodeName, serverName, serverid

        logDebug("getServerVariableID(%s) entry" % name)

        if ( len(serverVariableEntries ) == 0 ):
            serverVariableEntries = []
            logDebug("serverVariableEntries not found")


        if (serverVariableEntries == ""):
            serverVariableEntries = wsadminToList(AdminConfig.getid("/Node:"+nodeName+"/Server:"+serverName+"/VariableMap:/VariableSubstitutionEntry:/" ))
            logDebug("  serverVariableEntries refreshed")

        # Look for this name
        varid = ""
        for entry in serverVariableEntries:
            logDebug("entry = %s" % entry)
            # Some WAS's return the parent object too.  Check only VariableSubstitutionEntry
            index = entry.find("#VariableSubstitutionEntry_")
            if (index == -1):  continue
            try:
                varName = AdminConfig.showAttribute(entry, "symbolicName" )
            except:
                _type_, _value_, _tbck_ = sys.exc_info()
                errmsg = str(_value_)
                logDebug("No symbolicName for server entry %s" % entry)
                print " \n*Warning*  Invalid WAS server variable object %s" % (entry)
                print "           " + errmsg + "\n "
                continue
            if (name == varName):
                varid = entry
                break

        logDebug("getServerVariableID(%s)='%s' exit" % (name, varid))

        return varid


# both
##############################################################
#
#  Procedure  saveChanges
#  Arguments: modified
#
##############################################################

def saveChanges (  ):
        global AdminConfig
        global AdminControl
        global nodeName, serverName, serverid
        global runtime_path
        global itcamdc_home
        global SCRIPT, timestamp
        global configComplete

        logDebug("saveChanges() entry")

        #
        # Validate the server before attempting to save and display messages from log
        #

        print " "
        print "Validating state of server "+serverName+" on node "+nodeName
        logDebug(" Validating")
        try:
            AdminConfig.validate(serverid )
        except:
            _type_, _value_, _tbck_ = sys.exc_info()
            errmsg = str(_value_)
            print "Error  Failed to validate "+serverName
            print "Error message: "+errmsg+"\n "
            return "false"

        print "Validation completed successfully"

        # Extract and display the new log contents
        filename = subsNodeVariable("${LOG_ROOT}/wsadmin.valout", nodeName )
        print "WAS validation log file = "+filename
        try:
            inDesc = open( filename, "r" )
            buffer = ""
            tmp_list = []
            print " \n*** Validity log: ***\n "
            while 1:
                line = inDesc.readline()
                if (not line): break
                if (line.strip() == ""): continue
                print "%s" % line
                tmp_list.append(line)
            #endwhile
            buffer = ''.join(tmp_list)
            inDesc.close()

            print "*** End of log ***\n "

            dataDir = "data"
            fileName = "%s_%s_valCheck.log" % (nodeName, serverName)
            valFile = "%s%s%s%s%s" % (itcamdc_home, os.sep, dataDir, os.sep, fileName)
            try:
                valDesc = open( valFile, "w" )
                valDesc.write("%s Validity check log in %s for %s \n" % (timestamp, SCRIPT, serverName))
                valDesc.write(buffer)
                valDesc.close()
                logDebug("Validity check log file saved as %s" % valFile)
            except:
                _type_, _value_, _tbck_ = sys.exc_info()
                log.logException("Unable to copy validity check log")
                print " \n*Warning*  Unable to copy validity check log to %s\nError: %s" % (valFile, str(_value_))
        except:
            _type_, _value_, _tbck_ = sys.exc_info()
            print " \n*Warning*  Unable to open WAS validation log file\n Error: %s" % str(_value_)
            log.logException("Unable to open WAS validation log file")

        print " "
        print "Saving changes to server "+serverName+" on node "+nodeName

        logDebug("Saving")
        try:
            AdminConfig.save( )
            configComplete = 1
        except:
            _type_, _value_, _tbck_ = sys.exc_info()
            errmsg = str(_value_)
            print " \n*Error*  Failed to save"
            print "         Error message: " + errmsg + "\n "
            return "false"


        print "Saving complete"

        logDebug(" saveChanges exit")

        return "true"


def listSearch(list, value):
    '''
        return index of a value in the list
        returns -1 if no match found
    '''
    i = 0
    found = 0
    total = len(list)
    while i < total:
        item = list[i]
        if (item.startswith(value)):
            found = 1
            break
        i += 1
    if (found == 0): i = -1
    return i

# both
##############################################################
#
#  Procedure unconfigSystemProperties
#  Arguments server jvmId,
#            java system properties
#
##############################################################

def unconfigSystemProperties ( jvmId, newSysProperties ):
        global AdminConfig
        global quiet_flag

        logDebug("unconfigSystemProperties() entry")
        logDebug("JVM ID = "+jvmId)
        logDebug("System properties to remove:")
        for newSysProperty in newSysProperties:
            logDebug(newSysProperty)

        if ( not quiet_flag): print "Remove/Modify JVM system properties"
        oldSysPropertyIds = toList(AdminConfig.list("Property", jvmId ))

        logDebug("%d to examine" % len(oldSysPropertyIds))
        oldIds = {}
        for propid in oldSysPropertyIds:
            propName = AdminConfig.showAttribute(propid, "name" )
            propValue = AdminConfig.showAttribute(propid, "value" )
            logDebug("old ids %s=%s" % (propName, propValue))
            oldIds[propName] = [propName, [propid, propValue]]

        for newProperty in newSysProperties:
            objectId = ""
            newName = newProperty[0][1]
            newValue = newProperty[1][1]
            logDebug("To remove %s=%s" % (newName, newValue))
            ent  = oldIds.get(newName, None)
            if (ent != None):
                print "System property "+newName+" exists"
                objectId = ent[1][0]
                logDebug("Object ID = %s" % objectId)
            else:
                logDebug("object for %s not found" % newName )
            if (objectId != ""):
                print "Remove SystemProperty "+objectId
                AdminConfig.remove(objectId )
            else:
                if ( not quiet_flag):
                    print "Unable to remove systemProperty "+newName+", not present"
        logDebug(" unconfigSystemProperties exit")

# both
##############################################################
#
#  Procedure unconfigExtensionMBeanProvider
#  Arguments: xMBeanProv
#
##############################################################

def unconfigExtensionMBeanProvider ( xMBeanProv ):
        global AdminConfig
        global serverid
        global quiet_flag

        logDebug("unconfigExtensionMBeanProvider() entry")
        newName = ""
        newClasspath = ""
        for attr in xMBeanProv:
            attrName = attr[0]
            attrValue = attr[1]
            if ( attrName == "classpath" ):
                newClasspath = attrValue
            elif ( attrName == "name" ):
                newName = attrValue
            elif ( attrName == "default" ):
                print "ignoring "+attrName+"="+attrValue+" for ExtensionMBeanProvider"
        logDebug("    name="+newName)
        logDebug("classpath="+newClasspath)


        if (newName == ""):
            raise RuntimeError, "Missing name for ExtensionMBeanProvider\n "

        if (newClasspath == ""):
            raise RuntimeError, "Missing classpath for ExtensionMBeanProvider\n "


        hit = 0
        mbeanids = toList(AdminConfig.list("ExtensionMBeanProvider", serverid ))
        for tmpId in mbeanids:
            oldName = AdminConfig.showAttribute(tmpId, "name" )
            if (oldName == newName):
                print "Removing existing "+newName+" extension MBean provider"
                logDebug("Extension MBean Provider object = "+tmpId)
                logDebug("name        = "+oldName)
                try:
                    print "classpath=%s" % AdminConfig.showAttribute(tmpId, "classpath")
                    print "description =%s" % AdminConfig.showAttribute(tmpId, "description")
                except:
                    _type_, _value_, _tbck_ = sys.exc_info()
                    ermsg = str(_value_)
                    print "exception %s " % ermsg

                AdminConfig.remove(tmpId )
                hit = 1
                break


        if ( not hit  and  not quiet_flag):
                print "No existing "+newName+" extension MBean provider found, nothing to remove"

        logDebug("unconfigExtensionMBeanProvider() exit")



# unconfig
##############################################################
#
#  Procedure unconfigEnvs
#  Arguments config id, new environments
#
##############################################################

def unconfigEnvs ( javaProcId, updateEnvironments ):
        global AdminConfig
        global nodeName, serverName, serverid
        global quiet_flag

        logDebug(" unconfigEnvs() entry")
        if (quiet_flag == 0):
            print "Remove/Modify java process %d environments" % len(updateEnvironments)

        logDebug("nodeName="+nodeName+", serverName="+serverName)

        logDebug("%d updating environment entries" % len(updateEnvironments))
        for ue in updateEnvironments:
            pass
            #logDebug("%s=%s", % (ue[0][1],ue[1][1]) )

        # Read in the environment from the Java process

        logDebug("Reading in name/value pairs from existing environment")
        environs = wsadminToList(AdminConfig.showAttribute(javaProcId, "environment" ))
        print environs

        # If not z/OS.. zOS has two, LIBPATH and NLSPATH 
        if platform != "ZOS":  
            if (len(environs) != 1):
                print "Unexpected %d environment entries for %s, should be 1." % (len(environs), serverName)
                print "        Process will continue.  However, there may be a WAS configuration issue.\n "


        for envids in environs:
            print "'%s'" % envids
            logDebug("%d  environment IDs read for %s" % (len(envids.split()), serverName))
            for envid in envids.split():
                logDebug("envid: %s" % envid)


        #sys.exit(0)
        # Create the arrays from the update entries

        updates = {}
        hits = {}
        for ue in updateEnvironments:
            nm = ue[0][1]
            val = ue[1][1]
            logDebug("nm=%s val=%s" % (nm, val))
            updates[nm] = [nm, val]
            hits[nm] =  0


        logDebug("Updates specified:")
        for key in updates.keys():
            logDebug(" Update: %s=%s" % (key, updates[key]))



        # Process the environment variables
        # Special handling for paths such as NLSPATH and LIBPATH

        logDebug("   Processing environment entries:")

        for envids in environs:
            for envid in envids.split():
                # Validate the object id type
                index = envid.find("#Property_")
                if (index == -1): continue
                enm = AdminConfig.showAttribute(envid, "name" )
                logDebug("Environment name = "+enm)
                value = updates.get(enm, None)
                if (value == None):
                    logDebug("no match found")
                    continue
                nm = value[0]
                hits[nm] = 1
                eval = AdminConfig.showAttribute(envid, "value" )
                logDebug("Environment value = " + eval)
                val = value[1]
                logDebug("Update value      = "+val)
                # Must unmerge the path updates
                # We also remove anything that looks like an obsolete path entry

                if (enm == "NLSPATH" or enm == "LIBPATH" or enm == "LD_LIBRARY_PATH" or enm == "PATH" or enm == "SHLIB_PATH"):
                    logDebug("Merging "+enm+" with existing path list")
                    paths = split_path(val)
                    epaths = split_path(eval)
                    newPaths = []
                    for epath in epaths:
                        # Drop any existing path that is in the new paths
                        for path in paths:
                            if (path == epath):
                                logDebug(epath + " present in existing path")
                                epath = ""
                                break

                        if (epath == ""): continue
                        # Drop any ITCAMDCHOME path, it is assumed to be ours
                        if (epath.find("${ITCAMDCHOME}") != -1):
                            logDebug("Existing "+epath+" is an old ITCAM entry and will be dropped")
                            continue

                        # Drop any paths from a previous release
                        if (checkObsolete(epath ) ):
                            print " \n*Warning*  Obsolete "+enm+" entry removed: "+epath+"\n "
                            print "           If this entry was not originally set by ITCAM,"
                            print "           you may wish to manually replace it.\n "
                            continue

                        logDebug("Existing "+epath+" entry will be retained in "+enm)
                        newPaths.append(epath)

                    pathDelimiter = ":"
                    if platformUtil.isWindows():
                        pathDelimiter = ";"
                    newPaths = join( newPaths, pathDelimiter)
                    newVal = newPaths

                    # If there is no remaining path, remove it.
                    # Otherwise replace the environment variable.

                    if (newVal == ""):
                        print "Removing path environment variable "+enm+" = "+eval
                        AdminConfig.remove(envid )
                    else:
                        if (newVal == eval):
                            print "Path environment variable "+nm+" already set: "+eval
                        else:
                            print "Restoring path environment variable "+nm+" to "+newVal
                            newEnv = [["name", nm], ["value", newVal]]
                            AdminConfig.modify(envid, newEnv )


                    continue


                if ( not quiet_flag):
                    if (val != eval):
                            print " \n*Warning*  Current value or "+enm+" does not match its update value"
                            print "           Update value  = "+val
                            print "           Current value = "+eval+"\n "
                            print "           If this entry was not originally set by ITCAM,"
                            print "           you may wish to manually replace it.\n "

                print "Removing environment variable "+enm+" = "+eval
                AdminConfig.remove(envid )

        if ( not quiet_flag):
            space_flag = 1
            for nm in hits.keys():
                if (hits[nm]): continue
                if (space_flag):
                    print " "
                    space_flag = 0

                print "*Info*  Update variable "+nm+" not found in current environment for "+serverName+"."

            if ( not space_flag):
                print " "



        logDebug("unconfigEnvs() exit")


# both
##############################################################
#
#  Procedure  unconfigObsolete
#
#  Remove any obsolete items.
#
##############################################################


def unconfigObsolete ( platform ):
        global AdminConfig
        global product
        global servantJvmId
        global javaProcServant
        global javaProcControl
        global serverid
        global quiet_flag
        global server_amhome_flag
        global JVMSecurityPolicy_flag
        global ecam_flag
        global controlJvmId
        global server_classpath

        logDebug(" unconfigObsolete() entry")


        # Don't issue messages for missing obsolete parameters
        quiet_flag = 1

        # Get list of system properties for the servant JVM.  (It is a list of lists.)
        # Find the obsolete property java.security.policy and remove it if the file name
        # value looks okay.
        sysPropIds = toList(AdminConfig.showAttribute(servantJvmId, "systemProperties" ))
        logDebug("%d properties in the list to examine" %  len(sysPropIds))

        for propids in sysPropIds:
            propids = wsadminToList(propids)
            logDebug("%d properties to examine" % len(propids))
            for propid in propids:
                logDebug("Property ID = "+propid)
                index = propid.find("#Property_")
                if (index == -1): continue
                propName = AdminConfig.showAttribute(propid, "name" )
                logDebug("propName= %s" % propName)
                if (propName == "java.security.policy"):
                    propValue = AdminConfig.showAttribute(propid, "value" )
                    logDebug("System property "+propName+" found")
                    logDebug("value = \""+propValue+"\"")
                    logDebug("Object ID = "+propid)
                    # Remove obsolete wsam entry we know about
                    removeFlag = 0
                    if (propValue.find("/wsam/") != -1): removeFlag = 1
                    if (is_zOS(platform)):
                        if (propValue.find("/etc/datacollector.policy") != -1): removeFlag = 1
                    else:
                        if (propValue.endswith("datacollector.policy")): removeFlag = 1    
                    index = propValue.find("${ITCAMDCHOME}")
                    if index != -1: removeFlag = 1
                    if checkObsolete(propValue): removeFlag = 1
                    if (removeFlag):
                        print "Remove SystemProperty "+propName
                        AdminConfig.remove(propid )
                    else:
                        print " \n*Warning*  Unable to remove "+propName+", value"
                        print "           \""+propValue+"\" is not recognized.\n "
                        print "           You may wish to review and manually remove this entry.\n "

        # Custom Services
        # com.cyanea.ws5.ProbeService
        # com.cyanea.ws6.ProbeService
        # com.cyanea.ws6.ITCAMNotifierCustomService

        logDebug(" Removing obsolete Custom Service")
        customServiceIds = toList(AdminConfig.list("CustomService", serverid ))
        for tmpId in customServiceIds:
            logDebug("Service ID = %s" % tmpId)
            serviceName = AdminConfig.showAttribute(tmpId, "displayName" )
            logDebug("Service name = \""+serviceName+"\"")
            if (serviceName != "am"): continue
            className = AdminConfig.showAttribute(tmpId, "classname" )
            logDebug("Class name = \""+className+"\"")
            obsolete_flag = 0
            if (className == "com.cyanea.ws5.ProbeService"): obsolete_flag = 1
            if (className == "com.cyanea.ws6.ProbeService"): obsolete_flag = 1
            if (className == "com.cyanea.ws6.ITCAMNotifierCustomService"): obsolete_flag = 1
            if (obsolete_flag):
                print "Obsolete Custom Service exists, removing "+serviceName+"/"+className
                AdminConfig.remove(tmpId )
            else:
                 logDebug("Class name does not match, retaining")
        # com.ibm.websphere.management.enableConfigMBean in AdminService no longer needed
        adminSvc = AdminConfig.list("AdminService", serverid )
        adminProperties = toList(AdminConfig.list("Property", adminSvc ))
        for property in adminProperties:
            if (AdminConfig.showAttribute(property, "name" ) == "com.ibm.websphere.management.enableConfigMBean"):
                value = AdminConfig.showAttribute(property, "value" )
                print "AdminService property com.ibm.websphere.management.enableConfigMBean = "+value
                if (value == "true"):
                    print "removing com.ibm.websphere.management.enableConfigMBean true"
                    AdminConfig.remove(property )

                break
        # The system property ws.instrument is obsolete but it may also be used
        # by customers for their own reasons, so we can not safely remove it.

        # The am.enableCaptureZStub is a parameter invented by ITCAM so we may
        # safely remove it.

        # Servant system properties
        # am.enableCaptureZStub
        # am.ejb.rmilistener.enable
        # am.appserver
        # am.home - treat as obsolete only if am.home value is not supplied in input.properties
        # am.nodename
        # appserver.platform
        # com.ibm.tivoli.itcam.toolkit.a.bcm.helper.class
        # com.ibm.tivoli.itcam.toolkit.a.bcm.helper.classpath
        # dc.operation.mode
        # itcam61.home
        # java.rmi.server.codepage
        # jlog.common.dir
        # appserver.rmi.host
        # deploymentmgr.rmi.connection

        # Note although protocol_http_timeout_output_recovery was formerly
        # set by ITCAM configuration, we can not unset it as it is not
        # owned by us.

        # 
        # Identify and unconfigure obsolete server system properties 
        #
        props = []
        props.append([["name", "am.enableCaptureZStub"], ["value", []]])
        props.append([["name", "am.ejb.rmilistener.enable"], ["value", []]])
        props.append([["name", "am.appserver"], ["value", []]])
        # If server.systemproperty am.home is not supplied this run, handle as obsolete
        if ( not server_amhome_flag):
            props.append([["name", "am.home"], ["value", []]])

        props.append([["name", "am.nodename"], ["value", []]])
        props.append([["name", "appserver.platform"], ["value", []]])
        props.append([["name", "com.ibm.tivoli.itcam.toolkit.ai.bcm.helper.class"], ["value", []]])
        props.append([["name", "com.ibm.tivoli.itcam.toolkit.ai.bcm.helper.classpath"], ["value", []]])
        props.append([["name", "dc.operation.mode"], ["value", []]])
        props.append([["name", "itcam61.home"], ["value", []]])
        props.append([["name", "java.rmi.server.codebase"], ["value", []]])
        props.append([["name", "jlog.common.dir"], ["value", []]])
        props.append([["name", "appserver.rmi.host"], ["value", []]])
        props.append([["name", "deploymentmgr.rmi.connection"], ["value", []]])
        # If ecam property not supplied, consider "ITCAM_DC_ENABLED" obsolete
        if ( not ecam_flag):
            props.append([["name", "ITCAM_DC_ENABLED"], ["value", []]])
        unconfigSystemProperties(servantJvmId, props )


        # 
        # Identify and unconfigure obsolete control region system properties 
        #
        # Control region system properties
        if (is_zOS(platform)):
            props = []
            if ( not ecam_flag):  
               props.append([["name", "ITCAM_DC_ENABLED"], ["value", []]])

            if (len(props) != 0):
                unconfigSystemProperties(controlJvmId, props )
    

        # We may have set these but it would be disruptive to modify the values at this point
        # Servant protocol_iiop_propagate_unknown_service_ctxs
        # Control environment server_SMF_container_activity_enabled
        # Control environment server_SMF_container_interval_enabled
        # Control environment server_SMF_interval_length
        # Control environment server_SMF_server_activity_enabled
        # Control environment server_SMF_server_interval_enabled

        # ITCAM61HOME AM_HOME MS_AM_HOME

        # The catch logic is here because it was discovered during testing
        # that updating variables can put them in a state where the variables
        # exist but are not yet saved into variables.xml.  When we attempt
        # remove such variables we get an error, but restarting the AP server
        # or running wsc2n.sh resolves the problem and completes the remove.
        # Customers might encounter this situation so we render it harmless
        # with the catch so that the save will be completed and wsc2n.sh
        # will run.

        try:
            unconfigServerVariable("ITCAM61HOME" )
        except:
            _type_, _value_, _tbck_ = sys.exc_info()
            errmsg = str(_value_)
            print " \n*Warning*  Unable to remove ITCAM61HOME variable\n           errmsg="+errmsg+"\n "

        try:
            unconfigServerVariable("AM_HOME" )
        except:
            _type_, _value_, _tbck_ = sys.exc_info()
            errmsg = str(_value_)
            print " \n*Warning*  Unable to remove AM_HOME variable\n           errmsg="+errmsg+"\n "

        try:
            unconfigServerVariable("MS_AM_HOME" )
        except:
            _type_, _value_, _tbck_ = sys.exc_info()
            errmsg = str(_value_)
            print " \n*Warning*  Unable to remove MS_AM_HOME variable\n           errmsg="+errmsg+"\n "


        # Servant environment variables
        # CCLOG_COMMON_DIR environment variable

        envs = []
        envs.append([["name", "CCLOG_COMMON_DIR"], ["value", []]])
        unconfigEnvs(javaProcServant, envs )

        # Remove any Extension MBean Provider set by ITCAM in obsolete code

        unconfigExtensionMBeanProvider([["name", "ITCAM"], ["classpath", "n/a"]] )

        # eCAM doesn't configure the injector, but it may have been configured by ITCAM earlier
        # In any case, the injector is obsolete across the board

        unconfigInjector( )

        # Now we clean up the descriptions in the SMF environment variables.
        # We do not wish to modify them, as the customer may have already
        # have them turned on.  But we need to at least remove the description
        # comment so that it no longer references ITCAM.

        # List of names to remove if they are present and description indicates ITCAM
        remove_envnames = []
        remove_envnames.append("protocol_http_timeout_output_recovery")

        # List of names for which we will only clean the description
        nodesc_envnames = []
        if (is_zOS(platform)):
            nodesc_envnames.append("server_SMF_server_interval_enabled")
            nodesc_envnames.append("server_SMF_container_interval_enabled")
            nodesc_envnames.append("server_SMF_interval_length")


        # Get a clean list of environment properties for the control region
        # (This complex code may not be needed any more, but it is not hurting anything.)
        environments = []
        if (is_zOS(platform)):
            for envid in toList(AdminConfig.list("Property", javaProcControl )):
                index = envid.find("#Property_")
                if (index == -1): continue
                environments.append(envid)



        # Remove the properties which we feel may be safely removed
        for envname in remove_envnames:
            # Is an environment property with this name present?
            i = listSearch(environments,envname + "(")
            if (i > -1):
                envid = environments[i]
                # Does it have an ITCAM description?
                if (AdminConfig.showAttribute(envid, "description" ).find("ITCAM ") == 0):
                    # Remove the property
                    AdminConfig.remove(envid )
                    print envname+" control region environment property removed"
                else:
                    logDebug("%s was not set by ITCAM" % envname)

            else:
                logDebug(envname+" is not a current environment property")



        # Clean up the descriptors for properties we do not want to change
        for envname in nodesc_envnames:
            # Is an environment property with this name present?
            i = listSearch(environments,envname + "(")
            if (i > -1):
                # Does it have an ITCAM description?
                envid = environments[i]
                if (AdminConfig.showAttribute(envid, "description" ).find("ITCAM ") == 0):
                    # Blank out the description string for this property
                    AdminConfig.modify(envid, [["description", []]] )
                    print envname+" control region environment property retained"
                else:
                    logDebug(envname+" was not set by ITCAM")

            else:
                logDebug(envname+" is not a current environment property")



        # Clean up obsolete servant JVM arguments
        # ITCAM-supplied argument -Djava.security.policy is considered obsolete if current input.properties
        # does not contain this argument

        logDebug("Cleanup of obsolete JVM arguments")

        changed_flag = 0
        jvmArgs = ""
        genericJvmArgs_list = AdminConfig.showAttribute(servantJvmId, "genericJvmArguments" ).split( )
        for arg in genericJvmArgs_list:
            if (arg == ""): continue
            if ( not JVMSecurityPolicy_flag):
                if (arg.find("-Djava.security.policy=${ITCAMDCHOME}") == 0):
                    print "Removing obsolete JVM argument: "+arg
                    changed_flag = 1
                    continue

            if (arg.find("-Djlog.propertyFileDir.CYN=") == 0):
                print "Removing obsolete JVM argument: "+arg
                changed_flag = 1
                continue

            if (arg.find("-Djlog.common.dir.CYN=") == 0):
                print "Removing obsolete JVM argument: "+arg
                changed_flag = 1
                continue

            if (arg.find("-Dibm.common.log.dir=") == 0):
                print "Removing obsolete JVM argument: "+arg
                changed_flag = 1
                continue

            logDebug("Retaining JVM argument: %s" % arg)
            if (jvmArgs != ""):
                jvmArgs = jvmArgs + " " + arg
            else:
                jvmArgs = arg



        if (changed_flag):
            logDebug(" Updating cleaned JVM arguments")
            AdminConfig.modify(servantJvmId, [["genericJvmArguments", jvmArgs]] )
        else:
            logDebug("No changes to JVM arguments, update bypassed")


        # Clean up obsolete and residual ITCAM classpath entries if current input.properties contains no server.classpath.
        # Do not issue "$AdminConfig modify" if unsetAttributes fails; the modify concatenates.

        logDebug("Cleanup of residual and obsolete server classpath entries")

        changed_flag = 0
        if (server_classpath == ""):
            serverClasspath = AdminConfig.showAttribute(servantJvmId, "classpath" )
            if (serverClasspath.find("${ITCAMDCHOME}") > -1):
                newServerClasspath = mergePaths(serverClasspath, " " )
                parm = [["classpath", newServerClasspath]]
                if (newServerClasspath == ""):
                    print "Removing residual server classpath: "+serverClasspath
                else:
                    print "Updating server classpath to: "+newServerClasspath

                changed_flag = 1
                try:
                    AdminConfig.unsetAttributes(servantJvmId, "classpath" )
                    AdminConfig.modify(servantJvmId, parm )
                except:
                    _type_, _value_, _tbck_ = sys.exc_info()
                    ermsg = str(_value_)
                    print "********************* "
                    print "*Warning* Cleanup of residual and obsolete ITCAM classpath entries was unsuccessful."
                    print "*Warning* errorInfo: " + ermsg
                    print "********************* "


        if ( not changed_flag):
            logDebug("No changes to server classpath, update bypassed")


        quiet_flag = 0

        logDebug("unconfigObsolete exit")



##############################################################
#
#  Procedure setPMIStatisticSet
#  Arguments: server config id,
#             statistic set (e.g. "basic", "custom"...)
#
##############################################################
def setPMIStatisticSet (serverId, statisticSet ):
        global AdminConfig

        logDebug("setPMIStatisticSet() entry serverId=%s statisticsSet=%s" % (serverId, statisticSet))


        PMIServiceId = toList(AdminConfig.list("PMIService", serverId ))

        if (len(PMIServiceId) == 0):
            raise RuntimeError, "PMIService doesn't exist"

        if (len(PMIServiceId) > 1):
            raise RuntimeError, "more than 1 PMIService exists"

        logDebug("modify PMI statistic set to %s" % statisticSet)
        logDebug("Modifying PMI serversID '%s'" % PMIServiceId[0])
        AdminConfig.modify(PMIServiceId[0], [["statisticSet", statisticSet]] )
        logDebug("modify PMI statistic set complete")


        logDebug("setPMIStatisticSet() exit")



# both
##############################################################
#
#  Procedure unconfigExtraRMI
#  Arguments server jvmId,
#            java system properties
#
#  When deploymentmgr.rmi.connection is configured, then
#  appserver.rmi.host should be unconfigured, and vice-versa.
#  This applies to both config and unconfig.
##############################################################

def unconfigExtraRMI ( jvmId, systemproperties ):
    global AdminConfig

    logDebug("unconfigExtraRMI() entry")
    logDebug("JVM ID = " + jvmId)
    logDebug("System properties to examine: ")
    for newSysProperty in systemproperties:
        logDebug("property=" + newSysProperty)
    remove_deploymentmgr = 0
    remove_appserver = 0
    for propent in systemproperties:
        if (propent[0][1] == "deploymentmgr.rmi.connection"):
            remove_appserver = 1
        if (propent[0][1] == "appserver.rmi.host"):
            remove_deploymentmgr = 1

    props = []
    if (remove_appserver):
        print "ND mode, we will make sure appserver.rmi.host is not present"
        props.append([["name", "appserver.rmi.host"], ["value", []]])

    if (remove_deploymentmgr):
        print "BASE mode, we will make sure deploymentmgr.rmi.connection is not present"
        props.append([["name", "deploymentmgr.rmi.connection"], ["value", []]])

    if (len(props) != 0):
        logDebug("Unconfiguring system properties")
        unconfigSystemProperties(jvmId, props )


    logDebug("unconfigExtraRMI exit")


def printError(pname, line):
    print " \n*Error*  Invalid format, entry ignored"
    print "         Input file has an invalid entry."
    print "         Script error must be corrected\n "
    errorCode = None
    raise RuntimeError, "input.properties invalid parameter '%s' in line '%s'" % (pname, line)

def configure (input_record, progress):
    '''
        Configure
        @parm  input_record - Record form the input list file.
        @param progress - progress object reference
    '''
    global SCRIPT, VERSIONSTRING, timestamp
    #timestamp = strftime("%Y-%m-%d %H:%M:%S")
    idline = "#  "+SCRIPT+" Version "+VERSIONSTRING  + " " + timestamp + " #"
    blanks = "                                                                                                             "
    hashes = "#############################################################################################################"
    print " "
    print hashes[1:len(idline)+1]
    print "#" + blanks[1:len(idline) - 1] + "#"
    print idline
    print "#" + blanks[1:len(idline) - 1] + "#"
    print hashes[1:len(idline)+1]
    print " "

    global doSynchronize

    global AdminConfig
    global product
    global platform
    global nodeName
    global serverid
    global customservice
    global systemproperties
    global variables
    global control_environments
    global control_properties
    global genericargs
    global sharedLib
    global serverVersion
    global pipusc
    global pipusc_value
    global server_runtime
    global javaProcControl
    global javaProcServant
    global itcamdc_home
    global server_classpath
    global server_bootclasspat
    global smf
    global enableconfigmbean
    global containertimeouts
    global servantJvmId
    global controlJvmId
    global serverName
    global quietFlag
    global runtime_path
    global server_amhome_flag
    global JVMSecurityPolicy_flag
    global ecam_flag
    global maximumHeapSize
    global probe
    global wascell
    global wasnode
    global wasserver
    global current_inputproperties
    global config_type
    global common_constants
    global old_am_home
    global isStandalone
    global configComplete

    product = ""
    platform = ""
    serverid = ""
    customservice = []
    systemproperties = []
    environments = []
    variables = []

    # z/OS control region environment
    #control_environments = ""
    #control_properties = ""
    control_classpath = ""
    genericargs = ""
    sharedLib = ""
    serverVersion = ""
    pipusc = ""
    pipusc_value = ""
    server_runtime = ""
    javaProcControl = ""
    javaProcServant = ""
    itcamdc_home = ""
    server_classpath = ""
    server_bootclasspath = ""
    server_pmiservice_statisticSet = ""
    smf = "No"
    containertimeouts = ""
    doSynchronize = 1
    server_amhome_flag = 0
    JVMSecurityPolicy_flag = 0
    ecam_flag = 0
    control_environments = []
    control_properties = []
    isStandalone = 0
    configComplete = 0

    # This flag is turned on to enable the PMI service.  We do not
    # disable the PMI service under any condtion.

    pmiService_flag = 0
    extraRMI_flag = 0

    quiet_flag = 0
    failed_flag = 0

    old_am_home = ""

    #
    # The input file record has the following format:
    # 1st position = type that caused the config to be processed (config, reconfig, migrate)
    #                the type is an integer number.
    # 2nd position = 1 if server needs to be notified
    #                0 no need to notifiy server
    # 3rd position = Cell name
    # 4th position = Node name
    # 5th position = Server name
    # 6th position = input list location
    # The entries are separated by a "^".
    #
    input_entries = input_record.split("^")
    if (len(input_entries) != 6):
        print "input record is invalid - %s" % (input_record)
        raise RuntimeError, " input record is invalid - %s\n" %(input_record)
    else:
        config_type = input_entries[0]
        probe = input_entries[1]
        wascell = input_entries[2]
        wasnode = input_entries[3]
        wasserver = input_entries[4]
        inputfile = input_entries[5]
        current_inputproperties = inputfile
        print "configtype: %s  probe: %s  WAS cell: %s  WAS node: %s  WAS server: %s  inputfile: %s" % (config_type, probe, wascell, wasnode, wasserver, inputfile)
    #
    # Write the start message for the console in the progress file.
    #
    progress.write_progress_start(wasserver)

    # Read in the response file
    print "\nresponse file "+inputfile
    try:
        inDesc = codecs.open(inputfile, mode='r', encoding='utf-8')
        #inDesc = open( inputfile, "r" )
    except:
        errorCode = None
        raise RuntimeError, inputfile+" failed to open\n"

    lines = []
    while 1:
        buffer = inDesc.readline()
        if not buffer: break
        lines.append(buffer)
    inDesc.close()

    logDebug("%d lines read from %s" % (len(lines),inputfile))

    # Note: Any value which may contain equal signs must
    #       be handled with regexp instead of lindex.
    # Comments are not allowed with a switch (probably JACL bug)
    # Extension MBean Provider is obsolete
    #   server.xMBeanProvider
    # These parameters, while still valid, should be replaced by server.variable specifications
    #   server.itcamdchome server.itcam61home
    # These two parameters are obsolete so they are ignored
    #   server.am.home server.ms.am.home
    # Note:server.systemproperty=am.home is used in ITCAM 7.2(ecam) so it's conditionally obsolete.
    # These two options are now obsolete but retained to process input files
    # created previously.  They are only needed for unconfigDataCollector.jacl
    #   server.pmiservice.initialSpecLevel server.pmiservice.statisticSet

    print " "
    for line in lines:
        if (line.strip() == ""):
            #logDebug("blank line")
            continue
        elif (line.lstrip()[0] == "#"):
            #logDebug("comment: %s" % line)
            continue
        else:
            line = line.strip()
            tokens = line.split("=" )
            pname = tokens[0]
            #logDebug("pname=" + pname)
            logDebug("property_line=%s" % line)
            if ( pname == "product" ):
                product = tokens[1]
            elif ( re.compile("server\.genericJvmArguments[0-9]*").match(pname) ):
                if (len(line.split('=', 1)) == 2):
                    tmpargs = line.split('=', 1)[1].strip()
                    #TODO
                    genericargs += " " + tmpargs.strip()
                    logDebug("tmpargs=%s" % tmpargs)
                    if (tmpargs.find("-Djava.security.policy") == 0):
                        JVMSecurityPolicy_flag = 1
                else:
                    printError(pname, line)
            elif ( re.compile("server\.systemproperty[0-9]*").match(pname) ):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpname = tmpname.strip()
                    tmpvalue = tmpvalue.strip()
                    if (tmpname == "am.appserver"):
                        serverName = tmpvalue
                    if (tmpname == "am.nodename"):
                        nodeName = tmpvalue
                    if (tmpname == "deploymentmgr.rmi.connection"):
                        extraRMI_flag = 1
                    if (tmpname == "appserver.rmi.host"):
                        extraRMI_flag = 1
                    if (tmpname == "am.home" and tmpvalue != ""):
                        server_amhome_flag = 1
                    if (tmpname == "ITCAM_DC_ENABLED"):
                        ecam_flag = 1
                    systemproperties.append([["name", tmpname], ["value", tmpvalue]])
                else:
                    printError(pname, line)
            elif ( re.compile("server\.environment[0-9]*").match(pname) ):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpvalue = tmpvalue.strip()
                    logDebug("environment %s=%s" % (tmpname, tmpvalue))
                    environments.append([["name", tmpname], ["value", tmpvalue]])
                else:
                    printError(pname, line)
            elif ( re.compile("server\.variable[0-9]*").match(pname) ):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpvalue = tmpvalue.strip()
                    variables.append([["symbolicName", tmpname], ["value", tmpvalue]])
                    if (tmpname == "ITCAMDCHOME" or tmpname == "ITCAM61HOME"):
                        itcamdc_home = tmpvalue
                    elif (tmpname != "AM_HOME" and tmpname != "MS_AM_HOME" and tmpname != "ITCAMDCVERSION"):
                        printError(tmpname, line)
                else:
                    printError(pname, line)
            elif ( re.compile("control\.systemproperty[0-9]*").match(pname) ):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpvalue = tmpvalue.strip()
                    if (tmpname == "ITCAM_DC_ENABLED" and tmpvalue != ""): ecam_flag = 1
                    control_properties.append([["name", tmpname], ["value", tmpvalue]])
                else:
                    printError(pname, line)
            elif ( re.compile("control\.environment[0-9]*").match(pname)):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpvalue = tmpvalue.strip()
                    control_environments.append([["name", tmpname], ["value", tmpvalue]])
                else:
                    printError(pname, line)
            elif ( re.compile("control\.classpath").match(pname) ):
                control_classpath = tokens[1]
            elif ( re.compile("server\.customservice[0-9]*.classname").match(pname) ):
                tmpvalue = tokens[1]
                customservice.append(["classname", tmpvalue])
            elif ( re.compile("server\.customservice[0-9]*.displayName").match(pname) ):
                tmpvalue = tokens[1].strip()
                customservice.append(["displayName", tmpvalue])
                customservice.append(["externalConfigURL", tmpvalue])
            elif ( re.compile("server\.customservice[0-9]*.description").match(pname) ):
                tmpvalue = tokens[1].strip()
                customservice.append(["description", tmpvalue])
            elif ( re.compile("server\.customservice[0-9]*.enable").match(pname) ):
                tmpvalue = tokens[1].strip()
                customservice.append(["enable", tmpvalue])
            elif ( re.compile("server\.customservice[0-9]*.classpath").match(pname) ):
                tmpvalue = tokens[1].strip()
                if (tmpvalue == ""): tmpvalue = "null"
                customservice.append(["classpath", tmpvalue])
            elif ( pname == "library.WPSlib" ):
                tmpvalue = tokens[1].strip()
                libs = tmpvalue.split(";" )
                sharedLib = libs[0]
            elif ( pname =="server.platform"):
                platform = tokens[1].strip()
            elif ( pname == "server.smf" ):
                smf = tokens[1]
            elif ( pname == "server.classpath" ):
                server_classpath = tokens[1].strip()
            elif ( pname == "server.bootclasspath" ):
                server_bootclasspath = tokens[1]
            elif ( pname == "server.runtime" ):
                server_runtime = tokens[1].strip()
            elif ( pname == "server.version" ):
                serverVersion = tokens[1].strip()
                logDebug("version=%s" % serverVersion)
            elif ( pname == "server.id" ):
                serverid = tokens[1].strip()
            elif ( pname == "server.xMBeanProvider"  ):
                pass
            elif ( re.compile("server\.itcam(61|dc)home").match(pname) ):
                variables.append([["symbolicName", "ITCAMDCHOME"], ["value", tokens[1] ]])
                itcamdc_home = tokens[1]
            elif ( pname == "server.am.home" ):
                #obsolete parameter
                pass
            elif ( pname == "server.ms.am.home" ):
                #obsolete parameter
                pass
            elif ( pname == "server.protocol_iiop_propagate_unknown_service_ctxs" ):
                variables.append([["symbolicName", "protocol_iiop_propagate_unknown_service_ctxs"], ["value", tokens[1] ]])
            elif ( pname == "server.policyedit" ):
                if (tokens[1] == "Yes"):
                    if (SCRIPT == "configDataCollector"):
                        serverPolicy_flag = 1
                elif (tokens[1] == "No"):
                    serverPolicy_flag = 0
                else:
                    print " \n*Error*  Unexpected value for server.policyedit keyword:"
                    print "         "+line+"\n "
                    errorCode = None
                    raise RuntimeError, "Unexpected value for server.policyedit keyword\n"+line
            elif ( pname == "server.enablePMI" ):
                if (tokens[1] == "Yes"):
                    if (SCRIPT == "configDataCollector"):
                        pmiService_flag = 1
                elif (tokens[1] == "No"):
                        pmiService_flag = 0
                else:
                    print " \n*Error*  Unexpected value for server.enablePMI keyword: %s '%s'" % (line, tokens)
                    errorCode = None
                    raise RuntimeError, "Unexpected value for server.enablePMI keyword\n"+line
            elif ( pname == "server.enableconfigmbean" ):
                pass
            elif ( pname == "server.containertimeouts"  ):
                containertimeouts = tokens[1]
            elif ( pname == "server.pmiservice.initialSpecLevel" ):
                pass
            elif ( pname == "server.pmiservice.statisticSet" ):
                server_pmiservice_statisticSet = tokens[1]
            elif(pname == "server.maximumHeapSize"):
                maximumHeapSize = tokens[1]
                logDebug("server.maximumHeapSize=%s" % maximumHeapSize)

            else:
                print " \n*Error*  Can't find match for \""+pname+"\"\n "
                print "         Input file has an invalid entry."
                print "         Script error must be corrected\n "
                errorCode = None
                raise RuntimeError, "input.properties invalid parameter: \""+pname+"\""

    print " "
    if (product == ""):
        product = "ITCAM"
        pmiService_flag = 1


    ##################### Do some basic validation #####################

    print "product=%s" % product
    if (product != "ITCAM" and product != "eCAM"):
        print " \n*Error*  Invalid product specified, must be either ITCAM or eCAM\n "
        errorCode = None
        raise RuntimeError, "Invalid product specified"


    print "serverid=%s" % serverid
    if (serverid == ""):
        logDebug("No server id")
        errorCode = None
        raise RuntimeError, "server ID is missing from input.properties file"


    # In WAS 5.1 the server name is delimited on the right with ":", later it is "|".

    #server1(cells/x336r1s36-vm01Node01Cell/nodes/x336r1s36-vm01Node02/servers/server1|server.xml#Server_1183122130078)
    re_format = re.compile("^.*/nodes/.*/servers/.*(\||:).*$")
    if (re_format.match(serverid)):
        rex=re.compile("(/nodes/|/servers/|\||:)+")
        (tmp1, tmp2, tmpnode, tmp3, tmpserver, tmp4, tmp5) = rex.split(serverid)
        nodeName = tmpnode
        serverName = tmpserver
        logDebug("serverName=%s nodeName=%s" % (serverName, nodeName))
    else:
        logDebug("Unable to extract node and server from server ID %s" % serverid)
        errorCode = None
        raise RuntimeError, "Unable to extract node and server from server ID"


    print "server="+serverName
    if (serverName == ""):
        errorCode = None
        raise RuntimeError, "unable to extract server name from server ID"

    print "node="+nodeName
    if (nodeName == ""):
        errorCode = None
        raise RuntimeError, "unable to extract node name from server ID"

    # When a stand-alone node is brought under a deployment manager, the
    # server ID will change.  For unconfig we must allow for that possibility.

    currentServerid = AdminConfig.getid("/Node:%s/Server:%s" % (nodeName,serverName) )
    logDebug("currentServerId=%s serverid=%s" % (currentServerid, serverid))
    if (currentServerid != serverid):
        print " \n*Warning*  Server ID from response file does not match server ID for "+nodeName+"."+serverName
        print "           File value = "+serverid
        print "           Actual     = "+currentServerid+"\n "
        # If we are configuring, a changed server ID is not acceptable
        if (SCRIPT != "unconfigDataCollector"):
            errorCode = None
            raise RuntimeError, "input.properties server ID does not match server ID for\n"+nodeName+"."+serverName


        pos = serverid.find("#Server_")
        if (pos > -1):
            pos += 8
            idnum = serverid[pos:-1]
        else:
            idnum = "Bad ID number"

        logDebug("idnum=%s" % (idnum))

        pos = currentServerid.find("#Server_")
        if (pos > -1):
            pos += 8
            curidnum = currentServerid[pos:-1]
        else:
            curidnum = "Bad current ID number"

        logDebug("curidnum=%s" % (curidnum))

        if (idnum != curidnum):
            logDebug("Server ID number %s from reponse file does not match server ID number %s" % (idnum, curidnum))
            errorCode = None
            raise RuntimeError, "Server ID number "+idnum+" from reponse file does not match server ID number "+curidnum


        print "Processing continuing with actual server ID."
        serverid = currentServerid


    # If the server name doesn't match there is a serious problem.  Let us not make it worse.
    currentServerName = AdminConfig.showAttribute(serverid, "name" )
    if (currentServerName != serverName):
        print " \n*Error*  Server name from response file does not match server ID"
        print "         returned for server ID object."
        print "         File value = " + serverName
        print "         Actual     = " + currentServerName
        print "         Server ID  = " + serverid
        errorCode = None
        raise RuntimeError, "input.properties server name does not match server name for\n"+nodeName+"."+serverName+" = "+currentServerid


    print "product home="+itcamdc_home
    if (itcamdc_home == ""):
        errorCode = None
        raise RuntimeError, "product location is missing from input.properties file"


    print "product platform="+platform
    if (platform == ""):
        print " \n*Error*  Platform is missing from input.properties file\n "
        errorCode = None
        raise RuntimeError, "Product platform is missing from input.properties file"


    ####################################################################
    #                                                                  #
    #  For z/OS we must use the server.version supplied in the input.  #
    #                                                                  #
    #  This is because the server object name does not exist until the #
    #  server is actually started.                                     #
    #                                                                  #
    ####################################################################

#    if (platform != "z/OS"):
#        print " \n*Error*  Platform is \""+platform+"\", must be z/OS\n "
#        global errorInfo
#        errorInfo = str(_tbck_.dumpStack())
#        global errorCode
#        errorCode = None
#        raise RuntimeError, "Platform is "+platform+", must be z/OS"
#

    if (serverVersion == ""):
        #errorCode = None
        raise RuntimeError, "server version is missing from input.properties file"


    print "Server major version is "+serverVersion+"."

    if is_zOS(platform) and smf != "Yes" and smf != "No" and smf != "Leave":
        if (SCRIPT == "configDataCollector"):
            print "*Error*  Invalid value for server.smf="+smf+", must be one of Yes/No/Leave"
            failed_flag = 1
        else:
            smf = "Leave"



    # Ignore obsolete parameters
    if (server_bootclasspath != ""):
        print " "
        print "*Warning*  Obsolete parameter server.bootclasspath ignored"
        print " "
        server_bootclasspath = ""


    if (containertimeouts != ""):
        print " "
        print "*Warning*  Obsolete parameter server.containertimeouts ignored"
        print " "
        containertimeouts = ""


    ####################################################################
    # The following checks are only necessary for configDataCollector  #
    ####################################################################

    if (SCRIPT == "configDataCollector"):
        if (serverid == ""):
            print "*Error*  Can't get serverid"
            failed_flag = 1

        if (genericargs == ""):
            print "*Error*  Can't get jvm generic arguments"
            failed_flag = 1

        if (systemproperties == ""):
            print "*Error*  Can't get system properties"
            failed_flag = 1

        if (environments == ""):
            print "*Error*  Can't get environments"
            failed_flag = 1



    if (failed_flag):
        print " "
        print "           Input file has an invalid entry."
        print "           Script error must be corrected"
        print " "
        errorCode = None
        raise RuntimeError, "Processing error, unable to complete configuration"

    if hasNodeagent() != 0:
        msg = resourcebundle.getmsg("CFG1147E", "Unable to locate NodeSync for node {0}. Make sure nodeagent for the node is started.", [nodeName])
        progress_obj.write_progress(msg)
        raise RuntimeError, "Error: nodeagent for node %s not started" % nodeName



    #########################################################################
    # Validation step is complete                                           #
    #########################################################################

    getProcessDefs( platform )
    logDebug("Return from getProcessDefs")

    # Establish runtime path for file access

    runtime_path = server_runtime
    runtime_path.replace("${ITCAM61HOME}", "${ITCAMDCHOME}")
    runtime_path.replace("${ITCAMDCHOME}", itcamdc_home)
    logDebug("Resolved runtime path: %s" % runtime_path)

    ########################
    # End of matching code #
    ########################

    ################# config data collector #################

    print " "
    print "Start setting attributes for %s" % serverName

    # JVM

    initServerVariableMapCache( )

    if (genericargs != ""):
        print "Configuring JVM Args"
        configJVM(genericargs )

    #System Properties
    if (systemproperties != ""):
        print "Configuring System Properties"
        configSystemProperties(servantJvmId, systemproperties)

    # Environment variables
    if ( environments != "" ):
        print "Configuring Environment Variables"
        configEnvs(javaProcServant, environments)

    # Server variables
    if (len(variables) > 0 ):
        for varent in variables:
            print "Configuring server variable %s" % varent[0][1]
            logDebug("variable entry = %s" % varent)
            configServerVariable(varent)

    # Server region classpath
    if (server_classpath != "" and server_classpath != "None"):
        print "Configuring server region classpath"
        print "Adding server classpath entry: "+server_classpath
        oldClassPath = AdminConfig.showAttribute(servantJvmId, "classpath" )
        if oldClassPath != None:
            oldClassPath = oldClassPath.strip()
        logDebug("oldClassPath: " + oldClassPath)
        mergedClassPath = mergePaths(oldClassPath, server_classpath )
        #Remove special characters if there are any
        if mergedClassPath != None:
            mergedClassPath = mergedClassPath.strip()
        parm = [["classpath", mergedClassPath]]
        logDebug("classpath parm = %s" % parm)
        try:
            AdminConfig.unsetAttributes(servantJvmId, "classpath" )
        except:
            _type_, _value_, _tbck_ = sys.exc_info()
            ermsg = str(_value_)
            print "********************* "
            print "*Warning* unsetAttributes failed or is not supported in this WebSphere release."
            print "*Warning* Please check your server region classpath."
            print "*Warning* errorInfo: " + ermsg
            print "********************* "
        AdminConfig.modify(servantJvmId, parm )
    else:
        logDebug("no server.classpath specified, skipping the configuration")

    # Control region classpath

    if (control_classpath != "" and control_classpath != "None"):
        print "Configuring Control Region classpath"
        print "Adding control classpath entry: "+control_classpath
        oldClassPath = AdminConfig.showAttribute(controlJvmId, "classpath" )
        mergedClassPath = mergePaths(oldClassPath, control_classpath )
        parm = [["classpath", mergedClassPath]]
        logDebug("classpath parm = %s" % parm)

        try:
            AdminConfig.unsetAttributes(controlJvmId, "classpath" )
        except:
            _type_, _value_, _tbck_ = sys.exc_info()
            ermsg = str(_value_)
            print "********************* "
            print "*Warning* unsetAttributes failed or is not supported in this WebSphere release."
            print "*Warning* Please check your control region classpath."
            print "*Warning* errorInfo: " + ermsg
            print "********************* "
        AdminConfig.modify(controlJvmId, parm )


    # Control region system properties

    if (is_zOS(platform) and len(control_properties) >  0):
        print "Configuring Control Region system properties"
        configSystemProperties(controlJvmId, control_properties )


    # Control region environment variables
    # (Currently there aren't any.)

    if (len(control_environments) > 0):
        print "Configuring Control Environment Variables"
        configEnvs(javaProcControl, control_environments )


    # Custom Service

    if (len(customservice) > 0):
        print "Configuring CustomService"
        configCustomService(customservice )


    # PMI service

    if (pmiService_flag):
        configPMIService( )

    #  PMI service statistics set

    if (server_pmiservice_statisticSet != ""):
        print "Configuring PMI Service statistics_set"
        setPMIStatisticSet(serverid, server_pmiservice_statisticSet )


    # Shared Library

    if (sharedLib != ""):
        print "Configuring WPSlib Shared Library classPath"
        configSharedLib(sharedLib )


    # Class Path
    # (Class path is no longer configured)

    # Boot Class Path
    # (Boot class path is no longer configured)

    # Injector
    # (The injector is configured as part of configJVM.)

    # WAS SMF data recording

    if (is_zOS(platform)):
        if (smf == "Yes"):
                print "Enabling SMF data collection"
                configSmf("true" )
        elif (smf == "No"):
                print "Disabling SMF data collection"
                configSmf("false" )
        elif (smf == "Leave"):
                print "Making no changes to WebSphere SMF data collection"



    if (not is_zOS(platform)):
        if (maximumHeapSize != "" and str(maximumHeapSize) != "0"):
            print "INFO: Configuring maximumHeapSize attribute of jvmEntries"
            print "INFO: Set maximumHeapSize= %s" % str(maximumHeapSize)
            configHeapSize(maximumHeapSize )


    # Special case for deployment.rmi.connection and appserver.rmi.host,
    # which must be exclusive.

    if (extraRMI_flag):
        unconfigExtraRMI(servantJvmId, systemproperties )


    # Handle the special case of obsolete settings

    unconfigObsolete(platform )

    # The following may be uncommented for non-destructive testing
    # puts " \n*Warning*  Resetting for testing\n "; $AdminConfig reset; exit 0

    # Save changes and exit

    returncode = saveChanges( )
    if (returncode != "true"):
        raise RuntimeError, "saveChanges routine failed with RC( %s ) for server %s" % (str(returncode), serverName)


    print common_constants.SUCCESSFULLY_CONFIGURED_DC_MSG + serverName
    #print "Successfully configured data collector for server "+serverName




####################################################################################
#
#  Main
#
#
#
####################################################################################

#

print "configDataCollector running."


progress_obj = None
failed = 0

filename = sys.argv[0]
#
# Initialize logging
#
log.set_logging()
#
# Initialize the resource bundle
#
resourcebundle.set_resourcebundle()
log.logDebug("Resource bundle initialized")

#
# Instantiate the global constants class
#
common_constants = constants()

phase = "config"  

try:

    if (filename != ""):
        print "Input files "+filename
        #
        # Open the progress file
        #
        inputlist_file_loc = filename
        progress_obj = Progress(common_constants.CONFIG, inputlist_file_loc)
        inDesc = progress_obj.open_progressfile()
        lines = []
        while 1:
            buffer = inDesc.readline()
            if not buffer: break
            lines.append(buffer)
        inDesc.close()

        loaded_modules = dir()
        if "AdminConfig" in loaded_modules and not "AdminTask" in loaded_modules:
        #if not "AdminTask" in dir():
            msg = resourcebundle.getmsg("CFG5046E", "AdminTask object is required to complete the configuration, but is unavailable. Unable to continue.")
            progress_obj.write_progress(msg)
            raise Exception(msg)



        for line in lines:
            try:
                line = line.strip()
                if (line != ""):
                    print "\n\n\n *************************** Processing **************************"
                    print "\nConfigure using: "+line
                    print "\nCurrent input properties to be processed"
                    configure(line, progress_obj)
                    msgl =[]
                    msgl.append(wasserver)
                    msg = resourcebundle.getmsg("CONFIG_SUCCESSFUL", "Configuration done successfully for {0}", msgl)
                    progress_obj.write_progress(msg)
                    #
                    # Remove input.properties.bak if it exists for the server.
                    #
                    try:
                        current_inputpropertiesbak = current_inputproperties + ".bak"
                        if (os.path.isfile(current_inputpropertiesbak)):
                            os.remove(current_inputpropertiesbak)
                            log.logDebug("input.properties.bak deleted: %s" % (current_inputpropertiesbak))
                    except Exception, remerrmsg:
                        log.logError("Cannot remove: %s - %s" % (current_inputpropertiesbak, remerrmsg))
                    #
                    # Set the summary to indicate that the server status.
                    #
                    summary[wasserver] = common_constants.SERVER_OK
                    #
                    # Check if probe for the server has been changed, call mbean to notify the
                    # server to activate or deactivate the probe.
                    #
                    log.logDebug("Probe value is: %s,  server version: %s, isStandalone=%s" % (probe, serverVersion, isStandalone) )
                    if (probe != "1" or isStandalone == 1): # no chanage in probe or we are running on a standalone cell, don't call mbean
                        msgl =[]
                        msgl.append(wasserver)
                        msg = resourcebundle.getmsg("RESTARTSERVER", "Application Server ({0}) should be restarted", msgl)
                        progress_obj.write_progress(msg)
                    else:
                        log.logDebug("Processing probe for server %s" % (wasserver))
                        msgl =[]
                        msgl.append(wasserver)
                        msg = resourcebundle.getmsg("PROBE_CHANGED", "Probe has been changed for server {0}, server will be notified", msgl)
                        progress_obj.write_progress(msg)
                        server_info = wascell + "^" + wasnode + "^" + wasserver
                        mcall = mbeancall.process_mbean(server_info, progress_obj)
                        #
                        # If mbean call not successfull, set the summary to reflect that.
                        #
                        if (mcall == 1):
                            summary[wasserver] = common_constants.SERVER_OK_NOMBEAN
                        else:
                            summary[wasserver] = common_constants.SERVER_OK_MBEAN
                        

                    progress_obj.write_progress_finish(wasserver, common_constants.SUCCESSFUL)

            except:
                failed = 1
                log.logException("Error occurred while configuring server %s " % (wasserver))
                traceback_report = log.format_traceback()
                exception_report = log.format_traceback("true")
                #
                # Set the summary to indicate that the server has failed.
                #
                summary[wasserver] = common_constants.SERVER_NOT_OK
                #
                # Check the last processed input properties and delete it as the config
                #
                restore_input_properties()
                
                #
                # If progress object is present, put a message in progress file to be displayed for the wsadmin caller.
                #
                if (progress_obj != None):
                    msg = resourcebundle.getmsg("CONFIG_ERROR", "Error occurred while processing configuration - {0}")
                    if len(exception_report) > 0:
                        msg = msg + "\n" + exception_report
                    progress_obj.write_progress(msg)
                    if wasserver is None or wasserver == "None":
                        wasserver = ""
                    progress_obj.write_progress_finish(wasserver, common_constants.UNSUCCESSFUL)
        #
        # Set the summary so it will be shown as progress to the console.
        #
        msg = resourcebundle.getmsg("SUMMARY", "Summary:")
        progress_obj.write_progress_noindent(msg)
        for server_name in summary.keys():
            msgl =[]
            msgl.append(server_name)
            server_stat = summary[server_name]
            log.logDebug("Summary key: %s  status: %s" % (server_name, server_stat))
            if (server_stat == common_constants.SERVER_OK):
                msg = resourcebundle.getmsg("SERVER_OK", "{0} (OK)", msgl)
                progress_obj.write_progress(msg)
            elif (server_stat == common_constants.SERVER_OK_MBEAN):
                msg = resourcebundle.getmsg("SERVER_OK", "{0} (OK)", msgl)
                progress_obj.write_progress(msg)
            elif (server_stat == common_constants.SERVER_OK_NOMBEAN):
                msg = resourcebundle.getmsg("SERVER_OK", "{0} (OK)", msgl)
                progress_obj.write_progress(msg)
            else:
                progress_obj.write_progress("%s (X)" % (server_name))
    else:
        errorCode = None
        raise RuntimeError, "Input file not supplied. Configuration abandoned"

    if doSynchronize == 1:
        phase = "sync"
        syncstat = __doSynchronize()
        if (syncstat == 1):
            failed = 1
    else:
        log.logDebug("No Synchronization necessary")

    #
    # Close the progress file
    #
    if (progress_obj != None):
        progress_obj.close_file()
        progress_obj = None

    #
    # If processing each server caused an error, then a message is issued, and the exception
    # is raised, so the wsadmin gets a non zero return code.
    #
    if (failed == 1):
        log.logDebug("Processing servers had a failure")
        raise Exception("Processing servers had a failure")
    
    restore_input_properties()
    
# This needs to pick up the sys.exit(1) and exceptions     
except:
    log.logException(SCRIPTNAME+" ends abnormally\n ")
    traceback_report = log.format_traceback()
    exception_report = log.format_traceback("true")
    log.logDebug(traceback_report)
    #
    # Check the last processed input properties and delete it as the config failed
    #
    
    log.logDebug("configuration complete: %s input.properties=%s" % (str(configComplete), current_inputproperties))
    restore_input_properties()
    #
    # If progress object is present, put a message in progress file to be displayed for the wsadmin caller.
    #
    if (progress_obj is not None):
        if phase == "sync":
           msg = resourcebundle.getmsg("SYNC_ERROR", "Error occurred while testing for or executing synchronize nodes") 
        else:       
            msg = resourcebundle.getmsg("CONFIG_ERROR", "Error occurred while processing configuration")
        if len(exception_report) > 0:
            msg = msg + "\n" + exception_report
        progress_obj.write_progress(msg)
        if wasserver is None or wasserver == "None":
            wasserver = ""
        progress_obj.write_progress_finish(wasserver, common_constants.UNSUCCESSFUL)
        progress_obj.close_file()

    etype, value, tbck = sys.exc_info()
    raise Exception(value)


# When no errors, exit gracefully to reset 
# any error status previously stored by wsadmin
sys.exit(0)
 
    
