#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
    Configuration utility functions
'''
#####################################################################

fixlev = "41683" # Last APAR, defect or story to update this module

import sys
import os
import codecs
import types

import java.io.File

from types import *
from time import strftime
from java.lang import System
from java import lang
from java import util
from java import io

import shared_globals as globals
import utils
import platformUtil
import debugUtil as log

from resourcebundle import getmsg
from debugUtil import logDebug as debug
from migratecustomfiles import get_runtimes_created as get_runtimes_created

#####################################################################
# Set some constants                                                #
#####################################################################

YES="Yes"
NO="No"
listYesValues=[ "y", "Y", "yes", "Yes", "YES" ]
listNoValues=[ "n", "N", "no", "No", "NO" ]


###
# isLink(path) - Is path a symlink?
#
# The path can be a directory or file
# In z/OS, os.path.islink(path) is not working properly
#
# Returns:  1  if true,
#           0  if false,
#           16 if test failed
# TODO:  Capture and display a message that's often generated by the OS when there is a problem
###

def isLink(path):
    ''' isLink(path)
        Issues os.system('test -L ' + path
        Returns 1 if true, 0 if false, 16 if the test or command failed
    '''
    method_descr="isLink: " + path
    # TODO: do posix test
    try:
        rc=os.system("test -L " + path )
        if rc == 0:
            return True
        elif rc == 1:
            return False
        elif rc == 2:
            print "\n*ConfigUtils.isLink failed* - expression badly formed"
            return 16
        else:
            return 16
    except Exception, msgVar:
        log.logException(method_descr)
        return 16


###
# Fix Selected file names
###
def fixFilenames(pdl):
    '''
        Fix selected file names
    '''
    debug("Fixing selected file names")
    # Don't do PROFILEHOME, it is used as an index
    for pde in "CONFIGHOME", "JAVAHOME", "LOGROOT", "RUNTIME", "WASCONFIGROOT", "WASPLUGINSROOT", "WASPROPSDIR", "WASROOT", "WSADMIN":
        if pde not in pdl:
            debug("Entry %s not found in PDL" % pde)
            continue
        if pdl[pde][0] is None: continue
        if pdl[pde][0] == "": continue
        assert type(pdl[pde][0]) is not dict, "Entry %s is a dictionary" % pde
        assert type(pdl[pde][0]) is not tuple, "Entry %s is a tuple" % pde
        if type(pdl[pde][0]) is list:
            oldtag = pdl[pde][1]
            newtag = oldtag+":fix"
            for pdei in range(len(pdl[pde][0])):
                oldnm = pdl[pde][0][pdei]
                newnm = os.path.abspath(oldnm)
                if oldnm == newnm: continue
                debug("Changing %s[%i] from %s to %s" % (pde, pdei, oldnm, newnm))
                pdl[pde][0][pdei] = newnm
                pdl[pde][1] = newtag
        else:
            oldnm = pdl[pde][0]
            newnm = os.path.abspath(oldnm)
            if oldnm == newnm: continue
            debug("Changing %s from %s to %s" % (pde, oldnm, newnm))
            pdl[pde][0:2] = newnm, pdl[pde][1]+":fix"

    return


###
#  getDirectoryListing - a detailed listing
#     - type =  "summary" | "detail"
#  Returns a list
#  If z/OS, use command "ls -lsapFd <path>"
#  TODO: ** This needs some work for zos unless only a one line output is needed **
###

def getDirectoryListing(path, type="summary"):
    ''' getDirectoryListing(path, type='summary')  where type = detail | summary
        If detail and not z/os uses os.listdir(path)
                      zos, command issues is subject to change
        If summary, os.listdir(path) used
    '''
    listDir=[]
    if type == "detail":
        # print "getDirectoryListing for: " + path
        if platformUtil.IS_zOS_OS():
            listDir=CommandAndResponse("ls -lsapFd " + path)
        else:
            listDir=os.listdir(path)
    else:
        listDir=os.listdir(path)
    return listDir


###
# Issue OS command and get the return code
#
# Returns  - the "return code" from the command
#            or 99 if exception caught
###

def issueOSCommand(command):
    '''  issueOSCommand(command)
         Issue OS command and return the return code
    '''
    method_descr="issueOSCommand: " + command
    try:
        debug("    " + method_descr )
        rc = os.system(command)
       #debug("     ...os.system(%s) rc=%s" % (command, str(rc)) )
        debug("     ...rc=%s" % str(rc) )
        return rc
    except Exception, msgVar:
        log.logException(method_descr)
        return 8


###
# Issue OS command and obtain the response
# Returns a list of the output of the command
# To use example:
#     result_list = ConfigUtils.CommandAndResponse('id')
#     one_line_response  = result_list[0]
###

def CommandAndResponse(command):
    ''' CommandAndResponse(command)
        Issue command, retrieve and return the output from the command.
        In zOS, we somehow must get the EBCDIC response translated
        so we can 'see it" in our jvm
    '''
    method_descr="CommandAndResponse: " + command
    #TODO Make file name more unique
    #     Cleanup files
    cmd_output_filepath="/tmp/createcfg1.txt"
    iconv_cmd_output_filepath="/tmp/createcfg2.txt"

    output_list=[]

    try:
        rc = os.system(command + ">" + cmd_output_filepath)
        if rc == 0:
            if platformUtil.IS_zOS_OS():
                # covnert the output of the command so we can read it in jython
                command="iconv -f ibm-1047 -t iso8859-1 " + cmd_output_filepath + " > " + iconv_cmd_output_filepath
                os.system(command)
                original_file_name=cmd_output_filepath
                cmd_output_filepath = iconv_cmd_output_filepath
                os.remove(original_file_name)
            output_list = ReadFile(cmd_output_filepath)
            os.remove(cmd_output_filepath)
            return output_list
        else:
            # Log ..
            return []
    except Exception, msgVar:
        log.logException(method_descr)
        return 8


###
# CommandResponseRC
#
# Issue OS command and return the OS-generated return code and response.
# Returns a tuple ( rc, outputList) containing the return code and output of the command
# In zOS, need to examine EBCDIC output from command.
# Use example:
#     rc, resultList = ConfigUtils.CommandResponseRC('id')
#     one_line_response  = result_List[0]
# TODO: improve by testing for existence of the response list
###

def CommandResponseRC(command):
    ''' CommandResponseRC(command)
        Issue OS command and return the OS-generated return code and response.
    '''
    #TODO Make file name more unique
    method_descr="CommandResponseRC: " + command
    debug("    " + method_descr )
    cmd_output_filepath="/tmp/createcfg01.txt"
    iconv_cmd_output_filepath="/tmp/createcfg02.txt"
    response=""
    output_list=[]
    try:
        rc = os.system(command + ">" + cmd_output_filepath)
        if rc == 0:
            if platformUtil.IS_zOS_OS():
                command="iconv -f ibm-1047 -t iso8859-1 " + cmd_output_filepath + " > " + iconv_cmd_output_filepath
                rc1 = os.system(command)
                original_file_name=cmd_output_filepath
                cmd_output_filepath = iconv_cmd_output_filepath
                os.remove(original_file_name)
                output_list = ReadFile(cmd_output_filepath)
            os.remove(cmd_output_filepath)
        else:
            debug("      rc="  + str(rc))
        return rc, output_list
    except Exception, messageVar:
        print "CommandAndResponse Exception on command: ", command , messageVar
        rc = 8
        return rc, output_list


###
#  Read File
###

def ReadFile(cmd_output_filepath):
    ''' ReadFile(cmd_output_filepath)
    '''
    method_descr="ReadFile: " + cmd_output_filepath
    try:
        inFile  = open(cmd_output_filepath,"r")
        inLines = inFile.readlines()
        inFile.close()
        return inLines
    except Exception, msgVar:
        log.logException("Exception in " + method_descr)
        inFile.close()
        return []


###
# StringReplace
# Scan the string "str" for any matching occurance of the variables in the substitutions list.
# Replace if a match is found
#  List substitutionsList example
#       [ @{var-1}:value-1
#         @{var-2}:value-2 ]
# Returns the updated string
###

def stringReplace(str, substitutionsList, sep=":"):
    ''' stringReplace(str, substitutionsList, sep=':')
        Scan the string "str" for any matching occurance of the variables in the substitutions list.
        Replace if a match is found
    '''
    method_descr="stringReplace for: " + str
    # print " ..stringReplace: input - " + str
    try:
        for oldNewEntry in substitutionsList:
            oldNewEntryList=oldNewEntry.split(sep,1)
            # print " ..stringReplace: " +  oldNewEntryList[0]+  " : "  + oldNewEntryList[1]
            str=str.replace(oldNewEntryList[0],oldNewEntryList[1])
        #print " ..stringReplace updated string: " + str
        return str
    except Exception, msgVar:
        log.logException(method_descr)
        raise


###
# ScanReplaceFile scan/replace
#
# If z/OS, call a differend method because the encoding is EBCDIC on property files
#
#  Returns - 0 if successful
#            8 if failure - unable to complete the scan
#
# Example how to use:
#   substitutionsList = []
#   substitutionsList.append( '@{LOG_COMMON_DIR}:/var/IBM/commonLog' )
#   substitutionsList.append( '@{BLANK_TEST}:' )
#   ConfigUtils.scanReplaceFile(inFilePath, outFilePath,substitutionsList )
#
###

def scanReplaceFile(inFilePath, outFilePath, substitutionsList, sep=":"):
    ''' scanReplaceFile(inFilePath, outFilePath, substitutionsList, sep=':')
    '''
    method_descr="scanReplaceFile for: " + outFilePath
    file1 = None
    file2 = None
    try:
        if platformUtil.IS_zOS_OS():
            rc =  scanReplaceFileEBCDIC(inFilePath, outFilePath, substitutionsList, sep)
            return rc
        file1  = open(inFilePath,"r")
        file2  = open(outFilePath,"w")
        inLines = file1.readlines()
        for line in inLines:
            line = stringReplace(line,substitutionsList,sep)
            file2.write(line)
        file1.close()
        file2.close()
        return 0
    except Exception, msgVar:
        log.logException(method_descr)
        if file1 is not None: file1.close()
        if file2 is not None: file2.close()
        return 8


def scanReplaceFileASCII(inFilePath, outFilePath, substitutionsList, sep=":"):
    '''
        scanReplaceFileASCII(inFilePath, outFilePath, substitutionsList, sep=':')
    '''
    method_descr="scanReplaceFileASCII for: " + outFilePath
    try:
        file1  = open(inFilePath,"rt")
        file2  = open(outFilePath,"w")
        inLines = file1.readlines()
        for line in inLines:
            line = stringReplace(line,substitutionsList,sep)
            file2.write(line)
        file1.close()
        file2.close()
        return 0
    except Exception, msgVar:
        log.logException(method_descr)
        file1.close()
        file2.close()
        return 8


###
# scanReplaceFileEBCDIC  scan/replace over an ebcdic text file
#
#  Returns - 0 if successful
#            8 if failure
#
# Example how to use:
#   substitutionsList = []
#   substitutionsList.append( '@{LOG_COMMON_DIR}:/var/IBM/commonLog' )
#   substitutionsList.append( '@{BLANK_TEST}:' )
#   ConfigUtils.scanReplaceFile(inFilePath, outFilePath,substitutionsList )
#
###

def scanReplaceFileEBCDIC(inFilePath, outFilePath, substitutionsList, sep=":"):
    '''
        scanReplaceFileEBCDIC
        @param   inFilePath  in EBCDIC
        @param   outFilePath in EBCDIC
        @param   substitutionsList
    '''
    method_descr="scanReplaceFileEBCDIC for: " + outFilePath
    debug("entry: " + method_descr)
    try:
        fin  = codecs.open(inFilePath,  mode="r", encoding="cp1047")
        fout = codecs.open(outFilePath, mode="w", encoding="cp1047")
        lines = fin.readlines()
        for line in lines:
            line=stringReplace(line,substitutionsList,sep)
            fout.write(line)
        fin.close()
        fout.close()
        return 0
    except Exception, msgVar:
        log.logException("Exception in " + method_descr)
        fin.close()
        fout.close()
        return 8


###
# create_file - create a file using a list of text lines
#
# Returns: 0 - if successful
#          8 - if fails
###

def create_file(recordList, filePath, encoding="ASCII"):
    '''
        create_file(recordList, filePath, encoding='ASCII')
        Returns: 0 - if successful, 8 - if not successful
    '''
    method_descr="create_file for %s %s" % (filePath, encoding)
    if encoding == "EBCDIC":
        rc = create_file_ebcdic(recordList, filePath)
        return rc
    try:
        linesep=System.getProperty("line.separator")
        file = open(filePath, "wt")
        for record in recordList:
            file.write(record)
            file.write(linesep)
        file.close()
        return 0
    except Exception, msgVar:
        log.logException(method_descr)
        file.close()
        return 8


###
# create_file_ebcdic - Create file in EBDICIC format
#
#  Until we get a better java-based solution, use iconv
#
#  Returns - 0 if successful
#            8 if failure
#
# Example how to use:
#     create_ebcdic_file(filePath, records_list)
#
###

def create_file_ebcdic(recordList, filePath):
    ''' create_file_ebcdic(recordlist, filePath)
    '''
    method_name="create_file_ebcdic"
    method_descr=method_name + " for:  %s"  % (filePath)
    try:
        linesep=System.getProperty("line.separator")
        file = open(filePath, "wt")
        for record in recordList:
            record = record + "\x85"
            record = record.encode("cp037")
            file.write(record)
            #file.write(linesep)
        file.close()
        return 0

    except Exception, msgVar:
        log.logException(method_descr)
        file.close()
        return 8


###
# append_to_file - append to a file using a list of text lines
#
# Returns: 0 - if successful
#          8 - if fails
###

def append_to_file(recordList, filePath, encoding="ASCII"):
    '''
        create_file(recordList, filePath, encoding='ASCII' )
                                          encoding='EBDCIC'
        Returns: 0 - if successful, 8 - if not successful
    '''
    method_descr="append_to_file for %s %s" % (filePath, encoding)
    if encoding == "EBCDIC":
        rc = append_to_file_ebcdic(recordList, filePath)
        return rc
    try:
        linesep=System.getProperty("line.separator")
        file = open(filePath, "at")
        for record in recordList:
            file.write(record)
            file.write(linesep)
        file.close()
        return 0
    except Exception, msgVar:
        log.logException(method_descr)
        file.close()
        return 8


###
#  append_to_file_ebcdic - append to file in EBCDIC format
#
#  Returns - 0 if successful
#            8 if failure
#  TODO: convert to codecs
###

def append_to_file_ebcdic(recordList, filePath):
    ''' create_file_ebcdic(recordlist, filePath)
    '''
    method_name="create_file_ebcdic"
    method_descr=method_name + " for:  %s"  % (filePath)
    try:
        linesep=System.getProperty("line.separator")
        file = open(filePath, "a")
        for record in recordList:
            record = record + "\x85"
            record = record.encode("cp037")
            file.write(record)
        file.close()
        return 0
    except Exception, msgVar:
        log.logException(method_descr)
        file.close()
        return 8


###
#  convert_to_ascii using iconv
#
#  Use iconv command for now, piping the result to another file
#  Returns:  0 if successful
#            8 if not
#
###

def convert_to_ascii(ebcdic_filepath, ascii_filepath):
    '''
        convert_to_ascii(ebcdic_filepath, ascii_filepath)
        Converts an ebdcic file to ascii using iconv
        Returns:  0 if successful   8 if not
    '''
    method="convert_to_ascii"
    method_descr=method + ": %s --> %s"  % ( os.path.basename(ebcdic_filepath) , os.path.basename(ascii_filepath) )
    debug("  " + method_descr )
    cmd_response=""
    results = []
    try:
        # 1. validate existence of ebcdic_file
        if not os.path.exists(ebcdic_filepath):
            debug("    input file does not exist")
            return 8
        # 2. issue iconv:  ebcdic_filepath --> ascii_filepath
        command="iconv -f ibm-1047 -t iso8859-1  " + ebcdic_filepath + " > " + ascii_filepath
        rc = os.system(command)
        if not rc == 0:
            debug("         failed, rc=" + str(rc))
            return 8
        # 3. return
        debug("         successful")
        return 0
    except Exception, msgVar:
        log.logException(method_descr)
        return 8


###
#  convert_file_to_ascii(ebcdic_filepath) using iconv
#
#  Use iconv command for now, piping the result to another file
#  Returns:  0 if successful
#            8 if not
#
###

def convert_file_to_ascii(ebcdic_filepath):
    '''
        convert_file_to_ascii(filepath)
        Converts an ebdcic file to ascii using iconv
        Returns:  0 if successful   8 if not
    '''
    method="convert_file_to_ascii"
    method_descr="configUtils." + method + ": %s"  % ( ebcdic_filepath )
    debug("  " + method_descr )
    cmd_response=""
    results = []
    try:
        # 1. validate existence of ebcdic_file
        if not os.path.exists(ebcdic_filepath):
            debug("    input file does not exist")
            return 8
        # 2. issue iconv:  ebcdic_filepath --> ascii_filepath
        ascii_filepath = ebcdic_filepath + ".to.ascii.txt"
        command="iconv -f ibm-1047 -t iso8859-1  " + ebcdic_filepath + " > " + ascii_filepath
        rc = os.system(command)
        if not rc == 0:
            debug("       failed,  rc=" + str(rc))
            return 8
        # 3. delete original ebcdic_filepath
        os.remove(ebcdic_filepath)
        # 4. rename ascii_filepath --> ebcdic_filepath
        os.rename(ascii_filepath, ebcdic_filepath)
        # 5 return
        debug("         successful")
        return 0
    except Exception, msgVar:
        log.logException(method_descr)
        return 8


###
#  convert_file_to_ebcdic(ascii_filepath) using iconv
#
#  Use iconv command. We use command iconv to pipe the result to another file
#  Returns:  0 if successful
#            8 if not
#
###

def convert_file_to_ebcdic(ascii_filepath):
    ''' convert_file_to_ebcdic(ascii_filepath)
        Converts an ascii file to encdic using iconv
        Returns:  0 if successful   8 if not
    '''
    method="convert_file_to_ebcdic"
    method_descr="ConfigUtils." + method + ": %s"  % ( ascii_filepath )
    debug("  " + method_descr )
    # print "..." + method_descr
    cmd_response=""
    results = []
    try:
        # 1. validate existence of ascii_file
        if not os.path.exists(ascii_filepath):
            debug("    input file does not exist")
            return 8
        # 2. issue iconv:  ascii_filepath --> ebcdic_filepath
        ebcdic_filepath = ascii_filepath + ".ebcdic.txt"
        command="iconv -f iso8859-1 -t ibm-1047 " + ascii_filepath + " > " + ebcdic_filepath
        rc = os.system(command)
        if not rc == 0:
            debug("       failed,  rc=" + str(rc))
            return 8
        # 3. delete original ascii_filepath
        os.remove(ascii_filepath)
        # 4. rename ebcdic_filepath --> ascii_filepath
        os.rename(ebcdic_filepath, ascii_filepath)
        # 5 return
        debug("         successful")
        return 0
    except Exception, msgVar:
        log.logException(method_descr)
        return 8


###
#   display_exception
###

def display_exception(method_descr, Exception, msgVar):
    ''' display_exception(method_descr, Exception, msgVar):
    '''
    _type_, _value_, _tbck_ = sys.exc_info()
    ermsg = str(_value_)
    ertype= str(_type_)
    print "\n*configUtils.%s failed " % ( method_descr )
    print   "*.......   %s, %s " % ( ertype , ermsg )


###
# scan_file using a list of match strings
#    Returns a list of the strings that actually matched a line in the file
###

def scan_file(path, match_strings, ebcdic=False):
    '''
        scan_file(string_list,ebcdic=False)
        @param    path
        @param    match_strings - list of strings to match
        returns a list of the lines where a match was found
    '''
    debug("scan_file entry  - %s " % path )
    f = None
    results_list = []
    rc = 0
    try:
        if ebcdic == True:
            f = codecs.open(path, mode="r", encoding="cp1047")
        else:
            f = open(path, "r")
        lines = f.readlines()
        for line in lines:
            for string in match_strings:
                rc = line.find(string)
                if ( line.find(string) ) != -1:
                    if not string in results_list:
                        results_list.append(string)

        f.close()
        debug("scan_file exit")
        return [0, results_list]
    except:
        log.logException("scan_file exception on: %s" % path )
        if f is not None: f.close()
        return [8, results_list]


#####################################################################
# changeowner                                                       #
# Change ownership and access using chown & chmod                   #
# Notes:                                                            #
#####################################################################

def changeowner(dirent, owner, access):
    '''
         changeowner(dirent, owner, access)
         Change ownership and access priviliges using chmod and chown
         UNIX/LINUX only
         @param dirent - directory path
         @param owner  - new owner for chown command
         @param dirent - access flags for chmod command
    '''
    method_descr="ConfigUtils.changeowner"
    debug("configUtils.changeowner  entry")
    debug("  Directory or file= " + dirent)
    debug("  New owner= " + str(owner) + " New access=" + str(access))

    idu=99                # default "userid number"
    is_authorized=0       # default "authorized" is false
    owner_set=NO
    access_set=NO
    if owner is None:  owner  = ""
    if access is None: access = ""

    # If either owner or access input is supplied, do the work:
    if owner != ""  or  access != "":
      #login_user = ""
      #try:
      #    login_user = os.getlogin()
      #except OSError:
      #    log.logException("Failed to get session user")

        # Get the actual uid number of the console user
        rc, result = CommandResponseRC("id -u")
        if rc == 0  and result:
            idu = result[0]
        debug("  Userid number: " + str(idu))

        if idu == 0:
            debug("  Already running as root")
            is_authorized=1
            ownercase=1
        else:
            cmd="echo true | su 2>/dev/null"
            rc, result = CommandResponseRC(cmd)
            if rc == 0:
                is_authorized=1
                debug("  rc=0 Can change to root")
                ownercase=2
            else:
                debug("  rc=%s Can not use the root priviledge" % str(rc) )
                is_authorized=0
                ownercase=0
                print getmsg("CFG5048W","Your userid is not allowed to use root privileges, ownership will not be changed.")
                debug( "*Warning*  Unable to use root priviledges, ownership will not be changed")
        debug("  is_authorized flag=%s, ownercase=%s" % ( str(is_authorized), str(ownercase))  )

        # Running as root or can change(su) to root ?
        if is_authorized:
            # If OWNER-setting option supplied
            if owner != "":
                debug("  Setting ownership of %s to %s " % (dirent, owner) )
                cmd="chown -hR " + owner + " " + dirent
                if ownercase == 1:
                    rc = issueOSCommand(cmd)
                    if    rc == 0: owner_set = YES
                    debug("  command result: " + cmd + ", rc=" + str(rc))
                else:
                    # issue command piped into su:
                    cmd=cmd + " | su "
                    rc = issueOSCommand(cmd)
                    if    rc == 0: owner_set = YES
                    debug("  command result: " + cmd + ", rc=" + str(rc))

            # If ACCESS setting option supplied:
            if access != "":
                debug("  Setting permissions of %s to %s " % (dirent, access) )
                # Running as root user ?
                if ownercase == 1:
                    cmd="chmod -hR " + access + " " + dirent
                    rc = issueOSCommand(cmd)
                    if    rc == 0: access_set = YES
                    debug("  command result: " + cmd + ", rc=" + str(rc))
                else:
                    cmd= "chmod -hR " + access + " " + dirent + " | su "
                    rc = issueOSCommand(cmd)
                    if    rc == 0: access_set = YES
                    debug("  command result: " + cmd + ", rc=" + str(rc))
    else:
        debug("  No owner or access to set")

    debug("configUtils.changeowner exit: owner_set=%s access_set=%s"  % (str(owner_set), str(access_set))  )


def create_substitution_dictionary(pdl):
    '''
        This routine will create a substitution dictionary for
        tailoring input.properties.
    '''

    debug("Entry create_substitution_dictionary() %s" % fixlev)

    # This will create a tuple.  Each entry in the tuple will correspond
    # to the server entry in the SERVER entry of the PDL.  The substitution
    # values will be based on the contents of the PDL.

    #
    # Look for the "SERVERS" key in the PDL.  If it does not exist, then
    # raise an exception and exit.
    # The value for the SERVERS key is a tuple, and the first entry in the
    # tuple is the server list, which is also a tuple.  Each entry in the server
    # tuple represents a server.  This server entry is a dictionary which contains all
    # information about the server.
    #
    assert "SERVERS" in pdl.keys(), getmsg("CFG5015E", "Data element {0} is not present in the PDL", ["SERVERS"])
    assert pdl["SERVERS"][0] != None, getmsg("CFG5017E", "Data element {0} of PDL has no entries", ["SERVERS"])
    assert type(pdl["SERVERS"][0]) == type(()) or type(pdl["SERVERS"][0]) == type([]), getmsg("CFG5019E", "Unexpected value type {0} for PDL key {1}", [type(pdl["SERVERS"][0]), "SERVERS"])


    # This table specifies data taken directly from the PDL.  If the value is not in the PDL
    # and must be constructed, the value of the entry in this table will be None.
    parms = {
        "SERVER":"SERVERS",
        "RUNLOC":"RUNLOC",
        "RUNTIME":"RUNTIME",
        "RUNTIMEP":"RUNTIMEP",
        "SERVER_ID":"SERVERID",
        "WAS_VERSION":"WASVERSION",
        "GCLOGPATH":"GCLOGPATHLIST",
        "WAS_CELL":"WASCELL",
        "WAS_NODE":"WASNODE",
        "TTAPI_DIR_LIB" :"TTAPIDIR",
        "TTAPI_DIR_JAR" :"TTAPIJAR",
        "TTAPILIBPATH"  :"TTAPILIBPATH",
        "TTAPICLASSPATH":"TTAPICLASSPATH",
        "ZOSLIBPATH":"ZOSLIBPATH",
        "WAS_HOME":"WASINSTALLROOT",
        "WPS_HOME":"WPSHOME",
        "WBI_ROOT":"WBIHOME",
        "SUBPRODUCT":"SUBPRODUCT",
        "CONFIGRETURNCODE":"CONFIGRETURNCODE",
        "WAS_PROPS_DIR":"WASPROPSDIR",
        "SMF":"SMFLIST",
        "CONFIG_ROOT":"WASCONFIGROOT",
        "JAVA_HOME":"JAVAHOME",
        "JAVA_MODE":"JVMMODE",
        "JAVA_VERSION": "JVMVERSION",
        "ITCAMDCHOME":"CONFIGHOME",
        "GC_PARMS":"GC_PARMS",
        "MAXHEAPSIZE":"MAXHEAPSIZE",
        "ITCAMDC_INPUT":"ITCAMDCINPUT",
        "PROFILEHOME":"PROFILEHOME",
        "PROFILE":"PROFILE",
        "CONNTYPE":"CONNTYPELIST",
        "HOST":"HOSTLIST",
        "PORT":"PORTLIST",
        "TEMACONNECT":"TEMACONNECTLIST",
        "TEMAHOST":"TEMAHOSTLIST",
        "TEMAPORT":"TEMAPORTLIST",
        "MSCONNECT":"MSCONNECTLIST",
        "MSHOMECONNECT":"MSHOMECONNECTLIST",
        "KERNELIP":"KERNELIPLIST",
        "KERNELPORT01":"KERNELPORT01LIST",
        "RFSPORT01":"RFSPORT01LIST",
        "MSHOME":"MSHOMELIST",
        "RMIPORTSET":"RMIPORTSETLIST",
        "RMIPORTSET":"RMIPORTSETLIST",
        "PROBECONTROLLERRMIPORT":"PROBECONTROLLERRMIPORTLIST",
        "PROBERMIPORT":"PROBERMIPORTLIST",
        "TTAPI":"TTAPILIST",
        "TTAPISSN":"TTAPISSNLIST",
        "TTAPIDIR":"TTAPIDIRLIST",
        "TTTCHOST":"TTTCHOSTLIST",
        "TTTCPORT":"TTTCPORTLIST",
        "SOADC":"SOADCLIST",
        "GCOUTPUT":"GCOUTPUTLIST",
        "GCOUTPUTDIST":"GCOUTPUTDISTLIST",
        "CURRENTGCLOGLOCATION":"CURRENTGCLOGLOCATION",
        "HEAPSIZE":"HEAPSIZE",
        "CURRENTMAXHEAPSIZE":"CURRENTMAXHEAPSIZE",
        "STANDALONE":"STANDALONE",
        "MIGRATEDDCHOME":"MIGRATEDDCHOME",
        "MIGRATEDVERSION":"MIGRATEDVERSION",
        "ITCAMDCVERSION": "VERSION",
        "TPV": "TPVLIST",
        "DE": "DELIST",
        "WAS_CELL_MODIFIED": "WASCELLMODIFIED",
        "PROFILE_MODIFIED": "PROFILEMODIFIED",
        "WAS_NODE_MODIFIED": "WASNODEMODIFIED",
        "SERVER_MODIFIED": "SERVERSMODIFIED",
        "SERVERALIAS":"ALIASLIST",
        "ITCAMFEATUREVERSION": "ITCAMFEATUREVERSION",

        "AM_HOME":None,
        "AGENTLIB":None,
        "LOG_ROOT":None,
        "LIBPATH":None,
        "PATHDELIMITER":None,
        "PLATFORM":None,
        "PLATFORMBITMODE":None,
    }

    servers_dict = ()
    confighome = pdl["CONFIGHOME"][0]
    # Replace "\" in windows file path by "/" so the file path will work well in Java environment
    if platformUtil.isWindows():
        confighome = utils.replaceBackSlash(confighome)

    for serveri in range(len(pdl["SERVERS"][0])):
        server_ent = {}
        bitMode = platformUtil.getBitMode(confighome, pdl["JVMMODE"][0][serveri])
        for parm in parms.keys():
            # Handle special cases first
            if parm == "AM_HOME":
                server_ent[parm] = confighome + "/itcamdc"
            elif parm == "AGENTLIB":
                #runtimep = pdl["RUNTIMEP"][0][serveri]
                # For a distributed DC, agentlib format is like: -agentlib:am_ibm_<jdk version>
                # No runtime directory or other file directory involved
                # For now, comment out file path conversion
                #if platformUtil.isWindows():
                    # Replace "\" in windows file path by "/" so the file path will work well in Java environment
                    #runtimep = utils.replaceBackSlash(runtimep)
                #For the native library name in "-agentlib:xx":
                #z/OS: am
                #other platforms: am_<jkd vendor>_<jdk version>
                #for now, hard code as following until we have code to detect jdk_vendor:
                #Windows, Linux, AIX, and OS400: <jdk_vendor> = ibm
                #Solaris: <jdk_vendor> = sun
                #HP_UX: <jdk_vendor> = hp
                if platformUtil.IS_zOS_OS():
                    #server_ent[parm] = "-agentlib:am=" + runtimep + "/"
                    server_ent[parm] = "-agentlib:am=" + "${WAS_SERVER_NAME}"
                else:
                    #For non-z/OS platforms, get jdk version from pdl
                    jdkversion = "16"
                    debug("jdkversion: " + str(jdkversion))
                    amlib = "am_ibm_"
                    if platformUtil.getOSFamily() == "SOLARIS":
                        amlib = "am_sun_" + str(jdkversion)
                    elif platformUtil.getOSFamily() == "HP-UX":
                        amlib = "am_hp_" + str(jdkversion)
                    else:
                        amlib = "am_ibm_" + str(jdkversion)
                    server_ent[parm] = "-agentlib:" + amlib + "=${WAS_SERVER_NAME}"
                    if platformUtil.getOSFamily() == "OS400":
                        server_ent[parm] = server_ent[parm] + "," + pdl["WASNODE"][0][serveri]
                        debug("OS400 agentlib: " + server_ent[parm])
                    #endif
            # elif parm == "SMFLIST":
            elif parm == "SMF":
                smfvalue = pdl["SMFLIST"][0][serveri]
                # smfvalue as of 0127 is a string
                #assert smfvalue in (True,False,None), "Unexpected value for SMF PDL data = %s, type = %s" % (smfvalue, type(smfvalue))

                if smfvalue == "1" or smfvalue == True:
                    server_ent[parm] = "Yes"
                elif smfvalue == "0" or smfvalue == False:
                    server_ent[parm] = "No"
                else:
                    server_ent[parm] = "Leave"
            elif parm == "LOG_ROOT":
                server_ent[parm] = unicode(pdl["USERINSTALLROOT"][0][serveri], "utf-8") + "/logs"
            elif parm == "LIBPATH":
                # Change LIBPATH parameter name to right name base on platform:
                # For z/OS, AIX, and OS400: LIBPATH => LIBPATH
                # For Linux and Solaris: LIBPATH ==> LD_LIBRARY_PATH
                # For Windows, LIBPATH ==> PATH
                if platformUtil.isWindows():
                    server_ent[parm] = "PATH"
                elif platformUtil.IS_zOS_OS() or platformUtil.IS_AIX_OS() or platformUtil.IS_OS400_OS():
                    server_ent[parm] = "LIBPATH"
                elif platformUtil.getOSFamily() == "HP-UX":
                    server_ent[parm] = "SHLIB_PATH"
                else:
                    server_ent[parm] = "LD_LIBRARY_PATH"
            elif parm == "PATHDELIMITER":
                if platformUtil.isWindows():
                    server_ent[parm] = ";"
                else:
                    server_ent[parm] = ":"
            elif parm == "PLATFORM":
                server_ent[parm] = platformUtil.getOSFamily()
            elif parm == "PLATFORMBITMODE":
                server_ent[parm] = bitMode
            elif parm == "TTAPILIBPATH" and not platformUtil.IS_zOS_OS():
                server_ent[parm] = "${ITCAMDCHOME}/toolkit/lib" + bitMode + "/ttapi"
            elif parm == "ITCAMDC_INPUT":
                dcInputFilePath = pdl["ITCAMDCINPUT"][0][serveri]
                if dcInputFilePath != None:
                    if platformUtil.isWindows():
                        dcInputFilePath = utils.replaceBackSlash(dcInputFilePath)
                    dcInputFilePaths = dcInputFilePath.split("/")
                    #The last element is the array dcInputFilePaths is the file name
                    dcInputFileName = dcInputFilePaths[len(dcInputFilePaths) - 1]
                    debug("dcInputFileName: " + dcInputFileName)
                    server_ent[parm] = dcInputFileName
            elif parms[parm] == None:
                assert False, getmsg("CFG5015E", "Data element {0} is not present in the PDL", [parm])
            elif isinstance(pdl[parms[parm]][0],types.ListType):
                if  len(pdl[parms[parm]][0]) != 0:
                    server_ent[parm] = unicode(pdl[parms[parm]][0][serveri], "utf-8")
                else:
                    server_ent[parm] = "None"
            else:
                paramValue = unicode(pdl[parms[parm]][0], "utf-8")
                if paramValue != None:
                    paramValue = paramValue.strip()
                server_ent[parm] = paramValue
        servers_dict += (server_ent,)

    debug("Exit create_substitution_dictionary()")
    return servers_dict


def create_substitution_dictionary_xd(pdl):
    '''
        This routine will create a substitution dictionary for XD templates.
    '''

    # This will create a tuple.  Each entry in the tuple will correspond
    # to the server entry in the SERVER entry of the PDL.  The substitution
    # values will be based on the contents of the PDL.

    #
    # Look for the "SERVERS" key in the PDL.  If it does not exist, then
    # raise an exception and exit.
    # The value for the SERVERS key is a tuple, and the first entry in the
    # tuple is the server list, which is also a tuple.  Each entry in the server
    # tuple represents a server.  This server entry is a dictionary which contains all
    # information about the server.
    #
    assert "SERVERS" in pdl.keys(), getmsg("CFG5015E", "Data element {0} is not present in the PDL", ["SERVERS"])
    assert pdl["SERVERS"][0] != None, getmsg("CFG5017E", "Data element {0} of PDL has no entries", ["SERVERS"])
    assert type(pdl["SERVERS"][0]) == type(()) or type(pdl["SERVERS"][0]) == type([]), getmsg("CFG5019E", "Unexpected value type {0} for PDL key {1}", [type(pdl["SERVERS"][0]), "SERVERS"])


    parms = {
            "ITCAMDCVERSION": "VERSION",
            "ITCAMDCHOME":"TEMPLATEDCHOME",
            "ITCAMDC_INPUT":"ITCAMDCINPUT",
            "TTAPILIBPATH"  :"TTAPILIBPATH",
            "TTAPICLASSPATH":"TTAPICLASSPATH",
            "SMF":"SMFLIST",
            "ZOSLIBPATH":"ZOSLIBPATH",
            "CURRENTGCLOGLOCATION":"CURRENTGCLOGLOCATION",
            "GC_PARMS":"GC_PARMS",


            "AM_HOME":None,
            "AGENTLIB":None,
            "PATHDELIMITER":None,
            "PLATFORMBITMODE":None,
            "LIBPATH":None,
            "PLATFORM":None,
           }


    servers_dict = ()
    confighome = pdl["TEMPLATEDCHOME"][0]
    platform = pdl["TEMPLATEOS"][0]
    # Replace "\" in windows file path by "/" so the file path will work well in Java environment
    if platform.startswith("win"):
        confighome = utils.replaceBackSlash(confighome)

    for serveri in range(len(pdl["SERVERS"][0])):
        server_ent = {}
        bitMode = pdl["TEMPLATEINTERPLIST"][0][serveri]
        for parm in parms.keys():
            # Handle special cases first
            if parm == "AM_HOME":
                server_ent[parm] = confighome + "/itcamdc"
            elif parm == "AGENTLIB":
                #runtimep = pdl["RUNTIMEP"][0][serveri]
                # For a distributed DC, agentlib format is like: -agentlib:am_ibm_<jdk version>
                # No runtime directory or other file directory involved
                # For now, comment out file path conversion
                #if platformUtil.isWindows():
                    # Replace "\" in windows file path by "/" so the file path will work well in Java environment
                    #runtimep = utils.replaceBackSlash(runtimep)
                #For the native library name in "-agentlib:xx":
                #z/OS: am
                #other platforms: am_<jkd vendor>_<jdk version>
                #for now, hard code as following until we have code to detect jdk_vendor:
                #Windows, Linux, AIX, and OS400: <jdk_vendor> = ibm
                #Solaris: <jdk_vendor> = sun
                #HP_UX: <jdk_vendor> = hp
                if platform is "zos":
                    server_ent[parm] = "-agentlib:am=" + "${WAS_SERVER_NAME}"
                else:
                    #For non-z/OS platforms, get jdk version from pdl
                    jdkversion = "16"
                    debug("jdkversion: " + str(jdkversion))
                    amlib = "am_ibm_"
                    if platform.startswith("solaris"):
                        amlib = "am_sun_" + str(jdkversion)
                    elif platform.startswith("hpux"):
                        amlib = "am_hp_" + str(jdkversion)
                    else:
                        amlib = "am_ibm_" + str(jdkversion)
                    server_ent[parm] = "-agentlib:" + amlib + "=${WAS_SERVER_NAME}"
                    if platform.startswith("OS400"):
                        #server_ent[parm] = server_ent[parm] + "," + pdl["WASNODE"][0][serveri]
                        debug("OS400 agentlib: " + server_ent[parm])
                    #endif
            # elif parm == "SMFLIST":
            elif parm == "SMF":
                if platform.startswith("zos"):
                    smfvalue = pdl["SMFLIST"][0][serveri]
                    if smfvalue == "1" or smfvalue == True:
                        server_ent[parm] = "Yes"
                    elif smfvalue == "0" or smfvalue == False:
                        server_ent[parm] = "No"
                    else:
                        server_ent[parm] = "Leave"
                else:
                    server_ent[parm] = "No"

            #elif parm == "LOG_ROOT":
            #    server_ent[parm] = unicode(pdl["USERINSTALLROOT"][0][serveri], "utf-8") + "/logs"
            elif parm == "LIBPATH":
                # Change LIBPATH parameter name to right name base on platform:
                # For z/OS, AIX, and OS400: LIBPATH => LIBPATH
                # For Linux and Solaris: LIBPATH ==> LD_LIBRARY_PATH
                # For Windows, LIBPATH ==> PATH
                if platform.startswith("win"):
                    server_ent[parm] = "PATH"
                elif platform.startswith("zos") or platform.startswith("aix") or platform.startswith("OS400"):
                    server_ent[parm] = "LIBPATH"
                elif platform.startswith("hpux"):
                    server_ent[parm] = "SHLIB_PATH"
                else:
                    server_ent[parm] = "LD_LIBRARY_PATH"
            elif parm == "PATHDELIMITER":
                if platform.startswith("win"):
                    server_ent[parm] = ";"
                else:
                    server_ent[parm] = ":"
            elif parm == "PLATFORM":
                server_ent[parm] = platform
            elif parm == "PLATFORMBITMODE":
                server_ent[parm] = bitMode
            elif parm == "TTAPILIBPATH" and (platform is not "zos"):
                server_ent[parm] = confighome + "/toolkit/lib/" + bitMode + "/ttapi"
            elif parm == "ITCAMDC_INPUT":
                dcInputFilePath = pdl["ITCAMDCINPUT"][0][serveri]
                if dcInputFilePath != None:
                    if platform.startswith("win"):
                        dcInputFilePath = utils.replaceBackSlash(dcInputFilePath)
                    dcInputFilePaths = dcInputFilePath.split("/")
                    #The last element is the array dcInputFilePaths is the file name
                    dcInputFileName = dcInputFilePaths[len(dcInputFilePaths) - 1]
                    debug("dcInputFileName: " + dcInputFileName)
                    server_ent[parm] = dcInputFileName
            elif parms[parm] == None:
                assert False, getmsg("CFG5015E", "Data element {0} is not present in the PDL", [parm])
            elif isinstance(pdl[parms[parm]][0],types.ListType):
                if  len(pdl[parms[parm]][0]) != 0:
                    server_ent[parm] = unicode(pdl[parms[parm]][0][serveri], "utf-8")
                else:
                    server_ent[parm] = "None"
            else:
                paramValue = unicode(pdl[parms[parm]][0], "utf-8")
                if paramValue != None:
                    paramValue = paramValue.strip()
                server_ent[parm] = paramValue
        servers_dict += (server_ent,)

    return servers_dict


###
#   loadProperties
###

def loadProperties (path):
    '''
        Load a Java properties file(path) into a Dictionary.
        returns the dictionary object
    '''
    result = {}

    try:
        fis = io.FileInputStream(path)
        bis = io.BufferedInputStream(fis)
        props = util.Properties()
        props.load(bis)
        bis.close()
        for key in props.keySet().iterator():
            result[key] = props.get(key)
        return result

    except:
        log.logException("loadProperties failed for: " + path)
        msg = getmsg("CFG1114E", "Invalid path {0} - loadProperties failed ", [path])
        print msg
        raise

###
#   Read a property
###

def getProperty (properties, name, default=None):
    '''
        reads a property from the loaded properties
    '''
    return properties.get(name, default)

###
#   loadProperties basic style
#   The java-based style in loadProperties breaks down when properties contain single backslashes
###
def loadPropertiesBasic(path, encoding=None ):
    '''
        Load a standard properties file(path) into a Dictionary.
        returns the dictionary object
        Commented lines are ignored
    '''
    prop_dict = {}
    debug("loadPropertiesBasic entry - path: <%s> " % (path))
    encode = "utf-8"
    try:
        if encoding == None:
            if platformUtil.IS_zOS_OS():
                encode = "cp1047"
        else:
            encode = encoding
        infile = codecs.open(path, "r", encode)
        for record in infile.readlines():
            line = record.rstrip("\r\n")
            value = None
            if line == "":  continue
            if line.startswith("#"):
                continue
            ws = line.split("=")
            if len(ws) > 1:
                prop_dict[ws[0]] = ws[1]
            else:
                ws[0] = ws[0].rstrip()
                prop_dict[ws[0]] = ""
                debug("..property line no value: %s" % ws[0])
    except:
        log.logException("loadPropertiesBasic failed for: " + path)
        msg = getmsg("CFG1114E", "Invalid path {0} - loadProperties failed ", [path])
        print msg
        raise

    debug("loadPropertiesBasic exit")
    return prop_dict


###
#   Write the version.properties file to the log
###

def logVersionProperties(pdl=None):
    '''
        Write the version.properties to the debug log
        @param  pdl optional. If not supplied, config_home obtained from cwd
    '''
    ver_properties_path = ""
    try:
        if pdl is None:
            cwd = os.getcwd()
            if cwd.endswith("/bin/lib"):
                config_home = cwd.replace("/bin/lib","")
            else:
                if cwd.endswith("/bin"):
                    config_home = cwd.replace("/bin","")
                else:
                    debug("Unable to locate confighome from cwd: %s" % cwd)
                    return
        else:
            config_home = utils.getData(pdl, "CONFIGHOME")
        ver_properties_path = config_home+"/itcamdc/etc/version.properties"
        if not os.path.isfile(ver_properties_path):
            debug("Invalid location of version.properties: %s" % str(ver_properties_path))
            debug("  Unable to copy version.properties to this log")
            return
        f = None
        box_line_30 = "##############################"
        if platformUtil.IS_zOS_OS():
            f = codecs.open(ver_properties_path, mode="r", encoding="cp1047")
        else:
            f = open(ver_properties_path, "r")
        debug(box_line_30)
        for line in f.readlines():
            line = line.rstrip("\r\n")
            if line is None or line == "": continue
            debug(line)
        debug(box_line_30)
    except Exception, msgVar:
        debug("Exception %s reading version.properties file at %s" % (msgVar,ver_properties_path))


###
#   Convert an EBCDIC string to ASCII
###

def convert_ebcdic_to_ascii(string):
    '''
        Convert an EBCDIC string to ASCII
        @param  string

        @returns converted string
    '''
    if string is None or string == "": return string
    #  convert ebcdic --> unicode
    str_unicode, len = codecs.getdecoder("cp1047")(string)
    #  convert unicode  -->  ascii
    str_ascii, len   = codecs.getencoder("ascii")(str_unicode)
    return str_ascii


###
#   Convert an ASCII string to EBCDIC
###

def convert_ascii_to_ebcdic(string):
    '''
        Convert an ASCII string to EBCDIC
        @param  string
        @returns converted string
    '''
    if string is None or string == "": return string
    #  convert ebcdic --> unicode
    str_unicode, len = codecs.getdecoder("ascii")(string)
    #  convert unicode  -->  ascii
    str_ebcdic, len  = codecs.getencoder("cp1047")(str_unicode)
    return str_ebcdic


###
# read saved properties in format style "# prop=value"
###
def read_saved_properties(path, dict, enc="utf-8"):
    '''
        Read saved properties
        @param path - full path/filename
        @param dict - dictionary of properties to find
        @return - dictionary of properties and values read
    '''

    debug("Entry read_saved_properties %s - for %s using encoding %s" % (fixlev, path, enc))

    if not os.path.isfile(path):
        debug("  read_saved_properties - exit - file not found")
        return dict
    delim_counter = 0
    infile = codecs.open(path, mode="r", encoding=enc)
    for record in infile.readlines():
        line = record.rstrip("\r\n")
        if line.startswith("##############"):
            delim_counter +=1
            if  delim_counter == 1: continue
            if  delim_counter == 2: break
        if line.startswith("# "):
            line = line[len("# "):len(line)]
            ipprop = line.split("=")
            if len(ipprop) == 2:
                if dict.has_key(ipprop[0]):
                    dict[ipprop[0]] = ipprop[1]

    debug("read_saved_properties - result dict: %s" % str(dict))

    debug("Exit read_saved_properties")
    return dict

###
# z/OS update runtime access
###
def update_runtime_access_zOS(pdl):
    '''
        update_runtime_permissions_zOS(pdl):
        @param pdl
        @return True | False indicating mission accomplished
    '''

    debug("Entry update_runtime_access_zOS" % fixlev)

    rtn = True
    paths_with_errors = []

    if platformUtil.IS_zOS_OS():
        itcam_owner  = pdl["OWNER"][0]
        itcam_access = pdl["ACCESS"][0]
        runtime_root = pdl["CONFIGHOME"][0] + "/runtime"
        metadata     = runtime_root + "/wsBundleMetaData"
        paths_to_set = []

        # 1. runtime_root + /wsBundleMetaData
        paths_to_set.append(metadata)

        # 2. Server specific files
        for serveri in range(len(pdl["SERVERS"][0])):
            server = pdl["SERVERS"][0][serveri]
            input_prefix = utils.getServerBasedFilePrefix(pdl, serveri)
            wascell = utils.getData(pdl, "WASCELL", serveri)
            wasnode = utils.getData(pdl, "WASNODE", serveri)
            wasprofile = "default"
            wasserver = utils.getData(pdl, "SERVERS", serveri)
            dcmanualinput_filename = wasprofile + "." + wascell + "." + wasnode + "." + wasserver + "." + "DCManualInput.txt"
            dcmanualinput_path   = runtime_root + os.sep + dcmanualinput_filename
            # Do not update access and permissions to input.properties. WAS does not read them.
            # inputproperties_path = runtime_root + os.sep + input_prefix + "input.properties"
            # inputproperties_path.bak = runtime_root + os.sep + input_prefix + "input.properties.bak"
            node_server_dir = runtime_root+ "/%s.%s" %(wasnode, wasserver)

            paths_to_set.append(dcmanualinput_path)
            paths_to_set.append(node_server_dir)

        # 3. Migrated app server runtimes if any
        migr_runtimes = get_runtimes_created()
        if migr_runtimes is not None:
            for new_runtime in migr_runtimes:
                paths_to_set.append(new_runtime)

        for path in paths_to_set:
            try:
                if os.path.exists(path):
                    debug("updating permissions to path: %s" % path)
                    # 1. set default permissions
                    os.system("chmod -hRf a+rwX " + path)
                    # 2. call method to set owner and access using pdl values
                    if itcam_owner is not None and itcam_access is not None:
                        changeowner(path, itcam_owner, itcam_access)
                else:
                    rtn = False
                    paths_with_errors.append(path)
                    debug("path does not exist: %s" % path)
            except Exception, msgVar:
                rtn = False
                log.logException("Exception setting runtime_root ownership/permissions - server: %s " % server)

        # 3. Make sure the search permission is set
        try:
            os.system("chmod -hRf a+X " + runtime_root)
        except OSError:
            log.logException("Exception setting chmod -hRf a+X %s"  % runtime_root)
    else:
        debug("platform is not z/OS")
        rtn = False

    debug("Exit update_runtime_access_zOS - %s" % rtn)
    return rtn
