####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#
#####################################################################

'''
  This module configures Development Edition.
'''

fixlev = "40296" # Last APAR, defect or story to update this module

import doprompt as prompt
import utils
import platformUtil

from resourcebundle import getmsg as getmsg
from debugUtil import logDebug as debug


def config_de(pdl, soa71_migrate = False):
    '''
       Prompts the user for DE configuration options.

       @param pdl - PDL reference
       @param soa71_migrate - True if soa71 migration processing otherwise False
    '''

    debug("Entry config_de %s" % fixlev)

    (only_wlp_selected, wlp_servers) = utils.get_wlp_selected(pdl)
    zOS_flag = platformUtil.isZ()
    OS400_flag = platformUtil.IS_OS400_OS()

    debug("At entry we have  SOA mig: %s  wlp: %s  z/OS: %s  OS/400: %s DE: %s" % (soa71_migrate, only_wlp_selected, zOS_flag, OS400_flag, repr(pdl["DE"][0:2])))

    if only_wlp_selected:

        if pdl["DE"][1] != "configtema":
            debug("Forcing DE mode off because it wasn't set by configtema")
            pdl["DE"][0:2] = "0", "configde"
        else:
            debug("Retaining DE mode of '%s' because it was set by configtema" % pdl["DE"][0])


    elif zOS_flag or OS400_flag:

        debug("Forcing DE mode off because this is %s" % zOS_flag and "z/OS" or "OS/400")
        pdl["DE"][0:2] = "0", "configde"

    else:

        # If this is as the result of migrate call for soa71, then set the pdl
        # without asking question.

        if soa71_migrate and not pdl["SILENT"][0] and pdl["DE"][1] != "configtema":
            pdl["DE"][0:2] = "0", "configde"
        else:
            print "\n%s" % getmsg("FLOWERBOX_DE","Optional integration with Application Performance Diagnostics Lite")
            debug("pdl[DE]: %s  string: '%s'" % (pdl["DE"], utils.getDataString(pdl, "DE")))
            if pdl["DE"][1] == "configtema" and utils.getDataString(pdl, "DE") == "1":
               print "%s\n" % getmsg("CFG1184I", "Prompt bypassed, APD was already activated by configtema")
            else:
               prompt.do_prompt("DE", pdl)

    # Make the pdls for the DE as a list for the number of
    # servers in SERVERS pdl.

    de = utils.getDataString(pdl, "DE")
    nservers = len(pdl["SERVERS"][0])
    pdl["DELIST"][0:2] = [de]*nservers, "configde"
    debug("DE values set as DE: %s  DELIST: %s" % (repr(pdl["DE"][0:2]), repr(pdl["DELIST"][0:2])))

    debug("Exit config_de")


def set_defaultvalues(pdl):
    '''
       Sets the default values in the pdl.  The first selected server's values
       are used to set the default.

       @param pdl - PDL reference
    '''

    debug("Entry set_defaultvalues %s" % fixlev)

    # set default de values.

    pdl["DE"][0:2] = pdl["DELIST"][0][0], "configde"
    debug("DE defaults set as DE: %s  DELIST: %s" % (repr(pdl["DE"][0:2]), repr(pdl["DELIST"][0:2])))

    debug("Exit set_defaultvalues")
