#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved. 
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
from string import lower
'''
  This module prompts the user for GC data.
'''

import debugUtil as log
import doprompt as prompt
import utils
import platformUtil
import os

from debugUtil import logDebug as debug
from resourcebundle import getmsg as getmsg

import platform

ELIMINATE = '1'
YES = '1'

def config_gc(pdl, soa71_migrate = False):
    '''
       Propmts the user for GC data.
    
       @param pdl - PDL reference
       @param soa71_migrate - Ture if soa71 migration processing otherwise False
    '''
    
    debug("Entry config_gc")
    
    try:
        #
        # Create GCLOGPATH as a list, as each server must have its own
        # GC log path.  As the prompt gets the log location, the cell, node,
        # and server name is placed in the log location name to make it unique. 
        #
        #utils.setData(pdl, "GCLOGPATH", [], "configgc")
        (only_wlp_selected, wlp_servers) = utils.get_wlp_selected(pdl)
        java_vendor = platform.java_ver(vendor="ibm")[1]
        debug("java vendor=%s" % java_vendor)
        if (platformUtil.IS_zOS_OS()):
            
            #utils.setData(pdl, "GC_PARMS", "-verbose:gc ", "configgc")
            #
            # GC_PARMS is a list which cintain the GC parm data, biut for ZOS
            # environment, only -verbose:gc is specified, so to keep the format
            # the same, we need to iterate thru servers and set the list, which 
            # seems all have the same value.
            #
            utils.setData(pdl, "GC_PARMS", [], "configgc")
            for serveri in range(len(pdl["SERVERS"][0])):
                utils.getData(pdl,"GC_PARMS").append("-verbose:gc ")
            #
            # If this is as the result of migrate call for soa71, then set the pdl
            # without asking question.
            #
            if ((soa71_migrate) and (not pdl["SILENT"][0])):
                utils.setData(pdl, "GCOUTPUT", "0", "configgc")
            else:
                print
                print "%s" % getmsg("FLOWERBOX_GCZ","JVM Garbage Collection information will be gathered")
                prompt.do_prompt("GCOUTPUT", pdl)
                
            gcanswer =  utils.getDataString(pdl, "GCOUTPUT")   
            
            #
            # Based on the answer set the GCLOGPATH 
            #
            if (gcanswer == ELIMINATE):
                utils.setData(pdl, "GCLOGPATH", "JESLOG:SYSOUT:KEEP", "Prompt")
            else:
                utils.setData(pdl, "GCLOGPATH", "JESLOG:SYSOUT:DISCARD", "Prompt") 
            #
            # Make the pdls for the GCLOGPATH as a list for the number of
            # servers in SERVERS pdl.
            #
            gclogpath = utils.getData(pdl, "GCLOGPATH")
            utils.setData(pdl, "GCLOGPATHLIST", [], "configgc")
            utils.setData(pdl, "GCOUTPUTLIST", [], "configgc")
            utils.setData(pdl, "CURRENTGCLOGLOCATION", [], "configgc")
            serverlist = utils.getData(pdl, "SERVERS")
            for serveri in range(len(serverlist)):
                utils.getData(pdl, "GCLOGPATHLIST").append(gclogpath); 
                utils.getData(pdl, "CURRENTGCLOGLOCATION").append(gclogpath); 
                utils.getData(pdl, "GCOUTPUTLIST").append(gcanswer);     
                
        else:
            #
            # If this is as the result of migrate call for soa71, then set the pdl
            # without asking question.
            #
            if ((soa71_migrate) and (not pdl["SILENT"][0])):
                utils.setData(pdl, "GCOUTPUTDIST", "0", "configgc")
            else:
                print
                print "%s" % getmsg("FLOWERBOX_GCDIST","Advanced Settings")
                prompt.do_prompt("GCOUTPUTDIST", pdl)
                
            gcanswer =  utils.getDataString(pdl, "GCOUTPUTDIST")
            #
            # Based on the answer set the GCLOGPATH 
            #
            if (gcanswer == YES):
                debug("GC Log Path(Before) is: %s" % (utils.getData(pdl, "GCLOGPATH")))
                prompt.do_prompt("GCLOGPATH", pdl)
                gclogpath =  utils.getData(pdl, "GCLOGPATH") 
                
                #
                # If gc log path starts with "${SERVER_LOG_ROOT}", the modification
                # to the path is not taking place as the path will be unique, otherwise, 
                # check the path for "/" or "\".  We are not using os.sep
                # as the user could specify "/" for windows and that is an
                # ok specification, so we check for both characters.
                #
                if (not gclogpath.startswith("${SERVER_LOG_ROOT}")):
                    pathe = gclogpath.split("/")
                    if (len(pathe) == 1):
                        pathe = gclogpath.split("\\")
                    index = len(pathe)
                    if (len(pathe) == 1):
                        gclogfile = pathe[0]
                    else:
                        gclogfile = pathe[index-1]
                #
                # Create GC_PARMS as a list, as each server must have its own
                # GC log path.  As the prompt gets the log location, the node,
                # and server name is placed in the log location name to make it unique. 
                # User could have provided the gc log location already, and if it is
                # provided, then it will be used.
                #
                utils.setData(pdl, "GC_PARMS", [], "configgc")
                utils.setData(pdl, "GCLOGPATHLIST", [], "configgc")
                utils.setData(pdl, "CURRENTGCLOGLOCATION", [], "configgc")
                utils.setData(pdl, "GCOUTPUTDISTLIST", [], "configgc")
                reconfig = pdl["RECONFIG"][0]
                for serveri in range(len(pdl["SERVERS"][0])):
                        gclogpath =  utils.getData(pdl, "GCLOGPATH") 
                        orig_gclogpath = gclogpath
                        if (not gclogpath.startswith("${SERVER_LOG_ROOT}")):
                            wasprofile = utils.getData(pdl,"PROFILE", serveri)
                            wascell = utils.getData(pdl,"WASCELL", serveri)
                            wasnode = utils.getData(pdl,"WASNODE", serveri)
                            wasserver = utils.getData(pdl,"SERVERS", serveri)
                            if serveri in wlp_servers:
                                new_gclogfile = wasnode + "_" + wasserver + "." + gclogfile
                            else:
                                new_gclogfile = wasprofile + "." + wascell + "." + wasnode + "." + wasserver + "." + gclogfile
                            gclogpath = ""
                            if (len(pathe) == 1):
                                gclogpath = new_gclogfile
                            else:
                                for gclogi in range(len(pathe) - 1):
                                    if (gclogi == 0):
                                        gclogpath = pathe[gclogi]
                                    else:
                                        gclogpath = gclogpath + os.sep + pathe[gclogi]
                                gclogpath = gclogpath + os.sep + new_gclogfile
                        utils.getData(pdl, "GCLOGPATHLIST").append(orig_gclogpath)
                        utils.getData(pdl, "CURRENTGCLOGLOCATION").append(gclogpath)
                        utils.getData(pdl, "GCOUTPUTDISTLIST").append(gcanswer); 
                        debug("Gc log path is - %s" % (gclogpath))
                        #        
                        # Set the gc log path in the pdl, and set the gc parms based
                        # on the os platform.
                        #  
                        #get java vendor
                        
                        
                        if (platformUtil.IS_HPUX_OS() or platformUtil.IS_SOL_OS()) or (serveri in wlp_servers and lower(java_vendor).find("ibm") == -1):
                            gcparms = "-XX:+PrintGCTimeStamps -verbosegc -Xloggc:%s" %(gclogpath)
                        else:
                            gcparms = "-verbosegc -Xverbosegclog:%s" %(gclogpath)  
                            
                        debug("Gc parms is - %s" % (gcparms))
                        utils.getData(pdl,"GC_PARMS").append(gcparms) 
            else:
                #
                # Check the current gc log location.  If it is provided, then
                # GCLOGPATHLIST value is checked based on config or reconfig.
                # The GCLOGPATHLIST is already created for reconfig, but for config
                # the pdl entry is created.  
                # If current gc location is not present, then there is no gc log 
                # location previously, so no location is set.
                # If current gc log location is present, then based on GCLOGPATHLIST
                # the location is set.  If this is config, then there is no need to 
                # check GCLOGPATHLOIST, as there is no entry for it, so the current
                # log location will be set for gc.
                # If this is reconfig, then if GCLOGPATHLIST is present, then it inidcates
                # that gc log location was provided previously, so now is no longer needed.
                #
                utils.setData(pdl, "GC_PARMS", [], "configgc")
                reconfig = pdl["RECONFIG"][0]
                debug("Reconfig : %s" % reconfig)
                utils.setData(pdl, "GCOUTPUTDISTLIST", [], "configgc")
                #
                # Note that when doing config, the GCLOGPATHLIST is not created, so it is
                # created her, but in the case of reconfig, it is alreadt created.
                #
                if (not reconfig):
                    utils.setData(pdl, "GCLOGPATHLIST", [], "configgc")
                #
                # Iterate thru all the servers and set the GC parms accordingly.
                #    
                for serveri in range(len(pdl["SERVERS"][0])):
                    utils.getData(pdl, "GCOUTPUTDISTLIST").append(gcanswer); 
                    wasserver = utils.getData(pdl,"SERVERS", serveri)
                    gcloglocation = utils.getData(pdl, "CURRENTGCLOGLOCATION", serveri)
                    if (gcloglocation != None and gcloglocation not in "None"):
                        gclogpath = gcloglocation
                        if (reconfig):
                            gclogpath_pdl = utils.getData(pdl, "GCLOGPATHLIST", serveri)
                            if (gclogpath_pdl is None or gclogpath_pdl == "None"):
                                debug("GC log location already set, being reused: %s" % (gclogpath))
                                debug("GC log Location for server %s: %s" % (wasserver, gclogpath))
                                #
                                # based on a platform set the GC entries
                                #
                                if (platformUtil.IS_HPUX_OS() or platformUtil.IS_SOL_OS()):
                                    gcparms = "-XX:+PrintGCTimeStamps -verbosegc -Xloggc:%s" %(gclogpath)
                                else:
                                    gcparms = "-verbosegc -Xverbosegclog:%s" %(gclogpath)
                            else:
                                gcparms = "-verbosegc " 
                                utils.setData(pdl, "GCLOGPATHLIST", "None", "configgc", serveri)
                                utils.setData(pdl, "CURRENTGCLOGLOCATION", "None", "configgc", serveri)   
                        else:
                            #
                            # based on a platform set the GC entries
                            #
                            if (platformUtil.IS_HPUX_OS() or platformUtil.IS_SOL_OS()):
                                gcparms = "-XX:+PrintGCTimeStamps -verbosegc -Xloggc:%s" %(gclogpath)
                            else:
                                gcparms = "-verbosegc -Xverbosegclog:%s" %(gclogpath)
                            utils.getData(pdl, "GCLOGPATHLIST").append("None");
                    else:
                        gcparms = "-verbosegc"
                        if (reconfig):
                            utils.setData(pdl, "GCLOGPATHLIST", "None", "configgc", serveri)
                            utils.setData(pdl, "CURRENTGCLOGLOCATION", "None", "configgc", serveri)   
                        else:
                            utils.getData(pdl, "GCLOGPATHLIST").append("None");
                            utils.setData(pdl, "CURRENTGCLOGLOCATION", "None", "configgc", serveri)     
                        
                    debug("Gc parms is - %s" % (gcparms))    
                    utils.getData(pdl,"GC_PARMS").append(gcparms) 
                    
        debug("Exit config_gc")   
    
    except Exception, errmsg:
        log.logException("Error occurred while setting the GC - %s" % (errmsg))
        raise 
    
    
def set_defaultvalues(pdl):
    '''
       Sets the default values in the pdl.  The first selected server's values
       are used to set the default.
       
       @param pdl - PDL reference  
    '''   
    
    debug("Entry set_defaultvalues")    
    
    #
    # set default gc values.  
    #
    if (platformUtil.IS_zOS_OS()):
        gcanswer =  utils.getDataString(pdl, "GCOUTPUTLIST", 0) 
        utils.setData(pdl, "GCOUTPUT", gcanswer, "configgc")    
    else:    
        gcanswer =  utils.getDataString(pdl, "GCOUTPUTDISTLIST", 0)
        utils.setData(pdl, "GCOUTPUTDIST", gcanswer, "configgc")
      
    gclogpath = utils.getData(pdl, "GCLOGPATHLIST", 0)
    if (gclogpath is None or gclogpath == "None"):
        gclogpath = utils.getData(pdl, "CURRENTGCLOGLOCATION", 0)
        
    utils.setData(pdl, "GCLOGPATH", gclogpath, "configgc")
    
        
    debug("Exit set_defaultvalues")    
