#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved. 
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
   This module prompts the user for specifying max heap to be added to the current
   heap size or not.
'''

import debugUtil as log
import doprompt as prompt
import utils
import platformUtil

from debugUtil import logDebug as debug

CONFIGURE_HEAP = '1'


def config_maxheap(pdl, soa71_migrate = False):
    '''
       Prompts the user for max heap size.
    
       @param pdl - PDL reference
       @param soa71_migrate - Ture if soa71 migration processing otherwise False
    '''
    
    debug("Entry config_maxheap")
    
    if (not platformUtil.IS_zOS_OS()):
        #
        # If this is as the result of migrate call for soa71, then set the pdl
        # without asking question.
        #
        if ((soa71_migrate) and (not pdl["SILENT"][0])):
            utils.setData(pdl, "HEAPSIZE", "0", "configmaxheap")
        else:
            prompt.do_prompt("HEAPSIZE", pdl)
            
        heapanswer =  utils.getDataString(pdl, "HEAPSIZE") 
        utils.setData(pdl, "HEAPSIZELIST", [], "configmaxheap")
        #
        # If requested to increase heap size, Add 128m to the
        # current max heap size.  If max heap size is not provided,
        # we use 256M and add 128m to it.
        #
        if (heapanswer == CONFIGURE_HEAP):
            utils.setData(pdl, "MAXHEAPSIZE", [], "configmaxheap")
            for serveri in range(len(pdl["SERVERS"][0])):
                wasserver = utils.getData(pdl,"SERVERS")[serveri]
                currentmaxheap = utils.getData(pdl,"CURRENTMAXHEAPSIZE")[serveri]
                if (utils.isInt(currentmaxheap) == 1):
                    int_currentmaxheap = int(currentmaxheap)
                    if (int_currentmaxheap == 0):
                        int_currentmaxheap = 256
                    int_currentmaxheap = int_currentmaxheap + 128
                    utils.getData(pdl,"MAXHEAPSIZE").append(str(int_currentmaxheap))
                    utils.getData(pdl,"HEAPSIZELIST").append(heapanswer)
                    utils.setData(pdl, "CURRENTMAXHEAPSIZE", int_currentmaxheap, "configmaxheap", serveri)
                    log.logDebug("Max heap size set for server %s: %d" % (wasserver, int_currentmaxheap))
        else:
            utils.setData(pdl, "MAXHEAPSIZE", [], "configmaxheap")
            for serveri in range(len(pdl["SERVERS"][0])):
                wasserver = utils.getData(pdl,"SERVERS")[serveri]
                utils.getData(pdl,"MAXHEAPSIZE").append("0")
                utils.getData(pdl,"HEAPSIZELIST").append(heapanswer)
                log.logDebug("Max heap size set for server %s: %s" % (wasserver, "0"))
           
    debug("Exit config_maxheap")       
