#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved. 
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
   This module prompts the user for MS host and port if not already provided
   thru the environment variable or arguments.
'''

import sys

import java.net.URL
import java.util.Properties

import resourcebundle as rmsg
import debugUtil as log
import doprompt as prompt
import utils
import configproxy

from debugUtil import logDebug as debug
from ConfigurationException import ConfigurationException

CONFIGURE_MS = '1'
SET_RMIPORT = '1'
FILE_TO_REQ = '.ITCAM61_MS_CONTEXT.properties'
MSCONNECT = "MSCONNECT"
MSCONNECT_LIST = "MSCONNECTLIST"
MSHOME = "MSHOME"
MSHOME_LIST = "MSHOMELIST"
KERNELIP = 'KERNELIP'
KERNELPORT = 'KERNELPORT01'
RFSPORT  = 'RFSPORT01'
RMIPORTSET = 'RMIPORTSET'
PROBECONTROLLERRMIPORT = 'PROBECONTROLLERRMIPORT'
PROBERMIPORT = 'PROBERMIPORT'
KERNELIP_LIST = 'KERNELIPLIST'
KERNELPORT_LIST = 'KERNELPORT01LIST'
RFSPORT01_LIST  = 'RFSPORT01LIST'
RMIPORTSET_LIST = 'RMIPORTSETLIST'
PROBECONTROLLERRMIPORT_LIST = 'PROBECONTROLLERRMIPORTLIST'
PROBERMIPORT_LIST = 'PROBERMIPORTLIST'


def _get_mshome(pdl):
    '''
       Gets the MS home by sending a request to Kernel.
       
    '''   
    
    debug("Entry _get_mshome")
    
    msHome = None
    inpStr = None
    if pdl["MSHOMECONNECT"][0]:
        host = utils.getData(pdl, KERNELIP)
        port = utils.getDataString(pdl, KERNELPORT)
        iport = utils.getData(pdl, KERNELPORT)

        uri = "/../" + FILE_TO_REQ

        try:
            fileurl = java.net.URL("http", host, int(iport), uri)
            inpStr = fileurl.openStream()
        #except java.net.ConnectException, errmsg:
            #inpStr = None
            #log.logError("Error occurred while connecting to Managing Server %s %s" % (host, port))
            #print rmsg.getmsg("CFG2037E","Could not connect to Managing server - {0} - {1} - {2}", [host, port, errmsg])
        #java.net has tons of exceptions. We cannot catch them by name
        #except Exception: or except BaseException does not catch these
        #only except does it
        except:
            raised = 1
            log.logError("Unknown Exception: %s %s" %(sys.exc_info()[0], sys.exc_info()[1]))
            log.logError("Error occurred while connecting to Managing Server %s %s" % (host, port))
            print rmsg.getmsg("CFG2037E","Could not connect to Managing server - {0} - {1}", [host, port])


        #
        # If host can not be connected, then a prompt is issued
        # to get the MS home
        #
        if inpStr is None or inpStr.available() == 0:
            #if port number is incorrect, it comes here! 
            #so we need to warn the user
            log.logError("Error occurred while connecting to Managing Server %s %s" % (host, port))
            if (raised ==0) :print rmsg.getmsg("CFG2037E","Could not connect to Managing server - {0} - {1}", [host, port])

        else:
            prop = java.util.Properties()
            prop.load(inpStr);
            msHome = prop.getProperty("MS_HOME",None);
            
    debug("Exit _get_mshome - %s" % (msHome))        
        
    return msHome       
         


def config_ms(pdl, soa71_migrate = False):
    '''
       Prompts user for MS host, port if not already provided
    
       @param pdl - PDL reference
       @param soa71_migrate - Ture if soa71 migration processing otherwise False
    
    '''
    
    debug("Entry config_ms")
    
    (only_wlp_selected, wlp_servers) = utils.get_wlp_selected(pdl)
    
    try:
        
        #
        # If this is as the result of migrate call for soa71, then set the pdl
        # without asking question.
        #
        if ((soa71_migrate) and (not pdl["SILENT"][0])):
            utils.setData(pdl, "MSCONNECT", "0", "configms")
        elif only_wlp_selected:
            debug("Only WLP server(s) selected skipping MS config prompt")
            utils.setData(pdl, "MSCONNECT", "0", "configms")
        else: 
		    utils.setData(pdl, "MSCONNECT", "0", "configms")
            # print
            # print "%s" % rmsg.getmsg("FLOWERBOX_MS","Optional integration with ITCAM for Application Diagnostics Managing Server")
            # if len(wlp_servers) > 0:
            #    print "%s" % rmsg.getmsg("CFG1173W", "Managing Server Integration is not supported for WebSphere Liberty Profile")
            # prompt.do_prompt("MSCONNECT", pdl)
            
        msanswer =  utils.getDataString(pdl, "MSCONNECT")   
        #
        # If requested to config MS, then prompt for the kernel
        # host and port if not already provided.
        #
        if msanswer == CONFIGURE_MS:
            prompt.do_prompt(KERNELIP, pdl)
            prompt.do_prompt(KERNELPORT, pdl)
            #
            # Get the MS home by sending a request to Kernel
            #
            msHome = _get_mshome(pdl) 
            if ((msHome is None) and (not pdl["SILENT"][0])):
                prompt.do_prompt("MSHOME", pdl)
                msHome = utils.getData(pdl, "MSHOME")
                log.logDebug("Ms home is: %s" % (msHome))
                utils.setData(pdl, "MSHOME", msHome, "Prompt")
            elif ((msHome is None) and (pdl["SILENT"][0])):
                msHome = utils.getData(pdl, "MSHOME")
                if msHome is None:
                    log.logDebug("Ms home on silent configuration is: %s" % (msHome))
                    messg = rmsg.getmsg("CFG5040E", "MS home is not set in the silent file, and Managing server could not be connected, so exiting Configuration Utility")
                    raise ConfigurationException(messg)
                else :
                    log.logDebug("MS home on silent configuration is: %s" % (msHome))
                    print rmsg.getmsg("CFG5041E","MS cannot be contacted, so we are using the MS Home details provided in the silent file: {0}",[msHome])
            else:
                utils.setData(pdl, "MSHOME", msHome, "Kernel") 
            msh = []
            msh.append(msHome)       
            print "\n%s" % rmsg.getmsg("CFG1049I","MS Home is: {0}", msh)
            #
            # Prompt for Advanced settings including RMI ports and RFS port
            #
            prompt.do_prompt(RMIPORTSET, pdl)
            rmianswer =  utils.getDataString(pdl, "RMIPORTSET")  
            if rmianswer == SET_RMIPORT:
                prompt.do_prompt(PROBERMIPORT, pdl)
                prompt.do_prompt(PROBECONTROLLERRMIPORT, pdl)
                prompt.do_prompt(RFSPORT, pdl)
                
        else:
            log.logDebug("No Managing server is configured")   
        #
        # Make the pdls for the MS as a list for the number of
        # servers in SERVERS pdl.
        #
        kernelip = utils.getData(pdl, KERNELIP)
        kernelport = utils.getDataString(pdl, KERNELPORT)
        rfsport =  utils.getDataString(pdl,RFSPORT) 
        mshome = utils.getData(pdl, MSHOME)
        probermiport = utils.getDataString(pdl, PROBERMIPORT)
        probecntrlrmiport = utils.getDataString(pdl, PROBECONTROLLERRMIPORT)
        utils.setData(pdl, KERNELIP_LIST, [], "configms")
        utils.setData(pdl, KERNELPORT_LIST, [], "configms")
        utils.setData(pdl, RFSPORT01_LIST, [], "configms")
        utils.setData(pdl, MSHOME_LIST, [], "configms")
        utils.setData(pdl, PROBERMIPORT_LIST, [], "configms")
        utils.setData(pdl, PROBECONTROLLERRMIPORT_LIST, [], "configms")
        utils.setData(pdl, MSCONNECT_LIST, [], "configms")
        serverlist = utils.getData(pdl, "SERVERS")
        for serveri in range(len(serverlist)):
            if serveri in wlp_servers:
                utils.getData(pdl, MSCONNECT_LIST).append("0")
                utils.getData(pdl, KERNELIP_LIST).append("None")
                utils.getData(pdl, KERNELPORT_LIST).append("None") 
                utils.getData(pdl, RFSPORT01_LIST).append("None") 
                utils.getData(pdl, MSHOME_LIST).append("None")
                utils.getData(pdl, PROBERMIPORT_LIST).append("None")
                utils.getData(pdl, PROBECONTROLLERRMIPORT_LIST).append("None")
            else:
                utils.getData(pdl, MSCONNECT_LIST).append(msanswer)
                utils.getData(pdl, KERNELIP_LIST).append(kernelip)
                utils.getData(pdl, KERNELPORT_LIST).append(kernelport) 
                utils.getData(pdl, RFSPORT01_LIST).append(rfsport) 
                utils.getData(pdl, MSHOME_LIST).append(mshome)
                utils.getData(pdl, PROBERMIPORT_LIST).append(probermiport)
                utils.getData(pdl, PROBECONTROLLERRMIPORT_LIST).append(probecntrlrmiport)
               
            
    except Exception, msgVar:
        msg = "Error occurred while setting the Managing Server configuration: %s" % msgVar 
        log.logException(msg)
        raise 
    
    debug("Exit config_ms")
    
    
def set_defaultvalues(pdl):
    '''
       Sets the default values in the pdl.  The first selected server's values
       are used to set the default.
       
       @param pdl - PDL reference  
    '''       
    
    debug("Entry set_defaultvalues")
    
    #
    # set default ms values.
    #
    msanswer =  utils.getDataString(pdl, MSCONNECT_LIST, 0)   
    kernelip = utils.getData(pdl, KERNELIP_LIST, 0)
    kernelport = utils.getData(pdl, KERNELPORT_LIST, 0)
    rfsport = utils.getData(pdl,RFSPORT01_LIST, 0 )
    mshome = utils.getData(pdl, MSHOME_LIST, 0)
    probe_rmiport = utils.getData(pdl, PROBERMIPORT_LIST, 0)
    probecontroller_rmiport = utils.getData(pdl, PROBECONTROLLERRMIPORT_LIST, 0)
    utils.setData(pdl, KERNELIP, kernelip, "configms")                
    utils.setData(pdl, KERNELPORT, kernelport, "configms")  
    utils.setData(pdl, RFSPORT , rfsport, "configms") 
    utils.setData(pdl, MSHOME, mshome, "configms") 
    utils.setData(pdl, PROBECONTROLLERRMIPORT, probecontroller_rmiport, "configms") 
    utils.setData(pdl, PROBERMIPORT, probe_rmiport, "configms")   
    utils.setData(pdl, MSCONNECT, msanswer, "configms")   
    
    debug("Exit set_defaultvalues")
