####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved. 
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
   This module sets values for port consolidator script
'''

import sys
import resourcebundle as rmsg
import debugUtil as log
import utils
import fileinput
import platformUtil
import os
import re
import shutil

from debugUtil import logDebug as debug
from ConfigurationException import ConfigurationException


SCRIPT_NAME = "proxyserverctrl_ws"


def config_proxyserverctrl_script(pdl, overwrite=True):
    debug("config_proxyserverctrl_script entry")
    
    rex = re.compile('(@{.*})')
    config_home = pdl["CONFIGHOME"][0]
    am_home = config_home + os.sep + "itcamdc"
    ext = ".sh"
    if platformUtil.isWindows():
        ext = ".bat"
    elif platformUtil.IS_OS400_OS():
        ext = ""
    proxyserverstrcl_script = am_home + os.sep + "bin" + os.sep + SCRIPT_NAME + ext
    proxy_properties = am_home + os.sep + "etc" + os.sep + "proxy.properties"
    
    scripts = []
    scripts.append(proxyserverstrcl_script)
    scripts.append(proxy_properties)
    
    subst_dic, ms_configured = _create_subst_dictionary(pdl)
    if ms_configured:
        for filename in scripts:
            debug("Processing file=%s" % filename)
            if not os.path.exists(filename):
                debug("File %s not found, port consolidator configuration skipped" % filename)
                break
            backup_script = filename + ".template"
            try:
                if os.path.exists(backup_script):
                    if overwrite:
                        debug("script %s was configured previously, restoring from backup %s" % (filename, backup_script))
                        shutil.copyfile(backup_script, filename)
                    else:
                        debug("script %s was configured previously, skipping configuration" % filename)
                else:
                    debug("fresh configuration, backing up a copy of the script")
                    shutil.copyfile(filename, backup_script)
            except Exception, msgVar:
                debug("Error '%s' while accessing file %s, skipping the configuration" % (msgVar, filename))
                break
        
            try:
                if (platformUtil.IS_zOS_OS()):
                    encoded=fileinput.hook_encoded('cp1047')
                else:
                    encoded = None
                for line in fileinput.input(filename, inplace=1, openhook=encoded):
                    tokens = rex.split(line)
                    if len(tokens) == 3:
                        var = tokens[1]
                        debug("var=%s" % var)
                        if var in subst_dic.keys():
                            debug("replacing %s with '%s'" % (var, subst_dic[var]))
                            line = line.replace(var, str(subst_dic[var]))
                    sys.stdout.write(line)
                if filename.endswith(".sh"):
                    debug("changing permission for %s" % filename)
                    try:
                        os.chmod(filename, 0755)
                    except:
                        #there is a bug in python that throws Error 20000 on os.chmod on some versions of Unix
                        utils.runCommand("chmod 0755 %s" % filename)
            except Exception, msgVar:
                debug("error while in config_proxyserverctrl_script %s" % msgVar)
    else:
        debug("No MS connection configured, skipping port consolidator configuration")
    debug("config_proxyserverctrl_script exit")
        
def _create_subst_dictionary(pdl):
    
    debug("_create_subst_dictionary entry")
    ms_host = None
    ms_port = None
    ms_home = None
    was_root = None
    was_version = None
    ms_configured = False
    subst_dic = {}
    try:
        config_home = pdl["CONFIGHOME"][0]
        am_home = config_home + os.sep + "itcamdc"
        toolkit_dir = config_home + os.sep + "toolkit"
        findservers = utils.getData(pdl, "FULLSERVERS")
        for serveri in range(len(findservers)):
            findservers_entry = findservers[serveri]
            ms_host = pdl["KERNELIPLIST"][0][serveri]
            ms_port = pdl["KERNELPORT01LIST"][0][serveri]
            ms_home = pdl["MSHOMELIST"][0][serveri]
            was_root = pdl["WASINSTALLROOT"][0][serveri]
            was_version = pdl["WASVERSION"][0][serveri]
            debug("ms_host=%s ms_port=%s ms_home=%s" % (ms_host, ms_port, ms_home))
            if (ms_host is not None and ms_port is not None and ms_host != "None" and ms_posrt != "None"):
                ms_configured = True
                break
        
        if ms_configured:   
            subst_dic = {
                "@{WAS_HOME}": was_root,
                "@{TOOLKITHOME}": toolkit_dir,
                "@{AM_HOME}": am_home,
                "@{MS_AM_HOME_ORIG}": ms_home,
                #"@{AM_VERSION}": pdl["VERSION"][0],
                "@{AM_VERSION}": "7.1",
                "@{APPSERVER_VERSION}": was_version,
                "@{KERNEL_HOST01}": ms_host,
                "@{PORT_KERNEL_CODEBASE01}": ms_port,
                "@{AM_SOCKET_BINDIP}": pdl["LOCALHOSTIP"][0],
                "@{LOG_COMMON_DIR}": config_home + os.sep + "logs",
                "@{AM_SOCKET_EXPORTIP}": pdl["LOCALHOSTIP"][0]
                }        

    except Exception, msgVar:
        debug("error while in _create_subst_dictionary %s" % msgVar)
    debug("_create_subst_dictionary exit")
    return (subst_dic, ms_configured)
