####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved. 
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
  This module configures SOA DC.
'''

import doprompt as prompt
import utils

from resourcebundle import getmsg as getmsg
from debugUtil import logDebug as debug


def config_soa(pdl, soa71_migrate = False):
    '''
       Prompts the user for SOADC configuration options.
    
       @param pdl - PDL reference
       @param soa71_migrate - Ture if soa71 migration processing otherwise False
    '''
    
    debug("Entry config_soa")
    
    (only_wlp_selected, wlp_servers) = utils.get_wlp_selected(pdl)
    
    #
    # If this is as the result of migrate call for soa71, then set the pdl
    # without asking.
    #
    if (soa71_migrate):
        utils.setData(pdl, "SOADC", "1", "configsoa")
    elif only_wlp_selected:
        debug("Only WLP server(s) selected, skipping SOA config prompt")
        utils.setData(pdl, "SOADC", "0", "configsoa")
    else:
	    utils.setData(pdl, "SOADC", "0", "configsoa")
        # print
        # print "%s" % getmsg("FLOWERBOX_SOA","Optional integration with ITCAM for SOA Agent")
        # if len(wlp_servers) > 0:
        #    print "%s" % getmsg("CFG1175W", "ITCAM for SOA Integration is not supported for WebSphere Liberty Profile")
        # prompt.do_prompt("SOADC", pdl)
    
    #
    # Make the pdls for the SOA as a list for the number of
    # servers in SERVERS pdl.
    #
    soadc = utils.getDataString(pdl, "SOADC")
    utils.setData(pdl, "SOADCLIST", [], "configsoa")
    serverlist = utils.getData(pdl, "SERVERS")
    for serveri in range(len(serverlist)):
        if serveri in wlp_servers:
            utils.getData(pdl, "SOADCLIST").append("0")
        else:
            utils.getData(pdl, "SOADCLIST").append(soadc)

    debug("Exit config_soa")
    
def set_defaultvalues(pdl):
    '''
       Sets the default values in the pdl.  The first selected server's values
       are used to set the default.
       
       @param pdl - PDL reference  
    '''   
    
    debug("Entry set_defaultvalues")    
    
    #
    # set default soa values.
    #
    soa = utils.getData(pdl, "SOADCLIST", 0)
    utils.setData(pdl, "SOADC", soa, "configsoa")
    
    debug("Exit set_defaultvalues")    
        
    
