#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2014 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
   This module prompts the user for TEMA host, port.  If the
   values are already provided by an environment variable or arguments,
   the prompt is not executed.
'''

fixlev = "39182" # Last APAR, defect or story to update this module

import os
import codecs

import configdcless
import debugUtil as log
import doprompt
import platformUtil
import utils

from resourcebundle import getmsg
from debugUtil import logDebug as debug


def config_tema(pdl, soa71_migrate=False):
    '''
       Prompts the user for TEMA information.

       @param pdl - PDL reference
       @param soa71_migrate - True if soa71 migration processing otherwise False
    '''

    debug("Entry config_tema %s" % fixlev)

    # Read any existing connections properties file to get the default JMX admin port
    # and server aliases.  (It is quite possible this file does not yet exist.)

    conndict = configdcless.readConnFile(pdl)

    # Determine whether this is a Liberty server.  If so, we don't do any DC-less configuration.
    # (This will eventually be mooted when Liberty supports DC-less.)

    is_wlp = pdl["WLPINSTALLDIR"][0] is not None
    debug("Liberty status of this profile is %s" % is_wlp)

    is_scmai_install = utils.is_ccm(pdl)
    debug("SCM AI install status of this environment=%s" % is_scmai_install)

    # If this is as the result of migrate call for soa71, then set the pdl
    # tema off without asking any questions.

    if soa71_migrate and not pdl["SILENT"][0]:

        # TEMA's not going to happen for a SOA migration

        if pdl["TEMACONNECT"][0] != False: pdl["TEMACONNECT"][0:2] = False, "configtema"
        if not pdl["DCLESS"][0]: pdl["DCLESS"][0:2] = False, "configtema"

    elif pdl["MIGRATE"][0] and is_scmai_install:

        #if we are doing migrate inside SCM AI  install set TEMA connection to True

        debug("Migrate inside SCM AI  install")
        pdl["TEMACONNECT"][0:2] = 1, "configtema"
        #turn DC less off for both tWAS and Liberty
        if pdl["DCLESS"][0]: pdl["DCLESS"][0:2] = False, "configtema"
        if not pdl["TEMADCAPPSERVER"][0]: pdl["TEMADCAPPSERVER"][0:2] = True, "configtema"
       
    else:

        # Do we want to do TEMA at all?

        print "\n%s" % getmsg("FLOWERBOX_ADTEMA","Optional integration with ITCAM Agent for WebSphere Applications")
        doprompt.do_prompt("TEMACONNECT", pdl)

        if is_wlp:

            # For Liberty DC-less is not available yet.  (Will be eventually)

            if pdl["DCLESS"][0]: pdl["DCLESS"][0:2] = False, "configtema"
            if not pdl["TEMADCAPPSERVER"][0]: pdl["TEMADCAPPSERVER"][0:2] = True, "configtema"

        elif utils.getDataString(pdl, "TEMACONNECT") != "1":

            # Don't prompt and turn off DC-less

            if pdl["DCLESS"][0] != False: pdl["DCLESS"][0:2] = False, "configtema"
            if pdl["TEMADCAPPSERVER"][0] != False: pdl["TEMADCAPPSERVER"][0:2] = False, "configtema"

        else:

            # Turn DCLESS on and prompt for DC-ful configuration.

            if not pdl["DCLESS"][0]: pdl["DCLESS"][0:2] = True, "configtema"
            print "\n%s" % getmsg("FLOWERBOX_DCFUL","Optional integration with DC-ful data collection")
            doprompt.do_prompt("TEMADCAPPSERVER", pdl)

        # If full TEMA is in effect, prompt for APD full.
        if utils.getDataString(pdl, "TEMACONNECT") == "1" and pdl["TEMADCAPPSERVER"][0]:

            # Use DEPRO for the prompt to get a different message.
            pdl["DEPRO"][0:2] = pdl["DE"][0:2]
            doprompt.do_prompt("DEPRO", pdl)
            pdl["DE"][0:2] = pdl["DEPRO"][0:2]
            if utils.getDataString(pdl, "DE") == "1":
                pdl["DE"][0:2] = "1", "configtema"

        # If TEMACONNECT or DCLESSMORE is off, force DE off
        elif utils.getDataString(pdl, "DE") == "1" and pdl["DE"][1] == "configtema":
            pdl["DE"][0:2] = "0", "configtema"

        # Re-populate the DELIST in case this is a respecification
        if pdl["DELIST"][0] is not None:
            de = utils.getDataString(pdl, "DE")
            pdl["DELIST"][0:2] = [de for serveri in xrange(len(pdl["SERVERS"][0]))], "configtema"
            debug("Reconstructed DELIST: %s" % repr(pdl["DELIST"]))

    # The TEMA answer is a string "1" or "0" for historical reasons
    # having to do with Jython 2.1 compatibility.
    temaanswer = utils.getDataString(pdl, "TEMACONNECT")
    debug("temaanswer: %s %s" % (temaanswer, type(temaanswer)))

    # If requested to config TEMA, then prompt for the TEMA
    # host and port if not already provided.
    # (True or "1" will result for prompt and reprompt, respectively.)

    if temaanswer == "1":

        if pdl["MIGRATE"][0] and is_scmai_install:
            #migrate in SCM AI environment, do not prompt, use values from existing configuration

            pdl["TEMAHOST"][0:2] = pdl["TEMAHOSTLIST"][0][0], "configtema"
            pdl["TEMAPORT"][0:2] = pdl["TEMAPORTLIST"][0][0], "configtema"
        else:

            if pdl["TEMADCAPPSERVER"][0]:

                # Prompt for the tema host.
		localhostip = pdl["LOCALHOSTIP"][0] 
		utils.setData(pdl, "TEMAHOST", localhostip, "configtema") 
                doprompt.do_prompt("TEMAHOST", pdl)

                # Prompt for the Tema port.

                doprompt.do_prompt("TEMAPORT", pdl)

            if pdl["DCLESS"][0]:

                # Default the JMX server admin port to whatever was found in the connections.properties file.

                if "jmx.port" in conndict.keys():
                    debug("jmx.port read from connection properties: %s" % conndict["jmx.port"])
                    if pdl["JMXADMINPORT"][1] != "silentInputFileProcessor":
                        pdl["JMXADMINPORT"][0:2] = conndict["jmx.port"], "configtema"
                    else:
                        debug("JMXADMINPORT overridden by silent input file processor to %s" % pdl["JMXADMINPORT"][0])

    else:
        debug("Tema is not configured")
        if pdl["DCLESS"][0]: pdl["DCLESS"][0:2] = False, "configtema"

    # Make the pdls for the tema as a list for the number of
    # servers in SERVERS pdl.

    temahost = utils.getData(pdl, "TEMAHOST")
    temaport = utils.getDataString(pdl, "TEMAPORT")

    fullservers = pdl["FULLSERVERS"][0]
    serverlist = utils.getData(pdl, "SERVERS")
    nservers = len(pdl["SERVERS"][0])

    utils.setData(pdl, "TEMACONNECT", temaanswer, "configtema")
    utils.setData(pdl, "TEMACONNECTLIST", [temaanswer]*nservers, "configtema")
    utils.setData(pdl, "TEMAHOSTLIST", [temahost]*nservers, "configtema")
    utils.setData(pdl, "TEMAPORTLIST", [temaport]*nservers, "configtema")
    if utils.getData(pdl,"ALIASLIST") is None:
        debug("alias list is none")
        utils.setData(pdl, "ALIASLIST", [], "configtema")

    aliasdict = load_aliases(pdl)
    for serveri in xrange(nservers):

        # Prompt for the server alias name.

        servername = utils.getData(pdl,"SERVERS")[serveri]
        nodename = utils.getData(pdl,"WASNODE")[serveri]
        serverkey = "%s/%s" % (nodename,servername)

        if platformUtil.isZ():
            default_serveralias = servername + "_default"
        else:
            default_serveralias = nodename + servername

        # Find the FULLSERVERS entry for this server

        fullserver = None
        for fullserveri in xrange(len(fullservers)):
            if fullservers[fullserveri]["WASSERVER"] == servername:
                fullserver = fullservers[fullserveri]
                break
        assert fullserver is not None, getmsg("CFG5022E", "Internal error in {0}", ["configtema FULLSERVERS"])

        # Look in the connection dictionary for this server.
        # If we find it, use its existing alias name for our default.

        # As in configdcless, do our best to get the true path
        # (To do: Also handle Windows short paths by using java.lang.File.)
        key_wasinstallroot = pdl["WASINSTALLROOT"][0][serveri]
        key_wasinstallroot = os.path.normpath(key_wasinstallroot)
        key_wasinstallroot = os.path.abspath(key_wasinstallroot) or key_wasinstallroot
        key_wasinstallroot = os.path.normcase(key_wasinstallroot)
        key_wasinstallroot = utils.replaceBackSlash(key_wasinstallroot)
        key_wasinstallroot = configdcless.normalizeString(key_wasinstallroot)

        key_wasprofile = configdcless.normalizeString(fullserver["PROFILE"])
        key_cellname = configdcless.normalizeString(fullserver["WASCELL"])
        key_nodename = configdcless.normalizeString(nodename)
        key_servername = configdcless.normalizeString(servername)

        connkey = (key_wasinstallroot, key_wasprofile, key_cellname, key_nodename, key_servername)
        if connkey in conndict.keys():
            debug("Found server %s in existing connections file" % servername)
            connval = conndict[connkey]
            connflds = connval.split("|")
            if len(connflds) >= 12:
                default_serveralias = connflds[11]
                debug("Server alias from connections file is: %s" % default_serveralias)
            else:
                debug("No server alias in connection for server %s" % servername)
        else:
            debug("Did not find server %s in existing connections file" % servername)

        aliaslist = utils.getData(pdl,"ALIASLIST")
        debug("aliaslist=%s" % aliaslist)

        if (len(aliaslist) < serveri+1) or (aliaslist[serveri] is None) or (aliaslist[serveri] == ""):       # if no alias in aliaslist
            debug("setting default alias")
            utils.setData(pdl, "SERVERALIAS", default_serveralias, "configtema") #  set default for prompt
        else:
            debug("setting default from aliaslist")
            utils.setData(pdl, "SERVERALIAS", aliaslist[serveri], "configtema")  #  set default from aliaslist
        oldalias = pdl["SERVERALIAS"][0]
        serveralias = ""
        if temaanswer == "1":

            while True:

                prompt_msg = getmsg("CFG1167I", "Enter the server alias for server {0} in node {1}.", [servername, nodename])
                doprompt.do_prompt("SERVERALIAS", pdl, alt_prompt=prompt_msg, force_alt_prompt=True)

                serveralias = utils.getData(pdl, "SERVERALIAS")

                if serveralias in aliasdict.keys():
                    if len(aliasdict[serveralias]) != 1 or aliasdict[serveralias][0] != serverkey:
                        badalias_msg = getmsg("CFG1188I", "Bad alias {0} specified", [serveralias])
                        print "\n%s" % badalias_msg
                        pdl["SERVERALIAS"][0:2] = oldalias, "configtema"
                        continue

                break

            if serveralias not in aliasdict:
                debug("Adding to alias dictionary: %s %s" % (serveralias, [serverkey]))
                aliasdict[serveralias] = [serverkey]

        # Save the aliases for each monitored AppServer.
        # (Remember aliaslist and pdl["ALIASLIST"][0] are the same thing.)

        if len(aliaslist) < serveri+1:
            aliaslist.append(serveralias)
        else:
            aliaslist[serveri] = serveralias

    debug("Server aliases:\n    %s" % "\n    ".join(["%s : %s" % (pdl["SERVERS"][0][serveri], pdl["ALIASLIST"][0][serveri]) for serveri in xrange(len(pdl["SERVERS"][0]))]))

    # The JMX server port is persisted in the connections.properties file, so it doesn't participate in the serveri madness
    # However, at this time we don't create the connections.properties file for Liberty.  (Someday...)
    # In case of migrate done in SCM AI environment do not prompt for JMX server port, will reuse existing value
    
    if temaanswer == "1" and not is_wlp and not (pdl["MIGRATE"][0] and is_scmai_install):

        # Prompt for JMX server admin port

        print "\n%s" % getmsg("FLOWERBOX_JMXPORT","You are going to have to enter a JMX port now")
        doprompt.do_prompt("JMXADMINPORT", pdl)

    debug("Exit config_tema")


def set_defaultvalues(pdl):
    '''
       Sets the default values in the pdl.  The first selected server's values
       are used to set the default.

       @param pdl - PDL reference
    '''

    debug("Entry set_defaultvalues %s" % fixlev)

    # set default tema values.

    temaanswer = pdl["TEMACONNECTLIST"][0]
    if type(temaanswer) is type([]): temaanswer = temaanswer[0]
    if temaanswer not in ("0", "1"):
        debug("Unexpected value for TEMACONNECTLIST: %s %s" % (repr(temaanswer), type(temaanswer)))
        temaanswer = "0"

    temahost = utils.getData(pdl, "TEMAHOSTLIST", 0)
    temaport = utils.getData(pdl, "TEMAPORTLIST", 0)
    serveralias = utils.getData(pdl, "ALIASLIST", 0)

    utils.setData(pdl, "TEMAHOST", temahost, "configtema")
    utils.setData(pdl, "TEMAPORT", temaport, "configtema")
    utils.setData(pdl, "TEMACONNECT", temaanswer, "configtema")
    utils.setData(pdl, "SERVERALIAS", serveralias, "configtema")

    debug("Exit set_defaultvalues")


def load_aliases(pdl):
    '''
        Load the aliases from the input.properties files and return them as a directory.
    '''

    debug("Entry load_aliases %s" % fixlev)

    aliasdict = {}
    runtimepath = os.path.join(pdl["CONFIGHOME"][0], "runtime")
    debug("Runtime directory: %s" % runtimepath)
    if os.path.exists(runtimepath):
        for filename in os.listdir(runtimepath):
            filename = os.path.join(runtimepath,filename)
            if not filename.endswith(".input.properties"): continue
            if not os.path.isfile(filename): continue
            if not os.path.isfile(filename): continue
            debug("Input properties file: %s" % filename)
            nodename = None
            servername = None
            aliasname = None
            try:
                propfile = codecs.open(filename, "r", platformUtil.getEncoding())
                for ln in propfile.xreadlines():
                    ln = ln.rstrip("\n\r")
                    props = ln.split("=",1)
                    if props[0] == "# WAS_NODE":
                        debug("Node name line: %s" % ln)
                        nodename = props[1]
                    if props[0] == "# WAS_SERVER":
                        debug("Server name line: %s" % ln)
                        servername = props[1]
                    if props[0] == "# SERVERALIAS":
                        debug("Alias name line: %s" % ln)
                        aliasname = props[1]
                propfile.close()
            except IOError, errmsg:
                debug("Problem reading file: %s" % errmsg)
            if nodename is not None and servername is not None and aliasname is not None:
                serverkey = "%s/%s" % (nodename,servername)
                debug("%s is an alias of server %s" % (aliasname, serverkey))
                if aliasname not in aliasdict: aliasdict[aliasname] = []
                aliasdict[aliasname].append(serverkey)
            else:
                debug("Server and alias were not found in file")

    debug("*Existing aliases found:\n    %s" % "\n    ".join("%s %s" % (aliasname, aliasdict[aliasname]) for aliasname in sorted(aliasdict.keys())))


    debug("Exit load_aliases")
    return aliasdict


#############
# Unit Test #
#############

if __name__ == "__main__":

    import sys

    import createPCL_config as createPCL
    import createPDL

    from debugUtil import set_logging
    from resourcebundle import set_resourcebundle

    set_logging("./configtema.log")
    set_resourcebundle()

    pcl = createPCL.make_pcl()
    pdl = createPDL.make_pdl(pcl)

    pdl["CONFIGHOME"][0:2] = os.path.join(os.curdir, "data"), "configtema.__main__"
    if not os.path.isdir(pdl["CONFIGHOME"][0]): os.mkdir(pdl["CONFIGHOME"][0])
    pdl["SERVERS"][0:2] = ["server1"], "configtema.__main__"
    pdl["WASNODE"][0:2] = ["MyNode"], "configtema.__main__"
    pdl["FULLSERVERS"][0:2] = [{"WASSERVER":"server1","WASCELL":"MyCell","PROFILE":"MyProfile"}], "configtema.__main__"
    pdl["WASINSTALLROOT"][0:2] = [r"C:\Utl\IBM\WebSphere\AppServer855nd"], "configtema.__main__"

    config_tema(pdl)

    print "*Debug*  TEMACONFIG = '%s' %s %s" % (pdl["TEMACONNECT"][0], pdl["TEMACONNECT"][1], type(pdl["TEMACONNECT"][0]))
    print "*Debug*  DE = '%s' %s %s" % (pdl["DE"][0], pdl["DE"][1], type(pdl["DE"][0]))

    print "*Debug*  Test of configtema completed"
    sys.exit(0)

    from createPDL import getFormats

    pdl["CONFIGHOME"][0:2] = os.curdir, "configtema.__main__"
    print "CONFIGHOME: %s" % pdl["CONFIGHOME"][0]
    pdl["PROFILEHOME"][0:2] = [r"C:\Utl\IBM\WebSphere\AppServer855test"], "configtema.__main__"
    print "PROFILEHOME: %s" % pdl["PROFILEHOME"][0]
    datahome = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "data"))
    if not os.path.exists(datahome): os.mkdir(datahome)

    print "Encoded test #1: '%s'" % _encodePassword(pdl,"a")
    print "Encoded test #2: '%s'" % _encodePassword(pdl,"password")

    pass
