####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved. 
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
  This module gets information about creating the dynamic cluster template.
'''

import doprompt as prompt
import utils
import platformUtil

from debugUtil import logDebug as debug
from resourcebundle import getmsg as getmsg
from ConfigurationException import ConfigurationException

#
# Constants below are the valid operating system for xd 
#
LINUX_PPC = "linux-ppc"
LINUX_PPC64 = "linux-ppc64"
LINUX_S390 = "linux-s390"
LINUX_S390_64 = "linux-s390-64"
HPUX10 = "hpux10"
HPUX11 = "hpux11"
HPUX_64 = "hpux-64"
HPUX_IA64N = "hpux-ia64n"
HPUX_IA64 = "hpux-ia64"
AIX4_R1 = "aix4-r1"
AIX_64 = "aix-64"
SOLARIS2 = "solaris2"
SOLARIS_64 = "solaris-64"
SOLARIS2_IX86 = "solaris2-ix86"
SOLARIS2_IX86_64 = "solaris2-ix86-64"
LINUX_IX86 = "linux-ix86"
LINUX_IX64 = "linux-ix64"
LINUX_IA64 = "linux-ia64"
OS400 = "OS400"
OS400_64 = "OS400-64"
W32_IX86 = "win32-ix86"
W64_X64 = "win64-x64"
OS390 = "os390"
ZOS = "zos"

#
#The constants below are for toolkit arch name in toolkit native library path: toolkit/lib/<arch>
#
ARCH_AIX4_R1 = "aix533"
ARCH_AIX_64 = "aix536"
ARCH_LINUX_IX86 = "li6263"
ARCH_LINUX_IX64 = "lx8266"
ARCH_LINUX_PPC = "lpp263"
ARCH_LINUX_PPC64 = "lpp266"
ARCH_LINUX_S390 = "ls3263"
ARCH_LINUX_S390_64 = "ls3266"
ARCH_SOLARIS2 = "sol293"
ARCH_SOLARIS_64 = "sol296"
ARCH_SOLARIS2_IX86 = "sol603"
ARCH_SOLARIS2_IX86_64 = "sol606"
ARCH_HPUX10 = "hp11"
ARCH_HPUX_64 = "hp116"
ARCH_HPUX_IA64 = "hpi116"
ARCH_HPUX_IA64N = "hpi113"
ARCH_W32_IX86 = "win32"
ARCH_W64_X64 = "win64"
ARCH_OS400 = "os400-pase"
ARCH_OS400_64 = "os400-pase64"
ARCH_ZOS = "zos"

#
# Valid JDK versions
#
JDK_15 = "15"
JDK_16 = "16"

JDK_VERSIONS = [JDK_15,
                JDK_16
               ]

#
# Constants for the operating system types.  The order should not be changed.
#
OPERATING_SYSTEMS = [LINUX_PPC,
                     LINUX_PPC64,
                     LINUX_S390,
                     LINUX_S390_64,
                     HPUX10,
                     HPUX_64,
                     HPUX_IA64N,
                     HPUX_IA64,
                     AIX4_R1,
                     AIX_64,
                     SOLARIS2,
                     SOLARIS_64,
                     SOLARIS2_IX86,
                     SOLARIS2_IX86_64,
                     LINUX_IX86,
                     LINUX_IX64,
                     W32_IX86,
                     W64_X64,
                     ZOS,
                     
                   ]

#
# Constants for the toolkit directory based on the OS.  The order should not be changed.
#
TOOLKIT_ARCH = [ARCH_LINUX_PPC,
                ARCH_LINUX_PPC64,
                ARCH_LINUX_S390,
                ARCH_LINUX_S390_64,
                ARCH_HPUX10,
                ARCH_HPUX_64,
                ARCH_HPUX_IA64,
                ARCH_HPUX_IA64N,
                ARCH_AIX4_R1,
                ARCH_AIX_64,
                ARCH_SOLARIS2,
                ARCH_SOLARIS_64,
                ARCH_SOLARIS2_IX86,
                ARCH_SOLARIS2_IX86_64,
                ARCH_LINUX_IX86,
                ARCH_LINUX_IX64,
                ARCH_W32_IX86,
                ARCH_W64_X64,
                ARCH_ZOS,
               ]



def _config_os(pdl):
    '''
       Propmts the user with the list of the valid OS for selection.
    '''
    
    debug("_config_os Entry")
    
    prompt.prompt_list_no_pdl(pdl,
                              OPERATING_SYSTEMS,
                              getmsg("CFG1163I", "Specify Operating System type:"))
    os_resp = utils.getData(pdl, "LISTRESP")
    os_int = int(os_resp) - 1
    osname = OPERATING_SYSTEMS[os_int]
    interp = TOOLKIT_ARCH[os_int]
    utils.setData(pdl, "TEMPLATEOS", osname, "configtemplate")
    utils.setData(pdl, "TEMPLATEINTERP", interp, "configtemplate")
    #
    # In case of ZOS, the utility must be running on a ZOS system, because of
    # some required TT configuration.
    #
    if (osname.startswith("zos") and not platformUtil.IS_zOS_OS()):
        debug("Creating XD template creation for ZOS must be executed on a ZOS system, so exiting XD template Utility.")
        messg = getmsg("CFG5053E", "Creating XD template creation for ZOS must be executed on a ZOS system, so exiting XD template Utility.")
        raise ConfigurationException(messg)
        
       
    
    debug("_config_os Exit")
    

def _config_jdk(pdl):
    '''
       Prompts the user with the list of the valid JDKs for selection.
    '''
    
    debug("_config_jdk Entry")
    
    prompt.prompt_list_no_pdl(pdl,
                              JDK_VERSIONS,
                              getmsg("CFG1161I", "Specify JDK version:"))
    jdk_resp = utils.getData(pdl, "LISTRESP")
    jdk_int = int(jdk_resp) - 1
    jdkversion = JDK_VERSIONS[jdk_int]
    utils.setData(pdl, "TEMPLATEJDKVER", jdkversion, "configtemplate")
    
    debug("_config_os Exit")

    
def config_template(pdl):    
    '''
       Prompts the user for template configuration options.
    
       @param pdl - PDL reference
    '''
    
    debug("Entry config_template")
    
    # 
    # Prompt for the DC config home
    #
    prompt.do_prompt("TEMPLATEDCHOME", pdl)
    dchome = utils.getData(pdl, "TEMPLATEDCHOME")
    dchome = dchome.replace("\\", "/")
    utils.setData(pdl, "TEMPLATEDCHOME", dchome, "configtemplate")
    
    #
    # Propmpt for the OS type.
    #
    _config_os(pdl)
    
    #
    # Prompt for JDK version
    #
    _config_jdk(pdl)
    
    #
    # Make the pdls for the DE as a list for the number of
    # servers in SERVERS pdl.
    #
    templateos = utils.getDataString(pdl, "TEMPLATEOS")
    templatejdkver = utils.getDataString(pdl, "TEMPLATEJDKVER")
    templatejdkmode = utils.getDataString(pdl, "TEMPLATEINTERP")
    utils.setData(pdl, "TEMPLATEOSLIST", [], "configtemplate")
    utils.setData(pdl, "TEMPLATEJDKVERLIST", [], "configtemplate")
    utils.setData(pdl, "TEMPLATEINTERPLIST", [], "configtemplate")
    utils.setData(pdl, "ZOSLIBPATH", [], "configtemplate")
    #
    # The following PDLs are created as a dummy pdls as are needed for not breaking the
    # the architecture.
    #
    utils.setData(pdl, "WASCELL", [], "configtemplate")
    utils.setData(pdl, "WASNODE", [], "configtemplate")
    serverlist = utils.getData(pdl, "SERVERS")
    for serveri in range(len(serverlist)):
        utils.getData(pdl, "TEMPLATEOSLIST").append(templateos);
        utils.getData(pdl, "TEMPLATEJDKVERLIST").append(templatejdkver);
        utils.getData(pdl, "TEMPLATEINTERPLIST").append(templatejdkmode);
        utils.getData(pdl, "ZOSLIBPATH").append("");
        utils.getData(pdl, "WASCELL").append("WASCELL_XDDUMMY");
        utils.getData(pdl, "WASNODE").append("WASNODE_XDDUMMY");
        

    debug("Exit config_template")
