#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved. 
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
  This module prompts the user for GC data.
'''

import debugUtil as log
import doprompt as prompt
import utils
import platformUtil
import os

from debugUtil import logDebug as debug
from resourcebundle import getmsg as getmsg

ELIMINATE = '1'
YES = '1'

def config_gc(pdl):
    '''
       Sets the GC for the template xd.
    
       @param pdl - PDL reference
    '''
    
    debug("Entry config_gc")
    
    try:
        platform = utils.getData(pdl, "TEMPLATEOS")        
        if (platform.startswith("zos")):
            
            #utils.setData(pdl, "GC_PARMS", "-verbose:gc ", "configgc")
            #
            # GC_PARMS is a list which cintain the GC parm data, biut for ZOS
            # environment, only -verbose:gc is specified, so to keep the format
            # the same, we need to iterate thru servers and set the list, which 
            # seems all have the same value.
            #
            utils.setData(pdl, "GC_PARMS", [], "configgc")
            for serveri in range(len(pdl["SERVERS"][0])):
                utils.getData(pdl,"GC_PARMS").append("-verbose:gc ")
            
            print
            print "%s" % getmsg("FLOWERBOX_GCZ","JVM Garbage Collection information will be gathered")
            prompt.do_prompt("GCOUTPUT", pdl)
                
            gcanswer =  utils.getDataString(pdl, "GCOUTPUT")   
            
            #
            # Based on the answer set the GCLOGPATH 
            #
            if (gcanswer == ELIMINATE):
                utils.setData(pdl, "GCLOGPATH", "JESLOG:SYSOUT:KEEP", "Prompt")
            else:
                utils.setData(pdl, "GCLOGPATH", "JESLOG:SYSOUT:DISCARD", "Prompt") 
            #
            # Make the pdls for the GCLOGPATH as a list for the number of
            # servers in SERVERS pdl.
            #
            gclogpath = utils.getData(pdl, "GCLOGPATH")
            utils.setData(pdl, "GCLOGPATHLIST", [], "configgc")
            utils.setData(pdl, "GCOUTPUTLIST", [], "configgc")
            utils.setData(pdl, "CURRENTGCLOGLOCATION", [], "configgc")
            serverlist = utils.getData(pdl, "SERVERS")
            for serveri in range(len(serverlist)):
                utils.getData(pdl, "GCLOGPATHLIST").append(gclogpath); 
                utils.getData(pdl, "CURRENTGCLOGLOCATION").append(gclogpath); 
                utils.getData(pdl, "GCOUTPUTLIST").append(gcanswer);     
                
        else:
            
            utils.setData(pdl, "GC_PARMS", [], "configgc")
            utils.setData(pdl, "GCLOGPATHLIST", [], "configgc")
            utils.setData(pdl, "CURRENTGCLOGLOCATION", [], "configgc")
            utils.setData(pdl, "GCOUTPUTDISTLIST", [], "configgc")
            #
            # Iterate thru all the servers and set the GC parms accordingly.
            #    
            for serveri in range(len(pdl["SERVERS"][0])):
                gcparms = "-verbosegc " 
                utils.getData(pdl, "GCLOGPATHLIST").append("None");
                utils.getData(pdl, "CURRENTGCLOGLOCATION").append("None");
                
            debug("Gc parms is - %s" % (gcparms))    
            utils.getData(pdl,"GC_PARMS").append(gcparms) 
                    
        debug("Exit config_gc")   
    
    except Exception, errmsg:
        log.logException("Error occurred while setting the GC - %s" % (errmsg))
        raise 
    
    
def set_defaultvalues(pdl):
    '''
       Sets the default values in the pdl.  The first selected server's values
       are used to set the default.
       
       @param pdl - PDL reference  
    '''   
    
    debug("Entry set_defaultvalues")    
    
    #
    # set default gc values.  
    #
    if (platformUtil.IS_zOS_OS()):
        gcanswer =  utils.getDataString(pdl, "GCOUTPUTLIST", 0) 
        utils.setData(pdl, "GCOUTPUT", gcanswer, "configgc")    
    else:    
        gcanswer =  utils.getDataString(pdl, "GCOUTPUTDISTLIST", 0)
        utils.setData(pdl, "GCOUTPUTDIST", gcanswer, "configgc")
      
    gclogpath = utils.getData(pdl, "GCLOGPATHLIST", 0)
    if (gclogpath is None or gclogpath == "None"):
        gclogpath = utils.getData(pdl, "CURRENTGCLOGLOCATION", 0)
        
    utils.setData(pdl, "GCLOGPATH", gclogpath, "configgc")
    
        
    debug("Exit set_defaultvalues")  
