#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
    Creates and configures XD template.
'''
import sys
import re
import os
import codecs
import shutil

from time import strftime
from java.lang import System
from java.io   import File

# On Windows platforms the python.path property has problems.
# Make an explict addition to the the import path.
sys.path.append("lib")

import platformUtil
import debugUtil as log
import resourcebundle

from constants import constants
from Progress import Progress
from debugUtil import logDebug as debug
##Global variables
VERSIONSTRING = "ITCAM 7.2 AK0000"
SCRIPT = "configtemplatexd_wsadmin"
SCRIPTNAME = SCRIPT + ".py"

common_constants = None

# This is used for properties and variable substitution entries
timestamp = strftime("%Y-%m-%d %H:%M:%S")
tmp = ("%s %s" % (VERSIONSTRING, timestamp))
description_entry = ["description", tmp]

nodeName = ""
serverName = ""
serverid = ""

javaProcServant = ""
javaProcControl = ""

# These are set and checked in getProcessDefs
jvmId = ""
servantJvmId = ""
controlJvmId = ""

# This is set after parameters are validated
runtime_path = ""

variableMap = ""
variableMapArray = {}
variableMapServerid = ""
serverVariableEntries = []
maximumHeapSize = ""

product = ""
control_classpath = ""
genericargs = ""
sharedLib = ""
serverVersion = "61"
pipusc = ""
pipusc_value = ""
server_runtime = ""
itcamdc_home = ""
server_classpath = ""
server_bootclasspath = ""
server_pmiservice_statisticSet = ""
smf = "No"
containertimeouts = ""
doSynchronize = 1
server_amhome_flag = 0
JVMSecurityPolicy_flag = 0
ecam_flag = 0
control_environments = []
control_properties = []

pmiService_flag = 0
extraRMI_flag = 0

quiet_flag = 0
failed_flag = 0

probe = 0
wascell = "None"
wasnode = "None"
wasserver = "None"
current_inputproperties = "None"
summary = {}
config_type = -1
old_am_home = ""
isStandalone = 0
configComplete = 0

customservice = []
systemproperties = []
environments = []
variables = []

###End of global variables####


###Initialize#####
# Catch in case jiti.jar is not provided
try:
    from com.cyanea.installer.zos import SetupUtil
    jiti_flag = 1
except Exception:
    jiti_flag = 0
    print " \n*Warning*  Unable to load SetupUtil for jiti unconfigure\n "


# Get policy cleanup property for unconfigDataCollector only

# The server.policy file is common within a node, so we do not want to
# unconfigure it unless that choice has been approved earlier from
# the unconfig.sh script.

if (SCRIPT == "unconfigDataCollector" ):
    if (System.getProperty("itcam.cleanuppolicy") == "" ):
        serverPolicy_flag = 0
    else:
        serverPolicy_flag = 1
        print "itcam.cleanuppolicy option set"
else:
    serverPolicy_flag=0

# These values are overridden by system property entries

#######End initialize#########

def logDebug(msg):
    print msg

def is_zOS(platform):
    if (platform.startswith("zos")): return 1
    return 0

def to2DList(str):
    '''
        returns result of AdminConfig call as a list of lists
        like in AdminConfig.show(<PMIServiceId> )
    '''
    result = []
    item_list = toList(str)
    logDebug("item_list=%s" % item_list)
    for item in item_list:
        pair = wsadminToList(item)
        logDebug("pair values: %s" % pair)
        result.append(pair)
    return result

def toList(stri):
    if (stri.strip() == ""): return []
    lineSeparator = System.getProperty('line.separator')
    _list = stri.split(lineSeparator)
    return _list

def regexp(pattern, string, flags=0):
        if(re.compile(pattern, flags).search(string)==None): return 0
        else: return 1

def regexpn(pattern, string, flags=0):
        r = re.compile(pattern, flags).subn("", string)
        return r[1]

def join (lst, separator=" "):
    strg = ""
    for element in lst:
        strg += str(element) +separator
    strg = strg[0:len(strg)-len(separator)]
    return strg


def wsadminToList(inStr):
    outList=[]
    if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
        tmpList = inStr[1:-1].split() #splits space-separated lists,
    else:
        tmpList = inStr.split("\n")   #splits for Windows or Linux
    for item in tmpList:
        item = item.rstrip();         #removes any Windows "\r"
        if (len(item)>0):
            outList.append(item)
    return outList

def split_path(path_str):
    '''
        splits path by unix or windows style separator
        replacement for split(";:") in jacl

    '''
    rex = re.compile("[:]")
    if platformUtil.isWindows():
        rex = re.compile("[;]")
    return rex.split(path_str)


def isUnderAmHome(path, am_home):
    '''
        check if a path is under a directory
    '''
    logDebug("isUnderAmHome entry %s %s" % (path, am_home))
    isChildDir = 0
    
    if am_home != None and am_home != "":
        try:
            dc_home = os.path.dirname(am_home)
            canonical_path = File(path).getCanonicalPath()
            canonical_dc_home = File(dc_home).getCanonicalPath()
            logDebug("checking %s is under %s" % (canonical_path, canonical_dc_home))
            if canonical_path.startswith(canonical_dc_home): isChildDir = 1
        except Exception, errmsg:
            logDebug("Error in isUnderAmHome %s" % errmsg)
            
        logDebug("isUnderAmHome exit %s" % isChildDir)
    return isChildDir

errorCode = ""
errorInfo = ""

##############################################################
#
#  Procedure  checkObsolete
#
#  Check for obsolete path values, that is ones that contain
#
#   - /itcam/WebSphere/DC/
#   - /itcamdc/lib/wpsaspect.jar
#   - ${ITCAM61HOME}
#
##############################################################

def checkObsolete ( path ):

        global old_am_home
        
        obsolete_flag = 0
        if isUnderAmHome(path, old_am_home) == 1:
            logDebug("marking '%s' as obsolete, path is under old am.home " % path)
            obsolete_flag = 1
        elif (path.find("/itcam/WebSphere/DC/") != -1):
            obsolete_flag = 1
        elif (path.find("/itcamdc/lib/wpsaspect.jar") != -1):
            obsolete_flag = 1
        elif (path.find("${ITCAM61HOME}") != -1):
            obsolete_flag = 1
                
        logDebug("checkObsolete exit = %s" % str(obsolete_flag))
        return obsolete_flag


def attrAsList(stri):
    rex=re.compile("\[(.*)\]")
    attrValue = rex.split(stri)[1]
    return attrValue.split()


def getSubstitutionEntryValue(var_name):
    '''
        get variable value
    '''
    global AdminConfig
    global serverid
    logDebug("getSubstitutionEntryValue entry %s" % var_name)
    varValue = None
    try:
        variables = toList(AdminConfig.list("VariableSubstitutionEntry", serverid))
        for variable in variables:
            getVarName = AdminConfig.showAttribute(variable, "symbolicName")
            logDebug("varName %s" % (getVarName))
            if getVarName == var_name:
                varValue = AdminConfig.showAttribute(variable, "value" )
    except:
        logDebug("Error in getSubstitutionEntryValue")
    logDebug("varName %s is set to %s" % (getVarName, varValue))
    return varValue

def get_am_home(propId):
    global AdminConfig
    
    logDebug("get_am_home entry")
    prop_value = AdminConfig.showAttribute(propId, "value" )
    if prop_value != None and prop_value.find("${ITCAMDCHOME}") > -1:
        itcamdchome = getSubstitutionEntryValue("ITCAMDCHOME")
        if itcamdchome != None:
            logDebug("replacing value with %s" % itcamdchome) 
            prop_value = prop_value.replace("${ITCAMDCHOME}", itcamdchome)
    logDebug("get_am_home exit %s" % prop_value)    
    return prop_value

def getJVMMode():
    global AdminTask

    logDebug(" getJVMMode()")    
    logDebug("nodeName="+nodeName+", serverName="+serverName)
    
    serverSDK = AdminTask.getServerSDK('-nodeName ' + nodeName + ' -serverName ' + serverName)
    serverBitness = serverSDK[-5:-3]
     
    logDebug("serverBitness=" + serverBitness)
    logDebug(" getJVMMode() exit")    

    return serverBitness


####
# restore input.properties file from a backup if configuration failed
# remove backup file if configuration was successful
#
####
def restore_input_properties():
    global current_inputproperties
    global config_type
    global configComplete
    
    logDebug("restore_input_properties entry configComplete=%s configType=%s current_inputproperties=%s" % (str(configComplete), str(config_type), current_inputproperties))
    try:
        if (current_inputproperties != "None" and
           (config_type == str(common_constants.CONFIG) or config_type == str(common_constants.MIGRATE) or config_type == str(common_constants.RECONFIG))):
            current_inputproperties_org = current_inputproperties + "_org"
            if configComplete == 0:  #dataCollector wasn't configured, no AdminConfig.save() executed, restoring to saved input.properties
                if os.path.exists(current_inputproperties_org):
                    shutil.copy(current_inputproperties_org, current_inputproperties)
                    os.remove(current_inputproperties_org)
                    log.logDebug("restored original input.properties file %s" % (current_inputproperties_org))
                else:
                    os.remove(current_inputproperties)
                    log.logDebug("input.properties file was removed - %s" % (current_inputproperties))
            else:  #configuration complete, failure  after AdminConfig.save()
                if os.path.exists(current_inputproperties_org):
                    os.remove(current_inputproperties_org)
                    log.logDebug("AdminConfig.save() executed, removing stored input.properties file %s" % current_inputproperties_org)
                else:
                    log.logDebug("no backup file found %s" % current_inputproperties_org)
    except Exception, remerrmsg:
        print "input.properties file could not be restores - %s - %s" % (current_inputproperties, remerrmsg)
    logDebug("restore_input_properties exit")


def configEnvs ( javaProcId, updateEnvironments, platform):
    '''
       Configure the environment variables
       
       @param javaProcId - Process id
       @param updateEnvironments - Updated environments
       @param platform - Platform type
    '''   
    global AdminConfig
    global nodeName, serverName, serverid
    global description_entry, old_am_home


    #if (platformUtil.IS_OS400_OS() and config_type == str(common_constants.RECONFIG)):
    #    logDebug("nodeName="+nodeName+", serverName="+serverName)
    #    logDebug("calling getJVMMode()")
    #    bitness = getJVMMode()
        
    logDebug("configEnvs %s %s " %(javaProcId,updateEnvironments))
    print "Create/Modify java process with %s environment variables" % str(updateEnvironments)

    logDebug("%s updating environment entries " % str(len(updateEnvironments)))
    for ue in updateEnvironments:
        logDebug("item=%s" % ue)
    # Read in the environment from the Java process

    logDebug("Reading in name/value pairs from existing environment")
    '''
    '[PATH(cells/x336r1s39-vm02Node01Cell/nodes/x336r1s39-vm02Node02/servers/server1
    |server.xml#Property_1319649805390) _JVM_THREAD_DUMP_BUFFER_SIZE(cells/x336r1s39
    -vm02Node01Cell/nodes/x336r1s39-vm02Node02/servers/server1|server.xml#Property_1
    319649805562) NLSPATH(cells/x336r1s39-vm02Node01Cell/nodes/x336r1s39-vm02Node02/
                          servers/server1|server.xml#Property_1324073901586)]'
    '''
    environs = toList(AdminConfig.showAttribute(javaProcId, "environment" ))
    logDebug("environs %s" % environs)
    if (len(environs) != 1):
        print " \n*Warning*  Unexpected %s environment entries for %s should be 1.\n " % (str(len(environs)), serverName)
        print "           Process will continue.  However, there may be a WAS configuration issue.\n "


    env_list = attrAsList(environs[0])
    environs = env_list
    env_dic = {}
    for envid in environs:
        logDebug("envid=%s" % envid)
        logDebug( "%s environment IDs read for %s" % (str(len(env_list)), serverName))
        if (envid.find('(') > -1 and envid.find(')') > -1):
            var_name = envid.split('(')[0]
            var_value = envid[len(var_name):]
            env_dic[var_name] = var_value
        envid.split('(')

    # Create the arrays from the update entries

    updates = {}
    hits = {}
    for ue in updateEnvironments:
        nm = ue[0][1]
        val = ue[1][1]
        hits[nm] = 0
        updates[nm] = val
        logDebug("%s=%s" % (nm, val))




    # Process the environment variables
    # Special handling for paths such as NLSPATH and LIBPATH

    logDebug("  Processing environment entries:")
    
    logDebug("am.home value is %s" %  old_am_home)
    
    for envid in environs:
        has_property = envid.find("#Property_")
        if (has_property == -1): 
            continue
        enm = AdminConfig.showAttribute(envid, "name" )
        logDebug("Environment name = %s" % enm)
        if updates.get(enm, None) == None:
            continue
        hits[enm] = 1
        eval = AdminConfig.showAttribute(envid, "value" )
        logDebug("Environment value = %s" % eval)
        val = updates[enm]
        logDebug("Update value      = %s" % val)

        #if (enm == "LIBPATH" and platformUtil.IS_OS400_OS() and config_type == str(common_constants.RECONFIG)):
        #    if bitness == "32":
        #        val = "/lib:${ITCAMDCHOME}/toolkit/lib/os400-pase"
        #    else:
        #        val = "/lib:${ITCAMDCHOME}/toolkit/lib/os400-pase64"
                
        #    logDebug("Update value (overriden) = %s" % val)

        # Must merge the path updates
        if (enm == "NLSPATH" or enm == "LIBPATH" or enm == "LD_LIBRARY_PATH" or enm == "PATH" or enm == "SHLIB_PATH"):
            logDebug("Merging %s  with existing path list" % enm)
            paths = split_path(val)
            epaths = split_path(eval)
            newPaths = paths
            for epath in epaths:
                # Drop any existing path that is in the new paths
                for path in paths:
                    if (path == epath):
                        logDebug("%s present in existing path " % (epath))
                        epath = ""
                        break


                if (epath == ""):
                    continue
                # Drop any ITCAMDCHOME path, it is assumed to be ours
                if (epath.find("${ITCAMDCHOME}") != -1):
                    logDebug("Existing %s is an old ITCAM entry and will be dropped" % epath)
                    continue

                # Drop any paths from a previous release
                if (checkObsolete(epath ) ):
                    print " \n*Warning*  Obsolete "+enm+" entry removed: "+epath+"\n "
                    print "           If this entry was not originally set by ITCAM,"
                    print "           you may wish to manually replace it.\n "
                    continue


                logDebug("Existing %s entry will be retained in %s " % (epath, enm))
                newPaths.append(epath)

            pathDelimiter = ":"
            if (platform.startswith("win")):
                pathDelimiter = ";"
            newPaths = join( newPaths, pathDelimiter)
            newVal = newPaths

            # Update existing entry with merged paths
            # (We presume that the new value will never be empty)

            if (newVal == eval):
                print "Environment variable already set: "+enm+" = "+eval
            else:
                print "Updating environment variable "+nm+" to merged path: "+newVal
                newEnv = [["name", enm], ["value", newVal], description_entry]
                AdminConfig.modify(envid, newEnv )

            continue


        if (val == eval):
            print "Environment variable already set: "+enm+" = "+eval
        else:
            print "Updating environment variable "+nm+" from \""+eval+"\" to \""+val+"\""
            newEnv = [["name", enm], ["value", val], description_entry]
            AdminConfig.modify(envid, newEnv )

    logDebug(hits)
    for nm in updates.keys():
        if (hits[nm] > 0):
            logDebug("Value %s already processed, skipping..." % nm)
            continue
        logDebug("Found new name %s" % nm)
        val = updates[nm]
        print "Creating new environment "+nm+"="+val
        newEnv = [["name", nm], ["value", val], description_entry]
        AdminConfig.create("Property", javaProcId, newEnv, "environment" )


    logDebug(" configEnvs() exit")



def configCustomService ( newCustomService ):
        '''
            config
            Procedure configCustomService
            Arguments: server config id,
            newCustomService
        '''

        global AdminConfig
        global serverid


        logDebug("configCustomService() entry")
        for attr in newCustomService:
            logDebug("attr=%s" % attr)

        for attr in newCustomService:
            attrName = attr[0]
            attrValue = attr[1]
            if ( attrName == "classname" ):
                newClassName = attrValue
            elif ( attrName == "enable" ):
                newEnable = attrValue
            elif ( attrName == "classpath" ):
                newClassPath = attrValue
            elif ( attrName == "displayName" ):
                newServiceName = attrValue
            elif ( attrName == "description" ):
                newDesctiption = attrValue
            elif ( attrName == "default" ):
                logDebug("ignore "+attrName+"="+attrValue)



        print "Configure custom service "+newServiceName
        customServiceIds = AdminConfig.list("CustomService", serverid )
        if (customServiceIds != ""):
            customServiceIds_list = toList(customServiceIds)
            print "Custom Service "+customServiceIds+" exists"
            for tmpId in customServiceIds_list:
                oldClassName = AdminConfig.showAttribute(tmpId, "classname" )
                oldServiceName = AdminConfig.showAttribute(tmpId, "displayName" )
                oldEnable = AdminConfig.showAttribute(tmpId, "enable" )
                oldClassPath = AdminConfig.showAttribute(tmpId, "classpath" )
                if (oldClassName == newClassName):
                    print "Removing existing "+newServiceName+" custom service"
                    AdminConfig.remove(tmpId )
                    break
        AdminConfig.create("CustomService", serverid, newCustomService )
        logDebug("configCustomService() exit")



def unconfigSharedLib ( libraries, nodeName ):
    '''
        unconfig
        Procedure  unconfigSharedLib
        Arguments: libraries

        Note: This unconfigures the ITCAM/DC WPS shared library
            for the whole node.  This should be okay because
            there is only one WebSphere_Portal per node.
    '''

    logDebug("unconfigSharedLib(%s) entry" % libraries )
    libId = AdminConfig.getid("/Node:"+nodeName+"/Library:WPSlib/" )
    if (libId== ""):
        print "WPS not installed.  Skipping WPSlib classPath unconfiguration"
        logDebug("unconfigSharedLib() exit = false")
        return "false"
    logDebug("WPSlib ID = %s" % libId)
    nodeId = AdminConfig.getid("/Node:"+nodeName+"/" )
    logDebug("Node ID = %s" % nodeId)
    # Attributes for the new WPSlib Library object consist
    # of the old objects attributes with the classPath replaced

    currentPaths = AdminConfig.showAttribute(libId, "classPath" )
    newPaths = unmergePaths(currentPaths, libraries )
    classpathDelimiter = ":"
    if platformUtil.isWindows():
        classpathDelimiter = ";"
    if (newPaths == join( split_path(currentPaths), classpathDelimiter)  ):
        print "WPS shared library "+libraries+" already unconfigured"
    else:
        print "Removing ITCAM from WPSlib classPath"
        logDebug("Constructing new WPSlib attributes")
        attrs = to2DList(AdminConfig.show(libId ))
        newWPSlib = []
        for attr in attrs:
            if (attr[0] == "description"):
                logDebug("Skipping description attribute %s" % attr)
                continue
            if (attr[0] == "classPath"):
                # classPath normally uses semicolons for separator character
                newWPSlib.append(["classPath", join( newPaths.split(":" ), ";")  ])
                continue
            logDebug("Using existing attribute = %s, value = %s" % (attr[0], attr[1]))
            newWPSlib.append(attr)
        AdminConfig.remove(libId )
        libId = AdminConfig.create("Library", nodeId, newWPSlib )
        logDebug("New WPSlib ID = %s" % libId)

    logDebug("unconfigSharedLib() exit = true")
    return "true"


# config
##############################################################
#
#  Procedure  configSharedLib
#  Argument:  libraries
#
#  Because the classPath attribute is merged instead of replaced
#  by modify, we have to recreate the WPSlib object to change the
#  classPath.  This would not be necessary if we had unsetAttributes,
#  but that is available only in WAS 7.0 and later.
#
#  This is specifically for configuring the class path of an existing
#  WPSlib Library object.
#
#  Shared libararies are configured at the node, so ${ITCAMDCHOME} can
#  not be used because it is configured at the node.
#
#  Admin Console: Environment/Shared Libraries/WPSlib
#
##############################################################

def configSharedLib ( libraries ):
        global AdminConfig
        global nodeName
        global description_entry

        logDebug("  configSharedLib(%s) entry for node '%s'" % (libraries, nodeName))

        libId = AdminConfig.getid("/Node:"+nodeName+"/Library:WPSlib/" ).strip()
        if (libId == "" or libId == None):
            print "WPS not installed.  Skipping WPSlib classPath configuration"
            logDebug("configSharedLib() exit = false")
            return "false"

        logDebug(" WPSlib ID = %s" % libId)

        nodeId = AdminConfig.getid("/Node:"+nodeName+"/" )

        logDebug("Node ID = %s" % nodeId)
        # Attributes for the new WPSlib Library object consist
        # of the old objects attributes with the classPath replaced

        currentPaths = AdminConfig.showAttribute(libId, "classPath" )
        newPaths = mergePathsAfter(currentPaths, libraries )
        classpathDelimiter = ":"
        if platformUtil.isWindows():
            classpathDelimiter = ";"
        if (newPaths == join(split_path(currentPaths), classpathDelimiter) ):
            print "WPS shared library "+libraries+" already configured"
        else:
            print "Adding ITCAM to WPSlib classPath"
            logDebug("Constructing new WPSlib attributes")
            attrs = to2DList(AdminConfig.show(libId ))
            newWPSlib = []
            newWPSlib.append(description_entry)
            for attr in attrs:
                print "attr=%s" % attr
                if (attr[0] == "description"):
                    logDebug("Skipping description attribute = %s" % attr[1])
                    continue
                if (attr[0] == "classPath"):
                    logDebug("Existing classPath value    =\n  %s  " % join(split_path(attr[1]), "\n"))
                    logDebug("Replacement classPath value =\n  %s  " % join (split_path(newPaths), "\n" ))
                    # classPath normally uses semicolons for separator character
                    newWPSlib.append(["classPath", join( newPaths.split(":" ), ";")  ])
                    continue
                logDebug("Using existing attribute = %s value = %s" % (attr[0], attr[1]))
                newWPSlib.append(attr)

            AdminConfig.remove(libId )
            libId = AdminConfig.create("Library", nodeId, newWPSlib )
            logDebug("New WPSlib ID = %s" % libId)


        logDebug("configSharedLib() exit")
        return "true"


def configSmf(enable):
    '''
       Config SMF
       
       @param enable - True or false
    '''   
       
    global AdminConfig
    global serverid
    global javaProcControl

    logDebug("configSmf("+enable+") entry")

    environments = wsadminToList(AdminConfig.list("Property", javaProcControl ))

    # If we are unconfiguring, remove the SMF container and server activity
    # records because they may have been configured by a back-level
    # version of configDataCollector.

    # Note: Consider removing after 21-Oct-2009

    if (enable == "false"):
        setControlEnv(environments, "server_SMF_server_activity_enabled", enable )
        setControlEnv(environments, "server_SMF_container_activity_enabled", enable )


    setControlEnv(environments, "server_SMF_server_interval_enabled", enable )
    setControlEnv(environments, "server_SMF_container_interval_enabled", enable )

    setControlEnv(environments, "server_SMF_interval_length", 120 )

    logDebug("  configSmf() exit")



# unconfig
##############################################################
#
#  Procedure  deleteJvmArgs
#  Arguments: target, configured, current
#
#  Any JVM option not found in the current options which is
#  found in both the target and configured options is inferred
#  to have been deleted.  These are removed from the target
#  and returned to the caller.
#
#  Note that debug tracing is commented out in this routine
#  because the voluminous output obscures the flow of code
#  more likely to fail.  This low-level routine was debugged
#  and tested outside the wsadmin environment.
#
##############################################################

def deleteJvmArgs ( target_opts, config_opts, exist_opts ):

        # if { $debug_flag } { puts " \n[clock format [clock seconds] -format %T] *DEBUG*  unmergeJvmArgs() entry" }
        # if { $debug_flag } { puts   "[clock format [clock seconds] -format %T] *DEBUG*    target:  \"$target_opts\"" }
        # if { $debug_flag } { puts   "[clock format [clock seconds] -format %T] *DEBUG*    config:  \"$config_opts\"" }
        # if { $debug_flag } { puts   "[clock format [clock seconds] -format %T] *DEBUG*    current: \"$exist_opts\"\n " }

        #target_opts = join( target_opts, " ")
        #target_opts = target_opts.split( )
        #config_opts = join( config_opts, " ")
        #config_opts = config_opts.split( )
        #exist_opts = join( exist_opts, " ")
        #exist_opts = exist_opts.split( )

        merged = []
        for item in target_opts:
            # First check if the item was copied to the configured args
            dup_flag = 0
            if (item == ""): continue
            # if { $debug_flag } { puts "[clock format [clock seconds] -format %T] *DEBUG*    target_opts item: \"$item\"" }
            for configitem in config_opts:
                if (item == configitem):
                    dup_flag = 1
                    break

            # If it is in both, see if it has been removed from the current args
            if (dup_flag):
                keep_flag = 0
                for existitem in exist_opts:
                    if (item == existitem):
                        keep_flag = 1
                        break

            else:
                keep_flag = 1

            # Accumlate all non-deleted items
            if (keep_flag):
                merged.append( item )
            else:
                logDebug("deleted item = %s" % item)




        # if { $debug_flag } { puts "[clock format [clock seconds] -format %T] *DEBUG*    delete result:"; displayJvmArgs "" $merged }
        # if { $debug_flag } { puts "[clock format [clock seconds] -format %T] *DEBUG*  deleteJvmArgs() exit\n " }
        merged = join( merged, " ")
        return merged


# config
##############################################################
#
#  Procedure  doSynchronize
#  Arguments: none
#
#  Returns    none
##############################################################

def __doSynchronize():

        global AdminConfig, AdminControl
        global nodeName, serverName, serverid
        global common_constants

        stat = 0

        #
        # Synchronize with this node if Network Deployment
        # ND has a DeploymentManager object, Base does not
        # There will only be one cell returned.
        # Cell type will be STANDALONE or DISTRIBUTED.
        #
        logDebug("__doSynchronize entry")
        try:
            cell = AdminConfig.getid("/Cell:/") 
            celltype = AdminConfig.showAttribute(cell, "cellType" )
            logDebug("celltype=%s" % celltype)
            if (celltype == "DISTRIBUTED"):
                logDebug("Synchronizing domain manager")
                print "\n%s %s" % (common_constants.SYNC_STARTED, nodeName)
                #print "\nSynchronizing with node %s" % nodeName
                sync = AdminControl.completeObjectName("type=NodeSync,node="+nodeName+",*" )
                if (sync == "" or sync == None):
                    logDebug("Unable to locate NodeSync for node %s. Manual Synchronization is required after the configuration " % nodeName)
                    stat = 1
                    msg = resourcebundle.getmsg("CFG1150W", "Configuration completed, but synchronization of node {0} with the Deployment Manager failed. Manual synchronization is required.", [nodeName])
                    progress_obj.write_progress(msg) 
                else:
                    rc = AdminControl.invoke(sync, "sync" )
                    if (rc == "true"):
                        print "%s on %s" % (common_constants.SYNC_COMPLETED, nodeName)
                    else:
                        stat = 1
                        logDebug("*Error*  Failure synchronizing on node %s, result = %s" % (nodeName, str(rc)))
                        msg = resourcebundle.getmsg("CFG1150W", "Configuration completed, but synchronization of node {0} with the Deployment Manager failed. Manual synchronization is required.", [nodeName])
                        progress_obj.write_progress(msg) 
            else:
                logDebug("No DMGR, synchronization not necessary.")
        except Exception, msg:
            logDebug("Error occurred while synchronizing: %s" % msg )
            stat = 1
            msg = resourcebundle.getmsg("CFG1150W", "Configuration completed, but synchronization of node {0} with the Deployment Manager failed. Manual synchronization is required.", [nodeName])
            progress_obj.write_progress(msg)
            
        logDebug("__doSynchronize exit %s" % str(stat))
        return stat

def hasNodeagent():
    
    global AdminConfig, AdminControl
    global nodeName, doSynchronize, isStandalone
        
    result = 0
    
    logDebug("hasNodeagent entry")
    try:
        cell = AdminConfig.getid("/Cell:/" )
        celltype = AdminConfig.showAttribute(cell, "cellType" )
        logDebug("celltype=%s" % celltype) 
        if (celltype == "DISTRIBUTED"):
            logDebug("checking if nodeagent is up")
            print "\n%s %s" % (common_constants.SYNC_STARTED, nodeName)
            sync = AdminControl.completeObjectName("type=NodeSync,node="+nodeName+",*" )
            logDebug("nodeName=%s sync=%s" % (nodeName, str(sync)))
            if (sync == "" or sync == None):
                doSynchronize = 0
                result = 1
                logDebug("Nodeagent for node %s is not running " % nodeName)
            else:
                doSynchronize = 1
        else:
            logDebug("Standalone node, we are OK to proceed") 
            doSynchronize = 0
            isStandalone = 1
    except Exception, msg:
        logDebug("Error in hasNodeAgent: %s" % msg )
   
    logDebug("hasNodeagent, exit %s" % str(result))    
    return result


# both
##############################################################
#
#  Procedure  diffJvmArgs
#  Arguments:
#
#  Find the difference between two strings.  That is, remove every word
#  found in the second string from the first string.
#
#  Note that debug tracing is commented out in this routine
#  because the voluminous output obscures the flow of code
#  more likely to fail.  This low-level routine was debugged
#  and tested outside the wsadmin environment.
#
##############################################################

def diffJvmArgs ( exist_opts, minus_opts ):

        logDebug("diffJvmArgs() entry")
        logDebug(" current: %s" % exist_opts)
        logDebug("minus: %s" % minus_opts)

        #exist_opts = join( exist_opts, " ")
        #exist_opts = exist_opts.split( )
        #minus_opts = join( minus_opts, " ")
        #minus_opts = minus_opts.split( )

        extras = []
        for item in exist_opts:
            for olditem in minus_opts:
                if (item == olditem):
                    item = ""
                    break
            if (item != ""): extras.append(item)

        diff = " ".join(extras)

        logDebug(" diffJvmArgs() result = %s" % diff)
        return diff



# config
##############################################################
#
#  Procedure  getArg
#  Arguments: arg
#             argList
#  Returns argument from list if present, "" if not
#
##############################################################
def getArg ( arg, argList ):

    logDebug("arg=%s" % arg)
    retArg = ""
    for argument in argList:
        if (argument.find(arg) == 0):
            retArg = argument
            break
    logDebug("input arg=%s return arg=%s" % (arg, retArg))
    return retArg








# config
##############################################################
#
#  Procedure  removeJiti
#  Arguments: bootClasspath
#
#  Returns:   The modified bootClasspath
#
#
##############################################################

def removeJiti ( bootClasspath ):

        boots = bootClasspath.split(":" )
        newboots = []
        for boot in boots:
            if ("boot".find("jiti.jar") >= 0):
                    continue
            else:
                newboots.append(boot)

        retVal = join(newboots, ':')
        return retVal


# both
#############################################################
#
#  Procedure unconfigInjector
#  Unconfigure the JITI Primary Injector File, which is obsolete
#
#############################################################
def unconfigInjector (  ):
        global AdminConfig
        global server_runtime
        global quiet_flag
        global jvmId
        global jiti_flag


        logDebug("unconfigInjector() entry")

        PRIMARY_INJECTOR_FILE = "com.ibm.tivoli.jiti.injector.ProbeInjectorManagerChain.primaryInjectorFile"

        # Unlike other property scans, this will be quick, since there will probably be at most
        # one property for the Servant JVM ID.  So there is no point in keeping a cache of properties here.

        objectId = ""
        value = ""
        logDebug(" Looking for "+PRIMARY_INJECTOR_FILE+" property.")

        properties = toList(AdminConfig.list("Property", jvmId ))
        logDebug(" %d property objects found." % len(properties))
        for property in properties:
            logDebug(" Object ID = "+property)
            if ( property.find("#Property_") == -1 ): continue
            # Catch any errors in showAttribute
            try:
                name = AdminConfig.showAttribute(property, "name" )
            except:
                _type_, _value_, _tbck_ = sys.exc_info()
                print " \n*Warning*  showAttribute failed for property ID "+property
                print "           " + str(_value_) +"\n "
                continue
            logDebug("Name = "+name)
            if (name == PRIMARY_INJECTOR_FILE):
                value = AdminConfig.showAttribute(property, "value" )
                print "System property "+name+" exists,"
                print "  value = \""+value+"\""
                objectId = property
                break



        # Have the unconfigInjector remove the name from the properties file,
        # then it returns the name to revert to.  We either update the
        # property or remove it, depending upon whether a reversion name was returned.

        # However, if JITI is not available the best we can do is remove the property

        if (objectId != ""):
            # The catch is in case the jiti jar has not been provided to wsadmin
            # It should not be necessary because we already checked in the prolog
            if (jiti_flag):
                try:
                    setup_util = SetupUtil(  ) #?PROBLEM? (jacl 61) JAVA::NEW SetupUtil might require:  from ?? import SetupUtil
                    # Note: Java null is returned as empty string, not [java::null]
                    injectorPropertyValue = setup_util.unconfigInjector(value )
                    print "injectorPropertyValue being set to \""+injectorPropertyValue+"\""
                except:
                    _type_, _value_, _tbck_ = sys.exc_info()
                    ermsg = str(_value_)
                    # Note ermsg is included in $::errorInfo, so unneeded if the latter is dumped
                    print " *Warning*  Unconfig injector failure " + ermsg
                    print " processing continues\n "
                    injectorPropertyValue = ""
            else:
                logDebug("Removing obsolete injector property")
                injectorPropertyValue = ""
            if (injectorPropertyValue == ""):
                logDebug("Reversion injector file name is null")
                AdminConfig.remove(objectId )
                logDebug("Injector property "+PRIMARY_INJECTOR_FILE+" has been removed")
            else:
                logDebug("Reversion injector file name is \""+injectorPropertyValue+"\"")
                injectorProperty = [["name", PRIMARY_INJECTOR_FILE], ["value", injectorPropertyValue]]
                AdminConfig.modify(objectId, injectorProperty )
                logDebug("Injector property "+PRIMARY_INJECTOR_FILE+" has been reverted")

        else:
            logDebug("No injector property "+PRIMARY_INJECTOR_FILE+" found")
            if ( not quiet_flag):
                print "Injector property "+PRIMARY_INJECTOR_FILE+" does not exist"


        logDebug("  unconfigInjector() exit")




def subsNodeVariable (varstring, nodename):
    '''
        Procedure subsNodeVariable
        Arguments: variable string, node name
        This does recursive replacement of node variables.
    '''
    global variableEntries, variableEntriesMapNodename, variableEntriesMap
    global AdminConfig

    variableEntriesMapNodename = ""
    logDebug("subsNodeVariable(%s %s) "  % (varstring, nodename))
    # Check if we need to get a new variable map
    logDebug("variableEntries not set")
    variableEntries = []
    variableEntriesMapNodename = nodename
    variableEntriesMap={}

    if (nodename != variableEntriesMapNodename):
        logDebug("cached node was " + str(variableEntriesMapNodename))
        variableEntries = ""


    # Build a variable map array
    if (len(variableEntries) == 0):
        logDebug("need to build variableEntriesMap")
        variableEntries = toList(AdminConfig.getid("/Node:"+nodename+"/VariableMap:/VariableSubstitutionEntry:/" ))
        for entry in variableEntries:
            # Skip if not VariableSubstitutionEntry
            position = entry.find("#VariableSubstitutionEntry_")
            if (position == -1): continue
            try:
                varName = AdminConfig.showAttribute(entry, "symbolicName" )
            except:
                typee, value, tbck = sys.exc_info()
                logDebug("No symbolicName for node entry %s . Error: %s - %s" % (entry, str(value), typee))
                continue
            #endtry
            try:
                varValue = AdminConfig.showAttribute(entry, "value" )
            except:
                typee, value, tbck = sys.exc_info()
                logDebug("No value for node entry %s .  Error: %s " % (str(entry), str(value)))
                continue
            #endtry
            variableEntriesMap[varName]=varValue

        variableEntriesMapNodename = nodename


    while (varstring.find('{') > -1 and varstring.find('}') > -1):
        varName = varstring.split('${', 1)[1].split('}', 1)[0]
        leftPart=varstring.split('${',1)[0]
        rightPart=varstring.split('}',1)[1]
        #print "varstring=%s varName=%s leftPart=%s rightPart=%s" % (varstring, varName, leftPart, rightPart)
        if (variableEntriesMap.get(varName, -1) == -1):
            logDebug("Unable to locate ${"+varName+"}")
            varValue = ""
        else:
            varValue = variableEntriesMap[varName]
        logDebug("replace ${"+varName+"} => \""+varValue+"\"")
        var = leftPart+varValue+rightPart
        varstring=var
    #endWhile

    logDebug("subsNodeVariable %s=%s" % (varstring, var))
    return var






def getProcessDefs (serverTemplate, platform):
    '''
       Get the jvmid
       
       @param serverTemplate - Server template
       @param platform - Platform type
    '''   
    global AdminConfig
    global nodeName, serverName, serverid
    global javaProcServant
    global javaProcControl
    global jvmId, servantJvmId, controlJvmId

    logDebug("getProcessDefs(%s)" % serverid)


    jvmId = ""
    servantJvmId = ""
    controlJvmId = ""
        
    
    procDefs = convertToList(AdminConfig.showAttribute(serverTemplate, "processDefinitions"))
    for process in procDefs:
        if (process.find("JavaProcessDef") != -1):
            logDebug("getProcessDefs process = %s"  % process)

            # z/OS Has multiple ProcessDefs, Control, Servant, Adjunct
            ptype = AdminConfig.showAttribute(process, "processType" )
            # Carol: only z/OS has type value returned.
            logDebug("   type = %s" % ptype)

            if (is_zOS(platform) and ptype == "Control"):
                    javaProcControl = process
                    print "   jvmidstr=%s" % AdminConfig.showAttribute(process, "jvmEntries" )
                    jvmIdStr = wsadminToList(AdminConfig.showAttribute(process, "jvmEntries" ))
                    jvmIdStr = join( jvmIdStr, " ")
                    controlJvmId = jvmIdStr
                    logDebug("   getProcessDefs is Control")
            elif (ptype == "Servant" or not is_zOS(platform)):
                    javaProcServant = process
                    jvmIdStr = wsadminToList(AdminConfig.showAttribute(process, "jvmEntries" ))
                    print "jvmidstr=%s" % AdminConfig.showAttribute(process, "jvmEntries" )
                    jvmIdStr = join( jvmIdStr, " ")
                    servantJvmId = jvmIdStr
                    print "   servantJvmId: %s" % servantJvmId
                    logDebug("   getProcessDefs is Servant")
            elif (ptype == "Adjunct"):
                    # We don't currently care about an Adjunct
                    logDebug("   getProcessDefs is Adjunct")


            else:
                    logDebug("   getProcessDefs is unexpected type = %s " % type)



    if ( len(controlJvmId) == 0 ):
            #error "Unable to locate Control region JVM, configuration abandoned"
            print "  no control JvmId"

    if ( len(servantJvmId) == 0  ):
            #error "Unable to locate Servant region JVM, configuration abandoned"
            print "  no servntJvmid"


    # default "jvmId" is the servant on z/OS
    jvmId = servantJvmId

    logDebug(" default JVM ID: %s" % jvmId)
    logDebug(" control JVM ID: %s" % controlJvmId)
    logDebug(" servant JVM ID: %s" % servantJvmId)
    logDebug("getProcessDefs exit")

    return



def configPMIService ():
    '''
       Config PMI
    '''   
    global AdminConfig
    global serverid

    logDebug(" configPmiService() entry")


    PMIServiceId = AdminConfig.list("PMIService", serverid )
    logDebug("PMI service ID = %s" % PMIServiceId)


    if (PMIServiceId == "" or PMIServiceId == None):
        raise RuntimeError, "PMIService doesn't exist"


    #if (len(PMIServiceId) > 1):
    #    raise RuntimeError, "More than 1 PMIService exists"


    pmiats = to2DList(AdminConfig.show(PMIServiceId ))
    enable = ""
    for pmiat in pmiats:
        if ( pmiat[0] == "enable" ): enable = pmiat[1]


    if (enable != "true"):
        print "Enable PMI service"
        AdminConfig.modify(PMIServiceId, [["enable", "true"]] )
    else:
        print "PMI service is already enabled"


    logDebug("configPmiService() exit")





def setControlEnv( environments, entry, newValue ):
    '''
       Sets a control process environment variable
       Note that new entries will not be created if the value is to
       be "false".  This is to avoid creating unneeded entries for
       container and server activity records, which are obsolete.
       
       @param environments      current environment variables
       @param var               variable to add
       @param newValue          new value to set
    '''
       
    global AdminConfig
    global javaProcControl
    global description_entry

    logDebug(" setControlEnv(%s, %s) entry" % (entry, newValue))

    parm = [["name", entry], ["value", newValue], description_entry]
    index = 0
    i = -1
    envid = ""
    for var in environments:
        print var
        if (var.startswith(entry)):
            logDebug("found %s at position %d" % (entry, index))
            i = index
            envid = var
            break
        index += 1
    if (i < 0):
        if (newValue != "false"):
            logDebug("New environment variable %s" % entry)
            AdminConfig.create("Property", javaProcControl, parm, "environment" )
        else:
            logDebug("Not creating "+entry+" because it would be false")

    else:
        #envid = environments[i]
        value = AdminConfig.showAttribute(envid, "value" )
        if (value == newValue):
            logDebug("Already set to %s" % newValue)

        else:
            logDebug("Modifying existing Old value = %s" % value)
            logDebug("New value = %s" % newValue)
            AdminConfig.modify(envid, parm )


    logDebug(" setControlEnv() exit")







# config
##############################################################
#
#  Procedure  mergePaths
#  Arguments: base classpath
#             new classpath
#  Returns  Updated classpath string
#
#  Given a new classpath and an old classpath, merges the
#  new paths into the old paths, with the new paths before the
#  existing paths.
#
#  Note: Empty braces may have been introduced in previous configuration
#
##############################################################

def mergePaths ( basePaths, newPaths ):

        logDebug("mergePaths entry" )
        if basePaths != None:
            basePaths = basePaths.strip()
        if newPaths != None:
            newPaths = newPaths.strip()
        logDebug("Base:   "+basePaths)
        logDebug("New:    "+newPaths)


        results = []
        used = []
        # Put in the new path entries first, which includes the option keyword
        for path in split_path(newPaths):
            if (path.strip( ) == ""): continue
            if (path.strip( ) == "{}"): continue
            logDebug("New path entry: "+path)
            used.append(path)
            results.append(path)


        # Now put in all base path entries which are not duplicate, ITCAM product, or obsolete
        # Note that removing the duplicates will handle the keyword option, if present
        for path in split_path(basePaths):
            if (path.strip( ) == ""):  continue
            if (path.strip( ) == "{}"):  continue
            logDebug("Base path entry: %s" % path)

            # Do not add entries we have already added
            if path in used:
                logDebug("Duplicate entry %s dropped." % path)
                continue
            if (path.find("${ITCAMDCHOME}") != -1):
                logDebug("ITCAM product entry dropped.")
                continue

            if (checkObsolete(path ) ):
                print " \n*Warning*  Obsolete entry "+path+" removed "
                print "           If this entry was not originally set by ITCAM,"
                print "           you may wish to replace it manually. "
                continue
            results.append(path)


        results = join( results, ":")
        result = results

        logDebug("mergePaths exit = %s " % results)
        return result


# unconfig
##############################################################
#
#  Procedure  unmergePaths
#  Arguments: existing paths
#             added paths
#  Returns  Updated classpath string
#
#  Given an existing classpath, removes paths provided in
#  the addedPath parameter.
#
#  Note: Empty braces may have been introduced in previous configuration
#
##############################################################

def unmergePaths ( currentPaths, addedPaths ):
        global AdminConfig
        global product
        global jvmId
        global serverid

        logDebug("  unMergePaths entry")
        logDebug("Current %s" % currentPaths)
        logDebug("Added: %s " % addedPaths)


        # Build a lookup array of the added paths
        used = []
        for path in split_path(addedPaths):
            if (path.strip( ) == ""): continue
            if (path.strip( ) == "{}"): continue
            logDebug(" Path entry: %s " % path)
            used.append(path)


        results = []

        # Remove added paths, ITCAM product paths, and obsolete paths
        logDebug("Current paths: %s " % currentPaths)

        for path in split_path(currentPaths):
            if (path.strip( ) == ""): continue
            if (path.strip( ) == "{}"): continue
            logDebug("Path entry: %s" % path)
            if (path in used):
                logDebug("Added entry dropped.")
                continue

            if (path.find("${ITCAMDCHOME}") != -1):
                logDebug("ITCAM product entry dropped.")
                continue

            if (checkObsolete(path) ):
                print " \n*Warning*  Obsolete entry "+path+" removed\n "
                print "           If this entry was not originally set by ITCAM,"
                print "           you may wish to replace it manually.\n "
                continue

            results.append(path)

        results = join( results, ":")
        result = results

        logDebug("unmergePaths exit %s" % result)
        return result



# config
##############################################################
#
#  Procedure  mergePathsAfter
#  Arguments: base classpath
#             new classpath
#  Returns  Updated classpath string
#
#  Given a new classpath and an old classpath, merges the
#  new paths into the old paths, with the new paths following the
#  existing paths.
#
#  Note: Empty braces may have been introduced in previous configuration
#
##############################################################

def mergePathsAfter ( basePaths, newPaths ):

        logDebug(" mergePathsAfter entry" )
        if basePaths != None:
            basePaths = basePaths.strip()
        if newPaths != None:
            newPaths = newPaths.strip()
        logDebug("Base: %s  " % basePaths)
        logDebug("New:  %s  " % newPaths)

        used = []
        # Build an array of new path entries for checking
        for path in split_path(newPaths):
            if (path.strip() == ""): continue
            if (path.strip() == "{}"): continue
            used.append(path)


        results = []

        # Now put in all base path entries which are not duplicate, ITCAM product, or obsolete
        for path in split_path(basePaths):
            if (path.strip( ) == ""): continue
            if (path.strip() == "{}"): continue
            logDebug("Base path entry: " + path)
            # Drop the entries we are going to add


            if (path in used):
                logDebug("Duplicate entry dropped. %s " % path)
                continue


            if (path.find("${ITCAMDCHOME}") != -1):
                logDebug("ITCAM product entry dropped. %s " % path)
                continue


            if (checkObsolete(path ) ):
                print " \n*Warning*  Obsolete entry "+path+" removed\n "
                print "           If this entry was not originally set by ITCAM,"
                print "           you may wish to replace it manually.\n "
                continue


            results.append(path)


        # Put in the new path entries after the existing paths
        for path in split_path(newPaths):
            if (path.strip( ) == ""): continue
            if (path.strip( ) == "{}"): continue
            logDebug("New path entry: %s" % path)
            results.append(path)


        results = join( results, ":")
        result = results

        logDebug("mergePaths exit =\n  %s" % results)
        return result






def listSearch(list, value):
    '''
        return index of a value in the list
        returns -1 if no match found
    '''
    i = 0
    found = 0
    total = len(list)
    while i < total:
        item = list[i]
        if (item.startswith(value)):
            found = 1
            break
        i += 1
    if (found == 0): i = -1
    return i


def printError(pname, line):
    print " \n*Error*  Invalid format, entry ignored"
    print "         Input file has an invalid entry."
    print "         Script error must be corrected\n "
    raise RuntimeError, "input.properties invalid parameter '%s' in line '%s'" % (pname, line)

def validate (lines, progress):
    '''
        Validates the input.properties data.
        @parm  lines - A list contains the input.properties data
        @param progress - progress object reference
    '''
    global SCRIPT, VERSIONSTRING, timestamp
    #timestamp = strftime("%Y-%m-%d %H:%M:%S")
    idline = "#  "+SCRIPT+" Version "+VERSIONSTRING  + " " + timestamp + " #"
    blanks = "                                                                                                             "
    hashes = "#############################################################################################################"
    print " "
    print hashes[1:len(idline)+1]
    print "#" + blanks[1:len(idline) - 1] + "#"
    print idline
    print "#" + blanks[1:len(idline) - 1] + "#"
    print hashes[1:len(idline)+1]
    print " "

    global doSynchronize

    global AdminConfig
    global product
    global platform
    global nodeName
    global serverid
    global customservice
    global systemproperties
    global variables
    global control_environments
    global control_properties
    global genericargs
    global sharedLib
    global serverVersion
    global pipusc
    global pipusc_value
    global server_runtime
    global javaProcControl
    global javaProcServant
    global itcamdc_home
    global server_classpath
    global server_bootclasspat
    global smf
    global enableconfigmbean
    global containertimeouts
    global servantJvmId
    global controlJvmId
    global serverName
    global quietFlag
    global runtime_path
    global server_amhome_flag
    global JVMSecurityPolicy_flag
    global ecam_flag
    global maximumHeapSize
    global probe
    global wascell
    global wasnode
    global wasserver
    global current_inputproperties
    global config_type
    global common_constants
    global old_am_home
    global isStandalone
    global configComplete
    global xdtemplate_osname
    global environments

    product = ""
    platform = ""
    serverid = ""
    #customservice = []
    #systemproperties = []
    #environments = []
    #variables = []

    # z/OS control region environment
    #control_environments = ""
    #control_properties = ""
    genericargs = ""
    sharedLib = ""
    serverVersion = ""
    pipusc = ""
    pipusc_value = ""
    server_runtime = ""
    javaProcControl = ""
    javaProcServant = ""
    itcamdc_home = ""
    server_classpath = ""
    server_bootclasspath = ""
    smf = "No"
    containertimeouts = ""
    doSynchronize = 1
    server_amhome_flag = 0
    JVMSecurityPolicy_flag = 0
    ecam_flag = 0
    control_environments = []
    control_properties = []
    isStandalone = 0
    configComplete = 0

    # This flag is turned on to enable the PMI service.  We do not
    # disable the PMI service under any condtion.

    quiet_flag = 0
    failed_flag = 0

    old_am_home = ""

    
    # Note: Any value which may contain equal signs must
    #       be handled with regexp instead of lindex.
    # Comments are not allowed with a switch (probably JACL bug)
    # Extension MBean Provider is obsolete
    #   server.xMBeanProvider
    # These parameters, while still valid, should be replaced by server.variable specifications
    #   server.itcamdchome server.itcam61home
    # These two parameters are obsolete so they are ignored
    #   server.am.home server.ms.am.home
    # Note:server.systemproperty=am.home is used in ITCAM 7.2(ecam) so it's conditionally obsolete.
    # These two options are now obsolete but retained to process input files
    # created previously.  They are only needed for unconfigDataCollector.jacl
    #   server.pmiservice.initialSpecLevel server.pmiservice.statisticSet

    print " "
    for line in lines:
        if (line.strip() == ""):
            continue
        elif (line.lstrip()[0] == "#"):
            continue
        else:
            line = line.strip()
            tokens = line.split("=" )
            pname = tokens[0]
            logDebug("property_line=%s" % line)
            if ( pname == "product" ):
                product = tokens[1]
            elif ( re.compile("server\.genericJvmArguments[0-9]*").match(pname) ):
                if (len(line.split('=', 1)) == 2):
                    tmpargs = line.split('=', 1)[1].strip()
                    genericargs += " " + tmpargs.strip()
                    logDebug("tmpargs=%s" % tmpargs)
                    if (tmpargs.find("-Djava.security.policy") == 0):
                        JVMSecurityPolicy_flag = 1
                else:
                    printError(pname, line)
            elif ( re.compile("server\.systemproperty[0-9]*").match(pname) ):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpname = tmpname.strip()
                    tmpvalue = tmpvalue.strip()
                    if (tmpname == "am.appserver"):
                        serverName = tmpvalue
                    if (tmpname == "am.nodename"):
                        nodeName = tmpvalue
                    if (tmpname == "deploymentmgr.rmi.connection"):
                        extraRMI_flag = 1
                    if (tmpname == "appserver.rmi.host"):
                        extraRMI_flag = 1
                    if (tmpname == "am.home" and tmpvalue != ""):
                        server_amhome_flag = 1
                    if (tmpname == "ITCAM_DC_ENABLED"):
                        ecam_flag = 1
                    systemproperties.append([["name", tmpname], ["value", tmpvalue]])
                else:
                    printError(pname, line)
            elif ( re.compile("server\.environment[0-9]*").match(pname) ):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpvalue = tmpvalue.strip()
                    logDebug("environment %s=%s" % (tmpname, tmpvalue))
                    environments.append([["name", tmpname], ["value", tmpvalue]])
                else:
                    printError(pname, line)
            elif ( re.compile("server\.variable[0-9]*").match(pname) ):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpvalue = tmpvalue.strip()
                    variables.append([["symbolicName", tmpname], ["value", tmpvalue]])
                    if (tmpname == "ITCAMDCHOME" or tmpname == "ITCAM61HOME"):
                        itcamdc_home = tmpvalue
                    elif (tmpname != "AM_HOME" and tmpname != "MS_AM_HOME" and tmpname != "ITCAMDCVERSION"):
                        printError(tmpname, line)
                else:
                    printError(pname, line)
            elif ( re.compile("control\.systemproperty[0-9]*").match(pname) ):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpvalue = tmpvalue.strip()
                    if (tmpname == "ITCAM_DC_ENABLED" and tmpvalue != ""): ecam_flag = 1
                    control_properties.append([["name", tmpname], ["value", tmpvalue]])
                else:
                    printError(pname, line)
            elif ( re.compile("control\.environment[0-9]*").match(pname)):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpvalue = tmpvalue.strip()
                    control_environments.append([["name", tmpname], ["value", tmpvalue]])
                else:
                    printError(pname, line)
            elif ( re.compile("control\.classpath").match(pname) ):
                control_classpath = tokens[1]
            elif ( re.compile("server\.customservice[0-9]*.classname").match(pname) ):
                tmpvalue = tokens[1]
                customservice.append(["classname", tmpvalue])
            elif ( re.compile("server\.customservice[0-9]*.displayName").match(pname) ):
                tmpvalue = tokens[1].strip()
                customservice.append(["displayName", tmpvalue])
                customservice.append(["externalConfigURL", tmpvalue])
            elif ( re.compile("server\.customservice[0-9]*.description").match(pname) ):
                tmpvalue = tokens[1].strip()
                customservice.append(["description", tmpvalue])
            elif ( re.compile("server\.customservice[0-9]*.enable").match(pname) ):
                tmpvalue = tokens[1].strip()
                customservice.append(["enable", tmpvalue])
            elif ( re.compile("server\.customservice[0-9]*.classpath").match(pname) ):
                tmpvalue = tokens[1].strip()
                if (tmpvalue == ""): tmpvalue = "null"
                customservice.append(["classpath", tmpvalue])
            elif ( pname == "library.WPSlib" ):
                tmpvalue = tokens[1].strip()
                libs = tmpvalue.split(";" )
                sharedLib = libs[0]
            elif ( pname =="server.platform"):
                platform = tokens[1].strip()
            elif ( pname == "server.smf" ):
                smf = tokens[1]
            elif ( pname == "server.classpath" ):
                server_classpath = tokens[1].strip()
            elif ( pname == "server.bootclasspath" ):
                server_bootclasspath = tokens[1]
            elif ( pname == "server.runtime" ):
                server_runtime = tokens[1].strip()
            elif ( pname == "server.version" ):
                serverVersion = tokens[1].strip()
                logDebug("version=%s" % serverVersion)
            elif ( pname == "server.id" ):
                serverid = tokens[1].strip()
            elif ( pname == "server.xMBeanProvider"  ):
                pass
            elif ( re.compile("server\.itcam(61|dc)home").match(pname) ):
                variables.append([["symbolicName", "ITCAMDCHOME"], ["value", tokens[1] ]])
                itcamdc_home = tokens[1]
            elif ( pname == "server.am.home" ):
                #obsolete parameter
                pass
            elif ( pname == "server.ms.am.home" ):
                #obsolete parameter
                pass
            elif ( pname == "server.protocol_iiop_propagate_unknown_service_ctxs" ):
                variables.append([["symbolicName", "protocol_iiop_propagate_unknown_service_ctxs"], ["value", tokens[1] ]])
            elif ( pname == "server.policyedit" ):
                if (tokens[1] == "Yes"):
                    serverPolicy_flag = 1
                elif (tokens[1] == "No"):
                    serverPolicy_flag = 0
                else:
                    print " \n*Error*  Unexpected value for server.policyedit keyword:"
                    print "         "+line+"\n "
                    errorCode = None
                    raise RuntimeError, "Unexpected value for server.policyedit keyword\n"+line
            elif ( pname == "server.enablePMI" ):
                if (tokens[1] == "Yes"):
                    pmiService_flag = 1
                elif (tokens[1] == "No"):
                        pmiService_flag = 0
                else:
                    print " \n*Error*  Unexpected value for server.enablePMI keyword: %s '%s'" % (line, tokens)
                    errorCode = None
                    raise RuntimeError, "Unexpected value for server.enablePMI keyword\n"+line
            elif ( pname == "server.enableconfigmbean" ):
                pass
            elif ( pname == "server.containertimeouts"  ):
                containertimeouts = tokens[1]
            elif ( pname == "server.pmiservice.initialSpecLevel" ):
                pass
            elif ( pname == "server.pmiservice.statisticSet" ):
                server_pmiservice_statisticSet = tokens[1]
            elif(pname == "server.maximumHeapSize"):
                maximumHeapSize = tokens[1]
                logDebug("server.maximumHeapSize=%s" % maximumHeapSize)

            else:
                print " \n*Error*  Can't find match for \""+pname+"\"\n "
                print "         Input file has an invalid entry."
                print "         Script error must be corrected\n "
                errorCode = None
                raise RuntimeError, "input.properties invalid parameter: \""+pname+"\""

    print " "
    if (product == ""):
        product = "ITCAM"
        pmiService_flag = 1


    ##################### Do some basic validation #####################

    print "product=%s" % product
    if (product != "ITCAM" and product != "eCAM"):
        print " \n*Error*  Invalid product specified, must be either ITCAM or eCAM\n "
        errorCode = None
        raise RuntimeError, "Invalid product specified"


    '''
    print "serverid=%s" % serverid
    if (serverid == ""):
        logDebug("No server id")
        errorCode = None
        raise RuntimeError, "server ID is missing from input.properties file"
    '''

    # In WAS 5.1 the server name is delimited on the right with ":", later it is "|".

    #server1(cells/x336r1s36-vm01Node01Cell/nodes/x336r1s36-vm01Node02/servers/server1|server.xml#Server_1183122130078)
    '''
    re_format = re.compile("^.*/nodes/.*/servers/.*(\||:).*$")
    if (re_format.match(serverid)):
        rex=re.compile("(/nodes/|/servers/|\||:)+")
        (tmp1, tmp2, tmpnode, tmp3, tmpserver, tmp4, tmp5) = rex.split(serverid)
        nodeName = tmpnode
        serverName = tmpserver
        logDebug("serverName=%s nodeName=%s" % (serverName, nodeName))
    else:
        logDebug("Unable to extract node and server from server ID %s" % serverid)
        errorCode = None
        raise RuntimeError, "Unable to extract node and server from server ID"


    print "server="+serverName
    if (serverName == ""):
        errorCode = None
        raise RuntimeError, "unable to extract server name from server ID"

    print "node="+nodeName
    if (nodeName == ""):
        errorCode = None
        raise RuntimeError, "unable to extract node name from server ID"
    '''
    # When a stand-alone node is brought under a deployment manager, the
    # server ID will change.  For unconfig we must allow for that possibility.

    '''
    currentServerid = AdminConfig.getid("/Node:%s/Server:%s" % (nodeName,serverName) )
    logDebug("currentServerId=%s serverid=%s" % (currentServerid, serverid))
    if (currentServerid != serverid):
        print " \n*Warning*  Server ID from response file does not match server ID for "+nodeName+"."+serverName
        print "           File value = "+serverid
        print "           Actual     = "+currentServerid+"\n "
        # If we are configuring, a changed server ID is not acceptable
        if (SCRIPT != "unconfigDataCollector"):
            errorCode = None
            raise RuntimeError, "input.properties server ID does not match server ID for\n"+nodeName+"."+serverName


        pos = serverid.find("#Server_")
        if (pos > -1):
            pos += 8
            idnum = serverid[pos:-1]
        else:
            idnum = "Bad ID number"

        logDebug("idnum=%s" % (idnum))

        pos = currentServerid.find("#Server_")
        if (pos > -1):
            pos += 8
            curidnum = currentServerid[pos:-1]
        else:
            curidnum = "Bad current ID number"

        logDebug("curidnum=%s" % (curidnum))

        if (idnum != curidnum):
            logDebug("Server ID number %s from reponse file does not match server ID number %s" % (idnum, curidnum))
            errorCode = None
            raise RuntimeError, "Server ID number "+idnum+" from reponse file does not match server ID number "+curidnum


        print "Processing continuing with actual server ID."
        serverid = currentServerid
    '''

    # If the server name doesn't match there is a serious problem.  Let us not make it worse.
    '''
    currentServerName = AdminConfig.showAttribute(serverid, "name" )
    if (currentServerName != serverName):
        print " \n*Error*  Server name from response file does not match server ID"
        print "         returned for server ID object."
        print "         File value = " + serverName
        print "         Actual     = " + currentServerName
        print "         Server ID  = " + serverid
        errorCode = None
        raise RuntimeError, "input.properties server name does not match server name for\n"+nodeName+"."+serverName+" = "+currentServerid
    '''

    print "product home=" + itcamdc_home
    if (itcamdc_home == ""):
        errorCode = None
        raise RuntimeError, "product location is missing from input.properties file"


    print "product platform=" + platform
    if (platform == ""):
        print " \n*Error*  Platform is missing from input.properties file\n "
        errorCode = None
        raise RuntimeError, "Product platform is missing from input.properties file"


    ####################################################################
    #                                                                  #
    #  For z/OS we must use the server.version supplied in the input.  #
    #                                                                  #
    #  This is because the server object name does not exist until the #
    #  server is actually started.                                     #
    #                                                                  #
    ####################################################################

#    if (platform != "z/OS"):
#        print " \n*Error*  Platform is \""+platform+"\", must be z/OS\n "
#        global errorInfo
#        errorInfo = str(_tbck_.dumpStack())
#        global errorCode
#        errorCode = None
#        raise RuntimeError, "Platform is "+platform+", must be z/OS"
#

    if (serverVersion == ""):
        #errorCode = None
        raise RuntimeError, "server version is missing from input.properties file"


    print "Server major version is "+serverVersion+"."

    if is_zOS(platform) and smf != "Yes" and smf != "No" and smf != "Leave":
        print "*Error*  Invalid value for server.smf="+smf+", must be one of Yes/No/Leave"
        failed_flag = 1
        



    # Ignore obsolete parameters
    if (server_bootclasspath != ""):
        print " "
        print "*Warning*  Obsolete parameter server.bootclasspath ignored"
        print " "
        server_bootclasspath = ""


    if (containertimeouts != ""):
        print " "
        print "*Warning*  Obsolete parameter server.containertimeouts ignored"
        print " "
        containertimeouts = ""


    
    '''
    if (serverid == ""):
        print "*Error*  Can't get serverid"
        failed_flag = 1
    '''    

    if (genericargs == ""):
        print "*Error*  Can't get jvm generic arguments"
        failed_flag = 1

    if (systemproperties == ""):
        print "*Error*  Can't get system properties"
        failed_flag = 1

    if (environments == ""):
        print "*Error*  Can't get environments"
        failed_flag = 1



    if (failed_flag):
        print " "
        print "           Input file has an invalid entry."
        print "           Script error must be corrected"
        print " "
        errorCode = None
        raise RuntimeError, "Processing error, unable to complete configuration"

    '''
    if hasNodeagent() != 0:
        msg = resourcebundle.getmsg("CFG1147E", "Unable to locate NodeSync for node {0}. Make sure nodeagent for the node is started.", [nodeName])
        progress_obj.write_progress(msg)
        raise RuntimeError, "Error: nodeagent for node %s not started" % nodeName
    '''



def configure(serverTemplate, platform):
    '''
       Start configuring the server.xml.
       
       @param servletTemplate - Server template
       @param platform - Platform type
    '''   

    global doSynchronize
    global AdminConfig
    global product
    global nodeName
    global serverid
    global customservice
    global systemproperties
    global variables
    global control_environments
    global control_properties
    global genericargs
    global sharedLib
    global serverVersion
    global pipusc
    global pipusc_value
    global server_runtime
    global javaProcControl
    global javaProcServant
    global itcamdc_home
    global server_classpath
    global server_bootclasspat
    global smf
    global enableconfigmbean
    global containertimeouts
    global servantJvmId
    global controlJvmId
    global serverName
    global quietFlag
    global runtime_path
    global server_amhome_flag
    global JVMSecurityPolicy_flag
    global ecam_flag
    global maximumHeapSize
    global probe
    global wascell
    global wasnode
    global wasserver
    global current_inputproperties
    global config_type
    global common_constants
    global old_am_home
    global isStandalone
    global configComplete
    global xdtemplate_osname
    global environments
    
    getProcessDefs(serverTemplate, platform)
    logDebug("Return from getProcessDefs")

    # Establish runtime path for file access

    runtime_path = server_runtime
    runtime_path.replace("${ITCAM61HOME}", "${ITCAMDCHOME}")
    runtime_path.replace("${ITCAMDCHOME}", itcamdc_home)
    logDebug("Resolved runtime path: %s" % runtime_path)

    print " "
    print "Start setting attributes for %s" % serverTemplate

    # JVM

    initServerVariableMapCache(serverTemplate)
    
    if (genericargs != ""):
        print "Configuring JVM Args"
        configJVM(serverTemplate, genericargs, platform)
        
    

    #System Properties
    if (systemproperties != ""):
        print "Configuring System Properties"
        configSystemProperties(servantJvmId, systemproperties)

    # Environment variables
    if ( environments != "" ):
        print "Configuring Environment Variables"
        configEnvs(javaProcServant, environments, platform)

     
    # Server variables
    #if (len(variables) > 0 ):
    #    for varent in variables:
    #        print "Configuring server variable %s" % varent[0][1]
    #        logDebug("variable entry = %s" % varent)
    #        configServerVariable(serverTemplate, varent)

    # Server region classpath
    if (server_classpath != "" and server_classpath != "None"):
        print "Configuring server region classpath"
        print "Adding server classpath entry: "+server_classpath
        oldClassPath = AdminConfig.showAttribute(servantJvmId, "classpath" )
        if oldClassPath != None:
            oldClassPath = oldClassPath.strip()
        logDebug("oldClassPath: " + oldClassPath)
        mergedClassPath = mergePaths(oldClassPath, server_classpath )
        #Remove special characters if there are any
        if mergedClassPath != None:
            mergedClassPath = mergedClassPath.strip()
        parm = [["classpath", mergedClassPath]]
        logDebug("classpath parm = %s" % parm)
        try:
            AdminConfig.unsetAttributes(servantJvmId, "classpath" )
        except:
            _type_, _value_, _tbck_ = sys.exc_info()
            ermsg = str(_value_)
            print "********************* "
            print "*Warning* unsetAttributes failed or is not supported in this WebSphere release."
            print "*Warning* Please check your server region classpath."
            print "*Warning* errorInfo: " + ermsg
            print "********************* "
        AdminConfig.modify(servantJvmId, parm )
    else:
        logDebug("no server.classpath specified, skipping the configuration")

    # Control region classpath

    if (control_classpath != "" and control_classpath != "None"):
        print "Configuring Control Region classpath"
        print "Adding control classpath entry: "+control_classpath
        oldClassPath = AdminConfig.showAttribute(controlJvmId, "classpath" )
        mergedClassPath = mergePaths(oldClassPath, control_classpath )
        parm = [["classpath", mergedClassPath]]
        logDebug("classpath parm = %s" % parm)

        try:
            AdminConfig.unsetAttributes(controlJvmId, "classpath" )
        except:
            _type_, _value_, _tbck_ = sys.exc_info()
            ermsg = str(_value_)
            print "********************* "
            print "*Warning* unsetAttributes failed or is not supported in this WebSphere release."
            print "*Warning* Please check your control region classpath."
            print "*Warning* errorInfo: " + ermsg
            print "********************* "
        AdminConfig.modify(controlJvmId, parm )

    # Control region system properties

    if (platform.startswith("zos") and len(control_properties) >  0):
        print "Configuring Control Region system properties"
        configSystemProperties(controlJvmId, control_properties )


    # Control region environment variables
    # (Currently there aren't any.)

    if (len(control_environments) > 0):
        print "Configuring Control Environment Variables"
        configEnvs(javaProcControl, control_environments, platform)


    # Custom Service

    if (len(customservice) > 0):
        print "Configuring CustomService"
        configCustomService(customservice )


    # WAS SMF data recording

    if (platform.startswith("zos")):
        if (smf == "Yes"):
                print "Enabling SMF data collection"
                configSmf("true" )
        elif (smf == "No"):
                print "Disabling SMF data collection"
                configSmf("false" )
        elif (smf == "Leave"):
                print "Making no changes to WebSphere SMF data collection"



    # Save changes and exit

    try:
        AdminConfig.save( )
        configComplete = 1
    except Exception, errmsg:
        _type_, _value_, _tbck_ = sys.exc_info()
        errmsg = str(_value_)
        print " \n*Error*  Failed to save"
        print "         Error message: " + errmsg + "\n "
        raise RuntimeError, "saveChanges routine failed - " % (errmsg)
       
    #returncode = saveChanges( )
    #if (returncode != "true"):
    #    raise RuntimeError, "saveChanges routine failed with RC( %s ) for server %s" % (str(returncode), serverName)


    print common_constants.SUCCESSFULLY_CONFIGURED_DC_MSG
    #print "Successfully configured data collector for server "+serverName


def initServerVariableMapCache(serverTemplate):
    '''
       Save the varialbes
       
       @param serverTemplate - Server Template
    '''
    
       
    global AdminConfig
    global serverid
    global variableMap, variableMapArray, variableMapServerid, serverVariableEntries


    logDebug("  initServerVariableMapCache() entry")


    # The array doesn't need to be rebuilt if additions and deletions are maintained
    # If an entry has been added, the variable map must be rebuilt
    # If an entry has been removed, the variable map must be rebuilt

    if ( len(variableMap) == 0  ):
        logDebug("   First variable map cache")
        variableMap = ""
        variableMapServerid = serverTemplate
        # Add a dummy entry to define 2-dimensional array
        variableMapArray = {}


    if (variableMapServerid != serverTemplate):
        logDebug("Server for variable map cache has changed")
        variableMap = ""
        variableMapServerid = serverTemplate
        serverVariableEntries = []


    if (len(variableMap) == 0):
        logDebug("Creating new variable map entries cache")
        serverVariableEntries = []
        variableMapArray = {}
        # Build the array

        variableMap = AdminConfig.list("VariableMap", serverTemplate)
        if (variableMap == ""):
            print " \n*Error*  VariableMap doesn't exist in server " + serverTemplate + "\n "
            raise RuntimeError, "VariableMap doesn't exist in server %s" % serverTemplate

        serverVariableEntries = wsadminToList(AdminConfig.showAttribute(variableMap, "entries" ))
        for entryId in serverVariableEntries:
            subst_found = entryId.find("#VariableSubstitutionEntry_")
            if (subst_found == -1): 
                continue

            try:
                name = AdminConfig.showAttribute(entryId, "symbolicName" )
            except:
                logDebug("No symbolicName for server entry %s " % entryId)
                print " \n*Warning*  Invalid WAS server variable object "+entryId
                #print "           "+errmsg+"\n "
                continue

            try:
                value = AdminConfig.showAttribute(entryId, "value" )
            except:
                logDebug("No value for server entry %s " % entryId)
                print " \n*Warning*  Invalid WAS server variable object %s" % entryId
                #print "           "+errmsg+"\n "
                continue


            logDebug("Entry ID = %s" % entryId)
            logDebug("%s=%s" % (name, value))
            #variableMapArray[name] = [name, [entryId, value]]
            variableMapArray[name] = value
            
        logDebug("%d properties in cache" % (len(variableMapArray)))

    logDebug("initServerVariableMapCache() exit")



def configJVM (serverTemplate, newGenericJvmArguments, platform):
    '''
       Configure JVM
       
       @param serverTemplate - Server template
       @param newGenericJvmArguments - JVM arguments
       @param platform - Platform type
    '''   
    global AdminConfig
    global AdminTask
    global product
    global jvmId
    global nodeName, serverName, serverid
    global serverVersion
    global wasnode
    global wasserver
    

    logDebug("configJVM() entry")


    foundRTT = "false"
    runam = ""

    print "Modify JVM command line arguments"
    oldGenericJvmArguments = AdminConfig.showAttribute(jvmId, "genericJvmArguments" )

    print "oldGenericJvmArguments: ", oldGenericJvmArguments

    #Check if agentlib:am* is part of oldGenericJvmArguments.
    #If yes, remove it from oldGenericJvmArguments.
    updateOldArgList = []
    if oldGenericJvmArguments != None and oldGenericJvmArguments != "":
        oldArgList = oldGenericJvmArguments.split()
        addArgToList = "true"
        for oldArg in oldArgList:
            if oldArg.find("-agentlib:am") == 0:
                addArgToList = "false"
            elif (oldArg.startswith("-Xbootclasspath/a:") or oldArg.startswith("-Xbootclasspath/p:")):
                index = oldArg.index(":")
                bootCPList = oldArg[index+1:len(oldArg)]
                print "bootCPList: ", bootCPList
                #Check if any jar files from older version of ITCAM
                #If yes, remove them from bootclasspath as we do not want to save them in AM_OLD_JVM_ARG.
                bootCPToAdd = []
                classpathdelimiter = ":"
                if (platform.startswith("win")):
                    classpathdelimiter = ";"
                cpjars = bootCPList.split(classpathdelimiter)
                oldArg = ""
                for cpjar in cpjars:
                    print "cpjar: ", cpjar
                    if cpjar.find("${ITCAM") == 0:
                        continue
                    else:
                        bootCPToAdd.append(cpjar)
                if len(bootCPToAdd) > 0:
                    oldArg = join(bootCPToAdd, os.sep)
                    addArgToList = "true"
            if addArgToList == "true":
                updateOldArgList.append(oldArg)

    oldGenericJvmArguments = join( updateOldArgList, " ")
    print "oldGenericJvmArguments after cleaning up: ", oldGenericJvmArguments

    # Save the old generic JVM arguments in a variable, so unconfig can find it
    #am_old_jvm_args = [["symbolicName", "AM_OLD_JVM_ARGS"], ["value", oldGenericJvmArguments]]

    # Check if the variable already exists (the user is running config again)
    #oldvarid = getServerVariableID("AM_OLD_JVM_ARGS" )
    #if (oldvarid == ""):
    #configServerVariable(serverTemplate, am_old_jvm_args )
    #else:
    #    print "Variable AM_OLD_JVM_ARGS already exists - left in place"
    #    logDebug("Saved generic JVM arguments are: ")
    #    displayJvmArgs("", AdminConfig.showAttribute(oldvarid, "value" ) )


    logDebug("Current generic JVM arguments are: ")
    displayJvmArgs("", oldGenericJvmArguments )

    newArgList = newGenericJvmArguments.split()
    oldArgList = oldGenericJvmArguments.split()
    jvmArgList = []
    if (oldGenericJvmArguments != ""):
        logDebug("New generic JVM arguments are:")
        displayJvmArgs("", newGenericJvmArguments )
        oldArgList = oldGenericJvmArguments.split()

        # First go through oldGenericJvmArguments, and see if
        # there are replacements in newGenericArgs

        for oldArg in oldArgList:
            logDebug("oldArg=%s" % oldArg)
            if (oldArg.startswith("-Xrunjvmpi:")):
                runam = oldArg.split("-Xrunjvmpi:")[1]
                #if { [regexp -- {^-Xrunjvmpi:(.+)$} $oldArg "" runam] } {
                if (int(serverVersion) >= 61):
                    print "Found ITCAM for RTT in WAS 6.1. Leave -Xrunjvmpi"
                    jvmArgList.append(oldArg)
                else:
                    print "Found ITCAM for RTT libjvmpi. Converted to -Xrunam:"+runam
                    jvmArgList.append("-Xrunam:"+runam)


            # For WAS 6.1, -Xrunam is replaced by -agentlib:am, jiti.jar is
            # removed from boot class path, and the injector path is no
            # longer present.  This should allow graceful migration from
            # WAS 6.0 to WAS 6.1 by simply rerunning setupwas.sh.
            # The -Xrunam* in new list replaces the -Xrunam* in the old list
            elif (oldArg.find("-Xrunam") == 0):
                runam = getArg("-Xrunam", newArgList )
                if (runam == ""):  runam = getArg("-agentlib:am", newArgList )
                print "Found ITCAM -Xrunam for WAS. Migrating to "+runam
                jvmArgList.append(runam)

            # The -agentlib in new list replaces the -agentlib in the old list
            elif (oldArg.find("-agentlib:am") == 0):
                agentlib = getArg("-agentlib:am", newArgList )
                print "Found ITCAM -agentlib:am in old args. Now "+agentlib
                jvmArgList.append(agentlib)

            # Merge paths for -Xbootclasspath/a: and -Xbootclasspath/p:
            elif (oldArg.startswith("-Xbootclasspath/a:") or oldArg.startswith("-Xbootclasspath/p:")):
                # elseif { [regexp -- {^(-Xbootclasspath/[ap]:).*} $oldArg oldBootClasspath arg] } {
                arg = oldArg.split(':')[0] + ':'
                oldBootClasspath = arg
                newBootClasspath = getArg(arg, newArgList )
                bootClasspath = mergePaths(oldBootClasspath, newBootClasspath )
                # If using JVMTI now and RTT not present, remove any old jiti libraries
                if (newGenericJvmArguments.find("-agentlib:am") >= 0):
                    if (oldGenericJvmArguments.find("-Xrunjvmpi") < 0):
                        print "Removing unused jiti libraries from bootClasspath"
                        bootClasspath = removeJiti(bootClasspath )


                print "Merged boot class "+bootClasspath
                jvmArgList.append(bootClasspath)

            # Any properties in old list will be dropped if they are also in new list
            elif (oldArg.startswith("-D") and len(oldArg.split('=')) == 2):
                #TODO} elseif { [regexp -- {^-D([^=]+)=(.*)$} $oldArg x prop ] } {
                prop = oldArg.split('=')[1]
                arg = getArg("-D"+prop+"=", newArgList )
                if (arg == ""):
                    jvmArgList.append(oldArg)
                else:
                    print "Property "+prop+" updated to "+arg


            # Any -verbose from old list will be used if not found in new list
            elif (oldArg.find("-verbose") == 0):
                for arg in newArgList:
                    if (arg== oldArg):
                        oldArg = ""
                        break


                if (oldArg== ""):
                    print "Duplicate item "+arg+" dropped"
                else:
                    jvmArgList.append(oldArg)


            # Drop IProbeInectorManager if using JVMTI
            elif (oldArg.find("-Dcom.ibm.tivoli.jiti.injector.IProbeInjectorManager") == 0):
                if (newGenericJvmArguments.find("-agentlib:am") >= 0):
                    print " Using JVMTI. Dropping com.ibm.tivoli.jiti.injector.IProbeInjectorManager"
                else:
                    jvmArgList.append(oldArg)


            # Drop obsolete ITCAM_JVM_ARGS
            elif (oldArg == "${ITCAM_JVM_ARGS}"):
                print "Dropping obsolete entry "+oldArg

            # Drop erroneous entry jlog.common.dir.CYN=
            elif (oldArg.find("jlog.common.dir.CYN=") == 0):
                print "Dropping erroneous entry "+oldArg

            # Drop obsolete entry -Declipse.security=
            elif (oldArg.find("-Declipse.security=") == 0):
                print "Dropping erroneous entry "+oldArg

            # Drop no longer needed entry -Xnoclassgc
            elif ("-Xnoclassgc"== oldArg):
                print "Dropping obsolete entry "+oldArg

            # Drop duplicate entry -Xverbosegclog:
            elif (oldArg.find("-Xverbosegclog:") == 0):
                print "Dropping conflicting entry "+oldArg

            # Anything else will be passed through
            else:
                print "Pass through: "+oldArg
                jvmArgList.append(oldArg)


        logDebug("JVM arg list updates: ")
        #jvmArgList = join( jvmArgList)
        displayJvmArgs("", join(jvmArgList) )

        # Now go through the newArglist, see if there are new ones to go in.

        print "Examining new Jvm arg list"
        for newArg in newArgList:

            # Only one runam in the final list
            if (newArg.find("-Xrunam") == 0):
                arg = getArg("-Xrunam", jvmArgList )
                if (arg == ""): jvmArgList.append(newArg)


            # if a bootpath was merged, do not add the one in the new list
            elif (newArg.startswith("-Xbootclasspath/a:") or newArg.startswith("-Xbootclasspath/p:")):
                #TODO} elseif { [regexp -- {(-Xbootclasspath/[ap]:).*} $newArg x prop] } {
                prop = newArg.split(':')[0] + ':'
                arg = getArg(prop, jvmArgList )
                if (arg == ""):
                    jvmArgList.append(newArg)


            # Only one -verbose:gc in the final list
            elif (newArg.find("-verbose:gc") == 0):
                arg = getArg("-verbose:gc", jvmArgList )
                print "getArg -verbose:gc found "+arg
                if (arg == ""):
                    jvmArgList.append(newArg)

            # Add all new elements that are not already present in the final list
            else:
                for arg in jvmArgList:
                    if (arg == newArg):
                        print "newArg "+newArg+" already present"
                        newArg = ""
                        break


                if (newArg != ""):
                    print "Adding newArg "+newArg
                    jvmArgList.append(newArg)



        jvmArgList = join( jvmArgList, " ")
        genericJvmArgs = jvmArgList
    else:
        # No oldArgs
        genericJvmArgs = newGenericJvmArguments


    # If AM_CONFIG_JVM_ARGS exists, we have been configured before.
    # In that case, determine if any of the current parameters are extras
    # defined after configuration, and remove those from the parameters
    # from the value to be stored in AM_CONFIG_JVM_ARGS.  This is so that
    # unconfig will recognize those extras as new parameters.

    configJvmArgs = genericJvmArgs
    '''
    configvarid = getServerVariableID("AM_CONFIG_JVM_ARGS" )
    if (configvarid != ""):
        logDebug(" Will replace existing AM_CONFIG_JVM_ARG")
        # Find any options in the current JVM options which aren't in AM_CONFIG_JVM_ARGS.
        # These will be extras defined since the last configuration.

        configValue = AdminConfig.showAttribute(configvarid, "value" )
        extraValue = diffJvmArgs(oldGenericJvmArguments, configValue )
        if (extraValue != ""):
            logDebug("Added since last configuration: %s" % extraValue)
            # Remove the extra JVM options from the AM_CONFIG_JVM_ARGS options.
            # This will cause unconfig to recognize them as defined since configuration.
            configJvmArgs = diffJvmArgs(configJvmArgs, extraValue )


        # But things may also be removed.
        #   If in AM_OLD_JVM_ARGS
        #   And in AM_CONFIG_JVM_ARGS
        #   But not in current JVM args
        #   Then it was deleted,
        #   So it must be added back to the new AM_CONFIG_JVM_ARGS for unconfig.sh

        if (oldvarid != ""):
            oldValue = AdminConfig.showAttribute(oldvarid, "value" )
            configJvmArgsList = configJvmArgs.split( )
            for item in oldValue.split( ):
                if (item == ""): continue
                logDebug("item=%s" % item)
                # Is item from AM_OLD_JVM_ARGS in AM_CONFIG_JVM_ARGS?
                additem = ""
                for configitem in configValue.split( ):
                    if (configitem == item):
                        additem = "yes"
                        break


                # And is item not in current JVM args?
                if (additem != "yes"): continue
                for currentitem in genericJvmArgs.split( ):
                    if (currentitem == item):
                        additem = "no"
                        break
                if (additem != "yes"): continue
                # Then it must be reinstated into new AM_CONFIG_JVM_ARGS
                logDebug("Deleted item found to retain in AM_CONFIG_JVM_ARGS: %s"  % item)
                configJvmArgsList.append(item)

            #configJvmArgsList = join( configJvmArgsList, " ")
            configJvmArgs = join( configJvmArgsList, " ")
            print "configJvmArgs = %s" % configJvmArgs

    else:
        logDebug("No previous AM_CONFIG_JVM_ARG")
    '''
    displayJvmArgs("Final merged generic JVM arguments", genericJvmArgs )

    # Save the configured generic JVM arguments in a variable, so unconfig can identify changes
    #am_config_jvm_args = [["symbolicName", "AM_CONFIG_JVM_ARGS"], ["value", configJvmArgs]]
    #configServerVariable(serverTemplate, am_config_jvm_args )

    # modify genericJvmArguments
    process_type_arg = ""
    if is_zOS(platform): 
        process_type_arg = "-processType Servant"    
    print "Jvm Arguments  %s" % (genericJvmArgs)
    #argString = "[ %s -serverName %s -nodeName %s -genericJvmArguments \"%s\"]" % (process_type_arg, wasserver, wasnode, genericJvmArgs)
    argString = "[ %s -genericJvmArguments \"%s\"]" % (process_type_arg, genericJvmArgs)
    
    print "Setting jvm argument ARGS using  %s" % (argString)
    #AdminTask.setJVMProperties(argString)
    
    
    procDefs = convertToList(AdminConfig.showAttribute(serverTemplate, "processDefinitions"))
    for process in procDefs:
        if (process.find("JavaProcessDef") != -1):
            logDebug("getProcessDefs process = %s"  % process)

            # z/OS Has multiple ProcessDefs, Control, Servant, Adjunct
            ptype = AdminConfig.showAttribute(process, "processType" )
            logDebug("   type = %s" % ptype)

            if (is_zOS(platform) and ptype == "Control"):
                jvmEntries = AdminConfig.showAttribute(process, "jvmEntries")
            elif (is_zOS(platform) and ptype == "Servant"):
                continue 
            elif (is_zOS(platform) and ptype == "Adjunct"):
                continue        
            else:
                jvmEntries = AdminConfig.showAttribute(process, "jvmEntries") 
                    
            if (len(jvmEntries)>0 and jvmEntries[0]=='[' and jvmEntries[len(jvmEntries)-1]==']'):
                    jvmEntries = jvmEntries[1:len(jvmEntries)-1]
            AdminConfig.modify(jvmEntries, [["genericJvmArguments",genericJvmArgs]])
    
    
    

    logDebug("configJVM() exit")


def configServerVariable (serverTemplate, newVariable):
    '''
       Configure the server variables
       
       @param serverTemplate - Serevr Template
       @param newVariable - New Variable
    '''
       
    global AdminConfig
    global AdminTask
    global nodeName, serverName, serverid
    global variableMap, variableMapArray
    global description_entry
    global wascell
    global wasnode
    global wasserver

    logDebug(" configServerVariable(%s) entry" % newVariable)

    _J2J_bracket_ = newVariable[0][0]
    if (_J2J_bracket_ != "symbolicName" or newVariable[1][0] != "value"):
        raise RuntimeError, "configServerVariable bad parameter = "+newVariable


    newSymbolicName = newVariable[0][1]
    newValue = newVariable[1][1].strip()


    initServerVariableMapCache(serverTemplate)

    logDebug("description_entry=%s" % description_entry)
    newVariable.append(description_entry)
    logDebug("map array=%s" %  variableMapArray)
    logDebug("new symb name=%s new val=%s" % (newSymbolicName, newValue))
    ent = variableMapArray.get(newSymbolicName, None)
    logDebug("Entry from map array %s" % (ent))
    if (ent != None):
        oldValue = ent
        logDebug("variable %s is already defined with value '%s'" % (newSymbolicName, oldValue))
        if (oldValue != newValue):
            print "Modify variable "+newSymbolicName
            argString = u"[-variableName %s -scope Cell=%s,Node=%s,Server=%s ]" % (newSymbolicName, wascell, wasnode, wasserver)
            print "Removing variables ARGS     %s" % (argString)
            AdminTask.removeVariable(argString)
            if newValue.find('"') > -1:
                newValue = newValue.replace('"', '&quot;')
            argString = u"[-variableName %s -scope Cell=%s,Node=%s,Server=%s -variableValue \"%s\" -variableDescription '%s']" % (newSymbolicName, wascell, wasnode, wasserver, newValue, description_entry[1])
            print "Modifying variable ARGS     %s" % (argString)
            AdminTask.setVariable(argString)
            variableMapArray[newSymbolicName] = newValue
        else:
            print "No change for variable "+newSymbolicName

    else:
        if newValue.find('"') > -1:
            newValue = newValue.replace('"', '&quot;')
        print "create new variable "+newSymbolicName
        #argString = u"[-variableName %s -scope Cell=%s,Node=%s,Server=%s -variableValue \"%s\" -variableDescription '%s']" % (newSymbolicName, wascell, wasnode, wasserver, newValue, description_entry[1])
        argString = u"[-variableName %s -variableValue \"%s\" -variableDescription '%s']" % (newSymbolicName, newValue, description_entry[1])
        print "Setting variable ARGS     %s" % (argString)
        AdminTask.setVariable(argString)
        variableMapArray[newSymbolicName] = newValue


    logDebug("configServerVariable() exit")


def configSystemProperties (jvmId, newSysProperties):
    '''
        config
        Procedure configSystemProperties
        Arguments java system properties
    '''
    global AdminConfig
    global description_entry
    global old_am_home
    
    logDebug("configSystemProperties() entry")
    logDebug("JVM ID = %s" % jvmId)
    
    logDebug("System properties to create:" )
    for newSysProperty in newSysProperties:
        logDebug("   %s" % newSysProperty)

    print "Create/Modify JVM system properties"

    oldSysPropertyIds = wsadminToList(AdminConfig.list("Property", jvmId ))
    for newProperty in newSysProperties:
        #print "newProperty=%s" % newProperty
        logDebug("newProperty=%s" % newProperty)
        newProperty.append(description_entry)
        newName = newProperty[0][1]
        newValue = newProperty[1][1]
        #print "new "+newName+" = \""+newValue+"\""
        logDebug("new %s =%s" % (newName,newValue))
        objectId = ""
        for oldPropertyId in oldSysPropertyIds:
            if (oldPropertyId == ""): continue
            oldPropertyName = AdminConfig.showAttribute(oldPropertyId, "name" )
            if (oldPropertyName == "am.home"):
                if old_am_home == None or old_am_home == "":
                    old_am_home = get_am_home(oldPropertyId)
                debug("old_am_home=%s" % old_am_home)
            if (oldPropertyName == newName):
                #print "System property "+oldPropertyName+" exists"
                logDebug("System property "+oldPropertyName+" exists")
                objectId = oldPropertyId
                break


        if (objectId != ""):
            oldValue = AdminConfig.showAttribute(objectId, "value" )
            if (oldValue != newValue):
                print "Modify property "+newName+" from \""+oldValue+"\" to \""+newValue+"\""
                AdminConfig.modify(objectId, newProperty )
            else:
                print "No change for SystemProperty "+newName+" = "+oldValue

        else:
            print "create new property "+newName+" = \""+newValue+"\""
            AdminConfig.create("Property", jvmId, newProperty, "systemProperties" )



    logDebug("configSystemProperties() exit")


def convertToList(inlist):
    '''
       Convert the templates to a list format.
    '''
       
    outlist=[]
    
    print "Template List:  %s " % (inlist)
    if (len(inlist) > 0 and inlist[0] == '[' and inlist[len(inlist)-1] ==']'):
        inlist = inlist[1:len(inlist)-1]
        clist = inlist.split(" ")
    else:
        clist = inlist.split("\n")
        
    for elem in clist:
        elem = elem.rstrip();
        if (len(elem) > 0):
            outlist.append(elem)
            
    return outlist


def displayJvmArgs ( title, jvmargs ):
    '''
       Display JVM arguments.
       
       @param title - Title
       @param jvmargs - JVM args
    '''   
    if (title != ""):
        if (jvmargs == ""):
            print title+": (none)"
            return

        print " "
        print title+":"


    print " "
    hit_flag = 0
    for arg in jvmargs.split( ):
        if (arg == ""):  continue
        hit_flag = 1
        if (arg.find(":") == arg.rfind(":") ):
            print "  " + arg
        else:
            args = arg.split(":" )
            argi = 0  #forStart
            while ( argi < len(args)  ):  #forTest
                print "%d %s" % (argi, args[argi])
                argi += 1  #forNext


    if ( not hit_flag): print "  (none)"

    print " "




def getConfigIdForTemplate(templateName):
    '''
       Check if the template does already exist.
    '''
       
    print "templateName: %s" % (templateName)   
    templates = convertToList(AdminConfig.listTemplates("Server"))
    for template in templates:
        print "Template:  %s" %(template)
        cname = AdminConfig.showAttribute(template, "name")
        print "cname:  %s" % (cname)
        if (cname == templateName):
            return template
    return None


def getServerType(serverTemplate):
    '''
       Check the template server type, and get the template type
       
       @param  serverTemplate - Server template name
    '''
       
    idx = 0
    data = None
       
    tokens = serverTemplate.split("/")
    for token in tokens:
        #
        # serverTemplate is of the form 'itcam(templates/servertypes/APPLICATION_SERVER/servers/itcam|server.xml#Server_1210548827522)'
        # return third token 
        #
        if (idx == 2): 
            data = token
        idx = idx + 1
       
    return data


def process_templates(templatename, progress):
    '''
       Start changing the template adding the DC information to it
       
       @param templatename - Template name
       @param progress - Progress reference
    '''
    
    global platform 
    
    print "process_templates - %s" % (templatename)
    serverTemplates = convertToList(AdminConfig.listTemplates("Server"))
    for serverTemplate in serverTemplates:
        serverType = getServerType(serverTemplate)
        serverTemplateName = AdminConfig.showAttribute(serverTemplate,"name")
        if (serverTemplateName == templatename and serverType == "APPLICATION_SERVER"):   
            print "Found server template - %s" % (serverTemplateName)
            configure(serverTemplate, platform)
            logDebug("Return from configure")
           



def check_templates(templatename, progress):
    '''
       Check the templates and if already exists, skip, otherwise, create it.
       
       @param templatename - Template name
       @param progress - Progress reference
    '''
    
    global platform 
    
    print "creating xd template - %s " % (templatename)
        
    template = getConfigIdForTemplate(templatename)
    progress.write_progress_start(templatename)
    if (template == None):
        if (is_zOS(platform)):
            admin_command = '[-sourceTemplate defaultZOS -destinationTemplate ' + templatename + ' -description "WebSphere XD Default Template"]'
        else:
            admin_command = '[-sourceTemplate default -destinationTemplate ' + templatename + ' -description "WebSphere XD Default Template"]'
        print "admin command: %s" % (admin_command)
        AdminTask.createTemplateFromTemplate(admin_command)
        #template = getConfigIdForTemplate(template)
        threadpools=convertToList(AdminConfig.list("ThreadPool",template))
        for threadpool in threadpools:
            tname = AdminConfig.showAttribute(threadpool,"name")
            if (tname == "WebContainer"):
                AdminConfig.modify(threadpool,[['maximumSize','175']])
    else:
        print "template already exists, skipping"
        msg = resourcebundle.getmsg("XDTEMPLATE_ALEADYEXISTS", "XD template already exists - {0}", [templatename])
        progress.write_progress(msg)
        print "deleting template: " + templatename
        AdminTask.makeNonSystemTemplate("-templateName " + templatename +" -serverType APPLICATION_SERVER")
        AdminTask.deleteServerTemplate(template)
        msg = resourcebundle.getmsg("XDTEMPLATE_DELETED", "XD template deleted - {0}", [templatename])
        progress.write_progress(msg) 
        if (is_zOS(platform)):
            admin_command = '[-sourceTemplate defaultZOS -destinationTemplate ' + templatename + ' -description "WebSphere XD Default Template"]'
        else:
            admin_command = '[-sourceTemplate default -destinationTemplate ' + templatename + ' -description "WebSphere XD Default Template"]'
        print "admin command: %s" % (admin_command)
        AdminTask.createTemplateFromTemplate(admin_command)
        #template = getConfigIdForTemplate(template)
        threadpools=convertToList(AdminConfig.list("ThreadPool",template))
        for threadpool in threadpools:
            tname = AdminConfig.showAttribute(threadpool,"name")
            if (tname == "WebContainer"):
                AdminConfig.modify(threadpool,[['maximumSize','175']])
        

####################################################################################
#
#  Main
#
#
#
####################################################################################

#

print "configtemplatexd_wsadmin running."


progress_obj = None
failed = 0

filename = sys.argv[0]
#
# Initialize logging
#
log.set_logging()
#
# Initialize the resource bundle
#
resourcebundle.set_resourcebundle()
log.logDebug("Resource bundle initialized")

#
# Instantiate the global constants class
#
common_constants = constants()

phase = "config"  

try:

    if (filename != ""):
        print "Input files " + filename
        #
        # Read the input list file, and the reading is done thru the
        # Progress object, as the first record in the list is the progress file 
        # file location.
        #
        inputlist_file_loc = filename
        progress_obj = Progress(common_constants.CONFIGTEMPLATE, inputlist_file_loc)
        inDesc = progress_obj.open_progressfile()
        lines = []
        #
        # Read other records in the input list, which only consists of one record.
        # This record has the template name and the input properties file location
        #
        while 1:
            buffer = inDesc.readline()
            if not buffer: break
            lines.append(buffer)
        inDesc.close()

        loaded_modules = dir()
        if "AdminConfig" in loaded_modules and not "AdminTask" in loaded_modules:
            msg = resourcebundle.getmsg("CFG5046E", "AdminTask object is required to complete the configuration, but is unavailable. Unable to continue.")
            progress_obj.write_progress(msg)
            raise Exception(msg)

        #
        # The input file record has the following format:
        # 1st position = Profile name
        # 2nd position = input list location
        # The entries are separated by a "^".
        #
        input_entries = lines[0].split("^")
        if (len(input_entries) != 2):
            print "input record is invalid - %s" % (lines)
            raise RuntimeError, " input record is invalid - %s\n" %(lines)
        else:
            xdtemplate_osname = input_entries[0]
            inputfile = input_entries[1]
            current_inputproperties = inputfile
            print "XD template os: %s  inputfile: %s" % (xdtemplate_osname, inputfile)
        #    
        # Read in the input properties file data.
        #
        try:
            inputfile = inputfile.strip()
            inDesc = codecs.open(inputfile, mode='r', encoding='utf-8')
        except Exception, msgVar:
            errorCode = None
            print "Faild to open - %s" % (msgVar)
            raise RuntimeError, inputfile + " failed to open\n"
    
        lines = []
        while 1:
            buffer = inDesc.readline()
            if not buffer: break
            lines.append(buffer)
        inDesc.close()
        logDebug("%d lines read from %s" % (len(lines),inputfile))    
        #
        # Validate the input.properties data
        #
        validate(lines, progress_obj)
        #
        # Check the xd template about to be created.
        #
        templatename = xdtemplate_osname + "_itcam"
        check_templates(templatename, progress_obj)
        #
        # Start changing server.xml
        #
        process_templates(templatename, progress_obj)
        progress_obj.write_progress_finish(templatename, common_constants.SUCCESSFUL)
        
         

    #
    # Close the progress file
    #
    if (progress_obj != None):
        progress_obj.close_file()
        progress_obj = None

    #
    # If processing each server caused an error, then a message is issued, and the exception
    # is raised, so the wsadmin gets a non zero return code.
    #
    if (failed == 1):
        log.logDebug("Configuring templates had a failure")
        raise Exception("Configuring templates had a failure")
    
    
    
# This needs to pick up the sys.exit(1) and exceptions     
except:
    log.logException(SCRIPTNAME+" ends abnormally\n ")
    traceback_report = log.format_traceback()
     
    #
    # If progress object is present, put a message in progress file to be displayed for the wsadmin caller.
    #
    if (progress_obj is not None):
        msg = resourcebundle.getmsg("XDTEMPLATE_ERROR", "Error occurred while processing XD template creation")
        if len(traceback_report) > 0:
            msg = msg + "\n" + traceback_report
        progress_obj.write_progress(msg)
        progress_obj.write_progress_finish(common_constants.UNSUCCESSFUL)
        progress_obj.close_file()

    etype, value, tbck = sys.exc_info()
    raise Exception(value)


# When no errors, exit gracefully to reset 
# any error status previously stored by wsadmin
sys.exit(0)
 
    
