####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
  This module configures TPV.
'''

fixlev = "39451" # Last APAR, defect or story to update this module

import doprompt
import utils

from resourcebundle import getmsg
from debugUtil import logDebug as debug
import debugUtil


def config_tpv(pdl, soa71_migrate = False):
    '''
       Prompts the user for TPV configuration options.

       @param pdl - PDL reference
       @param soa71_migrate - Ture if soa71 migration processing otherwise False
    '''

    debug("Entry config_tpv %s" % fixlev)

    # If this is as the result of migrate call for soa71, then set the pdl
    # without asking question.

    portal_servers = ""
    portal_indexes = []
    (only_wlp_selected, wlp_servers) = utils.get_wlp_selected(pdl)

    if soa71_migrate and not pdl["SILENT"][0]:
        utils.setData(pdl, "TPV", "0", "configtpv")
    else:
        serverlist = utils.getData(pdl, "SERVERS")
        for serveri in range(len(serverlist)):
            wpshome = utils.getData(pdl, "WPSHOME")
            if wpshome is not None:
                wpshome = utils.getData(pdl, "WPSHOME")[serveri]
                if wpshome != None and wpshome != "None" and len(wpshome) > 0:
                    portal_indexes.append(serveri)
                    if len(portal_servers) > 0: portal_servers += ", "
                    portal_servers += (utils.getData(pdl, "WASSERVER")[serveri])
        only_portal_selected = len(portal_indexes) == len(serverlist)
		
        utils.setData(pdl, "TPV", "0", "configtpv")
        # only_portal_and_liberty_selected = len(portal_indexes) + len(wlp_servers) == len(serverlist)
        # if not only_portal_and_liberty_selected:
        #    print "\n%s" % getmsg("FLOWERBOX_TPV","Optional integration with ITCAM for TPV")
        #    if len(portal_servers) > 0:
        #        print "%s" % getmsg("CFG1160W", "TPV Integration is not supported for the portal server {0}", [portal_servers])
        #    if len(wlp_servers) > 0:
        #        print "%s" % getmsg("CFG1171W", "TPV Integration is not supported for WebSphere Liberty Profile")
        #    doprompt.do_prompt("TPV", pdl)

    # Make the pdls for the SOA as a list for the number of
    # servers in SERVERS pdl.

    tpv = utils.getDataString(pdl, "TPV")
    serverlist = utils.getData(pdl, "SERVERS")
    utils.setData(pdl, "TPVLIST", [], "configtpv")
    for serveri in range(len(serverlist)):
        if serveri in portal_indexes or serveri in wlp_servers:
            utils.getData(pdl, "TPVLIST").append("0");
        else:
            utils.getData(pdl, "TPVLIST").append(tpv);

    debug("Exit config_tpv")


def set_defaultvalues(pdl):
    '''
       Sets the default values in the pdl.  The first selected server's values
       are used to set the default.

       @param pdl - PDL reference
    '''

    debug("Entry set_defaultvalues %s" % fixlev)

    # set default tpv values.

    tpv = utils.getData(pdl, "TPVLIST", 0)
    utils.setData(pdl, "TPV", tpv, "configtpv")
    debug("Values set are  TPV: %s  TPVLIST: %s" % (repr(pdl["TPV"][0:2]), repr(pdl["TPVLIST"][0:2])))

    debug("Exit set_defaultvalues")
