####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
    Configure Transaction Tracking (TTAPI)
'''

######################################################################
#
# TTAPI     - enable TTAPI   true | false
#
# TTAPIDIR  - Transactions Tracking zOS directory  - zOS only
# TTAPISSN  - Transactions Tracking subsystem name - zOS only
#
# TTTCHOST  - TT Collector server host/ip_address  - distributed only
# TTTCPORT  - TT Collector server port             - distributed only
#
# In zOS, these are set here and used by input.properties
# to set LIBPATH and CLASSPATH.
#   TTAPILIBPATH
#   TTAPICLASSPATH
#
# All TTAPI configuration options have defaults.
#
######################################################################

import sys
import os

from java.lang import System

import doprompt
import platformUtil
import utils
import configUtils
import debugUtil
import createPDL
import shared_globals as globals
import debugUtil as log

from resourcebundle import getmsg
from debugUtil import logDebug as debug


def config_tt(pdl, soa71_migrate = False):
    '''
       Prompts the user for TTAPI configuration options.
       @param pdl - PDL reference
       @param soa71_migrate - Ture if soa71 migration processing otherwise False
    '''
    continue_config = True
    nonconfig_server_list = []
    error_msg = None
    try:
        debug("Configuring TTAPI - config_tt entry ")
        
        xdtemp = utils.getData(pdl, "CREATETEMPLATE")
        (only_wlp_selected, wlp_servers) = utils.get_wlp_selected(pdl)
        
        #For distributed platforms, check if ttapi library path exists.
        #TTAPI library path is like CONFIGHOME/toolkit/lib/<interp>/ttapi
        display_ttapi_prompt = _check_ttapi_path(pdl)
        
        ##################################################
        # Prompt / get option values that apply to all
        # servers in this config loop
        ##################################################
        utils.setData(pdl, "TTAPILIST", [], "configtt")
        if display_ttapi_prompt:
            #
            # If this is as the result of migrate call for soa71, then set the pdl
            # without asking question.
            #
            if ((soa71_migrate) and (not pdl["SILENT"][0])):
                utils.setData(pdl, "TTAPI", "0", "configtt")
            else:
                if platformUtil.IS_zOS_OS():
                    print getmsg("FLOWERBOX_CONFIGTT_ZOS", """\
                                    +----------------------------------------------------------------+
                                    |      WebSphere Application Server integration with             |
                                    |            Transaction tracking (TT)                           |
                                    |                                                                |
                                    | If you plan to integrate the WebSphere Application Server data |
                                    | collector with Transaction tracking, you must have installed   |
                                    | Transaction tracking.                                          |
                                    | On z/OS this requires a separate installation of ITCAM for     |
                                    | Transactions on the same host as WebSphere Application Server. |
                                    | You will need to supply the subsystem name and directory       |
                                    | path of the Transaction tracking container.                    |
                                    +----------------------------------------------------------------+""")
                    print
                else:
                    print getmsg("FLOWERBOX_CONFIGTT", """\
                                    +----------------------------------------------------------------+
                                    |      WebSphere Application Server integration with             |
                                    |            Transaction tracking (TT)                           |
                                    |                                                                |
                                    | If you plan to integrate the WebSphere Application Server data |
                                    | collector with Transaction tracking, you must have installed   |
                                    | Transaction tracking.                                          |
                                    +----------------------------------------------------------------+""")
                doprompt.do_prompt("TTAPI", pdl)
                
            tt_answer = utils.getDataString(pdl, "TTAPI")
            debug("  TTAPI: %s %s" % (tt_answer, type(tt_answer)) )
    
            sn_answer = utils.getData(pdl, "TTAPISSN")
            dir_answer = utils.getData(pdl, "TTAPIDIR")
            tchost_answer = utils.getData(pdl, "TTTCHOST")
            tcport_answer = utils.getData(pdl, "TTTCPORT")

            if tt_answer is True or tt_answer == "1":

                if platformUtil.IS_zOS_OS():
                    doprompt.do_prompt("TTAPISSN", pdl)
                    sn_answer = utils.getData(pdl, "TTAPISSN")
                    doprompt.do_prompt("TTAPIDIR", pdl)
                    dir_answer = utils.getData(pdl, "TTAPIDIR")
                    #assert is not really needed, checking should be done by doprompt
                    assert dir_answer != None and dir_answer != "", getmsg("CFG5017E", "Data element {0} of PDL has no entries", ["TTAPIDIR"])
                    #debug("  SSN: %s, DIR: %s" % (sn_answer, dir_answer))
                else:
                    log.logDumpPDL(pdl, "PDL dump in configtt")
                    doprompt.do_prompt("TTTCHOST", pdl)
                    tchost_answer = utils.getData(pdl, "TTTCHOST")
                    doprompt.do_prompt("TTTCPORT", pdl)
                    tcport_answer = utils.getData(pdl, "TTTCPORT")
                    debug("  TCHOST: %s, TCPORT: %s" % (tchost_answer , tcport_answer))
            #
            # Make the pdls for the TT as a list for the number of
            # servers in SERVERS pdl.
            #
            serverlist = utils.getData(pdl, "SERVERS")
            utils.setData(pdl, "TTAPISSNLIST", [], "configtt")
            utils.setData(pdl, "TTAPIDIRLIST", [], "configtt")
            utils.setData(pdl, "TTTCHOSTLIST", [], "configtt")
            utils.setData(pdl, "TTTCPORTLIST", [], "configtt")
    
            for serveri in range(len(serverlist)):
                utils.getData(pdl, "TTAPILIST").append(tt_answer)
                if platformUtil.IS_zOS_OS():
                    utils.getData(pdl, "TTAPISSNLIST").append(sn_answer)
                    utils.getData(pdl, "TTAPIDIRLIST").append(dir_answer)
                    utils.getData(pdl, "TTTCHOSTLIST").append(None)
                    utils.getData(pdl, "TTTCPORTLIST").append(None)
                else:
                    apmoffering = utils.getData(pdl, "IOFFERING")
                    debug("  product offering: %s " % (apmoffering))                                        
                    
                    if (apmoffering == "IAPM"  or apmoffering == "APMADV"):
                        # add tt.enabled settings 
                        ttconfigFileNameSuffix = utils.getServerBasedFilePrefix(pdl, serveri)
                        ttconfigFilePath = pdl["CONFIGHOME"][0] + os.sep + "runtime" + os.sep + "custom"
                        ttconfigFileName = ttconfigFilePath + os.sep + "ttdc.enabled." + ttconfigFileNameSuffix
                        ttconfigFileName = ttconfigFileName[:-1]
                        debug("ttenableFileNameSuffix = %s " % ttconfigFileNameSuffix)
                        debug("ttenableFileName = %s " % ttconfigFileName)
                        
                        if not os.path.exists(ttconfigFilePath):
                            debug(ttconfigFilePath + " path does not exist")
                            try:
                                os.makedirs(ttconfigFilePath)
                                debug(ttconfigFilePath + " path created")
                            except Exception, msgVar:
                                debug("An error occurred while creating %s path. Error details - %s" % (ttconfigFilePath, msgVar))
                                
                        if not os.path.isfile(ttconfigFileName):
                            debug(ttconfigFileName + " file does not exist")
                            try:
                                open(ttconfigFileName, 'a').close()
                                debug(ttconfigFileName + " file created")
                            except Exception, msgVar:
                                debug("An error occurred while creating %s file. Error details - %s" % (ttconfigFileName, msgVar))
                                                                                                                                       
                    utils.getData(pdl, "TTTCHOSTLIST").append(tchost_answer)
                    utils.getData(pdl, "TTTCPORTLIST").append(tcport_answer)
                    utils.getData(pdl, "TTAPISSNLIST").append(None)
                    utils.getData(pdl, "TTAPIDIRLIST").append(None)
    
            #################################################################
            # Special handling for z/OS only.
            # 1. Set TTAPILIBPATH and TTAPICLASSPATH even if TTAPI not enabled
            # 2. Update path variables
            # 3. Update directories.. crearting symlinks or directories
            # Refs: TTAPISSNLIST, TTAPIDIRLIST
            # Sets: TTAPICLASSPATH, TTAPILIBPATH
            #################################################################
    
            if  platformUtil.IS_zOS_OS():
                continue_config, error_msg, nonconfig_server_list = config_tt_zOS(pdl)
        else: 
            # print "%s" % getmsg("CFG1126I", "TTAPI is not supported on this platform.")
            # print "\n"
            #
            # Make the pdls for the TT as a list for the number of
            # servers in SERVERS pdl.
            #
            serverlist = utils.getData(pdl, "SERVERS")
            utils.setData(pdl, "TTAPISSNLIST", [], "configtt")
            utils.setData(pdl, "TTAPIDIRLIST", [], "configtt")
            utils.setData(pdl, "TTTCHOSTLIST", [], "configtt")
            utils.setData(pdl, "TTTCPORTLIST", [], "configtt")
    
            for serveri in range(len(serverlist)):
                utils.getData(pdl, "TTAPILIST").append("0");
                utils.getData(pdl, "TTAPISSNLIST").append(None);
                utils.getData(pdl, "TTAPIDIRLIST").append(None);
                utils.getData(pdl, "TTTCHOSTLIST").append(None);
                utils.getData(pdl, "TTTCPORTLIST").append(None);
    
    except:
        debug("Error occurred while setting the TTAPI data")
        raise

    if not continue_config:
        print "%s" % getmsg("CFG1125E", "TTAPI Configuration failed for servers {0}", [", ".join(nonconfig_server_list)])
        if error_msg != None: print "%s" % error_msg
        doprompt.do_prompt("CONTINUECONFIG", pdl, alt_prompt="CFG2013I Would you like to continue with config?")
        if (pdl["CONTINUECONFIG"][0] == False): continue_config = False
        else: continue_config = True

    debug("Configuring TTAPI - config_tt exit ")
    return continue_config

# For non z/OS platforms, check if CONFIGHOME/toolkit/lib/<interp>/ttapi exists.
def _check_ttapi_path(pdl):
    
        
    if platformUtil.IS_zOS_OS():
        return True
    
    
    support_ttapi = True
    confighome = pdl["CONFIGHOME"][0]
    serverlist = utils.getData(pdl, "SERVERS")
    nottapi_serverlist = []
    #Loop through each application server and find its jvmmode.
    #This for loop probably is only useful in 64-bit OS.
    #As for 64-bit platform, both 64-bit and 32-bit JVMs can work 
    ttapi_lib_path = ""
    for serveri in range(len(serverlist)):
        is_wlp_server = False
        wlphome = utils.getData(pdl, "WLPHOME")[serveri]
        xdtemp = utils.getData(pdl, "CREATETEMPLATE")
        if (xdtemp == True):
            interp = utils.getDataString(pdl, "TEMPLATEINTERP")
        else:
            jvmmode = utils.getData(pdl,"JVMMODE")[serveri]
            interp = platformUtil.getToolkitArch(confighome, jvmmode)
            
        ttapi_lib_path = confighome + "/toolkit/lib/" + interp + "/ttapi"
        debug("ttapi_lib_path: " + ttapi_lib_path)
        if not os.path.exists(ttapi_lib_path):
            server_name = utils.getServerName(pdl, serveri)
            nottapi_serverlist.append(server_name)
            #args = []
            #args.append(server_name)
            #args.append(ttapi_lib_path)
            #nottapi_msg = getmsg("CFG1127I", "For {0}, TTAPI library path: {1} does not exist", args)
            #print nottapi_msg
            debug("For " + server_name + ", TTAPI library path: " + ttapi_lib_path + " does not exist or server is WLP type")
    
    if len(nottapi_serverlist) == len(serverlist):
        support_ttapi = False
        
    return support_ttapi

################################################
# Configure TT for zOS
# Assumes all prompting and data input has been completed
# 1. Set TTAPILIBPATH and TTAPICLASSPATH even if TTAPI not enabled
# 2. Update path variables
# 3. Update configuration directories.
# Input: TTAPISSNLIST, TTAPIDIR
################################################

def config_tt_zOS(pdl):
    '''
        Configure TTAPI for zOS
          1. Set TTAPILIBPATH and TTAPICLASSPATH even if TTAPI not enabled
          2. Update path variables
          3. Update configuration directories.
        @param pdl  reference to PDL
    '''

    debug("config_tt_zOS - entry" )
    continue_config = True
    error_msg = None
    nonconfig_server_list = []

    # 1. Prime the PDL for our path variables if not already set by migrate
    # TODO: do a test of type and length before bypassing the init of these variables..
    migrate_mode = utils.getData(pdl, "MIGRATE")
    debug("  MIGRATE in pdl: %s %s" % (migrate_mode, type(migrate_mode)))
    if (migrate_mode is None) or (migrate_mode == '0') or (migrate_mode == 0):
        varlist = [ "TTAPICLASSPATH", "TTAPILIBPATH", ]
        init_pdl(pdl, varlist)
        debug("  ..config mode" )
    else:
        debug("  ..migration mode" )

    # 2. For each server
    CONFIGHOME = utils.getData(pdl, "CONFIGHOME")
    nonconfig_server_list = pdl["SERVERS"][0][:]    #make a copy of the original servers list
    for serveri in range(len(pdl["SERVERS"][0])):
        WASNODE    =  utils.getData(pdl, "WASNODE",     serveri)
        SERVER     =  utils.getData(pdl, "SERVERS",     serveri)
        libdir     =  "%s/%s/%s.%s/lib" % (CONFIGHOME, globals.CONFIGHOME_NODE_SERVER_ROOT, WASNODE, SERVER)

        # 1. Verify config lib directory exists
        #      <CONFIGHOME>/[node].[server]/lib
        rc = verify_dir_tree(libdir)
        if rc == False:
            error_msg = getmsg("CFG5021E", "Failed to create {0}", [libdir])
            continue_config = False
            break
        # 2. Update pdl path variables TTAPILIBPATH and TTAPICLASSPATH
        update_path_variables(pdl, CONFIGHOME, libdir, serveri)

        # 3. Update directory - independent of TTAPI enabled, exit from the loop on error
        confinue_config, error_msg = update_directory(pdl, serveri, libdir)
        if not confinue_config: break
        #removing server from the list if no error
        nonconfig_server_list.remove(SERVER)

    debug("config_tt_zOS - exit")
    return continue_config, error_msg, nonconfig_server_list


###############################################################################################
# Update path related PDL variables TTAPILIBPATH and TTAPICLASSPATH
# independent of TTAPI being enabled
#
#     TTAPICLASSPATH  =   ${ITCAMDCHOME}/<node>.<server>/lib/ttapi/cytapi4j.jar
#     TTAPILIBPATH    =   ${ITCAMDCHOME}/<node>.<server>/lib/ttapi
#
###############################################################################################

def update_path_variables(pdl, CONFIGHOME, libdir, serveri):
    '''
        set_path_variables(pdl, serveri)
        @param libdir in format  <CONFIGHOME>/[node].[server]/lib
    '''
    debug("update_path_variables entry ")
    server = pdl["SERVERS"][0][serveri]
    debug("   Updating path variables for server: %s"  % server )

    upv_libdir = libdir.replace(CONFIGHOME, "${ITCAMDCHOME}")
    TTAPICLASSPATH = upv_libdir + "/ttapi/cytapi4j.jar"
    TTAPILIBPATH   = upv_libdir + "/ttapi"

    # If TTAPICLASSPATH and TTAPILIBPATH haven't yet been set, prepare them as lists

    # This works because the pdl variables have been initialized earlier
    utils.setData(pdl,"TTAPICLASSPATH", TTAPICLASSPATH, "configtt", serveri)
    utils.setData(pdl,"TTAPILIBPATH",   TTAPILIBPATH,   "configtt", serveri)

    debug("   TTAPICLASSPATH set to: %s" % (TTAPICLASSPATH) )
    debug("   TTAPILIBPATH   set to: %s" % (TTAPILIBPATH) )
    debug("update_path_variables exit for server: %s" % (server) )


######################################################################
# Update the config_home ttapi directory
#    ${ITCAMDCHOME}/<node>.<server>/lib/ttapi
#
# If TTAPI enabled,  create dir "ttapi" as a aymbolic link
#             else,  we do not create a directory entry
#
# Assumptions:
#     The lib directory exists ${ITCAMDCHOME}/<node>.<server>/lib
# Input:
#     ttapipath in format: ${ITCAMDCHOME}/<node>.<server>/lib/ttapi

######################################################################

def update_directory(pdl, serveri, libdir):
    '''
        update_directory(pdl, serveri)
    '''
    debug("update_directory entry ")
    continue_config = True
    error_msg = None
    server = pdl["SERVERS"][0][serveri]
    debug("   update_directory for server: %s using %s " % (server, libdir) )

    TTAPI         = utils.getData(pdl, "TTAPI")
    ttapi_dirpath  = libdir + "/ttapi"

    debug(" TTAPI option and ttapi_dirpath: %s - %s"  % (str(TTAPI), ttapi_dirpath) )

    # 1. If a path is built thru the "ttapi" name, clean by deleting the "ttapi" level.
    #    We expect it to be a symlink. In earlier editions, "ttapi" could be a directory.
    #    Do not use os.remove if the path is a directory.
    result = test_and_delete_path(ttapi_dirpath)
    if result == False:
        error_msg = getmsg("CFG1116E", "Unable to remove {0} symlink {1}", ["TTAPI", ttapi_dirpath])
        continue_config = False
    # 2. Create the "ttapi" part of this path
    #    If TTAPI enabled: create a symlink named "ttapi"
    #    else, we create nothing.
    if (TTAPI == True or TTAPI == '1' )  and continue_config:
       debug("    TTAPI enabled.. creating symlink")
       TTAPIDIR  = utils.getData(pdl, "TTAPIDIR")

       cmd='ln -s '+TTAPIDIR+' '+ttapi_dirpath
       rc=configUtils.issueOSCommand(cmd)
       debug("    command and rc: %s rc= %s"  % (cmd, str(rc)))
       if not configUtils.isLink(ttapi_dirpath):
           continue_config = False
           error_msg = getmsg("CFG1123E", "Failed to create {0} symlink {1}", ["TTAPI", ttapi_dirpath] )
       else:
           debug("    ttapi symlink created: %s"  % (ttapi_dirpath) )
           debug("      to %s"   % (TTAPIDIR) )
           debug("   update_directory complete for server: %s" % (server) )
    else:
       # TTAPI is disabled: do not create a "ttapi" entry of any kind
       debug("    TTAPI disabled - no ttapi directory entry created")
    debug("update_directory exit ")    
    return continue_config, error_msg


###
# Test and delete the path.
###
def test_and_delete_path(path):
    '''
        Test existance of and delete the path
        @param   path
        @returns 0 if successful; 8 if not
    '''

    debug("test_and_delete_path entry - %s" % path  )
    if configUtils.isLink(path) or os.path.isfile(path):
        debug("   symlink or file exists")
        try:
            os.remove(path)
            debug("    removed")
            return True
        except Exception, msgVar:
            debugUtil.logException("  os.remove failed")
            return False
    else:
        if os.path.isdir(path):
            try:
                os.rmdir(path)
                debug("    directory removed")
                debug("update_directory exit ")
                return True
            except:
                debugUtil.logException("  os.rmdir failed")
                return False
        else:
            debug("  path does not exist")
            debug("update_directory exit ")
            return True


###
# Verify existance of a directory tree.  If not create it.
# Mode using integer of 775 not generating desired results.
# By default with no mode, permissions set to 777.
# TODO: test 2.5.2 using mode as a string.. which didn't work in Jython 2.1
# So call without 'mode' operand specified.
###

def verify_dir_tree(libdir, mode=None):
    '''
        verify_dir_tree(libdir, mode=None):
    '''
    debug("verify_dir_tree entry - %s " % str(libdir))
    rtn = True
    if not os.path.isdir(libdir):
        debug("Directory " +  libdir + " does not exist, will attempt to create.")
        try:
            if mode is None:
                os.makedirs(libdir)
            else:
                os.makedirs(libdir, mode)
            if os.path.isdir(libdir):
                rtn = True
            else:
                rtn = False
        except:
            debugUtil.logException("Cannot create: %s" % libdir )
            debug("....Unable to create it %s" % libdir)
            rtn = False
    else:
        rtn = True
    return rtn
    
    debug("verify_dir_tree exit - %s" % rtn )

# Initialize the TTAPI variables in the PDL
def init_pdl(pdl, varlist):
    '''
        init_pdl(pdl, varlist)
        Initialize the pdl for configtt if not already done
    '''
    debug("  Init configtt pdl for: %s" % (str(varlist)) )
    for name in varlist:
        # If type not set to type([]), initialize it
        if  type(pdl[name][00]) != type([]) or type(pdl[name][00]) != type(()):
            pdl[name][0]=[]            # Initialize as a list
            pdl[name][1]="configtt"
            for serveri in range(len(pdl["SERVERS"][0])):
                pdl[name][0].append("")
    debug("  Init configtt pdl complete")
    
    
def set_defaultvalues(pdl):
    '''
       Sets the default values in the pdl.  The first selected server's values
       are used to set the default.
       
       @param pdl - PDL reference  
    '''   
    
    debug("Entry set_defaultvalues")    
    
    #
    # set default tema values.
    #
    tt_answer = utils.getDataString(pdl, "TTAPILIST", 0)
    utils.setData(pdl, "TTAPI", tt_answer, "configtt")
    if platformUtil.IS_zOS_OS():
        TTssn = utils.getData(pdl, "TTAPISSNLIST", 0)
        TTdir = utils.getData(pdl, "TTAPIDIRLIST", 0)
        debug("TTssn is: %s   TTdir is: %s" % (TTssn, TTdir))
        utils.setData(pdl, "TTAPISSN", TTssn, "configtt")
        utils.setData(pdl, "TTAPIDIR", TTdir, "configtt")
    else:
        TThost = utils.getData(pdl, "TTTCHOSTLIST", 0)
        TTport = utils.getData(pdl, "TTTCPORTLIST", 0)
        debug("TThost is: %s   TTport is: %s" % (TThost, TTport))
        utils.setData(pdl, "TTTCHOST", TThost, "configtt")
        utils.setData(pdl, "TTTCPORT", TTport, "configtt")
        
        
    debug("Exit set_defaultvalues")    
