#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

fixlev = "40296" # Last APAR, defect or story to update this module

import os
import codecs
import urllib

import debugUtil
import utils
import platformUtil
from resourcebundle import getmsg
from ConfigurationException import ConfigurationException

#--------------------------------------------------------------------
# Set global constants
#--------------------------------------------------------------------

_TEMPLATE_FILENAME_ = "_Template_DCManualInput.txt"


#######################################################################
#This method will create $x$DCManualInput.txt for each of the servers.
#######################################################################

def createFile(pdl,fileCreationDir):
    #creating PDL entry for ITCAMDCINPUT
    utils.setData(pdl, "ITCAMDCINPUT", [], "createDCInput")
    mshome = None

    interp = pdl["TEMPLATEINTERP"][0]

    #Loop through servers in pdl["SERVERS"] and create all DC Manual Inputs files
    for serveri in range(len(pdl["SERVERS"][0])):

        dcManualInputFile = _TEMPLATE_FILENAME_
        dcManualInputFile = fileCreationDir + "/" + interp + _TEMPLATE_FILENAME_

        debugUtil.logDebug("Creating dcManualInputFile: " + dcManualInputFile)

        #if not there: this file is created
        #if there: its emptied out
        try:
            f = None
            platform = pdl["TEMPLATEOS"][0]
            if platform is "zos":
                f = codecs.open(dcManualInputFile, mode="w", encoding="cp1047")
                os.system("chmod 775 " + dcManualInputFile)
            else:
                f = codecs.open(dcManualInputFile, "w", "utf-8")

            f.write("#######################################################\n")
            f.write("# This file is machine generated; Do not edit or delete; \n")
            f.write("#######################################################\n")
            f.write("local.hostname=127.0.0.1" +"\n")
            f.write("was.profile.home=none"+ "\n")
            f.write("was.version=none" + "\n")
            f.write("itcam.home=" + pdl["TEMPLATEDCHOME"][0] +"\n")
            f.write("was.nodename=none" + "\n")
            f.write("was.profilename=none" + "\n")
            f.write("was.servername=none" + "\n")
            f.write("kwj.serveralias=none" + "\n")

            wr,ms,tt,soa,tpv,de = "0","0","0","0","0","0"

            if utils.getData(pdl, "TPVLIST", serveri) == "1":
                tpv = "1"
            if utils.getData(pdl, "DELIST", serveri) == "1":
                de = "1"
            if utils.getData(pdl, "TEMACONNECTLIST", serveri) == "1" and pdl["TEMADCAPPSERVER"][0]:
                wr="1"
                f.write("tema.host="+                    pdl["TEMAHOSTLIST"][0][serveri] +"\n")
                f.write("tema.port="+                    pdl["TEMAPORTLIST"][0][serveri] +"\n")
            if utils.getData(pdl, "MSCONNECTLIST", serveri) == "1":
                ms="1"
                f.write("ms.hostname="+                  pdl["KERNELIPLIST"][0][serveri] +"\n")
                f.write("kernel.codebase.port="+         pdl["KERNELPORT01LIST"][0][serveri] +"\n")
                f.write("kernel.rfs.port="+              pdl["RFSPORT01"][0] +"\n")
                f.write("probe.controller.rmi.port="+    pdl["PROBECONTROLLERRMIPORTLIST"][0][serveri] +"\n")
                f.write("probe.rmi.port="+               pdl["PROBERMIPORTLIST"][0][serveri] +"\n")
                mshome = urllib.quote(pdl["MSHOMELIST"][0][serveri],"\\/:").replace("\\","/")
                f.write("ms.home="+                      urllib.quote(pdl["MSHOMELIST"][0][serveri],"\\/:").replace("\\","/")  +"\n")
            if len(pdl["TTAPILIST"][0]) > 0:
                if utils.getData(pdl, "TTAPILIST", serveri) == "1":
                    tt="1"

            if platformUtil.getOSFamily() != "OS400":
                if utils.getData(pdl, "SOADCLIST", serveri) == "1": soa="1"

            #opmode = operation_mode[wr+ms]
            opmode = ""
            if wr == "1":  opmode = "WR,HC,"
            if ms == "1":  opmode = opmode + "MS,"
            if tt == "1":  opmode = opmode + "TT,"
            if soa == "1": opmode = opmode + "SOA,"
            if tpv == "1": opmode = opmode + "ECAM,"
            if de == "1": opmode = opmode + "DE,"

            #get rid of the last comma
            opmode=opmode.rstrip(",")

            f.write("am.camtoolkit.gpe.dc.operation.mode=" + opmode+ "\n")

            if platform is not "zos":
                jmode = pdl["TEMPLATEINTERPLIST"][0][serveri]
                f.write("interp=" +  jmode + "\n")

            if tt == "1":
                if platform is "zos":
                    ssn = pdl["TTAPISSNLIST"][0][serveri]
                    f.write("tt.connection.string=ssn:" + ssn +  "\n")
                else:
                    host=pdl["TTTCHOSTLIST"][0][serveri]
                    port=pdl["TTTCPORTLIST"][0][serveri]
                    f.write("tt.connection.string=tcp:" + host + ":" + port + "\n")

            #f.write("com.ibm.tivoli.itcam.dc.ttapi.portal=false \n")
            #endif
            f.write("***************************************** \n")
            f.write("PLEASE DO NOT CHANGE CODE BELOW THIS LINE \n")
            f.write("***************************************** \n")
            f.write("APPSERVER_PLATFORM=@{appserver.platform}\n")
            f.write("DC_OPER_MODE=@{am.camtoolkit.gpe.dc.operation.mode}\n")
            f.write("ITCAMDCHOME=@{itcam.home}\n")
            f.write("KERNEL_HOST01=@{ms.hostname}\n")
            f.write("MS_AM_HOME=@{ms.home}\n")
            f.write("RT_WAS_NODE_NAME=@{was.nodename}\n")
            f.write("RT_WAS_SERVER_NAME=@{was.servername}\n")
            f.write("WAS_BASE_SHORT_VERSION=@{was.version}\n")
            #f.write("WS_HOST=\n")
            f.write("STDOUT=@{was.profile.home}/@{RT_WAS_NODE_NAME}/native_stdout.log\n")
            f.write("AM_HOME=@{ITCAMDCHOME}/itcamdc\n")
            f.write("AM_SOCKET_BINDIP=@{local.hostname}\n")
            f.write("AM_SOCKET_EXPORTIP=@{local.hostname}\n")
            #f.write("DMGR_RMI=\n")
            #it really does not matter if MS is selected or not.
            #MSHOMELIST always has a default value and that is good enough
            if mshome == None:
                debugUtil.logDebug("MS was not selected")
                f.write("FILESEP=//\n")
            elif mshome.startswith("/"):
                debugUtil.logDebug("MS Platform is detected to be Unix")
                f.write("FILESEP=//\n")
            else:
                debugUtil.logDebug("MS Platform is detected to be Windows")
                f.write("FILESEP=/\n")
            #if (platformUtil.IS_WINDOWS_OS()):
            #    f.write("FILESEP=/\n")
            #else:
            #    f.write("FILESEP=//\n")
            f.write("ITCAM_ENABLED=enable\n")
            f.write("LOG_COMMON_DIR=@{ITCAMDCHOME}/logs\n")
            f.write("PORT_KERNEL_CODEBASE01=@{kernel.codebase.port}\n")
            f.write("PORT_KERNEL_RFS01=@{kernel.rfs.port}\n")
            f.write("PROBE_CONTROLLER_RMI_PORT=@{probe.controller.rmi.port}\n")
            f.write("PROBE_RMI_PORT=@{probe.rmi.port}\n")
            f.write("PROFILE=@{was.profilename}\n")
            f.write("RUNTIME_DIR=@{ITCAMDCHOME}/runtime/@{APPSERVER_PLATFORM}.@{RT_WAS_NODE_NAME}.@{RT_WAS_SERVER_NAME}\n")
            f.write("TEMA_HOST=@{tema.host}\n")
            f.write("TEMA_PORT=@{tema.port}\n")
            f.write("TIVOLI_COMMON_DIR=@{ITCAMDCHOME}/logs\n")
            f.write("TT_CONNECTION_STRING=@{tt.connection.string}\n")
            if platform is not "zos":
                f.write("TOOLKIT_LIB_PATH=@{ITCAMDCHOME}/toolkit/lib/@{interp}:@{ITCAMDCHOME}/toolkit/lib/@{interp}/ttapi\n")
            else:
                confighome    = pdl["CONFIGHOME"][0]
                libpathzos    = pdl["ZOSLIBPATH"][0][serveri]
                libpathzos_tt = pdl["TTAPILIBPATH"][0][serveri]
                zos_libpath   = get_zos_libpath(confighome, libpathzos, libpathzos_tt )
                f.write("TOOLKIT_LIB_PATH="+zos_libpath+"\n")
            #f.write("WS_RMI_PORT=\n")
            f.write("alias1=@{kwj.serveralias}\n")
            f.write("gclog=@{temagclog.path}\n")


        finally:
            if f != None: f.close()

        utils.getData(pdl,"ITCAMDCINPUT").append(dcManualInputFile)

###
#  get_zos_libpath using the input.properties.template,
#  and zos and TT libpath variables
###
def get_zos_libpath(confighome, libpathzos, libpathzos_tt):
    '''
        get_zos_libpath(confighome, libpathzos, libpathzos_tt)
    '''
    debugUtil.logDebug("  get_zos_libpath for %s, %s, %s" %(confighome, libpathzos, libpathzos_tt) )
    libpath_rec  = get_ipt_entry("server.environment=@{LIBPATH}", confighome)
    zos_libpath  = translate_zos_libpath(libpath_rec, libpathzos, libpathzos_tt)
    debugUtil.logDebug("    %s" % zos_libpath)
    return zos_libpath


#######################################################################
# get_ipt_entry(key, confighome) for zOS
# This method reads the input.properties.template file(ebcdic on z/OS),
# locates, and returns the value for the specified entry(key).
#######################################################################
def get_ipt_entry(key, confighome):
    '''
        get_ipt_entry(key, confighome)
    '''
    ipt_filepath = confighome+"/itcamdc/etc/input.properties.template"
    f = codecs.open(ipt_filepath, mode="r", encoding="cp1047")
    ip_lines = f.readlines()
    f.close()
    for line in ip_lines:
        if line.startswith("#@COMMENT@# "):
            line = line[len("#@COMMENT@# "):len(line)]
            line = line.strip()
            if line.startswith(key+"="):
                value=line[len(key+"="):len(line)]
                return value
    assert 1,"No input.properties.template entry for: %s in %s" %(key, ipt_filepath)


#######################################################################
# translate_zos_libpath record
# Example of input.properties.template record
#    @COMMENT@# server.environment=@{LIBPATH}= @{ZOSLIBPATH}/lib@{PATHDELIMITER}${ITCAMDCHOME}/toolkit/lib@{PLATFORMBITMODE}@{PATHDELIMITER}@{TTAPILIBPATH}
#######################################################################
def  translate_zos_libpath(libpathrec, ZOSLIBPATH, TTAPILIBPATH ):
    '''
         translate_zos_libpath(libpathrec, TTAPILIBPATH, TTAPILIBPATH )
         @param libpathrec    record from input.properties.template
         @param ZOSLIBPATH    zos   libpath PDL
         @param TTAPILIBPATH  ttapi libpath from PDL

    '''
    debugUtil.logDebug("translate_zos_libpath - entry ")
    debugUtil.logDebug("  libpathrec:   %s" % ZOSLIBPATH)
    debugUtil.logDebug("  ZOSLIBPATH:   %s" % str(ZOSLIBPATH))
    debugUtil.logDebug("  TTAPILIBPATH: %s" % str(TTAPILIBPATH))

    newlibpath = libpathrec.replace("@{PATHDELIMITER}",":")
    newlibpath = newlibpath.replace("@{PLATFORMBITMODE}","")
    newlibpath = newlibpath.replace("@{ZOSLIBPATH}",ZOSLIBPATH)
    if TTAPILIBPATH is None: TTAPILIBPATH = ""
    newlibpath = newlibpath.replace("@{TTAPILIBPATH}",TTAPILIBPATH)
    newlibpath = newlibpath.replace("${","@{")

    debugUtil.logDebug("  translate_zos_libpath - exit - returns: %s" % str(newlibpath))
    return newlibpath


#######################################################
#This IS the main routine that needs to be called
#######################################################

def createManualInputFile(pdl):
    _DC_HOME_= pdl["CONFIGHOME"][0]
    # Replace "\" in windows file path by "/" so the file path will work well in Java environment
    if platformUtil.isWindows(): _DC_HOME_ = utils.replaceBackSlash(_DC_HOME_)

    debugUtil.logDebug("Entering createManualInputFile()")

    assert "SERVERS" in pdl.keys(), getmsg("CFG5015E", "Data element {0} is not present in the PDL", ["SERVERS"])
    #assert pdl["SERVERS"][0] != None, "SERVERS no servers were selected (this should have been caught earlier)"
    assert len(pdl["SERVERS"][0]) != 0, getmsg("CFG5017E", "Data element {0} of PDL has no entries", ["SERVERS"])

    servercount = len(pdl["SERVERS"][0])
    debugUtil.logDebug("Number of servers selected: "+ str(servercount))

    runtime = _DC_HOME_ + "/runtime"
    if not utils.create_dir_tree(runtime):
        raise ConfigurationException(getmsg("CFG5021E", "Failed to create {0}", [runtime] ))

    createFile(pdl,runtime)

    debugUtil.logDebug("Exiting createManualInputFile()")


#####################################################################
# Unit Testing
#####################################################################

if __name__ == "__main__":

    import createPCL_config as createPCL
    import createPDL
    from debugUtil import logDebug as debug

    debugUtil.set_logging()
    debug("Unit Test for createDCInput")

    pdl = createPDL.make_pdl(createPCL.make_pcl())

    createManualInputFile(pdl)
