####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved. 
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
  This module discovers the itcam templates and displays it to the user
  to select for deletion.
'''

import doprompt as prompt
import utils
import os

from debugUtil import logDebug as debug
from resourcebundle import getmsg as getmsg
from ConfigurationException import ConfigurationException
from DeleteTemplateTask import DeleteTemplateTask
from DiscoverTemplateTask import DiscoverTemplateTask
from constants import constants


def process_discover(pdl):    
    '''
       Runs the discover jython thru the wasadmin to discover the itcam XD templates, and
       it is displayed for selections.
    
       @param pdl - PDL reference
    '''
    
    debug("Entry process_discover")
    
    discovered_records = []
    
    #
    # Instantiate the global constants class
    #
    common_constants = constants()
    
    #
    # Call template task to discover the itcam created templates.
    #
    discover_task = DiscoverTemplateTask(pdl, common_constants.DISCOVERTEMPLATE, "discovertemplatexd_inputlist", "discovertemplatexd.log", "discovertemplatexd_wsadmin.py", "discovertemplatexd.dat")
    discover_task.process()
    #
    # Check the return code
    #
    rc = pdl["RETURNCODE"][0]
    if (rc != 0):
        msg = getmsg("CFG2061E","Discovery of XD template did not complete successfully with return code {0}.",(pdl["RETURNCODE"][0],))
        raise ConfigurationException(msg) 
    
    #
    # Read the discovered templates and display them for deletion.
    #
    datadir = pdl["CONFIGHOME"][0] + os.sep + "data"
    data_fileloc = datadir + os.sep + "discovertemplatexd.dat"
    debug("Data discovery file loc: " + data_fileloc)
    if (os.path.exists(data_fileloc)):
        f = open(data_fileloc, 'r')
        discovered_data = f.readlines()
        if (len(discovered_data) != 0):
            #
            # Strip the records
            #
            for record in discovered_data:
                record = record.strip()
                discovered_records.append(record)
                
            utils.setData(pdl, "TEMPLATELIST", discovered_records, "deletetemplate")
            prompt.do_prompt("TEMPLATELIST", pdl)
    else:
        debug("Data file does not exist - %s" % (data_fileloc)) 
        messg = getmsg("CFG2054E", "Discovery data file does not exist")
        raise ConfigurationException(messg)       
        
    
    debug("Exit process_discover")

def process_delete(pdl):    
    '''
       Runs the discover jython thru the wasadmin to delete the selcted itcam XD templates.
    
       @param pdl - PDL reference
    '''
    
    debug("Entry process_delete")
    
    templatelist = utils.getData(pdl, "TEMPLATELIST")
    
    #
    # Instantiate the global constants class
    #
    common_constants = constants()
    
    if (templatelist is not None):
        #
        # Call template task to discover the itcam created templates.
        #
        delete_task = DeleteTemplateTask(pdl, common_constants.DELETETEMPLATE, "deletetemplatexd_inputlist", "deletetemplatexd.log", "deletetemplatexd_wsadmin.py")
        delete_task.process()
        #
        # Check the return code
        #
        rc = pdl["RETURNCODE"][0]
        if (rc != 0):
            msg = getmsg("CFG2059E","Deletion of XD template did not complete successfully with return code {0}.",(pdl["RETURNCODE"][0],))
            raise ConfigurationException(msg) 
    else:
        msg = getmsg("CFG2062I","There are no itcam templates to delete")
        print msg
            
    
        
    debug("Exit process_delete")

def delete_template(pdl):
    '''
       Discovers the itcam XD templates and displ;ays it for the user to select the
       templates to be deleted.
       
       #param pdl - PDL reference
    '''
    
    debug("Entry delete_template")
    
    #
    # Discover the itcam XD templates
    #
    process_discover(pdl)
    
    #
    # Delete the selected itcam XD templates.
    #
    process_delete(pdl)
    
    debug("Exit delete_template")
