#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
    Deletes the itcam xd templates .
'''
import sys

from time import strftime

# On Windows platforms the python.path property has problems.
# Make an explict addition to the the import path.
sys.path.append("lib")

import debugUtil as log
import resourcebundle

from constants import constants
from Progress import Progress
##Global variables
VERSIONSTRING = "ITCAM 7.2 AK0000"
SCRIPT = "deletetemplatexd_wsadmin"
SCRIPTNAME = SCRIPT + ".py"

common_constants = None

# This is used for properties and variable substitution entries
timestamp = strftime("%Y-%m-%d %H:%M:%S")
tmp = ("%s %s" % (VERSIONSTRING, timestamp))
description_entry = ["description", tmp]


def convertToList(inlist):
    '''
       Convert the templates to a list format.
    '''
       
    outlist=[]
    
    print "Template List:  %s " % (inlist)
    if (len(inlist) > 0 and inlist[0] == '[' and inlist[len(inlist)-1] ==']'):
        inlist = inlist[1:len(inlist)-1]
        clist = inlist.split(" ")
    else:
        clist = inlist.split("\n")
        
    for elem in clist:
        elem = elem.rstrip();
        if (len(elem) > 0):
            outlist.append(elem)
            
    return outlist


def delete_template(templatename, progress):
    '''
       Deletes the template.
       
       @param template - template name
       @param progress - Progress reference
    '''
    
    global AdminTask
    
    print "Entry delete_template - %s" % (templatename)   
    
    print "searching for templates"
    templatename = templatename.strip()
    templates = convertToList(AdminConfig.listTemplates("Server"))
    for template in templates:
        cname = AdminConfig.showAttribute(template,"name")
        if (cname != None):
            cname = cname.strip()
            print "cname: %s  " % (cname)
            if (cname == templatename):
                print "Found itcam template: " + cname
                AdminTask.makeNonSystemTemplate("-templateName " + cname +" -serverType APPLICATION_SERVER")
                AdminTask.deleteServerTemplate(template)
                msg = resourcebundle.getmsg("XDTEMPLATE_DELETED", "XD template deleted - {0}", [cname])
                progress.write_progress(msg) 
                break

    print "Exit delete_template"

def check_templates(templates, progress):
    '''
       Deletes the itcam templates
       
       @param datafileloc - Templates name separated by a '^'
       @param progress - Progress reference
    '''
    
    global AdminConfig
    
    print "check_templates - %s " % (templates)
    
    progress.write_progress_start()
    
    #
    # Get each template name and delete it.
    #
    template_entries = templates.split("^")
    for template in template_entries:
        print "Template to be deleted: %s" % (template)
        delete_template(template, progress)
        
        
    try:
        AdminConfig.save( )
        
    except Exception, errmsg:
        print " \n*Error*  Failed to save"
        print "         Error message: " + errmsg + "\n "
        raise RuntimeError, "saveChanges routine failed - " % (errmsg)    
    
    print "Exit check_templates"
        


                
    
####################################################################################
#
#  Main
#
#
#
####################################################################################

#

print "discovertemplatexd_wsadmin running."


progress_obj = None
failed = 0

filename = sys.argv[0]
#
# Initialize logging
#
log.set_logging()
#
# Initialize the resource bundle
#
resourcebundle.set_resourcebundle()
log.logDebug("Resource bundle initialized")

#
# Instantiate the global constants class
#
common_constants = constants()

phase = "config"  

try:

    if (filename != ""):
        print "Input files " + filename
        #
        # Read the input list file, and the reading is done thru the
        # Progress object, as the first record in the list is the progress file 
        # file location.
        #
        inputlist_file_loc = filename
        progress_obj = Progress(common_constants.DELETETEMPLATE, inputlist_file_loc)
        inDesc = progress_obj.open_progressfile()
        lines = []
        #
        # Read other records in the input list, which only consists of one record.
        # This record has the template name and the input properties file location
        #
        while 1:
            buffer = inDesc.readline()
            if not buffer: break
            lines.append(buffer)
        inDesc.close()

        loaded_modules = dir()
        if "AdminConfig" in loaded_modules and not "AdminTask" in loaded_modules:
            msg = resourcebundle.getmsg("CFG5046E", "AdminTask object is required to complete the configuration, but is unavailable. Unable to continue.")
            progress_obj.write_progress(msg)
            raise Exception(msg)

        templates = lines[0]
        templates = templates.strip()
        print "Templates: %s" % (templates)
        
        #
        # Get the itcam XD templates and save them in the data file.
        #
        check_templates(templates, progress_obj)
        progress_obj.write_progress_finish("", common_constants.SUCCESSFUL)
        
         

    #
    # Close the progress file
    #
    if (progress_obj != None):
        progress_obj.close_file()
        progress_obj = None

    #
    # If processing each server caused an error, then a message is issued, and the exception
    # is raised, so the wsadmin gets a non zero return code.
    #
    if (failed == 1):
        log.logDebug("Delete templates had a failure")
        raise Exception("Delete templates had a failure")
    
    
    
# This needs to pick up the sys.exit(1) and exceptions     
except:
    log.logException(SCRIPTNAME+" ends abnormally\n ")
    traceback_report = log.format_traceback()
     
    #
    # If progress object is present, put a message in progress file to be displayed for the wsadmin caller.
    #
    if (progress_obj is not None):
        msg = resourcebundle.getmsg("DELETE_XDTEMPLATE_ERROR", "Error occurred while processing deleting XD template")
        if len(traceback_report) > 0:
            msg = msg + "\n" + traceback_report
        progress_obj.write_progress(msg)
        progress_obj.write_progress_finish("", common_constants.UNSUCCESSFUL)
        progress_obj.close_file()

    etype, value, tbck = sys.exc_info()
    raise Exception(value)


# When no errors, exit gracefully to reset 
# any error status previously stored by wsadmin
sys.exit(0)
 
    
