#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
    Discovers the itcam xd templates and writes it to the data file.
'''
import sys

from time import strftime

# On Windows platforms the python.path property has problems.
# Make an explict addition to the the import path.
sys.path.append("lib")

import debugUtil as log
import resourcebundle

from constants import constants
from Progress import Progress
##Global variables
VERSIONSTRING = "ITCAM 7.2 AK0000"
SCRIPT = "discovertemplatexd_wsadmin"
SCRIPTNAME = SCRIPT + ".py"

common_constants = None

# This is used for properties and variable substitution entries
timestamp = strftime("%Y-%m-%d %H:%M:%S")
tmp = ("%s %s" % (VERSIONSTRING, timestamp))
description_entry = ["description", tmp]


def convertToList(inlist):
    '''
       Convert the templates to a list format.
    '''
       
    outlist=[]
    
    print "Template List:  %s " % (inlist)
    if (len(inlist) > 0 and inlist[0] == '[' and inlist[len(inlist)-1] ==']'):
        inlist = inlist[1:len(inlist)-1]
        clist = inlist.split(" ")
    else:
        clist = inlist.split("\n")
        
    for elem in clist:
        elem = elem.rstrip();
        if (len(elem) > 0):
            outlist.append(elem)
            
    return outlist


def check_templates(datafileloc, progress):
    '''
       Get the itcam XD templates and writes them to the data file.
       
       @param datafileloc - Data file location
       @param progress - Progress reference
    '''
    
    global AdminConfig
    
    print "check_templates - %s " % (datafileloc)
    
    progress.write_progress_start()
    
    #
    # Open the data file for writing
    #
    data_file = open(datafileloc, 'w')
    print "searching for templates"
    templates = convertToList(AdminConfig.listTemplates("Server"))
    for template in templates:
        cname = AdminConfig.showAttribute(template,"name")
        if (cname != None):
            print "found: " + cname
            if (cname.endswith("itcam")):
                print "Found itcam template: " + cname
                data_file.write(cname + "\n")
                
    data_file.close()          
        
    
####################################################################################
#
#  Main
#
#
#
####################################################################################

#

print "discovertemplatexd_wsadmin running."


progress_obj = None
failed = 0

filename = sys.argv[0]
#
# Initialize logging
#
log.set_logging()
#
# Initialize the resource bundle
#
resourcebundle.set_resourcebundle()
log.logDebug("Resource bundle initialized")

#
# Instantiate the global constants class
#
common_constants = constants()

phase = "config"  

try:

    if (filename != ""):
        print "Input files " + filename
        #
        # Read the input list file, and the reading is done thru the
        # Progress object, as the first record in the list is the progress file 
        # file location.
        #
        inputlist_file_loc = filename
        progress_obj = Progress(common_constants.DISCOVERTEMPLATE, inputlist_file_loc)
        inDesc = progress_obj.open_progressfile()
        lines = []
        #
        # Read other records in the input list, which only consists of one record.
        # This record has the template name and the input properties file location
        #
        while 1:
            buffer = inDesc.readline()
            if not buffer: break
            lines.append(buffer)
        inDesc.close()

        loaded_modules = dir()
        if "AdminConfig" in loaded_modules and not "AdminTask" in loaded_modules:
            msg = resourcebundle.getmsg("CFG5046E", "AdminTask object is required to complete the configuration, but is unavailable. Unable to continue.")
            progress_obj.write_progress(msg)
            raise Exception(msg)

        datafile = lines[0]
        datafile = datafile.strip()
        print "datafile: %s" % (datafile)
        
        #
        # Get the itcam XD templates and save them in the data file.
        #
        check_templates(datafile, progress_obj)
        progress_obj.write_progress_finish("", common_constants.SUCCESSFUL)
        
         

    #
    # Close the progress file
    #
    if (progress_obj != None):
        progress_obj.close_file()
        progress_obj = None

    #
    # If processing each server caused an error, then a message is issued, and the exception
    # is raised, so the wsadmin gets a non zero return code.
    #
    if (failed == 1):
        log.logDebug("Discovering templates had a failure")
        raise Exception("Discovering templates had a failure")
    
    
    
# This needs to pick up the sys.exit(1) and exceptions     
except:
    log.logException(SCRIPTNAME+" ends abnormally\n ")
    traceback_report = log.format_traceback()
     
    #
    # If progress object is present, put a message in progress file to be displayed for the wsadmin caller.
    #
    if (progress_obj is not None):
        msg = resourcebundle.getmsg("DISCOVER_XDTEMPLATE_ERROR", "Error occurred while processing discovering XD template")
        if len(traceback_report) > 0:
            msg = msg + "\n" + traceback_report
        progress_obj.write_progress(msg)
        progress_obj.write_progress_finish("", common_constants.UNSUCCESSFUL)
        progress_obj.close_file()

    etype, value, tbck = sys.exc_info()
    raise Exception(value)


# When no errors, exit gracefully to reset 
# any error status previously stored by wsadmin
sys.exit(0)
 
    
