#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
   This module displays configuration parameters and allows the user
   to redrive the selection process.
'''

fixlev = "41696" # Last APAR, defect or story to update this module

from types import StringType, UnicodeType

import utils
import platformUtil
import createPCL_config as createPCL
import createPDL
import debugUtil as log
import doprompt as prompt
import createservers
import configtema
import configms
import configsmf
import configtpv
import configgc
import configsoa
import configtt
import configmaxheap
import configde
import configdcless

from resourcebundle import getmsg
from debugUtil import logDebug as debug

_MAX_SERVERS_FOR_DETAILS = 3
is_portal = False
de_forced = False
only_wlp_selected = False

###################################################
# Sections and elements to display
###################################################

serverid_data = (                     # Driven by WAS server name
                  "WASCELL",
                  "WASNODE",
                )
wsadmin_data  = (                     # Driven by "WSADMIN"
                                      # No entries
                )
server_data   = (                     # driven by # WAS server name
                  "WASVER",           # WAS version
                  "WBIHOME",          # WBI home directory
                  "WPSHOME",          # WPS  home directory
                  "STANDALONE",       # SPECIAL  network Deployment or    STANDALONE
                  "JVMMODE",          #    64bit or   31bit
                  "ALIASLIST",        # Server Alias
                )
server_runtime =(                     # zOS only
                  "CONFIGHOME",       # ITCAM configuration home
                  "OWNER",            # Set owner
                  "ACCESS",           # Set access
                )
tema_data_dclessmore  =    (                     # Driven by "TEMACONNECT" ITCAM for Tivoli Enterprise Portal Interface
                  "JMXADMINPORT",
                  "TEMADCAPPSERVER",
                  "DEPRO",
                  "TEMAHOST",
                  "TEMAPORT",
                )
tema_data_dclessonly = (
                  "JMXADMINPORT",
                  "TEMADCAPPSERVER",
                 )
itcam_data =    (                     # Driven by "MSCONNECT"   ITCAM for Tivoli Application Monitor Interface
                  "KERNELIP",
                  "KERNELPORT01",
                  "MSHOME",
                )
z_ttapi_data  = (                     # zOS TTAPI config data
                  "TTAPISSN",
                  "TTAPIDIR",
                )
d_ttapi_data  = (                     # Distributed TTAPI config data
                  "TTTCHOST",         # Transations Collector host
                  "TTTCPORT",         #                       port
                )
soadc_data  =   (                     #SOADC currently has no config details

                )
z_misc_data =   (
                  "SMF",
                  "GCOUTPUT",
                )
d_misc_data =   (
                  "GCOUTPUTDIST",
                  "HEAPSIZE",
                )

# Special translation table for certain variable names
special_names = {
                  "OWNER":     "SETOWNER",
                  "ACCESS":    "SETACCESS",
                  "STANDALONE":"DEPLOYMENT",
                  "ALIASLIST": "SERVERALIAS",
                }


def display(pdl):
    '''
       Displays configuration parameters and drive re-prompting
        @param pdl - PDL reference
        returns   True  - configuration selections accepted
                  False - caller needs to redrive configuration selections
    '''

    # global line is needed since we want to change it - jython construct
    global is_portal, only_wlp_selected

    debug("Entry display %s" % fixlev)

    debug("  Interactive prompt to accept or change config values")

    CONFIG_SOA=-1
    CONFIG_TEMA=-1
    CONFIG_MS=-1
    CONFIG_TT=-1
    CONFIG_TPV=-1
    CONFIG_DE=-1
    CONFIG_MISC=-1
    validAnswer = []

    wpshome = pdl["WPSHOME"][0]
    (only_wlp_selected, wlp_servers) = utils.get_wlp_selected(pdl)
    if type(wpshome) == type([]):
        portal_count = 0
        for wps in range(len(wpshome)):
            if wpshome[wps] != None and wpshome[wps] != "None" and len(wpshome[wps]) > 0:
                portal_count += 1
        if portal_count == len(wpshome): is_portal = True
    else:
        is_portal = wpshome != None and wpshome != "None" and len(wpshome) > 0
    debug("is_portal: %s" % is_portal)

    ACCEPT = getmsg("ACCEPT_CHAR","a")
    ALL = getmsg("SELECT_ALL_CHAR","*")

    # Special handling for mocking
    if len(ACCEPT) > 1 and ACCEPT.endswith("]"): ACCEPT = ACCEPT[-2:-1]
    if len(ALL) > 1 and ALL.endswith("]"): ALL = ALL[-2:-1]

    while True:
        de_forced = pdl["DE"][1] == "configtema" and utils.getDataString(pdl, "DE") == "1"
        debug("de_forced: %s" % de_forced)

        # Display configuration choices
        display_config_info(pdl)

        # If batch, exit
        if pdl["BATCH"][0] or pdl["SILENT"][0]:
            break

        print
        print getmsg("CFG1040I","Configuration sections:")
        print
        print " 1) %s" % getmsg("LISTOFSERVERS","List of servers selected")

        validAnswer = [ACCEPT, ALL, "1"]

        if platformUtil.IS_zOS_OS():
            i = 2
            validAnswer.append(str(i))
            CONFIG_SOA=i
            print " %s) %s "  % (i, getmsg("SOADC","Service Oriented Architecture Interface"))
            i += 1
            validAnswer.append(str(i))
            CONFIG_TEMA=i
            print " %s) %s "  % (i, getmsg("TEMACONNECT","ITCAM for Tivoli Enterprise Portal Interface"))
            i += 1
            validAnswer.append(str(i))
            CONFIG_MS=i
            print " %s) %s "  % (i, getmsg("MSCONNECT","ITCAM for Tivoli Application Monitor Interface"))
            i += 1
            validAnswer.append(str(i))
            CONFIG_TT = i
            print " %s) %s "  % (i, getmsg("TTAPI","Transaction Tracking Integration"))
            if not is_portal:
                i += 1
                validAnswer.append(str(i))
                CONFIG_TPV=i
                print " %s) %s "  % (i, getmsg("TPVCONNECT","Integrate with Tivoli Performance Viewer"))
            i += 1
            validAnswer.append(str(i))
            CONFIG_MISC=i
            print " %s) %s "  % (i, getmsg("MISCELLANEOUS","Miscellaneous configuration selections"))

        elif  platformUtil.IS_OS400_OS():
            i = 2
            validAnswer.append(str(i))
            CONFIG_TEMA=i
            print " %s) %s "  % (i, getmsg("TEMACONNECT","ITCAM for Tivoli Enterprise Portal Interface"))
            i += 1
            validAnswer.append(str(i))
            CONFIG_MS=i
            print " %s) %s "  % (i, getmsg("MSCONNECT","ITCAM for Tivoli Application Monitor Interface"))
            i += 1
            validAnswer.append(str(i))
            CONFIG_TT = i
            print " %s) %s "  % (i, getmsg("TTAPI","Transaction Tracking Integration"))
            if not is_portal:
                i += 1
                validAnswer.append(str(i))
                CONFIG_TPV=i
                print " %s) %s "  % (i, getmsg("TPVCONNECT","Integrate with Tivoli Performance Viewer"))
            i += 1
            validAnswer.append(str(i))
            CONFIG_MISC = i
            print " %s) %s "  % (i, getmsg("MISCELLANEOUS","Miscellaneous configuration selections"))
        else:   #distributed
            if not only_wlp_selected:
                i = 2
                validAnswer.append(str(i))
                CONFIG_TEMA = i
                print " %s) %s "  % (i, getmsg("TEMACONNECT","ITCAM for Tivoli Enterprise Portal Interface"))
                i += 1
                validAnswer.append(str(i))
                CONFIG_TT = i
                print " %s) %s "  % (i, getmsg("TTAPI","Transaction Tracking Integration"))
                i += 1
				# If configtema set DE on, we don't allow it changed by this entry
                if not de_forced:
                    validAnswer.append(str(i))
                    CONFIG_DE = i
                    print " %s) %s "  % (i, getmsg("DE","Integrate with ITCAM diagnostics tool"))
                    i += 1	
                validAnswer.append(str(i))
                CONFIG_MISC = i
                print " %s) %s "  % (i, getmsg("MISCELLANEOUS","Miscellaneous configuration selections"))
            else:
                i = 2
                validAnswer.append(str(i))
                CONFIG_TEMA = i
                print " %s) %s "  % (i, getmsg("TEMACONNECT","ITCAM for Tivoli Enterprise Portal Interface"))
                i += 1
                validAnswer.append(str(i))
                CONFIG_TT = i
                print " %s) %s "  % (i, getmsg("TTAPI","Transaction Tracking Integration"))
                i += 1
                validAnswer.append(str(i))
                CONFIG_MISC = i
                print " %s) %s "  % (i, getmsg("MISCELLANEOUS","Miscellaneous configuration selections"))

        #Init prompt message - no default
        utils.setData(pdl,"PRCONFIGDISPLAY","","dispConfig")
        while True:
            prompt.do_prompt("PRCONFIGDISPLAY", pdl)
            answer =  utils.getData(pdl, "PRCONFIGDISPLAY")
            if answer in validAnswer:
                break
            print "%s\n" % getmsg("CFG5007E","Invalid response: {0} Please try again.", [answer])

        if  answer == ACCEPT:
            # Accept config selections and proceed: 'a'
            debug("display_config display - exit - accept choices, proceed")
            return True
        elif answer == ALL:
            # Respecify all options: '*', use config.py to redrive
            debug("display_config display - exit - redrive config.py prompts")
            return False
        elif answer == "1":
            # Prompt for servers and details
            createservers.process_servers(pdl)
            continue

        if answer == str(CONFIG_SOA):
            # Configure SOA DC
            configsoa.config_soa(pdl)
        elif answer == str(CONFIG_TEMA):
            # Configure Tema
            configtema.config_tema(pdl)
        elif answer == str(CONFIG_MS):
            # Configure Managing Server
            configms.config_ms(pdl)
        elif answer == str(CONFIG_TT):
            # Configure TTAPI
            configtt.config_tt(pdl)
        elif answer == str(CONFIG_TPV):
            # Configure TPVCONNECT
            configtpv.config_tpv(pdl)
        elif answer == str(CONFIG_DE):
            # Configure DE
            configde.config_de(pdl)
        elif answer == str(CONFIG_MISC):
            if platformUtil.IS_zOS_OS():
                configsmf.config_smf(pdl)
            # Configure GC - "GCOUTPUTDIST"
            configgc.config_gc(pdl)
                # Configure max heap size
                # configmaxheap.config_maxheap(pdl)
        else:   # should not occur
            debug("*Warning*  Checking of answer %s failed" % answer)
            print "%s\n" % getmsg("CFG5007E","Invalid response: {0} Please try again.", [answer])

    debug("display_config display - exit")
    return True


def display_config_info(pdl):
    '''
        Display configuration choices
        @param pdl - PDL reference
    '''
    debug("Entry display_config_info %s" % fixlev)

    global only_wlp_selected
    # Log the pdl with the results of the configuration selections
    log.logDumpPDL(pdl, "Before display config - %d entries" % len(pdl), short=False)

    # Get common messages and labels

    msgSERVER   = getmsg("SERVER","WAS server")
    msgCELL     = getmsg("WASCELL","WAS cell")
    msgNODE     = getmsg("WASNODE","WAS server")
    msgWSADMIN  = getmsg("WSADMIN","wsadmin location")
    msgPROFHOME = getmsg("WASPROFILEHOME","WebSphere Profile home")
    msgTEMA     = getmsg("TEMACONNECT","ITCAM for Tivoli Enterprise Portal Interface" )
    msgMSCONNECT= getmsg("MSCONNECT","ITCAM for Tivoli Application Monitor Interface")
    msgTTAPI    = getmsg("TTAPI","Transaction Tracking Integration")
    msgSOADC    = getmsg("SOADC","Service Oriented Architecture Interface")
    msgTPV      = getmsg("TPVCONNECT","Integrate with Tivoli Performance Viewer")
    msgDE       = getmsg("DE","Integrate with ITCAM diagnostics tool")
    msgMISCL    = getmsg("MISCELLANEOUS","Miscellaneous configuration selections")

    ######################################################
    # common information across all servers
    #######################################################
    temaconnect   = utils.getDataFlex(pdl, "TEMACONNECT")
    msconnect     = utils.getDataFlex(pdl, "MSCONNECT")
    ttapi         = utils.getDataFlex(pdl, "TTAPI")
    soadc         = utils.getDataFlex(pdl, "SOADC")
    tpv           = utils.getDataFlex(pdl, "TPV")
    de            = utils.getDataFlex(pdl, "DE")

    # Use the resource bundle to get Yes|No translations
    valTEMA      = _set_yn_val(temaconnect)
    valMSCONNECT = _set_yn_val(msconnect)
    valTTAPI     = _set_yn_val(ttapi)
    valSOADC     = _set_yn_val(soadc)
    valTPV       = _set_yn_val(tpv)
    valDE        = _set_yn_val(de)
    debug("   temaconnect=" + str(temaconnect))
    debug("   msconnect="   + str(msconnect))
    debug("   ttapi=" + str(ttapi))
    debug("   tpv:"   + str(tpv))
    debug("   de:"    + str(de))
    debug("   soadc:" + str(soadc))

    # If only Liberty servers were selected, the JMX port has no validity
    if pdl["TEMADCAPPSERVER"][0]:
        pdl["DEPRO"][0:2] = pdl["DE"][0], "display_config"
        tema_data = [e for e in tema_data_dclessmore if e != "JMXADMINPORT" or not only_wlp_selected]
    else:
        tema_data = [e for e in tema_data_dclessonly if e != "JMXADMINPORT" or not only_wlp_selected]

    # These tuples control the display for z/OS, OS/400, and distributed
    #                                                                Display
    #                          #  ID      Active ?     Description   Value        List of variables to display
    #                         --- ----    ----------   -----------   -----        ----------------------------
    #TPV data is not displayed for Portal Servers
    debug("...Is this a portal Server:" + str(is_portal))
    if is_portal:
        display_driver_ZOS  = (  [ 2, "SOA",  soadc,       msgSOADC,     valSOADC,     None ],
                                 [ 3, "TEMA", temaconnect, msgTEMA,      valTEMA ,     tema_data ],
                                 [ 4, "MS",   msconnect,   msgMSCONNECT, valMSCONNECT, itcam_data ],
                                 [ 5, "TT",   ttapi,       msgTTAPI,     valTTAPI,     z_ttapi_data ],
                                 [ 6, "MISC", "NA",        msgMISCL,     "",           z_misc_data ],
                              )

        display_driver_DIST = (  [ 2, "SOA",  soadc,       msgSOADC,     valSOADC,     None ],
                                 [ 3, "TEMA", temaconnect, msgTEMA,      valTEMA ,     tema_data ],
                                 [ 4, "MS",   msconnect,   msgMSCONNECT, valMSCONNECT, itcam_data ],
                                 [ 5, "TT",   ttapi,       msgTTAPI,     valTTAPI,     d_ttapi_data ],
                                 [ 6, "DE",   de,          msgDE,        valDE,        None ],
                                 [ 7, "MISC", "NA",        msgMISCL,     "",           d_misc_data ],
                              )

        display_driver_400  = (  [ 2, "TEMA", temaconnect, msgTEMA,      valTEMA ,     tema_data ],
                                 [ 3, "MS",   msconnect,   msgMSCONNECT, valMSCONNECT, itcam_data ],
                                 [ 4, "TT",   ttapi,       msgTTAPI,     valTTAPI,     d_ttapi_data ],
                                 [ 5, "MISC", "NA",        msgMISCL,     "",           d_misc_data ],
                              )
    elif only_wlp_selected:
        display_driver_DIST = ( [ 2, "TEMA", temaconnect, msgTEMA,      valTEMA ,     tema_data ],
                                [ 3, "TT",   ttapi,       msgTTAPI,     valTTAPI,     d_ttapi_data ],
                                [ 4, "MISC", "NA",        msgMISCL,     "",           d_misc_data ],
                             )
        display_driver_ZOS = ( [ 2, "TEMA", temaconnect, msgTEMA,      valTEMA ,     tema_data ],
                               [ 3, "MISC", "NA",        msgMISCL,     "",           d_misc_data ],
                             )
        display_driver_400 = ( [ 2, "TEMA", temaconnect, msgTEMA,      valTEMA ,     tema_data ],
                               [ 3, "MISC", "NA",        msgMISCL,     "",           d_misc_data ],
                             )
    else:
        display_driver_ZOS  = (  [ 2, "TEMA", temaconnect, msgTEMA,      valTEMA ,     tema_data ],
                                 [ 3, "MISC", "NA",        msgMISCL,     "",           z_misc_data ],
                              )

        display_driver_DIST = (  [ 2, "TEMA", temaconnect, msgTEMA,      valTEMA ,     tema_data ],
                                 [ 3, "TT",   ttapi,       msgTTAPI,     valTTAPI,     d_ttapi_data ],
                                 [ 4, "DE",   de,          msgDE,        valDE,        None ],
                                 [ 5, "MISC", "NA",        msgMISCL,     "",           d_misc_data ],
                              )

        display_driver_400  = (  [ 2, "TEMA", temaconnect, msgTEMA,      valTEMA ,     tema_data ],
                                 [ 3, "MISC", "NA",        msgMISCL,     "",           d_misc_data ],
                              )

    #
    #   Report title  Summary of chosen configuration parameters"
    #
    print
    print "%s" % getmsg("FLOWERBOX_DCSUMMARY", "Data collection configuration summary").rstrip()
    print

    # "Setup will create an ITCAM configuration for each of the servers selected"
    print "%s" % getmsg("CFG1029I","Each of the servers will be configured for data collection")
    print

    if pdl["DCLESS"][0]:
        print getmsg("TOBEREGISTERED","These servers will be registered for resource (PMI) monitoring by ITCAM Agent for WebSphere Applications")
        print

    ######################################################
    # Server display  - one subsection for each server
    ######################################################
    #       1) List of servers selected
    print " 1) %s"   % getmsg("LISTOFSERVERS","List of servers selected")
    print

    # For each server:

    single_line_per_server = False
    if len(pdl["SERVERS"][0]) > _MAX_SERVERS_FOR_DETAILS:
        single_line_per_server = True

    for serveri in range(len(pdl["SERVERS"][0])):
        server = pdl["SERVERS"][0][serveri]
        debug(" display_config_info for server %s - starting" % ( server) )
        cell    = utils.getDataFlex(pdl,"WASCELL", serveri)
        node    = utils.getDataFlex(pdl,"WASNODE", serveri)
        wsadmin = utils.getDataFlex(pdl,"WSADMIN", serveri)
        profile = utils.getDataFlex(pdl,"PROFILE", serveri)
        profilehome = utils.getDataFlex(pdl,"PROFILEHOME", serveri)
        WAS_server_name = "%s.%s.%s(%s)"  % (cell, node, server, profile)

        w = max(len(msgSERVER),len(msgCELL),len(msgNODE))

        #
        # Display server information, with details up to _MAX_SERVERS_FOR_DETAILS.
        #
        print "   %s %*s: %s "  % ("-", w, msgSERVER, WAS_server_name)

        if single_line_per_server: print
        else:

            print "     %*s: %s " % (w, msgNODE, node)
            print "     %*s: %s " % (w, msgCELL, cell)
            print

            #     "   WebSphere Profile home :
            if profilehome is not None and len(profilehome) > 0:
                print "       %-25s : "  % msgPROFHOME
                print "         %s "     % profilehome
                print
            #     "   wsadmin location       :
            if wsadmin is not None and len(wsadmin) > 0:
                print "       %-25s : "  % msgWSADMIN
                print "         %s "     % wsadmin
                print

            # Display the variables in server_data
            display_variables(pdl, server_data, serveri)
            # Display runtime information
            display_variables(pdl, server_runtime, serveri )
            print

    debug(" display_config_info for %s - complete" % server )

    if  platformUtil.IS_zOS_OS():
        display_driver = display_driver_ZOS
    elif platformUtil.IS_OS400_OS():
        display_driver = display_driver_400
    else:
        display_driver = display_driver_DIST

    # Display the SOA/TEMA/MS/TT/TPV, and Miscellaneous sections
    #   dr[0] - id number
    #   dr[1] - id value                    ex "TEMA"
    #   dr[2] - option is active True|False ex temaconnect
    #   dr[3] - descriptive message         ex msgTEMA,
    #   dr[4] - value to display translated ex valTEMA
    #   dr[5] - list of options to display  ex tema_data
    # Except for the "MISCL" section, if the option(ex temaconnect) is false,
    # we do not display the details.

    setWarning=True
    needConfigDataCollector = False
    # Display each row "dr", and in most cases, the details.
    for dr in display_driver:
        #  For example:
        #         2) Service Oriented Architecture Interface : Yes|No
        print " " + str(dr[0])  + ") %s : %-s " % (dr[3], dr[4])
        print
        if dr[5] is None:                        # No variables to display
            if dr[2] == True:
                setWarning=False
                needConfigDataCollector = True
            continue
        elif dr[2] != "NA":
            if dr[2] == False or dr[2] == "0":   # option is false
                continue
            else:
                setWarning=False
                if dr[1] != "TEMA" or pdl["TEMADCAPPSERVER"][0]: needConfigDataCollector = True
        display_variables(pdl, dr[5])
        print

    # Turn this flag on if no config data collector is needed
    pdl["NOCONFIGDC"][0:2] = not needConfigDataCollector, "display_config"

    if setWarning == True:
        print
        print getmsg("CFG1166W","**Warning**: You have not selected integration with any interfaces.").ljust(w-3)
        print

    # End of display
    debug(" display_config_info complete for %s exit" % ( server) )


def display_variables(pdl, varList, serveri=None ):
    '''
        Display the options defined in the variables list passed
        @param varList - a tuple or list of variables to be displayed
        @param serveri - optional server index

        The standalone variable now contains u'true' if true.
    '''

    debug("Entry display_variables %s" % fixlev)

    for var in varList:
        # Get the variable's value
        varValue  = utils.getDataFlex(pdl, var,  serveri)
        if (type(varValue) is StringType) or (type(varValue) is UnicodeType):
            varValue = varValue.strip()
        # Get the variables description/label
        msg_lookup_key = var
        if special_names.has_key(var):
            msg_lookup_key = special_names[var]
        varLabel  = getmsg(msg_lookup_key,msg_lookup_key)
        # Special variable cases:
        if var == "STANDALONE":
            if  varValue == "1" or varValue == 1 or varValue  == "true":
                varValue = getmsg("STANDALONE","Stand alone")
            else:
                varValue = getmsg("NETWORKDEPLOYMENT","Network deployment")
        if var == "SMF" and varValue is None:
            varValue =  getmsg("Leave","Leave")
        # Special values cases
        if varValue == "0" or varValue == 0 or varValue is False:
            varValue = getmsg("No","No")
        elif varValue == "1" or varValue == 1 or varValue is True:
            varValue = getmsg("Yes","Yes")
        #   Do not report variables that have no value
        elif varValue is None or len(varValue) == 0:
            continue
        if var == "HEAPSIZE":
            continue
        print "    %28s : %-s "  % (varLabel, varValue)

    debug("Exit display_variables")


def _set_yn_val(variable):
    '''
        Set yes-no variable labels
    '''
    if (variable == None) or (variable is False) or (variable == 0) or (variable == "0"):
        return getmsg("No","No")
    elif (variable is True) or (variable == 1) or (variable == "1"):
        return getmsg("Yes","Yes")
    else:
        return variable


###########################################################################
# Unit Testing
###########################################################################

if __name__ == "__main__":

    import sys
    import resourcebundle

    print "Unit testing of display_config %s" % fixlev
    print "test display config"
    # Initialize PDL
    pcl=createPCL.make_pcl()
    pdl=createPDL.make_pdl(pcl)
    createPDL.proc_envvar(pdl)
    createPDL.proc_cliopt(pdl,sys.argv)
    log.logDumpPDL(pdl,short=False,findservers=True)
    # Init log
    print " set logging to: " + str(utils.getData(pdl,"LOG"))
    log.set_logging(utils.getData(pdl,"LOG"))
    # Set example of discovered data
    utils.setData(pdl,"SERVERS",   ["b8sr00a","b8sr00b","b8sr00c"],   "*Debug*")
    utils.setData(pdl,"JVMMODE",   ["31bit","64bit","31bit"],         "*Debug*")
    utils.setData(pdl,"PROFILES",  ["default","default","default"],   "*Debug*")
    utils.setData(pdl,"WSADMIN",   "/u/WAS800B1/AppServer/bin/wsadmin.sh", "*Debug*")
    utils.setData(pdl,"PROFILE",   ["default","default","default"],   "*Debug*")
    utils.setData(pdl,"PROFILEHOME",["/u/WAS800B1/beta/tvt7004/AppServer/profiles/default",
                                     "",
                                     "/u/WAS800B3/beta/tvt7004/AppServer/profiles/default"],
                                                                      "*Debug*")
    utils.setData(pdl,"WBIHOME",      ["/u/wbi0",""," "], "*Debug*")
    utils.setData(pdl,"WPSHOME",      ["/u/wbi0",""," "], "*Debug*")

    utils.setData(pdl,"STANDALONE",   ["false","true","false"],        "*Debug*")
    utils.setData(pdl,"WASCELL",      ["beta","beta","mycell"],        "*Debug*")
    utils.setData(pdl,"WASNODE",      ["tvt7006","tvt7006","mynode"],  "*Debug*")
    utils.setData(pdl,"WASSERVER",    ["b8sr00a","b8sr00b","b8sr00c"], "*Debug*")
    utils.setData(pdl,"WASVERSION",   ["80","80","85"],                "*Debug*")
    utils.setData(pdl,"WASVER",       ["8.0.0.0 ND","8.0.0.0 ND","8.0.0.0 ND"], "*Debug*")
    utils.setData(pdl,"CURRENTMAXHEAPSIZE",[500000,500000,30000],      "*Debug*")
    utils.setData(pdl,"RUNLOC",       ["/u/config1","/u/config2","/u/config3"], "*Debug*")
    utils.setData(pdl,"GCOUTPUTDIST", "/u/config1/gc",                 "*Debug*")
    utils.setData(pdl,"HEAPSIZE",     128,                             "*Debug*")
    utils.setData(pdl,"TPV",          True,                            "*Debug*")


    log.logDumpPDL(pdl,short=False,findservers=True)
    # Initialize the resource bundle
    resourcebundle.set_resourcebundle()
    # Call this module
    display(pdl)
    log.logDumpPDL(pdl, "PDL after display ")
