#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

'''

 This module handles getting the configured servers for AD72 and previous
 releases AD71.

'''

fixlev = "41818" # Last APAR, defect or story to update this module

import os
import re

import utils
import doprompt
import platformUtil
import readconfigfiles
import configtema
import configms
import configtt
import configsoa
import configgc
import configtpv
import configde
import checkversion
import debugUtil as log
import migrate_unconfigservers

from constants import constants
from debugUtil import logDebug as debug
from resourcebundle import getmsg

# Create the pattern that matches input.properties files and can
# differentiate dcless.input.properties files.
# The value of result.group(1), which is the server,
# will not be None for a match against a valid input.properties file
# that is not also a dcless.input.properties file.

pat_input = re.compile(r"^.+\..+\..+\..+\.dcless\.input\.properties$|^.+\..+\..+\.(.+)\.input\.properties$")

# Instantiate the global constants class

common_constants = constants()

# The following tuple is the list of the entry names in the pdl for reconfig to
# be picked up by find config servers.

pdl_entry_names_probes = (
                          "TEMACONNECTLIST",
                          "MSCONNECTLIST",
                          "TTAPILIST",
                          "SOADCLIST",
                          "TPVLIST",
                          "DELIST",
                          "KERNELIPLIST",
                          "KERNELPORT01LIST",
                          "RFSPORT01LIST",
                          "TTTCHOSTLIST",
                          "TTTCPORTLIST",
                          "TTAPISSNLIST",
                          "TTAPIDIRLIST",
                          "TEMAHOSTLIST",
                          "TEMAPORTLIST",
                          "ALIASLIST",
                         )

# The following tuple is the list of the entry names in the FINDSERVER dictionary.  The
# FINDSERVERS dictionary items has all the following names in it.

pdl_entry_names = (
                   "WASCELL",
                   "WASNODE",
                   "WASSERVER",
                   "WASINSTALLROOT",
                   "PROFILEHOME",
                   "USERINSTALLROOT",
                   "PROFILE",
                   "CONNTYPELIST",
                   "HOSTLIST",
                   "PORTLIST",
                   "TEMACONNECTLIST",
                   "TEMAHOSTLIST",
                   "TEMAPORTLIST",
                   "MSCONNECTLIST",
                   "MSHOMECONNECTLIST",
                   "KERNELIPLIST",
                   "KERNELPORT01LIST",
                   "RFSPORT01LIST",
                   "MSHOMELIST",
                   "RMIPORTSETLIST",
                   "PROBECONTROLLERRMIPORTLIST",
                   "PROBERMIPORTLIST",
                   "TTAPILIST",
                   "TTAPISSNLIST",
                   "TTAPIDIRLIST",
                   "TTTCHOSTLIST",
                   "TTTCPORTLIST",
                   "TTAPILIBPATH" ,
                   "TTAPICLASSPATH",
                   "SOADCLIST",
                   "SMFLIST",
                   "GCOUTPUTLIST",
                   "GCLOGPATHLIST",
                   "GCOUTPUTDISTLIST",
                   "HEAPSIZELIST",
                   "MAXHEAPSIZE",
                   "CURRENTMAXHEAPSIZE",
                   "GC_PARMS",
                   "CURRENTGCLOGLOCATION",
                   "WASVERSION",
                   "JVMMODE",
                   "JVMVERSION",
                   "STANDALONE",
                   "SERVERID",
                   "CONFIGURED",
                   "MIGRATEDDCHOME",
                   "MIGRATEDVERSION",
                   "MIGRATEDCONFIGHOME",
                   "JAVAHOME",
                   "DCVERSION",
                   "TPVLIST",
                   "DELIST",
                   "WASCELLMODIFIED",
                   "PROFILEMODIFIED",
                   "WASNODEMODIFIED",
                   "SERVERSMODIFIED",
                   "WPSHOME",
                   "WBIHOME",
                   "SUBPRODUCT",
                   "ALIASLIST",
                   "WLPHOME",
                   "CONFIGRETURNCODE",
                  )

# This dictionary has the property names that are in input.properties, and
# a search is done for all the properties in this list.
# The value of dictionary is the name that is used in FINDSERVERS entry in pdl.
# Each entry in the FINDSERVERS is a dictionary which contains the same name
# as the value of this dictionary.  The value of the dictionary are the names
# that are matched in PDL.

inputproperties_props = {
                         "WAS_CELL": "WASCELL",
                         "WAS_NODE": "WASNODE",
                         "WAS_SERVER": "WASSERVER",
                         "WAS_HOME": "WASINSTALLROOT",
                         "WAS_HOMEPROFILE": "PROFILEHOME",
                         "WAS_PROFILE": "PROFILE",
                         "WAS_CONNTYPE": "CONNTYPELIST",
                         "WAS_HOST": "HOSTLIST",
                         "WAS_PORT": "PORTLIST",
                         "WAS_VERSION": "WASVERSION",
                         "WAS_SERVERID": "SERVERID",
                         "JAVA_MODE": "JVMMODE",
                         "JAVA_VERSION": "JVMVERSION",
                         "TEMACONNECT": "TEMACONNECTLIST",
                         "TEMAHOST": "TEMAHOSTLIST",
                         "TEMAPORT": "TEMAPORTLIST",
                         "MSCONNECT": "MSCONNECTLIST",
                         "MSHOMECONNECT": "MSHOMECONNECTLIST",
                         "KERNELIP": "KERNELIPLIST",
                         "KERNELPORT01": "KERNELPORT01LIST",
                         "RFSPORT01": "RFSPORT01LIST",
                         "MSHOME": "MSHOMELIST",
                         "RMIPORTSET": "RMIPORTSETLIST",
                         "PROBECONTROLLERRMIPORT": "PROBECONTROLLERRMIPORTLIST",
                         "PROBERMIPORT": "PROBERMIPORTLIST",
                         "TTAPI": "TTAPILIST",
                         "TTAPISSN": "TTAPISSNLIST",
                         "TTAPIDIR": "TTAPIDIRLIST",
                         "TTTCHOST": "TTTCHOSTLIST",
                         "TTTCPORT": "TTTCPORTLIST",
                         "TTAPILIBPATH"  :"TTAPILIBPATH",
                         "TTAPICLASSPATH":"TTAPICLASSPATH",
                         "SOADC": "SOADCLIST",
                         "SMF": "SMFLIST",
                         "GC_PARMS":"GC_PARMS",
                         "GCOUTPUT": "GCOUTPUTLIST",
                         "GCLOGPATH": "GCLOGPATHLIST",
                         "GCOUTPUTDIST": "GCOUTPUTDISTLIST",
                         "CURRENTGCLOGLOCATION": "CURRENTGCLOGLOCATION",
                         "HEAPSIZE": "HEAPSIZELIST",
                         "MAXHEAPSIZE": "MAXHEAPSIZE",
                         "CURRENTMAXHEAPSIZE": "CURRENTMAXHEAPSIZE",
                         "STANDALONE": "STANDALONE",
                         "MIGRATEDDCHOME":"MIGRATEDDCHOME",
                         "MIGRATEDVERSION":"MIGRATEDVERSION",
                         "JAVA_HOME": "JAVAHOME",
                         "ITCAMDCVERSION": "DCVERSION",
                         "TPV": "TPVLIST",
                         "DE": "DELIST",
                         "WAS_CELL_MODIFIED": "WASCELLMODIFIED",
                         "WAS_NODE_MODIFIED": "WASNODEMODIFIED",
                         "WAS_SERVER_MODIFIED": "SERVERSMODIFIED",
                         "WAS_PROFILE_MODIFIED": "PROFILEMODIFIED",
                         "WPS_HOME": "WPSHOME",
                         "SERVERALIAS": "ALIASLIST",
                         "WLP_HOME": "WLPHOME",
                         "SUBPRODUCT": "SUBPRODUCT",
                         "CONFIGRETURNCODE": "CONFIGRETURNCODE",
                        }

# This dictionary has the property names that are in input.properties, and
# a search is done for all the properties in this list.
# The value of dictionary is the pdl name, and this dictionary is used for
# getting the configured probe values.

inputproperties_probe_props = {
                                "TEMACONNECT": "TEMACONNECTLIST",
                                "MSCONNECT": "MSCONNECTLIST",
                                "TTAPI": "TTAPILIST",
                                "SOADC": "SOADCLIST",
                                "TPV": "TPVLIST",
                                "DE": "DELIST",
                                "KERNELIP": "KERNELIPLIST",
                                "KERNELPORT01": "KERNELPORT01LIST",
                                "RFSPORT01": "RFSPORT01LIST",
                                "TTTCHOST": "TTTCHOSTLIST",
                                "TTTCPORT": "TTTCPORTLIST",
                                "TTAPISSN": "TTAPISSNLIST",
                                "TTAPIDIR": "TTAPIDIRLIST",
                                "TEMAHOST": "TEMAHOSTLIST",
                                "TEMAPORT": "TEMAPORTLIST",
                                "SERVERALIAS": "ALIASLIST",
                               }

# This following 2 dictionaries have the property names that are in dcinput.txt, and
# a search is done for all the properties in this list.
# The value of dictionary is the name that is used in FINDSERVERS entry in pdl.
# Each entry in the FINDSERVERS is a dictionary which contains the same name
# as the value of this dictionary.  The value of the dictionary are the names
# that are matched in the PDL.

dcinput_props_dist = {
                       "APP_SERVER_NAME": "WASCELL",
                       "APP_SERVER_ID": "SERVERID",
                       "WAS_BASEDIR": "WASINSTALLROOT",
                       "WAS_HOME_SHORT_PATH": "PROFILEHOME",
                       "PROFILE": "PROFILE",
                       "CONNECTION": "CONNTYPELIST",
                       "WS_HOST": "HOSTLIST",
                       "WS_PORT": "PORTLIST",
                       "TEMA_HOST": "TEMAHOSTLIST",
                       "TEMA_PORT": "TEMAPORTLIST",
                       "KERNEL_HOST01": "KERNELIPLIST",
                       "PORT_KERNEL_CODEBASE01": "KERNELPORT01LIST",
                       "PORT_KERNEL_RFS01": "RFSPORT01LIST",
                       "MS_AM_HOME": "MSHOMELIST",
                       "PROBE_CONTROLLER_RMI_PORT": "PROBECONTROLLERRMIPORTLIST",
                       "PROBE_RMI_PORT": "PROBERMIPORTLIST",
                       "TTAS_HOST": "TTTCHOSTLIST",
                       "TTAS_PORT": "TTTCPORTLIST",
                       "GCOUTPUT": "GCOUTPUTLIST",
                       "GC_LOG_PATH": "CURRENTGCLOGLOCATION",
                       "HEAPSIZESETTING_SELECTED": "HEAPSIZELIST",
                       "ORG_MAXIMUMHEAPSIZE": "CURRENTMAXHEAPSIZE",
                       "WAS_BASE_SHORT_VERSION": "WASVERSION",
                       "JDK_DATA_MODEL": "JVMMODE",
                       "JDK_VERSION": "JVMVERSION",
                       "IS_ND": "STANDALONE",
                       "DC_OPER_MODE": "TPVLIST",
                       "SERVERALIAS": "ALIASLIST",
                    }

# This following 2 dictionaries have the property names that are in dcinputs.txt, and
# a search is done for all the properties in this list.
# The value of dictionary is the name that is used in FINDSERVERS entry in pdl.
# Each entry in the FINDSERVERS is a dictionary which contains the same name
# as the value of this dictionary.  The value of the dictionary are the names
# that are matched in PDL.
# This property is used for ecam .

dcinput_props_dist_ecam = {
                            "APP_SERVER_NAMES": "WASCELL",
                            "APP_SERVER_ID": "SERVERID",
                            "WAS_BASEDIR": "WASINSTALLROOT",
                            "WAS_HOME_SHORT_PATH": "PROFILEHOME",
                            "PROFILE": "PROFILE",
                            "CONNECTION": "CONNTYPELIST",
                            "WS_HOST": "HOSTLIST",
                            "WS_PORT": "PORTLIST",
                            "TEMA_HOST": "TEMAHOSTLIST",
                            "TEMA_PORT": "TEMAPORTLIST",
                            "KERNEL_HOST01": "KERNELIPLIST",
                            "PORT_KERNEL_CODEBASE01": "KERNELPORT01LIST",
                            "PORT_KERNEL_RFS01": "RFSPORT01LIST",
                            "MS_AM_HOME": "MSHOMELIST",
                            "PROBE_CONTROLLER_RMI_PORT": "PROBECONTROLLERRMIPORTLIST",
                            "PROBE_RMI_PORT": "PROBERMIPORTLIST",
                            "DC_TTAPI_HOST": "TTTCHOSTLIST",
                            "DC_TTAPI_PORT": "TTTCPORTLIST",
                            "GCOUTPUT": "GCOUTPUTLIST",
                            "GC_LOG_PATH": "CURRENTGCLOGLOCATION",
                            "HEAPSIZESETTING_SELECTED": "HEAPSIZELIST",
                            "ORG_MAXIMUMHEAPSIZE": "CURRENTMAXHEAPSIZE",
                            "WAS_BASE_SHORT_VERSION": "WASVERSION",
                            "JDK_DATA_MODEL": "JVMMODE",
                            "JDK_VERSION": "JVMVERSION",
                            "IS_ND": "STANDALONE",
                            "DC_OPER_MODE": "TPVLIST",
                            "SERVERALIAS": "ALIASLIST",
                           }

# This following 2 dictionaries have the property names that are in dcinputs.txt, and
# a search is done for all the properties in this list.
# The value of dictionary is the name that is used in FINDSERVERS entry in pdl.
# Each entry in the FINDSERVERS is a dictionary which contains the same name
# as the value of this dictionary.  The value of the dictionary are the names
# that are matched in PDL.

# This property dictionary is used for dc61.

dcinput_props_dist_dc61 = {
                            "APP_SERVER_NAMES": "WASCELL",
                            "APP_SERVER_ID": "SERVERID",
                            "WAS_BASEDIR": "WASINSTALLROOT",
                            "WAS_HOME_SHORT_PATH": "PROFILEHOME",
                            "PROFILE": "PROFILE",
                            "CONNECTION": "CONNTYPELIST",
                            "WS_HOST": "HOSTLIST",
                            "WS_PORT": "PORTLIST",
                            "TEMA_HOST": "TEMAHOSTLIST",
                            "TEMA_PORT": "TEMAPORTLIST",
                            "KERNEL_HOST01": "KERNELIPLIST",
                            "PORT_KERNEL_CODEBASE01": "KERNELPORT01LIST",
                            "PORT_KERNEL_RFS01": "RFSPORT01LIST",
                            "MS_AM_HOME": "MSHOMELIST",
                            "PROBE_CONTROLLER_RMI_PORT": "PROBECONTROLLERRMIPORTLIST",
                            "PROBE_RMI_PORT": "PROBERMIPORTLIST",
                            "TTAS_HOST": "TTTCHOSTLIST",
                            "TTAS_PORT": "TTTCPORTLIST",
                            "GCOUTPUT": "GCOUTPUTLIST",
                            "GC_LOG_PATH": "CURRENTGCLOGLOCATION",
                            "HEAPSIZESETTING_SELECTED": "HEAPSIZELIST",
                            "ORG_MAXIMUMHEAPSIZE": "CURRENTMAXHEAPSIZE",
                            "WAS_BASE_SHORT_VERSION": "WASVERSION",
                            "JDK_DATA_MODEL": "JVMMODE",
                            "JDK_VERSION": "JVMVERSION",
                            "IS_ND": "STANDALONE",
                            "SERVERALIAS": "ALIASLIST",
                           }

# This property dictionary is used for z/OS.

dcinput_props_zos = {
                       "TEMA_HOST": "TEMAHOSTLIST",
                       "TEMA_PORT": "TEMAPORTLIST",
                       "KERNEL_HOST01": "KERNELIPLIST",
                       "PORT_KERNEL_CODEBASE01": "KERNELPORT01LIST",
                       "PORT_KERNEL_RFS01": "RFSPORT01LIST",
                       "MS_AM_HOME": "MSHOMELIST",
                       "PROBE_CONTROLLER_RMI_PORT": "PROBECONTROLLERRMIPORTLIST",
                       "PROBE_RMI_PORT": "PROBERMIPORTLIST",
                       "TTAPI": "TTAPILIST",
                       "TTAPISSN": "TTAPISSNLIST",
                       "TTAPIDIR": "TTAPIDIRLIST",
                       "PROFILE": "PROFILE",
                       "JDK_VERSION": "JVMVERSION",
                       "WAS_BASE_SHORT_VERSION": "WASVERSION",
                       "DMGR_RMI": "STANDALONE",
                       "DC_OPER_MODE": "TPVLIST",
                       "SERVERALIAS": "ALIASLIST",
                    }


def process_ecam_dc61(pdl,
                      ptype,
                      version,
                      vtype):
    '''
       Process ecam/dc61 to find out the configured servers.  Based on platform, each directory
       that represents a server is read and information is acquired.

       @param pdl = PDL reference
       @param ptype - Process type
       @param version - version of the config
       @param vtype -  process type like AD71, AD72, ecam,....
    '''

    debug("Entry process_ecam_dc61 %s" % fixlev)

    serversfound = False
    dcinput_props = None
    config_directories = []

    os_platform = platformUtil.getOSFamily()
    config_dir_path = get_dc61_ecam_configdir(pdl)
    if not os.path.isdir(config_dir_path):
        debug("Configured directory %s" %(config_dir_path))
        print "%s" % getmsg("CFG1071I", "No servers have been configured in migrate home - {0}", [pdl["MIGRATEHOME"][0]])
    else:
        debug("Processing configured directory %s" %(config_dir_path))

        # In distributed environment, the directory structure is based on the following:
        # cells/CELLNAME/nodes/NODENAME/servers/SERVERNAME
        # so, all the servers directories are gathered, and put into a list to be processed.

        if os_platform != platformUtil.IS_zOS_OS() and os_platform != platformUtil.IS_OS400_OS():
            config_directories = _get_ecam_dc61_servers(pdl, config_dir_path)
        else:
            config_dir_listing = os.listdir(config_dir_path)
            for infile in config_dir_listing:
                config_dir = os.path.normpath(os.path.join(config_dir_path, infile))
                if os.path.isdir(config_dir):
                    config_directories.append(config_dir)
                    debug("Added configured directory to the list - %s" %(config_dir))

    if len(config_directories) > 0:
        if os_platform == platformUtil.IS_zOS_OS():
            debug("Processing ZOS....")
            process, find_serverlist = readconfigfiles.read_pre_ad72_config_files(pdl,
                                                                                  pdl_entry_names,
                                                                                  config_directories,
                                                                                  dcinput_props_zos,
                                                                                  ptype,
                                                                                  version,
                                                                                  vtype)
        elif os_platform == platformUtil.IS_OS400_OS():
            debug("Processing OS400....")
            process, find_serverlist = readconfigfiles.read_pre_ad72_config_files(pdl,
                                                                                  pdl_entry_names,
                                                                                  config_directories,
                                                                                  dcinput_props_dist,
                                                                                  ptype,
                                                                                  version,
                                                                                  vtype)
        else:

            # Based on dc61 or ecam, select the dcinput table and process it.  Some of the dc input properties
            # are different.

            if vtype == common_constants.DC61:
                debug("Processing DC 61....")
                dcinput_props = dcinput_props_dist_dc61
            else:
                debug("Processing ECAM....")
                dcinput_props = dcinput_props_dist_ecam

            process, find_serverlist = readconfigfiles.read_pre_ad72_config_files(pdl,
                                                                                  pdl_entry_names,
                                                                                  config_directories,
                                                                                  dcinput_props,
                                                                                  ptype,
                                                                                  version,
                                                                                  vtype,
                                                                                  "dcInputs.txt",
                                                                                  "APP_SERVER_NAMES")
        if process == True:

            # Prompt to select the servers for migrations.

            utils.setData(pdl, "FINDSERVERS", find_serverlist, "findconfigservers")
            doprompt.do_prompt("SERVERS", pdl, None, None, 1)

            # Check the selected servers in the FINDSERVERS and make sure it does
            # exists, and create pdls for the selected servers.

            serversfound = _check_findservers(pdl, ptype)

    else:
        debug("No Servers have been configured ")
        print getmsg("CFG1056I", "No servers have been configured")

    debug("Exit process_ecam_dc61 - %s" % serversfound)
    return serversfound


def process_ad71(pdl,
                 ptype,
                 version,
                 vtype):
    '''
       Process AD71 to find out the configured servers.  Based on platform, each directory
       that represents a server is read and information is acquired.


       @param pdl = PDL reference
       @param ptype - Process type
       @param version - AD71 version
       @param vtype -  process type like AD71, AD72, ecam,....
    '''

    debug("Entry process_ad71 %s - %s - %s - %s" % (fixlev, ptype, version, vtype))

    serversfound = False
    config_directories = []

    config_dir_path = get_ad71_configdir(pdl, version)
    if not os.path.isdir(config_dir_path):
        debug("Missing configured directory %s" % config_dir_path)
        print "%s" % getmsg("CFG1071I", "No servers have been configured in migrate home - {0}", [pdl["MIGRATEHOME"][0]])
    else:
        debug("Processing configured directory %s" %(config_dir_path))
        config_dir_listing = os.listdir(config_dir_path)
        for infile in config_dir_listing:
            if infile == "custom" and platformUtil.IS_zOS_OS(): continue
            config_dir = os.path.normpath(os.path.join(config_dir_path, infile))
            if os.path.isdir(config_dir):

                # If the file name contains the text that is already been migrated, then
                # it is skipped.  Once a file is migrated, the directory name is changed
                # to have a text that indicates the server has been migrated.

                rename_suffix = migrate_unconfigservers.get_renamed_suffix(pdl)
                if config_dir.find(rename_suffix) == -1:
                    config_directories.append(config_dir)
                    debug("Added configured directory to the list - %s" %(config_dir))
                else:
                    debug("Already has been configured - %s" % (config_dir))

    if len(config_directories) > 0:
        if platformUtil.IS_zOS_OS():
            process, find_serverlist = readconfigfiles.read_pre_ad72_config_files(pdl,
                                                                                  pdl_entry_names,
                                                                                  config_directories,
                                                                                  dcinput_props_zos,
                                                                                  ptype,
                                                                                  version,
                                                                                  vtype)
        else:
            process, find_serverlist = readconfigfiles.read_pre_ad72_config_files(pdl,
                                                                                  pdl_entry_names,
                                                                                  config_directories,
                                                                                  dcinput_props_dist,
                                                                                  ptype,
                                                                                  version,
                                                                                  vtype)
        if process == True:

            # Prompt to select the servers for migrations.

            utils.setData(pdl, "FINDSERVERS", find_serverlist, "findconfigservers")
            doprompt.do_prompt("SERVERS", pdl, None, None, 1)

            # Check the selected servers in the FINDSERVERS and make sure it does
            # exists, and create pdls for the selected servers.

            serversfound = _check_findservers(pdl, ptype)
    else:
        debug("No Servers have been configured ")
        print getmsg("CFG1056I", "No servers have been configured")

    debug("Exit process_ad71 - %s" % (serversfound))
    return serversfound


def process_ad72(pdl,
                 inputproperties_list,
                 ptype,
                 version):
    '''
       Process AD72 to find out the configured servers.  An input.properties file for
       a specific server that resides in "runtime" directory indicates that a server
       has been configured.

       @param pdl = PDL reference
       @param inputproperties_list - A list of input properties files, default is None
       @param ptype - Process type like reconfig, migrate
       @param version - Version of config
    '''

    global common_constants

    debug("Entry process_ad72 %s - %s - %s - %s" % (fixlev, inputproperties_list, ptype, version))

    # Read the files in runtime directory, and choose the files that have
    # suffix of "input.properties".  Based on the type, the DC home is
    # selected.

    files = []

    if ptype == common_constants.MIGRATE:
        path = os.path.normpath(os.path.join(pdl["MIGRATEHOME"][0], "runtime"))
    else:
        path = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "runtime"))
    debug("Path to process for input properties files - %s" % (path))

    # If input properties list is provided, then we don't need to search for it.
    # Otherwise we need to list the files and select those files that are
    # input.properties but not dcless.input.properties files.

    if inputproperties_list is None:
        if os.path.isdir(path):
            for inbase in os.listdir(path):
                infile = os.path.normpath(os.path.join(path, inbase))
                if os.path.isfile(infile):
                    debug("File name: %s" % infile)
                    # Make sure this is an input.properties file but not also 
                    # a dcless.input.properties file as well.
                    r = pat_input.match(infile)
                    if r is None or r.group(1) is None: continue
                    debug("Found input.properties - %s" % infile)
                    files.append(infile)
        else:
            debug("Config path does not exist - %s" % path)
    else:
        files = inputproperties_list

    serversfound = False
    if len(files) > 0:
        debug("Input properties files to be processed:\n    %s" % "\n    ".join(files))
        process, find_serverlist = readconfigfiles.read_ad72_inputproperty_file(pdl,
                                                                                pdl_entry_names,
                                                                                inputproperties_props,
                                                                                files,
                                                                                ptype,
                                                                                version)
        if process == True:

            # Prompt to select the servers for migrations.

            utils.setData(pdl, "FINDSERVERS", find_serverlist, "findconfigservers")
            doprompt.do_prompt("SERVERS", pdl, None, None, 0)

            # Check the selected servers in the FINDSERVERS and make sure it does
            # exists, and create PDLs for the selected servers.

            serversfound = _check_findservers(pdl, ptype)

            if serversfound and pdl["RECONFIG"][0]:

                # Set the default PDL entries.  This is done to show the
                # default values at the time of prompt.  It is only done for
                # reconfig.  Because the prompt values are for all the
                # selected servers, we only choose the first server values
                # and set the default.

                _set_default_pdlvalues(pdl)

    else:
        debug("No files with suffix of input.properties found")
        print "%s" % getmsg("CFG1056I", "No servers have been configured")

    debug("Exit process_ad72 - %s" % (serversfound))
    return serversfound


def get_dc61_ecam_configdir(pdl):
    '''
       This routine gets the DC61 or ecam config directroy.

       @param pdl - PDL reference
    '''

    debug("Entry get_dc61_ecam_configdir %s" % fixlev)

    os_platform = platformUtil.getOSFamily()
    if os_platform == platformUtil.IS_zOS_OS():
        config_dir_path = os.path.normpath(os.path.join(pdl["MIGRATEHOME"][0], "runtime"))
    elif os_platform == platformUtil.IS_OS400_OS():
        config_dir_path = os.path.normpath(os.path.join(pdl["MIGRATEHOME"][0], "config", "configured"))
    else:
        config_dir_path = os.path.normpath(os.path.join(pdl["MIGRATEHOME"][0], "_uninst", "configured"))

    debug("Exit get_dc61_ecam_configdir - %s" % (config_dir_path))
    return config_dir_path


def get_ad71_configdir(pdl, version):
    '''
       This routine gets the AD 71 config directory.  The AD71 vanilla has a different
       directory structure than the fixpack releases of AD71.

       @param pdl - PDL reference
       @param version - AD71 versions
    '''

    debug("Entry get_ad71_configdir %s - %s" % (fixlev, version))

    if platformUtil.IS_zOS_OS():
        config_dir_path = os.path.normpath(os.path.join(pdl["MIGRATEHOME"][0], "runtime"))
    elif platformUtil.IS_OS400_OS():
        if version == common_constants.BASE_AD71VERSION:
            config_dir_path = os.path.normpath(os.path.join(pdl["MIGRATEHOME"][0], "runtime"))
        else:
            config_dir_path = os.path.normpath(os.path.join(pdl["MIGRATEHOME"][0], "config", "configured"))
    else:
        if version == common_constants.BASE_AD71VERSION:
            config_dir_path = os.path.normpath(os.path.join(pdl["MIGRATEHOME"][0], "installer", "configured"))
        else:

            # The config dir is not under the migrate home, it is under the
            # migrate home parent one level higher, so we set it ourself to it.
            # We could have done it by using ".." but because we are comparing for
            # the same name in the configured xml file, then we need to have the
            # entire file path.

            configpath = pdl["MIGRATEHOME"][0]
            config_dir_path = os.path.normpath(os.path.join(configpath, os.pardir, "config", "configured"))

    debug("Exit get_ad71_configdir - %s" % (config_dir_path))
    return config_dir_path


def _get_ecam_dc61_servers(pdl, config_dir_path):
    '''
       Gets the configured servers directory.  The directory structure is in the form
       of  cells/CELLNAME/nodes/NODENAME/servers/SERVERNAMES.

       @param pdl - PDL reference
       @param config_dir_path - Config directory path
    '''

    debug("Entry _get_ecam_dc61_servers %s" % fixlev)

    config_directories = []

    #
    # Get the cells directory contents
    #
    cells_dir_path = os.path.normpath(os.path.join(config_dir_path, "cells"))
    debug("Cells directory path is: %s" % (cells_dir_path))
    if not os.path.isdir(cells_dir_path):
        debug("Cell directory %s" % config_dir_path)
        print "%s" % getmsg("CFG1071I", "No servers have been configured in migrate home - {0}", [pdl["MIGRATEHOME"][0]])
    else:
        cells_dir_listing = os.listdir(cells_dir_path)
        for cells_infile in cells_dir_listing:
            cell_dir = os.path.normpath(os.path.join(cells_dir_path, cells_infile))
            debug("Cells directory name is: %s" % (cell_dir))
            if os.path.isdir(cell_dir):

                # Get the nodes directory contents

                nodes_dir_path = os.path.normpath(os.path.join(cell_dir, "nodes"))
                debug("Nodes directory path is: %s" % (nodes_dir_path))
                if not os.path.isdir(nodes_dir_path):
                    debug("Node directory %s" %(nodes_dir_path))
                    print "%s" % getmsg("CFG1071I", "No servers have been configured in migrate home - {0}", [pdl["MIGRATEHOME"][0]])
                else:
                    nodes_dir_listing = os.listdir(nodes_dir_path)
                    for nodes_infile in nodes_dir_listing:
                        node_dir = os.path.normpath(os.path.join(nodes_dir_path, nodes_infile))
                        debug("Nodes directory name is: %s" % node_dir)
                        if os.path.isdir(node_dir):

                            # Get the servers directory contents

                            servers_dir_path = os.path.normpath(os.path.join(node_dir, "servers"))
                            debug("Servers directory path is: %s" % (servers_dir_path))
                            if not os.path.isdir(servers_dir_path):
                                debug("Servers directory %s" %(servers_dir_path))
                                print "%s" % getmsg("CFG1071I", "No servers have been configured in migrate home - {0}", [pdl["MIGRATEHOME"][0]])
                            else:
                                servers_dir_listing = os.listdir(servers_dir_path)
                                for servers_infile in servers_dir_listing:
                                    debug("Server directory name is: %s" % (servers_infile))
                                    server_dir = os.path.normpath(os.path.join(servers_dir_path, servers_infile))
                                    config_directories.append(server_dir)
                                    debug("Added configured directory to the list - %s" % server_dir)

    debug("Exit _get_ecam_dc61_servers")
    return config_directories


def _check_findservers(pdl, ptype):
    '''
       Each entry in the server list is checked against the FINDSERVERS
       pdl entries for its existence.

       @param pdl - PDL reference
       @param ptype - Process type
                       1 - reconfig
                       2 - migrate
    '''

    debug("Entry _check_findservers %s" % fixlev)

    index = 0
    serverfound = False
    findservers_indexlist = []

    serverlist = utils.getData(pdl, "FULLSERVERS")
    findservers = utils.getData(pdl, "FINDSERVERS")

    # For each entry in the FULLSERVERS, each server is checked to
    # see if it is already been configured in this release by checking
    # the input.properties of the specific server.  If it does exist,
    # a message is issued to the user that the server has already been
    # configured, otherwise, the server entry is checked the entries in
    # FINDSERVERS, and if there is match, remember the index in FINDSERVERS for later use.

    for server_entry in serverlist:
        debug("Server Entry - %s" % (server_entry))
        wasprofile = server_entry["PROFILE"]
        wascell = server_entry["WASCELL"]
        wasnode = server_entry["WASNODE"]
        wasserver = server_entry["WASSERVER"]
        for index in range(len(findservers)):
            findserver_entry = findservers[index]
            debug("FindServer Entry - %s" % (findserver_entry))

            # For each entry in the server list, check it against
            # this FINDSERVER entry and if the server name matches
            # then make a note of it.  The checking is done based
            # on cell, node and server name.

            if (wasprofile == findserver_entry["PROFILE"] and
                wascell == findserver_entry["WASCELL"] and
                wasnode == findserver_entry["WASNODE"] and
                wasserver == findserver_entry["WASSERVER"]):
                findservers_indexlist.append(index)
                debug("Server entry found - profile: %s  Cell: %s  Node: %s  Server: %s" % (wasprofile, wascell, wasnode, wasserver))
                break

    # If there are no find server indexes in the find server list, then no servers were
    # found so exit.

    if len(findservers_indexlist) > 0:
        serverfound = True

        # Transfer the server data to PDL

        tag = "findconfigservers"
        for fentry in pdl_entry_names:
            utils.setData(pdl,fentry,[],tag)
            for serveri in findservers_indexlist:
                utils.getData(pdl,fentry).append(utils.getData(pdl,"FINDSERVERS")[serveri][fentry])

        # If selected number of servers are not the total found servers,
        # then in the case of migrate, a warning message is issued to tell
        # user that there is a possibility of having a mixture of the plugin
        # jars from the previous release to the new release.

        if ptype == common_constants.MIGRATE:
            _check_profiles(pdl)

        # Delete the FINDSERVERS entries as it is no longer needed.

        del findservers[:]

    debug("Exit _check_findservers - %s" % (serverfound))
    return serverfound


def _check_profiles(pdl):
    '''
       Group all the servers within a profile from the FINDSERVERS, and also
       group all the selected servers based on its profile, and if not all
       the servers from the same profile selected, then a warning message is
       issued to notify the user that it could be a inconsistency in the plugin
       jars from a previous release to the new release.

       @param pdl - PDL reference
    '''

    debug("Entry _check_profiles %s" % fixlev)


    # Get total number of servers  for each profile in the FINDSERVERS based on a cell, node,
    # and profile name, and save it in a dictionary.

    findservers_group = utils.group_findservers(pdl)

    # Now get the group servers based on the selected servers, and compare it against the
    # findservers group which has the total number of servers, and if not equal, then
    # it indicates that not all the servers within the findservers profile is selected, so the
    # warning message is issued.
    # Note that the key for both findservers and servers dictionaries are the same.  The value
    # for the servers dictionary is an index list which contains the index into the SERVERS
    # pdl.

    servers_group = utils.group_servers(pdl)
    for groupservers_entry in servers_group.keys():
        debug("Group Servers key: %s   Total number of servers: %d" % (groupservers_entry, findservers_group[groupservers_entry]))
        servers_index = servers_group[groupservers_entry]
        if servers_index is not None:
            total_numberof_servers = findservers_group[groupservers_entry]
            numberof_selected_servers = len(servers_index)
            debug("Group Servers Key: %s   Total number of servers: %d   Selected number of servers: %d" % (groupservers_entry, total_numberof_servers, numberof_selected_servers))
            if total_numberof_servers != numberof_selected_servers:
                msgl =[]
                msgl.append(groupservers_entry)
                infomsg = getmsg("CFG1137W", "It is strongly recommended that all servers under the  profile: {0} be migrated at one time. Else it may cause instability.", msgl)
                print infomsg

    debug("Exit _check_profiles")


def _set_default_pdlvalues(pdl):
    '''
       Sets the default values in the pdl.  The first selected server' values
       are set in the pdl.

       @param pdl - PDL reference
    '''

    debug("Entry _set_default_pdlvalues %s" % fixlev)

    # set default tema values.

    configtema.set_defaultvalues(pdl)

    # set default MS values.

    configms.set_defaultvalues(pdl)

    # set default TT values.

    configtt.set_defaultvalues(pdl)

    # set default soa values.

    configsoa.set_defaultvalues(pdl)

    # set default gc values.

    configgc.set_defaultvalues(pdl)

    # set default tpv values.

    configtpv.set_defaultvalues(pdl)

    # set default de values.

    configde.set_defaultvalues(pdl)

    debug("Exit _set_default_pdlvalues")


def find_configservers(pdl,
                       ptype,
                       inputproperties_list = None,
                       version = None):
    '''
       Finds the configured servers by reading the input.properities file in
       the runtime directory, and prompt the user to choose server/s for unconfig.

       @param pdl - PDL reference
       @param ptype - Process type
       @param inputproperties_list - A list of input properties files, default is None
       @param version - Product version to process, based on the product version a
                        specific method is called to process

    '''

    debug("Entry find_configservers %s" % fixlev)
    debug("  ptype = %s" % ptype)
    if inputproperties_list is None:
        debug("  input properties list = None")
    else:
        debug("  input properties list =\n    %s" % "\n    ".join(inputproperties_list))
    debug("  version = %s" % version)

    serversfound = False

    try:
        if version is None:
            version = checkversion.convert_version(utils.getData(pdl, "VERSION"))
            debug("Version from PDL: %s" % version)

        # Based on config version, call the appropriate method to process it.

        mversion = version[0]
        if version >= common_constants.BASE_AD72VERSION:
            debug("Processing AD 72")
            utils.setData(pdl, "PREAD72MIGRATE", False, "findconfigservers")
            serversfound = process_ad72(pdl,
                                        inputproperties_list,
                                        ptype,
                                        version)
        elif version >= common_constants.BASE_ECAMVERSION and version < common_constants.BASE_AD72VERSION:
            debug("Processing ecam")
            utils.setData(pdl, "PREAD72MIGRATE", True, "findconfigservers")
            serversfound = process_ecam_dc61(pdl,
                                             ptype,
                                             version,
                                             common_constants.ECAM)
        elif version >= common_constants.BASE_AD71VERSION and version < common_constants.BASE_ECAMVERSION:
            debug("Processing AD71")
            utils.setData(pdl, "PREAD72MIGRATE", True, "findconfigservers")
            serversfound = process_ad71(pdl,
                                        ptype,
                                        version,
                                        common_constants.AD71)
        elif mversion < 7:
            debug("Processing DC 61")
            utils.setData(pdl, "PREAD72MIGRATE", True, "findconfigservers")
            serversfound = process_ecam_dc61(pdl,
                                             ptype,
                                             version,
                                             common_constants.DC61)
        else:
            log.logError("Invalid version %s" % (version))

        debug("Entry find_configservers - %s" % (serversfound))

        return serversfound

    except Exception, errmsg:
        log.logException("Error occurred processing find config servers - %s" % (errmsg))
        print getmsg("CFG2016E", "Error occurred while processing find config servers - {0}", [errmsg])
        raise


def find_configuredprobes(pdl, ptype=None):
    '''
       Finds the configured probes for the selected servers and saves it in the
       pdl for future use.

       @param pdl - PDL reference
       @param ptype - For reconfig which has the value defined in constants class
    '''

    debug("Entry find_configuredprobes %s" % fixlev)

    global common_constants
    process = False

    try:

        # If this is for reconfig, we already have all the information from the
        # input.properties, so we just get it from the pdl.

        if ptype == common_constants.RECONFIG:
            process = readconfigfiles.set_probes_for_reconfig(pdl, pdl_entry_names_probes)
        else:

            # Get the FULLSERVERS pdl, and for all the servers, check its
            # input.properties and determine if the probes are specified and
            # save it in the pdl.   The information is checked against the
            # current probes (After the interview) and it is determined if
            # a request is needed to send reconfig to data collector for start/stop
            # the probe.
            # This probe checking is only done if the server has already been
            # configured, otherwise, it is skipped.  In the CONFIGURED pdl, which
            # has the version of the configured app server if already configured.

            inputproperties_filelist = []
            key_list = []
            fullservers = utils.getData(pdl, "FULLSERVERS")
            path = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "runtime"))
            for serveri in range(len(fullservers)):
                configured = utils.getData(pdl, "CONFIGURED", serveri)

                # Check the version of the server and get its first major version
                # number which is its first digit, and if it is over version 7 and
                # higher, then we have input.properties, so it will be checked.

                mver = configured[0]
                mver_int = utils.isInt(mver)
                if mver_int == True:
                    if configured >= 7:
                        inputproperties_prefix = utils.getServerBasedFilePrefix(pdl, serveri)
                        key_list.append(serveri)
                        input_file = os.path.join(path, inputproperties_prefix + "input.properties")
                        debug("Processing probe for %s" % (input_file))
                        inputproperties_filelist.append(input_file)
                        process = True

            if process == True:
                debug("List of input properties files: %s" % (inputproperties_filelist))

                # set the pdl probes.

                process = readconfigfiles.set_probes(pdl, inputproperties_probe_props, inputproperties_filelist, key_list)

    except Exception, errmsg:
        process = False
        debug("Error occurred processing find probes - %s" % (errmsg))
        log.logException("Error occurred processing find probes")
        print "%s" % getmsg("CFG2017E", "Error occurred while processing find probes - {0}", [errmsg])

    debug("Exit find_configuredprobes")
    return process


def find_configservers_for_migrate_73(pdl,
                 ptype,
                 inputproperties_list,
                 migratehome_dic):
    
    
    '''
       Process AD73 SCM AI install to find the configured servers.  An input.properties file for
       a specific server that resides in "runtime" directory indicates that a server
       has been configured.

       @param pdl = PDL reference
       @param inputproperties_list - A list of input properties files, default is None
       @param ptype - Process type like reconfig, migrate
       @param migratehome_dic - dictionalry with dchome as key and version as value
    '''

    global common_constants

    debug("Entry find_configservers_for_migrate_73 %s - %s - %s - %s" % (fixlev, inputproperties_list, ptype, migratehome_dic))

    # Read the files in runtime directory, and choose the files that have
    # suffix of "input.properties".  Based on the type, the DC home is
    # selected.

    files = []
    available_servers_list = []
    serversfound = False
    
    # If input properties list is provided, then we don't need to search for it.
    # Otherwise we need to list the files and select those files that are
    # input.properties but not dcless.input.properties files.

    for migratehome in migratehome_dic.keys():
        path = os.path.normpath(os.path.join(migratehome, "runtime"))
        version = migratehome_dic[migratehome]
        debug("Path %s version=%s" % (path, version))
        files = []
        if inputproperties_list is None:
            if os.path.isdir(path):
                for inbase in os.listdir(path):
                    debug ("inbase= %s" % inbase)
                    infile = os.path.normpath(os.path.join(path, inbase))
                    if os.path.isfile(infile):
                        debug("File name: %s" % infile)
                        # Make sure this is an input.properties file but not also 
                        # a dcless.input.properties file as well.
                        r = pat_input.match(infile)
                        if r is None or r.group(1) is None: continue
                        debug("Found input.properties - %s" % infile)
                        files.append(infile)      
                process, server_list = read_configfile_for_dchome(pdl, ptype, files, migratehome, version)
                if process == True:
                    available_servers_list.extend(server_list)
            else:
                debug("Config path does not exist - %s" % path)
        
        #else:
        #    files = inputproperties_list
    if len(available_servers_list) > 0:
        # Prompt to select the servers for migrations.

        utils.setData(pdl, "FINDSERVERS", available_servers_list, "findconfigservers")
        doprompt.do_prompt("SERVERS", pdl, None, None, True)

        # Check the selected servers in the FINDSERVERS and make sure it does
        # exists, and create PDLs for the selected servers.

        serversfound = _check_findservers(pdl, ptype)

    
    debug("find_configservers_for_migrate_73 Exit - %s" % (serversfound))
    return serversfound

def read_configfile_for_dchome(pdl, ptype, files, migratehome, version):
    
    debug("read_configfile_for_dchome Entry %s" % migratehome)
    serversfound = False
    find_serverlist = []
    process = False
    pdl["MIGRATEHOME"][0:2] = migratehome, "findconfigservers"
    if len(files) > 0:
        debug("Input properties files to be processed:\n    %s" % "\n    ".join(files))
        process, find_serverlist = readconfigfiles.read_ad72_inputproperty_file(pdl,
                                                                                pdl_entry_names,
                                                                                inputproperties_props,
                                                                                files,
                                                                                ptype,
                                                                                version, 
                                                                                False)
    else:
        debug("No files with suffix of input.properties found")
        debug("No servers have been configured")

    debug("read_configfile_for_dchome Exit - %s, %s" % (process, find_serverlist))
    return process, find_serverlist