#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
    gdc_extract.py
    Usage: gdc_extract.py archive_file out_file
    Extracts archive_file to a directory selected by the user
    Writes the dir selected by the user to out_file
'''

import os
import sys
import codecs
import tarfile
import zipfile
import re

from java.lang import System
from PrintWrapper import PrintWrapper
from constants import constants

# Provide for Unicode output
#sys.stdout = codecs.getwriter('utf-8')(sys.stdout)

import utils
import debugUtil
import platformUtil
import resourcebundle
from debugUtil import logDebug as debug
from resourcebundle import getmsg

_YES = "1"
_NO = "2"
DEFAULT_INSTALL = "default"
CUSTOM_INSTALL = "custom"
EXIT = "q"
yesno_dic = {_YES: True, _NO: False}
dc_version=None

################################################################################
### Prompting functions.
### Consider re-using doprompt.py rather than more-or-less reimplementing
### similar functions here
################################################################################

def __prompt(prompt_message, default_value):
    debug("__prompt entry")
    local_answer = ""
    default_msg = resourcebundle.getmsg("DEFAULT", "default is:")
    _YESNO_OPS = resourcebundle.getmsg("YESNO", "[1 - YES, 2 - NO]")
    entervalue_msg = getmsg("ENTERVALUE", "Enter a number")
    default_str = default_value
    if (default_value == True): default_str = _YES
    if (default_value == False): default_str = _NO
    if (utils.isSet(default_value)):
        msg = "%s %s \n%s [%s %s]: " % (prompt_message, _YESNO_OPS, entervalue_msg, default_msg, default_str)
    else:
        msg = "%s %s: " % (prompt_message, _YESNO_OPS)
    print msg
    local_answer = raw_input()
    debug("__prompt exit %s" % local_answer)
    return str(local_answer)

def __validate_entry(local_answer, default_value):
    is_valid = 0
    answer = local_answer
    if (len(answer.strip()) == 0):  #user clicked on Enter
        if (default_value != None):
            #there is a default value, accepting it
            is_valid = 1
            answer = default_value
        else:
            is_valid = 0
    else:
        local_answer = local_answer.strip()
        if (local_answer == _YES or local_answer == _NO):
            is_valid = 1
            answer = yesno_dic[local_answer]
    return (is_valid, answer)


def prompt_dir_create(confighome):
    '''
        Prompt for permission to create the specified directory.
        TODO: might be good to include the dirname in the message.
    '''
    debug("prompt_dir_create entry")
    msg = resourcebundle.getmsg("CFG5006I", "Directory: {0} doesn't exist. Is it ok to create?", [confighome])
    default_value = True
    is_valid = 0
    while (not is_valid):
        local_answer = __prompt(msg, default_value)
        is_valid, answer = __validate_entry(local_answer, default_value)
    debug("prompt_dir_create exit %s" % answer)
    return answer



def prompt_confighome(dc_home_list, default_confighome):
    debug("prompt_confighome entry")
    confighome = ""
    if len(dc_home_list) == 0:
        prompt_message = resourcebundle.getmsg("CFG1037I", "Enter the directory to which the Data Collector should be installed: ")
        confighome = prompt_confighome_pristine(prompt_message, default_confighome)
    else:
        prompt_message = resourcebundle.getmsg("CFG1156I", "List of Data Collector installation directories discovered: ")
        confighome = prompt_confighome_update(prompt_message, dc_home_list, default_confighome)
    debug("prompt_confighome exit %s" % confighome) 
    return confighome

def prompt_confighome_pristine(prompt_message, default_confighome):
    '''
        Prompt for confighome and return the value entered.
        The default value is accepted if user hits RETURN
    '''
    debug("prompt_confighome_pristine entry")
    local_answer = ""
    default_msg = resourcebundle.getmsg("DEFAULT", "default is:")
    msg = "%s\n[%s %s]: " % (prompt_message, default_msg, default_confighome)
    is_valid = False
    
    print msg
    local_answer = raw_input()
    local_answer = local_answer.strip()
    if local_answer == "": 
        local_answer = default_confighome
        is_valid = True
        #elif local_answer == EXIT:
        #    is_valid = True
        #else:
        #    is_valid  = is_valid_confighome(local_answer)
        #if not is_valid:
        #    invalid_reponse_msg = getmsg("CFG5013E", "Invalid response:")
        #    print "\n%s '%s'\n" % (invalid_reponse_msg, local_answer)
        #    local_answer = None 
    debug("prompt_confighome_pritine exit %s" % local_answer)    
    return local_answer


def prompt_confighome_update(prompt_message, dc_home_list, default_value):
    debug("prompt_confighome_update")
    local_answer = None
    debug("defaul=%s num options=%d" % (default_value, len(dc_home_list)))
    entervalue_msg = resourcebundle.getmsg("CFG1157I", "Enter a number or enter the full path the directory to which the Data Collector should be installed:")
    default_msg = resourcebundle.getmsg("DEFAULT", "default is:")
    invalid_response_msg = getmsg("CFG5013E", "Invalid response:")
    #if we have only one option set it as default
    if len(dc_home_list) == 1: default_value = dc_home_list[0]
    msg1 = prompt_message
    index = 1
    items_dic = {}
    options_str = ""
    has_default = False
    default_index = -1
    answer = None
    for item in dc_home_list:
        options_str += "\n  %d. %s" % (index, item.encode('utf-8'))
        items_dic[str(index)] = item
        if item == default_value: default_index = str(index)
        index += 1
    if default_index > -1 and default_value in dc_home_list:
        has_default = True
    msg2 = "\n" + entervalue_msg
    if has_default:
        msg2 +=  " [%s %s]" % (default_msg, default_index)
    msg = "%s %s %s: " % (msg1, options_str.decode('utf-8'), msg2)
    is_valid = False
    while not is_valid:
        print msg
        local_answer = raw_input()
        local_answer = local_answer.strip()
        debug("user answer: %s" % local_answer)
        if local_answer == EXIT:
            is_valid = True
            answer = local_answer
        elif local_answer == "" and has_default:
            answer = default_index
            is_valid = True
        elif local_answer.isdigit():
            debug("user selected existing directory")
            answer = items_dic.get(local_answer, None)
            if (answer != None): 
                is_valid = True
        else:
            debug("user entered directory name")
            is_valid = True
            answer = local_answer
        if not is_valid:
            print "\n%s '%s'\n" % (invalid_response_msg, local_answer)
            local_answer = None
            answer = None
    return answer

def prompt_install_type():
    '''
        Prompt for the type of install to perform.
        Returns "default", "custom" or "q" if the user elected to quit
    '''

    is_answered = False;
    debug("prompt_install_type entry")
    prompt_msg = resourcebundle.getmsg("CFG1096I", "Choose the type of install to perform. Enter 'q' to quit the utility without installing the datacollector.\n1. default install \n2. custom install")
    default_msg = resourcebundle.getmsg("DEFAULT", "default is:")
    entervalue_msg = resourcebundle.getmsg("ENTERVALUE", "Enter a number")
    msg = "%s \n%s [%s %s]: " % (prompt_msg, entervalue_msg, default_msg, "1")

    while not is_answered:
        # prompt user for DC home
        print msg
        answer = raw_input()
        answer = answer.strip()
        if answer == "" or answer == "1":
            answer = DEFAULT_INSTALL
            is_answered = True
        elif (answer == "2"):
            answer = CUSTOM_INSTALL
            is_answered = True
        elif (answer == EXIT):
            is_answered = True
            
    debug("prompt_install_type exit '%s'" % answer)
    return answer



################################################################################
### Functions for zip/tar extraction
### utils.py candiate?
################################################################################

def extract_tar(archive, toDir):
    '''
        Extract the tar archive to toDir
    '''
    debug("extracting: " + archive + " to " + toDir + "...")
    if os.geteuid() != 0:
        #running as non-root user, need to use OS command to extract archive in order to keep file permission
        debug("Running extractor as non-root user")
        cmd = "`mkdir -p %s && cd %s && tar -xf %s`" % (toDir, toDir, os.path.abspath(archive))
        if archive.endswith(".gz"):
            cmd = "`mkdir -p %s && cd %s && gzip -dc %s | tar -xf -`" % (toDir, toDir, os.path.abspath(archive))
        if not os.system(cmd) == 0:
            raise Exception("Failed to extract %s to %s" % (archive, toDir))
    else:
        try:
            tar = tarfile.open(archive, "r");
            tar.extractall(toDir)
            tar.close()
        except NotImplementedError:
            # tar.extractall fails for root on AIX, so fall back to system command
            _type_, _value_, _tbck_ = sys.exc_info()
            debug("Failed extract from tar. %s %s" % (str(_type_), str(_value_)))
            if archive.endswith(".gz"):
                cmd = "`mkdir -p %s && cd %s && gzip -dc %s | tar -xf -`" % (toDir, toDir, os.path.abspath(archive))
            else:
                cmd = "`mkdir -p %s && cd %s && tar -xf %s`" % (toDir, toDir, os.path.abspath(archive))
    
            if not os.system(cmd) == 0:
                raise Exception("Failed to extract %s to %s" % (archive, toDir))
        except: 
            debug ("Failed to extract files from %s" % archive)
            raise 


def extract_zip(archive, toDir):
    '''
        Extract the zip archive to toDir.
        Note that there is no extractall for zipfile in python 2.5. A
        consequence is that file attributes (e.g. rwx) are not preserved by
        this function.
    '''
    debug("extracting: " + archive + " to " + toDir + "...")
    zip = zipfile.ZipFile(archive, "r");
    for name in zip.namelist():
        dest = os.path.join(toDir, name)
        destdir = os.path.dirname(dest)
        if not os.path.isdir(destdir):
            os.makedirs(destdir)
        if not name.endswith("/"):
            data = zip.read(name)
            f = open(dest, "wb")
            f.write(data)
            f.close()
    zip.close()


def extract_archive(archive, dest):
    '''
        Extract the archive to the dest directory.
        Supported archive formats (identified by file extension):
        zip, tar, tar.gz
    '''
    debug("extract_archive entry")
    if archive.endswith(".zip"):
        extract_zip(archive, dest)
    elif archive.endswith(".tar") or archive.endswith("tar.gz"):
        extract_tar(archive, dest)
    else:
        raise Exception("Cannot determine archive type from file name: "
                        + archive)

################################################################################


def is_env_var_set(name):
    '''
        Test whether the environment variable with the specified name is
        defined.
        Returns 1 if defined, else 0
    '''
    import os
    value = os.getenv(name)
    if (value != None):
        return len(value) > 0
    else:
        return 0;


def get_default_confighome():
    '''
        Return the default confighome location, calculated from CANDLE_HOME
    '''
    candleHome = os.getenv("CANDLE_HOME")
    return os.path.join(candleHome, "dchome")

def get_itm_log_dir():
    '''
       Return log directory under CANDLEHOME 
    ''' 
    debug("get_itm_log_dir entry")  
    candleHome = os.getenv("CANDLE_HOME")
    logDir = candleHome
    if platformUtil.isWindows():
        logDir = os.path.join(candleHome, "InstallITM")
    else:
        logDir = os.path.join(candleHome, "logs")
    debug("get_itm_log_dir exit %s" % logDir)
    return logDir

def get_install_history_file():
    '''
        Return the path/name of the file in which GDC install history is saved
    '''
    debug("get_install_history_file entry")
    candleHome = os.getenv("CANDLE_HOME")
    file = "gdc_install_history.txt"
    if platformUtil.isWindows():
        path = "TMAITM6"
    else:
        path = "config"

    debug("get_install_history_file exit %s" % os.path.join(candleHome, path, file))
    return os.path.join(candleHome, path, file)


def read_install_history(history_file):
    '''
        Read the install history file.
        Returns a dictionary of DC installation home dirs keyed by DC version
    '''
    line_list = []
    dc_version_dic = {}
    is_dir_removed = False
    dc_home_list = []
    debug("read_install_history entry")
    if os.path.isfile(history_file):
        try:
            fdescr = open(history_file, "r")
            lines = []
            while 1:
                buffer = fdescr.readline()
                if not buffer: break
                if len(buffer.strip()) > 0:
                    lines.append(buffer)
                
            fdescr.close()
            for line in lines:
                dir_name = line.split('\t')[0].strip()
                version = line.split('\t')[1].strip()
                installed_dc_version = get_dc_version(dir_name, version)
                if installed_dc_version == None:
                    debug("Found invalid installation dir: " + dir_name)
                    is_dir_removed = True
                else:
                    line_list.append(line)
                    if dir_name not in dc_home_list:
                        dc_home_list.append(dir_name)
                    if dc_version_dic.has_key(version):
                        dc_version_dic[version].append(dir_name)
                    else:
                        dc_version_dic.setdefault(version, []).append(dir_name)
        except:
            _type_, _value_, _tbck_ = sys.exc_info()
            errmsg = str(_value_)
            debug("Failed to read from the history file: %s. Error %s" % (history_file, errmsg))

   
    debug("line_list %s" % str(line_list))

    if is_dir_removed:
        # update the history file, removing invalid installation dirs
        try:
            fileh = open(history_file, 'w')
            for line in line_list:
                print >> fileh, line
            fileh.close()
            debug("Updated the history file: " + history_file)
        except:
            debug("Failed to update the history file: " + history_file)
    debug("read_install_history exit %s" % str(dc_version_dic))
    return (dc_version_dic, dc_home_list)

def write_install_history(history_file, confighome, dc_version):
    '''
        Update the install history file with the new dir to which
        the GDC has been installed

        History file is currently just a list of install dirs, one per line.
    '''
    msg = "Updating install history file %s with new ITCAM_CONFIGHOME %s." % (history_file, confighome)
    debug(msg);
    try:
        # append absolute path of the new directory into the file
        fileh = open(history_file, 'a')
        print >> fileh, "%s\t%s" % (os.path.abspath(confighome), dc_version)
        fileh.close()
    except:
        debug("Failed to write ITCAM_CONFIGHOME to history file: " + history_file);


def write_confighome(confighome, filename):
    '''
        Write the value of confighome to a file so it can be read by invoker
    '''
    debug("Writing ITCAM_CONFIGHOME to " + filename);
    try:
        fileh = open(filename, 'w')
        # write absolute path for the directory into the file
        print >> fileh, os.path.abspath(confighome)
        fileh.close()
    except:
        debug("Failed to write ITCAM_CONFIGHOME to file: " + filename);
        pass


def read_confighome(response_file):
    '''
        Read ITCAM_CONFIGHOME from the response file
        Return the value read from the file
        Raises an exception if the value could not be read
    '''
    debug("read_confighome entry")
    confighome = ""
    for line in open(response_file):
        if "ITCAM_CONFIGHOME" in line:
            confighome = line.split("=")[1]
            confighome = str.strip(confighome)
            
    if confighome == "":
        raise Exception("Failed to read ITCAM_CONFIGHOME from file: "+ response_file)
    debug("read_confighome exit %s" % confighome)
    return confighome


def create_confighome(confighome, isInteractive):
    '''
        Create the specified dir, or validate that it is empty.
        If the dir does not exist, create it.
        If isInteractive, prompt before creating the dir.
        If the dir does exist, validate that it is empty.
        Return True if the validation passes, otherwise False
    '''
    debug("create_confighome entry %s" % confighome)
    is_confighome_ok = True
    
    # check if dir exists, prompt to create it if not
    if (not os.path.exists(confighome)):
        answer = True
        if (isInteractive):
            # prompt for permission to create the dir
            answer = prompt_dir_create(confighome)
        
        if (answer == True):
            # try to create the dir
            result = utils.create_dir_tree(confighome)
            if (result != True):
                is_confighome_ok = False
                errmsg = resourcebundle.getmsg("CFG1038E", "Unable to create the directory - {0}.", [confighome])
                print errmsg
                raise Exception(errmsg)
        else:
            # user did not give permission to create the dir
            is_confighome_ok = False
            errmsg = resourcebundle.getmsg("CFG1155W", "User canceled from Data Collector install")
            print errmsg
            raise KeyboardInterrupt(errmsg) 
            

        
    debug("create_confighome exit %s" % str(is_confighome_ok))    
    return is_confighome_ok

def isValidLocalPath(filename_str):
    '''
        validates if string is a valid path to a file on a local machine
        this is restrictive checking, invalidates paths that contain characters
        not allowed by WebSphere
    '''

    is_valid = 1
    debug("isValidLocalPath(%s) entry" % filename_str)
    #check that filename is not exceeding 255 char length
    if (len(filename_str) > 255): 
        print "Maximum directory name length of 255 characters is exceeded"
        return 0
    
    file_name = os.path.basename(filename_str)
    debug("file_name: " + file_name)
    dir_path = os.path.dirname(filename_str)
    debug("dir_path: " + dir_path)
    
    
    folders = []
    folders.append(file_name)
    while 1:
        dir_path, dir = os.path.split(dir_path)
        if dir != "":
            folders.append(dir)
        else:
            folders.append(dir_path)
            break
    
    debug("path tokens %s" % folders)
    pattern = '[`\*:<\\\>\?/\"\|]'
    for index in range(len(folders) - 1):
        dir_name = folders[index]
        is_valid = not re.search(pattern, dir_name)
        if is_valid == 0:
            invalid_reponse_msg = getmsg("CFG5013E", "Invalid response:")
            print "\n%s '%s'\n" % (invalid_reponse_msg, filename_str)
            print "%s" % getmsg("NO_SPECIAL_CHARS_ALLOWED", "The following characters are not allowed as part of a directory or file name: *, \", <, >, :, ?, |, `")
            break
        
    if (is_valid == 0): debug("Invalid characters in path: %s" % dir_name)
        
    debug("isValidLocalPath exit %s" % str(is_valid))
    return is_valid


def is_valid_confighome(confighome):
    global dc_version
    debug("is_valid_confighome entry %s" % confighome)
    is_valid = False
    is_valid_dir_path = isValidLocalPath(confighome)
    if is_valid_dir_path:
        if get_dc_version(confighome, dc_version) == None:
            debug("%s is valid for install " % confighome)
            is_valid = True
        else:
            msg = resourcebundle.getmsg("CFG1158W", "Directory {0} already contains Data Collector files", [confighome])
            print
            print msg
    debug("is_valid_confighome exit %s" % is_valid)    
    return is_valid

def prompt_reuse_dc(confighome):
    debug("prompt_reuse_dc entry")
    msg = resourcebundle.getmsg("CFG1165W", "Directory {0} already contains Data Collector files. Do you want to continue with the existing DC install?", [confighome])
    default_value = True
    is_valid = 0
    while (not is_valid):
        local_answer = __prompt(msg, default_value)
        is_valid, answer = __validate_entry(local_answer, default_value)
    debug("prompt_reuse_dc exit %s" % answer)
    return answer

        
def has_dc_installed(confighome):
    global dc_version
    confighome_has_dc = False
    debug("has_dc_installed entry")
    if get_dc_version(confighome, dc_version) == None:
        debug("no dc install found in %s " % confighome)
        confighome_has_dc = False
    else:
        confighome_has_dc = True
        msg = resourcebundle.getmsg("CFG1158W", "Directory {0} already contains Data Collector files", [confighome])
        debug("Directory %s already contains Data Collector files" % confighome)
    debug("has_dc_installed exit %s" % confighome_has_dc)
    return confighome_has_dc
    
def get_dc_version(confighome, version):
    '''
        returns numeric DC version if confighome contains a valid DC install, otherwise will return None
    '''
    
    debug("get_dc_version entry %s %s" % (confighome, version))
    is_valid = True
    installed_dc_version = None
    dchome = os.path.join(confighome, version)
    dirs = [
         dchome,
         os.path.join(dchome, 'bin'),
         os.path.join(dchome, 'itcamdc'),
         os.path.join(dchome, 'toolkit')
    ]
    
    version_file = os.path.join(dchome, 'itcamdc', 'etc', 'version.properties')
    files = [version_file]
    
    for dir in dirs:
        debug("dir '%s'" % dir)
        if not os.path.exists(dir):
            is_valid = False
            msg = ("Directory '%s' not found" % dir)
            debug(msg)
            break

    if is_valid:
        for file in files:
            if not os.path.isfile(file):
                is_valid = False;
                msg = "File %s is missing from %s" % (file, confighome)
                debug(msg)
                break

    if not is_valid:
        msg = "Directory %s does not contain a valid Data Collector installation" %(confighome)
        debug(msg)
    else:
        for line in open(version_file):
            if line.startswith("probe.release.version="):
                version_str = line.split('=')[1]
                installed_dc_version = re.sub('[^0-9]', '', version_str)
    debug("get_dc_version exit %s" % str(installed_dc_version))
    return installed_dc_version;

################################################################################
### Main
################################################################################
try:
    log_dir = get_itm_log_dir()
    log_filename = os.path.join(log_dir, "gdc_install.log")
    msg_filename = os.path.join(log_dir, "gdc_install.msg")
    debugUtil.set_logging(log_filename, msg_filename)
    
    # validate command line and environment
    if len(sys.argv) < 3:
        raise Exception("Invalid command line: [archive to extract] [file to write ITCAM_CONFIGHOME]")
    if not is_env_var_set("CANDLE_HOME"):
        raise Exception("Error: Environment variable CANDLE_HOME is not defined")
    
    archive = sys.argv[1]
    confighome_file = sys.argv[2]
    
    debug("archive=%s confighome_file=%s" % (archive, confighome_file))
    # optional system properties
    response_file=System.getProperty("response.file")
    if response_file ==  "":
        response_file = None
    confighome=System.getProperty("config.home")
    if confighome ==  "":
        confighome = None
    dc_version = System.getProperty("dc.version") or "7.3"
    debug("response.file=%s confighome=%s dc.version=%s" % (response_file, confighome, dc_version))
    
    # Instantiate the global constants class
    common_constants = constants()
    
    # Set the print wrapper for stdout and stderr.
    PrintWrapper(common_constants.STDOUT)
    PrintWrapper(common_constants.STDERR)
    
    
    # initialize resource bundle
    resourcebundle.set_resourcebundle()
    
    user_exit_msg = resourcebundle.getmsg("CFG1155W", "User canceled from Data Collector install")

    DEFAULT_INSTALL = resourcebundle.getmsg("DEFAULT_STRING","default")
    CUSTOM_INSTALL = resourcebundle.getmsg("CUSTOM_STRING","custom")
    EXIT = resourcebundle.getmsg("QUIT_CHAR","q")

    # Special handling for mocking
    if len(EXIT) > 1 and EXIT.endswith("]"): EXIT = EXIT[-2:-1]
    
    do_extract = True
    confighome_set = False
    
    history_file = get_install_history_file()
    installed_dc_version_dic, config_home_list = read_install_history(history_file)
    
    if (installed_dc_version_dic.has_key(dc_version)):
        confighome = installed_dc_version_dic[dc_version][0]
        debug("%s version of the datacollector is already installed under %s, skipping datacollector extraction" % (dc_version, confighome))
        do_extract = False
        confighome_set = True
    else:
        msg = resourcebundle.getmsg("CFG1094I", "In the next steps, the ConfigWASDC utility for configuring the data collector will be installed and started from the directory you specify.")
        print msg    
        confighome_set = False
        if response_file == None and confighome == None:
            debug("Installing in interactive mode")
            # interactive mode
            confighome = None
            default_confighome = get_default_confighome()
            check_install_type = True
            install_type = CUSTOM_INSTALL
            
            while not confighome_set:    
                if check_install_type:
                    install_type = prompt_install_type()
                if install_type == DEFAULT_INSTALL:
                    debug("processing default install")
                    confighome = default_confighome
                elif install_type == CUSTOM_INSTALL:
                    debug("processing custom install")
                    local_answer = prompt_confighome(config_home_list, default_confighome);
                    if (local_answer == EXIT):
                        print user_exit_msg
                        sys.exit(1)
                    else:
                        confighome = local_answer
                elif install_type == EXIT:
                    print user_exit_msg
                    debug("Exiting per user input")
                    sys.exit(1)
                    
                if isValidLocalPath(confighome):
                    if not has_dc_installed(confighome):
                        confighome_set = create_confighome(confighome, True)
                    else:           #directory already contains DC files installed manually
                        if prompt_reuse_dc(confighome):
                            #skip extraction
                            confighome_set = True
                            do_extract = False
                            write_install_history(history_file, confighome, dc_version)
                        else:
                            confighome_set = False
                            if install_type == DEFAULT_INSTALL:
                                check_install_type = False
                                install_type = CUSTOM_INSTALL
                else:
                    if install_type == CUSTOM_INSTALL:
                        check_install_type = False
                        
                #if is_valid_confighome(confighome):
                    
        else:
            # silent mode
            debug("installing in silent mode")
            if response_file != None:
                # confighome defined in response file
                confighome = read_confighome(response_file)
           
            debug("confighome=%s" % confighome)
            if confighome != None:
                if is_valid_confighome(confighome):
                    confighome_set = create_confighome(confighome, False)
                else:
                    raise Exception("Unable to install into the directory %s" % confighome)
            else:
                raise Exception("DC confighome not set in silent mode")
    
    if confighome_set:
        if do_extract:
            debug("extracting datacollector files")
            extract_archive(archive, confighome)
            write_install_history(history_file, confighome, dc_version)
        else:
            msg = resourcebundle.getmsg("CFG1120I",
           "The installation of {0} version of  data collector found under {1} and will be reused. The data collector installation is skipped.", [dc_version, confighome])
            print
            print msg
    else:
        raise Exception('Unable to get datacollector home directory, exiting')
    
    infomsg = resourcebundle.getmsg("CFG1095I","In the next step, you configure the Data Collector using the ITCAM Data Collector for WebSphere Configuration Utility")
    print
    print infomsg
    
    write_confighome(confighome, confighome_file)
    sys.exit(0)
    
except KeyboardInterrupt, message:
    #suppress trace if user selected to quit
    debug(message)
    sys.exit(1)
except SystemExit, exitcode:
    # Suppress trace for intentional system exit
    if exitcode.code == 0: sys.exit(0)
    
    sys.exit(1)

except Exception, message:
    import traceback
    _exctype, _excval, _exctb = sys.exc_info()
    traceback.print_exception(_exctype,_excval,_exctb)
    try:
        print message
        #print "%s %s" % (resourcebundle.getmsg("CFG1159E", "Unexpected exception detected in gdc_extract.py"), message)
    except Exception:
        print "Unexpected exception detected in gdc_extract.py %s" % message
    sys.exit(1)
