#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

import sys
from java.lang import System
import codecs
import os
import re
from java.net import InetAddress

fileh = None

def isSet(var):
    if var != None and var != "":
        return 1
    return 0

def toList(str):
    lineSeparator = System.getProperty('line.separator')
    list = str.split(lineSeparator)
    return list

def write_to_outfile(record):
    
    global fileh
    fileh.write(record)
    fileh.write(os.linesep)
    fileh.flush()

def wsadminToList(inStr):
    outList=[]
    if len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']':
        tmpList = inStr[1:-1].split() #splits space-separated lists,
    else:
        tmpList = inStr.split("\n")   #splits for Windows or Linux
    for item in tmpList:
        item = item.rstrip();         #removes any Windows "\r"
        if (len(item)>0):
            outList.append(item)
    return outList

if (len(sys.argv) < 1):
        print "Usage: wsadmin -conntype NONE -f getTopology.jacl output"
        sys.exit(1)

filename = sys.argv[0]
fileh = codecs.open(filename, mode='w', encoding='utf-8')

########## get cell ###########
cellId = None
hostname = None
hostaddr = None
local_node_flag = 1
try:
    cellId = AdminConfig.getid("/Cell:/" )
    cellName = AdminConfig.showAttribute(cellId, "name")
    cellType = AdminConfig.showAttribute(cellId, "cellType" )
    nodes = []
    print "%s %s" % (cellName, cellType)
    nodes = toList(AdminConfig.getid("/Cell:"+cellName+"/Node:/" ))
    if (cellType != "STANDALONE"):
        # For ND environments, we want to discover the local Node and all 
        # the AppServers under that node
        try:
            inet = InetAddress.getLocalHost()
            hostname = inet.getCanonicalHostName()
            print "Hostname=" + hostname
        except:
            type, value, tbck = sys.exc_info()
            print "InetAddress.getLocalHost failed: %s" % value
        try:
            inets = InetAddress.getAllByName(hostname)
            hostaddr = inets[0].getHostAddress()
        except:
            type, value, tbck = sys.exc_info()
            print "InetAddress.getHostAddress failed: %s" % (str(value))
        shostname = hostname.split("." )[0].lower()
        

        # Locate all IP addresses for the local host
        if (not isSet(hostaddr)):
            try:
                inethosts = InetAddress.getAllByName(hostname)
                hostaddrs = []
                i = 0
                while i < len(inethosts):
                    hostaddrs.append(inethosts[i].getHostAddress())
                    i += 1
            except:
                type, value, tbck = sys.exc_info()
                print "InetAddress.getAllByName(" + hostname+ ") failed: " + str(value)
                hostaddrs = [hostaddr]
        else:
            hostaddrs = [hostaddr]
        print "All Host IP=" + str(hostaddrs)
        local_node_flag = 0
    else:
        # For non-ND environments, we want to discover all the Nodes 
        # and all the AppServers under those nodes
        local_node_flag = 1
    
    #print "nodes=%s" % nodes
    if len(nodes) == 0:
        print "INFO there are no nodes available on cell "+cellId
    else:
        for nodeid in nodes:
            if ( not local_node_flag):
                node_local_node_flag = 0
                #Filter only local nodes for the ND environment
                nodehostname = AdminConfig.showAttribute(nodeid, "hostName" ).lower()
                snodehostname = nodehostname.split("." )[0]
                print "Local Host: %s nodehostname: %s" % (shostname, snodehostname)
                if (isSet(shostname) and snodehostname== shostname):
                    print "Unqualified hostnames match: %s=%s" % (snodehostname, shostname)
                    node_local_node_flag = 1
    
                # Check if node hostname specified is an IP address that
                # matches one of our local IP addresses
    
                if ( not node_local_node_flag):
                    pattern = "[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+"
                    if (re.match(pattern,nodehostname)):
                        print " IP node host: %s " % (nodehostname)
                        for ip in hostaddrs:
                            if (ip!= "" and nodehostname == ip):
                               print "IP node hostname %s matches local IP" % (nodehostname)
                               node_local_node_flag = 1
                               break
    
                # Resolve all IP addresses for the node host and compare to local host's
                if ( not node_local_node_flag):
                    # Attempt to resolve the node's hostname
                    try:
                        inetnodes = InetAddress.getAllByName(nodehostname)
                        nodeaddrs = []
                        nodehosti = 0  #forStart
                        while ( nodehosti < len(inetnodes)):
                            inetnode = inetnodes[nodehosti]
                            nodeaddrs.append(inetnode.getHostAddress())
                            nodehosti += 1
                    except:
                        type, value, tb = sys.exc_info()
                        print "*Warning*  getAllByName failed for %s Error: %s " % (nodehostname, value)
                        nodeaddrs = []
    
                    # If any IP address match, we presume we have a local node
                    for nodeaddr in nodeaddrs:
                        print "Node IP: %s " % (nodeaddr)
                        for hostaddr in hostaddrs:
                            print " Host IP: %s" % (hostaddr)
                            if (hostaddr!= "" and nodeaddr == hostaddr):
                                node_local_node_flag = 1
                                break
                            if (node_local_node_flag): break
    
                if ( not node_local_node_flag): continue
            
            nodeName = AdminConfig.showAttribute(nodeid, "name")
            serverIds = toList(AdminConfig.getid("/Cell:"+cellName+"/Node:"+nodeName+"/Server:/" ))
            #print "found %s servers" % len(serverIds)
            for serverid in serverIds:
                servertype = AdminConfig.showAttribute(serverid, "serverType" )
                if (servertype == "APPLICATION_SERVER"):
                    serverName = AdminConfig.showAttribute(serverid, "name" )
                    #print "%s" % serverName
                    write_to_outfile("%s\t%s\t%s" % (serverName, nodeName, cellName))
    fileh.close()
    fileh = None
except:
    type, value, tbck = sys.exc_info()
    print "Error on getting cell config data"
    print "Error message is %s" % value
    if (fileh is not None):
        fileh.close()
    sys.exit(1)
