#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

'''
    Prompt and validate WebSphere root and wsadmin.sh location
'''

fixlev = "51424" # Last APAR, defect or story to update this module

import os
import codecs
from java.io import File

import platformUtil
import wasdiscover
import doprompt
import utils
import wlpConfigUtils

import debugUtil as log
from debugUtil import logDebug as debug
from resourcebundle import getmsg
from ConfigurationException import ConfigurationException

#####################################################################


def get_wasroot(pdl, soa71_migrate = None):
    '''
        Prompt for the WebSphere root
        Locate the wsadmin scripts and choose default
        If requested, prompt for selection of particular wsadmin script

        @param pdl - PDL reference
        @param soa71_migrate - Ture if soa71 migration processing otherwise False
    '''

    debug("Entry get_wasroot(pdl) %s" % fixlev)

    # Here we assume WSADMIN has been selected, after using the initial value
    # of WASROOT to search for it.  So now we reset WASROOT to its final value.

    if platformUtil.isZ():
        get_wasroot_zOS(pdl, soa71_migrate)

    elif platformUtil.IS_OS400_OS():
        get_wasroot_os400(pdl)

    else:
        profile_path_list = []
        was_dir_list = []
        msg = None

        #
        # This half-loop is to re-prompt for an invalid WASROOT
        #
        while True:

            wasroot_saved = pdl["WASROOT"][0]

            #
            # If this soa71 migrate, the WASROOT is already set.
            #
            if not soa71_migrate:
                if utils.is_initial_value(pdl, "WASHOMES"):
                    try:
                        was_dir_list = wasdiscover.discover_was()
                        msg = None
                    except Exception, msgVar:
                        msg = "Error occurred during WebSphere discovery %s" % msgVar
                        debug(msg)
                    pdl["WASHOMES"][0:2] = was_dir_list, "get_wasroot"
                if len(was_dir_list) > 0:
                    doprompt.do_prompt("WASHOMES", pdl, "WASROOT")
                else:
                    print "%s" % getmsg("CFG1134E", "Unable to discover WebSphere installation directory on the system.")
                    doprompt.do_prompt("WASROOT", pdl, alt_prompt=msg)

            wasroot = pdl["WASROOT"][0]
            if not wasdiscover.isWLPHome(wasroot):
                debug("Configuring WebSphere, profile directory is required in wasroot: %s" % wasroot)
                if utils.is_initial_value(pdl, "PROFILES") or wasroot != wasroot_saved:
                    try:
                        profile_name_list, profile_path_list , default_profile_name = wasdiscover.get_was_profiles(pdl, wasroot)
                        if len(profile_name_list) == 0:
                            print "%s" % getmsg("CFG1183E", "Invalid WebSphere home directory selected: {0}", [wasroot])
                            continue
                    except Exception, errmsg:
                        debug("Exception discovering profiles: %s" % errmsg)
                        cxdtemp = utils.getData(pdl, "CREATETEMPLATE")
                        dxdtemp = utils.getData(pdl, "DELETETEMPLATE")
                        if cxdtemp == True or dxdtemp == True:
                            raise ConfigurationException(getmsg("CFG1164E","Can not discover dmgr profiles. Make sure there is a local dmgr profile created"))
                        else:
                            continue
                            raise ConfigurationException(getmsg("CFG1073E","Can not discover profiles. Make sure there is a local application server profile created."))
                    pdl["PROFILES"][0:2] = profile_name_list, "get_wasroot"
                    if pdl["PROFILE"][0] == None or pdl["PROFILE"][0] == "":
                        pdl["PROFILE"][0:2] = default_profile_name, "get_wasroot"
                doprompt.do_prompt("PROFILES", pdl, "PROFILE", None)
                profile_name = pdl["PROFILE"][0]
                profile_home = get_was_profile_home(pdl, profile_name, profile_path_list)
                if profile_home != None:
                    wsadmin = os.path.normpath(os.path.join(wasroot, "bin", platformUtil.getWSAdminScript()))
                    pdl["PROFILEHOME"][0:2] = profile_home, "get_wasroot"
                    pdl["WSADMIN"][0:2] = wsadmin, "get_wasroot"
                else:
                    raise ConfigurationException(getmsg("CFG2042E", "Profile path not found for profile {0}", [profile_name]))
            else:   #this is Liberty profile, skipping profile discovery
                debug("Configuring WAS Liberty, profile discovery skipped for wasroot: %s" % wasroot)
                print "\n%s" % getmsg("CFG1170I", "Found WebSphere Liberty Profile")
                #expand windows path into long name (no tilde in the path)
                pdl["WASROOT"][0:2] = File(wasroot).getCanonicalPath(), "get_wasroot"
                pdl["WLPINSTALLDIR"][0:2] = pdl["WASROOT"][0], "get_wasroot"
                pdl["WLPUSRDIR"][0:2] = wlpConfigUtils.getWlpUserDir(pdl["WLPINSTALLDIR"][0]), "get_wasroot"
                pdl["PROFILE"][0:2] = "", "get_wasroot"
                pdl["PROFILEHOME"][0:2] = pdl["WLPUSRDIR"][0], "get_wasroot"

            # If we fall through to this point we may assume the WASROOT is acceptable and exit the prompt loop
            break

    if pdl["WLPINSTALLDIR"][0] is None:
        profile_home = pdl["PROFILEHOME"][0]
        if wasdiscover.isWASPortal(profile_home):
        
            debug("this is websphere portal server, checking version")
            # Check if this is Portal 7. Portal 7 profile has 2 servers: server 1 and portal server. Without the port parameter wsadmin would try to connect to server1 which might be down.
            portal_version = wasdiscover.getPortalVersion(profile_home)
            debug("portal version=%s" % portal_version)
            if (portal_version != None):
                versionList = portal_version.split(".")
                if versionList[0] == "7":
                    if pdl["PORT"][0] == None :
                        debug("found WAS Portal V7 installation, need to determine SOAP port")
                        soap_port = wasdiscover.getDefaultPortalSoapPort(profile_home)
                        if soap_port != None and utils.isInt(soap_port):
                            pdl["PORT"][0:2] = soap_port, "get_wasroot"
                            pdl["CONNTYPE"][0:2] = "SOAP", "get_wasroot"
                        doprompt.do_prompt("PORT", pdl)


    debug("  PROFILEHOME = %s" % pdl["PROFILEHOME"][0])
    debug("  WSADMIN = %s" % pdl["WSADMIN"][0])

    debug("Exit get_wasroot()")


def get_wasroot_zOS(pdl, soa71_migrate = None):
    '''
        get wasroot for z/OS platforms
        Sets: WASROOT PROFILE  WSADMIN  PROFILEHOME

        @param pdl - PDL reference
        @param soa71_migrate - True if soa71 migration processing otherwise False
    '''

    debug("Entry get_wasroot_zOS(pdl) %s" % fixlev)

    #
    # If this soa71 migrate, the WASROOT is already set.
    #
    derived_wasroot = None
    wsadmin_list = []
    wasroot_list = []
    profile_home = None
    wsadmin = None
    initial_wasroot_is_valid = 1
    default_profile_in_wsadmin_list = False
    wsadmin_supplied = False

    if not soa71_migrate:
        # If WAS home not supplied, derive it from WSADMIN if possible
        if pdl["WSADMIN"][0] is not None:
            wsadmin_supplied = True
            if utils.is_initial_value(pdl, "WASROOT"):
                derived_wasroot = get_wasroot_from_wsadmin(pdl["WSADMIN"][0])
                if derived_wasroot is not None:
                    pdl["WASROOT"][0:2] = derived_wasroot, "get_wasroot"

        # In zOS, we accept either 1) a complete WAS home directory, or 2) a partial directory path, where hopefully
        # a scan of wsadmin.sh's will locate a WAS home

        # Prompt for WAS_HOME
        doprompt.do_prompt("WASROOT", pdl)
        wasroot = pdl["WASROOT"][0]

        initial_wasroot_is_valid = wasdiscover.verifyValidWasHome(wasroot)
        debug("Initial wasroot: %s,  is_valid: %s"  % (wasroot, str(initial_wasroot_is_valid)))

    #
    # Prompt for WAS configuration home directory (WAS root)
    # Input:  derived_wasroot from WSADMIN
    #         WASROOT - pdl and prompt
    #         initial_wasroot_is_valid
    # Output:
    #         wsadmin_list - a 'smart' list of wsadmin.sh's under the supplied WAS home directory
    #         wasroot_list - corresponding WAS home locations
    #
    pdl["PROFILE"][0:2] = "default", "get_wasroot"
    default_wasroot = derived_wasroot or ""
    while len(wsadmin_list) == 0:
        wasroot = pdl["WASROOT"][0]
        # If initial wasroot appears to be valid, look for well known locations for wsadmin.sh.
        #   If found, use them; try to avoid the directory search.
        if initial_wasroot_is_valid:
            print "%s" % getmsg("CFG1143I", "Searching for wsadmin locations under WAS config home: {0}.", [wasroot])
            was_appserver_wsadmin = wasroot+"/bin/wsadmin.sh"
            was_profile_wsadmin = wasroot+"/profiles/default/bin/wsadmin.sh"
            if os.path.isfile(was_appserver_wsadmin) and os.path.isfile(was_profile_wsadmin):
                wsadmin_list = [was_appserver_wsadmin , was_profile_wsadmin]
            else: debug("Did not find wsadmin script directly under specified WAS root: %s" % wasroot)
        else:
            print "%s" % getmsg("CFG1144I", "Searching for wsadmin locations under directory: {0}.", [wasroot])

        # 1. Scan sub directories for wsadmin's using a depth of 8, but only if we don't have any yet
        #    (8 is just enough to find the profile entries from a standard install directory.)
        if len(wsadmin_list) == 0:
            wsadmin_list = wasdiscover.getwsadmin(wasroot, 8)

        # 2. Translate those directory paths to unique physical paths
        wsadmin_list = _to_physical_path(wsadmin_list)

        # 3. If initial WAS home entered is not a WAS home directory,
        #    verify each wsadmin directory contains a possibly valid WAS home
        #    Method _verify_wsadmin_list_zOS returns:
        #    - wsadmin_list - a list of wsadmin paths from wsadmin_list which contain a WAS home
        #    - wasroot_list - a list of WAS home directories corresponding to the returned wsadmin_list
        if not initial_wasroot_is_valid:
            wsadmin_list, wasroot_list = _verify_wsadmin_list_zOS(wsadmin_list)

        # 4. Verify the list contains the default profile's wsadmin in one of the entries
        #    That is, it contains an entry ending in "profiles/default/bin/wsadmin.sh"
        default_profile_in_wsadmin_list=_contains_default_profile(wsadmin_list)

        if len(wsadmin_list) == 0:
            debug("Unable to find wsadmin.sh under %s" % wasroot)
            # print "%s" % getmsg("CFG1141E", "Unable to discover WebSphere home directory under {0}.", [wasroot])
            if not initial_wasroot_is_valid:
                print "%s" % getmsg("CFG1142I", "You may need to supply more of the WAS home directory path")
            if isBatchMode(pdl):
                raise ConfigurationException(getmsg("CFG1141E", "Unable to discover WebSphere home directory under {0}.", [wasroot]))
            else:
                print "%s" % getmsg("CFG1141E", "Unable to discover WebSphere home directory under {0}.", [wasroot])
            pdl["WASROOT"][0] = default_wasroot
            doprompt.do_prompt("WASROOT", pdl)

    #
    # Prompt for which wsadmin.sh to use.
    # For batch mode, need to have a valid default wsadmin.
    # Input:  wsadmin_list and wasroot_list from above
    # Output: selected wsadmin to be used
    #
    pdl["WSADMINS"][0:2] = wsadmin_list, "get_wasroot"
    default_wsadmin = get_default_wsadmin(wsadmin_list)
    if default_wsadmin != None: pdl["WSADMIN"][0:2] = default_wsadmin, "get_wasroot"

    doprompt.do_prompt("WSADMINS", pdl, "WSADMIN", "Choose WebSphere wsadmin.sh to use")
    wsadmin = pdl["WSADMIN"][0]
    debug("Chosen wsadmin = %s" %wsadmin)
    if not initial_wasroot_is_valid:
        # Maybe we didn't find any wsadmin script
        if wsadmin is None:
            raise ConfigurationException(getmsg("CFG1168E", "Unable to locate a usable wsadmin.sh script under this WebSphere home {0}", [pdl["WASROOT"][0]]))
        # Locate the WAS home based on the selected wsadmin
        ix = wsadmin_list.index(wsadmin)
        derived_wasroot = wasroot_list[ix]
        pdl["WASROOT"][0:2] = derived_wasroot, "get_wasroot"
        print "%s" % getmsg("CFG1140I", "WebSphere home directory located at {0}", [pdl["WASROOT"][0]])
        print
    profile_home = pdl["WASROOT"][0] + "/profiles/default"
    debug("setting profile_home to %s" % profile_home)
    pdl["PROFILEHOME"][0:2] = profile_home, "get_wasroot"

    # Could display wsadmin selected here, especially in batch mode

    debug("get_wasroot_zOS()  WSADMIN selected: %s" % pdl["WSADMIN"][0])
    debug("get_wasroot_zOS()  WASROOT:          %s" % pdl["WASROOT"][0])
    debug("get_wasroot_zOS()  PROFILEHOME:      %s" % pdl["PROFILEHOME"][0])
    debug("Exit get_wasroot_zOS()")


def get_default_wsadmin(wsadmin_list):
    '''
        get_default_wsadmin
        @param   wsadmin_list
        @return  the wsadmin to be used as the default

        Locate the wsadmin provided by the "default" profile
    '''

    debug("Entry get_default_wsadmin %s" % fixlev)

    default_wsadmin = None
    for item in wsadmin_list:
        if item.endswith("/profiles/default/bin/wsadmin.sh"):
            default_wsadmin = item
            break

    debug ("Exit get_default_wsadmin")
    return default_wsadmin


def _verify_wsadmin_list_zOS(wsadmin_list):
    '''
        _verify_wsadmin_list_zOS
        @param wsadmin_list - a list of wsadmin.sh locations for a given WAS home "prefix" supplied by user
        @return
                 - a wsadmin_list that includes only wsadmin.sh's with a WAS home
                 - a list of WAS home's, associated with the wsadmin_list entries

        Verify there is a valid WAS home directory for each wsadmin.sh
    '''

    debug("Entry _verify_wsadmin_list_zOS %s" % fixlev)

    debug("  input wsadmin_list:\n\n%s" % "\n".join(wsadmin_list))
    revised_wsadmin_list = []
    wasroot_list = []
    for wsadmin in wsadmin_list:
        det_wasroot = get_wasroot_from_wsadmin(wsadmin)
        if det_wasroot is not None:
            if _verify_appserver_wasroot(det_wasroot):
                revised_wsadmin_list.append(wsadmin)
                wasroot_list.append(det_wasroot)
    debug(" revised_wsadmin_list:\n\n%s " % "\n".join(revised_wsadmin_list))
    debug(" wasroot_list:\n\n%s " % "\n".join(wasroot_list))

    debug("Exit _verify_wsadmin_list_zOS")
    return revised_wsadmin_list, wasroot_list


def _verify_appserver_wasroot(wasroot):
    '''
        Determine whether the was.env files within the passed WAS root
        indicate this is an App Server.
        @param wasroot - The WAS root to validate
    '''

    debug("Entry _verify_appserver_wasroot %s - entry wasroot=%s" % (fixlev, wasroot))

    # Assume it's not an App Server
    isAppServer = False

    ents = os.listdir(wasroot+os.sep+os.pardir)
    debug("  Under WASROOT:\n\n%s" % "\n".join(ents))
    for ent in ents:
        dirname = os.path.normpath(wasroot+os.sep+os.pardir+os.sep+ent)
        if not os.path.isdir(dirname):
            debug("    %s is not a directory" % ent)
            continue
        wasenv = dirname+os.sep+"was.env"
        if not os.path.isfile(wasenv):
            debug("    %s has no was.env file" % ent)
            continue
        debug("    Reading was.env under %s" % ent)
        try:
            file = codecs.open(wasenv, "r", "cp1047")
            lns = file.readlines();
        except:
            # If we can't read this directory we certainly won't configure it
            debug("Problem reading %s" % wasenv)
            log.logException("File problem in App Server check")
            #log.logDumpStack("File problem in App Server check")
        server_type = None
        for ln in lns:
            ln = ln.rstrip()
            if ln.startswith("server_type="):
                server_type = ln[12:]
                break
        if server_type is None:
            debug("    was.env in %s has no server_type" % ent)
            continue
        if server_type == "AppServer":
            debug("    Server type is AppServer found in was.env for %s" %  ent)
            isAppServer = True
            break
        else:
            debug("    Server type is %s for %s, not AppServer" % (server_type,ent))

    debug("Exit _verify_appserver_wasroot - result: %s" % isAppServer)
    return isAppServer


def _to_physical_path(wsadmin_list):
    '''
        @param wsadmin_list - a list of wsadmin.sh paths, whose directory which may contain symbolic links
        @return an updated wsadmin_list of unique path names whose dirnames are physical paths
    '''

    debug("Entry _to_physical_path(wsadmin_list) %s" % fixlev)
    debug("  input wsadmin_list:\n\n%s " % "\n".join(wsadmin_list))

    revised_wsadmin_list = []
    for wsadmin in wsadmin_list:
        if os.path.exists(wsadmin):
            wsadmin_dir = os.path.dirname(wsadmin)
            wsadmin_dir_canonical_path = File(wsadmin_dir).getCanonicalPath()
            updated_wsadmin = wsadmin_dir_canonical_path + os.sep + os.path.basename(wsadmin)
            if updated_wsadmin not in revised_wsadmin_list:
                revised_wsadmin_list.append(updated_wsadmin)
            else: debug("  wsadmin duplicate: %s" % wsadmin)
        else: debug("  wsadmin does not exist: %s" % wsadmin)

    debug("  revised wsadmin_list of unique paths:\n\n%s " % "\n".join(revised_wsadmin_list))
    debug("Exit _to_physical_path(wsadmin_list)")
    return revised_wsadmin_list


def _contains_default_profile(wsadmin_list):
    '''
        _contains_default_profile
        @param   wsadmin_list
        @return  True if wsadmin_list contains the default profile's wsadmin.sh
    '''

    debug("Entry _contains_default_profile(wsadmin_list) %s" % fixlev)

    rtn = False
    for wsadmin in wsadmin_list:
        if wsadmin.endswith("/profiles/default/bin/wsadmin.sh"):
            rtn = True
            break

    debug("Exit _contains_default_profile(wsadmin_list) returns: %s" % str(rtn))
    return rtn


def get_wasroot_os400(pdl):
    '''
        get wasroot for os400 platforms

        @param pdl - PDL reference
    '''

    debug("Entry get_wasroot_os400(pdl) %s" % fixlev)

    default_wasroot = pdl["WASROOT"][0]
    doprompt.do_prompt("WASROOT", pdl)
    wasroot = pdl["WASROOT"][0]

    debug("wasroot %s" % wasroot)
    profile_path_list = []
    profile_name_list = []
    default_profile_name = ""
    if utils.is_initial_value(pdl, "PROFILES"):
        try:
            profile_name_list, profile_path_list , default_profile_name = wasdiscover.get_was_profiles(pdl, wasroot)
            if len(profile_name_list) == 0: raise Exception
        except Exception:
            cxdtemp = utils.getData(pdl, "CREATETEMPLATE")
            dxdtemp = utils.getData(pdl, "DELETETEMPLATE")
            if cxdtemp == True or dxdtemp == True:
                raise ConfigurationException(getmsg("CFG1164E","Can not discover dmgr profiles. Make sure there is a local dmgr profile created"))
            else:
                raise ConfigurationException(getmsg("CFG1073E","Can not discover profiles. Make sure there is a local application server profile created."))
        pdl["PROFILES"][0:2] = profile_name_list, "get_wasroot"
        if pdl["PROFILE"][0] == None or pdl["PROFILE"][0] == "":
            pdl["PROFILE"][0:2] = default_profile_name, "get_wasroot"
    doprompt.do_prompt("PROFILES", pdl, "PROFILE", None)
    profile_name = pdl["PROFILE"][0]
    profile_home = get_was_profile_home(pdl, profile_name, profile_path_list)
    if profile_home != None:
        wsadmin = profile_home + os.sep + "bin" + os.sep + platformUtil.getWSAdminScript()
        pdl["PROFILEHOME"][0:2] = profile_home, "get_wasroot"
        pdl["WSADMIN"][0:2] = wsadmin, "get_wasroot"
    else:
        raise ConfigurationException(getmsg("CFG2042E", "Profile path not found for profile {0}", [profile_name]))

    debug("Exit get_wasroot_os400()")


def get_wasroot_from_wsadmin(wsadmin):
    '''
        get_wasroot_from_wsadmin
        @param wsadmin  - path to a wsadmin.sh
        @returns - a wasroot associated with this wsadmin.sh or None
    '''

    debug("Entry get_wasroot_from_wsadmin(%s) %s" % (wsadmin, fixlev))

    if wsadmin is None or len(wsadmin) == 0 or not wsadmin.endswith("wsadmin.sh"):
        return None
    was_root = None
    if wsadmin.endswith("/profiles/default/bin/wsadmin.sh"):
        was_root = wsadmin.replace("/profiles/default/bin/wsadmin.sh","")
    else:
        if wsadmin.endswith("/bin/wsadmin.sh"):
            was_root = wsadmin.replace("/bin/wsadmin.sh","")
    # validate - is was_root valid?
    if was_root is not None:
        if wasdiscover.verifyValidWasHome(was_root):
            pass
        else: was_root = None

    debug("Exit get_wasroot() result = %s" % was_root)
    return was_root


def get_was_profile_home(pdl, profile_name, profile_path_list):
    '''
       This routine gets the profile path.  The provided profile name is checked
       against the profile name list and based on the index in the profile name
       list, the profile path list is used to get its path based on the provided
       index.

       @param pdl - PDL reference
       @param profile_name - Profile name
       @param profile_path_list - A list of profile paths
       @return - Profile path locations
    '''

    debug("Entry get_was_profile_home(pdl, %s, profile_path_list) %s" % (profile_name, fixlev))

    index = 0
    profile_path_loc = None

    profile_name_list = utils.getData(pdl, "PROFILES")
    for profile_name_entry in profile_name_list:
        #debug("Profile name entry is: %s  profile name is: %s" % (profile_name_entry, profile_name))
        if profile_name == profile_name_entry and len(profile_path_list) > 0:
            profile_path_loc = profile_path_list[index]
            debug("Profile path location found - %s" % (profile_path_loc))
            break
        index += 1

    debug("Exit get_was_profile_home(), result = %s" % profile_path_loc)
    return profile_path_loc


def isBatchMode(pdl):
    batchMode = pdl["BATCH"][0]
    assert batchMode == True or batchMode == False, getmsg("CFG5019E", "Unexpected value type {0} for PDL key {1}", [type(batchMode), "BATCH"])
    return batchMode
