#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
    module sets WASVERSION, JVMVERSION, WASMAJORVER, WASVER keys in pdl
    based on version string
    input: version string
    output: none
    to call get_wasversion(pdl, version_str)
'''

import sys

import utils
import debugUtil as log
from debugUtil import logDebug as debug
from resourcebundle import getmsg as getmsg


def is_valid_version(version_str=None):
    '''
        check if passed version string is valid
    '''
    is_valid = True
    if version_str == None or version_str.strip() == "None" or len(version_str.strip()) == 0:
        is_valid = False
    return is_valid


def get_wasversion(pdl, version_str=None):
    '''
        Main entry for getting websphere server version
        Sets pcl values based on the version
    '''
    debug("get_wasversion entry: %s" % version_str)
    assert is_valid_version(version_str), getmsg("CFG1109E", "WebSphere version is not set, unable to continue.")
    process_version(pdl, version_str)
    debug("get_wasversion exit")


def process_version(pdl, version):
    debug("process_version entry: %s" % version)
    version_info_dic = {}
    package = ""
    version = version.strip()

    if (version.startswith("5.1")):
        version_info_dic["was_version"]=51
        version_info_dic["was_major_ver"] = 5
        version_info_dic["jvm_version"] = 14
    elif (version.startswith("6.0")):
        version_info_dic["was_version"]=60
        version_info_dic["was_major_ver"]=6
        version_info_dic["jvm_version"]=14
    elif (version.startswith("6.1")):
        version_info_dic["was_version"]=61
        version_info_dic["was_major_ver"]=6
        version_info_dic["jvm_version"]=15
    elif (version.startswith("7.0")):
        version_info_dic["was_version"]=70
        version_info_dic["was_major_ver"]=7
        version_info_dic["jvm_version"]=16
    elif (version.startswith("7.1")):
        version_info_dic["was_version"]=71
        version_info_dic["was_major_ver"]=7
        version_info_dic["jvm_version"]=16
    elif (version.startswith("8.0")):
        version_info_dic["was_version"]=80
        version_info_dic["was_major_ver"]=8
        version_info_dic["jvm_version"]=16
    elif (version.startswith("8.5")):
        version_info_dic["was_version"]=85
        version_info_dic["was_major_ver"]=8
        version_info_dic["jvm_version"]=16
    else:
        # todo: Create real failure message
        assert False, "Unsupported WAS version %s" % version

    debug("version params: %s" % version_info_dic)
    pdl["WASVER"][0] = version+" "+package
    pdl["WASVER"][1] = "WasVersion"

    pdl["WASVERSION"][0] = version_info_dic["was_version"]
    pdl["WASVERSION"][1] = "WasVersion"

    pdl["WASMAJORVER"][0] = version_info_dic["was_major_ver"]
    pdl["WASMAJORVER"][1] = "WasVersion"

    pdl["JVMVERSION"][0] = version_info_dic["jvm_version"]
    pdl["JVMVERSION"][1] = "WasVersion"
    debug("process_version exit")


#####################################################################
# Unit Testing
#####################################################################

if __name__ == "__main__":

    import createPCL_config as createPCL
    import createPDL

    log.set_logging()

    debug("Unit testing of get_wasversion")

    pdl = createPDL.make_pdl(createPCL.make_pcl())
    createPDL.proc_cliopt(pdl, ("", "-bypasswasver", "get_wasversion.dat", "-wasroot", "/u/WebSphere/AppServer"))

    get_wasversion(pdl)
    log.logDumpPDL(pdl,short=True)

