#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

'''
    Install the required SRVPGMs
'''

import os
import sys
import debugUtil
from resourcebundle import getmsg as getmsg

FAILEDSRVPGMS = 0

SCRIPTS_DIR = os.getcwd()
ITCAM_DIR = SCRIPTS_DIR.replace("/bin","")    
INSTSRVPGM_LOG = ITCAM_DIR + "/data/InstSRVPGM.log"

QGPL = "QGPL"
QTIVTMTP = "QTIVTMTP"
QTIVCCGTS = "QTIVCCGTS"

SRVPGMS = ["LIBCCLOG",                     "LIBCFFDC",                     "LIBMSG23",                     "LIBSJITI",      "AM72000014",          "AM72000015",          "AM72000016",          "AMPASS"]
SPFNAME = ["LIBCCLOG",                     "LIBCFFDC",                     "LIBMSG23",                     "SJITI",         "AM72000014",          "AM72000015",          "AM72000016",          "AMPASS"]
SRVLIBS = ["QTIVCCGTS",                    "QTIVCCGTS",                    "QTIVCCGTS",                    "QGPL",          "QGPL",                "QGPL",                "QGPL",                "QGPL"]   
SAVLIBS = ["QTIVCCGTS",                    "QTIVCCGTS",                    "QTIVCCGTS",                    "QTIVTMTP",      "QGPL",                "QGPL",                "QGPL",                "QGPL"]   
SAVFS =   ["CCLOG",                        "CFFDC",                        "MSG23",                        "SJITI",         "AMV5R314",            "AMV5R315",            "AMV5R416",            "AMPASS"]
SRCSAVF = ["QTIVCCGTS.LIBCCLOG.srvpgm.sf", "QTIVCCGTS.LIBCFFDC.srvpgm.sf", "QTIVCCGTS.LIBMSG23.srvpgm.sf", "libsjiti.savf", "am_v5r3_sun_14.savf", "am_v5r3_sun_15.savf", "am_v5r4_sun_16.savf", "ampass.savf"] 


#####################################################################

def installSRVPGMs():

    debugUtil.logDebug("Testing for SRVPGMs currently installed...")
    
    try:
        if isFullyInstalled():
            return FAILEDSRVPGMS
    
        ############################
        # Display a startup message
        ############################

        print "\n%s" % getmsg("FLOWERBOX_INST_SRVPGM","Installation of SRVPGMs will start")

        debugUtil.logDebug("Installation of SRVPGMs will start")
        recreateLibraries()
        installServicePrograms()

        if FAILEDSRVPGMS > 0:
            showExitErrorMsg(FAILEDSRVPGMS)
        else:
            print "\n%s\n\n" % getmsg("CFG1105I", "Server programs successfully installed.")
        
    except:
        debugUtil.logException("An error occurred while trying to install the required Service Programs. Please check the log for more details (%s)" % (INSTSRVPGM_LOG))
       
    return FAILEDSRVPGMS

#####################################################################

def isFullyInstalled():
    installed = True
   
    for index, value in enumerate(SRVPGMS):
        if not (SRVPGMexist(SRVLIBS[index], SPFNAME[index])):
            installed = False
            break
            
    return installed 

#####################################################################

def SRVPGMexist(trgLIB, trgSRVPGM):
    command = "/qsys.lib/" + trgLIB + ".lib/" + trgSRVPGM + ".SRVPGM"
    debugUtil.logDebug("Looking for " + trgLIB + "/" + trgSRVPGM + "...")
    srvpgmExist = os.path.exists(command)
    debugUtil.logDebug("os.path.exists(%s) came back with rc: %i" % (command, srvpgmExist))
    
    return srvpgmExist

#####################################################################

def recreateLibraries():
    _CRTLIB(QTIVTMTP)
    _CRTLIB(QTIVCCGTS)

#####################################################################

def installServicePrograms():
    for index, value in enumerate(SRVPGMS):
        installSRVPGM(SRVLIBS[index], SRVPGMS[index], SAVLIBS[index], SAVFS[index], SRCSAVF[index], SPFNAME[index])

#####################################################################

def installSRVPGM(spLIB, spSRVPGM, sfLIB, sfSAVF, sfSRCSAVF, spFNAME):
    global FAILEDSRVPGMS
    
    debugUtil.logDebug ("Installing service program " + spLIB + "/" + spSRVPGM + "...")
    #print "    Installing Service Program %s ... " % (spSRVPGM)
    
    _DLTSRVPGM(spLIB, spFNAME)    
    _DLTF_SAVF(sfLIB, sfSAVF)
    _CRTSAVF(sfLIB, sfSAVF)
    _copySAVF(sfSRCSAVF, sfLIB, sfSAVF)
    rc = _RSTOBJ_SRVPGM(sfLIB, spSRVPGM, sfSAVF)
    
    if sfLIB != QTIVCCGTS:
        rc = _MOVOBJ_SRVPGM(sfLIB, spSRVPGM, spLIB, spFNAME)
        
        if sfLIB == QGPL:
            rc = _UPDSRVPGM(spLIB, spFNAME)
     
    if rc != 0:
        print "%s" % getmsg("CFG1107E", "Error(s) occurred during the installation of the service programs. Please check the log for more information.")
        FAILEDSRVPGMS = FAILEDSRVPGMS + 1
            
    return rc

#####################################################################

def _copySAVF(srcSAVF, trgLIB, trgSAVF):
    debugUtil.logDebug("* Populating SAVF " + trgLIB + "/" + trgSAVF + " ...")
    command = "/usr/bin/cp " + ITCAM_DIR + "/toolkit/lib/os400/" + srcSAVF + " /QSYS.LIB/" + trgLIB + ".LIB/" + trgSAVF + ".FILE"
 
    return _executeCommand(command)

#####################################################################

def _CRTLIB(trgLIB):
    command = "system 'CRTLIB LIB(" + trgLIB + ")'"
    debugUtil.logDebug("* Creating library " + trgLIB + " ...")
    _executeCommand(command)

#####################################################################

def _DLTSRVPGM(trgLIB, trgOBJ):
    debugUtil.logDebug("* Deleting SRVPGM " + trgLIB + "/" + trgOBJ + " ...")
    command = "system 'DLTSRVPGM SRVPGM(" + trgLIB + "/" + trgOBJ + ")'"
    rc = _executeCommand(command)
 
    return rc
    
#####################################################################

def _DLTF_SAVF(trgLIB, trgOBJ):
    debugUtil.logDebug("* Deleting SAVF file " + trgLIB + "/" + trgOBJ + " ...")
    command = "system 'DLTF FILE(" + trgLIB + "/" + trgOBJ + ")'"
    rc = _executeCommand(command)
 
    return rc

#####################################################################

def _CRTSAVF(trgLIB, trgOBJ):
    debugUtil.logDebug("* Creating SAVF file " + trgLIB + "/" + trgOBJ + " ...")
    command = "system 'CRTSAVF FILE(" + trgLIB + "/" + trgOBJ + ")'"
    rc = _executeCommand(command)
 
    return rc
    
#####################################################################

def _RSTOBJ_SRVPGM(trgLIB, trgSRVPGM, trgSAVF):
    debugUtil.logDebug("*  Restoring SRVPGM " + trgSRVPGM + " ...")
    command = "system 'RSTOBJ OBJ(" + trgSRVPGM + ") SAVLIB(" + trgLIB + ") DEV(*SAVF) SAVF(" + trgLIB + "/" + trgSAVF + ") OBJTYPE(*SRVPGM) FRCOBJCVN(*YES)'"
    rc = _executeCommand(command)
    
    return rc

#####################################################################

def _MOVOBJ_SRVPGM(srcLIB, srcSRVPGM, trgLIB, trgSRVPGM):
    debugUtil.logDebug("* Renaming " + srcLIB + "/" + srcSRVPGM + " as " + trgLIB + "/" + trgSRVPGM + " ...")
    command = "system \"MOV OBJ('/QSYS.LIB/" + srcLIB + ".LIB/" + srcSRVPGM + ".SRVPGM') TOOBJ('/QSYS.LIB/" + trgLIB + ".LIB/" + trgSRVPGM + ".SRVPGM')\""
    rc = _executeCommand(command)
 
    return rc
    
#####################################################################

def _UPDSRVPGM(trgLIB, trgOBJ):
    debugUtil.logDebug("* Updating service program " + trgLIB + "/" + trgOBJ + " ...")
    command = "system 'UPDSRVPGM SRVPGM(" + trgLIB + "/" + trgOBJ + ") MODULE(*NONE)'"
    rc = _executeCommand(command)
 
    return rc
    
#####################################################################

def _executeCommand(command):
    fullCommand = "%s >> %s 2>&1" % (command, INSTSRVPGM_LOG)
    debugUtil.logDebug("  " + command + "\n")
    debugUtil.logDebug("Redirecting output of this command into into '%s'" % INSTSRVPGM_LOG)
    rc = os.system(fullCommand)
    debugUtil.logDebug("  rc=%i\n" % (rc))

    return rc

#####################################################################

def showExitErrorMsg(FAILEDSRVPGMS):
    if FAILEDSRVPGMS > 1:
        quantity = "Some"
        additional = "s"
    else:
        if FAILEDSRVPGMS == 1:
            quantity = "A"
            additional = ""
        
    debugUtil.logDebug("%s Server program%s failed to install properly. Please check the log for more details (%s)" % (quantity, additional, INSTSRVPGM_LOG))
            
    print "\n\n%s\n\n" % getmsg("CFG1129E", "{0} Server program {1} failed to install properly. Please check the log for more details {2}", [quantity, additional, INSTSRVPGM_LOG])

