#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved. 
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
#
# This module is  is called at the time of configuration or reconfig to
# dynamically activate or de-activate a probe (MS, TT, TEMA, SOA).  The
# probe activation or de-activation is done thru an mbean call to the
# server.
#
# Note that this module runs under Jython 2.1 under configDataCollector.
#
#####################################################################

from time import strftime

try:
    import resourcebundle as resourcebundle
    import debugUtil as log
    from debugUtil import debug
    from constants import constants
    from debugUtil import logDebug as debug
except ImportError:
    import lib.resourcebundle as resourcebundle
    import lib.debugUtil as log
    from lib.debugUtil import debug
    from lib.constants import constants
    from lib.debugUtil import logDebug as debug
    


##Global variables
VERSIONSTRING = "ITCAM 7.2 AK0000"
SCRIPT = "mbeancall"
SCRIPTNAME = SCRIPT + ".py"


#common_constants = None

timestamp = strftime("%Y-%m-%d %H:%M:%S")


def process_mbean(server_info, progress):
    '''
       This method gets the server record which is in the form of 
       cell^node^server and gets the object name based on the record
       information, and invokes the reconfigure mbean.
       
       @param server_info - Server information in the form of cell^node^server
       @param progress - progress object reference
       
    '''
    
    global SCRIPT, VERSIONSTRING, timestamp
    global AdminControl
    
    debug("Entry process_mbean")
    
    #
    # Instantiate the global constants class
    #
    common_constants = constants()
        
    wascell = None
    wasnode = None
    wasserver =  None
    object_name = None
    stat = 0
        
    idline = "#  "+SCRIPT+" Version "+VERSIONSTRING  + " " + timestamp + " #"
    blanks = "                                                                                                             "
    hashes = "#############################################################################################################"
    print " "
    print hashes[1:len(idline)+1]
    print "#" + blanks[1:len(idline) - 1] + "#"
    print idline
    print "#" + blanks[1:len(idline) - 1] + "#"
    print hashes[1:len(idline)+1]
    print " "

    try:
        #
        # get the cell, node, server name from the server information record.
        #
        server_attributes = server_info.split("^")
        if (len(server_attributes) != 3):
            log.logError("Server info has an invalid format - %s" % (server_info))
        else:
            wascell = server_attributes[0] 
            wasnode = server_attributes[1]
            wasserver = server_attributes[2]   
            #debug("Processing cell <%s>  node <%s>  server <%s>" % (wascell, wasnode, wasserver))
            #progress.write_progress_start(wasserver)
            #
            # Get the object name
            #
            objname_contents = "WebSphere:name=ConfigurationManager,cell=%s,node=%s,process=%s,*" % (wascell, wasnode, wasserver)
            debug("Object name - %s" % (objname_contents))
            object_name = AdminControl.completeObjectName(objname_contents)
            debug("Object name is - %s" % (object_name))
            #
            # Invoke the mbean
            #
            if (object_name != None and object_name != ""):
                mbean_args = ["", "false", "", "false", ""]
                mbean_msg = AdminControl.invoke(object_name, 'reloadConfiguration', mbean_args)
                debug("Mbean call invoked successfully for %s - %s" % (wasserver, mbean_msg))
                msgl =[]
                msgl.append(wasserver)
                msg = resourcebundle.getmsg("MBEAN_SUCCESSFUL", "Server notified successfully for {0}, probes are activated", msgl) 
                progress.write_progress(msg)
            else:
                #
                # make sure the following text is not changed.  If changed, there scan log
                # in checkprobes module must also be changed, because it is looking fot the
                # text.
                #
                debug("%s, object name is not available for %s" % (common_constants.COMMON_MBEAN_LOG_TEXT, wasserver)) 
                msgl =[]
                msgl.append(wasserver)
                msg = resourcebundle.getmsg("MBEAN_OBJECTNAME_NOTFOUND", "Server was not notified for {0} - dynamic reconfiguration is not taking place, server must be restarted", msgl) 
                progress.write_progress(msg) 
                stat = 1
                
                
    except Exception, errmsg:
        log.logException("%s, error occurred while calling mbean - %s" % (common_constants.COMMON_MBEAN_LOG_TEXT, errmsg)) 
        msgl =[]
        msgl.append(errmsg)
        msg = resourcebundle.getmsg("MBEAN_ERROR", "Error occurred while processing mbean call {0}", msgl) 
        progress.progress_file.write(msg)
        stat = 1
    
    debug("Exit process_mbean - %s" % (stat))
        
    return stat    
         


    
