#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''

 This module handles unconfiguring the config files for the previous releases.
 
'''



import os
import debugUtil as log
import platformUtil
import utils
import codecs
import shutil
import findconfigservers

from xml.dom import minidom
from debugUtil import logDebug as debug
from constants import constants

#
# Instantiate the global constants class
#
common_constants = constants()


def unconfig_dc61_ecam_configfiles(pdl):
    '''
       Deletes all the migrated configured files from the previous release for distributed,
       and for ZOS and OS400 the config file name is renamed
       
       @param pdl - PDL reference
       @param config_dir_path - DC61 or ecam configured path location
    '''
    
    debug("Entry delete_dc61_ecam_configfiles")
    
    os_platform = platformUtil.getOSFamily()
    config_dir_path = findconfigservers.get_dc61_ecam_configdir(pdl)
    serverlist = utils.getData(pdl, "SERVERS")
    for serveri in range(len(serverlist)):
        migrate_confighome = utils.getData(pdl, "MIGRATEDCONFIGHOME", serveri)
        debug("Migrate config home is - %s" % (migrate_confighome))
        if (os_platform != platformUtil.IS_zOS_OS() and (os_platform != platformUtil.IS_OS400_OS())):
            del_dir(migrate_confighome);
            debug("Migrate home deleted - %s" % (migrate_confighome))
        elif (os_platform == platformUtil.IS_zOS_OS()):
            #
            # Check for input.propereties file and if it does exist, rename it
            #
            inputproperties_file = migrate_confighome + os.sep + "input.properties"
            debug("input.properties file location is - %s " % (inputproperties_file))
            if (os.path.exists(inputproperties_file)):
                renamed_inputproperties_file = inputproperties_file + get_renamed_suffix(pdl)
                debug("Renamed file is - %s" % (renamed_inputproperties_file))
                rename_dir(inputproperties_file, renamed_inputproperties_file)
            else:
                debug("File not found - %s" % (inputproperties_file)) 
        elif (os_platform == platformUtil.IS_OS400_OS()):
            #
            # Check for dcInput.txt and if it does exist, rename it
            #
            dcinput_file = migrate_confighome + os.sep + "dcInput.txt"
            debug("dcInput.txt file location is - %s" % (dcinput_file))
            if (os.path.exists(dcinput_file)):
                renamed_dcinput_file = dcinput_file + get_renamed_suffix(pdl)
                debug("Renamed file is - %s" % (renamed_dcinput_file))
                rename_dir(dcinput_file, renamed_dcinput_file)
            else:
                debug("File not found - %s" % (dcinput_file)) 
                                    
    #
    # For distribute, check the servers directory and if it is empty, then delete
    # entire directory including the cells and nodes dirs.
    #        
    if (os_platform != platformUtil.IS_zOS_OS() and (os_platform != platformUtil.IS_OS400_OS())):
        _delete_ecam_dc61_configdirs_dist(pdl, config_dir_path);
              
    debug("Exit delete_dc61_ecam_configfiles")
    

def unconfig_ad71_configfiles(pdl, version):
    '''
       Delete the config entries from the config xml file for distributed and os400, and
       for ZOS the config file is renamed.  For just AD 71 (7100) for os400, there is no
       xml file and the config file is renamed.
       
       @param pdl - PDL reference
       @param version - AD71 version 
    '''
    
    debug("Entry delete_ad71_configfiles - %s" % (version))
    
    os_platform = platformUtil.getOSFamily()
    config_dir_path = findconfigservers.get_ad71_configdir(pdl, version)
    serverlist = utils.getData(pdl, "SERVERS")
    for serveri in range(len(serverlist)):
        migrate_confighome = utils.getData(pdl, "MIGRATEDCONFIGHOME", serveri)
        debug("Migrate config home is - %s" % (migrate_confighome))
        if not platformUtil.IS_zOS_OS() and not platformUtil.IS_OS400_OS():
            inactivate_configured_server(pdl, 
                                         config_dir_path, 
                                         serveri,
                                         version)
        elif platformUtil.IS_OS400_OS():
            if (version == common_constants.BASE_AD71VERSION):  
                #
                # Check for dcInput.txt and if it does exist, rename it.
                #
                dcinput_file = migrate_confighome + os.sep + "dcInput.txt"
                debug("dcInput.txt file location is - %s" % (dcinput_file))
                if (os.path.exists(dcinput_file)):
                    renamed_dcinput_file = dcinput_file + get_renamed_suffix(pdl)
                    debug("Renamed file is - %s" % (renamed_dcinput_file))
                    rename_dir(dcinput_file, renamed_dcinput_file)
                else:
                    debug("File not found - %s" % (dcinput_file)) 
            else:
                inactivate_configured_server(pdl,
                                             config_dir_path,
                                             serveri,
                                             version)              
        elif platformUtil.IS_zOS_OS():
            #
            # Check for input.properties, and if it does exist, rename it.
            #
            inputproperties_file = migrate_confighome + os.sep + "input.properties"
            debug("input.properties file location is - %s " % (inputproperties_file))
            if (os.path.exists(inputproperties_file)):
                renamed_inputproperties_file = inputproperties_file + get_renamed_suffix(pdl)
                debug("Renamed file is - %s" % (renamed_inputproperties_file))
                rename_dir(inputproperties_file, renamed_inputproperties_file)
            else:
                debug("File not found - %s" % (inputproperties_file)) 
        else:
            debug("No valid OS platform")        
                 
    debug("Exit delete_ad71_configfiles")


def unconfig_ad72_configfiles(pdl):
    '''
       Renames the input.properties in the config directory.  This is as a result of migrate.
              
       @param pdl - PDL reference
    '''
    
    debug("Entry delete_ad72_configfiles")
    
    serverlist = utils.getData(pdl, "SERVERS")
    for serveri in range(len(serverlist)):
        migrate_confighome = utils.getData(pdl, "MIGRATEDCONFIGHOME", serveri)
        debug("Migrate config home is - %s" % (migrate_confighome))
        rename_migrate_confighome = migrate_confighome + get_renamed_suffix(pdl)
        rename_dir(migrate_confighome, rename_migrate_confighome)
    
    debug("Exit delete_ad72_configfiles")

   
def _delete_ecam_dc61_configdirs_dist(pdl, config_dir_path):
    '''
       Checks the servers directroy and if it is empty, then the entire directory
       including the cell and nodes directory are deleted.  
       
       @param pdl - PDL reference
       @param config_dir_path - Config directory path
    '''
    
    
    debug("Entry _delete_ecam_dc61_configdirs - %s" % (config_dir_path))
    
    #
    # Get the cells directory contents
    #
    cells_dir_path = config_dir_path + os.sep + "cells"
    debug("Cells directory path is: %s" % (cells_dir_path))
    cells_dir_listing = os.listdir(cells_dir_path) 
    for cells_infile in cells_dir_listing:
        debug("Cell directory name is: %s" % (cells_infile))
        cell_dir = cells_dir_path + os.sep + cells_infile
        if (os.path.isdir(cell_dir)):
            #
            # Get the nodes directory contents
            #
            nodes_dir_path = cell_dir + os.sep + "nodes"
            debug("Nodes directory path is: %s" % (nodes_dir_path))
            nodes_dir_listing = os.listdir(nodes_dir_path) 
            for nodes_infile in nodes_dir_listing:
                debug("Node directory name is: %s" % (nodes_infile))
                node_dir = nodes_dir_path + os.sep + nodes_infile
                if (os.path.isdir(node_dir)):
                    #
                    # Get the servers directory contents
                    #
                    servers_dir_path = node_dir + os.sep + "servers"
                    debug("Servers directory path is: %s" % (servers_dir_path))
                    servers_dir_listing = os.listdir(servers_dir_path) 
                    #
                    # If servers directroy has no children, then delete the entire
                    # config home including cell and node directories.
                    #
                    if (len(servers_dir_listing) == 0):
                        del_dir(cell_dir)
                        debug("Complete config home deleted - %s" % (cell_dir))
    #
    # Check the "cells" directory and if there are no sub directories, then
    # delete the  "cells" directory.
    #
    cells_dir_listing = os.listdir(cells_dir_path)  
    if (len(cells_dir_listing) == 0):
        del_dir(cells_dir_path)
        debug("cells directory deleted - %s" % (cells_dir_path))                  
                    
    
    debug("Exit _delete_ecam_dc61_configdirs")


def inactivate_configured_server(pdl,
                                 config_dir_path,
                                 serveri,
                                 version):    
    '''
       Delete the xml tag for the server that just migrated.  There is an
       xml file that contains the configured server names.  The entry will be
       deleted and the config server directroy will be renamed.
       
       @param pdl - PDL reference
       @param config_dir_path - Configuration path
       @param serveri - Server index in pdl
       @param version - Migrated version
    '''
    
    debug("Entry inactivate_configured_server - %s - %s - %s" % (config_dir_path, serveri, version))
    
    try:
        os_platform = platformUtil.getOSFamily()
        config_mhome = utils.getData(pdl, "MIGRATEDCONFIGHOME", serveri)
        debug("Configure server home - %s" % (config_mhome))
        #
        # For distributed platforms, if this AD 7100, the configured xml file name 
        # is different than subsequent releases.
        #
        if (version == common_constants.BASE_AD71VERSION and os_platform != platformUtil.IS_OS400_OS()):
            config_xmlfile = config_dir_path + os.sep + ".." + os.sep + "WasConfigureServers.xml"
        else:    
            config_xmlfile = config_dir_path + os.sep + ".." + os.sep + "configuredServers.xml"
        debug("Configuration XML file location - %s" % (config_xmlfile))
        doc = minidom.parse(config_xmlfile)
        nodelist = doc.getElementsByTagName('node')
        #
        # search for node element, and check its value in the form of a property
        # with the property name as "CONFIG_FILE_STORE_DIR" which its value is the
        # configured directory.  This value is matched against the configured migrate
        # home and if equal, then get its node parent and delete the parent node.  
        # The XML format is:
        #    <instance id="$SERVERID">
        #       <node>CONFIG_FILE_STORE_DIR=$CONFIG_DIR</node> 
        #        <node>DC_HOME=$DCHOME</node> 
        #        <node>WAS_BASEDIR=$WASBASE_DIR</node> 
        #    </instance>
        #
        for node_element in nodelist:
            node_children = node_element.childNodes
            for node_child in node_children:
                if (node_child.nodeType == node_child.TEXT_NODE):
                    nodetext = node_child.nodeValue
                    debug("Node text is - %s" % (nodetext))
                    if (nodetext.startswith("CONFIG_FILE_STORE_DIR")):
                        config_filestoredir = nodetext.split("=")
                        if (config_filestoredir[1] == config_mhome):
                            instance_node = node_element.parentNode
                            vendor_node = instance_node.parentNode
                            vendor_node.removeChild(instance_node)
        #        
        # Rewrite the modified config file.
        #        
        f = codecs.open(config_xmlfile, mode='w', encoding='utf-8')
        f.write(doc.toxml("utf-8"))
        f.close() 
        #
        # Rename the configured server directroy.
        #
        rename_config_mhome = config_mhome + get_renamed_suffix(pdl)
        rename_dir(config_mhome, rename_config_mhome)
        
        debug("Exit inactivate_configured_server")    
        
    except Exception, errmsg:
        log.logException("Error occurred while processing xml file - %s" % (errmsg))
        raise Exception  
    
def get_renamed_suffix(pdl):
    '''
       Gets the suffix for the renamed files.
       
       @param pdl - PDL reference
    '''
    
    debug("Entry get_renamed_suffix")
    
    suffix = ".AD" + "_migrate"
    
    debug("Exit get_renamed_suffix - %s" % (suffix))
    
    return suffix
 

    
def del_dir(pathtodelete):
    '''
       Deletes a directroy.
       
       @param pathtodelete - Path location to delete
       @return - True if deleted successfully otherwise false
    '''
    
    stat = True
    
    debug("Entry del_dir")
    
    try:
        shutil.rmtree(pathtodelete)
        debug("Directory %s deleted successfully " % (pathtodelete))
        
    except Exception, errarg:
        log.logError("Unable to delete directory %s - %s" % (pathtodelete, errarg))
        stat = False
    
    debug("Exit del_dir - %s" %(stat))
        
    return stat    


def rename_dir(origpath, renamepath):
    '''
       Deletes a directroy.
       
       @param origpath - Original Path location to rename
       @param renamepath - Rename path
       @return - True if renamed successfully otherwise false
    '''
    
    stat = True
    
    debug("Entry rename_dir - %s - %s" % (origpath, renamepath))
    
    try:
        #
        # If rename path is already present, then just delete it.  This could be
        # that user configured it again with the old config and now migrate it
        # again, and now we try to rename it and the name already exist.
        #
        if os.path.isdir(renamepath):
            del_dir(renamepath)
            debug("Directory Removed successfully - %s" % (renamepath))
        os.rename(origpath, renamepath)
        debug("Directory renamed successfully from %s to %s" % (origpath, renamepath))
        
    except Exception, errarg:
        log.logError("Unable to rename directory %s to %s - %s" % (origpath, renamepath, errarg))
        stat = False
    
    debug("Exit rename_dir - %s" % (stat))
        
    return stat  

def unconfig_servers(pdl, version):
    '''
       Deletes/deactivates the config files from the migrated home.
       
       @param pdl - PDL reference
       @param version - Migrated version
    '''
    
    stat = True
    
    debug("Entry unconfig_servers - %s" % (version))
    
    try:
        mversion = version[0]
        if (version >= common_constants.BASE_AD72VERSION):
            debug("Processing AD 72")
            unconfig_ad72_configfiles(pdl)
        elif (version >= common_constants.BASE_ECAMVERSION and version < common_constants.BASE_AD72VERSION):
            debug("Processing ecam")
            unconfig_dc61_ecam_configfiles(pdl)
        elif (version >= common_constants.BASE_AD71VERSION and version < common_constants.BASE_ECAMVERSION):
            debug("Processing AD71")
            unconfig_ad71_configfiles(pdl, version)
        elif (mversion < 7):
            debug("Processing DC 61")
            unconfig_dc61_ecam_configfiles(pdl)
        else:
            log.logError("Invalid version %s" % (version)) 
            
    except Exception, errmsg:
        log.logException("Error occurred while unconfiguring servers for migrate - %s" % (errmsg))
        stat = False
                
    
    debug("Exit unconfig_servers - %s" % (stat))
    
    return stat           