#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved. 
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
   This module copies all the files from a custom directory for a 
   specific app server to the migrated custom directory.   
'''


import os

import utils
import shutil
import debugUtil as log
import resourcebundle as msg
import platformUtil
import configUtils
import checkversion

from constants import constants
from debugUtil import logDebug as debug

runtimes_created = []
old_runtime_path = []
new_runtime_path = []
old_serverdir_name = []

#
# Instantiate the global constants class
#
common_constants = constants()


def get_runtimes_created():
    '''
        get_runtimes_created
        @return a list of app server runtimes created    
    '''                         
    return runtimes_created


def del_file(pathtodelete):
    '''
       Deletes a directroy.
       
       @param pathtodelete - Path location to delete
       @return - True if deleted successfully otherwise false
    '''
    
    stat = True
    
    debug("Entry del_file")
    
    try:
        os.removedirs(pathtodelete)
        debug("Directory %s deleted successfully " % (pathtodelete))
        
    except OSError, errarg:
        log.logError("Unable to delete directory %s - %s" % (pathtodelete, errarg))
        stat = False
    
    debug("Exit del_file - %s" %(stat))
        
    return stat    

def rename_dir(origpath, renamepath):
    '''
       Deletes a directroy.
       
       @param origpath - Original Path location to rename
       @param renamepath - Rename path
       @return - True if renamed successfully otherwise false
    '''
    
    stat = True
    
    debug("Entry rename_dir")
    
    try:
        os.rename(origpath, renamepath)
        debug("Directory renamed successfully from %s to %s" % (origpath, renamepath))
        
    except OSError, errarg:
        log.logError("Unable to rename directory %s to %s - %s" % (origpath, renamepath, errarg))
        stat = False
    
    debug("Exit rename_dir")
        
    return stat    



def del_newrelease_files(pdl, serveri):
    '''
       Deletes all the new created files, input.properties and dcManualInut for the
       specific server and delete the server runtime directory
       
       @param pdl - PDL reference
       @param serveri - Index for the server that is requested to delete the files
    '''
    
    stat = True
    
    debug("Entry del_newrelease_files")
    
    try:
        new_release_runtime_dir_path = pdl["CONFIGHOME"][0] + os.sep + "runtime"
        new_release_data_dir_path = pdl["CONFIGHOME"][0] + os.sep + "data"
        input_prefix = utils.getServerBasedFilePrefix(pdl, serveri)
        wascell = utils.getData(pdl, "WASCELL", serveri)
        wasnode = utils.getData(pdl, "WASNODE", serveri)
        wasprofile = utils.getData(pdl, "PROFILE", serveri)
        wasserver = utils.getData(pdl, "SERVERS", serveri)
        dcmanualinput_filename = wasprofile + "." + wascell + "." + wasnode + "." + wasserver + "." + "DCManualInput.txt"
        inputproperties_path = new_release_data_dir_path + os.sep + input_prefix + "input.properties"
        dcmanualinput_path = new_release_runtime_dir_path + os.sep + dcmanualinput_filename
        os.remove(inputproperties_path)
        os.remove(dcmanualinput_path)
        
    except OSError, errarg:
        log.logError("Unable to delete new release file - %s " % (errarg))
        stat = False    
    
    debug("Exit del_newrelease_files")
        
    return stat    


def copy_propertyfile(old_propertypath, new_propertypath):
    '''
       Copy the files from the old directory to the new one.
       
       @param old_propertypath - Old release property file path location
       @param new_propertypath - New release property file path location
       @return - True if copied successfully otherwise false
    '''
    
    stat = True
    
    debug("Entry copy_propertyfile")
    
    try:
        if platformUtil.IS_zOS_OS():
            shutil.copyfile(old_propertypath, new_propertypath)
            os.system('chmod a+rwX ' + new_propertypath)
        else:
            shutil.copyfile(old_propertypath, new_propertypath)
        log.logDebug("Custom property %s copied successfully to %s" % (old_propertypath, new_propertypath))
    except OSError, errarg:
        log.logError("Unable to copy custom property file %s to %s - %s" % (old_propertypath, new_propertypath, errarg))
        msgl =[]
        msgl.append(old_propertypath)
        msgl.append(new_propertypath)
        msgl.append(errarg)
        errmsg = msg.getmsg("CFG1082E", "Unable to copy custom property file {0} to {1}", msgl)
        print errmsg 
        stat = False
    
    debug("Exit copy_propertyfile - %s" % (stat))
        
    return stat    

def copy_idfile(old_idpath, new_idpath):
    '''
       Copies the old id file to the new config runtime directory.
       
       @param old_idpath - Old release id file path location
       @param new_propertypath - New release id file path location
       @return - True if copied successfully otherwise false
    '''
    
    stat = True
    
    debug("Entry copy_idfile")
    
    try:
        if platformUtil.IS_zOS_OS():
            shutil.copyfile(old_idpath, new_idpath)
            os.system('chmod a+rwX ' + new_idpath)
        else:
            shutil.copyfile(old_idpath, new_idpath)
        log.logDebug("Id file %s copied successfully to %s" % (old_idpath, new_idpath))
        
    except OSError, errarg:
        log.logError("Unable to copy id file %s to %s - %s" % (old_idpath, new_idpath, errarg))
        msgl =[]
        msgl.append(old_idpath)
        msgl.append(new_idpath)
        msgl.append(errarg)
        errmsg = msg.getmsg("CFG1082E", "Unable to copy custom property file {0} to {1}", msgl)
        print errmsg 
        stat = False
    
    debug("Exit copy_propertyfile - %s" % (stat))
        
    return stat    

def copy_and_transform(old_propertypath, new_propertypath):
    '''
        Copy and transform the config home value in all instances within a property file 
        @param old_propertypath - Old release property file path location
        @param new_propertypath - New release property file path location
        @return - return code from copy/scan operation
    '''
    debug("Entry copy_and_transform: %s" % old_propertypath )
    from_home = old_propertypath.split('runtime')[0]
    to_home = new_propertypath.split('runtime')[0]
    debug("..transforming %s to %s" % (from_home,to_home ))    
    substitutionsList = [ from_home + ":" + to_home ]
    rc = configUtils.scanReplaceFile(old_propertypath, new_propertypath, substitutionsList )
    if rc == 0:
        if platformUtil.IS_zOS_OS():
            os.system('chmod a+rwX ' + new_propertypath) 
    else:
        log.logError("Unable to copy/scan custom property file %s to %s - rc: %s" % (old_propertypath, new_propertypath, str(rc)))
        msgl =[old_propertypath, new_propertypath]
        errmsg = msg.getmsg("CFG1082E", "Unable to copy custom property file {0} to {1}", msgl)            
        print errmsg 
        
    debug("Exit copy_and_transform - rc: %s" % str(rc))
    return rc  
        

def create_runtime_dir(pdl, new_release_runtime_dir_path, old_release_server_dir, serveri, migrate_version):
    '''
       Creates the runtime directory, specific server directory, custom dir
       if not exists already.  The runtime directory has the following format:
       SERVERTYPE.WASCELL.WASNODE.WASPROFILE.WASSERVER
       
       @param pdl - PDL reference
       @param new_release_runtime_dir_path - New release runtime dir path
       @param old_release_server_dir - Old release server directory name
       @param serveri - Server index
       @param migrate_version - Migrate version
       @return - True if created successfully otherwise false
    '''
    #global runtimes_created  
    #global migrate_version
    
    stat = True   
    customdir = None
    
    debug("Entry create_runtime_dir - %s - %s - %s - %s" % (new_release_runtime_dir_path, old_release_server_dir, serveri, migrate_version))
    
    wascell = utils.getData(pdl, "WASCELLMODIFIED", serveri)
    wasnode = utils.getData(pdl, "WASNODEMODIFIED", serveri)
    wasserver = utils.getData(pdl, "SERVERSMODIFIED", serveri)
    wasprofile = utils.getData(pdl, "PROFILEMODIFIED", serveri)
    debug("wascell = %s  wasnode = %s  wasserver = %s  wasprofile = %s" % (wascell, wasnode, wasserver, wasprofile))
    
    #
    # Get the WAS type from the old release server directory name.  It could be as 
    # WASTYPE.WASNODE.WASSERVER or WASTYPE.STACKPROD.WASNODE.WASSERVER.  If we have
    # 3 entries, then the first entry is WAS type, and if there are 4 entries, then
    # WAS type is the WASTYPE.STACKPROD.  This is for AD71 and earlier release.
    # For Release 7.2 and above the runtime could be as WASTYPE.WASCELL.WASNODE.WASPROFILE.WASSERVER
    # or WASTYPE.STACKPROD.WASCELL.WASNODE.WASPROFILE.WASSERVER.
    #
    if (migrate_version >= common_constants.BASE_AD72VERSION):
        debug("Processing AD 72")
        new_serverdir = old_release_server_dir
    else:
        debug("Processing pre AD 72")
        serverdir = old_release_server_dir.split(".")
        if (len(serverdir) == 3):
            wastype = serverdir[0]
        elif (len(serverdir) == 4):
            wastype = serverdir[0] + "." + serverdir[1] 
        else:
            log.logError("Old release server directory not valid - %s" % (old_release_server_dir))
            stat = False
        if (stat == True):           
            debug("WAS type is - %s" % (wastype))
            new_serverdir = wastype + "." + wascell + "." + wasnode + "." + wasprofile + "." + wasserver
    if (stat == True):        
        customdir = new_release_runtime_dir_path + os.sep + new_serverdir + os.sep + "custom"
        new_runtime_dir = new_release_runtime_dir_path + os.sep + new_serverdir 
        debug("New release custom directory is - %s" % (customdir))
        runtimes_created.append(new_runtime_dir)
        if not os.path.exists(customdir):
            try: 
                os.makedirs(customdir)
                debug("Created new release custom directory  %s " % (customdir))
            except OSError, errarg:
                log.logError("Unable to create %s- %s" % (customdir, errarg))
                msgl =[]
                msgl.append(customdir)
                msgl.append(errarg)
                errmsg = msg.getmsg("CFG1081E", "Unable to create {0} - {1}", msgl)
                print errmsg 
                stat = False
    else:
        log.logError("Old release server directory not valid - %s" % (old_release_server_dir))
        stat = False
    
    debug("Exit create_runtime_dir - %s" % (stat))
    
    return stat, customdir


def copy_global_custom_files(pdl):
    '''
       Copies all the files from a global custom directory.  All the files
       that are ended with "properties" will be copied.
    '''
        
    debug("Entry copy_global_custom_files")
    
    new_release_runtime_dir_path = pdl["CONFIGHOME"][0] + os.sep + "runtime"
    debug("New release runtime dir path: %s" % (new_release_runtime_dir_path))
    old_release_runtime_dir_path = pdl["MIGRATEHOME"][0] + os.sep + "runtime"
    debug("Old release runtime dir path: %s" % (old_release_runtime_dir_path))
    if os.path.exists(old_release_runtime_dir_path):
        config_dir_listing = os.listdir(old_release_runtime_dir_path) 
        for custom_dir in config_dir_listing:
            if (custom_dir == "custom"):
                old_global_custom_dir = old_release_runtime_dir_path + os.sep + custom_dir
                debug("Global Custom Directory - %s" % (old_global_custom_dir))
                if (os.path.isdir(old_global_custom_dir)):
                    debug("Global Custom Directory found - %s" % (old_global_custom_dir))
                    try: 
                        new_global_custom_dir = new_release_runtime_dir_path + os.sep + custom_dir
                        if not os.path.exists(new_global_custom_dir):
                            os.makedirs(new_global_custom_dir)
                            debug("Created new release global custom directory  %s " % (new_global_custom_dir))
                        else:
                            debug("Global custom directory already exist %s " % (new_global_custom_dir)) 
                    except OSError, errarg:
                        log.logError("Unable to create %s- %s" % (new_global_custom_dir, errarg))
                        msgl =[]
                        msgl.append(new_global_custom_dir)
                        msgl.append(errarg)
                        errmsg = msg.getmsg("CFG1081E", "Unable to create {0} - {1}", msgl)
                        print errmsg 
                        debug("Creating new release global custom directory failed %s - %s " % (new_global_custom_dir, errarg))
                        break
                    #
                    # Copy all the files and directories from the old version.
                    #
                    listing = os.listdir(old_global_custom_dir)
                    for custom_infile in listing:
                        old_direntry = old_global_custom_dir + os.sep + custom_infile
                        new_direntry = new_global_custom_dir + os.sep + custom_infile 
                        if (os.path.isdir(old_direntry)):
                            copy_dirs(pdl, old_direntry, new_direntry)
                        else:
                            #
                            # Copy the custom property file, and if copied ok, delete the old
                            # config directory, otherwise, delete all the new created files.
                            #
                            copied_ok = copy_propertyfile(old_direntry, new_direntry)
                            if (copied_ok == False):
                                msgl =[]
                                msgl.append(old_direntry)
                                msgl.append(new_direntry)
                                infomsg = msg.getmsg("CFG2008E", "Copying the custom file {0} to {1} failed", msgl)
                                print infomsg
                                    
                    break;               
                                    
    else:
        debug("Old runtime directory does not exist - %s" % (old_release_runtime_dir_path))
        
    debug("Exit copy_global_custom_files" )
                        
    return 

def copy_dirs(pdl, old_custom_dir, new_custom_dir):
    '''
       Copies all the contents of a directory to its new location 
       
       @param pdl - PDL reference
       @param old_custom_dir - Old custom directory
       @param new_custom_dir - New custom directory
    ''' 
    
    contd = True
    
    debug("Entry copy_dirs - %s - %s" %(old_custom_dir, new_custom_dir))  
    
    #
    # Make sure the new directory is created if not already exists.
    #
    try:
        if not os.path.exists(new_custom_dir):
            os.makedirs(new_custom_dir)
            debug("Created new release custom directory  %s " % (new_custom_dir))
        else:
            debug("Custom directory already exist %s " % (new_custom_dir)) 
    except OSError, errarg:
        log.logError("Unable to create %s- %s" % (new_custom_dir, errarg))
        msgl =[]
        msgl.append(new_custom_dir)
        msgl.append(errarg)
        errmsg = msg.getmsg("CFG1081E", "Unable to create {0} - {1}", msgl)
        print errmsg 
        debug("Creating new release custom directory failed %s - %s " % (new_custom_dir, errarg))  
        contd = False      
     
    if (contd == True):        
        listing = os.listdir(old_custom_dir)
        for custom_infile in listing:
            old_direntry = old_custom_dir + os.sep + custom_infile
            new_direntry = new_custom_dir + os.sep + custom_infile
            debug("Old custom file/dir name: %s   New custom file/dir name: %s" % (old_direntry, new_direntry))
            if (os.path.isdir(old_direntry)):
                #
                # Make sure the new directory is created if not already exists.
                #
                try:
                    if not os.path.exists(new_direntry):
                        os.makedirs(new_direntry)
                        debug("Created new release custom directory  %s " % (new_direntry))
                    else:
                        debug("Custom directory already exist %s " % (new_direntry)) 
                        
                    copy_dirs(pdl, old_direntry, new_direntry)
                except OSError, errarg:
                    log.logError("Unable to create %s- %s" % (new_direntry, errarg))
                    msgl =[]
                    msgl.append(new_direntry)
                    msgl.append(errarg)
                    errmsg = msg.getmsg("CFG1081E", "Unable to create {0} - {1}", msgl)
                    print errmsg 
                    debug("Creating new release custom directory failed %s - %s " % (new_direntry, errarg))
                    break    
            else:
                #
                # Copy the custom property file.
                #
                copied_ok = copy_propertyfile(old_direntry, new_direntry)
                if (copied_ok == False):
                    msgl =[]
                    msgl.append(old_direntry)
                    msgl.append(new_direntry)
                    infomsg = msg.getmsg("CFG2008E", "Copying the custom file {0} to {1} failed", msgl)
                    print infomsg
                    break
                

    debug("Exit copy_dirs")

def copy_custom_files(pdl, migrate_version):
    '''
       Copies all the files from a custom directory of a specific
       server to the migrated custom directory.  The directory name
       has a node and server name in it as xxxx.NODE.SERVER.  THe
       node and server name are acquired from pdl and for each
       entry in the custom directory with the match, all the files
       under the custom directory for the specific app server is
       copied over to the migrated runtime directory under the 
       specific server directory.
    '''
    copied = True
        
    debug("Entry copy_custom_files")
    
    new_release_runtime_dir_path = pdl["CONFIGHOME"][0] + os.sep + "runtime"
    debug("New release runtime dir path: %s" % (new_release_runtime_dir_path))
    old_release_runtime_dir_path = pdl["MIGRATEHOME"][0] + os.sep + "runtime"
    debug("Old release runtime dir path: %s" % (old_release_runtime_dir_path))
    if os.path.exists(old_release_runtime_dir_path):
        config_dir_listing = os.listdir(old_release_runtime_dir_path) 
        serverlist = utils.getData(pdl, "SERVERS")
        utils.setData(pdl, "CONFIGURED", [], "migratecustomfiles")
        for serveri in range(len(serverlist)):
            if (copied == False):
                break
            for server_dir in config_dir_listing:
                if (copied == False):
                    break
                old_server_runtime_dir = old_release_runtime_dir_path + os.sep + server_dir
                debug("Config Directory - %s" % (old_server_runtime_dir))
                if (os.path.isdir(old_server_runtime_dir)):
                    #
                    # Get the WAS type from the old release server directory name.  It could be as 
                    # WASTYPE.WASNODE.WASSERVER or WASTYPE.STACKPROD.WASNODE.WASSERVER.  If we have
                    # 3 entries, then the first entry is WAS type, and if there are 4 entries, then
                    # WAS type is the WASTYPE.STACKPROD.  This is for AD71 and earlier release.
                    # For Release 7.2 and above the runtime could be as WASTYPE.WASCELL.WASNODE.WASPROFILE.WASSERVER
                    # or WASTYPE.STACKPROD.WASCELL.WASNODE.WASPROFILE.WASSERVER.
                    #
                    if (migrate_version >= common_constants.BASE_AD72VERSION):
                        debug("Processing AD 72")
                        wascell = utils.getData(pdl, "WASCELLMODIFIED", serveri)
                        wasnode = utils.getData(pdl, "WASNODEMODIFIED", serveri)
                        wasserver = utils.getData(pdl, "SERVERSMODIFIED", serveri)
                        wasprofile = utils.getData(pdl, "PROFILEMODIFIED", serveri)
                        debug("wascell = %s  wasnode = %s  wasserver = %s  wasprofile = %s" % (wascell, wasnode, wasserver, wasprofile))
                        if (wasprofile is None or len(wasprofile) == 0):
                            debug("WAS Liberty found")
                            key = wascell + "." + wasnode + "." + wasserver
                        else:
                            key = wascell + "." + wasnode + "." + wasprofile + "." + wasserver
                    else:
                        wasnode = utils.getData(pdl, "WASNODE", serveri)
                        wasserver = utils.getData(pdl, "SERVERS", serveri)
                        #
                        # Convert the fields to its ascii presentation based on AD 71 conversion,
                        # if contains a non ascii character.
                        #
                        wasnode = utils.convert_nonascii_ad71(wasnode)
                        wasserver = utils.convert_nonascii_ad71(wasserver)
                        debug("Fields after conversion  wasnode = %s  wasserver = %s" % (wasnode, wasserver))
                        key = wasnode + "." + wasserver
                        
                    log.logDebug("Searching for old release runtime directory for - %s " %(key))
                    index = server_dir.find(key)
                    if (index >= 0):
                        log.logDebug("Found old release runtime directory - %s " %(server_dir))
                        #
                        # Now look for "custom" directory
                        #
                        old_customdir_path = old_server_runtime_dir + os.sep + "custom"
                        if (os.path.isdir(old_customdir_path)):
                            listing = os.listdir(old_customdir_path)
                            new_customdir_created = False
                            for custom_infile in listing:
                                if (copied == False):
                                    break
                                
                                #
                                # Create the custom directory if not already created
                                #
                                if (new_customdir_created == False):
                                    custom_created, new_customdir_path = create_runtime_dir(pdl, new_release_runtime_dir_path, server_dir, serveri, migrate_version)
                                    if (custom_created == True):
                                        new_customdir_created = True
                                        old_runtime_path.append(old_server_runtime_dir)
                                        old_serverdir_name.append(server_dir)
                                    else:
                                        copied = False 
                                        break   
                                if (new_customdir_created == True):
                                    old_property_filename = old_customdir_path + os.sep + custom_infile
                                    new_property_filename = new_customdir_path + os.sep + custom_infile
                                    if (os.path.isdir(old_property_filename)):
                                        copy_dirs(pdl, old_property_filename, new_property_filename)
                                    else:
                                        #
                                        # Copy the custom property file, and if copied ok, delete the old
                                        # config directory, otherwise, delete all the new created files.
                                        #
                                        copied_ok = copy_propertyfile(old_property_filename, new_property_filename)
                                        if (copied_ok == False):
                                            msgl =[]
                                            msgl.append(old_property_filename)
                                            msgl.append(new_property_filename)
                                            infomsg = msg.getmsg("CFG2008E", "Copying the custom file {0} to {1} failed", msgl)
                                            print infomsg
                                            copied = False
                                            
                        else:
                            log.logDebug("No custom directory exist in the old runtime server directory - %s" %(old_server_runtime_dir))
    else:
        debug("Old runtime directory does not exist - %s" % (old_release_runtime_dir_path))
        copied = False
        
    debug("Exit copy_custom_files - %s" %(copied))
                        
    return copied   
 

def copy_id_files(pdl, version):
    '''
       Copies the id file from the migrated runtime directory to the new
       config directroy if it does exist
       
       @param pdl - PDL reference
    '''                          
    
    debug("Enter copy_id_files - %s" % (version))
    
    for serveri in range(len(runtimes_created)):
        #
        # If this is a migrate from DC 6 version, the id file has the 
        # WASVER.NODE.SERVER prepend to the id file name.
        #
        if (version < 7):
            debug("Processing DC 61 for id file")
            old_id_file = old_runtime_path[serveri] + os.sep + old_serverdir_name[serveri] + ".id"
        else:
            old_id_file = old_runtime_path[serveri] + os.sep + "id"
            
        debug("Id file is - %s" % (old_id_file))
        if os.path.isfile(old_id_file):
            new_id_file = runtimes_created[serveri] + os.sep + "id"
            copy_idfile(old_id_file, new_id_file)
            
    debug("Exit copy_id_file")        
            
              
   
def copy_files(pdl, mversion):
    '''
       Copies the custom property files and also the id file from the
       migrated home directory to the new config home.
       
       @param pdl - PDL references
       @param mversion - Migrate version
    '''   
    
       
    debug("Entry copy_files")
    
    debug("Migrate version is: %s" %(mversion))
    
    #
    # Copy global custom properties.
    #
    copy_global_custom_files(pdl)
    #
    # Copy custom properties
    #
    copied = copy_custom_files(pdl, mversion)
    #
    # Copy the id file
    #
    version = mversion[0]
    copy_id_files(pdl, version)
    
    
    debug("Exit copy_files - %s" %(copied))
    
    return copied

def copy_files_smai(pdl):
    '''
       Specific to SCM AI 7.3 and up migrate, where user can select servers from multiple DC Home directories to migrate in a single run
       Copies the custom property files and also the id file from the
       migrated home directory to the new config home.
       
       @param pdl - PDL references
    '''   
    
       
    debug("Entry copy_files_smai for SCM AI migrate")
    
    
    migrate_dic = pdl["CCMMIGRATEHOMES"][0]
    debug("migrate dictionary=%s" % migrate_dic)
    copied = True
    
    for migratehome in migrate_dic.keys():
        mversion = migrate_dic[migratehome]
        debug("processing migratehome=%s migrate version=%s" % (migratehome, mversion))
        pdl["MIGRATEHOME"][0:2] = migratehome, "temp_data"
        #
        # Copy global custom properties.
        #
        copy_global_custom_files(pdl)
        #
        # Copy custom properties
        #
        copied = copied and copy_custom_files(pdl, mversion)
        #
        # Copy the id file
        #
        version = mversion[0]
        copy_id_files(pdl, version)
    
    
    debug("Exit copy_files_smai - %s" %(copied))
    
    return copied
    
