#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

'''
    Invoke findServers and interpret the output
'''

fixlev = "40544" # Last APAR, defect or story to update this module

import os
import codecs

import utils
import platformUtil
import get_wasversion
import wasdiscover
import wlpFindServers

from resourcebundle import getmsg as getmsg
from debugUtil import logDebug as debug
from FindserversTask import FindserversTask
from constants import constants

# TOTALATTRIBUTES is the minimum number of server attributes obtained from findServers.py
TOTALATTRIBUTES = 8

findServers_error = None


def processFile(pdl, outputFile, findServers_list):

    global findServers_error

    debug("Entry processFile %s" % fixlev)

    infile = None
    try:
        infile = codecs.open(outputFile, mode="r", encoding="utf-8")
        server_found = 0
        lines = ""
        for line in infile.readlines():
            lines = lines + line
            if line.startswith("#"):
                line = line.rstrip("\r\n")
                if len(line) > 0 and line.endswith("\n"):
                    line=line + "mec4daveend"
                    line=line.replace("\nmec4daveend","")
                ws = line.split("=")
                if len(ws) != 2: continue
                w0, w1, w2 = "", "", ""
                w0 = ws[0]          #property name
                value = ws[1]       #value (rest of the line)
                w1 = value
                if w0 == "#findServers.version":
                    assert w1 == "7.3", getmsg("CFG1070E", "Invalid version number  - {0}", [line])
                elif w0 == "#bootstrap.address":
                    props = w1.split()
                    if len(props) == 2:
                        pdl["RMIHOST"][0] = props[0]
                        pdl["RMIHOST"][1] = "findServers"
                        pdl["RMIPORT"][0] = props[1]
                        pdl["RMIPORT"][1] = "findServers"
                elif w0 == "#was.cell":
                    pdl["WASCELL"][0] = w1
                    pdl["WASCELL"][1] = "findServers"
                elif w0 == "#was.node":
                    pdl["WASNODE"][0] = w1
                    pdl["WASNODE"][1] = "findServers"
                elif w0 == "#was.install.root":
                    pdl["WASINSTALLROOT"][0] = w1
                    pdl["WASINSTALLROOT"][1] = "findServers"
                elif w0 == "#was.plugins.root":
                    pdl["WASPLUGINSROOT"][0] = w1
                    pdl["WASPLUGINSROOT"][1] = "findServers"
                elif w0 == "#user.install.root":
                    pdl["USERINSTALLROOT"][0] = w1
                    pdl["USERINSTALLROOT"][1] = "findServers"
                elif w0 == "#was.config.root":
                    pdl["WASCONFIGROOT"][0] = w1
                    pdl["WASCONFIGROOT"][1] = "findServers"
                elif w0 == "#was.props.dir":
                    pdl["WASPROPSDIR"][0] = w1
                    pdl["WASPROPSDIR"][1] = "findServers"
                elif w0 == "#java.home":
                    pdl["JAVAHOME"][0] = w1
                    pdl["JAVAHOME"][1] = "findServers"
                elif w0 == "#was.log.root":
                    pdl["LOGROOT"][0] = w1
                    pdl["LOGROOT"][1] = "findServers"
                elif w0 == "#wps.home":
                    pdl["WPSHOME"][0] = w1
                    pdl["WPSHOME"][1] = "findServers"
                elif w0 == "#wbi.install.root":
                    pdl["WBIHOME"][0] = w1
                    pdl["WBIHOME"][1] = "findServers"
                elif w0 == "#wlp.install.root":
                    pdl["WLPHOME"][0] = w1
                    pdl["WLPHOME"][1] = "findServers"
                elif w0 == "#error":
                    findServers_error = w1
                else:
                    assert False, getmsg("CFG1112E", "Unexpected line in findServers output {0}", [line])
                continue

            line = line.rstrip()
            debug("line: %s" % line)
            words = line.split("\t")
            totalWords = len(words)
            if totalWords > 0:
                if words[0].find("#") > -1:
                    continue
            if totalWords < TOTALATTRIBUTES:
                debug("Do not have enough server information to populate server list")
                continue
            #Get WAS version from PDL
            debug("was version from findServers: %s" % words[6])
            get_wasversion.get_wasversion(pdl, words[6])
            wasversion = pdl["WASVERSION"][0]
            wasinstallroot = pdl["WASINSTALLROOT"][0]
            profile = pdl["PROFILE"][0]
            profilehome = pdl["PROFILEHOME"][0]
            jvmversion = pdl["JVMVERSION"][0]
            wlphome = pdl["WLPHOME"][0] or "None"
            debug("WAS version: %s" % wasversion)
            server_id = words[-1]

            #Update CONFIGURED value if the application server is configured by ITCAM 7.2 or later version
            cellName = words[0]
            nodeName = words[1]
            serverName = words[2]
            configured = words[9]
            dcVersion = words[11]
            subProduct = words[12]

            defaultDCVersion = pdl["VERSION"][0]
            if configured == "None" and (dcVersion == None or dcVersion == "None"):
                # pristine server, set dcVersion to current ITCAM version.
                dcVersion = defaultDCVersion
            elif dcVersion == defaultDCVersion:
                configured = dcVersion
            if pdl["BYPASSCHECK"][0] and configured != "None" and configured != defaultDCVersion:
                #configured = "7.2.1"
                #configured = defaultDCVersion
                configured = "None"
            debug("configured: %s" % configured)

            aServer = {
                       "WASCELL": cellName,
                       "WASNODE": nodeName,
                       "WASSERVER": serverName,
                       "STANDALONE": words[3],
                       "JVMMODE": words[4],
                       "SHARECLASSESMODIFIED": words[5],
                       "CURRENTMAXHEAPSIZE": words[7],
                       "CURRENTGCLOGLOCATION": words[8],
                       "CONFIGURED": configured,
                       "JVMVERSION": jvmversion,
                       "SERVERID": server_id,
                       "WASVERSION": wasversion,
                       "WASINSTALLROOT": wasinstallroot,
                       "PROFILE": profile,
                       "PROFILEHOME": profilehome,
                       "MIGRATEDDCHOME": None,
                       "MIGRATEDVERSION": None,
                       "DCVERSION": dcVersion,
                       "WLPHOME": wlphome,
                       "SUBPRODUCT": subProduct,
                       "CONFIGRETURNCODE": 0,
                      }

            #For WAS8 and above, we need to check if we need to overwrite JDKVERSION in PDL
            #was8_jdkversion = words[10]
            #if (was8_jdkversion != "None" and was8_jdkversion.isdigit()):
            #    aServer["JVMVERSION"] = was8_jdkversion
                #Reset aServer to pass was8_jdkversion to createservers.py so pdl["JDKVERSION"][0] can be rewritten
                #aServer = {"WASCELL": words[0], "WASNODE": words[1], "WASSERVER": words[2], "STANDALONE": words[3], "JVMMODE": words[4], "SHARECLASSESMODIFIED": words[5], "CURRENTMAXHEAPSIZE": words[7], "CURRENTGCLOGLOCATION": words[8], "CONFIGURED": words[9], "JVMVERSION": was8_jdkversion, "SERVERID": server_id, "WASVERSION": wasversion, "WASINSTALLROOT": wasinstallroot}

            debug("findServers_list server entry %d: %s" % (len(findServers_list), aServer))
            findServers_list.append(aServer)

        debug("findServers data:\n\n%s" % lines)

    finally:
        if infile != None:
            infile.close();

    debug("Exit processFile")


def populateFindServers(pdl):

    debug("Entry populateFindServers %s" % fixlev)

    stat = True
    global findServers_error

    #
    # Instantiate common constants class
    #
    common_constants = constants()

    #
    # Call the FindserversTask to run the findservers module thru the wsadmin to get
    # the list of the servers for a specific profile.
    #
    if not pdl["BYPASSFINDSERVERS"][0]:
        if not pdl["WLPINSTALLDIR"][0]:
            findservers_task = FindserversTask(pdl, common_constants.FINDSERVERS, "findservers.inputlist", "findservers.log", "findServers.py", "findservers.dat")
            findservers_task.process()
            if pdl["RETURNCODE"][0] != 0:
                debug("The findservers task set a non-zero return code, findservers has failed.")
                stat = False
            dat_logfile_loc = findservers_task.get_dat_logfile_loc()
        else:   #WAS Liberty profile
            debug("Finding servers for WAS Liberty")
            dat_logfile_loc = wlpFindServers.findlibertservers(pdl)
            debug("1 dat_logfile_loc: " + dat_logfile_loc)
    else:
        debug("findServers has been bypassed, substitute output file is %s" % pdl["BYPASSFINDSERVERS"][0])
        dat_logfile_loc = pdl["BYPASSFINDSERVERS"][0]
        findServers_error = None
    debug("dat_logfile_loc: " + dat_logfile_loc)

    findServers_list = []
    if os.path.isfile(dat_logfile_loc):
        processFile(pdl, dat_logfile_loc, findServers_list)
    else:
        args = []
        args.append(dat_logfile_loc)
        print getmsg("CFG5032E", "The file {0} does not exist.", args)

    if len(findServers_list) > 0:
        pdl["FINDSERVERS"][0:2] = findServers_list, "findServers"
    else:
        print getmsg("CFG5012E", "Error - Failed to discover servers for the profile.")
        stat = False

    if findServers_error != None:
        print "\n%s\n%s" % (getmsg("CFG5011W", "Warning - Note that not all Node Agents connected to the Deployment Manager are running."), findServers_error)

    debug("Exit populateFindServers: %s" % stat)
    return stat


#####################################################################
# Unit Testing
#####################################################################

if __name__ == "__main__":

    import debugUtil as log
    import createPCL_config as createPCL
    import createPDL

    log.set_logging()
    debug("Unit Test for populateFindServers")

    pdl = createPDL.make_pdl(createPCL.make_pcl())
    createPDL.proc_cliopt(pdl, ("", "-bypassfindservers", "findServers.dat"))
    log.logDumpPDL(pdl,"\nBefore populateFindServers",short=True)

    populateFindServers(pdl)

    log.logDumpPDL(pdl,"\nAfter populateFindServers",short=True)
