#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

'''
    Handles silent input properties file.
    Mode handled = Config/Unconfig/Migrate
    Updates PDL
    As well as returns a list of file input.properties file
    for handling by findconfigservers
'''

fixlev = "41818" # Last APAR, defect or story to update this module

import os
import codecs
import re

import debugUtil as log
import wasdiscover
import utils
import createPDL
import checkversion
import platformUtil
import wlpConfigUtils

from debugUtil import logDebug as debug
from resourcebundle import getmsg
from ConfigurationException import ConfigurationException
from doprompt import server_alldefault
from configdcless import readConnFile

# To do: This is probably unused and should be deleted.
from java.io import File

# Create the pattern that matches input.properties files and can
# differentiate dcless.input.properties files.
# The value of result.group(1), which is the server,
# will not be None for a match against a valid input.properties file
# that is not also a dcless.input.properties file.

pat_input = re.compile(r"^.+\..+\..+\..+\.dcless\.input\.properties$|^.+\..+\..+\.(.+)\.input\.properties$")

# Hold list of files which findconfigservers can use as a filter
FILES = []

#ToDo: Use Constants!
CONFIG = 2
UNCONFIG = 3
MIGRATE = 4
RECONFIG = 5

_TRUE="true"
_FALSE="false"

######################################################################################
# Dictionary to hold the DEFAULT section data of the silent file
######################################################################################
DEFAULT_HOLDER = {}

######################################################################################
# List of dictionaries to hold the SERVER section data of the silent file
# The values are a dictionary of name-value of pair of each property for that section
# This is composite server property - DEFAULT + SERVER section for each server
######################################################################################
SERVER_HOLDER = []

#######################################################################
# This is a map between: silent file options we allow and
# the values we use for brevity in the program
# If the keys are to be changed in the silent file, this is the only
# place in the code where they need to be changed
#######################################################################
stcMap = {
          "ms.connect"                   :        "msconnect",
          "default.hostip"               :        "defaulthostip",
          "ms.kernel.host"               :        "kernelip",
          "ms.kernel.codebase.port"      :        "kernelport01",
          "ms.am.home"                   :        "mshome",
          "ms.am.socket.bindip"          :        "bindip",
          "ms.probe.controller.rmi.port" :        "probermicontrollerport",
          "ms.probe.rmi.port"            :        "probermiport",
          "ttapi.enable"                 :        "ttapi",
          "was.backup.configuration"     :        "backup",
          "was.backup.configuration.dir" :        "backupdir",
          "was.gc.custom.path"           :        "gcoutputdist",
          "was.gc.file"                  :        "gclogpath",
          "was.configure.heap"           :        "heapsize",
          "ttapi.host"                   :        "tthost",
          "ttapi.port"                   :        "ttport",
          "soa.enable"                   :        "soadc",
          "tpv.enable"                   :        "tpv",
          "de.enable"                    :        "de",
          "temaconnect"                  :        "temaconnect",
          "tema.jmxport"                 :        "jmxport",
          "tema.appserver"               :        "temadcappserver",
          "tema.host"                    :        "temahost",
          "tema.port"                    :        "temaport",
          "tema.serveralias"             :        "serveralias",
          "tema.dcless.conntype"         :        "dclessconntype",     # Connect type is not currently supported
          "was.client.props"             :        "useclientprops",
          "was.wsadmin.connection.host"  :        "host",
          "was.wsadmin.connection.type"  :        "conntype",
          "was.wsadmin.connection.port"  :        "port",
          "was.wsadmin.username"         :        "user",
          "was.wsadmin.password"         :        "password",
          "was.appserver.server.name"    :        "server",
          "was.appserver.cell.name"    :          "cellname",
          "was.appserver.node.name"    :          "nodename",
          "was.appserver.profile.name"   :        "profile",
          "was.appserver.home"           :        "wasroot",
          "itcam.migrate.home"           :        "migratehome",
          "soa.migrate.home"             :        "soamigratehome",
          "migrate.type"                 :        "listresp",
         }

#######################################################################
# These are the keys allowed for the server section, a subset
#######################################################################
vldServerKwds = (
                 "server",
                 "cellname",
                 "nodename",
                 "profile",
                 "useclientprops",
                 "wasroot",
                 "user",
                 "password",
                 "serveralias",
                )

#######################################################################
# This is a map between: values of the stc map & allowed data format
#######################################################################
vldMap = {
          "msconnect"                   :        "truefalse",
          "defaulthostip"               :        "hostname",
          "kernelip"                    :        "hostname",
          "kernelport01"                :        "port",
          "mshome"                      :        "string",
          "bindip"                      :        "hostname",
          "probermicontrollerport"      :        "portrange",
          "probermiport"                :        "portrange",
          "ttapi"                       :        "truefalse",
          "backup"                      :        "truefalse",
          "backupdir"                   :        "dirname",
          "gcoutputdist"                :        "truefalse",
          "gclogpath"                   :        "existingpath",
          "heapsize"                    :        "truefalse",
          "tthost"                      :        "hostname",
          "ttport"                      :        "port",
          "soadc"                       :        "truefalse",
          "tpv"                         :        "truefalse",
          "de"                          :        "truefalse",
          "temaconnect"                 :        "truefalse",
          "temahost"                    :        "hostname",
          "temaport"                    :        "port",
          "serveralias"                 :        "alphanumeric",
          "jmxport"                     :        "port",
          "useclientprops"              :        "truefalse",
          "host"                        :        "hostname",
          "conntype"                    :        "conntype",
          "port"                        :        "port",
          "user"                        :        "username",
          "password"                    :        "password",
          "server"                      :        "server",  #how to verify
          "cellname"                    :        "cellname",#how to verify
          "nodename"                    :        "nodename",#how to verify
          "profile"                     :        "profile",
          "wasroot"                     :        "wasroot",
          "migratehome"                 :        "dirname",
          "soamigratehome"              :        "wasroot",
          "listresp"                    :        "listresp",
          "is_wlp"                      :        "truefalse",
          "temadcappserver"                  :        "truefalse",
          "dclessconntype"              :        "conntype",
         }

##
## Values for some of the keys should be validated only if prerequisite key is set.
## i.e., validate that backup directory is valid only if user selected to backup the profile.
##
dependencyMap = {
                "gclogpath": {"gcoutputdist": True},
                "backupdir": {"backup": True},
                "temaport": {"temaconnect": True},
                "temahost": {"temaconnect": True},
                "kernelip": {"msconnect": True},
                "mshome":{"msconnect": True},
                "kernelport01": {"msconnect": True},
                "probermicontrollerport": {"msconnect": True},
                "probermiport": {"msconnect": True},
                "tthost": {"ttapi": True},
                "ttport": {"ttapi": True},
                "was.appserver.profile.name": {"is_wlp": False},
                "was.appserver.cell.name": {"is_wlp": False},
                "was.appserver.node.name": {"is_wlp": False},
                "was.wsadmin.connection.host": {"is_wlp": False}
                }

# The is_WLP flag is used globally in this module
is_WLP = False


def processFile(pdl, inputFile, ptype):
    '''
       Main entry program that calls all the modules.
       Reads the DEFAULT Section
       Creates Values for each SERVER
       Updates PDL
       As well as returns a list of file input.properties file
       for handling by findconfigservers
    '''

    debug("Entry processFile %s" % fixlev)
    debug("Dump Silent to CLI Mapper:\n\n    %s" % "\n    ".join("%28s : %s" % (k,v) for k, v in sorted(stcMap.items())))

    # We need a blank line to compensate for the elimination of prompts.
    print

    # On z/OS we need to use EBCDIC for the human-editable silent file
    enc = platformUtil.getEncoding()

    try:
        debug("Processing Silent File : %s" % inputFile)
        infile = codecs.open(inputFile, mode="r", encoding=enc)
        lines = infile.readlines()
        infile.close()

    except Exception, errmsg:
        log.logError("An error occurred processing silent input files - %s" % (errmsg))
        messg = getmsg("CFG5033E", "The silent input file cannot be processed")
        raise ConfigurationException(messg)

    # Remove CR in case we are reading a DOS files on Unix.
    lines = [line.rstrip("\r\n") for line in lines]

    # Must purify debug output
    debug_lines = lines[:]
    for linei in xrange(len(debug_lines)):
        keyval = debug_lines[linei].split("=",2)
        if len(keyval) != 2: continue
        if keyval[0].strip() == "was.wsadmin.password" and keyval[1].strip() != "":
            debug_lines[linei] = keyval[0]+"="+"********"
    debug("Silent file contents -\n\n    %s" % "\n    ".join(debug_lines))

    getDefaultSection(pdl, lines)
    getServerSection(pdl, lines)
    validateInputs(pdl)
    updatePDL(pdl,ptype)

    #log.logDumpPDL(pdl,"Complete PDL after Silent File Processing",short=True,findservers=True)
    # FILES is built in updatePDL
    debug("Dump Final copy of FILES being returned by the program: %s" % FILES)

    debug("Exiting silentInputFileProcessor.processFile")
    return FILES


def getDefaultSection(pdl, lines):
    '''
       Reads the DEFAULT Section
       And creates the Dictionary DEFAULT_HOLDER
    '''

    global is_WLP

    debug("Entry getDefaultSection %s" % fixlev)

    section = ""
    for line in lines:

        # Ignore any empty lines
        if line.startswith("#"): continue
        if line.strip() == "": continue

        if line.strip() == "[DEFAULT SECTION]":
            debug("**********Processing Default Section**********")
            section = "Default Section"
            continue

        if line.strip() == "[SERVER]":
            section = "Server Section"
            continue

        ##############################################
        # Handle Default Section
        ##############################################

        if section == "Default Section":
            if line.count("=") > 0:
                ws = line.split("=",1)
                ws[0] = ws[0].strip()
                ws[1] = ws[1].strip()
                if ws[0] not in stcMap.keys():
                    messg = getmsg("CFG5042E", "This value: {0} is not currently supported in the silent configuration file",ws[0])
                    raise ConfigurationException(messg)
                DEFAULT_HOLDER[stcMap[ws[0]]] = ws[1]
                # Don't display the value of the password unless it's blank
                if ws[0] != "was.wsadmin.password" or ws[1] == "":
                    debug(ws[0] + " = "+ws[1])
                else:
                    debug(ws[0] + " = ********")
                if ws[0] == "was.appserver.home":
                    is_WLP = wasdiscover.isWLPHome(ws[1])
                    DEFAULT_HOLDER["is_wlp"] = str(is_WLP)
            else:
                messg = getmsg("CFG5042E", "This value: {0} is not currently supported in the silent configuration file",line)
                raise ConfigurationException(messg)
            continue

        if section == "Server Section":
            continue

        # If we get to this point, we have an orphan
        messg = getmsg("CFG5042E", "This value: {0} is not currently supported in the silent configuration file",line)
        raise ConfigurationException(messg)

    ent = DEFAULT_HOLDER.copy()
    if "password" in ent and ent["password"].strip() != "": ent["password"] = "********"
    debug("Dump Final copy of DEFAULT_HOLDER:\n\n    %s" %("\n    ".join(["%14s : %s" % (ky,val) for ky, val in sorted(ent.items())])))
    debug("Exiting silentInputFileProcessor.getDefaultSection")


def getServerSection(pdl, lines):
    '''
       Reads the SERVER Section
       And creates the list of dictionaries SERVER_HOLDER
       This incorporates the details from DEFAULT_HOLDER
    '''

    global is_WLP

    debug("Entry getServerSection %s" % fixlev)

    section = ""
    for line in lines:

        # Ignore empty lines
        if line.startswith("#"): continue
        if line.strip() == "": continue

        if line.strip() == "[DEFAULT SECTION]":
            section = "Default Section"
            continue

        if line.strip() == "[SERVER]":
            section = "Server Section"
            debug("**********Processing Server Section #%i" % len(SERVER_HOLDER))
            SERVER_HOLDER.append(DEFAULT_HOLDER.copy())
            continue

        if section == "Default Section":
            continue

        ##############################################
        # Handle Server Section
        ##############################################

        if section == "Server Section":
            if line.count("=") > 0:
                ws = line.split("=",1)
                ws[0] = ws[0].strip()
                ws[1] = ws[1].strip()
                # Only a subset of the keys are allowed in the server section
                if ws[0] not in stcMap.keys() or stcMap[ws[0]] not in vldServerKwds:
                    messg = getmsg("CFG5042E", "This value: {0} is not currently supported in the silent configuration file",ws[0])
                    raise ConfigurationException(messg)
                SERVER_HOLDER[-1][stcMap[ws[0]]] = ws[1]
                if ws[0] != "was.wsadmin.password":
                    debug(ws[0] + " = "+ws[1])
                else:
                    debug(ws[0] + " = ********")
            else:
                messg = getmsg("CFG5042E", "This value: {0} is not currently supported in the silent configuration file",line)
                raise ConfigurationException(messg)
            continue

        # If we get to this point, we have an orphan
        messg = getmsg("CFG5042E", "This value: {0} is not currently supported in the silent configuration file",line)
        raise ConfigurationException(messg)

    for serveri in range(len(SERVER_HOLDER)):
        ent = SERVER_HOLDER[serveri].copy()
        if "password" in ent and ent["password"].strip() != "": ent["password"] = "********"
        debug("Dump SERVER_HOLDER[%i]:\n\n    %s" % (serveri,"\n    ".join(["%14s : %s" % (ky,val) for ky, val in sorted(ent.items())])))
    debug("Exiting silentInputFileProcessor.getServerSection")


def validateInputs(pdl):
    '''
       All entries in the silent file are validated here
       This validation is done before setting the PDL with these values
       Validation is done from the data in DEFAULT_HOLDER and SERVER_HOLDER
    '''

    global is_WLP

    debug("Entry validateInputs %s" % fixlev)

    debug("Verifying data for DEFAULT section")
    for key, value in DEFAULT_HOLDER.iteritems():
        if key in dependencyMap:
            preqMap = dependencyMap.get(key)
            skip_validation = False
            for prereq_key in preqMap.keys():
                prereq_value = preqMap[prereq_key]
                default_value = DEFAULT_HOLDER[prereq_key]
                if str(default_value).lower() != str(prereq_value).lower():
                    debug("Skipping validation for key '%s' prequisite is not satisfied %s=%s" % (key, prereq_key, default_value))
                    skip_validation = True
            if skip_validation: continue

        format = vldMap.get(key)
        answer = value.strip()
        is_valid = False

        if format == "hostname":
            debug("Scrubbing  %s = %s" %(key,answer))
            is_valid = utils.isValidHostname(answer)
        elif format == "truefalse":
            debug("Scrubbing  %s = %s" %(key,answer))
            if answer.lower() == _TRUE or answer.lower() == _FALSE:
                is_valid = True
                #changing the value to lower case for downstream processing clarity
                DEFAULT_HOLDER[key]=answer.lower()
        elif format == "port":
            debug("Scrubbing  %s = %s" %(key,answer))
            is_valid = utils.isInt(answer)
        elif format == "portrange":
            debug("Scrubbing  %s = %s" %(key,answer))
            is_valid = utils.isValidRange(answer)
        elif format == "dirname":
            debug("Scrubbing  %s = %s" %(key,answer))
            is_valid = os.path.dirname(answer)
        elif format == "existingpath":
            debug("Scrubbing  %s = %s" %(key,answer))
            is_valid = len(answer) > 0 and (answer.find(" ") == -1) and os.path.exists(os.path.dirname(answer))
        elif format == "conntype":
            debug("Scrubbing  %s = %s" %(key,answer))
            is_valid = answer.lower() in ("soap","rmi")
        elif format == "wasroot":
            debug("Scrubbing  %s = %s" %(key,answer))
            is_valid = wasdiscover.verifyValidWasHome(answer,pdl)
        elif format == "server":
            debug("Scrubbing  %s = %s" %(key,answer))
            is_valid = not (len(answer.strip()) == 0 or answer == None)
        elif format == "listresp":
            debug("Scrubbing  %s = %s" %(key,answer))
            is_valid = (answer.lower() == "ad" or answer.lower() == "soa")
        elif format == "alphanumeric":
            debug("Scrubbing  %s = %s" %(key,answer))
            is_valid = utils.isAlphaNumericString(answer)

        # There are some formats which have no validity checking at this time
        elif format in ["string","username","cellname","profile","nodename","password"]:
            debug("Scrubbing bypassed for key <%s> format <%s>" % (key,format))
            is_valid = True

        else:
            debug("Invalid DEFAULT_HOLDER format %s for key %s" % (format,key))

        if not is_valid:
            messg = getmsg("CFG5038E", "The value of {0} which is: {1} in the silent file is invalid",[key, value])
            raise ConfigurationException(messg)

    for serveri in range(len(SERVER_HOLDER)):
        debug("Verifying data for [SERVER]  %s  " % serveri)
        if SERVER_HOLDER[serveri].get("server") == None:
            messg = getmsg("CFG5039E", "[SERVER] is uncommented for server {0}, but was.appserver.server.name is not defined",[serveri])
            raise ConfigurationException(messg)
        for key,value in SERVER_HOLDER[serveri].iteritems():
            if key in dependencyMap:
                preqMap = dependencyMap.get(key)
                skip_validation = False
                for prereq_key in preqMap.keys():
                    prereq_value = preqMap[prereq_key]
                    default_value = SERVER_HOLDER[serveri][prereq_key]
                    if str(default_value).lower() != str(prereq_value).lower():
                        debug("Skipping validation for key '%s' prequisite is not satisfied %s=%s" % (key, prereq_key, default_value))
                        skip_validation = True
                if skip_validation:
                    continue

            format = vldMap.get(key)
            answer = value.strip()
            is_valid = False

            if format == "hostname":
                debug("Scrubbing  %s = %s" %(key,answer))
                is_valid = utils.isValidHostname(answer)
            elif format == "truefalse":
                debug("Scrubbing  %s = %s" %(key,answer))
                if answer.lower() == _TRUE or answer.lower() == _FALSE:
                    is_valid = True
                    #changing the value to lower case for downstream processing clarity
                    SERVER_HOLDER[serveri][key]=answer.lower()
            elif format == "port":
                debug("Scrubbing  %s = %s" %(key,answer))
                is_valid = utils.isInt(answer)
            elif format == "portrange":
                debug("Scrubbing  %s = %s" %(key,answer))
                is_valid = utils.isValidRange(answer)
            elif format == "dirname":
                debug("Scrubbing  %s = %s" %(key,answer))
                is_valid = os.path.dirname(answer)
            elif format == "existingpath":
                debug("Scrubbing  %s = %s" %(key,answer))
                is_valid = len(answer) > 0 and (answer.find(" ") == -1) and os.path.exists(os.path.dirname(answer))
            elif format == "conntype":
                debug("Scrubbing  %s = %s" %(key,answer))
                is_valid = answer.lower() in ("soap","rmi")
            elif format == "wasroot":
                debug("Scrubbing  %s = %s" %(key,answer))
                is_valid = wasdiscover.verifyValidWasHome(answer,pdl)
            elif format == "server":
                debug("Scrubbing  %s = %s" %(key,answer))
                is_valid = not (len(answer.strip()) == 0 or answer == None)
            elif format == "listresp":
                debug("Scrubbing  %s = %s" %(key,answer))
                is_valid = (answer.lower() == "ad" or answer.lower() == "soa")
            elif format == "alphanumeric":
                debug("Scrubbing  %s = %s" %(key,answer))
                is_valid = utils.isAlphaNumericString(answer)

            # There are some formats which have no validity checking
            elif format in ["string","username","cellname","profile","nodename","password"]:
                debug("Scrubbing bypassed for key <%s> format <%s>" % (key,format))
                is_valid = True

            else:
                debug("Invalid SERVER_HOLDER format %s for key %s" % (format,key))

            if not is_valid:
                messg = getmsg("CFG5038E", "The value of {0} which is: {1} in the silent file is invalid",[key, value])
                raise ConfigurationException(messg)

    debug("Exiting silentInputFileProcessor.validateInputs")


def updatePDL(pdl, ptype):
    '''
       All PDL updates are handled from this method
       PDL updates are dependant on the mode CONFIG/UNCONFIG/MIGRATE
       This incorporates the details from DEFAULT_HOLDER
       Returns a list of file input.properties file
       for handling by findconfigservers
    '''

    debug("Entry updatePDL %s" % fixlev)

    migrateMode = "AD"

    utils.setData(pdl, "SERVERS", [], "silentInputFileProcessor")
    utils.setData(pdl, "USERLIST", {}, "silentInputFileProcessor")
    utils.setData(pdl, "PASSWORDLIST", {}, "silentInputFileProcessor")
    utils.setData(pdl, "USECLIENTPROPSLIST", {}, "silentInputFileProcessor")
    utils.setData(pdl, "FULLSERVERS", [], "silentInputFileProcessor")

    # Must default the JMX admin port
    conndict = readConnFile(pdl)
    if "jmx.port" in conndict:
        debug("Setting default JMX admin port: %s" % conndict["jmx.port"])
        pdl["JMXADMINPORT"][0:2] = conndict["jmx.port"], "silentInputFileProcessor"

    ####################################################
    #Special Processing for MIGRATE ONLY
    ####################################################
    #Point to migrate files
    #Set TEMA related PDLs
    if ptype == MIGRATE:

        #check if basics in silent file are ok
        ccm_install = utils.is_ccm(pdl)
        debug("Is scm ai install: %s" % ccm_install)
        if not ccm_install:
            checkBasicSilentFile(ptype)
        else:
            checkBasicSilentFileSCMAI(ptype)
            DEFAULT_HOLDER["listresp"] = "AD"
            DEFAULT_HOLDER["temaconnect"] = _TRUE
        if DEFAULT_HOLDER.get("listresp") == "AD" :
            utils.setData(pdl, "LISTRESP", "1", "InputFileProcessor")
            #get MIGRATEHOME
            mhome = DEFAULT_HOLDER.get("migratehome")
            utils.setData(pdl, "MIGRATEHOME", mhome, "silentInputFileProcessor")
            path = os.path.normpath(os.path.join(mhome, "runtime"))
            # MIGRATEHOME PDL is set later
            debug("Migration files at  %s" %(path))
            if DEFAULT_HOLDER.get("temaconnect") == _TRUE :
                pdl["TEMACONNECT"][0:2] = "1", "silentInputFileProcessor"
                pdl["TEMAHOST"][0:2] = DEFAULT_HOLDER.get("temahost"), "silentInputFileProcessor"
                pdl["TEMAPORT"][0:2] = DEFAULT_HOLDER.get("temaport"), "silentInputFileProcessor"
            else: pdl["TEMACONNECT"][0:2] = "0", "silentInputFileProcessor"
        elif DEFAULT_HOLDER.get("listresp") == "SOA" :
            migrateMode = "SOA"
            utils.setData(pdl, "LISTRESP", "5", "InputFileProcessor")
            pdl["MIGRATESOA71WASROOT"][0:2] = DEFAULT_HOLDER.get("wasroot"), "silentInputFileProcessor"
            path = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "runtime"))
    else:
        path = os.path.normpath(os.path.join(pdl["CONFIGHOME"][0], "runtime"))

    ####################################################
    # For unconfigure of all servers, we identify all
    # configured servers by locating the input.properties
    # files previously created for them when they were
    # configured.  For configure we have to delay
    # handling asterisks until we have done findservers.
    ####################################################

    if ptype == UNCONFIG or ptype == MIGRATE:

        # Remove the asterisk servers from the list of servers
        # and insert all currrently configured servers.
        for serveri in reversed(range(len(SERVER_HOLDER))):
            if SERVER_HOLDER[serveri].get("server") == "*":

                # Identify servers that are configured by enumeration of the input.properties files
                debug("Asterisk processing for SERVER_HOLDER entry %i" % serveri)

                profile = SERVER_HOLDER[serveri].get("profile") or ""

                # Normal WebSphere has a profile, cell name and node name.
                # Liberty has no profile, and the cell name and node name are derived.

                cellname = utils.convert_nonascii(SERVER_HOLDER[serveri].get("cellname") or "")
                nodename = utils.convert_nonascii(SERVER_HOLDER[serveri].get("nodename") or "")

                if "profile" in SERVER_HOLDER[serveri]:
                    # When the profile has been provided, we conclude it is not Liberty.
                    # In that case, cell and node and server must be specified.
                    debug("Profile %s was specified, non-Liberty WebSphere assumed" % profile)
                    assert cellname != "" and nodename != "", getmsg("CFG2106E", "Profile, cell name and node name must all be specified for WebSphere silent unconfiguration")
                    filePrefix = profile + "." + cellname + "." + nodename + "."

                else:
                    # When the profile is not provided, we conclude it is Liberty.
                    # In that case, cell and node and server must not be specified.
                    debug("No profile specified, Liberty assumed")
                    assert cellname == "" and nodename == "", getmsg("CFG2107E", "Neither profile, cell name nor node name may be specified for Liberty silent unconfiguration")
                    cellname = utils.convert_nonascii(wlpConfigUtils.getCellName())
                    nodename = utils.convert_nonascii(wlpConfigUtils.getNodeName(SERVER_HOLDER[serveri].get("wasroot")))
                    filePrefix = cellname + "." + nodename + "."

                debug("Finding all %s*.input.properties files under %s" % (filePrefix, path))
                for basename in os.listdir(path):
                    filename = os.path.join(path,basename)
                    if not os.path.isfile(os.path.join(path,filename)): continue
                    if not basename.startswith(filePrefix): continue
                    # Make sure this is an input.properties file but not also 
                    # a dcless.input.properties file as well.
                    r = pat_input.match(basename)
                    if r is None or r.group(1) is None: continue
                    servername = r.group(1)
                    debug("  Server name: %s for %s" % (servername,basename))
                    # Notice we have to make a copy of the dictionary here to replicate and change it.
                    SERVER_HOLDER.append(SERVER_HOLDER[serveri].copy())
                    SERVER_HOLDER[-1]["server"] = servername
                del SERVER_HOLDER[serveri]

        debug("Servers selected for unconfigure:")
        for serveri in xrange(len(SERVER_HOLDER)):
            if "profile" in SERVER_HOLDER[serveri]:
                debug("%3i. %s.%s.%s.%s" % (serveri,SERVER_HOLDER[serveri].get("profile"),SERVER_HOLDER[serveri].get("cellname"),SERVER_HOLDER[serveri].get("nodename"),SERVER_HOLDER[serveri].get("server")))
            else:
                debug("%3i. %s" % (serveri,SERVER_HOLDER[serveri].get("server")))

    ####################################################
    # No matter what mode we are in we need to do certain things
    ####################################################
    #Create SERVER PDL
    #Create USERLIST PDL
    #CREATE PASSWORDLIST PDL
    #CREATE BACKUP PDL
    #CREATE DEFAULTHOSTIP

    # Because silent mode has created the server entries and fullservers entries
    # from the same source, for silent mode SERVERS and FULLSERVERS have
    # identical indexes.  When not in silent mode these indexes are not related.

    for serveri in range(len(SERVER_HOLDER)):
        utils.getData(pdl, "SERVERS").append(SERVER_HOLDER[serveri].get("server"))
        fullservers_dict = {}
        fullservers_dict["PROFILE"] = SERVER_HOLDER[serveri].get("profile")
        fullservers_dict["WASCELL"] = SERVER_HOLDER[serveri].get("cellname")
        fullservers_dict["WASNODE"] = SERVER_HOLDER[serveri].get("nodename")
        fullservers_dict["WASSERVER"] = SERVER_HOLDER[serveri].get("server")
        utils.getData(pdl, "FULLSERVERS").append(fullservers_dict)
        debug("Printing wasroot:  %s" % (SERVER_HOLDER[serveri].get("wasroot")))
        is_liberty_server = wasdiscover.isWLPHome(SERVER_HOLDER[serveri].get("wasroot"))
        # Profile discovery if not Liberty
        if not is_liberty_server and "profile" in SERVER_HOLDER[serveri]:
            debug("Profile discovery for non-Liberty WebSphere")
            profile_name_list, profile_path_list, default_profile_name = wasdiscover.get_was_profiles(pdl, SERVER_HOLDER[serveri].get("wasroot"))
            j = 0
            debug("Printing Profile list  %s" % profile_name_list)
            for index in range(len(profile_name_list)):
                if profile_name_list[index] == SERVER_HOLDER[serveri].get("profile"):
                    j=index
                    debug("Matching Profile  %s" %(SERVER_HOLDER[serveri].get("profile")))
                    break
            if SERVER_HOLDER[serveri].get("useclientprops") == _TRUE :
                pdl["USECLIENTPROPS"][0:2] = True, "silentInputFileProcessor"
                pdl["USECLIENTPROPSLIST"][0][profile_path_list[j]] = True
                pdl["USECLIENTPROPSLIST"][1] = "silentInputFileProcessor"
            else:
                pdl["USECLIENTPROPS"][0:2] = False, "silentInputFileProcessor"
                pdl["USECLIENTPROPSLIST"][0][profile_path_list[j]] = False
                pdl["USECLIENTPROPSLIST"][1] = "silentInputFileProcessor"
                user = SERVER_HOLDER[serveri].get("user")
                password = SERVER_HOLDER[serveri].get("password")
                debug("User is %s -  Password is %s" % (user, "********"))
                if user != None and password != None and len(user) > 0 and len(password) > 0:
                    pdl["USERLIST"][0][profile_path_list[j]]     = SERVER_HOLDER[serveri].get("user")
                    pdl["USERLIST"][1] = "silentInputFileProcessor"
                    pdl["PASSWORDLIST"][0][profile_path_list[j]] = SERVER_HOLDER[serveri].get("password")
                    pdl["PASSWORDLIST"][1] = "silentInputFileProcessor"

        # Profile discovery for Liberty
        else:
            debug("Profile discovery for Liberty")
            wasroot = SERVER_HOLDER[serveri].get("wasroot")
            wasroot_expanded = File(wasroot).getCanonicalPath()
            SERVER_HOLDER[serveri]["wasroot"] = wasroot_expanded
            pdl["WLPINSTALLDIR"][0:2] = wasroot_expanded, "silentInputFileProcessor"
            wlpInstallDir = pdl["WLPINSTALLDIR"][0]
            pdl["WLPUSRDIR"][0:2] = wlpConfigUtils.getWlpUserDir(wlpInstallDir), "silentInputFileProcessor"
            cellname = wlpConfigUtils.getCellName()
            nodename = wlpConfigUtils.getNodeName(wlpInstallDir)
            #liberty doesn't have cell/node/profile filling in with 'our' values
            SERVER_HOLDER[serveri]["profile"] = ""
            SERVER_HOLDER[serveri]["cellname"] = cellname
            SERVER_HOLDER[serveri]["nodename"] = nodename
            fullservers_dict["PROFILE"] = ""
            fullservers_dict["WASCELL"] = cellname
            fullservers_dict["WASNODE"] = nodename
            fullservers_dict["WASSERVER"] = SERVER_HOLDER[serveri].get("server")

        # For Liberty, the None profile just got replaced by ""
        if SERVER_HOLDER[serveri].get("profile") != "":
            filePrefix = utils.convert_nonascii(SERVER_HOLDER[serveri].get("profile")) + "."
        else:
            filePrefix = ""
        filePrefix = filePrefix + utils.convert_nonascii(SERVER_HOLDER[serveri].get("cellname")) + "."
        filePrefix = filePrefix + utils.convert_nonascii(SERVER_HOLDER[serveri].get("nodename")) + "."
        filePrefix = filePrefix + utils.convert_nonascii(SERVER_HOLDER[serveri].get("server")) + "."
        filename = os.path.normpath(os.path.join(path, filePrefix + "input.properties"))

        # Check if the input.properties file exists for the specified profile, cell, node and server.

        if not os.path.isfile(filename) and ptype == UNCONFIG:
            filename += ".bak"
            if os.path.isfile(filename):
                messg = getmsg("CFG5036E", "This combination of profile name {0), cell name {1}, node name {2}, server name {3} is already unconfigured.",[SERVER_HOLDER[serveri].get("profile"), SERVER_HOLDER[serveri].get("cellname"), SERVER_HOLDER[serveri].get("nodename"),SERVER_HOLDER[serveri].get("server")])
                raise ConfigurationException(messg)
            messg = getmsg("CFG5035E", "This combination of profile name {0}, cell name {1}, node name {2}, server name {3} is not a valid combination.",[SERVER_HOLDER[serveri].get("profile"), SERVER_HOLDER[serveri].get("cellname"), SERVER_HOLDER[serveri].get("nodename"),SERVER_HOLDER[serveri].get("server")])
            raise ConfigurationException(messg)

        FILES.append(filename)

        debug("Adding %s to target file list" % filename)

    debug("Final Target file list:%s" % (len(FILES) == 0 and " []" or "\n    "+"\n    ".join(FILES)))

    #Processing Backups. This is done from the default holder
    #It is common across all servers
    if DEFAULT_HOLDER.get("backup") == _TRUE :
        pdl["BACKUP"][0:2] = True, "silentInputFileProcessor"
        pdl["BACKUPDIR"][0:2] = DEFAULT_HOLDER.get("backupdir"), "silentInputFileProcessor"
    else:
        pdl["BACKUP"][0:2] = False, "silentInputFileProcessor"

    if DEFAULT_HOLDER.get("defaulthostip") != None:
        pdl["DEFAULTHOSTIP"][0:2] = DEFAULT_HOLDER.get("defaulthostip"), "silentInputFileProcessor"
        debug("Defaulthost ip set at: %s" %(DEFAULT_HOLDER.get("defaulthostip")))


    if ptype == MIGRATE and migrateMode != "SOA":
        #read in tema alias if specified
        pdl["ALIASLIST"][0:2] = [], "silentInputFileProcessor"
        for serveri in range(len(SERVER_HOLDER)):
            if "serveralias" in SERVER_HOLDER[serveri]:
                debug("found alias for server %s" % serveri)
                pdl["ALIASLIST"][0].append(SERVER_HOLDER[serveri]["serveralias"])
                pdl["SERVERALIAS"][0:2] = SERVER_HOLDER[serveri]["serveralias"], "silentInputFileProcessor"
            else:
                pdl["ALIASLIST"][0].append(None)
    ####################################################
    # This is only for Config or Migrate SOA Mode      #
    ####################################################
    elif ptype == CONFIG or migrateMode == "SOA":

        pdl["ALIASLIST"][0:2] = [], "silentInputFileProcessor"
        for serveri in range(len(SERVER_HOLDER)):
            if "serveralias" in SERVER_HOLDER[serveri]:
                pdl["ALIASLIST"][0].append(SERVER_HOLDER[serveri]["serveralias"])
            else:
                pdl["ALIASLIST"][0].append(None)

        if DEFAULT_HOLDER.get("temadcappserver") is not None:
            pdl["TEMADCAPPSERVER"][0:2] = DEFAULT_HOLDER.get("temadcappserver") == _TRUE, "silentInputFileProcessor"
        if DEFAULT_HOLDER.get("dclessconntype") is not None:
            pdl["DCLESSCONNTYPE"][0:2] = DEFAULT_HOLDER.get("dclessconntype").upper(), "silentInputFileProcessor"

        if DEFAULT_HOLDER.get("conntype") != None:
            pdl["CONNTYPE"][0] = DEFAULT_HOLDER.get("conntype").upper()
            pdl["CONNTYPE"][1] = "silentInputFileProcessor"
        if DEFAULT_HOLDER.get("port") != None :
            pdl["PORT"][0] = DEFAULT_HOLDER.get("port")
            pdl["PORT"][1] = "silentInputFileProcessor"
        pdl["WASROOT"][0:2] = DEFAULT_HOLDER.get("wasroot"), "silentInputFileProcessor"
        pdl["PROFILE"][0:2] = DEFAULT_HOLDER.get("profile"), "silentInputFileProcessor"
        if DEFAULT_HOLDER.get("temaconnect") == _TRUE :
            pdl["TEMACONNECT"][0:2] = "1", "silentInputFileProcessor"
            pdl["TEMAHOST"][0:2] = DEFAULT_HOLDER.get("temahost"), "silentInputFileProcessor"
            pdl["TEMAPORT"][0:2] = DEFAULT_HOLDER.get("temaport"), "silentInputFileProcessor"
        else:
            pdl["TEMACONNECT"][0:2] = "0", "silentInputFileProcessor"
        if DEFAULT_HOLDER.get("jmxport") is not None:
            pdl["JMXADMINPORT"][0:2] = DEFAULT_HOLDER.get("jmxport"), "silentInputFileProcessor"

        if DEFAULT_HOLDER.get("msconnect") == _TRUE :
            pdl["MSCONNECT"][0:2] = "1", "silentInputFileProcessor"
            pdl["KERNELIP"][0:2] = DEFAULT_HOLDER.get("kernelip"), "silentInputFileProcessor"
            pdl["KERNELPORT01"][0:2] = DEFAULT_HOLDER.get("kernelport01"), "silentInputFileProcessor"
            pdl["MSHOME"][0:2] = DEFAULT_HOLDER.get("mshome"), "silentInputFileProcessor"
            pdl["PROBECONTROLLERRMIPORT"][0:2] = DEFAULT_HOLDER.get("probermicontrollerport"), "silentInputFileProcessor"
            pdl["RMIPORTSET"][0:2] = DEFAULT_HOLDER.get("probermiport"), "silentInputFileProcessor"
        else:
            pdl["MSCONNECT"][0:2] = "0", "silentInputFileProcessor"

        if DEFAULT_HOLDER.get("ttapi") == _TRUE :
            pdl["TTAPI"][0:2] = "1", "silentInputFileProcessor"
            pdl["TTTCHOST"][0:2] = DEFAULT_HOLDER.get("tthost"), "silentInputFileProcessor"
            pdl["TTTCPORT"][0:2] = DEFAULT_HOLDER.get("ttport"), "silentInputFileProcessor"
        else:
            pdl["TTAPI"][0:2] = "0", "silentInputFileProcessor"

        if DEFAULT_HOLDER.get("soadc") == _TRUE :
            pdl["SOADC"][0:2] = "1", "silentInputFileProcessor"
        else:
            pdl["SOADC"][0:2] = "0", "silentInputFileProcessor"
        if migrateMode == "SOA":
            pdl["SOADC"][0:2] = "1", "silentInputFileProcessor"

        if DEFAULT_HOLDER.get("tpv") == _TRUE :
            pdl["TPV"][0:2] = "1", "silentInputFileProcessor"
        else:
            pdl["TPV"][0:2] = "0", "silentInputFileProcessor"

        # This is before configtema is called, so we don't have to worry about
        # forcing DE on for TEMA at this point.
        if DEFAULT_HOLDER.get("de") == _TRUE :
            pdl["DE"][0:2] = "1", "silentInputFileProcessor"
        else:
            pdl["DE"][0:2] = "0", "silentInputFileProcessor"

        #For config only, we process GC Settings and Heap Size
        if DEFAULT_HOLDER.get("gcoutputdist") == _TRUE :
            pdl["GCOUTPUTDIST"][0:2] = "1", "silentInputFileProcessor"
            pdl["GCLOGPATH"][0] = DEFAULT_HOLDER.get("gclogpath")
        else:
            pdl["GCOUTPUTDIST"][0:2] = "0", "silentInputFileProcessor"
        if DEFAULT_HOLDER.get("heapsize") == _TRUE :
            pdl["HEAPSIZE"][0:2] = "1", "silentInputFileProcessor"
        else: pdl["HEAPSIZE"][0:2] = "0", "silentInputFileProcessor"

    debug("Exiting silentInputFileProcessor.updatePDL")


def checkBasicSilentFile(ptype):
    '''
       Checks - to extent reasonable - the contents of
       silent file depending on the mode CONFIG/UNCONFIG/MIGRATE
    '''

    debug("Entry checkBasicSilentFile %s" % fixlev)

    if ptype == MIGRATE:
        if "listresp" not in DEFAULT_HOLDER.keys():
            pname = "migrate.type"
            messg = getmsg("CFG5044E", "Parameter: {0} is missing from the silent input file",[pname])
            raise ConfigurationException(messg)
        if DEFAULT_HOLDER.get("listresp") == "SOA" :
            #make sure SOA WAS ROOT HOME is given
            if DEFAULT_HOLDER.get("wasroot") == None :
                pname = "was.appserver.home"
                messg = getmsg("CFG5044E", "Parameter: {0} is missing from the silent input file",[pname])
                raise ConfigurationException(messg)
        if DEFAULT_HOLDER.get("listresp") == "AD" :
            #make sure ITCAM MIGRATE HOME is given
            if DEFAULT_HOLDER.get("migratehome") == None :
                pname = "itcam.migrate.home"
                messg = getmsg("CFG5044E", "Parameter: {0} is missing from the silent input file",[pname])
                raise ConfigurationException(messg)

    debug("Exiting silentInputFileProcessor.checkBasicSilentFile")
    
def checkBasicSilentFileSCMAI(ptype):
    '''
       Checks - to extent reasonable - the contents of
       silent file depending on the mode CONFIG/UNCONFIG/MIGRATE
    '''

    debug("Entry checkBasicSilentFileSCMAI %s" % fixlev)

    if ptype == MIGRATE:
        if DEFAULT_HOLDER.get("migratehome") == None :
            pname = "itcam.migrate.home"
            messg = getmsg("CFG5044E", "Parameter: {0} is missing from the silent input file",[pname])
            raise ConfigurationException(messg)

    debug("Exiting silentInputFileProcessor.checkBasicSilentFileSCMAI")


def check_servers(pdl):
    '''
       Checks the FULLSERVERS pdl against the FINDSERVERS and check the configured
       flag, to determine if the server has already been configured or this is a
       pristine server.
    '''

    debug("Entry check_servers %s" % fixlev)

    # Process asterisk defaults
    server_alldefault(pdl)

    stat = False
    new_fullservers = []
    new_servers = []
    findservers = utils.getDataFlex(pdl, "FINDSERVERS")
    fullservers = utils.getDataFlex(pdl, "FULLSERVERS")
    current_version = utils.getData(pdl, "VERSION")

    # Loop thru all the servers, and check it against the FINDSERVERS, and
    # if already configured by the previous release, then issue a message and
    # do not process it.

    for serveri in range(len(fullservers)):
        fullservers_entry = fullservers[serveri]
        fulls_wasprofile = fullservers_entry["PROFILE"]
        fulls_wascell = fullservers_entry["WASCELL"]
        fulls_wasnode = fullservers_entry["WASNODE"]
        fulls_wasserver = fullservers_entry["WASSERVER"]
        for sindex in range(len(findservers)):
            findservers_entry = findservers[sindex]
            finds_wascell = findservers_entry["WASCELL"]
            finds_wasprofile = findservers_entry["PROFILE"]
            finds_wasnode = findservers_entry["WASNODE"]
            finds_wasserver = findservers_entry["WASSERVER"]
            if fulls_wasprofile == finds_wasprofile and fulls_wascell == finds_wascell and fulls_wasnode == finds_wasnode and fulls_wasserver == finds_wasserver:
                debug("Found a match in FINDSERVERS pdl - %s, %s, %s, %s" %(finds_wasprofile, finds_wasnode, finds_wascell, finds_wasserver))
                configured = findservers_entry["CONFIGURED"]
                configured_version = checkversion.convert_version(configured)
                if configured is None or configured == "None" or configured == current_version:
                    debug("Application Server is a pristine server - %s, %s, %s, %s" %(finds_wasprofile, finds_wasnode, finds_wascell, finds_wasserver))
                    new_fullservers.append(fullservers_entry)
                    new_servers.append(fulls_wasserver)
                elif configured == "VE":
                    debug("Application Server configured with VE - %s, %s, %s, %s" %(finds_wasprofile, finds_wasnode, finds_wascell, finds_wasserver))
                    msgl =[]
                    msgl.append(finds_wasprofile)
                    msgl.append(finds_wascell)
                    msgl.append(finds_wasnode)
                    msgl.append(finds_wasserver)
                    msMsg = "\n" + getmsg("CFG1151I", "Application server configured by ITCAM Managing Server. {0} - {1} - {2} - {3}", msgl)
                    print msMsg
                elif configured_version > checkversion.convert_version(current_version):
                    debug("Servers configured with newer ITCAM - %s, %s, %s, %s, %s" %(finds_wasprofile, finds_wasnode, finds_wascell, finds_wasserver, configured_version))
                    msgl =[]
                    msgl.append(finds_wasprofile)
                    msgl.append(finds_wascell)
                    msgl.append(finds_wasnode)
                    msgl.append(finds_wasserver)
                    msgl.append(configured_version)
                    msMsg = "\n" + getmsg("CFG1152I", "Application server configured with newer version of ITCAM. {0} - {1} - {2} - {3} - {4}", msgl)
                    print msMsg
                else:
                    debug("Servers configured with older ITCAM - %s, %s, %s, %s, %s" %(finds_wasprofile, finds_wasnode, finds_wascell, finds_wasserver, configured_version))
                    msgl =[]
                    msgl.append(finds_wasprofile)
                    msgl.append(finds_wascell)
                    msgl.append(finds_wasnode)
                    msgl.append(finds_wasserver)
                    msgl.append(configured_version)
                    msMsg = "\n" + getmsg("CFG1153I", "Application server configured with older version of ITCAM. {0} - {1} - {2} - {3} - {4}", msgl)
                    print msMsg

    # Save the findservers entries and servers entries into the pdl.  These entries
    # are the entries that are valid for configuration.

    utils.setData(pdl, "FULLSERVERS", new_fullservers, "silentInputFileProcess")
    utils.setData(pdl, "SERVERS", new_servers, "silentInputFileProcess")

    if len(new_fullservers) > 0:
        stat = True
    else:
        debug("There are No Application servers to configure")
        msMsg = "\n" + getmsg("CFG1154I", "There are no application servers to configure.")
        print msMsg

    debug("Exit check_servers - %s" %(stat));

    return stat


if __name__ == "__main__":

    import sys
    import createPDL as createPDL
    import createPCL_config as createPCL
    import resourcebundle

    pdl = createPDL.make_pdl(createPCL.make_pcl())
    createPDL.proc_envvar(pdl)
    createPDL.proc_cliopt(pdl,sys.argv)

    log.set_logging()
    debug("Unit Test for silentInputFileProcessor")

    processFile(pdl,r"C:\Source\ant\33584\silent_config.txt",CONFIG)

    log.logDumpPDL(pdl,"Complete PDL after Silent File Processing",short=False)
