#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

'''
    Prompt for websphere backup
'''

fixlev = "40296" # Last APAR, defect or story to update this module

import os

from time import strftime

import doprompt
import wasdiscover
import utils
import debugUtil as log

from debugUtil import logDebug as debug
from resourcebundle import getmsg


def process_backup(pdl, backup_dir, serveri):
    '''
       Do the actual backup.  The backup file name contains the
       cellname.nodename.profilename.prefix_date.zip.

       @param pdl - PDL reference
       @param backup_dir - Backup directory path
       @param serveri - Index for a specific server that corresponds to SERVERS pdl
    '''

    backup_created = True
    continue_config = True
    done = False

    timestamp = strftime("%Y%m%d%H%M%S")
    wascell = utils.getData(pdl, "WASCELL", serveri)
    wasnode = utils.getData(pdl, "WASNODE", serveri)
    wascell = wascell.replace(" ", "_")
    wasnode = wasnode.replace(" ", "_")
    profile = pdl["PROFILE"][0][serveri]
    wlphome = utils.getData(pdl, "WLPHOME")[serveri]
    is_wlp = False
    if not utils.isSet(wlphome):
        file_name = wascell + "." + wasnode + "." + profile + "." + "WebSphereConfig_" + timestamp + ".zip"
        backup_file = os.path.normpath(os.path.join(backup_dir, file_name))
    else:
        backup_file = None
        is_wlp = True
        debug("backing up Liberty profile, no WAS-provided backup script")

    # If file already exists, prompt if to reuse it or not

    while not done:
        if backup_file != None and os.path.exists(backup_file):
            msgl = []
            msgl.append(backup_file)
            bmsg = getmsg("CFG2014I", "File  {0}  already exists. Do you want to overwrite?", msgl)
            answer = doprompt.prompt_file_overwrite(pdl, bmsg)
            if answer == 0:
                doprompt.do_prompt("BACKUPSUFFIX", pdl, alt_prompt="Specify the suffix for the backup file")
                suffix = utils.getData(pdl, "BACKUPSUFFIX")
                file_name = wascell + "." + wasnode + "." + profile + "." + suffix + "_" + timestamp + ".zip"
                backup_file = os.path.normpath(os.path.join(backup_dir, file_name))
                continue
            else:
                done = True
        else:
            done = True

    # Do the backup

    try:
        backup_failed = False
        rc, output = wasdiscover.process_backup(pdl, backup_file, serveri, is_wlp)
        #rc, output = wasdiscover.backup_was(pdl, backup_file, serveri)
        if rc == 0:
            if (backup_file != None and os.path.exists(backup_file)) or backup_file == None:
                msgl =[]
                if backup_file != None:
                    msgl.append(backup_file)
                else:
                    msgl.append(output)
                bmsg = getmsg("CFG2011I", "Backup file {0} is successfully created", msgl)
                print bmsg
            else:
                backup_failed = True
        else:
            backup_failed = True

        if backup_failed:
            msgl =[]
            msgl.append(output)
            bmsg = getmsg("CFG2012E", "CFG2012E Failed to create websphere backup {0} ", msgl)
            print bmsg
            backup_created = False
    except:
        backup_created = False
        log.logException("Unable to perform WAS backup")
        msgl =[]
        msgl.append(backup_dir)
        bmsg = getmsg("CFG2012E", "CFG2012E Failed to create websphere backup {0} ", msgl)
        print bmsg

    # Check to see if error took place, then ask a question to continue with the
    # config or not

    if backup_created == False:
        doprompt.do_prompt("CONTINUECONFIG", pdl, alt_prompt="CFG2013I Would you like to continue with config?")
        if pdl["CONTINUECONFIG"][0] == False:
            continue_config = False

    return continue_config


def do_backup(pdl):
    '''
       This method call backs up the was home.  A prefix of cell name and node name
       is appended to the backup file name to make a unique name.

       @param pdl - PDL reference
       @param backup_file - Backup file name
    '''

    debug("Entry do_backup %s" % fixlev)

    done = False
    continue_config = True
    (only_wlp_selected, wlp_servers) = utils.get_wlp_selected(pdl)
    doprompt.do_prompt("BACKUP", pdl, alt_prompt="Do you want to backup current websphere configuration?")
    if pdl["BACKUP"][0] == True:
        confighome = pdl["CONFIGHOME"][0]
        backup_dir_default = confighome
        if utils.createDataDir(confighome):
            backup_dir_default = os.path.normpath(os.path.join(backup_dir_default, "data"))

        # we want to update this PDL only if we are not in silent config
        # otherwise this would override what is set in the silent file

        if not pdl["SILENT"][0]:
            pdl["BACKUPDIR"][0] = backup_dir_default
            pdl["BACKUPDIR"][1] = "Init"
        if not only_wlp_selected:
            while not done:
                doprompt.do_prompt("BACKUPDIR", pdl, alt_prompt="Enter backup directory name")
                backup_dir = pdl["BACKUPDIR"][0]
                if not os.path.isdir(backup_dir):
                    try:
                        os.makedirs(backup_dir)
                        debug("Created backup directory  %s" % (backup_dir))
                        done = True
                    except OSError, errarg:
                        log.logError("Unable to create %s- %s" % (backup_dir, errarg))
                        msgl = []
                        msgl.append(backup_dir)
                        msgl.append(errarg)
                        errmsg = getmsg("CFG1081E", "Unable to create {0} - {1}", msgl)
                        print errmsg
                else:
                    done = True

        # For each server in SERVERS pdl, search each entry and group all the servers
        # that belongs to the same cell, node.

        backup_dir = pdl["BACKUPDIR"][0]
        group_servers = utils.group_servers(pdl)
        continue_config = True

        # Each group belongs to a profile, so we only get the first index which has all
        # the information about the profile.

        for group_index in group_servers.keys():
            index_list = group_servers[group_index]
            if group_index == "liberty":
                i = 0
                while i < len(index_list):
                    if continue_config:
                        continue_config = process_backup(pdl, backup_dir, index_list[i])
                    i += 1
            else:
                continue_config = process_backup(pdl, backup_dir, index_list[0])
            if continue_config == False:
                break

    debug("Exit do_backup")
    return continue_config
