#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2014 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
#from test.test___future__ import features
'''
    wlpConfigDataCollector.py
'''
import sys
import re
import os
import time
import codecs
import traceback
import shutil
import utils
import errno

from time import strftime
from java.lang import System
import java.io.File
from xml.dom import minidom

import platformUtil
import debugUtil as log
import resourcebundle
from constants import constants
from debugUtil import debug as debug
import wlpConfigUtils
import checkversion
##Global variables

timestamp = strftime("%Y-%m-%d %H:%M:%S")
VERSIONSTRING = "ITCAM 7.2 AK0000"
SCRIPT = "wlpConfigDataCollector"
SCRIPTNAME = SCRIPT + ".py"

common_constants = None
nodeName = ""
serverName = ""
serverid = ""

# This is set after parameters are validated
runtime_path = ""

maximumHeapSize = ""

product = ""
genericargs = []
sharedLib = ""
serverVersion = ""
server_runtime = ""
itcamdc_home = ""
server_classpath = ""
server_bootclasspath = ""
smf = "No"
server_amhome_flag = 0
JVMSecurityPolicy_flag = 0
ecam_flag = 0
control_environments = []
control_properties = []

pmiService_flag = 0
extraRMI_flag = 0

probe = 0
wascell = "None"
wasnode = "None"
wasserver = "None"
current_inputproperties = "None"
summary = {}
config_type = -1
old_am_home = ""
configComplete = 0
wlpInstallRoot = ""
wlpUsrDirPath = ""
old_itcamdc_home = "${ITCAMDCHOME}"
wlpServerEnvFile = ""
wlpJvmOptionsFile = ""
wlpServerXmlFile = ""
current_server_env_dic = {}
old_jvm_args = None

def configure(pdl, serveri, inputPropertyFile, ctype, progress_obj):
    '''
        Configure
        @parm  input_record - Record form the input list file.
        @param progress - progress object reference
    '''
    global SCRIPT, VERSIONSTRING, timestamp
    

    global product
    global platform
    global nodeName
    global serverid
    global customservice
    global systemproperties
    global variables
    global control_environments
    global control_properties
    global genericargs
    global sharedLib
    global serverVersion
    global server_runtime
    global itcamdc_home
    global server_classpath
    global smf
 
    global serverName
    global runtime_path
    global server_amhome_flag
    global JVMSecurityPolicy_flag
    global ecam_flag
    global maximumHeapSize
    global probe
    global wascell
    global wasnode
    global wasserver
    global current_inputproperties
    global config_type
    global common_constants
    global old_am_home
    global configComplete
    global wlpInstallRoot
    global wlpUsrDirPath
    global old_itcamdc_home
    global server_bootclasspath
    global wlpServerEnvFile 
    global wlpJvmOptionsFile
    global wlpServerXmlFile
    global old_jvm_args

    product = ""
    platform = ""
    serverid = ""
    customservice = []
    systemproperties = []
    environments = []
    variables = []
    variables_dic = {}

    genericargs = []
    sharedLib = ""
    serverVersion = ""
    server_runtime = ""
    itcamdc_home = ""
    server_classpath = ""
    server_bootclasspath = ""
    smf = "No"
    JVMSecurityPolicy_flag = 0
    ecam_flag = 0
    control_environments = []
    control_properties = []
    configComplete = 0
    failed_flag = 0

    # This flag is turned on to enable the PMI service.  We do not
    # disable the PMI service under any condtion.

    pmiService_flag = 0
    extraRMI_flag = 0

    old_am_home = ""
    server_dic = {}
    
    debug("configure() Entry")
    
    wlpInstallRoot = utils.getData(pdl, "WLPHOME", serveri)
    wlpUsrDirPath = utils.getData(pdl, "USERINSTALLROOT", serveri)
    
    wascell = utils.getData(pdl,"WASCELL", serveri)
    wasnode = utils.getData(pdl,"WASNODE", serveri)
    wasserver = utils.getData(pdl,"SERVERS", serveri)
    wasprofile = utils.getData(pdl, "PROFILE", serveri)
    
    itcam_feature_list = []
    
    config_type = ctype
    runtimedir = pdl["CONFIGHOME"][0] + os.sep + "runtime"
    #inputPropertyFilePrefix = utils.getServerBasedFilePrefix(pdl, server_dic)
    inputPropertyFilePrefix = wascell.strip() + "." + wasnode.strip() + "." + wasserver.strip() + "." + "input.properties"
    
    inputfile = runtimedir + os.sep + inputPropertyFilePrefix
    debug("configtype: %s  probe: %s  WAS cell: %s  WAS node: %s  WAS server: %s  inputfile: %s" % (config_type, probe, wascell, wasnode, wasserver, inputfile))
    
    #Liberty configuration files for the server
    wlpServerRoot = wlpUsrDirPath + os.sep + "servers" + os.sep + wasserver
    wlpServerEnvFile = wlpServerRoot + os.sep + "server.env"
    wlpJvmOptionsFile = wlpServerRoot + os.sep + "jvm.options"
    wlpServerXmlFile = wlpServerRoot + os.sep + "server.xml"

    # Read in the response file
    debug ("\nresponse file "+inputfile)
    try:
        inDesc = codecs.open(inputfile, mode='r', encoding='utf-8')
        #inDesc = open( inputfile, "r" )
    except:
        errorCode = None
        raise RuntimeError, inputfile+" failed to open\n"

    lines = []
    while 1:
        buffer = inDesc.readline()
        if not buffer: break
        lines.append(buffer)
    inDesc.close()

    debug("%d lines read from %s" % (len(lines),inputfile))

    # Note: Any value which may contain equal signs must
    #       be handled with regexp instead of lindex.
    # Comments are not allowed with a switch (probably JACL bug)
    # Extension MBean Provider is obsolete
    #   server.xMBeanProvider
    # These parameters, while still valid, should be replaced by server.variable specifications
    #   server.itcamdchome server.itcam61home
    # These two parameters are obsolete so they are ignored
    #   server.am.home server.ms.am.home
    # Note:server.systemproperty=am.home is used in ITCAM 7.2(ecam) so it's conditionally obsolete.
    # These two options are now obsolete but retained to process input files
    # created previously.  They are only needed for unconfigDataCollector.jacl
    #   server.pmiservice.initialSpecLevel server.pmiservice.statisticSet

    debug(" ")
    for line in lines:
        if (line.strip() == ""):
            continue
        elif (line.lstrip()[0] == "#"):
            #logDebug("comment: %s" % line)
            continue
        else:
            line = line.strip()
            tokens = line.split("=" )
            pname = tokens[0]
            #logDebug("pname=" + pname)
            debug("property_line=%s" % line)
            if ( pname == "product" ):
                product = tokens[1]
            elif ( re.compile("server\.genericJvmArguments[0-9]*").match(pname) ):
                if (len(line.split('=', 1)) == 2):
                    tmpargs = line.split('=', 1)[1].strip()
                    tmpargs = " %s" % (tmpargs.strip())
                    tokens = tmpargs.split(" -")
                    for token in tokens:
                        param = token.strip()
                        if len(param) == 0: continue
                        if not token.startswith('-'): param = "-%s" % param 
                        genericargs.append(param)
                        debug("tmpargs=%s" % param)
                    if (tmpargs.find("-Djava.security.policy") == 0):
                        JVMSecurityPolicy_flag = 1
                else:
                    printError(pname, line)
            elif ( re.compile("server\.systemproperty[0-9]*").match(pname) ):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpname = tmpname.strip()
                    tmpvalue = tmpvalue.strip()
                    tmpargs = line.split('=', 1)[1].strip()
                    if (tmpname == "am.appserver"):
                        serverName = tmpvalue
                    if (tmpname == "am.nodename"):
                        nodeName = tmpvalue
                    if (tmpname == "deploymentmgr.rmi.connection"):
                        extraRMI_flag = 1
                    if (tmpname == "appserver.rmi.host"):
                        extraRMI_flag = 1
                    if (tmpname == "am.home" and tmpvalue != ""):
                        server_amhome_flag = 1
                    if (tmpname == "ITCAM_DC_ENABLED"):
                        ecam_flag = 1
                    systemproperties.append([["name", tmpname], ["value", tmpvalue]])
                    genericargs.append("-D" + tmpargs.strip())
                else:
                    printError(pname, line)
            elif ( re.compile("server\.environment[0-9]*").match(pname) ):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpvalue = tmpvalue.strip()
                    debug("environment %s=%s" % (tmpname, tmpvalue))
                    environments.append([["name", tmpname], ["value", tmpvalue]])
                else:
                    printError(pname, line)
            elif ( re.compile("server\.variable[0-9]*").match(pname) ):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpvalue = tmpvalue.strip()
                    #variables.append([["symbolicName", tmpname], ["value", tmpvalue]])
                    varvalue = tmpvalue
                    environments.append([["symbolicName", tmpname], ["value", tmpvalue]])
                    if (tmpname == "ITCAMDCHOME"):
                        itcamdc_home = tmpvalue
                        varvalue = utils.replaceBackSlash(itcamdc_home)
                    elif (tmpname != "AM_HOME" and tmpname != "MS_AM_HOME" and tmpname != "ITCAMDCVERSION"):
                        printError(tmpname, line)
                    variables_dic["${" + tmpname + "}"] = varvalue
                else:
                    printError(pname, line)
            elif ( re.compile("control\.systemproperty[0-9]*").match(pname) ):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpvalue = tmpvalue.strip()
                    if (tmpname == "ITCAM_DC_ENABLED" and tmpvalue != ""): ecam_flag = 1
                    control_properties.append([["name", tmpname], ["value", tmpvalue]])
                else:
                    printError(pname, line)
            elif ( re.compile("control\.environment[0-9]*").match(pname)):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpvalue = tmpvalue.strip()
                    control_environments.append([["name", tmpname], ["value", tmpvalue]])
                else:
                    printError(pname, line)
            elif ( re.compile("control\.classpath").match(pname) ):
                control_classpath = tokens[1]
            elif ( re.compile("server\.customservice[0-9]*.classname").match(pname) ):
                tmpvalue = tokens[1]
                customservice.append(["classname", tmpvalue])
            elif ( re.compile("server\.customservice[0-9]*.displayName").match(pname) ):
                tmpvalue = tokens[1].strip()
                customservice.append(["displayName", tmpvalue])
                customservice.append(["externalConfigURL", tmpvalue])
            elif ( re.compile("server\.customservice[0-9]*.description").match(pname) ):
                tmpvalue = tokens[1].strip()
                customservice.append(["description", tmpvalue])
            elif ( re.compile("server\.customservice[0-9]*.enable").match(pname) ):
                tmpvalue = tokens[1].strip()
                customservice.append(["enable", tmpvalue])
            elif ( re.compile("server\.customservice[0-9]*.classpath").match(pname) ):
                tmpvalue = tokens[1].strip()
                if (tmpvalue == ""): tmpvalue = "null"
                customservice.append(["classpath", tmpvalue])
            elif ( pname == "library.WPSlib" ):
                tmpvalue = tokens[1].strip()
                libs = tmpvalue.split(";" )
                sharedLib = libs[0]
            elif ( pname =="server.platform"):
                platform = tokens[1].strip()
            elif ( pname == "server.smf" ):
                smf = tokens[1]
            elif ( pname == "server.classpath" ):
                server_classpath = tokens[1].strip()
            elif ( pname == "server.bootclasspath" ):
                server_bootclasspath = tokens[1]
            elif ( pname == "server.runtime" ):
                server_runtime = tokens[1].strip()
            elif ( pname == "server.version" ):
                serverVersion = tokens[1].strip()
                debug("version=%s" % serverVersion)
            elif ( pname == "server.id" ):
                serverid = tokens[1].strip()
            elif (pname == "server.feature"):
                itcam_feature_list.append(tokens[1].strip())
            elif(pname == "server.maximumHeapSize"):
                maximumHeapSize = tokens[1]
                debug("server.maximumHeapSize=%s" % maximumHeapSize)
                
            elif ( pname == "server.enablePMI" ):
                if (tokens[1] == "Yes"):
                    if (SCRIPT == "configDataCollector"):
                        pmiService_flag = 1
                elif (tokens[1] == "No"):
                        pmiService_flag = 0
                else:
                    debug(" \n*Error*  Unexpected value for server.enablePMI keyword: %s '%s'" % (line, tokens))
                    errorCode = None
                    raise RuntimeError, "Unexpected value for server.enablePMI keyword\n"+line
            else:
                debug(" \n*Error*  Can't find match for '%s' \n" % pname)
                debug("         Input file has an invalid entry.")
                debug("         Script error must be corrected\n ")
                errorCode = None
                raise RuntimeError, "input.properties invalid parameter: \""+pname+"\""

    debug(" ")
    if (product == ""):
        product = "ITCAM"
        pmiService_flag = 1


    ##################### Do some basic validation #####################

    debug("product=%s" % product)
    if (product != "ITCAM" and product != "eCAM"):
        debug(" \n*Error*  Invalid product specified, must be either ITCAM or eCAM\n ")
        errorCode = None
        raise RuntimeError, "Invalid product specified"


    debug("serverid=%s" % serverid)
    serverName = wasserver
    variables_dic["${WAS_SERVER_NAME}"] = wasserver

    if (itcamdc_home == ""):
        errorCode = None
        raise RuntimeError, "product location is missing from input.properties file"

    if (platform == ""):
        debug(" \n*Error*  Platform is missing from input.properties file\n ")
        errorCode = None
        raise RuntimeError, "Product platform is missing from input.properties file"


    if (serverVersion == ""):
        #errorCode = None
        raise RuntimeError, "server version is missing from input.properties file"


    ####################################################################
    # The following checks are only necessary for configDataCollector  #
    ####################################################################
    failed_flag = 0

    if (len(genericargs) == 0):
        debug("*Error*  Can't get jvm generic arguments")
        failed_flag = 1

    if (systemproperties == ""):
        debug("*Error*  Can't get system properties")
        failed_flag = 1

    if (environments == ""):
        debug("*Error*  Can't get environments")
        failed_flag = 1

    if (failed_flag):
        debug(" ")
        debug("           Input file has an invalid entry.")
        debug("           Script error must be corrected")
        debug(" ")
        errorCode = None
        raise RuntimeError, "Processing error, unable to complete configuration"

  

    #########################################################################
    # Validation step is complete                                           #
    #########################################################################

    
    runtime_path = server_runtime
    runtime_path.replace("${ITCAM61HOME}", "${ITCAMDCHOME}")
    runtime_path.replace("${ITCAMDCHOME}", itcamdc_home)
    debug("Resolved runtime path: %s" % runtime_path)

    ########################
    # End of matching code #
    ########################

    ################# config data collector #################

    debug("Start setting attributes for %s" % serverName)

    # Environment variables
    if ( environments != "" ):
        debug("Configuring Environment Variables")
        newEnvList = configEnvs(environments, variables_dic)
                                  
    # JVM
    if (len(genericargs) >  0):
        debug("Configuring JVM Args")
        genericJvmArgs = configJvmOptions(genericargs, variables_dic)

    #read server.xml and see if we need to add in features
    featurever = pdl["ITCAMFEATUREVERSION"][0]
    config_home = pdl["CONFIGHOME"][0]
    runtime_dir = config_home + os.sep + "runtime"
    feature_location = config_home + os.sep + "itcamdc" + os.sep + "etc"
    #jar_file_prefix = "com.ibm.tivoli.itcam.liberty.classloader." + featurever + "feature"
    jar_file_prefix = "com.ibm.tivoli.itcam.liberty.classloader." 
    jar_file_suffix = ".jar"
    mf_file_prefix = "itcam"
    mf_file_suffix = ".mf"
    updated_server_xml = None
    #debug("wlpuserdir=%s type=%s profile_home=%s" % (pdl["WLPUSRDIR"][0], type(pdl["WLPUSRDIR"][0]), pdl["PROFILEHOME"][0]))
    feature_destination = wlpUsrDirPath + os.sep + "extension"
    #jar_file_destination = ""
    #getting jar file name
    jar_file_list = get_matching_files(feature_location, jar_file_prefix, jar_file_suffix)
    mf_file_list = get_matching_files(feature_location, mf_file_prefix, mf_file_suffix)
    if len(jar_file_list) == 0 or len(mf_file_list) == 0:
        jar_file = None
        mf_file = None
        errmsg = "One or both files (%s, %s) are missing from %s" % (feature_location, jar_file_prefix, mf_file_prefix)
        debug("required file not found %s" % errmsg)
        raise RuntimeError, errmsg
    try: 
        i = 0
        feature_path = feature_destination + os.sep + "lib" + os.sep + "features"
        if (not os.path.exists(feature_path)):
            try:
                os.makedirs(feature_path)
            except OSError:
                if not os.path.isdir(feature_path):
                    raise RuntimeError, "Unable to create directory '%s'" % (feature_path) 
        for jar_file in jar_file_list:
            if jar_file != None and mf_file_list[i] != None:    #copy jar and mf files under runtime/liberty
                debug("files for ITCAM feature: %s %s" % (jar_file, mf_file_list[i]))
                dest = feature_destination + os.sep + "lib"
                if not os.path.isfile(dest + os.sep + jar_file):  
                    src = feature_location + os.sep + jar_file
                    cmd="cp -rf " + src + " " + dest + " "
                    debug("The cmd is: %s %s" % (cmd,cmd))
                    if(platformUtil.isWindows()):
                        debug("detected as windows platform to copy file")
                        shutil.copy(src, dest)
                    else:
                        debug("detected as *nix platform to copy file")
                        os.system(cmd)#shutil.copy(src, dest)  
                dest = feature_destination + os.sep + "lib" + os.sep + "features"
                if not os.path.isfile(dest + os.sep + mf_file_list[i]):
                    src = feature_location + os.sep + mf_file_list[i]
                    cmd="cp -rf " + src + " " + dest+ " "
                    debug("The cmd is: %s %s" % (cmd,cmd))
                    if(platformUtil.isWindows()):
                        debug("detected as windows platform to copy file")
                        shutil.copy(src, dest)
                    else:
                        debug("detected as *nix platform to copy file")
                        os.system(cmd)#shutil.copy(src, dest)  
                i = i + 1    
        new_feature_list, updated_server_xml = add_itcam_feature(itcam_feature_list)
        if len(new_feature_list) > 0:
            newEnvList.append("AM_NEW_FEATURES=" + wlpConfigUtils.join (new_feature_list, separator=","))
    except Exception, errmsg:
        debug("failed to add enable feature %s" % errmsg)
        raise RuntimeError, errmsg
    
    #save old JVM parameters for unconfig
    if str(config_type) != str(common_constants.MIGRATE):
        if old_jvm_args != None:
            if "AM_OLD_JVM_ARGS" in current_server_env_dic:
                for arg in newEnvList:
                    if arg.startswith("AM_OLD_JVM_ARGS"): #and config_type != str(common_constants.MIGRATE):
                        newEnvList.remove(arg)
                        break
            newEnvList.append("AM_OLD_JVM_ARGS=" + old_jvm_args)
            
    #saving values into server.env, jvm.options and server.xml
    wlpServerEnvFile_bak = None
    wlpJvmOptionsFile_bak = None
    wlpServerXmlFile_bak = None
    try:
        wlpServerEnvFile_bak = wlpConfigUtils.create_backup_file(wlpServerEnvFile)
        wlpJvmOptionsFile_bak = wlpConfigUtils.create_backup_file(wlpJvmOptionsFile)
        wlpServerXmlFile_bak = wlpConfigUtils.create_backup_file(wlpServerXmlFile)
        fileSaved = wlpConfigUtils.writeDataToFile(wlpServerEnvFile, newEnvList)
        if fileSaved:
            fileSaved = wlpConfigUtils.writeDataToFile(wlpJvmOptionsFile, genericJvmArgs)
        if fileSaved and updated_server_xml != None:
            debug("writing data into %s file new xml string=\n%s" % (wlpServerXmlFile, updated_server_xml))
            #fileSaved = wlpConfigUtils.writeDataToFile(wlpServerXmlFile, line_list.append(updated_server_xml))
            fw = codecs.open(wlpServerXmlFile, mode='w', encoding='utf-8')
            fw.write(updated_server_xml)
            fw.close()
        if not fileSaved:
            raise Exception
    except Exception, remerrmsg:
        wlpConfigUtils.restoreFileFromBackup(wlpServerEnvFile_bak, wlpServerEnvFile)
        wlpConfigUtils.restoreFileFromBackup(wlpJvmOptionsFile_bak, wlpJvmOptionsFile)
        wlpConfigUtils.restoreFileFromBackup(wlpServerXmlFile_bak, wlpServerXmlFile)
        raise RuntimeError, "Unable to save configuration into a file: '%s'" % remerrmsg
    finally:
        #delete backup files
        if wlpServerEnvFile_bak!= None and os.path.isfile(wlpServerEnvFile_bak):
            os.remove(wlpServerEnvFile_bak)
        if wlpJvmOptionsFile_bak != None and os.path.isfile(wlpJvmOptionsFile_bak):
            os.remove(wlpJvmOptionsFile_bak)
        if wlpServerXmlFile_bak != None and os.path.isfile(wlpServerXmlFile_bak):
            os.remove(wlpServerXmlFile_bak)
    print "\t%s" % common_constants.SUCCESSFULLY_CONFIGURED_DC_MSG + serverName
    #print "Successfully configured data collector for server "+serverName




def startConfig(pdl, probe_serverindex_list, ctype):

    debug("wlpConfigDataCollector Entry " )
    
    global configCompletem, common_constants
    global newEnvList
    global genericJvmArgs, current_inputproperties
    progress_obj = None
    failed = 0
    
    common_constants = constants()
    
    phase = "config"  
    
    try:
        for serveri in probe_serverindex_list:
            try:
               
                inputPropertyFilePrefix = utils.getServerBasedFilePrefix(pdl, serveri)
                wasserver = utils.getData(pdl, "SERVERS", serveri)
                runtimedir = pdl["CONFIGHOME"][0] + os.sep + "runtime"
                inputPropertyFile = runtimedir + os.sep + inputPropertyFilePrefix + "input.properties"
                debug("inputPropertyFile: " + inputPropertyFile)
                debug("\nCurrent server to be processed %s" % wasserver)
                configure(pdl, serveri, inputPropertyFile, ctype, progress_obj)
                msgl =[]
                msgl.append(wasserver)
                msg = resourcebundle.getmsg("CONFIG_SUCCESSFUL", "Configuration done successfully for {0}", msgl)
                #
                # Remove input.properties.bak if it exists for the server.
                #
                current_inputproperties = inputPropertyFile
                try:
                    current_inputpropertiesbak = current_inputproperties + ".bak"
                    if (os.path.isfile(current_inputpropertiesbak)):
                        os.remove(current_inputpropertiesbak)
                        debug("input.properties.bak deleted: %s" % (current_inputpropertiesbak))
                except Exception, remerrmsg:
                    log.logError("Cannot remove: %s - %s" % (current_inputpropertiesbak, remerrmsg))
                #
                # Set the summary to indicate that the server status.
                #
                summary[wasserver] = common_constants.SERVER_OK
                #
                # Check if probe for the server has been changed, call mbean to notify the
                # server to activate or deactivate the probe.
                #
                msg = resourcebundle.getmsg("RESTARTSERVER", "Application Server ({0}) should be restarted", msgl)
                print("\t%s" % msg)
            except:
                failed = 1
                log.logException("Error occurred while configuring server %s " % (wasserver))
                traceback_report = log.format_traceback()
                #
                # Remove input.properties if it exists for the server, so unconfigurator doesn't consider the server configured
                debug("Removing file %s" % current_inputproperties)
                current_inputproperties = inputPropertyFile
                try:
                    if (os.path.isfile(current_inputproperties)):
                        os.remove(current_inputproperties)
                        debug("input.properties deleted: %s" % (current_inputproperties))
                except Exception, remerrmsg:
                    log.logError("Cannot remove: %s - %s" % (current_inputproperties, remerrmsg))
                
                # Set the summary to indicate that the server has failed.
                #
                summary[wasserver] = common_constants.SERVER_NOT_OK
                #
                # Check the last processed input properties and delete it as the config
                #
                restore_input_properties()
               
        #
        # Set the summary so it will be shown as progress to the console.
        #
        msg = resourcebundle.getmsg("SUMMARY", "Summary:")
        print "\t%s" % (msg)
        for server_name in summary.keys():
            msgl =[]
            msgl.append(server_name)
            server_stat = summary[server_name]
            debug("Summary key: %s  status: %s" % (server_name, server_stat))
            if (server_stat == common_constants.SERVER_OK):
                msg = resourcebundle.getmsg("SERVER_OK", "{0} (OK)", msgl)
                print("\t\t%s" % msg)
            elif (server_stat == common_constants.SERVER_OK_MBEAN):
                msg = resourcebundle.getmsg("SERVER_OK", "{0} (OK)", msgl)
                print("\t\t%s" % msg)
            elif (server_stat == common_constants.SERVER_OK_NOMBEAN):
                msg = resourcebundle.getmsg("SERVER_OK", "{0} (OK)", msgl)
                print("\t\t%s" % msg)
            else:
                print("\t\t%s (X)" % (server_name))

        #
        # Close the progress file
        #
        if (progress_obj != None):
            progress_obj.close_file()
            progress_obj = None
    
        #
        # If processing each server caused an error, then a message is issued, and the exception
        # is raised, so the wsadmin gets a non zero return code.
        #
        if (failed == 1):
            log.logDebug("Processing servers had a failure")
            raise Exception("Processing servers had a failure")
        
        restore_input_properties()
        
    # This needs to pick up the sys.exit(1) and exceptions     
    except:
        log.logException(SCRIPTNAME+" ends abnormally\n ")
        traceback_report = log.format_traceback()
         
        #
        # Check the last processed input properties and delete it as the config failed
        #
        
        debug("configuration complete: %s " % (str(configComplete)))
        restore_input_properties()
        #
        # If progress object is present, put a message in progress file to be displayed for the wsadmin caller.
        #
        etype, value, tbck = sys.exc_info()
        raise Exception(value)


##############################################################
#
#  Procedure configEnvs
#  Arguments  new environments, list of system variable
#
##############################################################
#updateEnvironments list of dictionaries
def configEnvs ( updateEnvironments, variables_dic ):
    
        global serverName, itcamdc_home
        global old_am_home
        global wlpInstallRoot, wlpUsrDirPath, old_itcamdc_home
        global wlpServerEnvFile
        global current_server_env_dic
        
        #print "Update env variables in server.env " % str(updateEnvironments)

        newEnvList = []
        debug("%s updating environment entries " % str(len(updateEnvironments)))
        for ue in updateEnvironments:
            debug("item=%s" % ue)
        # Read in the environment from the Java process

        # Create the arrays from the update entries
        updates = {}
        hits = {}
        for ue in updateEnvironments:
            nm = ue[0][1]
            val = ue[1][1]
            for var in variables_dic:
                if var in val:
                    val = val.replace(var, variables_dic[var])
            hits[nm] = 0
            updates[nm] = val
            debug("%s=%s" % (nm, val))

        debug("Reading in name/value pairs from existing environment for server %s" % serverName)
        evar_names_list = []
        evar_values_list = []
        
        debug ("reading environment entries from file %s" % wlpServerEnvFile)
        evar_names_list, evar_values_list = wlpConfigUtils.getEnvVarList(wlpUsrDirPath, serverName, wlpInstallRoot)
        if len(evar_names_list) > 0:    #server.env already exist, will modify
            #current_server_env_dic = env_dic
            if "ITCAMDCHOME" in evar_names_list:
                old_itcamdc_home = utils.replaceBackSlash(evar_values_list[evar_names_list.index("ITCAMDCHOME")])
            for enm in evar_names_list:
                eval = evar_values_list[evar_names_list.index(enm)]
                if not enm.startswith("#"):
                    current_server_env_dic[enm] = eval
                if enm in updates: 
                    val = updates[enm]
                    nm = enm
                    hits[nm] += 1
                    debug("existing variable name = %s" % nm)
                    # Must merge the path updates
                    if (enm == "NLSPATH" or enm == "LIBPATH" or enm == "LD_LIBRARY_PATH" or enm == "PATH" or enm == "SHLIB_PATH"):
                        debug("Merging %s  with existing path list" % enm)
                        paths = wlpConfigUtils.split_path(val)
                        epaths = wlpConfigUtils.split_path(eval)
                        newPaths = paths
                        for epath in epaths:
                            # Drop any existing path that is in the new paths
                            for path in paths:
                                if (path == epath):
                                    debug("%s present in existing path " % (epath))
                                    epath = ""
                                    break
                            if (epath == ""):  continue
                            # Drop any ITCAMDCHOME path, it is assumed to be ours
                            if (epath.find("${ITCAMDCHOME}") != -1 or epath.find(old_itcamdc_home) != -1):
                                debug("Existing %s is an old ITCAM entry and will be dropped" % epath)
                                continue                    
                            debug("Existing %s entry will be retained in %s " % (epath, enm))
                            newPaths.append(epath)
        
                        pathDelimiter = ":"
                        if platformUtil.isWindows():
                            pathDelimiter = ";"
                        newPaths = wlpConfigUtils.join( newPaths, pathDelimiter)
                        newVal = newPaths
                        
                        # Update existing entry with merged paths
                        # (We presume that the new value will never be empty)
                        
                        if (newVal == eval):
                            debug("Environment variable already set: %s=%s " % (enm,eval))
                            newEnvList.append(enm + "=" + eval)
                        else:
                            debug("Updating environment variable %s to merge path %s " % (nm, newVal))
                            newEnvList.append(enm + "=" + newVal)                 
                    else:
                        #val = updates[nm]
                        if enm.startswith("#"):
                            newEnvList.append(enm)
                        else:
                            debug("Keeping existing variable %s with new value %s" % (enm, val))
                            newEnvList.append(enm + "=" + val)
                else:
                    debug("Adding existing environment entry %s=%s" % (enm,eval))
                    if enm.startswith("#"):
                        newEnvList.append(enm)
                    else:
                        debug("Keeping existing variable %s with original value %s" % (enm, eval))
                        newEnvList.append(enm + "=" + eval)
        

        debug("server.env values=%s" % str(newEnvList))

        debug(hits)
        #adding in new entries that had no match in the original server.env
        debug("adding in new entries that had no match in the original server.env")
        for nm in updates:
            if nm not in hits.keys() or hits[nm] == 0: 
                debug("adding %s as %s" % (nm, updates[nm]))
                newEnvList.append(nm + "=" + updates[nm])

        
        #resolve variables, WLP doesn't allow variables in server.env
        debug("itcamdc_home=%s" % itcamdc_home)
        itcamdc_home = utils.replaceBackSlash(itcamdc_home)
        newEnvList_resolved = []
        for prop in newEnvList:
            debug("prop=%s" % prop)
            if not prop.startswith("#"):
                prop = prop.replace("${ITCAMDCHOME}", itcamdc_home)
            newEnvList_resolved.append(prop)
        newEnvList = newEnvList_resolved
        debug(" configEnvs() exit %s" % newEnvList)
        return newEnvList

####
# restore input.properties file from a backup if configuration failed
# remove backup file if configuration was successful
#
####
def restore_input_properties():
    global current_inputproperties
    global config_type
    global configComplete
    
    debug("restore_input_properties entry configComplete=%s configType=%s current_inputproperties=%s" % (str(configComplete), str(config_type), current_inputproperties))
    try:
        if (current_inputproperties != "None" and
           (config_type == str(common_constants.CONFIG) or config_type == str(common_constants.MIGRATE) or config_type == str(common_constants.RECONFIG))):
            current_inputproperties_org = current_inputproperties + "_org"
            if configComplete == 0:  #dataCollector wasn't configured restoring to saved input.properties
                if os.path.exists(current_inputproperties_org):
                    shutil.copy(current_inputproperties_org, current_inputproperties)
                    os.remove(current_inputproperties_org)
                    log.logDebug("restored original input.properties file %s" % (current_inputproperties_org))
                else:
                    os.remove(current_inputproperties)
                    log.logDebug("input.properties file was removed - %s" % (current_inputproperties))
            else:  #configuration complete, failure  after AdminConfig.save()
                if os.path.exists(current_inputproperties_org):
                    os.remove(current_inputproperties_org)
                    log.logDebug("AdminConfig.save() executed, removing stored input.properties file %s" % current_inputproperties_org)
                else:
                    log.logDebug("no backup file found %s" % current_inputproperties_org)
    except Exception, remerrmsg:
        debug("input.properties file could not be restores - %s - %s" % (current_inputproperties, remerrmsg))
    debug("restore_input_properties exit")
    


# config
##############################################################
#
#  Procedure configJVM
#  Arguments: new java command line argument
#
##############################################################

def configJvmOptions (newGenericJvmArguments, variables_dic):
       
        global serverName
        global wlpUsrDirPath
        global old_itcamdc_home
        global wlpJvmOptionsFile
        global current_server_env_dic
        global old_jvm_args
        global wlpInstallRoot
        
        debug("configJVM() entry")


        foundRTT = "false"
        runam = ""
        genericJvmArgs = []
        debug("Modify JVM command line arguments")
        debug ("reading jvm options entries from file %s" % wlpJvmOptionsFile)
        jvmOptionsFile = None
        oldGenericJvmArguments = []
        if not os.path.exists(wlpJvmOptionsFile):
            etc_dir = wlpInstallRoot + os.sep + "etc"
            etc_jvmOptions = etc_dir + os.sep + "jvm.options"
            if os.path.exists(etc_jvmOptions):
                jvmOptionsFile = etc_jvmOptions
        else:
            jvmOptionsFile = wlpJvmOptionsFile
            
        infile = None
        if (jvmOptionsFile != None):
            debug("Reading current jvm parameters from file %s" % jvmOptionsFile)
            try:
                infile = codecs.open(jvmOptionsFile, mode='r', encoding='utf-8')
                lines = ""
                for line in infile.readlines():
                    debug ("line: %s" % line)
                    line = line.rstrip()
                    #if line.startswith("#"): continue
                    if len(line.strip()) == 0: continue
                    oldGenericJvmArguments.append(line)
            finally:
                if infile != None:
                    infile.close();
                    debug("finished reading WLP properties")
                    
                    
        #list of JVM args that were in the file prior to configuration            
        saved_args = []
        if "AM_OLD_JVM_ARGS" in current_server_env_dic:
            saved_args = current_server_env_dic["AM_OLD_JVM_ARGS"].split('#')
            addArgToListDefault = False
        else:       #inital configuration
            addArgToListDefault = True
            
        debug("oldGenericJvmArguments: %s " % oldGenericJvmArguments)
      
        #Check if agentlib:am* is part of oldGenericJvmArguments.
        #If yes, remove it from oldGenericJvmArguments.
        updateOldArgList = []
        
        if len(oldGenericJvmArguments) > 0:
            oldArgList = oldGenericJvmArguments
            for oldArg in oldArgList:
                addArgToList = True
                #if server configured by some other DC before
                if oldArg.startswith("#"):  #keep comments at they are 
                     addArgToList = True
                else:
                    oldArg = oldArg.replace(old_itcamdc_home, "${ITCAMDCHOME}")
                    if oldArg.find("-agentlib:am") == 0:
                        addArgToList = False
                    elif (oldArg.startswith("-Xbootclasspath/a:") or oldArg.startswith("-Xbootclasspath/p:")):
                        addArgToList = "false"
                        index = oldArg.index(":")
                        arg = oldArg[0:index + 1]
                        bootCPList = oldArg[index+1:len(oldArg)]
                        debug("bootCPList: %s" % bootCPList)
                        #Check if any jar files from older version of ITCAM
                        #If yes, remove them from bootclasspath as we do not want to save them in AM_OLD_JVM_ARG.
                        bootCPToAdd = []
                        classpathdelimiter = ":"
                        if platformUtil.isWindows():
                            classpathdelimiter = ";"
                        cpjars = bootCPList.split(classpathdelimiter)
                        #oldArg = ""
                        for cpjar in cpjars:
                            debug("cpjar: %s " % (cpjar))
                            if cpjar.find("${ITCAMDC") == 0:
                                continue
                            else:
                                bootCPToAdd.append(cpjar)
                        if len(bootCPToAdd) > 0:
                            #oldArg = arg + join(bootCPToAdd, os.sep)
                            addArgToList = True
                    else:
                        if oldArg in saved_args:
                            addArgToList = True
                            
                if addArgToList:
                    updateOldArgList.append(oldArg)

        oldGenericJvmArguments = updateOldArgList
        debug("oldGenericJvmArguments after cleaning up: %s" % oldGenericJvmArguments)

        # Save the old generic JVM arguments in a variable, so unconfig can find it
        #am_old_jvm_args["AM_OLD_JVM_ARGS"] = oldGenericJvmArguments
        noCommentsJVMArguments = []
        for arg in oldGenericJvmArguments:
            if not arg.startswith('#') and arg.find("${ITCAMDC") == -1:
                noCommentsJVMArguments.append(arg) 
        old_jvm_args = '#'.join(noCommentsJVMArguments)
        
        # Check if the variable already exists (the user is running config again)
        #oldvarid = getServerVariableID("AM_OLD_JVM_ARGS" )
        #check  server.env entries
        if ("AM_OLD_JVM_ARGS" in current_server_env_dic):
            debug("Variable AM_OLD_JVM_ARGS already exists - left in place")
            debug("Saved generic JVM arguments are: ")
            wlpConfigUtils.displayJvmArgs("", current_server_env_dic["AM_OLD_JVM_ARGS"].split(' '))
            #old_jvm_args = oldGenericJvmArguments
        else:
            debug("will add new old args variable")
            #old_jvm_args = oldGenericJvmArguments
            #configServerVariable(am_old_jvm_args )
            
    
        debug("Current generic JVM arguments are: ")
        wlpConfigUtils.displayJvmArgs("", oldGenericJvmArguments )

        newArgList = newGenericJvmArguments
        oldArgList = oldGenericJvmArguments
        jvmArgList = []
        if (oldGenericJvmArguments != ""):
            debug("New generic JVM arguments are:")
            wlpConfigUtils.displayJvmArgs("", newGenericJvmArguments )
            oldArgList = oldGenericJvmArguments

            # First go through oldGenericJvmArguments, and see if
            # there are replacements in newGenericArgs

            for oldArg in oldArgList:
                debug("oldArg=%s" % oldArg)
        
                # The -agentlib in new list replaces the -agentlib in the old list
                if (oldArg.find("-agentlib:am") == 0):
                    agentlib = wlpConfigUtils.getArg("-agentlib:am", newArgList )
                    debug("Found ITCAM -agentlib:am in old args. Now %s" % agentlib)
                    jvmArgList.append(agentlib)
                    if oldArg in noCommentsJVMArguments: noCommentsJVMArguments.remove(oldArg)
                    
                # Merge paths for -Xbootclasspath/a: and -Xbootclasspath/p:
                elif (oldArg.startswith("-Xbootclasspath/a:") or oldArg.startswith("-Xbootclasspath/p:")):
                    arg = oldArg.split(':')[0] + ':'
                    oldBootClasspath = oldArg[len(arg):]
                    newBootClasspath = wlpConfigUtils.getArg(arg, newArgList)[len(arg):]
                    debug("oldBootClasspath=" + oldBootClasspath)
                    bootClasspath = wlpConfigUtils.mergePaths(oldBootClasspath, newBootClasspath )
                    debug("Merged boot class %s " % bootClasspath)
                    if len(bootClasspath.strip()) > 0: jvmArgList.append(arg + bootClasspath)
                    if oldArg in noCommentsJVMArguments: noCommentsJVMArguments.remove(oldArg)
                    bootClasspathList = wlpConfigUtils.unmergePaths (oldBootClasspath, newBootClasspath )
                    bootClasspath = wlpConfigUtils.join( bootClasspathList, ":")
                    debug("Unmerged boot class %s " % bootClasspath)
                    if len(bootClasspath.strip()) > 0: noCommentsJVMArguments.append(arg + bootClasspath)
                    
                # Any properties in old list will be dropped if they are also in new list
                elif (oldArg.startswith("-D") and len(oldArg.split('=')) == 2):
                    prop = oldArg.split('=')[0]
                    arg = wlpConfigUtils.getArg(prop+"=", newArgList )
                    if (arg == ""):
                        jvmArgList.append(oldArg)
                    else:
                        if oldArg in noCommentsJVMArguments: noCommentsJVMArguments.remove(oldArg)
                        debug("Property %s %s updated to" % (prop, arg))


                # Any -verbose from old list will be used if not found in new list
                elif (oldArg.find("-verbose") == 0):
                    for arg in newArgList:
                        if (arg== oldArg):
                            oldArg = ""
                            break
                    if (oldArg== ""):
                        debug("Duplicate item %s dropped" % arg)
                        if oldArg in noCommentsJVMArguments: noCommentsJVMArguments.remove(oldArg)
                    else:
                        jvmArgList.append(oldArg)


                
                # Drop duplicate entry -Xverbosegclog: 
                elif oldArg.find("-Xverbosegclog:") == 0:
                     debug("Dropping conflicting entry %s" % oldArg)
                #    if oldArg in noCommentsJVMArguments: noCommentsJVMArguments.remove(oldArg)
                # Anything else will be passed through
                else:
                    debug("Pass through: %s" % oldArg)
                    jvmArgList.append(oldArg)


            debug("JVM arg list updates: ")
            #jvmArgList = join( jvmArgList)
            wlpConfigUtils.displayJvmArgs("", jvmArgList)

            # Now go through the newArglist, see if there are new ones to go in.

            debug("Examining new Jvm arg list")
            for newArg in newArgList:

                # if a bootpath was merged, do not add the one in the new list
                if (newArg.startswith("-Xbootclasspath/a:") or newArg.startswith("-Xbootclasspath/p:")):
                    #TODO} elseif { [regexp -- {(-Xbootclasspath/[ap]:).*} $newArg x prop] } {
                    prop = newArg.split(':')[0] + ':'
                    arg = wlpConfigUtils.getArg(prop, jvmArgList )
                    if (arg == ""):
                        jvmArgList.append(newArg)


                # Only one -verbose:gc in the final list
                elif (newArg.find("-verbosegc") == 0):
                    arg = wlpConfigUtils.getArg("-verbosegc", jvmArgList )
                    debug("wlpConfigUtils.getArg -verbosegc found %s " % arg)
                    if (arg == ""):
                        jvmArgList.append(newArg)
                
                # drop if no file specified
                elif (newArg == "-Xverbosegclog:None"):
                    continue

                # Add all new elements that are not already present in the final list
                else:
                    for arg in jvmArgList:
                        if (arg == newArg):
                            debug("newArg %s already present" % newArg )
                            newArg = ""
                            break


                    if (newArg != ""):
                        debug("Adding newArg %s" % newArg)
                        jvmArgList.append(newArg)



            genericJvmArgs = jvmArgList
        else:
            # No oldArgs
            genericJvmArgs = newGenericJvmArguments


        # If AM_CONFIG_JVM_ARGS exists, we have been configured before.
        # In that case, determine if any of the current parameters are extras
        # defined after configuration, and remove those from the parameters
        # from the value to be stored in AM_CONFIG_JVM_ARGS.  This is so that
        # unconfig will recognize those extras as new parameters.

        configJvmArgs = genericJvmArgs
        
        '''
        configvarid = "AM_OLD_JVM_ARGS"
        if configvarid in current_server_env_dic:
            debug(" Will replace existing AM_OLD_JVM_ARGS")
            # Find any options in the current JVM options which aren't in AM_CONFIG_JVM_ARGS.
            # These will be extras defined since the last configuration.
    
            configValue = current_server_env_dic[configvarid]
            configValueList = configValue.split()
            extraValue = wlpConfigUtils.diffJvmArgs(oldGenericJvmArguments, configValueList)
            if (len(extraValue) > 0):
                logDebug("Added since last configuration: %s" % extraValue)
                # Remove the extra JVM options from the AM_CONFIG_JVM_ARGS options.
                # This will cause unconfig to recognize them as defined since configuration.
                configJvmArgs = wlpConfigUtils.diffJvmArgs(configJvmArgs, extraValue )
        else:
            logDebug("No previous AM_OLD_JVM_ARGS")
        '''    
        
        wlpConfigUtils.displayJvmArgs("Final merged generic JVM arguments ", genericJvmArgs )

        # Save the configured generic JVM arguments in a variable, so unconfig can identify changes
        #am_config_jvm_args = [["symbolicName", "AM_CONFIG_JVM_ARGS"], ["value", configJvmArgs]]
        #configServerVariable(am_config_jvm_args )

        # resolve variables, since WLP doesn't allow variables in jvm.options
        debug("itcamdc_home=%s" % itcamdc_home)
        genericJvmArgs_resolved = []
        for option in genericJvmArgs:
            debug("option=%s" % option)
            for var in variables_dic:
                debug ("var=%s" % var)
                if var in option:
                    debug("substituting variable for value %s->%s" % (var, variables_dic[var]))
                    option = option.replace(var, variables_dic[var])
            genericJvmArgs_resolved.append(option)
        genericJvmArgs = genericJvmArgs_resolved
        old_jvm_args = '#'.join(noCommentsJVMArguments)
        debug("configJVM() exit")
        return genericJvmArgs
    
def add_itcam_feature(itcam_feature_list):
    '''
       Update server.xml with itcam feature
    '''
    
    global serverName
    global wlpServerXmlFile
    debug("add_itcam_feature entry %s" % serverName)
    
    feature_list = []
    
    server_xml_file = wlpServerXmlFile
    
    try:

        debug("Getting current list of features from server.xml" )
       
        server_xml = java.io.File(server_xml_file)
        server_xml_found = True
        debug("looking for server xml file %s" % server_xml_file)
        if not server_xml.exists():        #something is absolutely wrong
            debug ("server.xml not found under %s" % server_xml_file)
            server_xml_found = False

        if not server_xml_found:
            #pmsg = getmsg("CFG2041E", "Profile registry file not found  {0}", [server_xml_file])
            raise ConfigurationException("Server xml file %s not found" % wlpServerXmlFile)
        doc= minidom.parse(server_xml_file)
        server = doc.documentElement
        featureManager = server.getElementsByTagName("featureManager")
        #feature dictionary, key=feature name, value=feature version
        #stripe the prefix usr: or itcam:
        feature_dic = {}  #feature format usr:itcam-73.xxx or itcam:itcam-73.xxx
        addon_feature_dic = {}
        addon_feature_list = itcam_feature_list #["itcam:itcam-72.004", "monitor-1.0"]
        xml_modified = False
        xml_string = None
        new_itcam_features = []
        
        
        for addon_feature in addon_feature_list:
            feature_name, feature_version = wlpConfigUtils.get_feature_name_version(addon_feature)
            addon_feature_dic[feature_name] = feature_version
            
        if featureManager.length == 0:
            raise Exception, "Invalid server.xml, expecting to find featureManager node"
        
        for featureManagerNode in featureManager:
            feature_nodes = featureManagerNode.getElementsByTagName("feature")
            for feature in feature_nodes:
                current_feature = wlpConfigUtils.getNodeText(feature.childNodes).strip()
                cfeature_name, cfeature_version = wlpConfigUtils.get_feature_name_version(current_feature)
                feature_list.append(cfeature_name)
                if cfeature_name in addon_feature_dic.keys():
                    #compare versions, if addon feature has newer version, remove old the feature
                    afeature_version = addon_feature_dic[cfeature_name]
                    if afeature_version > cfeature_version:
                        debug("Found newer version=%s for feature=%s" % (afeature_version, feature_name))
                        feature.parentNode.removeChild(feature)
                        xml_modified = True
                        debug("Removing old feature %s" % current_feature)
                        feature_list.pop()
                        
        debug("current features %s" % feature_list)
        for addon_feature in addon_feature_list:
            afeature_name, afeature_version = wlpConfigUtils.get_feature_name_version(addon_feature)
            if afeature_name not in feature_list:
                debug("adding new feature %s" % addon_feature)
                xml_modified = True
                new_itcam_features.append(addon_feature);
                itcam_feature = doc.createElement("feature")
                featureManager[0].appendChild(itcam_feature)
                text = doc.createTextNode(addon_feature)
                itcam_feature.appendChild(text)
                
        if xml_modified:
            xml_string = doc.toxml()        
    except Exception, errmsg:
        log.logException("Error occurred while reading server.xml - %s" % (errmsg))
        raise 
    debug("add_itcam_feature exit, features defined: %s" % feature_list)
    return new_itcam_features, xml_string
        
       
    
       
    
def printError(pname, line):
    debug(" \n*Error*  Invalid format, entry ignored")
    debug("         Input file has an invalid entry.")
    debug("         Script error must be corrected\n ")
    errorCode = None
    raise RuntimeError, "input.properties invalid parameter '%s' in line '%s'" % (pname, line)

def get_matching_files(dir, file_prefix, file_suffix=None):
    '''    
    get files matching prefix
    '''
    import glob
    file_list = []
    try:
        os.chdir(dir)
        for f in os.listdir('.'):
            if f.startswith(file_prefix):
                if file_suffix == None:
                    file_list.append(f) 
                else:
                    if f.endswith(file_suffix):
                        file_list.append(f)
    except Exception, errmsg:
        debug("error %s" % errmsg)
    debug("found files %s" % file_list)
    return file_list  