#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################

'''
    Get list of defined WLP servers
'''

fixlev = "40544" # Last APAR, defect or story to update this module


import os
import sys
import codecs
import socket

import utils
import platformUtil
from resourcebundle import getmsg as getmsg
from debugUtil import logDebug as debug
from constants import constants
import debugUtil
from xml.dom import minidom

from java.lang import System

import wlpConfigUtils
from ConfigurationException import ConfigurationException


VERSIONSTRING = "ITCAM 7.3 HAAD730 AK00000"
SCRIPT = "wlpFindServers"
SCRIPTNAME = SCRIPT+".py"

fileh = None

##locate servers for liberty profile
def findlibertservers(pdl):
    '''
        Locate WLP servers, write results into findservers.dat file
    '''
    global fileh

    debug("findlibertservers Entry")

    initdatFile(pdl)
    usrDirPath = pdl["WLPUSRDIR"][0]
    wlpInstallDir = pdl["WLPINSTALLDIR"][0]
    serversDirPath = usrDirPath + os.sep + "servers"
    cellname = wlpConfigUtils.getCellName()
    nodename = wlpConfigUtils.getNodeName(wlpInstallDir)
    #nodename = ".".join(wlpInstallDir.split(os.sep)[-2:])
    standalone = "true"
    jvmmode = System.getProperty("com.ibm.vm.bitmode") or  System.getProperty("sun.arch.data.model") or "None"
    shareclassesmodified = "false"
    maxHeapSize = "None"
    verbosegclog = "None"
    isConfigured = "None"
    java_version = "None"
    itcam_dcversion = "None"
    serverversion = getLibertyVersion(wlpInstallDir)

    write_to_findservers_dat("#was.cell=%s" % (cellname))
    write_to_findservers_dat("#was.node=%s" % (nodename))
    write_to_findservers_dat("#was.install.root=%s" % wlpInstallDir)
    write_to_findservers_dat("#user.install.root=%s" % usrDirPath)
    write_to_findservers_dat("#was.plugins.root=%s" % "")
    write_to_findservers_dat("#was.config.root=%s" % "")
    write_to_findservers_dat("#was.props.dir=%s" % wlpInstallDir)
    write_to_findservers_dat("#was.log.root=%s" % wlpInstallDir)
    write_to_findservers_dat("#java.home=%s" % wlpInstallDir)
    write_to_findservers_dat("#wps.home=%s" % (""))
    write_to_findservers_dat("#wbi.install.root=%s" % (""))
    write_to_findservers_dat("#wlp.install.root=%s" % wlpInstallDir)
    #write_to_findservers_dat("#wlp.user.dir=%s" % usrDirPath)

    if os.path.isdir(serversDirPath):
        dirList = os.listdir(serversDirPath)
        for entry in dirList:
            debug("Getting liberty servers from %s" % serversDirPath)
            server_xml_file = serversDirPath + os.sep + entry + os.sep + "server.xml"
            if os.path.exists(server_xml_file) and isValidServerXml(server_xml_file):
                isConfigured = "None"
                subProduct = "None"
                itcam_dcversion = "None"
                serverDir = serversDirPath + os.sep + entry
                debug("found WLP server servername=%s" % entry)
                servername = entry
                var_names_list, var_values_list = wlpConfigUtils.getEnvVarList(usrDirPath, servername)
                if "ITCAMDCHOME" in var_names_list:
                    isConfigured = "DC"
                if "ITCAMDCVERSION" in var_names_list:
                    itcam_dcversion = str(var_values_list[var_names_list.index("ITCAMDCVERSION")])

                # Positional data for each server:
                #
                #   0. Cell name
                #   1. Node name
                #   2. Server name
                #   3. Standalone or Distributed
                #   4. JVM bitness
                #   5. Shared classes
                #   6. Server version
                #   7. Maximum heap size
                #   8. Verbose GC log
                #   9. ITCAM DC configured
                #  10. Java version
                #  11. ITCAM DC version
                #  12. Portal or other subproduct
                #  13. Server ID (Server name for Liberty) - Always last

                write_to_findservers_dat("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s" % (cellname,
                                                                                                     nodename,
                                                                                                     servername,
                                                                                                     standalone,
                                                                                                     jvmmode,
                                                                                                     shareclassesmodified,
                                                                                                     serverversion,
                                                                                                     maxHeapSize,
                                                                                                     verbosegclog,
                                                                                                     isConfigured,
                                                                                                     java_version,
                                                                                                     itcam_dcversion,
                                                                                                     subProduct,
                                                                                                     servername))

    fileh.close()
    fileh = None
    dchome = pdl["CONFIGHOME"][0]
    dat_file = dchome + os.sep + "data" + os.sep + "findservers.dat"
    debug("returning from findlibertservers %s" % dat_file)
    return dat_file

def isValidServerXml(server_xml_file):
    '''
        sanity check to validate the we have real xml file
    '''
    debug("isValidServerXml entry %s" % server_xml_file)
    isValid = True
    try:
        doc = minidom.parse(server_xml_file)
        if doc is None: isValid = False
        #server = doc.documentElement
    except:
        isValid = False
        file_size = os.path.getsize(server_xml_file)
        debug("Server xml file %s is not valid, file size=%s" % (server_xml_file, file_size))
    debug("isValidServerXml exit %s" % isValid)
    return isValid


def initdatFile(pdl):
    '''
        Initialize findservers.dat
    '''
    global fileh

    debug("initdatFile entry")
    datadir = pdl["CONFIGHOME"][0] + os.sep + "data"
    dat_filename = "findservers.dat"
    debug("dat file location is %s" % (dat_filename))
    #
    # Rename any existing dat files.
    #
    dat_fileloc = datadir + os.sep + dat_filename
    if dat_filename is not None:
        utils.rename_archival_file(dat_fileloc)
    if dat_fileloc != "":
        #print "Input files " dat_file
        try:
            fileh = codecs.open(dat_fileloc, mode="w", encoding="utf-8")
            debug ("fileh = %s" % fileh)
            write_to_findservers_dat("#findServers.version=7.3")
        except Exception, errmsg:
            debugUtil.logException("Error occurred while executing wlpFindserver - %s" % (errmsg))
            debugUtil.logException("Unexpected Error: %s . %s ended" % (str(sys.exc_info()[0]), SCRIPTNAME))
            raise Exception, errmsg
    else:
        errorCode = None
        raise RuntimeError, "Input file not supplied. wlpFindservers abandoned"


def write_to_findservers_dat(record):
    '''
       Write a record into findServers.dat

       @param record - Record to be written
    '''
    global fileh

    debug("Record to be written to: %s" % (record))
    fileh.write(record)
    fileh.write(os.linesep)
    fileh.flush()


def getLibertyVersion(wlpInstallDir):
    '''
        get version of Liberty server
    '''
    debug("getting WLP version")
    versionDirPath = wlpInstallDir + os.sep + "lib" + os.sep + "versions"
    dirList = os.listdir(versionDirPath)
    version = None
    infile = None
    for versionfile in dirList:
        if versionfile.lower() == "websphereapplicationserver.properties":
            debug("opening file %s" % versionDirPath + os.sep + versionfile)
            try:
                infile = codecs.open(versionDirPath + os.sep + versionfile, mode="r", encoding="utf-8")
                lines = ""
                for line in infile.readlines():
                    debug ("line: %s" % line)
                    if line.startswith("com.ibm.websphere.productVersion"):
                        version = line.split("=")[1].strip()
                        break
            finally:
                if infile != None:
                    infile.close();
                    debug("finished reading WLP properties")
            break
    debug ("WLP version %s" % version)
    return version
