#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012, 2014 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
'''
    wlpUnconfigDC - unconfigures WAS Liberty Profile
'''

import sys
import re
import os
import time
import codecs
import traceback
import utils

from time import strftime
from java.lang import System

import wlpConfigUtils
import platformUtil
import debugUtil as log
import resourcebundle as resourcebundle
from constants import constants
from debugUtil import logDebug as debug
from xml.dom import minidom

##Global variables
VERSIONSTRING = "ITCAM 7.2 AK0000"
SCRIPT = "wlpUnconfigDC"
SCRIPTNAME = SCRIPT + ".py"

nodeName = ""
serverName = ""
serverid = ""

# This is set after parameters are validated
runtime_path = ""
customservice = []
product = ""
control_classpath = ""
genericargs = []
sharedLib = ""
serverVersion = ""
server_runtime = ""
itcamdc_home = ""
server_classpath = ""
server_bootclasspath = ""
smf = "No"
containertimeouts = ""
server_amhome_flag = 0
ecam_flag = 0
control_environments = []
control_properties = []
environments = []
pmiService_flag = 0
extraRMI_flag = 0
old_itcamdc_home = "${ITCAMDCHOME}"
failed_flag = 0
wasserver = "None"
summary = {}
current_inputproperties = "None"
old_jvm_args = None
wlpServerEnvFile = ""
wlpJvmOptionsFile = ""
wlpServerXmlFile = ""
updated_server_xml = None
genericJvmArgs = []
itcam_feature_list = []
configured_wlp_servers = {}
wlp_home_list = []

###End of global variables####

def printError(pname, line):
    debug(" \n*Error*  Invalid format, entry ignored")
    debug("         Input file has an invalid entry.")
    debug("         Script error must be corrected\n ")
    errorCode = None
    raise RuntimeError, "input.properties invalid parameter '%s' in line '%s'" % (pname, line)

def unconfigure (pdl, serveri, inputPropertyFile):
    '''
        UnConfigure
        @parm  input_record - Record form the input list file.
        @param progress - progress object reference
    '''
    global containertimeouts
    global control_environments
    global control_properties
    global customservice
    global environments
    global genericargs
    global itcamdc_home
    global nodeName
    global platform
    global product
    global runtime_path
    global serverName
    global serverVariableEntries
    global serverVersion
    global server_classpath
    global server_runtime
    global systemproperties
    global variables
    global current_inputproperties
    global common_constants
    global wlpInstallRoot
    global wlpUsrDirPath
    global old_itcamdc_home
    global newEnvList
    global genericJvmArgs
    global server_bootclasspath
    global wlpServerEnvFile 
    global wlpJvmOptionsFile
    global wlpServerXmlFile
    global failed_flag
    global old_jvm_args
    global updated_server_xml
    global itcam_feature_list
    
    product = ""
    platform = ""
    serverid = ""
    customservice = []
    systemproperties = []
    environments = []
    variables = []

    control_classpath = ""
    genericargs = []
    sharedLib = ""
    serverVersion = ""
    server_runtime = ""

    itcamdc_home = ""
    server_classpath = ""
    server_bootclasspath = ""
    smf = "No"
    containertimeouts = ""
   
    control_environments = []
    control_properties = []

    pmiService_flag = 0
    extraRMI_flag = 0

    failed_flag = 0
    variables_dic = {}
    old_jvm_args = None
    
    inputfile = inputPropertyFile
    current_inputproperties = inputfile
    genericJvmArgs = []
    
    wlpInstallRoot = utils.getData(pdl, "WLPHOME", serveri)
    wlpUsrDirPath = utils.getData(pdl, "USERINSTALLROOT", serveri)
    
    wasserver = utils.getData(pdl,"SERVERS", serveri)
    
    #Liberty configuration files for the server
    wlpServerRoot = wlpUsrDirPath + os.sep + "servers" + os.sep + wasserver
    wlpServerEnvFile = wlpServerRoot + os.sep + "server.env"
    wlpJvmOptionsFile = wlpServerRoot + os.sep + "jvm.options"
    wlpServerXmlFile = wlpServerRoot + os.sep + "server.xml"
    
    #
    # Write the start message for the console in the progress file.
    #

    # Read in the response file
    lines = []
    inDesc = None
    try:
        inDesc = codecs.open(inputfile, mode='r', encoding='utf-8')
        #inDesc = open( inputfile, "r" )
        while 1:
            buffer = inDesc.readline()
            if not buffer: break
            lines.append(buffer)
    except:
        errorCode = None
        raise RuntimeError, inputfile+" failed to open\n"
    finally:
        if inDesc != None:
            inDesc.close()

    debug("%d lines read from %s" % (len(lines),inputfile))
    common_constants = constants()

    # Note: Any value which may contain equal signs must
    #       be handled with regexp instead of lindex.
    # Comments are not allowed with a switch (probably JACL bug)
    # Extension MBean Provider is obsolete
    #   server.xMBeanProvider
    # These parameters, while still valid, should be replaced by server.variable specifications
    #   server.itcamdchome server.itcam61home
    # These two parameters are obsolete so they are ignored
    #   server.am.home server.ms.am.home
    # Note:server.systemproperty=am.home is used in ITCAM 7.2(ecam) so it's conditionally obsolete.
    # These two options are now obsolete but retained to process input files
    # created previously.  They are only needed for unconfigDataCollector.jacl
    #   server.pmiservice.initialSpecLevel server.pmiservice.statisticSet

    for line in lines:
        if (line.strip() == ""):
            debug("blank line")
            continue
        elif (line.lstrip()[0] == "#"):
            debug("comment: %s" % line)
            continue
        else:
            line = line.strip()
            tokens = line.split("=" )
            pname = tokens[0]
            #logDebug("pname=" + pname)
            if ( pname == "product" ):
                product = tokens[1]
            elif ( re.compile("server\.genericJvmArguments[0-9]*").match(pname) ):
                if (len(line.split('=', 1)) == 2):
                    tmpargs = line.split('=', 1)[1].strip()
                    tmpargs = " %s" % (tmpargs.strip())
                    tokens = tmpargs.split(" -")
                    for token in tokens:
                        param = token.strip()
                        if len(param) == 0: continue
                        if not token.startswith('-'): param = "-%s" % param 
                        genericargs.append(param)
                        debug("tmpargs=%s" % param)
                        if token.find("am.wasnode") > -1:
                            nodeName = token.split('=')[1].strip()
                            debug("nodeName=%s" % nodeName)
                        elif (token.find("-Djava.security.policy") == 0):
                            JVMSecurityPolicy_flag = 1
                else:
                    printError(pname, line)
            elif ( re.compile("server\.systemproperty[0-9]*").match(pname) ):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpname = tmpname.strip()
                    tmpvalue = tmpvalue.strip()
                    tmpargs = line.split('=', 1)[1].strip()
                    if (tmpname == "am.appserver"):
                        serverName = tmpvalue
                    if (tmpname == "am.nodename"):
                        nodeName = tmpvalue
                    if (tmpname == "deploymentmgr.rmi.connection"):
                        extraRMI_flag = 1
                    if (tmpname == "appserver.rmi.host"):
                        extraRMI_flag = 1
                    if (tmpname == "am.home" and tmpvalue != ""):
                        server_amhome_flag = 1
                    if (tmpname == "ITCAM_DC_ENABLED"):
                        ecam_flag = 1
                    systemproperties.append([["name", tmpname], ["value", tmpvalue]])
                    genericargs.append("-D" + tmpargs.strip())
                else:
                    printError(pname, line)
            elif ( re.compile("server\.environment[0-9]*").match(pname) ):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpvalue = tmpvalue.strip()
                    debug("environment %s=%s" % (tmpname, tmpvalue))
                    environments.append([["name", tmpname], ["value", tmpvalue]])
                else:
                    printError(pname, line)
            elif ( re.compile("server\.variable[0-9]*").match(pname) ):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpvalue = tmpvalue.strip()
                    variables.append([["symbolicName", tmpname], ["value", tmpvalue]])
                    environments.append([["symbolicName", tmpname], ["value", tmpvalue]])
                    if (tmpname == "ITCAMDCHOME"):
                        itcamdc_home = tmpvalue
                        varvalue = utils.replaceBackSlash(itcamdc_home)
                    elif (tmpname != "AM_HOME" and tmpname != "MS_AM_HOME" and tmpname != "ITCAMDCVERSION"):
                        printError(tmpname, line)
                    variables_dic["${" + tmpname + "}"] = varvalue
                else:
                    printError(pname, line)
            elif ( re.compile("control\.systemproperty[0-9]*").match(pname) ):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpvalue = tmpvalue.strip()
                    #Per 15907: ITCAM_DC_ENABLED is no longer in use. So comment the following line out.
                    #if (tmpname == "ITCAM_DC_ENABLED" and tmpvalue != ""): ecam_flag = 1
                    control_properties.append([["name", tmpname], ["value", tmpvalue]])
                else:
                    printError(pname, line)
            elif ( re.compile("control\.environment[0-9]*").match(pname)):
                if (len(line.split('=')) == 3):
                    (pname, tmpname, tmpvalue) = line.split('=')
                    tmpvalue = tmpvalue.strip()
                    control_environments.append([["name", tmpname], ["value", tmpvalue]])
                else:
                    printError(pname, line)
            elif ( re.compile("control\.classpath").match(pname) ):
                control_classpath = tokens[1]
            elif ( re.compile("server\.customservice[0-9]*.classname").match(pname) ):
                tmpvalue = tokens[1]
                customservice.append(["classname", tmpvalue])
            elif ( re.compile("server\.customservice[0-9]*.displayName").match(pname) ):
                tmpvalue = tokens[1].strip()
                customservice.append(["displayName", tmpvalue])
                customservice.append(["externalConfigURL", tmpvalue])
            elif ( re.compile("server\.customservice[0-9]*.description").match(pname) ):
                tmpvalue = tokens[1].strip()
                customservice.append(["description", tmpvalue])
            elif ( re.compile("server\.customservice[0-9]*.enable").match(pname) ):
                tmpvalue = tokens[1].strip()
                customservice.append(["enable", tmpvalue])
            elif ( re.compile("server\.customservice[0-9]*.classpath").match(pname) ):
                tmpvalue = tokens[1].strip()
                if (tmpvalue == ""): tmpvalue = "null"
                customservice.append(["classpath", tmpvalue])
            elif ( pname == "library.WPSlib" ):
                tmpvalue = tokens[1].strip()
                libs = tmpvalue.split(";" )
                sharedLib = libs[0]
            elif ( pname =="server.platform"):
                platform = tokens[1].strip()
            elif ( pname == "server.smf" ):
                smf = tokens[1]
            elif ( pname == "server.classpath" ):
                server_classpath = tokens[1].strip()
            elif ( pname == "server.bootclasspath" ):
                server_bootclasspath = tokens[1]
            elif ( pname == "server.runtime" ):
                server_runtime = tokens[1].strip()
            elif ( pname == "server.version" ):
                serverVersion = tokens[1].strip()
                debug("version=%s" % serverVersion)
            elif ( pname == "server.id" ):
                serverid = tokens[1].strip()
            elif ( pname == "server.xMBeanProvider"  ):
                pass
            elif ( re.compile("server\.itcam(61|dc)home").match(pname) ):
                variables.append([["symbolicName", "ITCAMDCHOME"], ["value", tokens[1] ]])
                itcamdc_home = tokens[1]
            elif ( pname == "server.am.home" ):
                #obsolete parameter
                pass
            elif ( pname == "server.ms.am.home" ):
                #obsolete parameter
                pass
            elif ( pname == "server.protocol_iiop_propagate_unknown_service_ctxs" ):
                variables.append([["symbolicName", "protocol_iiop_propagate_unknown_service_ctxs"], ["value", tokens[1] ]])
            elif ( pname == "server.policyedit" ):
                if (tokens[1] == "Yes"):
                    if (SCRIPT == "configDataCollector"):
                        serverPolicy_flag = 1

                elif (tokens[1] == "No"):
                    serverPolicy_flag = 0
                else:
                    debug(" \n*Error*  Unexpected value for server.policyedit keyword: %s" % line )
                    errorCode = None
                    raise RuntimeError, "Unexpected value for server.policyedit keyword\n"+line
            elif ( pname == "server.enablePMI" ):
                if (tokens[1] == "Yes"):
                    if (SCRIPT == "configDataCollector"):
                        pmiService_flag = 1
                elif (tokens[1] == "No"):
                        pmiService_flag = 0
                else:
                    debug(" \n*Error*  Unexpected value for server.enablePMI keyword: %s '%s'" % (line, tokens))
                    errorCode = None
                    raise RuntimeError, "Unexpected value for server.enablePMI keyword\n"+line
            elif ( pname == "server.enableconfigmbean" ):
                pass
            elif ( pname == "server.containertimeouts"  ):
                containertimeouts = tokens[1]
            elif ( pname == "server.pmiservice.initialSpecLevel" ):
                pass
            elif ( pname == "server.pmiservice.statisticSet" ):
                #server_pmiservice_statisticSet = tokens[1]
                pass
            elif(pname == "server.maximumHeapSize"):
                maximumHeapSize = tokens[1]
                debug("server.maximumHeapSize=%s" % maximumHeapSize)
            elif (pname == "server.feature"):
                itcam_feature_list.append(tokens[1].strip())
                
            else:
                debug(" \n*Error*  Can't find match for '%s' " % pname)
                debug("         Input file has an invalid entry.")
                debug("         Script error must be corrected")
                errorCode = None
                raise RuntimeError, "input.properties invalid parameter: \""+pname+"\""

    if (product == ""):
        product = "ITCAM"
        pmiService_flag = 1

    ##################### Do some basic validation #####################

    debug("product=%s" % product)
    if (product != "ITCAM" and product != "eCAM"):
        debug(" \n*Error*  Invalid product specified, must be either ITCAM or eCAM\n ")
        errorCode = None
        raise RuntimeError, "Invalid product specified"

    
    serverName = wasserver
    variables_dic["${WAS_SERVER_NAME}"] = wasserver
    
    debug("server=%s" % serverName)
    if (serverName == ""):
        errorCode = None
        raise RuntimeError, "unable to extract server name from server ID"

    debug("node=%s" % nodeName)
    if (nodeName == ""):
        errorCode = None
        raise RuntimeError, "unable to extract node name from server ID"

    
    debug("product home=%s" % itcamdc_home)
    if (itcamdc_home == ""):
        errorCode = None
        raise RuntimeError, "product location is missing from input.properties file"


    debug("product platform=%s" % platform)
    if (platform == ""):
        debug(" \n*Error*  Platform is missing from input.properties file\n ")
        errorCode = None
        raise RuntimeError, "Product platform is missing from input.properties file"

    if (serverVersion == ""):
        #errorCode = None
        raise RuntimeError, "server version is missing from input.properties file"

    if (failed_flag):
        debug("Input file has an invalid entry.\nScript error must be corrected")
        errorCode = None
        raise RuntimeError, "Processing error, unable to complete configuration"


    #########################################################################
    # Validation step is complete                                           #
    #########################################################################

    
    # Establish runtime path for file access

    runtime_path = server_runtime
    runtime_path.replace("${ITCAM61HOME}", "${ITCAMDCHOME}")
    runtime_path.replace("${ITCAMDCHOME}", itcamdc_home)
    debug("Resolved runtime path: %s" % runtime_path)

   
    ################# unconfig data collector #################

    debug("Start unconfiguring attributes for %s" % serverName)

    #Server feature
    

    # Environment variables
    if ( environments != "" ):
        debug("Unconfiguring Environment Variables")
        unconfigEnvs(environments, variables_dic)

    if (genericargs != ""):
        debug("Unconfiguring JVM generic args")
        unconfigJVM(genericargs, variables_dic)
   
    #saving values into server.env, jvm.options and server.xml
    wlpServerEnvFile_bak = None
    wlpJvmOptionsFile_bak = None
    try:
        wlpServerEnvFile_bak = wlpConfigUtils.create_backup_file(wlpServerEnvFile)
        wlpJvmOptionsFile_bak = wlpConfigUtils.create_backup_file(wlpJvmOptionsFile)
        wlpServerXmlFile_bak = wlpConfigUtils.create_backup_file(wlpServerXmlFile)
        fileSaved = wlpConfigUtils.writeDataToFile(wlpServerEnvFile, newEnvList, True)
        if fileSaved:
            fileSaved = wlpConfigUtils.writeDataToFile(wlpJvmOptionsFile, genericJvmArgs, True)
        if fileSaved and updated_server_xml != None:
            debug("writing data into %s file new xml string=\n%s" % (wlpServerXmlFile, updated_server_xml))
            #fileSaved = wlpConfigUtils.writeDataToFile(wlpServerXmlFile, line_list.append(updated_server_xml))
            fw = codecs.open(wlpServerXmlFile, mode='w', encoding='utf-8')
            fw.write(updated_server_xml)
            fw.close()
        if not fileSaved:
            raise Exception
    except Exception, remerrmsg:
        wlpConfigUtils.restoreFileFromBackup(wlpJvmOptionsFile_bak, wlpServerEnvFile)
        wlpConfigUtils.restoreFileFromBackup(wlpJvmOptionsFile_bak, wlpJvmOptionsFile)
        wlpConfigUtils.restoreFileFromBackup(wlpServerEnvFile_bak, wlpServerXmlFile)
        raise RuntimeError, "Unable to save configuration into a file: '%s'" % remerrmsg
    finally:
        #delete backup files
        if wlpServerEnvFile_bak!= None and os.path.isfile(wlpServerEnvFile_bak):
            os.remove(wlpServerEnvFile_bak)
        if wlpJvmOptionsFile_bak != None and os.path.isfile(wlpJvmOptionsFile_bak):
            os.remove(wlpJvmOptionsFile_bak)
        if wlpServerXmlFile_bak != None and os.path.isfile(wlpServerXmlFile_bak):
            os.remove(wlpServerXmlFile_bak)
    
    
    #returncode = saveChanges( )
    returncode = "true"
    if (returncode != "true"):
        raise RuntimeError, "saveChanges routine failed with RC( %s ) for server %s" % (str(returncode), serverName)

    # remove tt.enabled settings
    ttconfigFileNameSuffix = utils.getServerBasedFilePrefix(pdl, serveri)
    ttconfigFileName = pdl["CONFIGHOME"][0] + os.sep + "runtime" + os.sep + "custom" + os.sep + "ttdc.enabled." + ttconfigFileNameSuffix
    ttconfigFileName = ttconfigFileName[:-1]
    debug("ttenableFileNameSuffix = %s " % ttconfigFileNameSuffix)
    debug("ttenableFileName = %s " % ttconfigFileName)
    
    if os.path.isfile(ttconfigFileName):
        debug(ttconfigFileName + " file exists")
        try:
            os.remove(ttconfigFileName)
            debug(ttconfigFileName + " file removed")
        except Exception, msgVar:
           debug("An error occurred while removing %s file. Error details - %s" % (ttconfigFileName, msgVar))        
    
    debug("Successfully unconfigured data collector for server %s" % serverName)
    #print common_constants.SUCCESSFULLY_UNCONFIGURED_DC_MSG + serverName    



def startUnconfig(pdl, server_indexlist):
    
    global configured_wlp_servers
    
    failed = 0
    
    debug("wlpUnconfigDC running.")
    
    
    # Instantiate the global constants class
    #
    common_constants = constants()
    
    phase = "unconfig"  
    
    try:
            for serveri in server_indexlist:
                inputPropertyFilePrefix = utils.getServerBasedFilePrefix(pdl, serveri)
                wasserver = utils.getData(pdl, "SERVERS", serveri)
                runtimedir = pdl["CONFIGHOME"][0] + os.sep + "runtime"
                inputPropertyFile = runtimedir + os.sep + inputPropertyFilePrefix + "input.properties"
                debug("inputPropertyFile: " + inputPropertyFile)
                try:
                        debug("getting configured servers for WLP home")
                        wlpInstallRoot = utils.getData(pdl, "WLPHOME", serveri)
                        get_configured_servers(wlpInstallRoot, runtimedir)
            
                        debug("Unconfigure using: %s " % wasserver)
                        unconfigure(pdl, serveri, inputPropertyFile)
                        msgl =[]
                        msgl.append(wasserver)
                        msg = resourcebundle.getmsg("UNCONFIG_SUCCESSFUL", "UnConfiguration done successfully for {0}", msgl)
                        print "\t%s" % (msg)
                        
                        current_inputproperties = inputPropertyFile
                        #
                        # Set the summary to indicate the server status.
                        #
                        summary[wasserver] = common_constants.SERVER_OK
                        #
                        # Check the last processed input properties and rename it
                        #
                        if (current_inputproperties != "None"):
                            inputPropertyFileBak = current_inputproperties + ".bak"
                            debug("input.properties file name: %s" % (current_inputproperties))
                            debug("Backup input.properties file name: %s" % (inputPropertyFileBak))
                            configured_wlp_servers[wlpInstallRoot].remove(wasserver)
                            try:
                                if (os.path.exists(inputPropertyFileBak)):
                                    debug("backup file %s already exists, trying to delete" % inputPropertyFileBak)
                                    os.remove(inputPropertyFileBak)
                                rename_rc = os.rename(current_inputproperties, inputPropertyFileBak)
                                if (rename_rc > 0):
                                    debug("Cannot rename input.properties: %s rc=%s" % (current_inputproperties, str(rename_rc)))
                                else:
                                    debug("input.properties renamed from %s to %s" % (current_inputproperties, inputPropertyFileBak))
                            except Exception, renerrmsg:
                                debug("input.properties file could not be renamed - %s - %s" % (current_inputproperties, renerrmsg))
                        msgl =[]
                        msgl.append(wasserver)
                        msg = resourcebundle.getmsg("RESTARTSERVER", "Application Server ({0}) should be restarted", msgl)
                        print "\t%s" % msg
                        
                except:
                    failed = 1
                    log.logException("Error occurred while unconfiguring server %s " % (wasserver))
                    traceback_report = log.format_traceback()
                    
                    #
                    # Set the summary to indicate that the server has failed.
                    #
                    summary[wasserver] = common_constants.SERVER_NOT_OK
                    msg = resourcebundle.getmsg("UNCONFIG_ERROR", "Error occurred while processing unconfiguration")
                    if len(traceback_report) > 0:
                        msg = msg + "\n" + traceback_report
                    print "%s" % (msg)
                    msgl =[]
                    stat = resourcebundle.getmsg("UNSUCCESSFULLY", "unsuccessfully") 
                    msgl.append(wasserver)
                    msgl.append(stat)
                    finish_msg = resourcebundle.getmsg("FINISH_UNCONFIGURING", "Finished Unconfiguring {0} {1}", msgl)   
                    print "%s%s" % (finish_msg, os.linesep) 
    
            #
            # Set the summary so it will be shown as progress to the console.
            #
            
            msg = resourcebundle.getmsg("SUMMARY", "Summary:")
            print "\t%s " % (msg)
            for server_name in summary.keys():
                msgl =[]
                msgl.append(server_name)
                server_stat = summary[server_name]
                log.logDebug("Summary key: %s  status: %s" % (server_name, server_stat))
                if (server_stat == common_constants.SERVER_OK):
                    msg = resourcebundle.getmsg("SERVER_OK", "{0} (OK)", msgl)
                    print "\t\t%s " % (msg)
                elif (server_stat == common_constants.SERVER_OK_MBEAN):
                    msg = resourcebundle.getmsg("SERVER_OK", "{0} (OK)", msgl)
                    print "\t\t%s " % (msg)
                elif (server_stat == common_constants.SERVER_OK_NOMBEAN):
                    msg = resourcebundle.getmsg("SERVER_OK", "{0} (OK)", msgl)
                    print "\t\t%s " % (msg)
                else:
                    print "\t\t%s (X)" % (server_name)
    
            debug("checking if we can remove itcam.properties from <wlp_home>/etc/extensions")
            config_home = pdl["CONFIGHOME"][0]
            remove_wlp_itcam_extensions(config_home)
            #
            # If processing each server caused an error, then a message is issued, and the exception
            # is raised, so the wsadmin gets a non zero return code.
            #
            if (failed == 1):
                log.logDebug("Processing servers had a failure")
                raise Exception("Processing servers had a failure")
    
    
    except:
        log.logException(SCRIPTNAME+" ends abnormally\n ")
        traceback_report = log.format_traceback()
        #
        # If progress object is present, put a message in progress file to be displayed for the wsadmin caller.
        #
        msg = resourcebundle.getmsg("UNCONFIG_ERROR", "Error occurred while processing unconfiguration")
        if len(traceback_report) > 0:
            msg = msg + "\n" + traceback_report
        print "%s" % (msg)
        msgl =[]
        stat = resourcebundle.getmsg("UNSUCCESSFULLY", "unsuccessfully") 
        msgl.append(wasserver)
        msgl.append(stat)
        finish_msg = resourcebundle.getmsg("FINISH_UNCONFIGURING", "Finished Unconfiguring {0} {1}", msgl)
        print "%s%s" % (finish_msg, os.linesep)     
        etype, value, tbck = sys.exc_info()
        #print value
        raise Exception(value)
    
# config
##############################################################
#
#  Procedure unconfigJVM
#  Arguments: new java command line argument
#
##############################################################

def unconfigJVM(itcamJvmArguments, variables_dic):
       
        global serverName
        global old_itcamdc_home
        global genericJvmArgs, newEnvList
        global wlpJvmOptionsFile
        global old_jvm_args
        
        debug("unconfigJVM() entry")


        foundRTT = "false"
        runam = ""

        debug("Modify JVM command line arguments")
        debug ("reading jvm options entries from file %s" % wlpJvmOptionsFile)
        currentGenericJvmArguments = []
        if os.path.exists(wlpJvmOptionsFile):
            infile = None
            try:
                infile = codecs.open(wlpJvmOptionsFile, mode='r', encoding='utf-8')
                lines = ""
                for line in infile.readlines():
                    debug ("line: %s" % line)
                    line = line.rstrip()
                    #if line.startswith("#"): continue
                    if len(line.strip()) == 0: continue
                    currentGenericJvmArguments.append(line)
            finally:
                if infile != None:
                    infile.close();
                    debug("finished reading WLP properties")
        
        else:
            raise Exception, "File %s not found, unable to run unconfig" % wlpJvmOptionsFile

        debug("currentGenericJvmArguments: %s " % currentGenericJvmArguments)
        unconfigArgList = []

        #Check if agentlib:am* is part of currentGenericJvmArguments.
        #If yes, remove it from currentGenericJvmArguments.
        updatecurrentArgList = []
        itcamArgList = itcamJvmArguments
        debug("itcam arguments=%s" % itcamArgList)
        currentArgList = currentGenericJvmArguments
        for currentArg in currentArgList:
            if currentArg.startswith("#"):      #keep comments as they are
                unconfigArgList.append(currentArg)
            else:
                #if server configured by some other DC before 
                currentArg = currentArg.replace(old_itcamdc_home, "${ITCAMDCHOME}")
                if currentArg.find("-agentlib:am") == 0:
                    continue
                elif currentArg.find("-Dam.home") == 0:
                    continue
                elif (currentArg.startswith("-Xbootclasspath/a:") or currentArg.startswith("-Xbootclasspath/p:")):
                    arg = currentArg.split(':')[0] + ':'
                    currentBootClasspath = currentArg[len(arg):]
                    itcamBootClasspath = wlpConfigUtils.getArg(arg, itcamArgList)[len(arg):]
                    debug("currentBootClasspath='%s' itcam add-on path='%s'" % (currentBootClasspath, itcamBootClasspath))
                    bootClasspathList = wlpConfigUtils.unmergePaths (currentBootClasspath, itcamBootClasspath )
                    bootClasspath = wlpConfigUtils.join( bootClasspathList, ":")
                    debug("Unmerged boot class %s " % bootClasspath)
                    if len(bootClasspath.strip()) > 0: unconfigArgList.append(arg + bootClasspath)
                else:
                    if currentArg in itcamArgList:
                        debug("Droping argument %s" % currentArg)
                        continue
                    else:    
                        unconfigArgList.append(currentArg)

        currentGenericJvmArguments = updatecurrentArgList
        debug("currentGenericJvmArguments after unconfig : %s" % unconfigArgList)

        debug("merging with old jvm args list")
        if old_jvm_args != None:
            oldJvmArgsList = old_jvm_args.split('#')
            for arg in oldJvmArgsList:
                if arg not in unconfigArgList:
                    unconfigArgList.append(arg)

        debug("Current generic JVM arguments are: ")
        wlpConfigUtils.displayJvmArgs("", unconfigArgList )

        for arg in unconfigArgList:
            if len(arg.strip()) == 0: continue
            genericJvmArgs.append(arg)

        debug("configJVM() exit")    
        



##############################################################
#
#  Procedure unconfigEnvs
#  Arguments  new environments, list of system variable
#  removes entries added during DC config from server.env file  
#
##############################################################
#updateEnvironments list of dictionaries
def unconfigEnvs ( updateEnvironments, variables_dic ):
    
        global serverName, itcamdc_home
        global old_am_home
        global wlpInstallRoot, wlpUsrDirPath, old_itcamdc_home
        global newEnvList
        global wlpServerEnvFile
        global old_jvm_args
        
        #print "Update env variables in server.env " % str(updateEnvironments)

        debug("%s updating environment entries " % str(len(updateEnvironments)))
        for ue in updateEnvironments:
            debug("item=%s" % ue)
        # Read in the environment from the Java process

        # Create the arrays from the update entries
        itcam_config_env = {}
        hits = {}
        for ue in updateEnvironments:
            nm = ue[0][1]
            val = ue[1][1]
            for var in variables_dic:
                if var in val:
                    val = val.replace(var, variables_dic[var])
            hits[nm] = 0
            itcam_config_env[nm] = val
            debug("%s=%s" % (nm, val))

        debug("Reading in name/value pairs from existing environment for server %s" % serverName)
        current_env = {}
        cvar_names_list = []
        cvar_values_list = []
        unconfig_env = {}
        newEnvList = []
        am_itcam_features_list = []
        
        debug ("reading environment entries from file %s" % wlpServerEnvFile)
        if os.path.exists(wlpServerEnvFile):
            cvar_names_list, cvar_values_list = wlpConfigUtils.getEnvVarList(wlpUsrDirPath, serverName)
            if "ITCAMDCHOME" in cvar_names_list:
                old_itcamdc_home = utils.replaceBackSlash(cvar_values_list[cvar_names_list.index("ITCAMDCHOME")])
            for enm in cvar_names_list:
                eval = cvar_values_list[cvar_names_list.index(enm)]
                if enm in itcam_config_env:
                    if (enm == "NLSPATH" or enm == "LIBPATH" or enm == "LD_LIBRARY_PATH" or enm == "PATH" or enm == "SHLIB_PATH"):
                        #need to unmerge
                        current_path = eval
                        addedPath = itcam_config_env[enm]
                        pathList = wlpConfigUtils.unmergePaths (current_path, addedPath)
                        newPaths = []
                        for path in pathList:
                            # Drop any ITCAMDCHOME path, it is assumed to be ours
                            if (path.find("${ITCAMDCHOME}") != -1 or path.find(old_itcamdc_home) != -1):
                                debug("Existing %s is an old ITCAM entry and will be dropped" % path)
                                continue                    
                            debug("Existing %s entry will be retained in %s " % (path, enm))
                            newPaths.append(path)
            
                        pathDelimiter = ":"
                        if platformUtil.isWindows():
                            pathDelimiter = ";"
                        newPaths = wlpConfigUtils.join( newPaths, pathDelimiter)
                        if len(newPaths) > 0:
                            unconfig_env[enm] = newPaths
                    else:
                        #val = itcam_config_env[enm]
                        debug("dropping variable %s" % enm)
                else:
                    #drop AM_OLD_JVM_ARGS variable
                    if enm == "AM_OLD_JVM_ARGS":
                        old_jvm_args = eval
                        continue
                    elif enm == "AM_NEW_FEATURES":
                        #remove itcam-enabled features from server.xml
                        itcam_features = eval
                        am_itcam_features_list.extend(itcam_features.split(','))
                    else:    
                        unconfig_env[enm] = eval
                        
            #remove features from server.xml
            remove_itcam_features(am_itcam_features_list) 
               
        else:
            raise Exception, "File %s doesn't exist, can not proceed with unconfig " % wlpServerEnvFile
          
        debug("values=%s" % str(unconfig_env))

      
        for key in unconfig_env.keys():
            if key.startswith("#"):         #comment line, put back as is
                newEnvList.append(key)
            else:
                newEnvList.append(key + "=" + unconfig_env[key])
        
        
        debug(" configEnvs() exit %s" % newEnvList)
         
def remove_itcam_features(toremove_feature_list):
    '''
    remove features enabled by DC configurator from server.xml
    '''
    debug("remove_itcam_features Entry %s" % toremove_feature_list)
    global serverName
    global wlpServerXmlFile
    global updated_server_xml
    global itcam_feature_list
    debug("add_itcam_feature entry %s" % serverName)
    
    
    server_xml_file = wlpServerXmlFile
    xml_modified = False
    xml_string = None
    updated_server_xml = None
    unconfig_feature_list = []
   
    if len(toremove_feature_list) > 0:
        unconfig_feature_list = toremove_feature_list
    else:
        unconfig_feature_list = itcam_feature_list
    
   
    debug("Features to be removed from server.xml %s" % unconfig_feature_list)
    if len(unconfig_feature_list) > 0:
        try:
    
            debug("Getting current list of features from server.xml" )
            #
            # Search for the registry profile file name.  The registry resides in the
            # WASHOME/properties/profileRegistry.xml
            #
            
            #server_xml = java.io.File(server_xml_file)
            debug("looking for server xml file %s" % server_xml_file)
            if not os.path.isfile(server_xml_file):        #something is absolutely wrong
                debug ("server.xml not found under %s" % server_xml_file)
                raise ConfigurationException("Server xml file %s not found" % server_xml_file)
            doc= minidom.parse(server_xml_file)
            server = doc.documentElement
            featureManager = server.getElementsByTagName("featureManager")
            for featureManagerNode in featureManager:
                feature_nodes = featureManagerNode.getElementsByTagName("feature")
                for feature in feature_nodes:
                    feature_name = wlpConfigUtils.getNodeText(feature.childNodes).strip()
                    fname, fversion = wlpConfigUtils.get_feature_name_version(feature_name)
                    debug("feature name=%s" % feature_name)
                    if feature_name in unconfig_feature_list or fname.lower() == "itcam":
                        feature.parentNode.removeChild(feature)
                        xml_modified = True
                        debug("Removing feature %s" % feature_name)
                        
            if xml_modified:
                xml_string = doc.toxml()        
        except Exception, errmsg:
            log.logException("Error occurred while reading server.xml - %s" % (errmsg))
            raise 
    else:
       debug("Unable to get features that were added by the configurator utility")
            
    debug("remove_itcam_features exit, features deleted: %s" % xml_string)
    updated_server_xml = xml_string
    return xml_string
           
def get_configured_servers(wlpInstallRoot, runtimedir):
    
    global configured_wlp_servers
    global wlp_home_list
    
    '''
        find all configured wlp servers for the given WLP home
    '''
    debug("get_configured_servers Entry %s" % wlpInstallRoot)
    cellname = utils.convert_nonascii(wlpConfigUtils.getCellName())
    nodename = utils.convert_nonascii(wlpConfigUtils.getNodeName(wlpInstallRoot))
    if wlpInstallRoot not in configured_wlp_servers.keys():
        #processing new WLP install root
        filePrefix = cellname + "." + nodename + "."
        path = runtimedir
        fname = "input.properties"
        debug("Finding all %s*.%s files under %s" % (filePrefix, fname, path))
        for basename in os.listdir(path):
            filename = os.path.join(path,basename)
            if not os.path.isfile(os.path.join(path,filename)): continue
            if not (basename.startswith(filePrefix) and basename.endswith("."+fname)): continue
            servername = basename[len(filePrefix):-(len(fname)+1)]
            debug("Configured  server name: %s for %s" % (servername,basename))
            configured_wlp_servers.setdefault(wlpInstallRoot, [  ]).append(servername)
    debug("get_configured_servers Exit")

def remove_wlp_itcam_extensions(dc_home):
    '''
        remove itcam.properties file from <wlp_home>/etc/extensions is no servers are configured under <wlp_home>
    '''
    
    global configured_wlp_servers
    
    debug("remove_wlp_itcam_extensions entry")
    for key in configured_wlp_servers.keys():
        server_list = configured_wlp_servers.get(key, [  ])
        if len(server_list) == 0:
            wlpInstallRoot = key
            server_ext_dir = wlpInstallRoot + os.sep + "etc" + os.sep + "extensions"
            debug("no configured server under %s, removing itcam.properties from %s" % (wlpInstallRoot, server_ext_dir))
            itcam_properties_file = server_ext_dir + os.sep + "itcam.properties"
            if os.path.isfile(itcam_properties_file):
                os.remove(itcam_properties_file)
            feature_location = dc_home + os.sep + "itcamdc" + os.sep + "etc"
            jar_file_prefix = "com.ibm.tivoli.itcam.liberty.classloader." 
            jar_file_suffix = ".jar"
            mf_file_prefix = "itcam"
            mf_file_suffix = ".mf"
            jar_file_list = get_matching_files(feature_location, jar_file_prefix, jar_file_suffix)
            mf_file_list = get_matching_files(feature_location, mf_file_prefix, mf_file_suffix)
            if len(jar_file_list) == 0 or len(mf_file_list) == 0:
                jar_file = None
                mf_file = None
                errmsg = "One or both files (%s, %s) are missing from %s" % (feature_location, jar_file_prefix, mf_file_prefix)
                debug("required file not found %s" % errmsg)
                raise RuntimeError, errmsg
            features_dir = wlpInstallRoot + os.sep + "usr" + os.sep + "extension" + os.sep + "lib" + os.sep + "features"
            features_lib_dir = wlpInstallRoot + os.sep + "usr" + os.sep + "extension" + os.sep + "lib"
            i = 0
            for jar_file in jar_file_list:
                if jar_file != None and mf_file_list[i] != None:    #copy jar and mf files under runtime/liberty
                    target_jar_file = features_lib_dir + os.sep + jar_file
                    target_mf_file = features_dir + os.sep + mf_file_list[i]
                    if (os.path.exists(target_jar_file)):
						if(platformUtil.isWindows()):
							os.remove(target_jar_file)
						else:
							cmd="rm -f " + target_jar_file
							debug("The cmd is: %s %s" % (cmd,cmd))
							os.system(cmd)
                    if (os.path.exists(target_mf_file)):
						if(platformUtil.isWindows()):
							os.remove(target_mf_file)
						else:
							cmd="rm -f " + target_mf_file
							debug("The cmd is: %s %s" % (cmd,cmd))
							os.system(cmd)
                i = i + 1    
        else:
            debug ("configured servers exist under %s, leaving itcam.properties in place" % key)
            
    debug("remove_wlp_itcam_extensions exit")
	
def get_matching_files(dir, file_prefix, file_suffix=None):
    '''    
    get files matching prefix
    '''
    import glob
    file_list = []
    try:
        os.chdir(dir)
        for f in os.listdir('.'):
            if f.startswith(file_prefix):
                if file_suffix == None:
                    file_list.append(f) 
                else:
                    if f.endswith(file_suffix):
                        file_list.append(f)
    except Exception, errmsg:
        debug("error %s" % errmsg)
    debug("found files %s" % file_list)
    return file_list  	