@echo off

@REM #####################################################################         
@REM # Licensed Materials - Property of IBM
@REM #  
@REM # 5724-L62  
@REM #
@REM # (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.       
@REM #
@REM # US Government Users Restricted Rights - Use, duplication or
@REM # disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
@REM # 
@REM ##################################################################### 

setLocal enableExtensions enableDelayedExpansion

@REM Note: ITCAM_CONFIGHOME is only needed for the logic of this script.
@REM       The Jython program defaults if it is not specified.

@REM Check ITCAM_CONFIGHOME environment variable

@REM If not defined, set it relative to this script
if not defined ITCAM_CONFIGHOME set ITCAM_CONFIGHOME=%~dp0..

@REM Remove any quotes from ITCAM_CONFIGHOME
set dequote=###%ITCAM_CONFIGHOME%###
set dequote=%dequote:###"=%
set dequote=%dequote:"###=%
set dequote=%dequote:###=%
set ITCAM_CONFIGHOME=%dequote%

@REM Validate ITCAM_CONFIGHOME
if not exist "%ITCAM_CONFIGHOME%\bin\config.bat"   goto err_noConfigHome
if not exist "%ITCAM_CONFIGHOME%\bin\reconfig.bat" goto err_noConfigHome
if not exist "%ITCAM_CONFIGHOME%\bin\migrate.bat"  goto err_noConfigHome
if not exist "%ITCAM_CONFIGHOME%\bin\jython\jython.jar" goto err_noConfigHome
if not exist "%ITCAM_CONFIGHOME%\bin\lib\installbundle.jar" goto err_noConfigHome

@REM If JAVA_HOME is not defined, check the default ITM location
set trial_javahome=%ITCAM_CONFIGHOME%\..\..\java\java70_x64\jre
if not defined JAVA_HOME (
  if exist "%trial_javahome%\bin\java.exe" (
    set JAVA_HOME="%trial_javahome%"
    echo.
    echo *Warning*  JAVA_HOME was not defined, defaulting to !JAVA_HOME!
    echo.
  )
)

@REM Locate Java Home in the path if JAVA_HOME is still not defined
@REM Note that the java.exe in system32 is not acceptable because that doesn't locate a real JAVA_HOME
set temp_path=%PATH:"=%
:findJavaHome
if not defined JAVA_HOME if defined temp_path (
  for /F "tokens=1,* delims=;" %%D in ("%temp_path%") do (
    if not "%%~D" == "" if exist "%%~D\java.exe" if "%%~nD" == "bin" (
      set JAVA_HOME="%%~dpD"
      set "JAVA_HOME=!JAVA_HOME:~1,-2!"
      set JAVA_HOME="!JAVA_HOME!"
      echo.
      echo *Warning*  JAVA_HOME was not defined, but Java is found in PATH.
      echo            Using !JAVA_HOME! as JAVA_HOME.
      echo.
    )
    set temp_path=%%E
  )
  goto findJavaHome
)

@REM Remove any quotes from JAVA_HOME
set dequote=###%JAVA_HOME%###
set dequote=%dequote:###"=%
set dequote=%dequote:"###=%
set dequote=%dequote:###=%
set JAVA_HOME=%dequote%

@REM Validate Java Home
if not exist "%JAVA_HOME%\bin\java.exe" if exist "%JAVA_HOME%\jre\bin\java.exe" set JAVA_HOME=%JAVA_HOME%\jre
set JAVA_EXE=%JAVA_HOME%\bin\java
if not exist "%JAVA_EXE%.exe" set JAVA_EXE=%JAVA_HOME%\jre\bin\java
if not exist "%JAVA_EXE%.exe" goto err_noJavaHome
@REM for %%F in ("%JAVA_EXE%") do set JAVA_EXE=%%~sF

if     defined CLASSPATH set C_PATH=%ITCAM_CONFIGHOME%\bin\jython\jython.jar;%ITCAM_CONFIGHOME%\bin\lib\installbundle.jar;%CLASSPATH%
if not defined CLASSPATH set C_PATH=%ITCAM_CONFIGHOME%\bin\jython\jython.jar;%ITCAM_CONFIGHOME%\bin\lib\installbundle.jar
set PYTHONPATH=%ITCAM_CONFIGHOME%\bin;%ITCAM_CONFIGHOME%\bin\lib

@REM Set the JVM options
set JAVA_OPTS=
set JAVA_PROPS=-Dpython.path="%PYTHONPATH%"
set JAVA_TOOL_OPTIONS=

@REM echo.
@REM echo Launching ITCAM reconfiguration...
@REM echo.
@REM echo   ITCAM_CONFIGHOME: %ITCAM_CONFIGHOME%
@REM echo.
@REM echo   JAVA_HOME:        %JAVA_HOME%
@REM echo.
@REM set p=%C_PATH%
@REM set t=  CLASSPATH:       x
@REM set t=%t:x=%
@REM :loop_CLASSPATH
@REM if not "%p%" == "" (
@REM   for /f "tokens=1,* delims=;" %%I in ("%p%") do (
@REM     if not "%%I" == "" echo %t% %%I
@REM     set p=%%J
@REM     set t=                   x
@REM     set t=!t:x=!
@REM   )
@REM   goto loop_CLASSPATH
@REM )
@REM echo.
@REM set p=%PYTHONPATH%
@REM set t=  PYTHONPATH:      x
@REM set t=%t:x=%
@REM :loop_PYTHONPATH
@REM if not "%p%" == "" (
@REM   for /f "tokens=1,* delims=;" %%I in ("%p%") do (
@REM     if not "%%I" == "" echo %t% %%I
@REM     set p=%%J
@REM     set t=                   x
@REM     set t=!t:x=!
@REM   )
@REM   goto loop_PYTHONPATH
@REM )
@REM echo.

@REM Execute the DC reconfigurator
set launchcommand="%JAVA_EXE%" -cp "%C_PATH%" %JAVA_OPTS% %JAVA_PROPS% org.python.util.jython "%ITCAM_CONFIGHOME%\bin\config.py" -reconfig %*
@REM echo %launchcommand%
cd /d %ITCAM_CONFIGHOME%\bin
%launchcommand%
set RC=%ERRORLEVEL%
goto done

:err_noConfigHome
    echo *Error*  The environment variable ITCAM_CONFIGHOME is not correctly defined.
    set RC=1
    goto done
    
:err_noJavaHome
    echo.
    echo *Error*  The JAVA_HOME specified is invalid.  Please correct and resubmit.
    echo.
    set RC=1
    goto done

:done
if not defined RC set RC=0
endlocal & set RC=%RC%
exit /b %RC%
