@REM Licensed Materials - Property of IBM

@REM  Copyright IBM Corp. 2012  All Rights Reserved.

@REM US Government Users Restricted Rights - Use, duplication or
@REM disclosure restricted by GSA ADP Schedule Contract with
@REM IBM Corp.


@echo off

setlocal
set LDIR="%~dp0"
cd %LDIR%

set USER=
set PASSWORD=
set RE_INSTRUMENTATION=false
set CLASS_SELECTION_PROPERTY_MASK=


:processArgs

if "%1"=="" goto invokeJacl

if "%1"=="-profilehome" (
  if "%2"=="" goto usage
  set SRVDIR=%~2
  shift
  shift
  goto processArgs
)

if "%1"=="-node" (
  if "%2"=="" goto usage
  set WAS_NODE=%2
  shift
  shift
  goto processArgs
)

if "%1"=="-server" (
  if "%2"=="" goto usage
  set WAS_SERVER=%2
  shift
  shift
  goto processArgs
)

if "%1"=="-username" (
  if "%2"=="" goto usage
  set USER=%1 %2
  shift
  shift
  goto processArgs
)

if "%1"=="-password" (
  if "%2"=="" goto usage
  set PASSWORD=%1 %2
  shift
  shift
  goto processArgs
)

if "%1"=="-reinstrumentation" (
  if not "%2"=="true" (
    if not "%2"=="false" (
      goto usage
    )
  )    
  set RE_INSTRUMENTATION="%2"
  shift
  shift
  goto processArgs
)

if "%1"=="-propertymask" (
  if "%2"=="" goto usage
  set CLASS_SELECTION_PROPERTY_MASK=%2
  shift
  shift
  goto processArgs
)


if "%1"=="-classmasks" (
  if "%2"=="" goto usage
  set CLASS_SELECTION_MASKS=%2
  shift
  shift
  goto processArgs
)

:usage

echo.
echo "Usage: reloadDCConfig.bat  [-profilehome WebSphere_profile_home]  [-node WebSphere_node_name]  [-server WebSphere_server_name] [-username user_name] [-password pass_word]"
echo Dynamically reload the configuration of ITCAM Data Collectors from the file system.
echo.
echo WebSphere_profile_name: the home directory of the WebSphere profile
echo.
echo WebSphere_node_name: the WebSphere node name
echo.
echo WebSphere_server_name: the WebSphere server name
echo.
echo user_name: WebSphere user name
echo.
echo pass_word: Login password of the username 
echo.
echo Example 1: reloadDCConfig.bat
echo.
echo Example 2: reloadDCConfig.bat -profilehome "c:\WebSphere v7\profiles\p1" -node node1 -server server1
echo.
echo Example 3: reloadDCConfig.bat -profilehome "c:\WebSphere v7\profiles\p1" -node node1 -server server1 -username admin -password password
echo.
echo If you don't want to use -profilehome to specify WebSphere profile home in the commandline options, you can define the environment variable like, set WAS_PROFILE_HOME="c:\WebSphere v7\profiles\p1"
echo.
echo If you don't want to use -node to specify WebSphere node name in the commandline options, you can define the environment variable like, set RT_WAS_NODE_NAME=node1
echo.
echo If you don't want to use -server to specify WebSphere server name in the commandline options, you can define the environment variable like, set RT_WAS_SERVER_NAME=server1

goto done

:invokeJacl

if not "%WAS_PROFILE_HOME%"=="" (
	set WAS_PROFILE_HOME=%WAS_PROFILE_HOME:"=%
)

if "%SRVDIR%"=="" (
  if "%WAS_PROFILE_HOME%"=="" (
	echo "WebSphere profile home needs to be specified in the commandline options or in the environment variable,WAS_PROFILE_HOME."
	goto usage
  )	
  set SRVDIR=%WAS_PROFILE_HOME%
)

if not exist "%SRVDIR%\config\cells" (	
     	echo "WebSphere profile home, %SRVDIR%, is incorrect."
	goto usage
)

if "%WAS_NODE%"=="" (
  if "%RT_WAS_NODE_NAME%"=="" (
	echo "WebSphere node name needs to be specified in the commandline options or in the environment variable,RT_WAS_NODE_NAME."
	goto usage
  )	
  set WAS_NODE=%RT_WAS_NODE_NAME%
)

if not exist "%SRVDIR%\temp\%WAS_NODE%" (	
     	echo "The node name, %WAS_NODE%, is incorrect."
	goto usage
)

if "%WAS_SERVER%"=="" (
  if "%RT_WAS_SERVER_NAME%"=="" (
	echo "WebSphere server name needs to be specified in the commandline options or in the environment variable, RT_WAS_SERVER_NAME."
	goto usage
  )	
  set WAS_SERVER=%RT_WAS_SERVER_NAME%
)

if not exist "%SRVDIR%\temp\%WAS_NODE%\%WAS_SERVER%" (	
     	echo "The server name, %WAS_SERVER%, is incorrect."
	goto usage
)

if "%CLASS_SELECTION_PROPERTY_MASK%"=="" set CLASS_SELECTION_PROPERTY_MASK="null"


call "%SRVDIR%"\bin\wsadmin.bat %USER% %PASSWORD% -f ..\toolkit\etc\reconfig_dc_was.jacl %WAS_NODE% %WAS_SERVER% %RE_INSTRUMENTATION% %CLASS_SELECTION_PROPERTY_MASK% %CLASS_SELECTION_MASKS%

if errorlevel 1 exit /b 1

echo Reconfiguration complete.

:done
exit /b 0
