#!/bin/sh
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2012  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# All Rights Reserved

DIR=`dirname $0`
cd $DIR

SYNTAX_ERROR=false
USER=""
PASSWORD=""
RE_INSTRUMENTATION="false"
CLASS_SELECTION_PROPERTY_MASK=""

ParseOptions () {
   while [ "$1" != "" ]
   do
     if [ "$1" = "-profilehome" ]
     then
       if [ "$2" = "" ]
       then
         SYNTAX_ERROR=true
         return        
       else
         SRVDIR="$2"
         shift
         shift
       fi
     elif [ "$1" = "-node" ]
       then
         if [ "$2" = "" ]
         then
           SYNTAX_ERROR=true
           return        
         else
           WAS_NODE="$2"
           shift
           shift
         fi  
     elif [ "$1" = "-server" ]
       then
         if [ "$2" = "" ]
         then
           SYNTAX_ERROR=true
           return        
         else
           WAS_SERVER="$2"
           shift
           shift
         fi  
     elif [ "$1" = "-username" ]
       then
         if [ "$2" = "" ]
         then
           SYNTAX_ERROR=true
           return        
         else
           USER="$1 $2"
           shift
           shift
         fi  
     elif [ "$1" = "-password" ]
     then
       if [ "$2" = "" ]
       then
         SYNTAX_ERROR=true
         return
       else
         PASSWORD="$1 $2"
         shift
         shift
       fi
     elif [ "$1" = "-reinstrumentation" ]
     then
       if [ "$2" != "true" ] && [ "$2" != "false" ] 
       then
         SYNTAX_ERROR=true
         return
       fi
       RE_INSTRUMENTATION="$2"
       shift 
       shift
     elif [ "$1" = "-propertymask" ]
     then

       if [ "$2" = "" ]
       then
         SYNTAX_ERROR=true
         return           
       else
         CLASS_SELECTION_PROPERTY_MASK="$2"
         shift
         shift
       fi
     elif [ "$1" = "-classmasks" ]
     then
       if [ "$2" = "" ]
       then
         SYNTAX_ERROR=true
         return           
       else
         CLASS_SELECTION_MASKS="$2"
         shift
         shift
       fi
     else
       SYNTAX_ERROR=true
       return
     fi
   done
}

ParseOptions $*

if [ "$CLASS_SELECTION_PROPERTY_MASK" = "" ]
then
  CLASS_SELECTION_PROPERTY_MASK="null"
fi


if [ "$SRVDIR" = "" ]
then
  if [ "$WAS_PROFILE_HOME" = "" ]
  then
    echo "WebSphere profile home needs to be specified in the commandline options or in the environment variable,WAS_PROFILE_HOME."
    SYNTAX_ERROR=true
  else
    SRVDIR=$WAS_PROFILE_HOME
  fi
fi

if [ $SYNTAX_ERROR = false ] && [ ! -d "$SRVDIR/config/cells" ]
then
    echo "WebSphere profile home, $SRVDIR, is incorrect."
    SYNTAX_ERROR=true
fi

if [ "$WAS_NODE" = "" ]
then
  if [ "$RT_WAS_NODE_NAME" = "" ]
  then
    echo "WebSphere node name needs to be specified in the commandline options or in the environment variable,RT_WAS_NODE_NAME."
    SYNTAX_ERROR=true
  else
    WAS_NODE=$RT_WAS_NODE_NAME
  fi
fi

if [ $SYNTAX_ERROR = false ] && [ ! -d "$SRVDIR/temp/$WAS_NODE" ]
then
    echo "The node name, $WAS_NODE, is incorrect."
    SYNTAX_ERROR=true
fi

if [ "$WAS_SERVER" = "" ]
then
  if [ "$RT_WAS_SERVER_NAME" = "" ]
  then
    echo "WebSphere server name needs to be specified in the commandline options or in the environment variable, RT_WAS_SERVER_NAME."
    SYNTAX_ERROR=true
  else
    WAS_SERVER=$RT_WAS_SERVER_NAME
  fi
fi

if [ $SYNTAX_ERROR = false ] && [ ! -d "$SRVDIR/temp/$WAS_NODE/$WAS_SERVER" ]
then
      echo "The server name, $WAS_SERVER, is incorrect."
      SYNTAX_ERROR=true
fi

if [ $SYNTAX_ERROR = false ]
then
  $SRVDIR/bin/wsadmin.sh $USER $PASSWORD -f ../toolkit/etc/reconfig_dc_was.jacl $WAS_NODE $WAS_SERVER  $RE_INSTRUMENTATION $CLASS_SELECTION_PROPERTY_MASK $CLASS_SELECTION_MASKS |tee ../data/reloadDCConfig.out

  if [ ! -s ../data/reloadDCConfig.out ]
  then
    echo "Reconfiguration failed because the current user is not a WebSphere Administrator. Please run this command from an user ID with WebSphere administrative privileges." 
    exit $?
  fi

  grep Error ../data/reloadDCConfig.out >../data/reloadDCConfig.err

  if [ "$?" = "0" ] && [ ! -s ../data/reloadDCConfig.err ]
  then
      echo "Reconfiguration complete."
  fi

  exit $?
else
  # print usage
  echo ""
  echo "Usage: ./reloadDCConfig.sh [-profilehome WebSphere_profile_home]  [-node WebSphere_node_name]  [-server WebSphere_server_name] [-username user_name] [-password pass_word]"

  echo "Dynamically reload the configuration of ITCAM Data Collectors from the file system."
  echo ""
  echo "WebSphere_profile_name: the home directory of the WebSphere profile"
  echo ""
  echo "WebSphere_node_name: the WebSphere node name"
  echo ""
  echo "WebSphere_server_name: the WebSphere server name"
  echo ""
  echo "user_name: WebSphere user name"
  echo ""
  echo "pass_word: Login password of the username"
  echo ""

  echo ""
  echo "Example 1: ./reloadDCConfig.sh"
  echo ""
  echo "Example 2: ./reloadDCConfig.sh -profilehome /WebSphere7/profiles/p1 -node node1 -server server1"
  echo ""
  echo "Example 3: ./reloadDCConfig.sh -profilehome /WebSphere7/profiles/p1 -node node1 -server server1 -username admin -password password"
  echo ""
  echo "If you don't want to use -profilehome to specify WebSphere profile home in the commandline options, you can define the environment variable like, export WAS_PROFILE_HOME=/WebSphere7/profiles/p1"
  echo ""
  echo "If you don't want to use -node to specify WebSphere node name in the commandline options, you can define the environment variable like, export RT_WAS_NODE_NAME=node1"
  echo ""
  echo "If you don't want to use -server to specify WebSphere server name in the commandline options, you can define the environment variable like, export RT_WAS_SERVER_NAME=server1"
  echo ""

  exit 1
fi


