#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################


'''
    Create silent configuration file and perform silent DC config
'''


import os
import sys
import platform
import traceback
import codecs
import shutil
import platformUtil
import re
import utils
from time import strftime
import wasdiscover

import debugUtil as log
from debugUtil import logDebug as debug

from PrintWrapper import PrintWrapper

if platformUtil.isWindows():
    import getpass_loc as getpass
    agent_home = "C:/IBM/ITM"
    default_was_home = "C:\\Program Files\\IBM\\WebSphere\\AppServer"
    config_script = "config.bat"
else:
    import getpass_notty as getpass
    agent_home = "/opt/ibm/ccm/agent/install-images/kyn"
    default_was_home = "/opt/IBM/WebSphere/AppServer"
    config_script = "config.sh"

fixlev = "42288"
entervalue_msg = ""

DEFAULT_WAS_CONFIG = [
"[DEFAULT SECTION]",
"default.hostip=localhost",
"ms.connect=False",
"ttapi.enable=True",
"ttapi.host=127.0.0.1",
"ttapi.port=5457",
"soa.enable=False",
"tpv.enable=False",
"de.enable=True",
"temaconnect=True",
"tema.appserver=True",
"tema.host=127.0.0.1",
"tema.port=63335",
"tema.jmxport=63355",
"was.backup.configuration=False",
"was.backup.configuration.dir=/opt/IBM/ITM/dchome",
"was.wsadmin.username=",
"was.wsadmin.password=",
"was.client.props=False",
"was.appserver.profile.name=AppSrv02",
"was.appserver.home=/opt/IBM",
"was.appserver.cell.name=yourITCAMCell",
"was.appserver.node.name=yourITCAMNode",
"[SERVER]",
"was.appserver.server.name=server1"
]

DEFAULT_LIBERTY_CONFIG = [
"[DEFAULT SECTION]",
"default.hostip=localhost",
"de.enable=True",
"ttapi.enable=True",
"ttapi.host=127.0.0.1",
"ttapi.port=5457",
"temaconnect=True",
"tema.appserver=True",
"tema.jmxport=63355",
"tema.host=127.0.0.1",
"tema.port=63335",
"was.backup.configuration=False",
"was.backup.configuration.dir=/opt/IBM/ITM/dchome",
"was.appserver.home=/opt/IBM/wlp",
"[SERVER]",
"was.appserver.server.name=server1"
]

TERMINATE = 'q'

def prompt_list(option_list, prompt_msg, default_value=None, allow_custom=False):

    global entervalue_msg
    global default_msg
    global TERMINATE
    
    debug("Entry prompt_list_no_pdl %s" % fixlev)

    default_msg = getmsg("DEFAULT", "default is:")
    invalid_reponse_msg = "Invalid response: " 
    local_answer = None
    stored_answer = local_answer
    if len(option_list) == 0:
        raise Exception("zero length list provided as an input")
    errorcount = 0
    option_str = ""
    index_list = []
    for i in range(len(option_list)):
        index = i + 1
        index_list.append(str(index))
        option_str += "\n  %s. %s" % (str(index), option_list[i])

    while local_answer == None :
        #debug("Prompt for %s Pass '%d' out of '%d'" % (option_str, errorcount, 5)) #common_constants.MAX_TRY))
        if errorcount == 5: #common_constants.MAX_TRY:
            print "Error: Maximum number of retries have been reached. Unable to prompt for a valid selection"
            #print "%s" % getmsg("CFG5003E", "Error: Maximum number of retries have been reached. Unable to prompt for a valid selection")
            raise KeyboardInterrupt("Received terminate request in doprompt")
                #raise Exception("Unable to prompt for a valid selection ")
        msg = "%s %s " % (prompt_msg, option_str)
        if default_value != None:
            msg = "%s\n%s [%s %s]: " % (msg, entervalue_msg, default_msg, str(default_value))
        else:
            msg = "%s\n%s:" % (msg, entervalue_msg)

        print msg
        local_answer = raw_input()
        stored_answer = local_answer
        if local_answer == TERMINATE:
            debug("User requested to terminate")
            raise KeyboardInterrupt("Received terminate request in prompt_list")
       
        if local_answer == '':
            if default_value != None:
                local_answer = default_value
            else:
                local_answer = None
        else:
            if local_answer not in index_list:
                if allow_custom:
                    is_valid = wasdiscover.verifyValidWasHome(local_answer, None)
                    if is_valid == 0:
                        local_answer = None
                else:
                    local_answer = None
        if local_answer is None:
            errorcount += 1
            print "\n%s '%s'\n" % (invalid_reponse_msg, str(stored_answer))
    return local_answer

def prompt_for_answer(prompt_message, options_str, default_value, format):
    
    global TERMINATE
    
    local_answer = None
    debug("prompt_for_answer entrance %s" % (prompt_message))
    if format == "password" and utils.isSet(default_value): default_value = "******"
    invalid_reponse_msg = "Invalid response: " #getmsg("CFG5013E", "Invalid response:")
    errorcount = 0
    stored_answer = local_answer
    while local_answer is None:
        if errorcount == 5: #common_constants.MAX_TRY:
            print "Error: Maximum number of retries have been reached. Unable to prompt for a valid selection"
            #print "%s" % getmsg("CFG5003E", "Error: Maximum number of retries have been reached. Unable to prompt for a valid selection")
            raise KeyboardInterrupt("Received terminate request in doprompt")
        if utils.isSet(default_value):
            msg = "%s %s [%s %s]: " % (prompt_message, options_str, default_msg, default_value.encode(None,"backslashreplace"))
        else:
            if options_str != "":
                msg = "%s %s: " % (prompt_message, options_str)
            else:
                msg = "%s: " % (prompt_message)
        if format == "password":
            msg = msg + "\n"
            local_answer = getpass.getpass(msg)
            stored_answer = "******"
        else:
            print msg
            local_answer = raw_input()
            stored_answer = local_answer
            
        if local_answer == TERMINATE:
            debug("User requested to terminate")
            raise KeyboardInterrupt("Received terminate request in prompt_for_answer")
        if format == "washome":
            is_valid = wasdiscover.verifyValidWasHome(local_answer, None)
            if is_valid == 0:
                local_answer = None
        
        if local_answer is None:
            errorcount += 1
            print "\n%s '%s'\n" % (invalid_reponse_msg, stored_answer)
    return local_answer



# Outermost exception handler
try:

    # Eliminate need to prefix import our modules with "lib."
    sys.path.append("lib")

    # Jython 2.5.2 compatibility
    _Jython_vrm = os.sys.version_info[0:3]
    assert _Jython_vrm >= (2,5,2), "Unsupported Jython version %s" % ".".join([str(_vrm) for _vrm in _Jython_vrm])

    # Java 1.5 compatibility
    _Java_vrm = [int(_vrm) for _vrm in platform.java_ver()[0].split(".")[0:2]]
    assert _Java_vrm >= [1,5], "Unsupported Java JVM version %s" % ".".join([str(_vrm) for _vrm in _Java_vrm])

    import get_wasroot
    import tcpipstack
    import wasdiscover
    import utils
    import wlpConfigUtils

    from constants import constants
    from ConfigurationException import ConfigurationException

    import resourcebundle
    from resourcebundle import getmsg

    import debugUtil as log
    from debugUtil import logDebug as debug

    from PrintWrapper import PrintWrapper
    
    
    #TERMINATE = getmsg("QUIT_CHAR","q")          #termination character

    TERMINATE = 'q'
    # Exception handler for expected exceptions such
    # as ConfigurationException, KeyboardException and SystemExit.
    try:

        #############
        # Initialize
        #############

        # Instantiate the global constants class
        common_constants = constants()

        # Set the print wrapper for stdout and stderr.
        PrintWrapper(common_constants.STDOUT)
        PrintWrapper(common_constants.STDERR)

        # Initialize the resource bundle
        resourcebundle.set_resourcebundle()

        # There should be no debug() statement before this point
        debug("Enter config %s" % fixlev)

        logfile = "simpleconfig-trace.log"
        config_dir = None
        if len(sys.argv) > 0 and sys.argv[0]:
            cwd = os.path.dirname(os.path.abspath(sys.argv[0]))
            debug("Current working directory based on sys.arg[0]: %s" % cwd)
        else:
            cwd = os.path.abspath(os.getcwd())
            debug("Current working directory based on os.getcwd(): %s" % cwd)
        if os.path.basename(cwd) == "bin":
            config_dir = os.path.join(cwd, os.pardir)
            debug("Configuration home default: %s" % config_dir)
        else:
            config_dir = None

        log.set_logging(log_filename=logfile, msg_filename=None, config_home=config_dir)
        masked_pswrd = "*******"
        isWLP = False


        cwd = os.getcwd()
        config_home = None
        if len(sys.argv) > 0 and sys.argv[0]:
            cwd = os.path.dirname(os.path.abspath(sys.argv[0]))
            debug("Current working directory based on sys.arg[0]: %s" % cwd)
        else:
            cwd = os.path.abspath(os.getcwd())
            debug("Current working directory based on os.getcwd(): %s" % cwd)
        if os.path.basename(cwd) == "bin":
            config_cmd = cwd + os.sep + config_script
            config_home = os.path.join(cwd, os.pardir)
            debug("Configuration home default: %s" % config_home)
        else:
            raise ConfigurationException(getmsg("CFG5028E", "ITCAM_CONFIG is not set and current directory is not a valid config_home"))

        entervalue_msg = getmsg("ENTERVALUE", "Enter a number")

        fullLogName = log.getLogFilename()
        
        print "-------------------------------------------------------------------------------\t"
        print "-  Monitoring Agent for WebSphere Applications                                -\t"
        print "-  Data Collector Simple Configuration                                        -\t"
        print "-                                                                             -\t"
        print "-  This utility will perform a simple DC configuration.                       -\t"
        print "-  If you require to configure the DC with more options please run:           -\t"
        print "-  %s           -\t" % config_cmd
        print "-                                                                             -\t"
        print "-  At each prompt, enter 'q' to quit the utility.                             -\t"
        print "-------------------------------------------------------------------------------\t"
        print ""
        print "Log file: %s" % fullLogName

        was_dir_list = []
        # Determine the local host name
        #tcpipstack.get_iphost(pdl)


        was_dir_list = wasdiscover.discover_was()
        washome = None
        profile_home = None
        profile_name = None
        server_name = "*"
        if len(was_dir_list) > 0:
            print "List of WebSphere Application Server home directories discovered:"
            prompt_msg = getmsg("ENTERWASROOT", "Enter a number or enter the full path to a home directory")
            answer = prompt_list(was_dir_list, prompt_msg, default_value=None, allow_custom=True)
            if answer.isdigit():
                washome = was_dir_list[int(answer) - 1]
            else:
                washome = answer
        else:
            print "%s" % getmsg("CFG1134E", "Unable to discover WebSphere installation directory on the system. ")
            washome = prompt_for_answer("Specify WebSphere home directory", "", None, "washome")

        # Call findServers
        pdl = {}
        pdl["CREATETEMPLATE"] = [False,"Initial",1]
        pdl["DELETETEMPLATE"] = [False, "Initial", 1]

        if washome is None:
            debug("No valid websphere home directory specified")
            raise ConfigurationException("No valid websphere home directory specified")
        isWLP = wasdiscover.isWLPHome(washome)
        if  isWLP:
            print "\n%s" % getmsg("CFG1170I", "Found WebSphere Liberty Profile")
            print "Configuring all servers under Liberty Profile home"
            server_list = wlpConfigUtils.getWlpServersList(washome)
            #index = int(prompt_list(server_list, "Select server to configure", default_value=None)) - 1
            #server_name = server_list[int(index)]
            server_name = "*"
            wsadmin_user = ""
            wsadmin_password = ""
            profile_name = ""
            cellName = ""
            nodeName = ""
        else:
            try:
                profile_name_list, profile_path_list , default_profile_name = wasdiscover.get_was_profiles(pdl, washome)
                if len(profile_name_list) == 0: raise Exception
            except Exception:
                raise ConfigurationException(getmsg("CFG1073E","Can not discover profiles. Make sure there is a local application server profile created."))
            if len(profile_name_list) == 1:
                index = 0
            elif len(profile_name_list) > 1:
                index = int(prompt_list(profile_name_list, "Specify WAS profile", default_value=None)) - 1
            else:
                print "No application server profile exists under WAS home %s" % washome
                raise ConfigurationException(getmsg("CFG1073E","Can not discover profiles. Make sure there is a local application server profile created."))
            profile_name = profile_name_list[int(index)]
            profile_home = profile_path_list[int(index)]
            isPortal = wasdiscover.isWASPortal(profile_home)
            if (isPortal):
                okToConfigure = True
                error_msg = ""
                debug("that is websphere portal server, checking version")
                #check if this is Portal 7 and tell users that it is not supported
                portal_version = wasdiscover.getPortalVersion(profile_home)
                debug("portal version=%s" % portal_version)
                if (portal_version != None):
                    versionList = portal_version.split(".")
                    if versionList[0] == "7":   #do not configure Portal 7 due to potential issues with the base server
                        okToConfigure = False
                        error_msg = "This tool doesn't support configuration of WebSphere Portal Server 7. Please use advanced configuration utility to configure."
                else:
                    okToConfigure = False
                    error_msg = "You have WebSphere stack product installed in the directory. Please use advanced configuration utility to configure."
                if not okToConfigure:
                    debug("exiting from simpleconfig, unsupported version of WPS")
                    print "\n%s" % error_msg
                    raise KeyboardInterrupt(error_msg)
            print "Looking for servers under profile %s" % profile_name

            wsadmin_user = None
            wsadmin_password = None
            securityOn = wasdiscover.is_security_enabled(profile_home)
            #print "securityOn=%s" % securityOn
            if securityOn == "true":
                wsadmin_user = prompt_for_answer("Enter wsadmin user ", "", "", "")
                if wsadmin_user is not None and len(wsadmin_user) > 0:
                    wsadmin_password = prompt_for_answer("Enter wsadmin password ", "", "", "password")
            cur_dir = os.getcwd()
            print "Discovering profile. Please wait..."
            wsadmin_cmd_string = os.path.normpath(os.path.join(profile_home, "bin", platformUtil.getWSAdminScript()))
            wsadmin_cmd_string = utils.add_quotes(wsadmin_cmd_string)

            #wsadmin_cmd_string = profile_home + os.sep + "bin" + os.sep + wsadmin_script + " -lang jyhton"
            
            wsadmin_cmd_debug = wsadmin_cmd_string
            wsadmin_user_tmp = wsadmin_user
            wsadmin_password_tmp = wsadmin_password        
            if wsadmin_password is not None and len(wsadmin_password) == 0:
                wsadmin_password = '"  "'
                debug("PASSWORD is blank")
            
            if wsadmin_user is not None:
                if not platformUtil.isWindows():
                    wsadmin_user = re.escape(wsadmin_user)
                wsadmin_cmd_string = wsadmin_cmd_string + " -user " + wsadmin_user
                wsadmin_cmd_debug  = wsadmin_cmd_string

            if wsadmin_password is not None:
                if not platformUtil.isWindows():
                    wsadmin_password = re.escape(wsadmin_password)
                wsadmin_cmd_string = wsadmin_cmd_string + " -password " + wsadmin_password
                wsadmin_cmd_debug  = wsadmin_cmd_debug + " -password *********"

            if wsadmin_user is None:
                wsadmin_user = ""
                
            if wsadmin_password is None :
                wsadmin_password = ""

            server_list = []
            out_file_no_quotes = config_home + os.sep + "data" +  os.sep + "was_info.dat"
            out_file_no_quotes = utils.replaceBackSlash(out_file_no_quotes)
            out_file = utils.add_quotes(out_file_no_quotes)
            jython_script = utils.add_quotes(config_home + os.sep + "bin" + os.sep + "lib" + os.sep + "getTopology.py")
            wsadmin_cmd_string = wsadmin_cmd_string + " -f " + jython_script + " " + out_file
            wsadmin_cmd_debug  = wsadmin_cmd_debug + " -f " + jython_script + " " + out_file

            rc, out_list = utils.runCommand(wsadmin_cmd_string, wsadmin_cmd_debug)
            #print "rc=%s out_list=%s" % (rc, out_list)
            if rc != 0:
               if any("authentication failure" in s for s in out_list):
                   raise ConfigurationException("WebSphere authentication failure")
               else:
                   raise ConfigurationException("Failed to get WAS topology info, check that application server is started")
            wsadmin_user = wsadmin_user_tmp
            wsadmin_password = wsadmin_password_tmp 
            
            if wsadmin_user is None:
                wsadmin_user = ""
                
            if wsadmin_password is None :
                wsadmin_password = ""             
            if os.path.isfile(out_file_no_quotes):
                infile = codecs.open(out_file_no_quotes, mode='r', encoding='utf-8')
                server_list = []
                serverNameList = []
                aServer = {}
                lines = ""
                for line in infile.readlines():
                    line = line.rstrip('\r\n')
                    tokens = line.split('\t')
                    if len(tokens) == 3:
                        serverName = tokens[0]
                        nodeName = tokens[1]
                        cellName = tokens[2]
                        serverNameList.append(serverName)
                        aServer = {"WASCELL": cellName, "WASNODE": nodeName, "WASSERVER": serverName}
                        server_list.append(aServer)
                infile.close()
        silent_simpleconfig_base_filename = "silent_simpleconfig.txt"
        template_config_list = DEFAULT_WAS_CONFIG
        if len(server_list) > 0:
           if isWLP:
              silent_simpleconfig_base_filename = "silent_liberty_simpleconfig.txt"
              template_config_list = DEFAULT_LIBERTY_CONFIG
              config_values_map = {
                   "was.appserver.home":          washome,
                   "was.appserver.server.name":   server_name
               }
           else:
              config_values_map = {
                   "was.wsadmin.username":        wsadmin_user,
                   "was.wsadmin.password":        wsadmin_password,
                   "was.appserver.profile.name":  profile_name,
                   "was.appserver.home":          washome,
                   "was.appserver.cell.name":     cellName,
                   "was.appserver.node.name":     nodeName,
                   "was.appserver.server.name":   server_name
               }
           silent_config_file = config_home + os.sep + "bin" + os.sep + silent_simpleconfig_base_filename
           log._rename(silent_config_file)
           debug("silent response file used for configuration %s " % silent_config_file)
           
           destination= open( silent_config_file, "w" )
           destination.write("#Sample silent response file created by simpleconfig script on %s%s" % (strftime("%Y/%m/%d %H:%M"), os.linesep))
           for line in template_config_list:
               line.lstrip()
               token = line.split('=')
               line_changed = False
               if len(token) == 2:
                   if token[0] in config_values_map:
                        new_line = token[0].lstrip('#') + "=" + str(config_values_map.get(token[0]))
                        destination.write(new_line)
                        line_changed = True
               if not line_changed:
                   destination.write( line )
               destination.write(os.linesep)
               
           destination.close()
        else:
            raise ConfigurationException("No Application servers found for the profile")

        if platformUtil.isWindows():
           utils.runCommand("attrib +h " + silent_config_file , None)

        print "Configuring. Please wait..."
        config_dc_cmd = config_home + os.sep + "bin" + os.sep + config_script
        config_dc_cmd = utils.add_quotes(config_dc_cmd)
        silent_config_file_quotes = utils.add_quotes(silent_config_file)
        config_dc_cmd = config_dc_cmd + " -silent " + silent_config_file_quotes
        rc, out_list = utils.runCommand(config_dc_cmd, None)
        #console_wtr.log("%s" % out_list)
        #print "rc=%s out_list=%s" % (rc, out_list)

        s=open(silent_config_file).read()
        if wsadmin_password in s:
           s=s.replace("was.wsadmin.password="+wsadmin_password, "was.wsadmin.password=*******")
           f=open(silent_config_file, 'w')
           f.write(s)
           f.flush()
           f.close()

        if rc != 0:
           raise ConfigurationException("Configuration of Data collector failed. Please check logs for details.")

        # If we get here everything should be okay, and we exit with RC(0)
        print "Configuration completed successfully."
        print "Please restart the WebSphere servers under the selected profile to exploit the full power of Monitoring Agent for WebSphere Applications Data Collector."
        sys.exit(0)

    # This exception is raised when a problem is detected which may possibly
    # be corrected by the user.
    except ConfigurationException, instance:

        print "%s" % getmsg("CFG5026E", "Critical error in {0}", ["simpleconfig.py"])
        print "%s" % instance.parameter

        log.logException("Configuration error in config.py")
        #log.logDumpStack("Stack data for diagnostic purposes")


        sys.exit(1)

    # assert during execution, detected internal program error
    except AssertionError:

        print "%s" % getmsg("CFG5022E", "Internal program error detected in {0}", ["simpleconfig.py"])

        log.logException("Assertion error in config.py")
        log.logDumpStack("Stack data for diagnostic purposes")
        sys.exit(1)

    # User entered terminate character in prompt
    except KeyboardInterrupt:

        print "%s" % getmsg("CFG1998I", "Configuration of Data Collector has been bypassed")
        print "%s" % getmsg("CFG1088I", "Exiting configuration script")
        sys.exit(1)

    # For sys.exit(0) we will quietly exit and suppress dumping the log to the console.
    # For sys.exit(1) or indeed otherwise, we presume an erroneous situation and diagnose.
    except SystemExit, exitcode:

        if exitcode.code == 0: sys.exit(0)
        print "%s" % getmsg("CFG1088I", "Exiting configuration script")
        sys.exit(1)


######################################################
# This is the last-ditch attempt to gather diagnostic
# information for unexpected failures.
######################################################

except SystemExit, exitcode:

    # Suppress trace for intentional system exit
    if exitcode.code == 0: sys.exit(0)

    # In case logging was never started, we need to dump the log buffer and set final exit code.
    sys.exit(1)

except ConfigurationException:

    _exctype, _excval, _exctb = sys.exc_info()

    # In case logging was never started, we need to dump the log buffer and set final exit code.
    print "Failed to create silent configuration file %s" % _exctb
    sys.exit(1)

except Exception:

    # In unlikely case an exception happened before imports were
    # completed, we need to do some minimal imports to insure
    # we can put out diagnostic information.
    import debugUtil as log
    from resourcebundle import getmsg as getmsg

    _exctype, _excval, _exctb = sys.exc_info()

    print
    try:
        print "%s" % getmsg("CFG1002E", "Unexpected exception detected in simpleconfig.py")
    except Exception:
        print "Unexpected exception detected in simpleconfig.py"
    print

    try:
        log.logException("Error in main routine")
    except Exception:
        _type, _val, _tb = sys.exc_info()
        print
        try:
            print "%s" % getmsg("CFG1003E", "Logging routine failed, traceback follows")
        except:
            print "Logging routine failed, traceback follows"
        print
        print
        print "Original traceback"
    #traceback.print_exception(_exctype,_excval,_exctb)
    #print

    # In case logging was never started, we need to dump the log buffer.
    log.set_logging()
    sys.exit(1)




