#!/bin/sh

#####################################################################
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2012 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################


# Set ITCAM_CONFIGHOME if undefined
if [ -z "$ITCAM_CONFIGHOME" ]; then
    TMP=`dirname "$0"`/..
    ITCAM_CONFIGHOME=`cd $TMP; pwd`
    export ITCAM_CONFIGHOME
fi

# Check ITCAM_CONFIGHOME is valid
if [ ! -f "$ITCAM_CONFIGHOME/bin/simpleconfig.sh" ]; then
    echo The ITCAM_CONFIGHOME environment variable is not defined correctly
    echo This environment variable is needed to run this program
    exit 1
fi

# Check if using javaless installer
if [ -d "${ITCAM_CONFIGHOME}/../../yndchome" ]; then
   AGENT_HOME=`cd ${ITCAM_CONFIGHOME}/../..;pwd -P`
else
   AGENT_HOME=`cd ${ITCAM_CONFIGHOME}/../../../..;pwd -P`
fi

# Check JAVA_HOME is valid
if [ -z "$JAVA_HOME" ]; then
    foundAltJRE=0
    for i in ${AGENT_HOME}/JRE/lx8266/jre ${AGENT_HOME}/JRE/aix526
    do
        if [ -x "$i/bin/java" ]; then
            foundAltJRE=1
            echo "*Warning*  JAVA_HOME was not set, using $i"
            export JAVA_HOME=$i
            break
        fi
    done
    if [ "$foundAltJRE" != "1" ]
    then
        echo "*Error*  JAVA_HOME was not set and we are unable to locate a valid default."
        exit 1
    fi

elif [ ! -x "$JAVA_HOME"/bin/java ]; then
    echo "*Error*  The JAVA_HOME specified was invalid.  Please correct and resubmit."
    exit 1
fi

echo Please note, if you have installed your Monitored WebSphere instance using non-root credentials, 
echo Ensure that this user has read and write access to files under $ITCAM_CONFIGHOME. 
echo For more information, see the Monitoring Agent for WebSphere Applications Installation Guide.

#
# TODO: Should we preserve user CLASSPATH and PYTHONPATH or reset?
#
if [ -z "$CLASSPATH" ]; then
    CLASSPATH=$ITCAM_CONFIGHOME/bin/jython/jython.jar:$ITCAM_CONFIGHOME/bin/lib/installbundle.jar
else
    CLASSPATH=$CLASSPATH:$ITCAM_CONFIGHOME/bin/jython/jython.jar:$ITCAM_CONFIGHOME/bin/lib/installbundle.jar
fi
if [ -z "$PYTHONPATH" ]; then
    PYTHONPATH=$ITCAM_CONFIGHOME/bin/lib
else
    PYTHONPATH=$PYTHONPATH:$ITCAM_CONFIGHOME/bin/lib
fi
PYTHONPATH=$PYTHONPATH:$ITCAM_CONFIGHOME/bin/lib


JAVA_CMD=$JAVA_HOME/bin/java
if [ "$(uname)" = "AIX" ]
then
 JAVA_OPTS="-Xms256m -Xmx256m"
else
 JAVA_OPTS=
fi
JAVA_PROPS="-Dpython.path=$PYTHONPATH "

echo
#echo Launching ITCAM configuration...
echo
#echo JAVA_HOME:        $JAVA_HOME
#echo
#echo ITCAM_CONFIGHOME: $ITCAM_CONFIGHOME
#echo
#echo CLASSPATH:        $CLASSPATH
#echo
#echo PYTHONPATH:       $PYTHONPATH
#echo
#echo Log file:         $ITCAM_LOG

# Execute the DC configurator
"$JAVA_CMD" \
  -classpath "$CLASSPATH" $JAVA_OPTS $JAVA_PROPS \
  org.python.util.jython \
  "$ITCAM_CONFIGHOME"/bin/simpleconfig.py $*

#get return code from the DC configurator execution
rc=$?

if [ -t 0 ]; then
    # jython turns echo off
    stty echo
fi
echo Please ensure the account that was used to run the application server has the read and write priviledges to DCHome/runtime directories.

#return the exit code
exit $rc
