#!/bin/sh

#####################################################################         
# Licensed Materials - Property of IBM
#  
# 5724-L62     
#
# (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved.      
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# 
##################################################################### 

# Note: ITCAM_CONFIGHOME is only needed for the logic of this script.
# The Jython program defaults if it is not specified.

# Set ITCAM_CONFIGHOME if undefined
if [ -z "$ITCAM_CONFIGHOME" ]; then
    temp_configname=$(dirname "$0")/..
    ITCAM_CONFIGHOME=$(cd $temp_configname; pwd)
    export ITCAM_CONFIGHOME
fi

# Check ITCAM_CONFIGHOME is valid
if [ ! -f "$ITCAM_CONFIGHOME/bin/unconfig.sh" ]; then
    echo "*Error*  The environment variable ITCAM_CONFIGHOME is not correctly defined."
    exit 1
fi

# Check if using javaless installer
if [ -d "${ITCAM_CONFIGHOME}/../../yndchome" ]; then
   AGENT_HOME=`cd ${ITCAM_CONFIGHOME}/../..;pwd -P`
else
   AGENT_HOME=`cd ${ITCAM_CONFIGHOME}/../../../..;pwd -P`
fi


# Check JAVA_HOME is valid
if [ -z "$JAVA_HOME" ]; then
    foundAltJRE=0
    for i in ${AGENT_HOME}/JRE/lx8266/jre ${AGENT_HOME}/JRE/aix526
    do
        if [ -x "$i/bin/java" ]; then
            foundAltJRE=1
            echo "*Warning*  JAVA_HOME was not set, using $i"
            export JAVA_HOME=$i
            break
        fi
    done
    if [ "$foundAltJRE" != "1" ]
    then
        echo "*Error*  JAVA_HOME was not set and we are unable to locate a valid default."
        exit 1
    fi

elif [ ! -x "$JAVA_HOME"/bin/java ]; then
    echo "*Error*  The JAVA_HOME specified was invalid.  Please correct and resubmit."
    exit 1
fi


# Add Jython and the message bundle to the class path
if [ -z "$CLASSPATH" ]; then
    CLASSPATH=$ITCAM_CONFIGHOME/bin/jython/jython.jar:$ITCAM_CONFIGHOME/bin/lib/installbundle.jar
else
    CLASSPATH=$ITCAM_CONFIGHOME/bin/jython/jython.jar:$ITCAM_CONFIGHOME/bin/lib/installbundle.jar:$CLASSPATH
fi

# Set PYTHONPATH to our library
PYTHONPATH=$PYTHONPATH:$ITCAM_CONFIGHOME/bin/lib

# Set Java options
JAVA_EXE=$JAVA_HOME/bin/java
if [ "$(uname)" = "AIX" ]; then
    JAVA_OPTS="-Xms256m -Xmx256m"
else 
    JAVA_OPTS=
fi
JAVA_PROPS="-Dpython.path=$PYTHONPATH"

JAVA_CMD="$JAVA_EXE -classpath $CLASSPATH $JAVA_OPTS $JAVA_PROPS org.python.util.jython $ITCAM_CONFIGHOME/bin/unconfig.py $*"

#echo
#echo "Launching ITCAM unconfiguration..."
#echo
#echo "JAVA_HOME:        $JAVA_HOME"
#echo
#echo "ITCAM_CONFIGHOME: $ITCAM_CONFIGHOME"
#echo
#echo "CLASSPATH:        $CLASSPATH"
#echo
#echo "PYTHONPATH:       $PYTHONPATH"
#echo
#echo "Log file:         $ITCAM_LOG"
#echo
#echo "Command:          $JAVA_CMD"

echo
echo

# Execute the DC unconfigurator
$JAVA_CMD
rc=$?

# In some cases where Jython does not terminate correctly it leaves echo turned off.
# If file descriptor 0 is open and allocated to the terminal, make sure echo is back on.
if [ -t 0 ]; then
    stty echo
fi

# Return the exit code from Jython as this script's exit code.
exit $rc
