#!/bin/sh
# THIS PRODUCT CONTAINS RESTRICTED MATERIALS OF IBM
# 5724-i63, 5724-H88 (C) COPYRIGHT International Business Machines Corp., 2011
# All Rights Reserved * Licensed Materials - Property of IBM
# US Government Users Restricted Rights - Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.

curdir=`pwd`

# process args looking for --options filename
args=$@

#default is config
action="config"

while : 
do
    case $1 in
        --options) shift; optfile="$1";;
        --unconfig) action="unconfig";;
        "") break;;
    esac
    shift
done

if [ "$optfile" == "" ]
then
    optfile="dev-edition.properties";
fi

if [ ! -f $optfile ]
then
    echo "Option file $optfile not found... please check and try again"
    exit
fi

WAS_HOME=`grep appserver.basedir $curdir/$optfile | cut -d= -f2`
DC_HOME=`grep itcamdc.home $curdir/$optfile | cut -d= -f2`

JAVA_EXE=${WAS_HOME}/java/jre/bin/java
CONFIG_DC_HOME=${DC_HOME}/config_dc
LOG_CYN_DIR=${DC_HOME}/logs/CYN/logs

if [ ! -d ${LOG_CYN_DIR} ]
then    
    mkdir -p ${LOG_CYN_DIR}
fi

LOG_CYN_FILE=${LOG_CYN_DIR}/trace_config.logs

CLASSPATH=./config_DC.jar

start_msg() {
	echo "[INFO] Launching ITCAM for WebSphere Application Server Data Collector Configuration Tool"
}

start_msg

cd "${CONFIG_DC_HOME}"

"${JAVA_EXE}" \
-classpath ${CLASSPATH} \
com.ibm.tivoli.itcam.install.was.de.PreConfigurator $args > "${LOG_CYN_FILE}"
retcode=$?
if [ "$retcode" -ne "0" ]; then
  echo "[ERROR] PreConfigurator step ended with returncode $retcode; cannot continue"
  exit 1
fi

"${JAVA_EXE}" \
-Djlog.propertyFileDir.CYN="${DC_HOME}/etc" \
-jar config_DC.jar $args > "${LOG_CYN_FILE}"

retcode=$?
if [ "$retcode" -ne "0" ]; then
	if [ "$action" == "config" ]
    then
    	echo "[ERROR] Configuration failed with returncode $retcode; cannot continue"	
    else
        echo "[ERROR] Unconfiguration failed with returncode $retcode; cannot continue"	
    fi
    echo "[ERROR] See logs in ${LOG_CYN_DIR} for more details"
    exit 1
else
    if [ "$action" == "config" ]
    then
        echo "[INFO] Configuration was successful. Restart the server to activate the new configuration"
    else
        echo "[INFO] Unconfiguration was successful. Restart the server to activate the new configuration"
    fi
    exit 0
fi

# go back to where we started from
cd $curdir
