#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#
# * The source code for this program is not published or otherwise
# * divested of its trade secrets, irrespective of what has been
# * deposited with the US Copyright Office.
# *
#  @(#)48 1.7 itcam.dc/ppe.probe_ws/scripts/createItcamTemplate.py, am_ppe.probe_ws, itcam_wr_720, 201108112205 10/22/09 17:31:39
#
#  MAINT="F197369, 198667, 200142, 201758, 202494, 202905, 202905.1"
#
#
# Script for creating WebSphere XD default template
# with ITCAM DataCollector pre-defined 
#
# 
#
#

import sys
import java

nl = java.lang.System.getProperty("line.separator")
pathsep = java.lang.System.getProperty("path.separator")
filesep = java.lang.System.getProperty("file.separator")

startingOverlayTCPPort = 0
startingOverlayUDPPort = 0
endpoints = []
serverEntryTemplates = []
overlayTCPEndpointName = "OVERLAY_TCP_LISTENER_ADDRESS"
overlayUDPEndpointName = "OVERLAY_UDP_LISTENER_ADDRESS"
foundTCP = 0
foundUDP = 0
hostName = "*"
dcInputsFile = ""
templateRespFile = ""
itcamhome=""
inputs={}

def loadDcInputs(filepath):
    try:
        f = open(filepath)
    except IOError:
        print 
        print "Inputs file: %s not found; please check and retry" % (filepath)
        print
        print "Please note that there is a restriction on using wsadmin on the windows"
        print " platform that requires pathnames to use either forward slashes or"
        print " doubled backslashes. For example:"
        print
        print "     c:/path_to_file/template_inputs.txt"
        print "        or"
        print "     c:\\\\path_to_file\\\\template_inputs.txt"
        print 
        sys.exit(0)
        
    lines = f.readlines()

#---------------------------------
#
# on z/OS the input file will be in ebcdic
# so convert to unicode
# (use lines[0] as input to the unicode function since readline will
# return only a single "line" since there are no newlines
# to process)
#
# then we have to split on the \x85 (\U0085) which is
# the unicode "Next Line" character
# This results in an array of lines
#
#---------------------------------
    osversion=java.lang.System.getProperty("os.name")
    if osversion == "z/OS":
        lines=unicode(lines[0],'cp037')
        lines=lines.split("\x85")
    
    for line in lines:
        try:
            key,val = line.strip().split("=")
            inputs[key]=val
        except:
            pass

def validateInputs():
    abortFlag=0
    try:
        inputs["JDK_VERSION"]
    except:
        print "JDK_VERSION is missing"
        print "should be one of 14, 15, 16, representing Java 1.4.x, JDK5, JDK6"
        abortFlag=1

    try:
        inputs["JDK_TYPE"]
    except:
        print "JDK_TYPE is missing"
        print "should be ibm or sun"
        abortFlag=1

    try:
        inputs["APPSERVER_PLATFORM"]
    except:
        print "APPSERVER_PLATFORM is missing"
        print "should be WebSphere shortname, for example, was60, was61, was7, etc"
        abortFlag=1

    try:
        inputs["OS"]
    except:
        print "OS is missing"
        print "should be unix, HP, windows, or aix"
        abortFlag=1

    try:
        inputs["ITCAMDCHOME"]
    except:
        print "ITCAMDCHOME is missing"
        print "should be the path to the ITCAM home directory"
        abortFlag=1

    try:
        inputs["TOOLKIT_LIB_PATH"]
    except:
        print "TOOLKIT_LIB_PATH is missing"
        print "should be the path to the directory containing ITCAM native library"
        abortFlag=1

    if abortFlag == 1:
        print ""
        print "Missing one or more values from template inputs; see earlier messages."
        print "The template cannot be created. Please add the missing value(s) and retry."
        sys.exit(-1)

def printDcInputs():
    for key in inputs.keys():
        print "%s = %s" % (key, inputs[key])
    print "... done"


startingOverlayTCPPort = 11001
startingOverlayUDPPort = 11002

argc = len(sys.argv)

print ""
print ""
print "Creating the ITCAM WebSphere XD Dynamic Cluster Template"
print ""
print "This template can be used to as the basis for a Dynamic Cluster definition"
print "Server instances created based on this template will be preconfigured"
print " to include the ITCAM Datacollector"
print ""


if len(sys.argv) < 1:
    print
    print "You must supply the path to valid template response file"
    print
    print "syntax is:"
    print "createITCAMtemplate.py template_response_file <dcInput.txt>"
    print
    print "template_response_file is a standard text file"
    print "containing at minimum the following properties:"
    print
    print "ITCAMDCHOME=<itcam installation dir>, for example: C:/IBM/ITM/TMAITM6/wasdc/7.1"
    print "APPSERVER_PLATFORM=<websphere version abbrev>,  for example: was61, was7, etc"
    print "OS=<operating system>, must be one of unix,aix,linux,HP,windows"
    print "JDK_VERSION=<jdk version>, for example 13, 14, 15"
    print "JDK_TYPE=<jdk vendor>, for example ibm, sun"
    print "TOOLKIT_LIB_PATH=<ITCAMDCHOME>/toolkit/lib/<arch>, arch is for ex, win32, sol283, etc"
    print
    print "<dcInput.txt> is the response file created by itcam configuration"
    print "  and must be available at runtime"
    print "  if <dcInput.txt> is already available it can be used"
    print "  as the template_response_file"
    print
    print
    print "  please see product documentation for more information"
    sys.exit(0)
elif argc == 1:
    templateRespFile = sys.argv[0]
    print 
    print "Using template inputs file: ",templateRespFile
    print 
    print
elif argc == 2:
    templateRespFile = sys.argv[0]
    dcInputsFile = sys.argv[1]
    print
    print "Using template inputs file: ",templateRespFile
    print
    print "Specified runtime dcInputs file is: ",dcInputsFile
    print 
    print "Note: the specified file must exist in the <itcam_home>/runtime directory when"
    print " servers configured using this template are started"
    print
    
elif argc > 3:    
    print "Using provided values for Overlay Ports"
    startingOverlayTCPPort = sys.argv[1]
    startingOverlayUDPPort = sys.argv[2]


loadDcInputs(templateRespFile)
validateInputs()


# ---------------------------------------------

def getItcamHome():
    try:                               # the replaces below fix some common things seen in dcInputs
        home=inputs["ITCAMDCHOME"]   
        home=home.replace('\\','/')    # set to forward slash
        home=home.replace('//','/')    # covers "\\\\" above
        home=home.replace('\:',':')    # un-escape colon
        home=home.replace('/:',':')    # un-escape colon (covers c\\: -> c/: from above)
        return home
    except:
        print "Encountered exception retrieving ITCAMDCHOME, cannot continue"
        sys.exit(-1)


    
def getVersion():
    objName = AdminControl.completeObjectName('WebSphere:type=Server,*')
    res = AdminControl.getAttributes(objName,'[platformVersion]')
    vers = res.replace('[','')
    vers = vers.replace(']','')
    vers = vers.split(' ')[1]
    vers = vers.replace('.','')
    print 'was'+vers

def buildNativeLibString(libsuf=None):
    if libsuf==None:
        libsuf=''
    jdk = getJDKVersion()
    jdktype = getJDKType()
    
    if jdk == "14":
        return "-Xrunam_%s_%s%s:%s/runtime/${WAS_SERVER_NAME}/" % (jdktype, jdk,libsuf, itcamhome)
    else:
        return "-agentlib:am_%s_%s%s=%s/runtime/${WAS_SERVER_NAME}/" % (jdktype, jdk, libsuf, itcamhome)

def buildVerboseGCString():
    jdktype = getJDKType()

    if jdktype == "ibm":
        return "-Xverbosegclog:${SERVER_LOG_ROOT}/itcam_gc.log,5,3000"
    else:
        return "-XX:+PrintGCTimeStamps -verbosegc -Xloggc:${SERVER_LOG_ROOT}/itcam_gc.log"


def getAppServerPlatformString():
    try:
        return inputs["APPSERVER_PLATFORM"]
    except:
        print "Encountered exception retrieving APPSERVER_PLATFORM, cannot continue"
        sys.exit(-1)

def getPathEnvName():
    try:
        ostype=inputs["OS"]
        if ostype == "HP":
            return "SHLIB_PATH"
        elif ostype == "windows":
            return "PATH"
        elif ostype == "zos":
            return "LIBPATH"
        else:
            return "LD_LIBRARY_PATH"
    except:
        print "Encountered exception retrieving OS, cannot continue"
        sys.exit(-1)


def getJDKVersion():
    try:
        jdk=inputs["JDK_VERSION"]
        return jdk
    except:
        print "Encountered exception retrieving JDK_VERSION, cannot continue"
        sys.exit(-1)

def getJDKType():
    try:
        type=inputs["JDK_TYPE"]
        return type
    except:
        print "Encountered exception retrieving JDK_TYPE, cannot continue"
        sys.exit(-1)
                
def getToolkitLibPath():
    try:                               # the replaces below fix some common things seen in dcInputs
        type=inputs["TOOLKIT_LIB_PATH"]
        type=type.replace('\\','/')    # set to forward slash
        type=type.replace('//','/')    # covers "\\\\" above
        type=type.replace('\:',':')    # un-escape colon
        type=type.replace('/:',':')    # un-escape colon (covers c\\: -> c/: from above)
        return type
    except:
        print "Encountered exception retrieving TOOLKIT_LIB_PATH, cannot continue"
        sys.exit(-1)

def buildJvmArgs(libsuf=None):
    native = buildNativeLibString(libsuf)
    bootcp = "-Xbootclasspath/p:%s/toolkit/lib/bcm-bootstrap.jar%s%s/itcamdc/lib/ppe.probe-bootstrap.jar" % (itcamhome,pathsep,itcamhome)
    verbosegcString = buildVerboseGCString()
    propString = "-Ditcamdc.dyncluster=true -Dam.probe.stdout=${SERVER_LOG_ROOT}/native_stdout.log"
    jvmargs = bootcp + " " + native + " " + verbosegcString + " " + propString
    if len(dcInputsFile) > 0:
        workFilename = dcInputsFile.replace('\\','/')    # replace any \\ with /
        workFNtokens = workFilename.split('/')
        workFilename = workFNtokens[len(workFNtokens)-1] # last token is the basename
        jvmargs = jvmargs + " -Ditcamdc.inputs=" + workFilename

    return jvmargs

def createEndPoint(serverTemplate, endPointName, endPointHost, endPointPort):
    attrs = []
    namedEndpointAttrs = ["endPointName", endPointName]
    attrs.append(namedEndpointAttrs)
    namedEndPoint = AdminConfig.create("NamedEndPoint", serverTemplate, attrs)
    attrs = []
    hostAttr = ["host", endPointHost]
    portAttr = ["port", endPointPort]
    attrs.append(hostAttr)
    attrs.append(portAttr)
    try:
       newEndPoint = AdminConfig.create("EndPoint", namedEndPoint, attrs)
    except:
        print "Exception while creating new end point"
    
def convertToList(inlist):
     outlist=[]
     if (len(inlist)>0 and inlist[0]=='[' and inlist[len(inlist)-1]==']'):
        inlist = inlist[1:len(inlist)-1]
        clist = inlist.split(" ")
     else:
        clist = inlist.split("\n")
     for elem in clist:
        elem=elem.rstrip();
        if (len(elem)>0):
           outlist.append(elem)
     return outlist

def getConfigIdForTemplate(templateName):
   templates=convertToList(AdminConfig.listTemplates("Server"))
   for template in templates:
       cname = AdminConfig.showAttribute(template,"name")
       if (cname == templateName):
          return template
   return None
   
def getServerType(serverTemplate):
    tokens = serverTemplate.split("/")
    idx = 0
    for token in tokens:
#      serverTemplate is of the form 'itcam(templates/servertypes/APPLICATION_SERVER/servers/itcam|server.xml#Server_1210548827522)'
#      return third token 
       if (idx == 2): 
         return token
       idx=idx+1
    return None

print "creating xd template"
template=getConfigIdForTemplate("itcam")
if (template == None):
   AdminTask.createTemplateFromTemplate('[-sourceTemplate default -destinationTemplate itcam -description "WebSphere XD Default Template"]')
   template=getConfigIdForTemplate("itcam")
   threadpools=convertToList(AdminConfig.list("ThreadPool",template))
   for threadpool in threadpools:
      tname = AdminConfig.showAttribute(threadpool,"name")
      if (tname == "WebContainer"):
         AdminConfig.modify(threadpool,[['maximumSize','175']])
else:
   print "template already exists, skipping"


print "creating xd ZOS template"
template=getConfigIdForTemplate("itcamZOS")
if (template == None):
   AdminTask.createTemplateFromTemplate('[-sourceTemplate defaultZOS -destinationTemplate itcamZOS -description "WebSphere XD ZOS Default Template"]')
   template=getConfigIdForTemplate("itcamZOS")
   threadpools=convertToList(AdminConfig.list("ThreadPool",template))
   for threadpool in threadpools:
      tname = AdminConfig.showAttribute(threadpool,"name")
      if (tname == "WebContainer"):
         AdminConfig.modify(threadpool,[['maximumSize','175']])
else:
    print "template already exists, skipping"

print "creating xd ZOS 64bit template"
template=getConfigIdForTemplate("itcamZOS64")
if (template == None):
   AdminTask.createTemplateFromTemplate('[-sourceTemplate defaultZOS -destinationTemplate itcamZOS64 -description "WebSphere XD ZOS 64bit Default Template"]')
   template=getConfigIdForTemplate("itcamZOS64")
   threadpools=convertToList(AdminConfig.list("ThreadPool",template))
   for threadpool in threadpools:
      tname = AdminConfig.showAttribute(threadpool,"name")
      if (tname == "WebContainer"):
         AdminConfig.modify(threadpool,[['maximumSize','175']])
else:
   print "template already exists,skipping"

setname = ""

serverEntryTemplates = convertToList(AdminConfig.listTemplates("ServerEntry"))
for serverEntryTemplate in serverEntryTemplates:
    setname = AdminConfig.showAttribute(serverEntryTemplate,"serverName")
    foundTCP = 0
    foundUDP = 0
    if ((setname == "itcam") or (setname == "itcamZOS") or (setname == "itcamZOS64")):
        endpoints = convertToList(AdminConfig.showAttribute(serverEntryTemplate,"specialEndpoints"))
        for endpoint in endpoints:
            epname = AdminConfig.showAttribute(endpoint,"endPointName")
            if (epname == overlayTCPEndpointName):
                foundTCP = 1
            if (epname == overlayUDPEndpointName):
                foundUDP = 1

        if (foundTCP == 0):
           createEndPoint(serverEntryTemplate, overlayTCPEndpointName, hostName, startingOverlayTCPPort)

        if (foundUDP == 0):
           createEndPoint(serverEntryTemplate, overlayUDPEndpointName, hostName, startingOverlayUDPPort)

# ------------------------------------------------------

print "Templates created, applying ITCAM customizations"

itcamhome = getItcamHome()
args = buildJvmArgs()
args64 = buildJvmArgs('_64')      # get 64 bit version of native lib in args
plat = getAppServerPlatformString()
toolkitlib = getToolkitLibPath()
pathenvname = getPathEnvName()

print "appserver.platform is ", plat
print "genericJVMargs are: ", args
print "genericJVMargs for 64 bit are: ", args64
print "path env variable name is: ", pathenvname
           
serverTemplates = convertToList(AdminConfig.listTemplates("Server"))
for serverTemplate in serverTemplates:
    serverType = getServerType(serverTemplate)
    serverTemplateName = AdminConfig.showAttribute(serverTemplate,"name")
    if (((serverTemplateName == "itcam" or serverTemplateName == "itcamZOS" or serverTemplateName == "itcamZOS64") and 
         (serverType == "APPLICATION_SERVER"))):
       procDefs = convertToList(AdminConfig.showAttribute(serverTemplate, "processDefinitions"))
       for procDef in procDefs:
           if (procDef.find("JavaProcessDef") != -1):
               # for itcamZOS or itcamZOS64 template, skip anything that isn't a Servant
               if (serverTemplateName == "itcamZOS" or serverTemplateName == "itcamZOS64"):
                   processType = AdminConfig.showAttribute(procDef,"processType")
                   if (processType != "Servant"):
                       continue
               
               jvmEntries = AdminConfig.showAttribute(procDef, "jvmEntries")
               if (len(jvmEntries)>0 and jvmEntries[0]=='[' and jvmEntries[len(jvmEntries)-1]==']'):
                   jvmEntries = jvmEntries[1:len(jvmEntries)-1]

               genericArgs = AdminConfig.showAttribute(jvmEntries, "genericJvmArguments")
               if serverTemplateName == "itcamZOS64":
                   genericArgs = args64 + " " + genericArgs
               else:
                   genericArgs = args + " " + genericArgs
               AdminConfig.modify(jvmEntries, [["genericJvmArguments", genericArgs]])
               name=['name',pathenvname]
               value=['value',toolkitlib]
               enventry=[name,value]
               AdminConfig.modify(procDef,[['environment',[enventry]]])

print "Customizations complete; saving workspace"
AdminConfig.save()
print "Finished."

