#!/bin/sh

#    Licensed materials property of IBM
#
#    5724-L62
#
#    (C) Copyright IBM Corp. 2011, 2013  All rights reserved.
#
#    US Government Users Restricted Rights - Use, duplication or disclosure
#    restricted by GSA ADP Schedule Contract with IBM Corp.

if [ $# = 0 ]; then
  echo " "
  echo "*Error*  No command line arguments provided for offlineCollector.sh"
  echo " "
  echo "         At least the WAS_HOME directory must be provided as"
  echo "         the first parameter of the offlineCollector.sh script."
  echo " "
  echo "         Please note that all parameters are case-sensitive unless"
  echo "         explicitly specified otherwise."
  echo " "
  echo "         Subsequent arguments are specified as key=value pairs"
  echo "         on the command line and in any file specified by the"
  echo "         input= keyword parameter:"
  echo " "
  echo "    outfile=<filename> - The file name of the output JSON file."
  echo "                         To avoid problems a complete path should be used."
  echo "                         The directory which will contain this file must"
  echo "                         already exist.  The file will be written by the"
  echo "                         ITCAM Data Collector running within the WebSphere"
  echo "                         Application Server.  The offline collector will"
  echo "                         prompt you for a value if this is not otherwise"
  echo "                         supplied."
  echo " "
  echo "    userid=<WAS userid> - Use if and only if global security is enabled."
  echo "    password=<WAS password> - Use if and only if global security is enabled."
  echo " "
  echo "    port=<connection port> - Connection port number, default is 8880."
  echo "                         This is required if global security is enabled"
  echo "                         and the default is not adequate."
  echo "    conntype=<SOAP|RMI> - Connection type defaults to SOAP."
  echo "    host=<name or IP address> - Where WAS is running, default is localhost."
  echo " "
  echo "    cell=<cell>        - The cell name is not required, use for validation."
  echo "    node=<node>        - Usually not required.  May be needed for ND."
  echo "    server=<server>    - Usually not required.  May be needed for ND."
  echo " "
  echo "    remoteKeyName=<name> - Needed only when global security is enabled"
  echo "                         and the remote key store name is not the usual"
  echo "                         default of NodeDefaultTrustStore."
  echo " "
  echo "    trustStore=<trust file> - Not required, default is to create and sign a"
  echo "                         temporary trust store file.  If specified it must"
  echo "                         be the full path to the WAS trust store file or a"
  echo "                         copy of that file."
  echo "    trustStorePassword=<trust password> - Not required, needed only when the"
  echo "                         trust store is specified and the password for"
  echo "                         that file is not the default of WebAS."
  echo " "
  echo "    input=<filename>   - May be used to specify a file containing an"
  echo "                         alternative source of key/value pairs."
  echo " "
  exit 1
fi

# Go to the directory containing the script and check for the Jar file

cd $(dirname $0)

if [ ! -e ../lib/ext/gdcclient.jar ]; then
  echo " "
  echo "*Error*  Unable to locate GDC client Jar file"
  echo " "
  exit 1
fi

# Make sure the WAS_HOME is somewhat valid

if [ ! -d $1 ]; then
  echo " "
  echo "*Error*  Not a valid directory: $1"
  echo "         First argument must be the WAS_HOME directory"
  echo " "
  exit 1
fi

export WAS_HOME=$1

# Locate a Java program in WebSphere

JAVA_EXE=""
[ -x $WAS_HOME/java/bin/java ] && JAVA_EXE=$WAS_HOME/java/bin/java
[ -x $WAS_HOME/java/jre/bin/java ] && JAVA_EXE=$WAS_HOME/java/jre/bin/java

if [ -z "$JAVA_EXE" ]; then
  echo " "
  echo "*Error*  No executable Java program found within supplied WAS_HOME"
  echo "         Is $WAS_HOME a valid WebSphere home?"
  echo " "
  exit 1
fi

# Accumulate the remaining parameters

parms="env=was"
while [ -n "$2" ]; do
   parms="$parms $2"
   shift
done

# Execute the program in the Jar file

${JAVA_EXE} -jar ../lib/ext/gdcclient.jar $parms
