#!/bin/sh
# *************************************************************************
#
# Licensed Materials - Property of IBM
#
# 5724-CYN
#
# (C) Copyright IBM Corp. 2005,2014  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *************************************************************************
# @(#)49 1.19.1.21 itcam.dc/ppe.probe/bin/cyn_collector.sh, itcam.dc-ppe.probe, itcam_wr_720, 201108112205 3/10/11 09:34:05
# *************************************************************************
# ######################################################################################
# ######################################################################################
# ######################################################################################
# 
#                               F U N C T I O N S
#
# ######################################################################################
# ######################################################################################
# ######################################################################################

# ##############################################
#    Sends a string to the screen and the log
# ##############################################
print_syntax ()
{
  cat << EOF

   $CYN_COLLECTOR       Version $VERSION

Command line syntax is:

   $CYN_COLLECTOR [[-a] | [-s server]]

   where..
            a - Collect information from all App.Servers configured by this DC installation.
       server - Name of the only Application Server you want to collect
                information from.
EOF

  exit 1
}

#Removes duplicate logs from a profile
#Params: the log dir of the profile,  server name, target log directory
remove_extra_logs()
{
	echo in remove_extra_logs
	for i in `ls $1 | grep -v ffdc`
	do
		if  [ "$i" != "$2" ] && [ -d "$1/$i" ]
		then
			rm -rf $3/$(basename $1)/$i
		fi
	done

	for i in `ls $3/$(basename $1)/ffdc | grep -v $(basename $2)`
	do
		rm -rf $3/$(basename $1)/ffdc/$i
	done
}
# ######################################################################################
# ######################################################################################
# ######################################################################################
# 
#                             E N D   O F   F U N C T I O N S
#
# ######################################################################################
# ######################################################################################
# ######################################################################################


# ###############################
#   Define Basic Variables
# ###############################
# Version of this tool
VERSION=7.3.0.10.0
#VERSION=@cdc.version@

# This tool's log
TOOL_LOGNAME=cyn_collector.log

# Store this tool's name for global references
CYN_COLLECTOR=`basename $0`
CYNCOL_RELATIVE_PATH=`dirname $0`
CYNCOLLECTOR_PATH=`cd $CYNCOL_RELATIVE_PATH; pwd`
CURR_DIR=`pwd`
PROD_TYPE=
ZERO="0"
CONFIG="config"

PATH="/usr/bin:/usr/sbin:/usr/ucb:$PATH"

echo "============================"
echo " $CYN_COLLECTOR v$VERSION"
echo "============================"



# ####################################################
#   Define product's main directory 
# ####################################################

# test -z "$CYNCOL_RELATIVE_PATH" && CYNCOL_RELATIVE_PATH=.

if [ -d "$CYNCOL_RELATIVE_PATH"/../../bin ]
then
  echo "AD 7.2 (or higher) detected"
  cd $CYNCOL_RELATIVE_PATH/../..
  PROD_TYPE="AD72"
elif [ -d "$CYNCOL_RELATIVE_PATH"/../wasdc ]
then
  echo "ITCAM detected"
  cd $CYNCOL_RELATIVE_PATH/../wasdc
else
  echo "eCAM detected"
  eCAM=1
  cd $CYNCOL_RELATIVE_PATH/../../../DC
fi

# This is AD's path
PRODUCT_PATH=`pwd`

# ##########################################
#        Support for old AD versions
# ##########################################

cyn_tmp=`cd $PRODUCT_PATH/../../; pwd`
PLATFORM_BASE=`basename $cyn_tmp`
ITM_HOME=`cd $PRODUCT_PATH/../../..; pwd`

# ##################################################
#     Define the path for collection and logging
# ##################################################

if [ "$PROD_TYPE" = AD72 ]
then
   DATADIR=$CYNCOLLECTOR_PATH/../../data 
   
   if [ -w "$DATADIR" ]
   then
     CYN_TRGT_HOME_BASE=$DATADIR
   else
     CYN_TRGT_HOME_BASE=`~`
   fi

   CYN_TRGT_HOME=`cd $CYN_TRGT_HOME_BASE; pwd`/CYN
   
elif [ -z "$ITM_HOME" -o "$ITM_HOME" = "/" ] 
then
  CYN_TRGT_HOME=${ITM_HOME}tmp/CYN
else
  CYN_TRGT_HOME=$ITM_HOME/tmp/CYN
fi

#create the target directory
if [ ! -d $CYN_TRGT_HOME ]
then
  mkdir -p $CYN_TRGT_HOME || exit 1
fi

# Define this tool's log file
CYNCOLLOG=$CYN_TRGT_HOME/$TOOL_LOGNAME
echo "Tool information is being logged in file:
  $CYNCOLLOG"
  
# ############################
#    Initialize part of OS-related variables
# ############################

# Determine the platform
echo "Initializing part of OS-related variables"  >> $CYNCOLLOG
OS_TYPE=`uname -s`

if [ "$OS_TYPE" = "OS400" ]
then
  echo "This is i5"  >> $CYNCOLLOG
  HOST_NAME=`hostname`
  OS_VERSION=`uname -v`
  OS_RELEASE=`uname -r`
  OS_VERSION=V${OS_VERSION}R${OS_RELEASE}
else  
  HOST_NAME=`uname -n`
  OS_VERSION=`uname -r`
fi

if [ "$OS_TYPE" = "AIX" ]
then
  OS_VERSION=`uname -v`
  OS_RELEASE=`uname -r`
  OS_VERSION=${OS_VERSION}.${OS_RELEASE}
  OS_ARCH=`uname -p`
else
  OS_ARCH=`uname -m`
fi

# ###############################
#           Clean up
# ###############################
# cleaning last log
cat /dev/null > $CYNCOLLOG

# ###############################
#  define Collection's target 
#  and work directory
# ###############################
# set timestamp to provide unique name to data collection
TIMESTAMP=`date +%Y%m%d%H%M%S`
if [ "$OS_TYPE" = OS400 ]
then
  I5_HOSTNAME=`uname -n`
  CYN_WRKDIR=$I5_HOSTNAME-CYNenv
else
  CYN_WRKDIR=$HOST_NAME-CYNenv
fi

# the target dir can be set by the 2nd input argument
if [ -n "$2" ]; then
   if [ -d "$2" ]; then
      CYN_TRGT_HOME=$2
   else
      CYN_WRKDIR=$CYN_WRKDIR-$TIMESTAMP
      echo "Incorrect DC target directory specified as parameter: $2"
      echo "Setting DC target directory to: $CYN_TRGT_HOME"
   fi
else
    CYN_WRKDIR=$CYN_WRKDIR-$TIMESTAMP
fi


# ###############################
#       Handle Parameters 
# ###############################

  # Extract options passed as parameters

  # Name of the only App.Server we want to collect information from
  APP_SERVER_NAME=
  # Switch to indicate that we want to collect info from all servers
  ALL_SERVERS=

  while getopts ":as:" opt; do
      case $opt in
        a )
           ALL_SERVERS="Y";;
        s )
           APP_SERVER_NAME=$OPTARG;;
        ? )
           print_syntax
           exit 0;;
      esac
  done

  echo "ALL_SERVERS=$ALL_SERVERS
  APP_SERVER_NAME=$APP_SERVER_NAME
  " >> $CYNCOLLOG

  # Vaidate the syntax

  if [ -n "$ALL_SERVERS" ]
  then
    if [ -n "$APP_SERVER_NAME" ]
    then
      echo "...ERROR: You can not specify all servers (-a) and a single"
      echo "          server (-s) at the same time."
      print_syntax
      exit 1
    fi
  fi

  hasBeenConfigured=
  InputFilesdir=$PRODUCT_PATH/runtime
  if [ -d "$InputFilesdir" ]
  then
    InputFilesFound=`ls $InputFilesdir/*.input.properties 2>/dev/null`
    echo "InputFilesFound=$InputFilesFound" >> $CYNCOLLOG
  
    if [ -n "$InputFilesFound" ]
    then
      echo "DC contains *.input.properties files" >> $CYNCOLLOG
      hasBeenConfigured=true
    else
      echo "DC does not contain *.input.properties files" >> $CYNCOLLOG
    fi
  else
    echo "DC has never been configured..." >> $CYNCOLLOG
  fi

  if [ -z "$hasBeenConfigured" ]
  then
    if [ -n "$ALL_SERVERS" -o -n "$APP_SERVER_NAME" ]
    then
      echo "...ERROR: You can not specify all servers (-a) and/or a single"
      echo "          server (-s) when ITCAMfWAS has not been configured."
      echo "!!! ERROR: Parameters (-a) and/or (-s) were used with an unconfigured DC." >> $CYNCOLLOG
      print_syntax
      exit 1
    fi
  fi
  
  echo "!!! Valid parameters detected...
  " >> $CYNCOLLOG

# #########################################################################
# #########################################################################
# #########################################################################
#
#                             Declarations
#
# #########################################################################
# #########################################################################
# #########################################################################
#                              Variables
# #########################################################################

# Starting the log
echo "======================================================================
Executing $CYN_COLLECTOR v$VERSION on `date`
With the following parameters: $@
======================================================================
------------------------------------
         Main Definitions...              
------------------------------------
CYN_COLLECTOR=$CYN_COLLECTOR
CYNCOL_RELATIVE_PATH=$CYNCOL_RELATIVE_PATH
CYNCOLLECTOR_PATH=$CYNCOLLECTOR_PATH
CURR_DIR=$CURR_DIR
PRODUCT_PATH=$PRODUCT_PATH
CYN_TRGT_HOME=$CYN_TRGT_HOME
CYNCOLLOG=$CYNCOLLOG" >> $CYNCOLLOG

COMDIR=

# #################################
#   Verifying PRODUCT_PATH
# #################################
# If PRODUCT_PATH is not a directory flag the error
if [ ! -d "$PRODUCT_PATH" ]
then
  echo "Error: $PRODUCT_PATH does not appear to be a valid AD directory."
  echo "!!! ERROR: $PRODUCT_PATH does not appear to be a valid AD directory." >> $CYNCOLLOG
  exit 1
# If PRODUCT_PATH is not readable flag the error
elif [ ! -r "$PRODUCT_PATH" ]
then
  echo "Error: $PRODUCT_PATH is not readable."
  echo "!!! ERROR: $PRODUCT_PATH is not readable." >> $CYNCOLLOG
  exit 1
else
  echo "PRODUCT_PATH=$PRODUCT_PATH"  >> $CYNCOLLOG
  echo ""  >> $CYNCOLLOG
  echo "$PRODUCT_PATH seems like a valid AD path." >> $CYNCOLLOG
fi

# #########################################################################
#               Setup the environment & required directories
# #########################################################################

# ##########################################
#
#        Support for old AD versions
#
# ##########################################
if [ "$PLATFORM_BASE" = "sol293" ]
then
  DC71_JRE="sol283"
elif [ "$PLATFORM_BASE" = "sol603" ]
then
  DC71_JRE="sol606"
elif [ "$PLATFORM_BASE" = "aix533" ]
then
  DC71_JRE="aix523"
else
  DC71_JRE=$PLATFORM_BASE
fi
# ##########################################

# ##########################################
# For i5, determine if the right tar command is present
# ##########################################

if [ "$OS_TYPE" = "OS400" ]
then
  echo "This is i5, so we'll check for the right tar command"  >> $CYNCOLLOG
  TAR_CMD=/QOpenSys/usr/bin/tar
  
  if [ -f "$TAR_CMD" ]
  then
    echo "  tar was found..." >> $CYNCOLLOG

    if [ -x "$TAR_CMD" ]
    then
      echo "  tar is executable..." >> $CYNCOLLOG
    else
      echo "  tar command is not executable." >> $CYNCOLLOG
      TAR_CMD=
    fi
  else
    echo "  tar command was not found." >> $CYNCOLLOG
    TAR_CMD=
  fi
fi

# ##########################################
# Find Java
# ##########################################

if [ "$PROD_TYPE" = AD72 ]
then
  echo "This is AD7.2 (or above)"  >> $CYNCOLLOG
  
  if [ -z "$TAR_CMD" ]
  then
    echo "Tar command will not be used, defining the location for java"  >> $CYNCOLLOG

    # Check whether JAVA_HOME is already set
    if [ ! -z "$JAVA_HOME" ]
    then
      echo "JAVA_HOME is not empty... we are going to use it"  >> $CYNCOLLOG
      JAVA_EXE=$JAVA_HOME/bin/java
	  
      if [ ! -x "$JAVA_HOME"/bin/java ]
      then
        echo "*Error*  The JAVA_HOME specified was invalid.  Please correct and resubmit."
        echo "*Error*  The JAVA_HOME specified was invalid.  Please correct and resubmit. " >> $CYNCOLLOG
        exit 1
      fi		
    else
      product_javahome="/opt/ibm/ccm/agent/JRE/lx8266/jre"
    
      if [ -x "$product_javahome/bin/java" ]
      then
        echo
        echo "*Warning*  JAVA_HOME was not set, using $product_javahome"
        echo "*Warning*  JAVA_HOME was not set, using $product_javahome" >> $CYNCOLLOG
        JAVA_EXE=$product_javahome/bin/java
        export JAVA_HOME=$product_javahome
      else
        echo "*Error*  JAVA_HOME was not set and we are unable to locate a valid default JAVA_HOME. "
        echo "*Error*  JAVA_HOME was not set and we are unable to locate a valid default JAVA_HOME. " >> $CYNCOLLOG
      exit 1
    fi
    fi
  else
    echo "Tar command will be used, no need to define the location for java"  >> $CYNCOLLOG
  fi
elif [ "$eCAM" ]
then
  echo "This is eCAM so we'll use our own java"  >> $CYNCOLLOG
  JAVA_EXE="$PRODUCT_PATH/../../java/jre/bin/java"
else
  echo "We'll use ITM's java"  >> $CYNCOLLOG
  JAVA_EXE="$ITM_HOME/JRE/$DC71_JRE/bin/java"
fi

echo "------------------------------------
     Additional Global Definitions...     
------------------------------------
HOST_NAME=$HOST_NAME
OS_TYPE=$OS_TYPE
OS_VERSION=$OS_VERSION
OS_ARCH=$OS_ARCH
JAVA_EXE=$JAVA_EXE
" >> $CYNCOLLOG

# Test Java executable
if [ -z "$TAR_CMD" ]
then
  if [ -f "$JAVA_EXE" ]
  then
    echo "JAVA was found..." >> $CYNCOLLOG

    if [ -x "$JAVA_EXE" ]
    then
      echo "JAVA is executable..." >> $CYNCOLLOG
    else
      echo "Error: JAVA ($JAVA_EXE) is not executable."
      echo "!!! ERROR: JAVA ($JAVA_EXE) is not executable." >> $CYNCOLLOG

      exit 1
    fi
  else
    echo "Error: JAVA ($JAVA_EXE) was not found."
    echo "!!! ERROR: JAVA ($JAVA_EXE) was not found." >> $CYNCOLLOG
  
    exit 1
  fi
fi

# #########################################################
#   Defintion of main names for the collection structure
# #########################################################

# Final jar file
CYNJAR="$CYN_WRKDIR.jar"
# Final tar file
CYNTAR="$CYN_WRKDIR.tar"
# Full collection work directory
CYN_DIR=$CYN_TRGT_HOME/$CYN_WRKDIR
# Directory for general system information 
CYN_HOSTDIR=$CYN_DIR/host
# Main Directory for DC stuff
CYN_DCROOTDIR=$CYN_DIR/DC
# Directory for individual DC stuff
CYN_DCDIR=$CYN_DCROOTDIR
# Directory for individual DC logs
CYN_DCLOGSDIR=$CYN_DCROOTDIR/common_logs
# Directory for individual DC logs
CYN_DCINSTDIR=$CYN_DCROOTDIR/install_config
# Main directory for WebSphere's data
CYN_WASDIR=$CYN_DIR/was

echo "
------------------------------------------
 Main Collection Structure Definitions...              
------------------------------------------
CYN_WRKDIR=$CYN_WRKDIR
CYNJAR=$CYNJAR
CYNTAR=$CYNTAR
CYN_DIR=$CYN_DIR
CYN_HOSTDIR=$CYN_HOSTDIR
CYN_DCROOTDIR=$CYN_DCROOTDIR
CYN_DCDIR=$CYN_DCDIR
CYN_DCLOGSDIR=$CYN_DCLOGSDIR
CYN_DCINSTDIR=$CYN_DCINSTDIR
CYN_WASDIR=$CYN_WASDIR" >> $CYNCOLLOG

# ############################################
#   Defintion of collection's "general" file names
# ############################################
SYS_INFO=$CYN_HOSTDIR/system.info
DIR_INFO=$CYN_HOSTDIR/dir.info
ENV_INFO=$CYN_HOSTDIR/env.info
DISK_INFO=$CYN_HOSTDIR/disk.info
NET_INFO=$CYN_HOSTDIR/netstat.info
HOSTS_INFO=$CYN_HOSTDIR/hosts.info
TOP_INFO=$CYN_HOSTDIR/top.info
DMESG_INFO=$CYN_HOSTDIR/dmesg.info
REV_INFO=$CYN_HOSTDIR/rev.info
CLASS_INFO="classmaint.log"

# Log all definitions
echo "
----------------------------------
   General Files Definitions...              
----------------------------------
SYS_INFO=$SYS_INFO
DIR_INFO=$DIR_INFO
ENV_INFO=$ENV_INFO
DISK_INFO=$DISK_INFO
NET_INFO=$NET_INFO
HOSTS_INFO=$HOSTS_INFO
TOP_INFO=$TOP_INFO
DMESG_INFO=$DMESG_INFO
REV_INFO=$REV_INFO
CLASS_INFO=$CLASS_INFO
REV_INFO=$REV_INFO
" >> $CYNCOLLOG

# ###############################
#   Create collection structure
# ###############################
echo "
----------------------------------------
 Creating Basic Collection Structure...              
----------------------------------------" >> $CYNCOLLOG

# Create subdirectory for general files
echo "Creating directory for general files ($CYN_HOSTDIR)" >> $CYNCOLLOG
mkdir -p $CYN_HOSTDIR || exit 1

# Create collection structure (up to DC's main dir)
echo "Creating DC's main directory ($CYN_DCROOTDIR)" >> $CYNCOLLOG
mkdir -p $CYN_DCROOTDIR || exit 1

# Buffers for the different variables.xml files
SERVERVARIABLESXML=
NODEVARIABLESXML=
CLUSTERVARIABLESXML=
CELLVARIABLESXML=
CLUSTERNAME=

# #########################################################################
# #########################################################################
# #########################################################################
#                    End of variables' declarations
# #########################################################################
# #########################################################################
# #########################################################################

# #########################################################################
#                              Functions
# #########################################################################
collect_dc_maintenance_logs()
{
  echo "!!! Attempting to get CYN log path from dc.env.properties" >> $CYNCOLLOG

  matched_inst=
  
  RUNTIME=$PRODUCT_PATH/$level/runtime
  if [ -d "$RUNTIME" ]
  then
    for currRuntime in `ls $RUNTIME`
    do
      ENV_PROPS=$PRODUCT_PATH/$level/runtime/$currRuntime/dc.env.properties
      if [ -f "$ENV_PROPS" ]
      then
        # Extract CCLOG_COMMON_DIR variable
        COMMLOGSDIR=`grep CCLOG_COMMON_DIR= $ENV_PROPS | sed 's/.*=//'`
        echo "COMMLOGSDIR=$COMMLOGSDIR" >> $CYNCOLLOG
  
        CYN_DCDIR=$CYN_DCROOTDIR/$level/common_logs
        echo "!!! NEW CYN_DCDIR=$CYN_DCDIR" >> $CYNCOLLOG

        if [ -d "$COMMLOGSDIR"/CYN ]
        then
          echo "!!! Common logs directory exist ($COMMLOGSDIR/CYN)" >> $CYNCOLLOG

          for PNAME in logs FFDC
          do
            echo "!!! Creating collection directory for specific logs ($CYN_DCDIR/CYN/$PNAME)" >> $CYNCOLLOG
            mkdir -p $CYN_DCDIR/CYN/$PNAME || exit 1

            echo "!!! Copying log files ($COMMLOGSDIR/CYN/$PNAME/*.log to $CYN_DCDIR/CYN/$PNAME)" >> $CYNCOLLOG
            cp -Rpf $COMMLOGSDIR/CYN/$PNAME/*.log $CYN_DCDIR/CYN/$PNAME/ 2>/dev/null

            if [ -z "$ALL_SERVERS" -a -z "$APP_SERVER_NAME" ]
            then
              echo "!!! Copying specific servers logs" >> $CYNCOLLOG
              cp -Rpf $COMMLOGSDIR/CYN/$PNAME/* $CYN_DCDIR/CYN/$PNAME/ 2>/dev/null
            else 
              if [ -n "$ALL_SERVERS" ]
              then
                echo "!!! Copying specific servers logs" >> $CYNCOLLOG
                cp -Rpf $COMMLOGSDIR/CYN/$PNAME/* $CYN_DCDIR/CYN/$PNAME/ 2>/dev/null
              else
                matched_inst=`ls -d $COMMLOGSDIR/CYN/$PNAME/*.$APP_SERVER_NAME 2>/dev/null`
                if [ -n "$matched_inst" ]
                then
                  cp -Rpf $matched_inst $CYN_DCDIR/CYN/$PNAME/ 2>/dev/null
                fi
              fi
            fi
          done
        else
          echo "!!! Common logs directory ($COMMLOGSDIR/CYN) does not exist" >> $CYNCOLLOG
        fi
      fi
    done
  fi
}

collect_dc_maintenance_config()
{

  DC_VERSION=`grep -v '#' $PRODUCT_PATH/$level/itcamdc/etc/version.properties | grep probe.release.version= | sed 's/.*=//'`
  DC_PTF=`grep -v '#' $PRODUCT_PATH/$level/itcamdc/etc/version.properties | grep probe.ptf.version= | sed 's/.*=//'`

  RUNTIME=$PRODUCT_PATH/$level/runtime
  
  echo "======================================================================
  DC_VERSION=$DC_VERSION
  DC_PTF=$DC_PTF
  RUNTIME=$RUNTIME
  ======================================================================" >> $CYNCOLLOG

  echo "!!! Collecting configuration files" >> $CYNCOLLOG
  echo "!!! Creating $CYN_DCROOTDIR/itcamdc" >> $CYNCOLLOG
  mkdir -p $CYN_DCROOTDIR/$level/itcamdc/etc || exit 1

  echo "!!! Copying $PRODUCT_PATH/itcamdc/etc to $CYN_DCROOTDIR/itcamdc" >> $CYNCOLLOG
  cp -Rpf $PRODUCT_PATH/$level/itcamdc/etc $CYN_DCROOTDIR/$level/itcamdc

  echo "!!! Creating $CYN_DCROOTDIR/runtime" >> $CYNCOLLOG
  mkdir -p $CYN_DCROOTDIR/$level/runtime || exit 1

  if [ -d  "$RUNTIME" ]
  then
    for currRuntime in `ls $RUNTIME`
    do
      echo "!!! Copying $currRuntime" >> $CYNCOLLOG
      cp -Rpf $PRODUCT_PATH/$level/runtime/$currRuntime $CYN_DCROOTDIR/$level/runtime
      DC_RUNTIME_INSTANCE=$PRODUCT_PATH/$level/runtime/$currRuntime
    done
  fi
  
  TOOLKIT="$PRODUCT_PATH/$level/toolkit"
  CLASSPATH="$TOOLKIT/lib/bcm-bootstrap.jar:$TOOLKIT/lib/ext/cdn.jar:$TOOLKIT/lib/ext/jffdc.jar:$TOOLKIT/lib/ext/jflt.jar:$TOOLKIT/lib/ext/jlog.jar:$TOOLKIT/lib/ext/tk_jflt.jar:$TOOLKIT/lib/ext/aspectjrt.jar"
  echo "   ...Collecting maintenance levels for all classes"
  $JAVA_EXE -cp $CLASSPATH com.ibm.tivoli.itcam.toolkit.ai.scanner.ClassScanner -classpath $PRODUCT_PATH/$level>$CYN_DCROOTDIR/$level/$CLASS_INFO 2>/dev/null
}

extractValue ()
{
  inSetFile=$1
  inVARIABLE=$2

  echo "
  ****************************
  *   !!! extractValue()...
  ****************************
  *  inSetFile(Parm1)=$inSetFile
  *  inVARIABLE(Parm2)=$inVARIABLE" >> $CYNCOLLOG
  xVALUE=`grep -v '#' $inSetFile | grep $inVARIABLE= | sed 's/.*=//'`
  echo "  *  xVALUE=$xVALUE" >> $CYNCOLLOG
  echo "  ****************************
" >> $CYNCOLLOG

  return
}

# Function used to extract a value out of server.xml and define
# any unresolved symbol it may contain by the means of the
# variables.xml found @ server->node->cell levels (in that order)
extractDefinition ()
{
  reqDEFIN=$1
  newDEFINITION=

  echo "
  ****************************
  * !!! extractDefinition()...
  ****************************
  *  reqDEFIN(Parm1)=$reqDEFIN" >> $CYNCOLLOG

  # Check for unresolved symbols within the definiton
  unrSYMBOL=`echo $reqDEFIN | grep '\${' | sed -e 's/.*\${//' -e 's/}.*//'`
  echo "  *  unrSYMBOL=$unrSYMBOL" >> $CYNCOLLOG

  # If an unresolved symbol was found within the definition...
  while [ -n "$unrSYMBOL" ]
  do
    echo "  *  !!! Resolving $unrSYMBOL" >> $CYNCOLLOG

    if [ -n "$CLUSTERNAME" ]
    then
      # Extract the definition for the symbol out of all 4 variables.xml files
      symVALUE=`grep symbolicName=\"$unrSYMBOL\" $CELLVARIABLESXML $CLUSTERVARIABLESXML $NODEVARIABLESXML $SERVERVARIABLESXML | sed -e 's/.*value="//' -e 's/".*//' | tail -1`
    else
      # Extract the definition for the symbol out of all 3 variables.xml files
      symVALUE=`grep symbolicName=\"$unrSYMBOL\" $CELLVARIABLESXML $NODEVARIABLESXML $SERVERVARIABLESXML | sed -e 's/.*value="//' -e 's/".*//' | tail -1`
    fi

    echo "  *  symVALUE=$symVALUE" >> $CYNCOLLOG

    # If a definition for the symbol was not found flag the error and leave
    if [ -z "$symVALUE" ]
    then
      echo "...ERROR: Unable to locate definition for variable $unrSYMBOL"
      echo "!!! ERROR: Unable to locate definition for variable $unrSYMBOL" >> $CYNCOLLOG
      exit 1
    fi

    # Capture the initial portion of the definition
    preSYM=`echo $reqDEFIN | grep $ | sed 's/\${.*//'`
    echo "  *  preSYM=$preSYM" >> $CYNCOLLOG
    # Capture the final portion of the definition
    posSYM=`echo $reqDEFIN | grep $ | sed 's/.*}//'`
    echo "  *  posSYM=$posSYM" >> $CYNCOLLOG
    # Create the new definition
    newDEFINITION=$preSYM$symVALUE$posSYM        
    echo "  *  newDEFINITION=$newDEFINITION" >> $CYNCOLLOG

    reqDEFIN=$newDEFINITION
    echo "  *  reqDEFIN=$reqDEFIN" >> $CYNCOLLOG

    # Check for undefined symbols within the definiton
    unrSYMBOL=`echo $newDEFINITION | grep '\${' | sed -e 's/.*\${//' -e 's/}.*//'`
    echo "  *  unrSYMBOL=$unrSYMBOL" >> $CYNCOLLOG
  done                            

  if [ -z "$newDEFINITION" ]
  then
    echo "  *  !!! newDEFINITION is empty... using original value instead." >> $CYNCOLLOG
    newDEFINITION=$reqDEFIN
    echo "  *  newDEFINITION=$newDEFINITION" >> $CYNCOLLOG
  fi

  echo "  ****************************
" >> $CYNCOLLOG

  return
}

process_dcInputs ()
{
  echo "!!! $DCINPUTS is readable..." >> $CYNCOLLOG

  # #######################################################
  #  Extract the required settings out of the inputs file
  # #######################################################
  # Extract WAS_HOME variable
  extractValue $DCINPUTS WAS_HOME
  WAS_HOME=$xVALUE
  WAS_BIN_BASE=$WAS_HOME

  echo "WAS_HOME=$WAS_HOME" >> $CYNCOLLOG
  echo "WAS_BIN_BASE=$WAS_BIN_BASE" >> $CYNCOLLOG

  # Extract APPSERVER_VERSION variable
  extractValue $DCINPUTS APPSERVER_VERSION
  APPSERVER_VERSION=$xVALUE

  echo "APPSERVER_VERSION=$APPSERVER_VERSION" >> $CYNCOLLOG

  # Extract WS_INSTANCE_NAME variable
  extractValue $DCINPUTS WS_INSTANCE_NAME
  WS_INSTANCE_NAME=$xVALUE

  echo "WS_INSTANCE_NAME=$WS_INSTANCE_NAME" >> $CYNCOLLOG

  # For WAS 5.1.* replace ProdData with UserData and add the instance at the end
  WAS51HOME=`echo $WAS_HOME | sed -e 's/ProdData/UserData/'`
  WAS51HOME=$WAS51HOME/$WS_INSTANCE_NAME

  echo "WAS51HOME=$WAS51HOME" >> $CYNCOLLOG

  # If the App.Server is version 5.1.* then use the WAS 5.1 home format instead of the regular one
  if [ "$OS_TYPE" = "OS400" -a "$APPSERVER_VERSION" = "51" ]
  then
    WAS_HOME=$WAS51HOME
    echo "!!! New WAS_HOME=$WAS_HOME" >> $CYNCOLLOG
  fi

  if [ -n "$ALL_SERVERS" -o "$APP_SERVER_NAME" = "$currSERVER" ]
  then
    echo "!!! $currSERVER is a selected server..." >> $CYNCOLLOG
    serverFOUND=Y
  else
    echo "!!! $currSERVER is not a selected server..." >> $CYNCOLLOG
  fi

  serverXML=$WAS_HOME/config/cells/$currCELL/nodes/$currNODE/servers/$currSERVER/server.xml

  # Extract the cluster name (if any) out of server.xml
  CLUSTERNAME=`grep "<process:Server" $serverXML | grep clusterName | sed -e 's/.*clusterName="//' -e 's/".*//'`
  echo "CLUSTERNAME=$CLUSTERNAME" >> $CYNCOLLOG

  # #######################################################
  #  Define the full name for all required xml files
  # #######################################################
  CELLVARIABLESXML=$WAS_HOME/config/cells/$currCELL/variables.xml
  CLUSTERVARIABLESXML=$WAS_HOME/config/cells/$currCELL/clusters/$CLUSTERNAME/variables.xml
  NODEVARIABLESXML=$WAS_HOME/config/cells/$currCELL/nodes/$currNODE/variables.xml
  SERVERVARIABLESXML=$WAS_HOME/config/cells/$currCELL/nodes/$currNODE/servers/$currSERVER/variables.xml
  serverindexXML=$WAS_HOME/config/cells/$currCELL/nodes/$currNODE/serverindex.xml

  echo "CELLVARIABLESXML=$CELLVARIABLESXML" >> $CYNCOLLOG
  echo "CLUSTERVARIABLESXML=$CLUSTERVARIABLESXML" >> $CYNCOLLOG
  echo "NODEVARIABLESXML=$NODEVARIABLESXML" >> $CYNCOLLOG
  echo "SERVERVARIABLESXML=$SERVERVARIABLESXML" >> $CYNCOLLOG
  echo "serverXML=$serverXML" >> $CYNCOLLOG
  echo "serverindexXML=$serverindexXML" >> $CYNCOLLOG

  # #######################################################
  #  Find full names for all 4 logs we want to collect
  # #######################################################
  # Extract the full name for SystemOut.log out of server.xml
  SYSTEMOUT_LOG=`grep outputStreamRedirect $serverXML | sed -e 's/.*fileName="//' -e 's/".*//'`
  echo "SYSTEMOUT_LOG=$SYSTEMOUT_LOG" >> $CYNCOLLOG
  # Resolve any unresolved symbols
  extractDefinition $SYSTEMOUT_LOG
  # Store the resulting defintion 
  SYSTEMOUT_LOG=$newDEFINITION
  echo "!!! Resolved SYSTEMOUT_LOG=$SYSTEMOUT_LOG" >> $CYNCOLLOG

  # Extract the full name for SystemErr.log out of server.xml
  SYSTEMERR_LOG=`grep errorStreamRedirect $serverXML | sed -e 's/.*fileName="//' -e 's/".*//'`
  echo "SYSTEMERR_LOG=$SYSTEMERR_LOG" >> $CYNCOLLOG
  # Resolve any unresolved symbols
  extractDefinition $SYSTEMERR_LOG 
  # Store the resulting defintion 
  SYSTEMERR_LOG=$newDEFINITION
  echo "!!! Resolved SYSTEMERR_LOG=$SYSTEMERR_LOG" >> $CYNCOLLOG

  # Extract the full name for native_stdout.log out of server.xml
  NATSTDOUT_LOG=`grep ioRedirect $serverXML | sed -e 's/.*stdoutFilename="//' -e 's/".*//'`
  echo "NATSTDOUT_LOG=$NATSTDOUT_LOG" >> $CYNCOLLOG
  # Resolve any unresolved symbols
  extractDefinition $NATSTDOUT_LOG
  # Store the resulting defintion 
  NATSTDOUT_LOG=$newDEFINITION
  echo "!!! Resolved NATSTDOUT_LOG=$NATSTDOUT_LOG" >> $CYNCOLLOG

  # Extract the full name for native_stderr.log out of server.xml
  NATSTDERR_LOG=`grep ioRedirect $serverXML | sed -e 's/.*stderrFilename="//' -e 's/".*//'`
  echo "NATSTDERR_LOG=$NATSTDERR_LOG" >> $CYNCOLLOG
  # Resolve any unresolved symbols
  extractDefinition $NATSTDERR_LOG
  # Store the resulting defintion 
  NATSTDERR_LOG=$newDEFINITION
  echo "!!! Resolved NATSTDERR_LOG=$NATSTDERR_LOG" >> $CYNCOLLOG

  COMMLOGSDIR=`grep CCLOG_COMMON_DIR $serverXML | sed -e 's/.*value="//' -e 's/".*//'`
  echo "COMMLOGSDIR=$COMMLOGSDIR" >> $CYNCOLLOG
  # Resolve any unresolved symbols
  extractDefinition $COMMLOGSDIR
  # Store the resulting defintion 
  COMMLOGSDIR=$newDEFINITION
  echo "!! Resolved COMMLOGSDIR=$COMMLOGSDIR" >> $CYNCOLLOG

  # Extract RUNTIME_NAME variable
  TMP_RUNTIME_NAME=`grep RUNTIME_NAME= $DCINPUTS | sed 's/.*=//'`
  # Make it available
  RUNTIME_NAME=$TMP_RUNTIME_NAME

  if [ "$serverFOUND" = "Y" ]
  then
echo "
------------------------------------
  Copying WebSphere information...              
------------------------------------
" >> $CYNCOLLOG

    # If collector's was directory does not exist yet...
    if [ ! -d "$CYN_WASDIR" ]
    then
      # Create subdirectory for was files
      echo "!!! Creating WAS' collection dir ($CYN_WASDIR)" >> $CYNCOLLOG
      mkdir -p $CYN_WASDIR || exit 1
    fi

    # Create full structure for current server
    echo "!!! Creating full log collection dir for this server ($CYN_WASDIR/cells/$currCELL/nodes/$currNODE/servers/$currSERVER/logs)" >> $CYNCOLLOG
    mkdir -p $CYN_WASDIR/cells/$currCELL/nodes/$currNODE/servers/$currSERVER/logs || exit1

    # Copy server.xml to collector's structure
    echo "!!! Copying server.xml ($serverXML) to $CYN_WASDIR/cells/$currCELL/nodes/$currNODE/servers/$currSERVER" >> $CYNCOLLOG
    cp -Rpf $serverXML $CYN_WASDIR/cells/$currCELL/nodes/$currNODE/servers/$currSERVER
    # Copy server's variables.xml to collector's structure
    echo "!!! Copying <server>/variables.xml ($SERVERVARIABLESXML) to $CYN_WASDIR/cells/$currCELL/nodes/$currNODE/servers/$currSERVER" >> $CYNCOLLOG
    cp -Rpf $SERVERVARIABLESXML $CYN_WASDIR/cells/$currCELL/nodes/$currNODE/servers/$currSERVER
    # Copy serverindex.xml to collector's structure
    echo "!!! Copying serverindex.xml ($serverindexXML) to $CYN_WASDIR/cells/$currCELL/nodes/$currNODE" >> $CYNCOLLOG
    cp -Rpf $serverindexXML $CYN_WASDIR/cells/$currCELL/nodes/$currNODE
    # Copy node's variables.xml to collector's structure
    echo "!!! Copying <node>/variables.xml ($NODEVARIABLESXML) to $CYN_WASDIR/cells/$currCELL/nodes/$currNODE" >> $CYNCOLLOG
    cp -Rpf $NODEVARIABLESXML $CYN_WASDIR/cells/$currCELL/nodes/$currNODE
    # Copy cell's variables.xml to collector's structure
    echo "!!! Copying <cell>/variables.xml ($CELLVARIABLESXML) to $CYN_WASDIR/cells/$currCELL" >> $CYNCOLLOG
    cp -Rpf $CELLVARIABLESXML $CYN_WASDIR/cells/$currCELL

    if [ -n "$CLUSTERNAME" ]
    then
      echo "!!! Creating collection dir for cluster files ($CYN_WASDIR/cells/$currCELL/clusters/$CLUSTERNAME)" >> $CYNCOLLOG
      mkdir -p $CYN_WASDIR/cells/$currCELL/clusters/$CLUSTERNAME || exit1

      echo "!!! Copying <cluster>/variables.xml ($CLUSTERVARIABLESXML) to $CYN_WASDIR/cells/$currCELL/clusters/$CLUSTERNAME" >> $CYNCOLLOG
      cp -Rpf $CLUSTERVARIABLESXML $CYN_WASDIR/cells/$currCELL/clusters/$CLUSTERNAME
    fi 

    if [  -d  "$WAS_HOME/logs/ffdc" ]
    then
      echo "!!! Creating collection dir for this server's ffdc's ($CYN_WASDIR/cells/$currCELL/nodes/$currNODE/servers/$currSERVER/logs/ffdc)" >> $CYNCOLLOG
      mkdir -p $CYN_WASDIR/cells/$currCELL/nodes/$currNODE/servers/$currSERVER/logs/ffdc || exit 1
 
      echo "!!! Copying WAS ffdc logs to this server's log collection dir." >> $CYNCOLLOG
      cp -Rpf $WAS_HOME/logs/ffdc/$currSERVER_* $CYN_WASDIR/cells/$currCELL/nodes/$currNODE/servers/$currSERVER/logs/ffdc
    fi
    
    echo "!!! Copying ${WAS_HOME}/logs/$currSERVER to this server's log collection dir." >> $CYNCOLLOG
    cp -Rpf $WAS_HOME/logs/$currSERVER $CYN_WASDIR/cells/$currCELL/nodes/$currNODE/servers/$currSERVER/logs

    # Copy SystemOut.log
    if [ -f "$SYSTEMOUT_LOG" ]
    then
        echo "!!! Copying SystemOut.log ($SYSTEMOUT_LOG) to this server's log collection dir." >> $CYNCOLLOG
        cp -Rpf $SYSTEMOUT_LOG $CYN_WASDIR/cells/$currCELL/nodes/$currNODE/servers/$currSERVER/logs/$currSERVER
    else
        echo "!!! Unable to locate SystemOut.log($SYSTEMOUT_LOG)" >> $CYNCOLLOG
    fi

    # Copy SystemErr.log
    if [ -f "$SYSTEMERR_LOG" ]
    then
        echo "!!! Copying SystemErr.log ($SYSTEMERR_LOG) to this server's log collection dir." >> $CYNCOLLOG
        cp -Rpf $SYSTEMERR_LOG $CYN_WASDIR/cells/$currCELL/nodes/$currNODE/servers/$currSERVER/logs/$currSERVER
    else
        echo "!!! Unable to locate SystemErr.log($SYSTEMERR_LOG)" >> $CYNCOLLOG
    fi

    # Copy native_stdout.log
    if [ -f "$NATSTDOUT_LOG" ]
    then
        echo "!!! Copying native_stdout.log ($NATSTDOUT_LOG) to this server's log collection dir." >> $CYNCOLLOG
        cp -Rpf $NATSTDOUT_LOG $CYN_WASDIR/cells/$currCELL/nodes/$currNODE/servers/$currSERVER/logs/$currSERVER
    else
        echo "!!! Unable to locate native_stdout.log($NATSTDOUT_LOG)" >> $CYNCOLLOG
    fi

    # Copy native_stderr.log
    if [ -f "$NATSTDERR_LOG" ]
    then
        echo "!!! Copying native_stderr.log ($NATSTDERR_LOG) to this server's log collection dir." >> $CYNCOLLOG
        cp -Rpf $NATSTDERR_LOG $CYN_WASDIR/cells/$currCELL/nodes/$currNODE/servers/$currSERVER/logs/$currSERVER
    else
        echo "!!! Unable to locate native_stderr.log($NATSTDERR_LOG)" >> $CYNCOLLOG
    fi

    echo "!!! Collecting versionInfo ($WAS_HOME/bin/versionInfo.sh)." >> $CYNCOLLOG
    $WAS_HOME/bin/versionInfo.sh > $CYN_WASDIR/cells/$currCELL/nodes/$currNODE/servers/$currSERVER/versionInfo.txt
  fi #if [ "$serverFOUND" = "Y" ]

  dcRuntimes="$dcRuntimes$RUNTIME_NAME "
  
  dcInputFiles="$DCINPUTS "
}

process_collectWASInfo_classic ()
{
  # ####################################
  #   Collect WAS' information
  # ####################################

  WAS_HOME=
  WAS_BIN_BASE=
  APPSERVER=
  WS_NODE_NAME=
  DC_HOME=
  TIVOLI_COMMON=
  dcConfigNODES=
  dcConfigSERVERS=
  DCINPUTS=
  serverFOUND=
  dcPTF=
  noWAS=

  if [ -n "$ALL_SERVERS" -o -n "$APP_SERVER_NAME" ]
  then
    echo "* Collecting WAS information"
    echo "!!! Collecting WAS information..." >> $CYNCOLLOG
  else
    echo "!!! WAS information won't be collected..." >> $CYNCOLLOG
    noWAS=Y
  fi

  echo "noWAS=$noWAS" >> $CYNCOLLOG

  if [ -z "$noWAS" ]
  then
echo "
------------------------------------
 Analizing WebSphere Environment...              
------------------------------------" >> $CYNCOLLOG

    dcRuntimes=
    dcInputFiles=

    # If cells were found
    echo "dcConfigCELLS=$dcConfigCELLS
    CONFIGURED=$CONFIGURED" >> $CYNCOLLOG

    if [ -n "$dcConfigCELLS" -a -n "$CONFIGURED" ]
    then
      echo "!!! Cycling thru discovered cells..." >> $CYNCOLLOG
      # Cycle thru discovered cells
      for currCELL in $dcConfigCELLS 
      do
        if [ "$eCAM" ] 
        then
          dcConfigNODESdir=$dcConfigCELLSdir/cells/$currCELL/nodes
        else
          dcConfigNODESdir=$dcConfigCELLSdir/$currCELL/nodes
        fi

        if [ -d "$dcConfigNODESdir" ]
        then
          dcConfigNODES=`ls $dcConfigNODESdir`
        else
          dcConfigNODES=
        fi

        echo "currCELL=$currCELL" >> $CYNCOLLOG
        echo "dcConfigNODESdir=$dcConfigNODESdir" >> $CYNCOLLOG
        echo "dcConfigNODES=$dcConfigNODES" >> $CYNCOLLOG

        if [ -n "$dcConfigNODES" ]
        then
          echo "!!! Cycling thru discovered nodes..." >> $CYNCOLLOG
          for currNODE in $dcConfigNODES 
          do
            dcConfigSERVERSdir=$dcConfigNODESdir/$currNODE/servers

            if [ -d "$dcConfigSERVERSdir" ]
            then
              dcConfigSERVERS=`ls $dcConfigSERVERSdir`
            else
              dcConfigSERVERS=
            fi
              
            echo "currNODE=$currNODE" >> $CYNCOLLOG
            echo "dcConfigSERVERSdir=$dcConfigSERVERSdir" >> $CYNCOLLOG
            echo "dcConfigSERVERS=$dcConfigSERVERS" >> $CYNCOLLOG

            if [ -n "$dcConfigSERVERS" ]
            then
              echo "!!! Cycling thru discovered servers..." >> $CYNCOLLOG

              for currSERVER in $dcConfigSERVERS
              do
                DCINPUTS=$dcConfigSERVERSdir/$currSERVER/dcInputs.txt

                echo "currSERVER=$currSERVER" >> $CYNCOLLOG
                echo "DCINPUTS=$DCINPUTS" >> $CYNCOLLOG

                if [ -f "$DCINPUTS" ]
                then
                  echo "!!! $DCINPUTS exists..." >> $CYNCOLLOG

                  if [ -r "$DCINPUTS" ]
                  then
                    process_dcInputs
                  else
                    echo "...ERROR: $DCINPUTS is not readable."
                    echo "!!! $DCINPUTS is not readable." >> $CYNCOLLOG
                    exit 1
                  fi
                else
                  echo "...ERROR: $DCINPUTS does not exist."
                  echo "!!! $DCINPUTS does not exist." >> $CYNCOLLOG
                  exit 1
                fi 
              done
            else # If no servers found
              echo "...ERROR: No servers found."
              echo "!!! ERROR: No servers found." >> $CYNCOLLOG
            fi 
          done #for currNODE in $dcConfigNODES 
        else # If no nodes found
          echo "...ERROR: No nodes found."
          echo "!!! ERROR: No nodes found." >> $CYNCOLLOG
        fi
      done #for currCELL in $dcConfigCELLS 
    elif [ -n "$configuredAppServers" -a -n "$CONFIGURED" ]
    then
      echo "!!! Cycling thru discovered runtime directories..." >> $CYNCOLLOG
      # Cycle thru discovered runtime directories

      configuredAppServers=`ls $dcConfigCELLSdir`
      echo "configuredAppServers=$configuredAppServers" >> $CYNCOLLOG

      if [ -n "$configuredAppServers" ]
      then
        for runtimeDir in $configuredAppServers
        do
          echo "Analyzing $runtimeDir" >> $CYNCOLLOG
          DCINPUTS=$dcConfigCELLSdir/$runtimeDir/dcInput.txt
          echo "DCINPUTS=$DCINPUTS" >> $CYNCOLLOG

          if [ -f "$DCINPUTS" ]
          then
            echo "$DCINPUTS exists !!!" >> $CYNCOLLOG

            echo "Extracting APP_SERVER_NAME out of $DCINPUTS !!!" >> $CYNCOLLOG
            extractValue $DCINPUTS APP_SERVER_NAME 
            echo "Extracted APP_SERVER_NAME is: $xVALUE !!!" >> $CYNCOLLOG
       
            echo "Parsing cell name out of $xVALUE !!!" >> $CYNCOLLOG
            currCELL=`echo $xVALUE | sed -e 's/cells\///' -e 's/\/nodes.*//'`
            echo "currCELL=$currCELL !!!" >> $CYNCOLLOG

            echo "Parsing node name out of $xVALUE !!!" >> $CYNCOLLOG
            currNODE=`echo $xVALUE | sed -e 's/.*nodes\///' -e 's/\/servers.*//'`
            echo "currNODE=$currNODE" >> $CYNCOLLOG

            echo "Parsing server name out of $xVALUE !!!" >> $CYNCOLLOG
            currSERVER=`echo $xVALUE | sed -e 's/.*servers\///'`
            echo "currSERVER=$currSERVER" >> $CYNCOLLOG

            if [ "$APP_SERVER_NAME" = "" -o "$currSERVER" = "$APP_SERVER_NAME" ]
            then
              echo "Invoking function process_dcInputs!!!" >> $CYNCOLLOG
              process_dcInputs
            fi
          fi
        done
      else
        echo "...ERROR: No configured App.Servers found."
        echo "!!! ERROR: No configured App.Servers found." >> $CYNCOLLOG
      fi
    else # If no cells found
      echo "!!! No cells found. Checking for WAS CE configuration..." >> $CYNCOLLOG

      if [ -d "$WASCE_DIR" ]
      then
        COMMLOGSDIR=$COMDIR

        cd $WASCE_DIR
        propsFile=`ls *.properties`

        # Extract SERVER_HOME variable
        TMP_SERVER_HOME=`grep SERVER_HOME= $propsFile | sed 's/.*=//'`
        # Make it available
        CE_SERVER_HOME=$TMP_SERVER_HOME
        # Export it
        export CE_SERVER_HOME

        # Extract SERVER_UNAME variable
        TMP_SERVER_UNAME=`grep SERVER_UNAME= $propsFile | sed 's/.*=//'`
        # Make it available
        CE_SERVER_NAME=$TMP_SERVER_UNAME
        # Export it
        export CE_SERVER_NAME

        if [ -n "$ALL_SERVERS" -o "$APP_SERVER_NAME" = "$CE_SERVER_NAME" ]
        then
          serverFOUND=Y
          mkdir -p $CYN_WASDIR/$CE_SERVER_NAME/var/catalina || exit1

          cp -Rpf $CE_SERVER_HOME/var/log $CYN_WASDIR/$CE_SERVER_NAME/var 2>/dev/null
          cp -Rpf $CE_SERVER_HOME/var/catalina/logs $CYN_WASDIR/$CE_SERVER_NAME/var/catalina 2>/dev/null
        else
          if [ -z "$ALL_SERVERS" -a -z "$APP_SERVER_NAME" ]
          then
          serverFOUND=Y
          fi
        fi

        for DNAME in BWM CYN
        do
          if [ -d "$COMMLOGSDIR"/"$DNAME" ]
          then
            mkdir -p $CYN_DCLOGSDIR/WASCE/$DNAME 2>/dev/null
            for PNAME in logs FFDC
            do
              cp -Rpf $COMMLOGSDIR/$DNAME/$PNAME $CYN_DCLOGSDIR/WASCE/$DNAME/ 2>/dev/null
            done
          fi
        done

      else
        echo "!!! No cells found. Checking for WAS CE configuration..." >> $CYNCOLLOG
        extractValue $PRODUCT_PATH/itcamdc/etc/version.properties probe.ptf.version
        dcPTF=$xVALUE   

        echo "...ERROR: $DCINPUTS does not exist."
        echo "!!! $DCINPUTS does not exist." >> $CYNCOLLOG

        echo "dcPTF=$dcPTF"
        exit 1

        if [ ! -f dcInputs.txt ]
        then
          echo "...ERROR: Non-supported WAS version found."
          exit 1  
        fi
      fi
    fi

    if [ -z "$serverFOUND" ]
    then
      if [ -n "$ALL_SERVERS" ]
      then
        echo "*** ERROR: No servers found !!! ***"
        exit 1
      elif [ -n "$APP_SERVER_NAME" ]
      then
        echo "*** ERROR: App.Server $APP_SERVER_NAME not found !!! ***"
        exit 1
      fi
    fi
  fi
}

collect_configuration_AD72 ()
{
  echo "   ...Collecting product configuration logs"
  echo "   ...Collecting product configuration logs" >> $CYNCOLLOG
  configLogsDir=$CYN_DCROOTDIR/config_logs
  echo "!!! Creating $configLogsDir" >> $CYNCOLLOG
  mkdir -p $configLogsDir || exit 1

  echo "!!! Copying logs and data from $PRODUCT_PATH/data to $configLogsDir" >> $CYNCOLLOG
  cp -Rpf $PRODUCT_PATH/data/*.log $configLogsDir 2>/dev/null
  cp -Rpf $PRODUCT_PATH/data/*.dat $configLogsDir 2>/dev/null
}

collect_etc_AD72 ()
{
  echo "   ...Collecting main configuration files"

  echo "!!! Collecting configuration files" >> $CYNCOLLOG
  echo "!!! Creating $CYN_DCROOTDIR/itcamdc" >> $CYNCOLLOG
  mkdir -p $CYN_DCROOTDIR/itcamdc/etc || exit 1

  echo "!!! Copying $PRODUCT_PATH/itcamdc/etc to $CYN_DCROOTDIR/itcamdc" >> $CYNCOLLOG
  cp -Rpf $PRODUCT_PATH/itcamdc/etc $CYN_DCROOTDIR/itcamdc
}

collect_runtime_info_AD72()
{
    # Creating main DC collection directory
    trgAPPSrvDir=$CYN_DCROOTDIR/$iAppSrvID
    echo "!!! Creating $trgAPPSrvDir" >> $CYNCOLLOG
    mkdir -p $trgAPPSrvDir || exit 1

    # Copying runtime
    trgRTDir=$CYN_DCROOTDIR/$iAppSrvID/
    srcRTDir=$iDCRuntime
    echo "!!! Creating $trgAPPSrvDir" >> $CYNCOLLOG
    mkdir -p $trgRTDir || exit 1
    echo "!!! Copying $iDCRuntime to $trgRTDir" >> $CYNCOLLOG
    mkdir $trgRTDir/runtime/
    cp -Rpf $srcRTDir/*$iWASServer* $trgRTDir/runtime/
}

collect_runtime_info_AD72_DCLess()
{
    # Creating main DC collection directory
    trgAPPSrvDir=$CYN_DCROOTDIR/DCLessLogs/${iAppSrvID}_DCLess_${iTempWASProfile}_${iWASCell}_${iWASNode}_${iWASServer}
    echo "!!! Creating $trgAPPSrvDir" >> $CYNCOLLOG
    mkdir -p $trgAPPSrvDir || exit 1

    # Copying runtime
    trgRTDir=$CYN_DCROOTDIR/DCLessLogs/${iAppSrvID}_DCLess_${iTempWASProfile}_${iWASCell}_${iWASNode}_${iWASServer}/
    srcRTDir=$iDCRuntime
    echo "!!! Creating $trgAPPSrvDir" >> $CYNCOLLOG
    mkdir -p $trgRTDir || exit 1
    echo "!!! Copying $iDCRuntime to $trgRTDir" >> $CYNCOLLOG
    
	if [ -e "$srcRTDir" ]
	then
        mkdir $trgRTDir/runtime/
        cp -Rpf $srcRTDir/*$iWASServer* $trgRTDir/runtime/
	fi
}

collect_DCLessLogs_AD72_DCLess()
{
	trgDir=$CYN_DCROOTDIR/DCLessLogs

	#Need to cater for the case where a server is configured for DC but had been configured for DCLess and so has logs in the DC_HOME/logs dir.
	#May result in duplicate logs but better that than missing them.
	#cp -rf ${iTempDCHome}/logs/* ${trgDir}
	
	#Alternative case where we don't care about old servers that aren't
	#used any more
	mkdir -p ${trgDir}/FFDC
	mkdir -p ${trgDir}/logs
	if [ -e "${iTempDCHome}/logs/CYN/FFDC/${iAppSrvID}" ]
	then 
		cp -rf ${iTempDCHome}/logs/CYN/FFDC/${iAppSrvID} ${trgDir}/FFDC
	fi
	
	if [ -e "${iTempDCHome}/logs/CYN/logs/${iAppSrvID}" ]
	then
		cp -rf ${iTempDCHome}/logs/CYN/logs/${iAppSrvID} ${trgDir}/logs
	fi
}

collect_DCLogs_AD72_DCLess()
{
    # Creating DC logs directory
    trgDCLogsDir=$CYN_DCROOTDIR/DCLessLogs/${iAppSrvID}_DCLess_${iTempWASProfile}_${iWASCell}_${iWASNode}_${iWASServer}/DCLogs/logs
    echo "Creating $trgDCLogsDir" >> $CYNCOLLOG
    mkdir -p $trgDCLogsDir || exit 1

    # Creating DC FFDCs directory
    trgDCFFDCsDir=$CYN_DCROOTDIR/DCLessLogs/${iAppSrvID}_DCLess_${iTempWASProfile}_${iWASCell}_${iWASNode}_${iWASServer}/DCLogs/FFDC
    echo "Creating $trgDCFFDCsDir" >> $CYNCOLLOG
    mkdir -p $trgDCFFDCsDir || exit 1

    # Copying logs
    if [ -d "$iDCLogs" ]
    then
      echo "Copying $iDCLogs/* to $trgDCLogsDir" >> $CYNCOLLOG
      cp -Rpf $iDCLogs/* $trgDCLogsDir
    else
      echo "Unable to find $iDCLogs" >> $CYNCOLLOG
    fi
    
    # Copying FFDCs
    if [ -d "$iDCFFDC" ]
    then
      cd $iDCFFDC
      FFDCs=`ls 2>&1`
      cd $OLDPWD
      if [ -n "$FFDCs" ]
      then
        echo "!!! Copying $iDCFFDC/* to $trgDCFFDCsDir" >> $CYNCOLLOG
        cp -Rpf $iDCFFDC/* $trgDCFFDCsDir 2>&1
      else
        echo "!!! No FFDCs found under $iDCFFDC" >> $CYNCOLLOG
      fi
    else
      echo "Unable to find $iDCFFDC" >> $CYNCOLLOG
    fi
}
collect_DCLogs_AD72()
{
    # Creating DC logs directory
    trgDCLogsDir=$CYN_DCROOTDIR/$iAppSrvID/DCLogs/logs
    echo "Creating $trgDCLogsDir" >> $CYNCOLLOG
    mkdir -p $trgDCLogsDir || exit 1

    # Creating DC FFDCs directory
    trgDCFFDCsDir=$CYN_DCROOTDIR/$iAppSrvID/DCLogs/FFDC
    echo "Creating $trgDCFFDCsDir" >> $CYNCOLLOG
    mkdir -p $trgDCFFDCsDir || exit 1

    # Copying logs
    if [ -d "$iDCLogs" ]
    then
      echo "Copying $iDCLogs/* to $trgDCLogsDir" >> $CYNCOLLOG
      cp -Rpf $iDCLogs/* $trgDCLogsDir
    else
      echo "Unable to find $iDCLogs" >> $CYNCOLLOG
    fi
    
    # Copying FFDCs
    if [ -d "$iDCFFDC" ]
    then
      cd $iDCFFDC
      FFDCs=`ls 2>&1`
      cd $OLDPWD
      if [ -n "$FFDCs" ]
      then
        echo "!!! Copying $iDCFFDC/* to $trgDCFFDCsDir" >> $CYNCOLLOG
        cp -Rpf $iDCFFDC/* $trgDCFFDCsDir 2>&1
      else
        echo "!!! No FFDCs found under $iDCFFDC" >> $CYNCOLLOG
      fi
    else
      echo "Unable to find $iDCFFDC" >> $CYNCOLLOG
    fi
}

collect_WAS_info_AD72_DCLess()
{
	# Creating main server directory
	trgWASDir=$CYN_DCROOTDIR/DCLessLogs/${iAppSrvID}_DCLess_${iTempWASProfile}_${iWASCell}_${iWASNode}_${iWASServer}/WAS
	echo "Creating $trgWASDir" >> $CYNCOLLOG
	mkdir -p $trgWASDir || exit 1
  
	trgtCellDir=$trgWASDir/config/cells/$iWASCell
	trgtNodeDir=$trgWASDir/config/cells/$iWASCell/nodes/$iWASNode
	if [ -z "$isLiberty" ]; then
		trgtServerDir=$trgWASDir/config/cells/$iWASCell/nodes/$iWASNode/servers/$iWASServer
	else
		trgtServerDir=$trgWASDir/servers/$iWASServer
	fi
 
	echo "
  trgtCellDir=$trgtCellDir
  trgtNodeDir=$trgtNodeDir
  trgtServerDir=$trgtServerDir" >> $CYNCOLLOG
   
	if [ -z "$isLiberty" ]; then
		trgtServerDir=$trgWASDir/config/cells/$iWASCell/nodes/$iWASNode/servers/$iWASServer
	else
		trgtServerDir=$trgWASDir/servers/$iWASServer
	fi

	echo "
  trgtCellDir=$trgtCellDir
  trgtNodeDir=$trgtNodeDir
  trgtServerDir=$trgtServerDir" >> $CYNCOLLOG

	if [ -z "$isLiberty" ]; then
		echo "Creating $trgtServerDir" >> $CYNCOLLOG
		mkdir -p $trgtServerDir || exit 1

		if [ -e "$iCvariablesXML" ]
		then
			echo "Copying $iCvariablesXML to $trgtCellDir" >> $CYNCOLLOG
			cp -Rpf $iCvariablesXML $trgtCellDir
		fi

		if [ -e "$iNvariablesXML" ]
		then
			echo "Copying $iNvariablesXML to $trgtNodeDir" >> $CYNCOLLOG
			cp -Rpf $iNvariablesXML $trgtNodeDir
		fi

		if [ -e "$iNServerIndex" ]
		then
			echo "Copying $iNServerIndex to $trgtNodeDir" >> $CYNCOLLOG
			cp -Rpf $iNServerIndex $trgtNodeDir
		fi

		if [ -e "$iSvariablesXML" ]
		then
			echo "Copying $iSvariablesXML to $trgtServerDir" >> $CYNCOLLOG
			cp -Rpf $iSvariablesXML $trgtServerDir
		fi
	fi

	if [ -e "$iSServerXML" ]
	then
		echo "Copying $iSServerXML to $trgtServerDir" >> $CYNCOLLOG
		cp -Rpf $iSServerXML $trgtServerDir
	fi

	if [ -e "$iLogRoot" ]
	then
		echo "Copying $iLogRoot to $trgWASDir" >> $CYNCOLLOG
		cp -Rpf $iLogRoot $trgWASDir
		remove_extra_logs $iLogRoot $iWASServer $trgWASDir
	fi
}

collect_WAS_info_AD72()
{
  # Creating main server directory
  trgWASDir=$CYN_DCROOTDIR/$iAppSrvID/WAS
  echo "Creating $trgWASDir" >> $CYNCOLLOG
  mkdir -p $trgWASDir || exit 1
  
  trgtCellDir=$trgWASDir/config/cells/$iWASCell
  trgtNodeDir=$trgWASDir/config/cells/$iWASCell/nodes/$iWASNode
 if [ -z "$isLiberty" ]; then
  trgtServerDir=$trgWASDir/config/cells/$iWASCell/nodes/$iWASNode/servers/$iWASServer
 else
  trgtServerDir=$trgWASDir/servers/$iWASServer
 fi
 
  echo "
  trgtCellDir=$trgtCellDir
  trgtNodeDir=$trgtNodeDir
  trgtServerDir=$trgtServerDir" >> $CYNCOLLOG
   
 if [ -z "$isLiberty" ]; then
  trgtServerDir=$trgWASDir/config/cells/$iWASCell/nodes/$iWASNode/servers/$iWASServer
 else
  trgtServerDir=$trgWASDir/servers/$iWASServer
 fi

  echo "
  trgtCellDir=$trgtCellDir
  trgtNodeDir=$trgtNodeDir
  trgtServerDir=$trgtServerDir" >> $CYNCOLLOG

  echo "Creating $trgtServerDir" >> $CYNCOLLOG
  mkdir -p $trgtServerDir || exit 1


 if [ -z "$isLiberty" ]; then

  echo "Copying $iCvariablesXML to $trgtCellDir" >> $CYNCOLLOG
  cp -Rpf $iCvariablesXML $trgtCellDir

  echo "Copying $iNvariablesXML to $trgtNodeDir" >> $CYNCOLLOG
  cp -Rpf $iNvariablesXML $trgtNodeDir

  echo "Copying $iNServerIndex to $trgtNodeDir" >> $CYNCOLLOG
  cp -Rpf $iNServerIndex $trgtNodeDir

  echo "Copying $iSvariablesXML to $trgtServerDir" >> $CYNCOLLOG
  cp -Rpf $iSvariablesXML $trgtServerDir

 fi
  echo "Copying $iSServerXML to $trgtServerDir" >> $CYNCOLLOG
  cp -Rpf $iSServerXML $trgtServerDir

  echo "Copying $iLogRoot to $trgWASDir" >> $CYNCOLLOG
  cp -Rpf $iLogRoot $trgWASDir
  remove_extra_logs $iLogRoot $iWASServer $trgWASDir
}


collect_runtimeAndWAS_info_AD72()
{
  echo "Iterating thru the list of configured App.Servers" >> $CYNCOLLOG
  echo "...sel_appServers=$sel_appServers" >> $CYNCOLLOG
  for configDC in $sel_appServers
  do
  	if [ -s "$configDC" ]
    then
      unset isLiberty
      echo "...configDC=$configDC" >> $CYNCOLLOG
      iWASTmpVersion=`grep '# WAS_VERSION=' $configDC | cut -d= -f2 | tr -d '[:cntrl:]'`
      iWASNode=`grep '# WAS_NODE=' $configDC | cut -d= -f2 | tr -d '[:cntrl:]'`
      iWASServer=`grep '# WAS_SERVER=' $configDC | cut -d= -f2 | tr -d '[:cntrl:]'`
      iWASCell=`grep '# WAS_CELL=' $configDC | cut -d= -f2 | tr -d '[:cntrl:]'`
      iWASProfile=`grep '# WAS_HOMEPROFILE=' $configDC | cut -d= -f2 | tr -d '[:cntrl:]'`
      iConfigRoot=`grep '# CONFIG_ROOT=' $configDC | cut -d= -f2 | tr -d '[:cntrl:]'`   
      if grep -q "server.systemproperty=liberty.home=" $configDC 
      then
        isLiberty=y
      fi
 
      if [ "$iConfigRoot" = "None" ]
      then
        iConfigTmpRoot=`grep '# WAS_HOMEPROFILE=' $configDC | cut -d= -f2 | tr -d '[:cntrl:]'`
        iConfigRoot=$iConfigTmpRoot/$CONFIG
      fi
    
      iWASProfileName=`basename $iWASProfile`
      iDCHome=`grep 'server.variable=ITCAMDCHOME=' $configDC | cut -d= -f3 | tr -d '[:cntrl:]'`

      if [ "$iWASTmpVersion" = "6" -o "$iWASTmpVersion" = "7" -o "$iWASTmpVersion" = "8" ]
      then
        iWASVersion="$iWASTmpVersion$ZERO"
      else
        iWASVersion="$iWASTmpVersion"
      fi
    
 	  # if iConfigRoot = None then use WAS_HOMEPROFILE + '/config
      if [ -z "$isLiberty" ]; then
         iLogRoot=`grep '# LOG_ROOT=' $configDC | cut -d= -f2 | tr -d '[:cntrl:]'` 
      else
         iLogRoot=$iWASProfile/servers/$iWASServer
      fi
      iDCRuntime=`grep server.runtime= $configDC | cut -d= -f2 | sed 's/\(.*\/runtime\).*/\1/'| tr -d '[:cntrl:]'`
      if [ ! -d "$iDCRuntime" ]
      then
        iDCRuntime=$iDCHome/runtime/
      fi
      #iDCRuntime=$iDCHome/runtime/was$iWASVersion.$iWASCell.$iWASNode.$iWASProfileName.$iWASServer
      iAppSrvID=$iWASNode.$iWASServer
      iDCLogs=$PRODUCT_PATH/logs/CYN/logs/$iAppSrvID
      iDCFFDC=$PRODUCT_PATH/logs/CYN/FFDC/$iAppSrvID
    
      iCvariablesXML=$iConfigRoot/cells/$iWASCell/variables.xml
      iNvariablesXML=$iConfigRoot/cells/$iWASCell/nodes/$iWASNode/variables.xml
      iNServerIndex=$iConfigRoot/cells/$iWASCell/nodes/$iWASNode/serverindex.xml
      iSvariablesXML=$iConfigRoot/cells/$iWASCell/nodes/$iWASNode/servers/$iWASServer/variables.xml
      if [ -z "$isLiberty" ]; then
         iSServerXML=$iConfigRoot/cells/$iWASCell/nodes/$iWASNode/servers/$iWASServer/server.xml
         iServerPath=$iConfigRoot/cells/$iWASCell/nodes/$iWASNode/servers/$iWASServer
      else
         iSServerXML=$iWASProfile/servers/$iWASServer/server.xml
         iServerPath=$iWASProfile/servers/$iWASServer
      fi


  echo "
      iWASVersion=$iWASVersion
      iWASNode=$iWASNode
      iWASServer=$iWASServer
      iWASCell=$iWASCell
      iWASProfile=$iWASProfile
      iConfigRoot=$iConfigRoot
      iLogRoot=$iLogRoot
      iDCRuntime=$iDCRuntime
    
      iRunTime=$iRunTime
      iAppSrvID=$iAppSrvID
      iDCLogs=$iDCLogs
      iDCFFDC=$iDCFFDC
    
      iCvariablesXML=$iCvariablesXML
      iNvariablesXML=$iNvariablesXML
      iNServerIndex=$iNServerIndex
      iSvariablesXML=$iSvariablesXML
      iSServerXML=$iSServerXML" >> $CYNCOLLOG
    
      collect_runtime_info_AD72
      collect_DCLogs_AD72
      collect_WAS_info_AD72 
    fi   
  done

	echo \* Collecting DC-less\'s information
	###RETRIEVE VALUES FROM CONNECTIONS.PROPERTIES###
	iDCVersion=7.3.0.5.0
	#iDCVersion=@cdc.version@
	iTempDCHome=/opt/ibm/ccm/agent/install-images/kyn/dchome/$iDCVersion
	CONN_PROPS=$iTempDCHome/runtime/custom/connections.properties
	if [ -f $CONN_PROPS ] 
	then
		#Now get the connections list from the file
		for i in `grep connectionstring $CONN_PROPS`
		do
			#Break on = but -f2- in case there are other = signs (such as the encoded version of the wasadmin password)
			line=`echo $i | cut -d= -f2-`
			iWASVersion=`echo $line | cut -d\| -f1 | sed "s/was//g"`
			iWASNode=`echo $line | cut -d\| -f5`
			iWASServer=`echo $line | cut -d\| -f7`
			iWASCell=`echo $line | cut -d\| -f4`
			iTempWASProfile=`echo $line | cut -d\| -f6`
			tmpWASHome=`echo $line | cut -d\| -f11`
			profileRegistry=$tmpWASHome/properties/profileRegistry.xml
			if [ -f $profileRegistry ]
			then
				tempProfileLine=`grep $iTempWASProfile $profileRegistry`
				iWASProfile=`echo $tempProfileLine | grep -o path=.*\"\  | sed "s/ $//g" | cut -d= -f2 | sed -e "s/^\"//g" -e "s/\"$//g"`
			fi

			#Need to figure out if server alreadys exists
			#If it does then there will be an entry in the log file of the form
			#AppSrv03.CO9128108221Node02Cell.CO9128108221Node04.server1.input.properties
			if [ "`grep $iTempWASProfile\.$iWASCell\.$iWASNode\.$iWASServer\.input\.properties $CYNCOLLOG`" = "" ]
			then
				iConfigRoot=$iWASProfile/config
				iLogRoot=$iWASProfile/logs
                iDCRuntime=$iTempDCHome/runtime/
				iRunTime=

				#We need to retrieve was.nodename and was.servername from $iTempDCHome/jmxdc/etc/jmxdcInputs.txt
				iTempJMXProfile=`grep ^was.nodename $iTempDCHome/jmxdc/etc/jmxdcInputs.txt | cut -d= -f2`
				iTempJMXServer=`grep ^was.servername $iTempDCHome/jmxdc/etc/jmxdcInputs.txt | cut -d= -f2`
				iAppSrvID=$iTempJMXProfile.$iTempJMXServer
				iDCLogs=$iLogRoot/CYN/logs/$iAppSrvID
				iDCFFDC=$iLogRoot/CYN/FFDC/$iAppSrvID
				tempWASCell=$iWASProfile/config/cells/$iWASCell
				tempWASNode=$tempWASCell/nodes/$iWASNode
				iCvariablesXML=$tempWASCell/variables.xml
				iNvariablesXML=$tempWASNode/variables.xml
				iNServerIndex=$tempWASNode/serverindex.xml
				iSvariablesXML=$tempWASNode/servers/$iWASServer/variables.xml
				iSServerXML=$tempWASNode/servers/$iWASServer/server.xml

				echo iWASVersion=$iWASVersion>> $CYNCOLLOG
				echo iWASNode=$iWASNode>> $CYNCOLLOG
				echo iWASServer=$iWASServer>> $CYNCOLLOG
				echo iWASCell=$iWASCell>> $CYNCOLLOG
				echo iWASProfile=$iWASProfile>> $CYNCOLLOG
				echo tmpWASHome=$tmpWASHome>> $CYNCOLLOG
				echo iConfigRoot=$iConfigRoot>> $CYNCOLLOG
				echo iLogRoot=$iLogRoot>> $CYNCOLLOG
				echo iDCRuntime=$iDCRuntime>> $CYNCOLLOG
				echo >> $CYNCOLLOG
				echo iRunTime=$iRunTime>> $CYNCOLLOG
				echo iAppSrvID=$iAppSrvID>> $CYNCOLLOG
				echo iDCLogs=$iDCLogs>> $CYNCOLLOG
				echo iDCFFDC=$iDCFFDC>> $CYNCOLLOG
				echo >>$CYNCOLLOG	
				echo iCvariablesXML=$iCvariablesXML>> $CYNCOLLOG
				echo iNvariablesXML=$iNvariablesXML>> $CYNCOLLOG
				echo iNServerIndex=$iNServerIndex>> $CYNCOLLOG
				echo iSvariablesXML=$iSvariablesXML>> $CYNCOLLOG
				echo iSServerXML=$iSServerXML >> $CYNCOLLOG

      				collect_runtime_info_AD72_DCLess
      				collect_DCLogs_AD72_DCLess
      				collect_WAS_info_AD72_DCLess 
			else
				echo $iTempWASProfile $iWASCell $iWASNode $iWASServer has already been collected by the DC collection
			fi
		done
		collect_DCLessLogs_AD72_DCLess
	fi
}

analyze_inputsFile_AD72 ()
{
  echo "Analyzing all App.Servers found for this DC installation" >> $CYNCOLLOG

  if [ -n "$APP_SERVER_NAME" ]
  then
    echo "Specified App.Server is $APP_SERVER_NAME" >> $CYNCOLLOG
    
    for currInpFile in $configuredAppServers
    do
      echo "...Checking $currInpFile" >> $CYNCOLLOG
     
      currServer=`grep '# WAS_SERVER=' $currInpFile | cut -d= -f2 | tr -d '[:cntrl:]'`
      
      echo "...currServer=[$currServer]" >> $CYNCOLLOG
      echo "...APP_SERVER_NAME=[$APP_SERVER_NAME]" >> $CYNCOLLOG
      if  [ "$currServer" = $APP_SERVER_NAME ]
      then
        sel_appServers="$sel_appServers$currInpFile"
        echo "...match found." >> $CYNCOLLOG
      fi
    done
    
    echo "sel_appServers=$sel_appServers" >> $CYNCOLLOG
      
    if [ -z "$sel_appServers" ]
    then
      echo "...Error: The specified App.Server ($APP_SERVER_NAME) has not been configured. Unable to collect related information." 
      echo "...Error: The specified App.Server ($APP_SERVER_NAME) has not been configured. Unable to collect related information." >> $CYNCOLLOG
        
      exit 1 
    fi
  else
    echo "User selected all App.Servers from this installation" >> $CYNCOLLOG
    sel_appServers=$configuredAppServers
  fi

  collect_runtimeAndWAS_info_AD72
}
collect_dc_info_AD72 ()
{
  collect_configuration_AD72
  collect_etc_AD72
  if [ -z "$ALL_SERVERS" -a -z "$APP_SERVER_NAME" ]
  then
    echo "No App.Server(s) specified, therefore no further collection will take place." >> $CYNCOLLOG
  else
    analyze_inputsFile_AD72
  fi
}


# #########################################################################
# #########################################################################
# #########################################################################
# #########################################################################
#
#                            Implementation
#
# #########################################################################
# #########################################################################
# #########################################################################
# #########################################################################
#                        Analyzing Configuration
# #########################################################################

CONFIGURED=
runtimeDIRS=
dcConfigCELLS=

echo "Searching for configured Data Collectors" >> $CYNCOLLOG

if [ "$PROD_TYPE" = "AD72" ]
then
  echo "Structure type is AD7.2" >> $CYNCOLLOG
  dcConfigCELLSdir=$PRODUCT_PATH/runtime
elif [ -d "$PRODUCT_PATH/config/configured" ]
then
  echo "Structure type is config/configured" >> $CYNCOLLOG
  dcConfigCELLSdir=$PRODUCT_PATH/config/configured
elif [ -d "$PRODUCT_PATH/7.1/installer/configured" ]
then
  echo "Structure type is 7.1/installer/configured" >> $CYNCOLLOG
  dcConfigCELLSdir=$PRODUCT_PATH/7.1/installer/configured
elif [ "$eCAM" ]
then
  # eCAM mode
  echo "Structure type is 7.1/_uninst/configured" >> $CYNCOLLOG
  dcConfigCELLSdir=$PRODUCT_PATH/_uninst/configured
  dcConfigCELLS=`ls $dcConfigCELLSdir/cells`
fi

WASCE_DIR=$dcUnistConfigDir/wasce

echo "
------------------------------------
  Detecting Configuration Status...
------------------------------------" >> $CYNCOLLOG

CONFIGURED=
if [ "$PROD_TYPE" = "AD72" ]
then
  echo "Looking for configured DC's in AD72 mode" >> $CYNCOLLOG
  
  echo "dcConfigCELLSdir=$dcConfigCELLSdir" >> $CYNCOLLOG

  if [ -d "$dcConfigCELLSdir" ]
  then
    configuredAppServers=`ls $dcConfigCELLSdir/*.input.properties 2>&1`
    echo "configuredAppServers=$configuredAppServers" >> $CYNCOLLOG
  
    if [ -n "$configuredAppServers" ]
    then
      CONFIGURED=true
    fi
  fi

  if [ -z "$CONFIGURED" ]
  then
    echo "No App.Server has been configured with this DC"
    echo "No App.Server has been configured with this DC" >> $CYNCOLLOG
  fi
else
  echo "Looking for configured DC's in standard mode" >> $CYNCOLLOG
  configuredAppServers=`ls $dcConfigCELLSdir`
fi

echo "configuredAppServers=$configuredAppServers" >> $CYNCOLLOG

if [ "$PROD_TYPE" != "AD72" ]
then
  if [ -n "$configuredAppServers" ]
  then
    echo "Starting to search for runtimes" >> $CYNCOLLOG
    echo "eCAM=$eCAM" >> $CYNCOLLOG

    for runtimeDirs in $configuredAppServers
    do
      echo "$runtimeDirs was found" >> $CYNCOLLOG
      if [ "$eCAM" ]
      then
        echo "$runtimeDirs was found" >> $CYNCOLLOG
        CONFIGURED=`find $dcConfigCELLSdir/cells | grep dcInputs.txt`
        RUNTIME=$PRODUCT_PATH/runtime
      else
        echo "  looking for dcInput.txt under $runtimeDirs" >> $CYNCOLLOG
        if [ -f "$dcConfigCELLSdir/$runtimeDirs/dcInput.txt" ]
        then
          echo "  dcInput.txt found" >> $CYNCOLLOG
          CONFIGURED=true
        fi
      fi
    done
  fi

  if [ -d "$RUNTIME" ]
  then
    echo "!!! Runtime directory located..." >> $CYNCOLLOG
    runtimeDIRS=`ls $RUNTIME`

    echo "runtimeDIRS=$runtimeDIRS" >> $CYNCOLLOG

    if [ -n "$runtimeDIRS" ]
    then
      echo "!!! Runtime directory contains subdirectories..." >> $CYNCOLLOG
    fi
  else
    echo "!!! Runtime directory not located..." >> $CYNCOLLOG
  fi
fi

echo "CONFIGURED=$CONFIGURED
dcUnistConfigDir=$dcUnistConfigDir
dcConfigCELLSdir=$dcConfigCELLSdir
WASCE_DIR=$WASCE_DIR
" >> $CYNCOLLOG

# #########################################################################
#               Start collecting information
# #########################################################################

if [ "$PROD_TYPE" != "AD72" ]
then
  process_collectWASInfo_classic
fi

# ####################################
#   Collect system information
# ####################################
echo "* Collecting general information"
echo "   ...Collecting system information"

echo "
------------------------------------
  Collecting general information...              
------------------------------------" >> $CYNCOLLOG

echo "cd $PRODUCT_PATH" >> $CYNCOLLOG
cd $PRODUCT_PATH || exit 1

WHOAMI=
OSBits=
GROUPS_CMD=
DOMAIN_CMD=
PROC_INFO=
TOP=
WHICHTOP=
PRSTAT=
SHOWREV=
DMESG=
NETSTAT_OPT="-a -n"

GROUPS_CMD='groups'
DOMAIN_CMD='domainname'
HOSTNAME=`hostname 2>&1`

echo "OS_TYPE=$OS_TYPE" >> $CYNCOLLOG
if [ "$OS_TYPE" = "OS400" ]
then
  WHOAMI=`id -un`
  GROUPS_CMD=`id -Gn`
  HOSTNAME=`uname -n`
else
  WHOAMI=`whoami 2>/dev/null`
  [ -z "$WHOAMI" ] && WHOAMI=`id | cut -d"(" -f2 | cut -d")" -f1`
fi

echo "GROUPS_CMD=$GROUPS_CMD
DOMAIN_CMD=$DOMAIN_CMD
HOSTNAME=$HOSTNAME
WHOAMI=$WHOAMI
OS_TYPE=$OS_TYPE" >> $CYNCOLLOG

case "$OS_TYPE" in
  HP-UX)        
      OSBits=`getconf KERNEL_BITS 2>&1`
      TOP="top -d1 -u -h"
      SHOWREV="swlist -v"
      ;;
    AIX)
      OS_VERSION=`lslpp -L bos.rte.boot | grep Boot`     # boot fileset version
      OS_VERSION=`echo $OS_VERSION | cut -f 2 -d " "`     # Get the version number
      [ -z "$OS_VERSION" ] && OS_VERSION=`oslevel`        # if fails, use oslevel

      Kernel=`lsconf -k 2>/dev/null`
      [ -z "$Kernel" ] || OSBits=`echo "$Kernel" | cut -d " " -f3 | cut -c-2 2>/dev/null`
      [ -z "$OSBits" ] && OSBits=`getconf KERNEL_BITMODE 2>&1`

      PROC_INFO=`lscfg -l "proc*" 2>/dev/null`
      SHOWREV="lslpp -hav"
      ;;
  Linux)
      case "$OS_ARCH" in
        s390x|ppc64)
                OSBits=64
                ;;
              *)
                OSBits=32
                ;;
      esac

      [ -r /proc/cpuinfo ] && PROC_INFO=`cat /proc/cpuinfo`

      DMESG=dmesg
      TOP="top b n1"
      NETSTAT_OPT="$NETSTAT_OPT -p"
      SHOWREV="rpm -qaiv"
    ;;
  SunOS)
      OSBits=`isainfo -b 2> /dev/null`
      PROC_INFO=`psrinfo && echo && isainfo -vk && isainfo -v`

        WHICHTOP=`which top | grep "no top"`

        if [ -z "$WHICHTOP" ]
        then
        TOP="top -n"
        else
          PRSTAT=`which prstat | grep "no prstat"`
       
          if [ -z "$PRSTAT" ]
          then
            TOP="prstat 5 1"
          else
            TOP="echo No top or prstat command found on this host."
          fi
        fi

      SHOWREV="showrev -a"
      DMESG="dmesg"
    ;;
  OS400)
        SHOWREV="system DSPSFWRSC"
        ;;
esac

cat > $SYS_INFO << EOF
OS type     : $OS_TYPE 
OS version  : $OS_VERSION
OS bits     : $OSBits

User        : $WHOAMI
Group       : $GROUPS_CMD
Host name   : $HOSTNAME
Domain name : $DOMAIN_CMD
Date        : `date 2>&1`
Runtime     : $PRODUCT_PATH
Common      : $COMMLOGSDIR

==> uname:
`uname -a 2>&1`

==> ulimit:
`ulimit -a 2>&1`
EOF

if [ -n "$PROC_INFO" ]
then
cat >> $SYS_INFO << EOF

==> Processor(s) :
$PROC_INFO
EOF
fi

case "$OS_TYPE" in
  Linux)
      echo >> $SYS_INFO
      echo "==> Memory:" >> $SYS_INFO
      free >> $SYS_INFO
      echo >> $SYS_INFO

      if [ -r /proc/net/sockstat ]
        then
        echo "==> Socket state:" >> $SYS_INFO
        cat /proc/net/sockstat >> $SYS_INFO
      fi
      ;;
esac

echo >> $SYS_INFO

echo "   ...Collecting environment information"
ls -laRL $PRODUCT_PATH >> $DIR_INFO
env > $ENV_INFO 2>&1

if [ "$OS_TYPE" != "OS400" ]
then
  echo "   ...Collecting disk space information"
  df -k > $DISK_INFO 2>&1
else
  echo "   ...Collecting system status"
  system "wrksyssts" > $DISK_INFO 2>&1
fi

echo "   ...Collecting network state"
case "$OS_TYPE" in
  OS400)
        cp -f /etc/netconfig $NET_CGF 2>/dev/null
        cp -f /etc/NETGROUP $NET_GRP 2>/dev/null
        ;;
      *)
        netstat $NETSTAT_OPT > $NET_INFO 2>&1
        cp -f /etc/hosts $HOSTS_INFO 2>/dev/null
      ;;
esac

if [ -n "$TOP" ]; then
  echo "   ...Collecting system load information"
  $TOP > $TOP_INFO 2>&1
fi

if [ -n "$DMESG" ]; then
  echo "   ...Collecting kernel ring buffer"
  $DMESG > $DMESG_INFO 2>&1
fi

if [ -n "$SHOWREV" ]; then
  echo "   ...Collecting software revision information"
  $SHOWREV > $REV_INFO 2>&1
fi

# ####################################
#       Collect DC information
# ####################################

echo "* Collecting DC's information"
echo "
------------------------------------
     Collect DC information...              
------------------------------------" >> $CYNCOLLOG

if [ "$PROD_TYPE" != "AD72" ]
then
  if [ "$eCAM" ]
  then
    level=''
    collect_dc_maintenance_config
    collect_dc_maintenance_logs
  else
    for level in `ls $PRODUCT_PATH | egrep "^7\.([0-9]+)(\.([0-9]+))?(\.([0-9]+))?(\.([0-9]+))?$"`
    do
      if [ -d "$level/itcamdc/etc" ]
      then
        collect_dc_maintenance_config
        collect_dc_maintenance_logs
      fi
    done
  fi
else
  collect_dc_info_AD72
fi

if [ -d "$PRODUCT_PATH/config" ]
then
  cp -Rpf $PRODUCT_PATH/config $CYN_DCROOTDIR/
fi

echo "
------------------------------------
     Done collection DC information              
------------------------------------" >> $CYNCOLLOG

# ####################################
#    Packing collected information
# ####################################

echo "* Packing collected information"

JAR_PROC="$JAVA_EXE sun.tools.jar.Main"
PACKAGE=$CYN_TRGT_HOME/$CYNJAR
PKG_CMD="$JAR_PROC -cMf $CYNJAR $CYN_WRKDIR"

if [ -n "$TAR_CMD" ]
then
  echo "We are using tar command, so we are switching package and command" >> $CYNCOLLOG
  PACKAGE=$CYN_TRGT_HOME/$CYNTAR
  PKG_CMD="/QOpenSys/usr/bin/tar -cf $CYNTAR $CYN_WRKDIR"
fi

echo "
------------------------------------
  Packing collected information...              
------------------------------------
JAR_PROC=$JAR_PROC
PACKAGE=$PACKAGE

Packing will be executed as: $PKG_CMD
" >> $CYNCOLLOG

cp $CYNCOLLOG $CYN_DCDIR

cd $CYN_TRGT_HOME || exit 1

$PKG_CMD || exit 1

rm -Rf $CYN_WRKDIR

echo "
********************************************************************
$PACKAGE
This file has been created and contains diagnostic information.
Forward this binary file to your IBM Support representative.
********************************************************************
"
cd $CURR_DIR
