@echo off

@REM #####################################################################         
@REM # Licensed Materials - Property of IBM
@REM #  
@REM # 5724-L62  
@REM #
@REM # (C) Copyright IBM Corp. 2013 All Rights Reserved.       
@REM #
@REM # US Government Users Restricted Rights - Use, duplication or
@REM # disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
@REM # 
@REM ##################################################################### 

setLocal enableExtensions enableDelayedExpansion
cd /d %~dp0

@REM #########################################################
@REM # Configure these environment specific properties       #
@REM #########################################################

@REM There should be no such modifiables

@REM ################################################
@REM # End of user configured environment variables #
@REM ################################################

@REM The ITCAMDCVERSION is constant for a given fix level
set ITCAMDCVERSION=7.3.0.10.0

@REM Parse the command line arguments and extract the -temaport=<value> keyword
set TEMA_PORT=
set ARGS=%*
:parseloop
@REM Break the arguments into the next token and the rest of the line at the first blank
for /F "tokens=1* delims= " %%A in ("%ARGS%") do (
  @REM Break the token into keyword and argument at the first equals
  for /F "tokens=1* delims==" %%C in ("%%A") do (
    if "%%C" == "-temaport" set "TEMA_PORT=%%D"
  )
  @REM The remainder of the line
  set "ARGS=%%B"
)
@REM Continue with next optionn until nothing left
if defined ARGS goto parseloop

@REM If we didn't get a TEMA_PORT, use the default value
if not defined TEMA_PORT set TEMA_PORT=63335

@REM Locate and normalize the DC home
if not exist "%~dp0../../jmxdc" (
  echo *Error*  JMX home not found, execution impossible
  exit /b 1
)
setlocal
cd /d %~dp0../..
endlocal & set ITCAMDCHOME=%CD%

set JAVA_HOME=
set JMX_PORT=
if not exist "%ITCAMDCHOME%\runtime\custom\connections.properties" (
  echo *Error*  No connections properties file found, configuration required
  exit /b 1
)

for /F "usebackq tokens=1* delims==" %%A in ("%ITCAMDCHOME%\runtime\custom\connections.properties") do (
  if "%%A" == "java.home"  set "JAVA_HOME=%%B"
  if "%%A" == "jmx.port"   set "JMX_PORT=%%B"
)
if not defined JAVA_HOME (
  echo *Error*  java.home missing from connections property file
  exit /b 1
)
if not exist "!JAVA_HOME!\bin\java.exe" (
  echo *Error*  java.home does not contain a valid Java executable
  exit /b 1
)
if not defined JMX_PORT set JMX_PORT=63355

@REM If there is no JMX DC inputs file in runtime, we must create one.

if exist "%ITCAMDCHOME%\runtime\jmxdcInputs.txt" (
  echo *Info*  Overwriting existing JMX DC inputs file
) else (
  echo *Info*  Creating new JMX DC inputs file in runtime
)

if not exist "%ITCAMDCHOME%\runtime" md "%ITCAMDCHOME%\runtime"
echo # Created by JMXServer.bat %DATE% %TIME%>"%ITCAMDCHOME%\runtime\jmxdcInputs.txt"
for /F "usebackq tokens=* delims=" %%L in ("%ITCAMDCHOME%\jmxdc\etc\jmxdcInputs.txt") do (
  set "line=%%L"
  if "!line:~0,11!" == "itcam.home=" set "line=itcam.home=%ITCAMDCHOME%"
  if "!line:~0,10!" == "tema.port=" set "line=tema.port=%TEMA_PORT%"
  echo !line!>>"%ITCAMDCHOME%\runtime\jmxdcInputs.txt"
)

@REM The following dummy NODE_NAME and AGENT_NAME values will be used to populate JMXServer runtime directory. 
set NODE_NAME=ibmjmxdcnode
set AGENT_NAME=jmxserver

@REM ######################################################################
@REM # No need for the user to touch the following. Needs some refinement #
@REM ######################################################################

set HOST=localhost

set RUNTIME_HOME=%ITCAMDCHOME%\runtime\JMXProfile.jmxdc.%NODE_NAME%.%AGENT_NAME%

copy "%ITCAMDCHOME%\jmxdc\etc\apdstore" "%ITCAMDCHOME%\runtime\" >nul
copy "%ITCAMDCHOME%\jmxdc\etc\itcam_jd.properties" "%ITCAMDCHOME%\" >nul

set AM_HOME=%ITCAMDCHOME%\itcamdc

@REM We presume that the Java home is within the WebSphere home at the
@REM usual location.  If that is not true none of this will work.

@REM We can find the jar directories based relative to the assumed
@REM location of the Java home within the WebSphere home.

set RUNTIMES_DIR=%JAVA_HOME%\..\runtimes
if not exist "%RUNTIMES_DIR%" (
  echo *Error*  Unable to locate WebSphere runtime directory
  exit /b 1
)
set PLUGINS_DIR=%JAVA_HOME%\..\plugins
if not exist "%PLUGINS_DIR%" (
  echo *Error*  Unable to locate WebSphere plugins directory
  exit /b 1
)

@REM Normalize the paths for tidyness
setlocal
cd /d %RUNTIMES_DIR%
endlocal & set RUNTIMES_DIR=%CD%
setlocal
cd /d %PLUGINS_DIR%
endlocal & set PLUGINS_DIR=%CD%

@REM Locate the WebSphere jars
set WASJARS=
for %%F in ("%RUNTIMES_DIR%\com.ibm.ws.admin.client*.jar") do set WASJARS="%%~F"
for %%F in ("%RUNTIMES_DIR%\com.ibm.ws.ejb.thinclient*.jar") do set WASJARS=!WASJARS!;"%%~F"
for %%F in ("%RUNTIMES_DIR%\com.ibm.ws.orb*.jar") do set WASJARS=!WASJARS!;"%%~F"
for  %%F in ("%RUNTIMES_DIR%\com.ibm.ws.ejb.embeddableContainer_*.jar") do (
  set JARFILE_NAME=%%~nF
  if "!JARFILE_NAME:nls=!" == "%%F" set WASJARS=!WASJARS!;"%%~F"
)
@REM Must use %%~F here to avoid too many quotes
for %%F in ("%PLUGINS_DIR%\com.ibm.ws.security.crypto.jar") do set WASJARS=!WASJARS!;"%%~F"

set TOOLKIT_JARS="%ITCAMDCHOME%\toolkit\lib\bcm-bootstrap.jar";"%ITCAMDCHOME%\toolkit\lib\tk_bci_engine.jar";"%ITCAMDCHOME%\toolkit\etc\toolkit_config_gpe.jar";"%ITCAMDCHOME%\toolkit\lib\ext\tk_ws_helpers.jar";"%ITCAMDCHOME%\toolkit\lib\ext\tk_ws_handlers.jar";"%ITCAMDCHOME%\toolkit\lib\ext\tk_thread_callbacks.jar";"%ITCAMDCHOME%\toolkit\lib\ext\tk_runtime.jar";"%ITCAMDCHOME%\toolkit\lib\tk_reqmap_extract.jar";"%ITCAMDCHOME%\toolkit\lib\tk_nodyninst.jar";"%ITCAMDCHOME%\toolkit\lib\ext\cdn.jar";"%ITCAMDCHOME%\toolkit\lib\ext\jmxengine.jar";"%ITCAMDCHOME%\toolkit\lib\ext\was\was6handler.jar";"%ITCAMDCHOME%\toolkit\lib\ext\was\websphereconnector.jar";"%ITCAMDCHOME%\toolkit\lib\ext\tk_jflt.jar";"%ITCAMDCHOME%\toolkit\lib\ext\copyright.jar";"%ITCAMDCHOME%\toolkit\lib\ext\tk_utils.jar";"%ITCAMDCHOME%\itcamdc\lib\ext\was\pmicollector.jar";"%ITCAMDCHOME%\toolkit\lib\ext\glswrapper.jar"

set TOOLKIT_GPEX_JARS="%ITCAMDCHOME%\plugins\com.ibm.tivoli.itcam.bundlemanager_7.2.0.jar";"%ITCAMDCHOME%\toolkit\lib\ext\was\gpex.mbeans_was.jar";"%ITCAMDCHOME%\toolkit\etc\toolkit_config_gpex.jar";"%ITCAMDCHOME%\toolkit\lib\ext\was\gpex.extractors_was.jar";"%ITCAMDCHOME%\toolkit\lib\ext\was\gpex.monitor_was.jar";"%ITCAMDCHOME%\toolkit\lib\ext\was\gpex.ws_handlers_was.jar";"%ITCAMDCHOME%\toolkit\lib\ext\was\gpex.runtimebuilder_washelper.jar";"%ITCAMDCHOME%\toolkit\lib\ext\gpex.turbo_intf.jar";"%ITCAMDCHOME%\toolkit\lib\ext\was\gpex.runtimebuilder_washelper.jar"

set YNJARS="%ITCAMDCHOME%\itcamdc\lib\ext\kwjmbeans.jar";"%ITCAMDCHOME%\itcamdc\lib\ext\was\kynmbeans.jar";"%ITCAMDCHOME%\toolkit\lib\ext\reqaggr.jar";"%ITCAMDCHOME%\itcamdc\lib\ext\kwjmbeans_extension.jar";"%JMX_LIB_HOME%\kwj.java\lib\kwjwb.jar";"%JMX_LIB_HOME%\kwj.java\lib\kwjplugin.jar";"%JMX_LIB_HOME%\kwj.java\lib\kwjwb-tools.jar";"%JMX_LIB_HOME%\kwj.java\lib\migration-tool.jar";"%JMX_LIB_HOME%\itm.common\lib\kynwb.jar";"%JMX_LIB_HOME%\cg.mbean\lib\itcam.cg.mbean.jar;"

set DCJARS="%ITCAMDCHOME%\toolkit\lib\ext\jflt.jar";"%ITCAMDCHOME%\toolkit\lib\ext\jlog.jar";"%ITCAMDCHOME%\toolkit\lib\ext\jffdc.jar";"%ITCAMDCHOME%\toolkit\lib\ext\jflt.jar";"%ITCAMDCHOME%\toolkit\lib\ext\gls.jar";"%ITCAMDCHOME%\itcamdc\lib\ext\jakarta-regexp-1.3.jar"

set MSDCJARS="%ITCAMDCHOME%\itcamdc\lib\ext\ppe.service.jar"

@REM TODO passing java options as below is not working, fix it later
@REM set DEFS="-Dkwj.tema.configDir=config -Dkwj.tema.configFile=kyn.cfg -Djlog.propertyFileDir=config -Djlog.propertyFile=/kynjlog.properties -Djlog.common.dir=. -Ditm.logging.configuration=/kwjlogging.properties;/kynlogging.properties -Dkwj.registry=config/kynwb.registry -Dkwj.app.builder=com.ibm.tivoli.monitoring.itcam.tema.websphere.zos.ZosItcamWebsphereAppBuilder -Dkwj.app.ipcserver=com.ibm.tivoli.monitoring.toolkit.workbench.ipc.msg.socket.SocketServer -Dkwj.logging.ras1handler.enabled=false -Dkwj.tema.DCUnconfigChecker.interval=5   -Dkwj.jmx.settings.deep.count=2 -Dkwj.jvm.stat.interval=20 -Dkwj.jmx.settings.timeout=30000"

@REM OPTS="-Xms96m -Xmx96m -Xgcpolicy:optavgpause"
@REM giving this in options was not letting the process start. Gave it directly in the command line
@REM set OPTS="-Xmx256m -Xdebug -Xrunjdwp:transport=dt_socket,server=y,address=9999,suspend=y"

@REM Add Java to PATH
path "%JAVA_HOME%\bin";%PATH%

@REM does not work
@REM javaw -classpath %YNJARS%;%TOOLKIT_JARS%;%TOOLKIT_GPEX_JARS% %OPTS% %DEFS% com.ibm.tivoli.itcam.tema.was.remote.WasJMXServer 

@REM stop all the existing JMXServer instances
FOR /F "skip=1" %%A IN ('wmic process where "name='java.exe' and commandline like '%%WasJMXServer%%'" get processid' ) DO (
  FOR /F %%B IN ("%%A") DO (
  echo *Info*  Stopping existing JMXServer instance %%B
  taskkill /f /pid %%B
  )
)

@REM TODO remove WAS properties, they may not be needed
java -Xmx256m -Xdebug -DXrunjdwp:transport=dt_socket,server=y,address=9999,suspend=n -Dapd.truststorekeytype=JKS -Dapd.truststorepassword=ibmtivoli -Dapd.truststorefile="%ITCAMDCHOME%\runtime\apdstore" -Djmxdcmode=true -Dcom.sun.management.jmxremote -Dcom.sun.management.jmxremote.port=%JMX_PORT% -Dcom.sun.management.jmxremote.authenticate=false -Dcom.sun.management.jmxremote.ssl=false -Dam.home="%AM_HOME%" -DWAS_SERVER_NAME=%AGENT_NAME% -Dam.wasserver=%AGENT_NAME% -Dam.wasnode=%HOST% -Dam.wascell=jmxdc -Dam.wasprofile=JMXProfile -Dam.nodename=%HOST% -Dappserver.platform=jmxdc -Dam.profilename=JMXProfile -Dam.appserver=%AGENT_NAME% -Dam.camtoolkit.gpe.alt.runtime="%RUNTIME_HOME%" -Dcom.ibm.tivoli.itcam.toolkit.runtime.dir="%RUNTIME_HOME%" -DITCAMDCHOME="%ITCAMDCHOME%" -DITCAMDCVERSION=%ITCAMDCVERSION% -Dam.camtoolkit.gpe.dc.operation.mode=JD -Dcom.ibm.tivoli.itcam.toolkit.ai.runtimebuilder.enable.rebuild=true  -Dcom.ibm.tivoli.toolkit.pmicollector.checkpoint.read=true -Dcom.ibm.tivoli.itcam.toolkit.ai.read.config.checkpoint=true -Ddisable.toolkit.configdir=true -Ddisable.native.lib=true -DDtoolkit.initialize=false -Druntimebuilder.product.masks="itcam_jd.properties" -classpath %YNJARS%;%TOOLKIT_JARS%;%TOOLKIT_GPEX_JARS%;%DC_JARS%;%WASJARS%;%DCJARS%;%MSDCJARS% com.ibm.tivoli.itcam.tema.was.remote.WasJMXServer

echo "JMXServer process exit error level %ERRORLEVEL%"
exit /b %ERRORLEVEL%
