#!/bin/sh

#####################################################################         
# Licensed Materials - Property of IBM
#  
# 5724-L62  
#
# (C) Copyright IBM Corp. 2013, 2014 All Rights Reserved.       
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
##################################################################### 

###### MODIFIABLE CONFIGURATION SECTION ######
# (there should be no such modifiables)
###### End of modifiable configuration section ######

ITCAMDCVERSION=7.3.0.10.0

###PARSE COMMAND LINE ARGUMENTS###
TEMA_PORT=63335
if [ "$#" != "0" ]
then
    for i in `echo $*`
    do
        key=`echo $i | cut -d= -f1`
        if [ "$key" = "-temaport" ]
        then
            value=`echo $i | cut -d= -f2`
            TEMA_PORT=$value
        fi
    done
fi
###END PARSE###

# Locate and normalize the DC home

if [ ! -d $(dirname $0)/../../jmxdc ]; then
  echo "*Error*  JMX home not found, execution impossible"
  exit 1
fi
ITCAMDCHOME=$(cd $(dirname $0)/../..;pwd)
cd $ITCAMDCHOME/jmxdc/bin

# Validate that the existence of the connections properties file

if [ ! -f $ITCAMDCHOME/runtime/custom/connections.properties ]; then
  echo "*Error*  No connections properties file found, configuration required"
  exit 1
fi

# Extract the JAVA_HOME for WebSphere from the connections properties file

JAVA_HOME=$(grep "^java.home=" $ITCAMDCHOME/runtime/custom/connections.properties | cut -d '=' -f2)
if [ -z "$JAVA_HOME" ]; then
  echo "*Error*  java.home missing from connections property file"
  exit 1
fi
if [ ! -x $JAVA_HOME/bin/java ]; then
  echo "*Error*  java.home does not contain a valid Java executable"
  exit 1
fi 
JAVA_BIN=$JAVA_HOME/bin/java

# Extract the JMX_PORT from the connections properties files

JMX_PORT=$(grep "^jmx.port=" $ITCAMDCHOME/runtime/custom/connections.properties | cut -d '=' -f2)
if [ -z "$JMX_PORT" ]; then
  JMX_PORT=63355
fi

# Extract the jmx.localhostip from the connections properties files
LOCALHOST_IP=$(grep "^jmx.localhostip=" $ITCAMDCHOME/runtime/custom/connections.properties | cut -d '=' -f2)
if [ -z "$LOCALHOST_IP" ]; then
  LOCALHOST_IP=127.0.0.1
fi

# If there is no JMX DC inputs file in runtime, we must create one.

if [ -f $ITCAMDCHOME/runtime/jmxdcInputs.txt ]; then
  echo "*Info*  Overwriting existing JMX DC inputs file"
else
  echo "*Info*  Creating new JMX DC inputs file in runtime"
fi

if [ ! -d $ITCAMDCHOME/runtime ]; then
  mkdir $ITCAMDCHOME/runtime
fi

# Copy the JMX DC inputs file (jmxdcInputs.txt) from the template and replace the relevant properties
sed "s%^itcam.home=.*%itcam.home=$ITCAMDCHOME%g" $ITCAMDCHOME/jmxdc/etc/jmxdcInputs.txt > $ITCAMDCHOME/runtime/jmxdcInputs.txt.1
sed "s/^tema.host=[^\n]*/tema.host=$LOCALHOST_IP/g" $ITCAMDCHOME/runtime/jmxdcInputs.txt.1 > $ITCAMDCHOME/runtime/jmxdcInputs.txt.2
sed "s/^tema.port=[^\n]*/tema.port=$TEMA_PORT/g" $ITCAMDCHOME/runtime/jmxdcInputs.txt.2 > $ITCAMDCHOME/runtime/jmxdcInputs.txt

WAS_SERVER_NAME=server1
AM_WAS_NODE=localhost
NODE_NAME=ibmjmxdcnode

###### This following property will be used to populate JMXDC runtime directory. ######
AM_CAMTOOLKIT_GPE_ALT_RUNTIME=$ITCAMDCHOME/runtime/JMXProfile.jmxdc.$NODE_NAME.jmxserver

############

###### STATIC PROPERTIES SECTION ######
AM_WASCELL=jmxdc
AM_WASPROFILE=JMXProfile
AM_WASSERVER=$WAS_SERVER_NAME
AM_APPSERVER=$WAS_SERVER_NAME
COM_IBM_TIVOLI_ITCAM_TOOLKIT_RUNTIME_DIR=$AM_CAMTOOLKIT_GPE_ALT_RUNTIME
AM_HOME=$ITCAMDCHOME/itcamdc

####SETUP WAS JARS####
#Need to find WAS jars based on the JAVA_HOME presumed to lie with WebSphere
WAS_BASE_DIR=$(dirname $JAVA_HOME)
WAS_RUNTIMES_DIR=$WAS_BASE_DIR/runtimes
WAS_PLUGINS_DIR=$WAS_BASE_DIR/plugins
ADMIN_CLIENT_JAR=$(find $WAS_RUNTIMES_DIR -name "com.ibm.ws.admin.client*.jar")
ADMIN_THINCLIENT_JAR=$(find $WAS_RUNTIMES_DIR -name "com.ibm.ws.ejb.thinclient*.jar")
ORB_JAR=$(find $WAS_RUNTIMES_DIR -name "com.ibm.ws.orb*.jar")
EMBEDDABLE_CONTAINER_JAR=$(find $WAS_RUNTIMES_DIR -name "com.ibm.ws.ejb.embeddableContainer_[0-9].*.jar")
CRYPTO_JAR=$(find $WAS_PLUGINS_DIR -name "com.ibm.ws.security.crypto.jar")
WAS_JARS=$ADMIN_CLIENT_JAR:$ADMIN_THINCLIENT_JAR:$ORB_JAR:$EMBEDDABLE_CONTAINER_JAR:$CRYPTO_JAR
ITCAM_LIB_EXT_DIR=$AM_HOME/lib/ext
ITCAM_TOOLKIT_LIB_EXT_DIR=$ITCAMDCHOME/toolkit/lib/ext
ITCAM_TOOLKIT_LIB_DIR=$ITCAMDCHOME/toolkit/lib
ITCAM_TOOLKIT_ETC=$ITCAMDCHOME/toolkit/etc
ITCAM_PLUGINS_DIR=$ITCAMDCHOME/plugins
AM_NODENAME=$AM_WAS_NODE
APPSERVER_PLATFORM=$AM_WASCELL
AM_PROFILENAME=$AM_WASPROFILE
AM_CAMTOOLKIT_GPE_DC_OPERATION_MODE=JD
COM_IBM_TIVOLI_ITCAM_TOOLKIT_AI_RUNTIMEBUILDER_ENABLE_REBUILD=true
COM_IBM_TIVOLI_TOOLKIT_PMICOLLECTOR_CHECKPOINT_READ=true
COM_IBM_TIVOLI_ITCAM_TOOLKIT_AI_READ_CONFIG_CHECKPOINT=true
DISABLE_TOOLKIT_CONFIGDIR=true
DISABLE_NATIVE_LIB=true
TOOLKIT_INITIALIZE=false
RUNTIMEBUILDER_PRODUCT_MASKS=itcam_jd.properties
APD_TRUST_STORE_TYPE=JKS
APD_TRUST_STORE_PASSWORD=ibmtivoli
APD_TRUST_STORE_FILE=$ITCAMDCHOME/runtime/apdstore
############
 
manageprocs()
{
  OS=`uname`
  case $OS in 
    Linux )
    PID_CMD="ps auxww | grep "Djmxdcmode=true" | grep -v grep ";
  ;;
    AIX)
    PID_CMD="( export COLUMNS=5000 ; ps -eaf | grep "Djmxdcmode=true" | grep -v grep )";
  ;;
    SunOS )
    PID_CMD="/usr/ucb/ps -auxww | grep "Djmxdcmode=true" | grep -v grep ";
  ;;
    Interix )
    PID_CMD="ps -eafwwXunix | grep "Djmxdcmode=true" | grep -v grep";
  ;;
  esac

  pids=`eval ${PID_CMD} | awk '{ print $2 }'`
  if [ ! -z "$pids" ]; then
     echo *Info*  Stopping existing JMXServer instance $pids
     kill -9 $pids;
  fi
}
manageprocs

cp $ITCAMDCHOME/jmxdc/etc/apdstore $APD_TRUST_STORE_FILE
cp $ITCAMDCHOME/jmxdc/etc/itcam_jd.properties $ITCAMDCHOME

$JAVA_BIN -Xmx256m -Xdebug -DXrunjdwp:transport=dt_socket,server=y,address=9999,suspend=n -Dapd.truststorekeytype=$APD_TRUST_STORE_TYPE -Dapd.truststorepassword=$APD_TRUST_STORE_PASSWORD -Dapd.truststorefile=$APD_TRUST_STORE_FILE -Djmxdcmode=true -Dcom.sun.management.jmxremote -Dcom.sun.management.jmxremote.port=$JMX_PORT -Dcom.sun.management.jmxremote.authenticate=false -Dcom.sun.management.jmxremote.ssl=false -Dam.home=$AM_HOME -DWAS_SERVER_NAME=$WAS_SERVER_NAME -Dam.wasserver=$AM_WASSERVER -Dam.wasnode=$AM_WAS_NODE -Dam.wascell=$AM_WASCELL -Dam.wasprofile=$AM_WASPROFILE -Dam.nodename=$AM_NODENAME -Dappserver.platform=$APPSERVER_PLATFORM -Dam.profilename=$AM_PROFILENAME -Dam.appserver=$AM_APPSERVER -Dam.camtoolkit.gpe.alt.runtime=$AM_CAMTOOLKIT_GPE_ALT_RUNTIME -Dcom.ibm.tivoli.itcam.toolkit.runtime.dir=$COM_IBM_TIVOLI_ITCAM_TOOLKIT_RUNTIME_DIR -DITCAMDCHOME=$ITCAMDCHOME -DITCAMDCVERSION=$ITCAMDCVERSION -Dam.camtoolkit.gpe.dc.operation.mode=$AM_CAMTOOLKIT_GPE_DC_OPERATION_MODE  -Dcom.ibm.tivoli.itcam.toolkit.ai.runtimebuilder.enable.rebuild=$COM_IBM_TIVOLI_ITCAM_TOOLKIT_AI_RUNTIMEBUILDER_ENABLE_REBUILD -Dcom.ibm.tivoli.toolkit.pmicollector.checkpoint.read=$COM_IBM_TIVOLI_TOOLKIT_PMICOLLECTOR_CHECKPOINT_READ -Dcom.ibm.tivoli.itcam.toolkit.ai.read.config.checkpoint=$COM_IBM_TIVOLI_ITCAM_TOOLKIT_AI_READ_CONFIG_CHECKPOINT -Ddisable.toolkit.configdir=$DISABLE_TOOLKIT_CONFIGDIR -Ddisable.native.lib=$DISABLE_NATIVE_LIB -Dtoolkit.initialize=$TOOLKIT_INITIALIZE -Druntimebuilder.product.masks=$RUNTIMEBUILDER_PRODUCT_MASKS -classpath $ITCAM_LIB_EXT_DIR/kwjmbeans.jar:$ITCAM_LIB_EXT_DIR/was/kynmbeans.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/reqaggr.jar:$ITCAM_LIB_EXT_DIR/kwjmbeans_extension.jar:/kwj.java/lib/kwjwb.jar:/kwj.java/lib/kwjplugin.jar:/kwj.java/lib/kwjwb-tools.jar:/kwj.java/lib/migration-tool.jar:/itm.common/lib/kynwb.jar:/cg.mbean/lib/itcam.cg.mbean.jar::$ITCAM_TOOLKIT_LIB_DIR/bcm-bootstrap.jar:$ITCAM_TOOLKIT_LIB_DIR/tk_bci_engine.jar:$ITCAM_TOOLKIT_ETC/toolkit_config_gpe.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/tk_ws_helpers.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/tk_ws_handlers.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/tk_thread_callbacks.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/tk_runtime.jar:$ITCAM_TOOLKIT_LIB_DIR/tk_reqmap_extract.jar:$ITCAM_TOOLKIT_LIB_DIR/tk_nodyninst.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/cdn.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/jmxengine.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/was/was6handler.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/was/websphereconnector.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/tk_jflt.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/copyright.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/tk_utils.jar:$ITCAM_LIB_EXT_DIR/was/pmicollector.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/glswrapper.jar:$ITCAM_PLUGINS_DIR/com.ibm.tivoli.itcam.bundlemanager_7.2.0.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/was/gpex.mbeans_was.jar:$ITCAM_TOOLKIT_ETC/toolkit_config_gpex.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/was/gpex.extractors_was.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/was/gpex.monitor_was.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/was/gpex.ws_handlers_was.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/was/gpex.runtimebuilder_washelper.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/gpex.turbo_intf.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/was/gpex.runtimebuilder_washelper.jar:$WAS_JARS:$ITCAM_TOOLKIT_LIB_EXT_DIR/jflt.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/jlog.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/jffdc.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/jflt.jar:$ITCAM_TOOLKIT_LIB_EXT_DIR/gls.jar:$ITCAM_LIB_EXT_DIR/jakarta-regexp-1.3.jar:$ITCAM_LIB_EXT_DIR/ppe.service.jar com.ibm.tivoli.itcam.tema.was.remote.WasJMXServer
